
echo "Restoring Metadata on Unix after BRM Restore from another OS"
echo
echo "Solutions Configuration directory: $SASCONFIG"
echo "Solutions Main directory: $SASMAIN"
echo "SAS Executable: $SASEXE"
echo "Temporary directory: $TEMPDIR"
echo
echo "This script will perform the operations required to convert"
echo "metadata to the proper format for this operating system, after"
echo "a restore has been performed with data from a different "
echo "operating system."
echo
echo "Press Ctrl-C to exit, Enter to continue..."
read line

if [ ! -d "$SASCONFIG" ]; then
    echo "Missing Solutions Configuration directory" 1>&2
    exit 1
fi

if [ ! -d "$SASMAIN" ]; then
    echo "Missing Solutions Main directory" 1>&2
    exit 1
fi

if [ ! -x "$SASEXE" ]; then
    echo "Missing SAS Executable" 1>&2
    exit 1
fi

if [ ! -d "$TEMPDIR" ]; then
    echo "Missing Temporary directory" 1>&2
    exit 1
fi

echo "Un-registering repositories"
cd $SASMAIN/MetadataServer/
./MetadataServer.sh stop
sleep 2
if [ -d rposmgr.bak ]; then
    echo "Removing previous rposmgr.bak"
    rm -f -r rposmgr.bak
fi
mv rposmgr rposmgr.bak
mkdir rposmgr
./MetadataServer.sh start
sleep 5
./MetadataServer.sh stop
sleep 2

echo "Converting repositories"
cd $SASCONFIG/Utilities/SESContent/
cp -R $SASMAIN/MetadataServer/MetadataRepositories/ $TEMPDIR/
omaport_ok=0
if $SASEXE config/omaport.sas; then
    omaport_ok=1
fi
if [ "$omaport_ok" = "0" ]; then
    echo "There was an error running omaport.sas, conversion failed." 1>&2
    echo "Look for an omaport.log file." 1>&2
    exit 1
fi
if grep -i error omaport.log; then
    echo "There were OMAPORT errors, see omaport.log" 1>&2
    echo "Delete $TEMPDIR/MetadataRepositories before re-running $0" 1>&2
    exit 1
fi
cd $SASMAIN/MetadataServer/
if [ -d MetadataRepositories.bak ]; then
    echo "Removing previous MetadataRepositories.bak"
    rm -f -r MetadataRepositories.bak
fi
mv MetadataRepositories MetadataRepositories.bak
mv $TEMPDIR/MetadataRepositories/ .
./MetadataServer.sh start

echo 
echo
echo "The metadata has now been converted and the metadata server has been"
echo "restarted."
echo
echo "Next, connect to the metadata server on this machine via the "
echo "SAS Management Console and add back the repositories, starting"
echo "with the Foundation repository."
echo
echo "Follow the directions in the document:"
echo "SAS Solutions Services 1.4:"
echo "Backup, Restore, and Migration Tool"
echo "(BRMTool.pdf)"
echo 
