// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas;

/**
 * ComponentException is a RuntimeException used by the com.sas components to
 * throw exceptions.  A message and/or Throwable may be included with the exception.
 * <P>
 * Examples:<BR>
 * <pre>
 *     throw new ComponentException( "Something bad happened" );
 *     throw new ComponentException( someOtherException );
 *     throw new ComponentException( "Something bad happened", someOtherException );
 * </pre>
 * <P>
 * @author Scott P. Leslie
 */
public class ComponentException extends RuntimeException
{
	/**
	 * The exception which caused this exception to be generated.
	 */
	protected Throwable throwable;

	/**
	 * Constructor with a null description and no causal exception.
	 */
	public ComponentException()
	{
		this(null, null);
	}

	/**
	 * Constructor with a description and no causal exception.
	 * @param description The reason the exception occurred.
	 */
	public ComponentException( String description )
	{
		this(description, null);
	}

	/**
	 * Constructor with a null description and a causal exception.
	 * @param throwable The exception which caused this exception to be generated. 
	 */
	public ComponentException( Throwable throwable )
	{
		this(null, throwable);
	}

	/**
	 * Constructor with a null description and no causal exception.
	 * @param description The reason the exception occurred.
	 * @param throwable The exception which caused this exception to be generated. 
	 */
	public ComponentException( String description, Throwable throwable )
	{
		super( description == null &&  throwable != null ? throwable.getMessage() : description );

		this.throwable = throwable;
	}

	/**
	 * Returns the exception which caused this exception to be thrown.
	 * @return the exception which caused this exception to be thrown.
	 */
	public Throwable getTargetException()
	{
		return throwable;
	}
}
