// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas;

import com.sas.util.Classes;

/**
 * GlobalInfoTable is the location of applet wide 'globals'.
 * It provides classes the ability to access the current
 * applet.
 *
 * @author: Chris Bailey
 * @version 1.0
 **/


public class GlobalInfoTable
{
    private static Object object;

    /**
     * Store the current object to use when loading classes.
     * <code>setObject()</code> should be called by the applet or application in its
     * <code>init()</code> method:
     * <pre>
     * com.sas.GlobalInfoTable.setObject(this);
     * </pre>
     * @param object the currrent applet or application
     **/

    public static void setObject(Object objectParm )
    {
        object = objectParm;
    }

    /**
     * Return the current object to use when loading classes.
     * getObject() is for use by any component which wishes to reference the
     * base applet or application to showDocument, load classes, or getParameter, etc.
     * @return the object handle. This may be null if {@link #setObject(java.lang.Object)} was not called.
     **/

    public static Object getObject()
    {
        return object;
    }

    /**
     * Store the current applet.
     * <code>setApplet()</code> should be called by the applet in its
     * <code>init()</code> method:
     * <pre>
     * com.sas.GlobalInfoTable.setApplet(this);
     * </pre>
     * @param pApplet the currrent applet
     **/

    public static void setApplet(java.applet.Applet pApplet)
    {
        object = pApplet;
    }

    /**
     * Return the current applet.
     * getApplet() is for use by any component which wishes to reference the
     * base applet to showDocument or getParameter, etc.
     * @return the applet handle. This may be null if the applet did not
     * call {@link #setApplet(java.applet.Applet)}
     **/

    public static java.applet.Applet getApplet()
    {
        return (java.applet.Applet) object;
    }
  /**
   * Try to load a class relative to the current applet's class loader.
   * @param className the class name
   * @return a class
   * @throws java.lang.ClassNotFoundException if the class could not be loaded.
   * @see com.sas.util.Classes#loadClass
   */
  public static Class loadClass(String className) throws ClassNotFoundException
    {
      return Classes.loadClass(className, null, getObject());
    }

  /**
   * Try to load a class relative to the current applet's class loader.  This differs from loadClass
   * in that it throws a runtime exception.  This is used in place of uses of the dot class syntax
   * which does not throw an exception.
   * <P>
   * Note that an IllegalArgumentException may sitll be thrown from Class.forName() if an invalid
   * name for a class is passed in.
   *
   * @param className the class name
   * @return a class
   * @see com.sas.util.Classes#loadClass
   */
  public static Class _loadClass(String className)
    {
		try
		{
			return Classes.loadClass(className, null, getObject());
		}
		catch (ClassNotFoundException cnfe)
		{
			throw new ComponentException( cnfe );
		}
    }

}
