// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513


package com.sas;

/**
  * A base interface that can be used to check whether to call initialize by
  * determining if a component implements it or not. 
  */

public interface InitializeInterface
{
		/**
		 * The initialize() method was designed to be a synchronization point after the constructor
		 * for the object has completed and a number of methods have been called on the component.
		 * This allows components with "expensive" set method calls to delay processing
		 * until all the sets have occurred.
		 * <P>
		 * initialize() should be called on a component after is it constructed :
		 * <P>
		 * <PRE>
		 *     ListBox listBox = new ListBox();
		 *     listBox.initialize();
		 *     container.add( listBox );
		 * </PRE>
		 * <P>
		 * By default (in the ComponentInterface implementation), initialize() does nothing.
		 * However, various components override initialize() to perform operations, and it should be called.
		 * <P>
		 * This allows components which need processing during creation and deserialization to have
		 * a common place to perform operations.  Adding event handlers to subcomponents is a typical
		 * use of the initialize method.
		 */
		public void initialize();
        
} // InitializeInterface
