// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas;

import java.beans.BeanInfo;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Methods to support an implementation of com.sas.visuals.MultipleValueEventSourceInterface.
 */
public class MultipleValueEventSourceInterfaceSupport
{

    public static boolean supportsListenerInterface(Object object, java.lang.Class listenerInterface)
	{
//System.out.println("supportsListenerInterface: " + object + ", " + listenerInterface );
		if ( listenerInterface == java.beans.PropertyChangeListener.class  ||
			 listenerInterface == java.beans.VetoableChangeListener.class )
			return true;

//System.out.println("supportsListenerInterface: returning false" );
	 	return false;
	}

	public static java.lang.reflect.Method getEventMethod(Object object, java.lang.reflect.Method listenerMethod)
	{
//System.out.println("getEventMethod: " + object + ", " + listenerMethod );
		if ( listenerMethod.getName().equals( "propertyChange" )  ||
			 listenerMethod.getName().equals( "vetoableChange" ) )
		{
			try
			{
				Class[] params = new Class[0];
				return java.beans.PropertyChangeEvent.class.getMethod( "getPropertyName", params );
			}
			catch (NoSuchMethodException nsme )
			{
//System.out.println("getEventMethod: Exception :" + nsme );
				// Fall through and return null
			}
			catch (SecurityException se )
			{
//System.out.println("getEventMethod: Exception :" + se );
				// Fall through and return null
			}
		}

//System.out.println("getEventMethod: returning null" );
		return null; 
	}

	public static String[] getEventValues(Object object, java.lang.reflect.Method listenerMethod)
	{
		boolean returnContrainedProperties = false;
		boolean returnBoundProperties = false;

//System.out.println("getEventValues: " + object + ", " + listenerMethod );
		String methodName = listenerMethod.getName();
		if ( methodName.equals( "vetoableChange" ) )
		{
			returnContrainedProperties = true;
		}
		else if ( methodName.equals( "propertyChange" ) )
		{
			returnBoundProperties = true;
		}

		if ( methodName.equals( "propertyChange" ) ||
			 methodName.equals( "vetoableChange" ) )
		{
			try
			{
				BeanInfo beanInfo = com.sas.beans.Introspector.getBeanInfo( object.getClass() );
				PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();

				Vector propertyVector = new Vector( propertyDescriptors.length );
				for ( int i = 0; i < propertyDescriptors.length; i++ )
				{
//					System.out.println( propertyDescriptors[i].getName() + " : c=" + propertyDescriptors[i].isConstrained() + " : b=" + propertyDescriptors[i].isBound());
					if ( propertyDescriptors[i].getName().length() != 0  &&
						 //propertyDescriptors[i].isBound()  &&
						 !propertyDescriptors[i].isHidden() )
					{
						if ( returnContrainedProperties == propertyDescriptors[i].isConstrained()  ||
							 (returnBoundProperties  &&  propertyDescriptors[i].isBound() ) )
						{
							propertyVector.addElement( propertyDescriptors[i].getName() );
						}
					} // if
				} // for


				int count = propertyVector.size();
				String properties[] = new String[ count ];
				for ( int i = 0; i < count; i++ )
				{
					properties[i] = "\"" + (String) propertyVector.elementAt(i) + "\"";
				}

//System.out.println("getEventValues: returning " + properties.length );
				return properties;
			}
			catch (IntrospectionException ie )
			{
				// Fall through and return null
			}
		}

//System.out.println("getEventValues: returning null" );
		return null; 
	}
}
