// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas;

/**
 * The PropertyValueQueryInterface allows a property value on a bean to be get/set by passing
 * in the bean reference and the name of the property.  In the case of a set, the new value of
 * the property is also passed in.
 * <P>
 * This is used to allow the LinkPropertiesInterface implementation to avoid using reflection
 * in applets since reflection is a secure operation under some browsers.  Typically,
 * applets which have links and are generated by the IDE will implement the
 * PropertyValueQueryInterface and set/return property property values for the 
 * beans which have links in the applet.
 * <P>
 * @see com.sas.PropertyLinkAdaptor
 */
public interface PropertyValueQueryInterface
{
	/**
	 * Return the value of the property specified.
	 * @param bean The bean to get the property value from.
	 * @param property The name of the property to return the value of.
	 * @return The value of the property.
	 */
	public Object getPropertyValue( Object bean, String property );

	/**
	 * Set the value of the specified property.
	 * @param bean The bean to set the property value on.
	 * @param property The name of the property to set the value of.
	 * @param value The new value of the property.
	 */
	public void setPropertyValue( Object bean, String property, Object value );

	/**
	 * Return the type of the specified property.
	 * @param bean The bean which has the property.
	 * @param property The name of the property to get the type of.
	 * @return The type of the property.
	 */
	public Class getPropertyType( Object bean, String property );
}
