// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.beans;

/**
 * This is a abstract helper class which provides a variety of methods dealing with
 * JavaBeans Customizers.
 */
public abstract class BaseCustomizerUtil
{
	/**
	 * Returns the class of the bean customizer.
	 * @param beanClass The class to determine the customizer for.  Should not be null.
	 * @return The Customizer class or null if no Customizer is found.
	 */
	public static Class getCustomizerClass( Class beanClass )
	{
     	// Lookup the customizer class and create one

		java.beans.BeanInfo beanInfo;
		try
		{
			beanInfo = com.sas.beans.Introspector.getBeanInfo( beanClass );
		}
		catch ( java.beans.IntrospectionException e )
		{
			return null;
		}
		java.beans.BeanDescriptor beanDescriptor = beanInfo.getBeanDescriptor();
		if ( beanDescriptor == null )
		{
			return null;
		} // if

		Class customizerClass = beanDescriptor.getCustomizerClass();
		return customizerClass;
	}

	/**
	 * Returns the class of the bean customizer.
	 * @param beanClass The class to determine the customizer for.  Should not be null.
	 * @param useSuperclass Whether to look up the class hierarchy of the JavaBean
	 * for a Customizer if one is not available at the JavaBean's classs level.
	 * @return The Customizer class or null if no Customizer is found.
	 */
	public static Class getCustomizerClass( Class beanClass, boolean useSuperclass )
	{
		if ( useSuperclass == false )
		{
			return getCustomizerClass( beanClass );
		}
		
		while ( beanClass != null )
		{
			Class customizerClass = getCustomizerClass( beanClass );
			if ( customizerClass != null )
				return customizerClass;
			
			beanClass = beanClass.getSuperclass();
		}
		
		return null;
	}

	/**
	 * Returns the class of the bean customizer.
	 * @param bean The bean to determine the customizer for.  Should not be null.
	 * @param useSuperclass Whether to look up the class hierarchy of the JavaBean
	 * for a Customizer if one is not available at the JavaBean's classs level.
	 * @return The Customizer class or null if no Customizer is found.
	 */
	public static Class getCustomizerClass( Object bean, boolean useSuperclass )
	{
		return getCustomizerClass( bean.getClass(), useSuperclass );
	}

	/**
	 * Returns the class of the bean customizer.
	 * @param bean The bean to determine the customizer for.  Should not be null.
	 * @return The Customizer class or null if no Customizer is found.
	 */
	public static Class getCustomizerClass( Object bean )
	{
		return getCustomizerClass( bean.getClass() );
	}
	
	/**
	 * Returns an instance of the bean customizer.
	 * @param bean The bean to determine the customizer for.  Should not be null.
	 * @param useSuperclass Whether to look up the class hierarchy of the JavaBean
	 * for a Customizer if one is not available at the JavaBean's classs level.
	 * @return The Customizer or null if no Customizer is found.
	 */
	public static java.awt.Component getCustomizer( Object bean, boolean useSuperclass )
	{
		java.awt.Component customizer;
		try
		{
			Class customizerClass = getCustomizerClass( bean, useSuperclass );
			customizer = (java.awt.Component)customizerClass.newInstance();
		}
		catch (Exception e)
		{
			return null;
		}

		return customizer;
	}
	
	/**
	 * Returns an instance of the bean customizer.
	 * @param bean The bean to determine the customizer for.  Should not be null.
	 * @return The Customizer or null if no Customizer is found.
	 */
	public static java.awt.Component getCustomizer( Object bean )
	{
		return getCustomizer( bean, false );
	}
}
