// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.beans;

/**
 * BeanInfo is an extension to the standard java.beans.BeanInfo class to provide
 * additional information in the webAF classes.  Static property descriptors, default
 * input properties, and default output properties are provided via BeanInfo.
 */
public interface BeanInfo extends java.beans.BeanInfo
{
	/**
	 * getStaticPropertyDescriptors returns an array of PropertyDescriptor objects
	 * which describe the static properties of a webAF extended Bean.
	 * @returns An array of PropertyDescriptor objects or null if there are no static properties on the Bean.
	 */
	public abstract java.beans.PropertyDescriptor[] getStaticPropertyDescriptors();
	
	/**
	 * A wevAF extended Bean may have a default input property which is used to establish
	 * a default link between two components in the webAF IDE.  When a webAF extended 
	 * Bean is the destination for a link operation, the default input property is setup as
	 * the linked property.
	 * @returns Index of the default property in the PropertyDescriptor array returned by
	 *          getPropertyDescriptors.  Returns -1 if there is no default input property.
	 * @see #getDefaultOutputPropertyIndex
	 * @see java.beans.BeanInfo#getDefaultPropertyIndex
	 */
	public abstract int getDefaultInputPropertyIndex();

	/**
	 * A webAF extended Bean may have a default output property which is used to establish
	 * a default link between two components in the webAF IDE.  When a webAF extended 
	 * Bean is the source for a link operation, the default output property is setup as
	 * the source property.
	 * @returns Index of the default property in the PropertyDescriptor array returned by
	 *          getPropertyDescriptors.  Returns -1 if there is no default output property.
	 * @see #getDefaultInputPropertyIndex
	 * @see java.beans.BeanInfo#getDefaultPropertyIndex
	 */
	public abstract int getDefaultOutputPropertyIndex();
}
