// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.beans;

import java.beans.IntrospectionException;
import java.lang.reflect.Method;


/**
 * IndexedPropertyDescriptor extends java.beans.PropertyDescriptor to provide more information about
 * properties for use within the webAF environment.
 */
public class IndexedPropertyDescriptor extends PropertyDescriptor
{
	/**
	 * Constructor used to construct a com.sas.beans.IndexedPropertyDescriptor from either a
	 * java.beans.IndexedPropertyDescriptor or a com.sas.beans.IndexedPropertyDescriptor.
	 * @param property A java.beans.IndexedPropertyDescriptor to use to create this property descriptor.
	 * @exception java.beans.IntrospectionException Exception thrown by java.beans.PropertyDescriptor.
	 */
	public IndexedPropertyDescriptor( java.beans.IndexedPropertyDescriptor property ) throws IntrospectionException
	{
		super( property.getName(), property.getReadMethod(), property.getWriteMethod() );
	}

	/**
	 * Constructor to create an IndexedPropertyDescriptor based on the property name and the get and set methods.
	 * @exception java.beans.IntrospectionException Exception thrown by java.beans.PropertyDescriptor.
	 */
	public IndexedPropertyDescriptor( String propertyName, Method getter, Method setter ) throws IntrospectionException
	{
		super( propertyName, getter, setter );
	}
	
	/**
	 * Constructor to create a IndexedPropertyDescriptor based on a property name and class.
	 * @exception java.beans.IntrospectionException Exception thrown by java.beans.PropertyDescriptor.
	 */
	public IndexedPropertyDescriptor( String propertyName, Class beanClass ) throws IntrospectionException
	{
		super( propertyName, beanClass );
	}

	/**
	 * Constructor to create an IndexedPropertyDescriptor based on a property name, class, and accessor method names.
	 * @exception java.beans.IntrospectionException Exception thrown by java.beans.PropertyDescriptor.
	 */
	public IndexedPropertyDescriptor( String propertyName, Class beanClass, String getterName, String setterName ) throws IntrospectionException
	{
		super( propertyName, beanClass, getterName, setterName );
	}

//	/**
//	* Makes a copy of the PropertyDescriptor object.
//	* @return A copy of the PropertyDescriptor object.
//	*/
//	public Object clone()
//	{
//        try
//        {
//			PropertyDescriptor clone = (PropertyDescriptor) super.clone();
//			
//			return clone;
//        }
//        catch (CloneNotSupportedException cnse)
//        {
//			System.err.println("PropertyDescriptor.clone(): " + cnse.getMessage() );
//        }
//        return null;
//	} // clone
}
