////////////////////////////////////////////////////////////////////////
// Do not edit this source file.
// This source was generated by com.sas.tools.GenerateEventListenerList
// on February 10, 2006 6:04:13 PM EST
// Program args:
//     -serial -4659371781248063852L -silent -D./com/sas/beans java.beans.PropertyChangeEvent java.beans.PropertyChangeListener com.sas.beans.PropertyChangeListenerList com.sas.beans.PropertyChangeSource
////////////////////////////////////////////////////////////////////////

package com.sas.beans;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Manage a list of listeners for the java.beans.PropertyChangeEvent event.
 * This class is normally used in classes which implement
 * the <com.sas.beans.PropertyChangeSource class
 * @see java.beans.PropertyChangeEvent
 * @see java.beans.PropertyChangeListener
 * @see com.sas.tools.GenerateEventListenerList
 */

public final class PropertyChangeListenerList
                   implements java.io.Serializable, java.beans.PropertyChangeListener
{
  ////////////////////////////////////////////////////////////////////////
  // Instance Variables
  ////////////////////////////////////////////////////////////////////////

  static final long serialVersionUID = -4659371781248063852L;

  // the number of actual listeners in the listener list
  private int numberOfListeners;

  // the array of listeners. Items [0..numberOfListeners-1] are used;
  // the remaining are available for add calls.
  private PropertyChangeListener[] listenerList;
  private static PropertyChangeListener[] emptyListenerList;

  // an array to hand back to getListeners()
  private PropertyChangeListener[] listeners;

  // how much to add to the array
  private int increment;

  // default array increment.
  private static final int DEFAULT_INCREMENT = 8;

  ////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////

  /**
   * Construct a default PropertyChangeListenerList
   */
  public PropertyChangeListenerList()
  {
    this(DEFAULT_INCREMENT, true, false);
  }

  /**
   * Construct a PropertyChangeListenerList
   */
  public PropertyChangeListenerList(int initialIncrement, boolean logExceptions, boolean reThrowExceptions)
  {
    if (initialIncrement <= 0)
      this.increment = DEFAULT_INCREMENT;
    else
      this.increment = initialIncrement;
    this.logExceptions = logExceptions;
    this.reThrowExceptions = reThrowExceptions;
  }

  /**
   * Construct a default PropertyChangeListenerList with a
   * specific initialIncrement.
   * @param initialIncrement how much to grow the internal
   * array of listeners when it fills up and must be extended
   * to allow adding new listeners.
   */
  public PropertyChangeListenerList(int initialIncrement)
  {
    this(initialIncrement, true, false);
  }


  ////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////

  /**
   * return the number of listeners.
   * @return the number of listeners.
   */
  public int count()
  {
     return numberOfListeners;
  }

  private boolean reThrowExceptions;
  private boolean logExceptions;

  /**
   * Set whether this listener list logs exceptions caught when delivering
   * an event to a listener.
   * @param logExceptions if <code>true</code>, exceptions are logged to <code>System.err</code>
   * @see #setExceptionReThrown(boolean)
   */
   public void setExceptionLoggingEnabled(boolean logExceptions)
   {
     this.logExceptions = logExceptions;
   }

   /**
    * Return true if this listener list is currently logging exceptions
    * that are caught when delivering events to listeners in the list.
    * @return <code>true</code> if caught exceptiosn will be logged.
    */
    public boolean isExceptionLoggingEnabled()
    {
      return logExceptions;
    }

   /**
    * Handle an exception caught while delivering an event to a listener in the list.
    * If exception logging is enabled, print the exception and traceback to System.err.
    * If exception rethrow is enabled, rethtrow the exception.
    * @see #setExceptionLoggingEnabled(boolean)
    * @see #setExceptionReThrown(boolean)
    */
   protected void handleException(RuntimeException exception)
   {
     if (isExceptionLoggingEnabled())
       {
         System.err.println(exception.getMessage());
         exception.printStackTrace(System.err);
       }
     if (isExceptionReThrown())
       {
         throw(exception);
       }
   }


  /**
   * Specify whether this listener list should rethrow exceptions caught
   * when delivering an event to a listener. If <code>true</code>, then the event
   * is not delivered to any other listeners in this listener class.
   * It is expected that the class that owns this listener list
   * will handle the exception in some way.
   * @param reThrow if <code>true</code>, an exception caught
   * when delivering an event to a listener is rethrown and the event
   * is not delivered to any remaining listeners. Use extreme caution
   * when setting this to <code>true</code>.  If <code>false</code>,
   * exceptions are ignored (although they may be logged).
   * @see #setExceptionLoggingEnabled(boolean)
   */
   public void setExceptionReThrown (boolean reThrow)
   {
     reThrowExceptions = reThrow;
   }

   /**
    * Return the setting indicating whether the event delivery loops
    * re-throw any caught exceptions.
    * @return true if caught exceptions are rethrown.
    */
    public boolean isExceptionReThrown()
    {
      return reThrowExceptions;
    }

  /**
   * find the index of a listener in the listener list.
   * @return the listener index, or -1 if either the listener is not found
   *         or if listener == null.
   */
  private int indexOf(PropertyChangeListener listener)
  {
    // note: this is not synchronized because it is only called
    // from other synchronized methods.
    if (listenerList != null)
      for (int i = 0; i < numberOfListeners; i++)
        if (listenerList[i] == listener)
          return i;
    return -1;
  }

  /**
   * Add a listener to the list of listeners.
   * @param listener a new listener for the PropertyChangeEvent event
   */
  synchronized public void add(PropertyChangeListener listener)
  {
    if (listener == null) return;

    if (indexOf(listener) != -1)
      return;

    // invalidate the listeners cache array.
    listeners = null;

    if (    (listenerList == null)
         || (numberOfListeners >= listenerList.length))
      {
        // growing too fast? adjust incrment.
        if (numberOfListeners > (4 * increment))
          increment *= 2;
        PropertyChangeListener newListenerList[] =
              new PropertyChangeListener[ numberOfListeners + increment ];
        if (listenerList != null)
           System.arraycopy(listenerList, 0, newListenerList, 0, numberOfListeners);
        listenerList = newListenerList;
      }
    listenerList[ numberOfListeners++ ] = listener;
  }

  /**
   * Return a safe copy of the list of listeners for this event.
   * The client may iterate over this list and send events
   * to the clients in a thread safe manner; calls to
   * addPropertyChangeListener or removePropertyChangeListener may take
   * place but do not affect this copied listener list.
   * @return a safe arry copy of the listener list. The array contains no null items.
   */
  synchronized public PropertyChangeListener[] getListeners()
  {
    if (listeners == null)
    {
       if ( numberOfListeners > 0 )
         {
           listeners = new PropertyChangeListener[numberOfListeners];
           System.arraycopy(listenerList, 0, listeners, 0, numberOfListeners);
         }
       else
         {
           if (emptyListenerList == null)
             emptyListenerList = new PropertyChangeListener[0];
	   return emptyListenerList;
         }
    }
    return listeners;
  }

  /**
   * Remove a listener from the list of listeners.
   * @param listener the listener to remove from the list.
   */
  synchronized public void remove(PropertyChangeListener listener)
  {
    // if the listener is not in the list, or if listener == null,
    // indexOf will return -1.
    int index = indexOf(listener);
    if (index == -1)
      return;

    // invalidate the getListeners() array cache.
    listeners = null;

    numberOfListeners--;
    if (numberOfListeners == 0)
      {
        if (emptyListenerList == null)
          emptyListenerList = new PropertyChangeListener[0];
        listenerList = null;
        return;
      }

    // set the item to null so that serialization won't
    // write this object. Do this unconditionally, since
    // the below loop won't overwrite the item if it is the last one.
    listenerList[index] = null;

    // shift down all items above the found listener index.
    while (index < numberOfListeners)
      listenerList[index] = listenerList[++index];
  }

   /**
   * Send a {@link PropertyChangeEvent} event to all the {@link PropertyChangeListener}
   * objects in the listener list by invoking {@link java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent event)}.
   * If necessary, the listener list is copied first, so the event is sent only to each
   * item which is in the list at the time this method is called, even if
   * adds or removes are performed after the event send has started.
   * @param event the event to send
   */
  public void propertyChange(java.beans.PropertyChangeEvent event) 
  {
    if ( numberOfListeners == 0 )
       return;
    PropertyChangeListener[] listenerList = getListeners();
    for (int i = 0, n = listenerList.length; i < n; i++)
      {
        PropertyChangeListener listener = listenerList[i];
        // send the event to the listener:
        try {
        listener.propertyChange(event);
        } catch (RuntimeException exception) { handleException(exception); }
      }
      return ;
  }

  // Serialization
  private void readObject(java.io.ObjectInputStream stream)
    throws java.io.IOException, ClassNotFoundException
    {
      stream.defaultReadObject();
      // Older versions of the class saved 0-length arrays
      // or all null arrays. We can now safely replace these
      // with null; getListeners will return emptyListenerList
      if ( numberOfListeners == 0 )
        listenerList = null;
      // other stuff
    }

}
