// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.beans;

import java.beans.BeanInfo;
import java.beans.BeanDescriptor;
import java.beans.EventSetDescriptor;
import java.beans.MethodDescriptor;

public class SimpleBeanInfo extends java.beans.SimpleBeanInfo implements com.sas.beans.BeanInfo
{
	private BeanInfo additionalBeanInfo[];
	private BeanDescriptor beanDescriptor;
	private EventSetDescriptor[] events;
	private int defaultEvent;
	private java.beans.PropertyDescriptor[] properties;
	private int defaultProperty;
	private MethodDescriptor[] methods;
	private java.beans.PropertyDescriptor[] staticProperties;
	private int defaultInputPropertyIndex;
	private int defaultOutputPropertyIndex;
	private java.awt.Image icon;
	//private BeanInfo targetBeanInfo;
  
	protected SimpleBeanInfo( BeanDescriptor beanDescriptor,
						EventSetDescriptor[] events,
						int defaultEvent,
						java.beans.PropertyDescriptor[] properties,
						int defaultProperty,
						MethodDescriptor[] methods,
						java.beans.PropertyDescriptor[] staticProperties,
						int defaultInputPropertyIndex,
						int defaultOutputPropertyIndex,
						java.awt.Image icon )
	{
		this.beanDescriptor = beanDescriptor;
		this.events = events;
		this.defaultEvent = defaultEvent;
		this.properties = properties;
		this.defaultProperty = defaultProperty;
		this.methods = methods;
		this.staticProperties = staticProperties;
		this.defaultInputPropertyIndex = defaultInputPropertyIndex;
		this.defaultOutputPropertyIndex = defaultOutputPropertyIndex;
		this.icon = icon;
	}
	
	public java.beans.PropertyDescriptor[] getPropertyDescriptors() {
		return properties;
	}
	
	public int getDefaultPropertyIndex() {
		return defaultProperty;
	}
	
	public EventSetDescriptor[] getEventSetDescriptors() {
		return events;
	}
	
	public int getDefaultEventIndex() {
		return defaultEvent;
	}
	
	public MethodDescriptor[] getMethodDescriptors() {
		return methods;
	}
	
	public BeanDescriptor getBeanDescriptor() {
		return beanDescriptor;
	}

	public java.awt.Image getIcon( int iconSize )
	{
		return icon;
	}

	/**
	 * @see com.sas.beans.BeanInfo#getStaticPropertyDescriptors
	 */
	public java.beans.PropertyDescriptor[] getStaticPropertyDescriptors()
	{
		return staticProperties;
	}

	/**
	 * @see com.sas.beans.BeanInfo#getDefaultInputPropertyIndex
	 */
	public int getDefaultInputPropertyIndex()
	{
		return defaultInputPropertyIndex;
	}

	/**
	 * @see com.sas.beans.BeanInfo#getDefaultOutputPropertyIndex
	 */
	public int getDefaultOutputPropertyIndex()
	{
		return defaultOutputPropertyIndex;
	}

	/**
	 *
	 */
	public void dump()
	{
		System.out.println( "SimpleBeanInfo dump() starting" );

		System.out.println("");

		System.out.println( "    BeanDescriptor.getName = " + beanDescriptor.getName() );

		System.out.println( "    default property index = " + defaultProperty );
		System.out.println( "    default event index = " + defaultEvent );
		System.out.println( "    default input property index = " + defaultInputPropertyIndex );
		System.out.println( "    default output property index = " + defaultOutputPropertyIndex );
		System.out.println( "    icon = " + icon );

		System.out.println("");
		
		for ( int i = 0; i < properties.length; i++ )
		{
			System.out.println( "    Property[" + (new Integer(i)) + "] = " + properties[i].getName() );/*I18nOK:LINE*/
			System.out.println( "        Type = " + properties[i].getPropertyType().toString() );
			System.out.println( "        Read Method = " + properties[i].getReadMethod() );
			System.out.println( "        Write Method = " + properties[i].getWriteMethod() );
			System.out.println( "        PropertyEditorClass = " + properties[i].getPropertyEditorClass() );

			if ( properties[i] instanceof com.sas.beans.PropertyDescriptor )
			{
				com.sas.beans.PropertyDescriptor pd = (com.sas.beans.PropertyDescriptor) properties[i];
				
				System.out.println( "        DefaultValue = " + pd.getDefaultValue() );
			} // if
		} // for

		if ( staticProperties != null )
		{
			for ( int i = 0; i < staticProperties.length; i++ )
			{
				System.out.println( "    Static Property[" + (new Integer(i)) + "] = " + staticProperties[i].getName() );/*I18nOK:LINE*/
				System.out.println( "        Type = " + staticProperties[i].getPropertyType().toString() );
				System.out.println( "        Read Method = " + staticProperties[i].getReadMethod() );
				System.out.println( "        Write Method = " + staticProperties[i].getWriteMethod() );
				System.out.println( "        PropertyEditorClass = " + staticProperties[i].getPropertyEditorClass() );

				if ( staticProperties[i] instanceof com.sas.beans.PropertyDescriptor )
				{
					com.sas.beans.PropertyDescriptor pd = (com.sas.beans.PropertyDescriptor) staticProperties[i];
					
					System.out.println( "        DefaultValue = " + pd.getDefaultValue() );
				} // if
			}
		}

		if ( events!= null )
		{
			for ( int i = 0; i < events.length; i++ )
			{
				System.out.println( "    Event[" + (new Integer(i)) + "] = " + events[i].getName() );/*I18nOK:LINE*/
				System.out.println( "        Listener Type = " + events[i].getListenerType().toString() );
			}
		}

		System.out.println( "SimpleBeanInfo dump() done" );
	}
}