// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.beans;

import java.beans.VetoableChangeListener;

/**
 * An interface for objects which generate <code>java.beans.VetoableChangeEvents</code>.
 * @see java.beans.PropertyChangeEvent
 * @see com.sas.beans.PropertyChangeSource
 * @see com.sas.ComponentInterface
 * 
 * @author <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @version 1.0
 *
 */
public interface VetoableChangeSource
{
  /////////////////////////////////////////////////////////////////////
  // VetoableChangeEvent source Implementation
  /////////////////////////////////////////////////////////////////////

  /**
    * Adds a constrained property change listener to this event source.  The listener
    * will be notified of any property changes in the source via a <code>java.beans.VetoableChangeEvent</code>
    * and given a chance to deny the property set by throwing a <code>java.beans.PropertyVetoException</code>.
    * @param listener The property change listener
    * @see java.beans.PropertyChangeEvent
    * @see java.beans.PropertyVetoException
    */
  void addVetoableChangeListener( VetoableChangeListener listener );

  /**
    * Removes a constrained property change listener to this event source.  The listener
    * will no longer be notified of any property changes in the source
    * @param listener The property change listener
    * @see java.beans.PropertyChangeEvent
    */
  void removeVetoableChangeListener( VetoableChangeListener listener );

}
