// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.collection;

/**
 * The ContentsChangedEvent is fired when the contents of
 * an object has changed.  For example, Collections which implement
 * the com.sas.util.CollectionInterface interface will
 * fire this event when items are added, removed, reordered, etc.
 * <p>
 * ContentsChangedEvent is a subclass of java.beans.PropertyChangeEvent.
 * It has a null propertyName and null old/new values; this indicates
 * that unspecified attributes have changed, i.e. that the
 * entire component has changed; clients/viewers should refresh.
 * <p>
 * Your class can manage a set of ContentsChangedListener objects
 * with the ContentsChangedListenerList support class.  Also note
 * that there are several subclasses of ContentsChangedEvent
 * which provide more explicit, detailed information about how a
 * collection has changed. A ContentsChangedListener may actually
 * receive one of these subclasses.  For example, a listener for
 * a ContentsChangedEvent on an OrderedCollection will receive
 * a com.sas.util.CollectionChangedInfo, which extends from ContentsChangedEvent.
 * This object tells more information about how the ordered collection
 * changed; i.e. specifically which items were added, removed, reordered,
 * or replaced.  A viewer may be able to perform incremental updates
 * given this more accurate data about how the contents changed.
 * <p>
 * If your class fires this event, it should implement the ContentsChangedSource
 * interface.
 *
 * @see com.sas.collection.CollectionInterface
 * @see com.sas.collection.ContentsChangedSource
 * @see com.sas.collection.ContentsChangedListener
 * @see com.sas.collection.ContentsChangedListenerList
 * @see com.sas.collection.CollectionChangedInfo
 * @see com.sas.collection.DictionaryChangedInfo
 *
 * @author sasdjb <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @release 1.1
 */
public class ContentsChangedEvent
       extends java.beans.PropertyChangeEvent implements Cloneable
{
  static final long serialVersionUID = 4726121474057349339L;

//  allChanged removed as per Ken/Diane's request, sasdjb 31Aug98
//  /**
//   * Indicates that the change represents a significant
//   * change in contents, such as a complete replacement of the contents,
//   * as opposed to a incremental or less drastic
//   * change if <strong><code>false</code></strong> (the default),
//   * such as altering a portion of the contents or adding or
//   * removing or replacing some elements of a collection.
//   * <p>
//   * For example, some views may maintain a <em>cursor</em>
//   * or scroll position when viewing an ordered collection.
//   * When <var>allChanged</var>
//   * is <strong><code>true</code></strong>, such views
//   * may reset their cursor/scroll position to the beginning
//   * of the collection.
//   */
//  public boolean allChanged;

  /**
   * Default constructor.
   * @param source the object whose contents have changed.
   */
  public ContentsChangedEvent(Object source)
  {
    // this(source, null, null, null, false);
    super(source, null, null, null);
  }

//  allChanged removed as per Ken/Diane's request, sasdjb 31Aug98
//  /**
//   * Default constructor.
//   * @param source the object whose contents have changed.
//   * @param <var>allChanged</var> if true, the event reflects a drastic or complete
//   *        change to the contents. If false, the event indicates a
//   *        less drastic change in the model.
//   */
//  public ContentsChangedEvent(Object source, boolean allChanged)
//  {
//    this(source, null, null, null, allChanged);
//  }

  /**
   * Construct a property change event with the specified paramedters;
   * @param source the object whose contents have changed.
   * @param propertyName the property name of the changed property; may be null
   * @param oldValue the old value; may be null
   * @param newValue the new value; may be null
   */
  public ContentsChangedEvent(Object source, String propertyName, Object oldValue, Object newValue)
  {
    // this(source, propertyName, oldValue, newValue, false);
    super(source, propertyName, oldValue, newValue);
  }

//  allChanged removed as per Ken/Diane's request, sasdjb 31Aug98
//  /**
//   * Property change constructor
//   * @param source the object whose contents have changed.
//   * @param propertyName the property name of the changed property; may be null
//   * @param oldValue the old value; may be null
//   * @param newValue the new value; may be null
//   * @param allChanged if true, the event reflects a drastic or complete
//   *        change to the contents. If false, the event indicates a
//   *        less drastic change in the model.
//   */
//  public ContentsChangedEvent(Object source, String propertyName, Object oldValue, Object newValue, boolean allChanged)
//  {
//    // Construct a default java.beans.PropertyChangeEvent
//    // which indicates no specific property has changed; i.e. the entire
//    // component has changed.
//    super(source, propertyName, oldValue, newValue);
//    this.allChanged = allChanged;
//  }

  	/**
     * Clone the ContentsChangedEvent and sets the source to be the newSource
     * @return a clone of the ContentsChangedEvent, with the source changed to the newSource.
     * @exception CloneNotSupportedException if the ContentsChangedEvent cannot be cloned.
     **/
  	public Object clone(ContentsChangedSource newSource) throws CloneNotSupportedException
  	{
		ContentsChangedEvent clone = (ContentsChangedEvent)super.clone();
	  	clone.source = newSource;
		return clone;
  	}
}