// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

/**
 * Nil is a subclass of Pair, used to implement the nil singleton object.
 * The nil object is a special Pair object used to represent and empty list,
 * or the tail of a list. nil is implemented via a subclass so that the
 * special case code is handled by method dispatch rather than sprinkling
 * checks for the nil object throughout the Pair class.
 * @version 1.1
 * @author sasdjb <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 */

package com.sas.collection;
import com.sas.collection.Pair;
import com.sas.util.ApplyInterface;
import java.io.PrintStream;
import com.sas.util.transforms.TransformInterface;
import com.sas.util.transforms.TransformException;

final class Nil extends Pair
{
  static final long serialVersionUID = 2411960590071316906L;

  static final String RB_KEY = "Nil.";

  // no-public constructor. Nil is a singleton instantiated once, when Pair is loaded
  Nil()
  {
    super(null, null);
  }

  public void setcar(Object car)
  {
    throw new IllegalStateException(RB.getStringResource(RB_KEY, "nilImmutable.ex"));
  }

  public void setcdr(Object cdr)
  {
    throw new IllegalStateException(RB.getStringResource(RB_KEY, "nilImmutable.ex"));
  }

  public int length()
  {
    return 0;
  }

  /**
   * Clone the object.
   * @return a clone of the nil object.
   * This does not really clone it, since nil is a singleton
   * @exception CloneNotSupportedException if the clone fails.
   */
  public Object clone()
  {
    return nil;
  }

  public Pair getLast()
  {
    return nil;
  }

  public boolean nullp()
  {
    return true;
  }

  public boolean equals (Pair other)
  {
    return other == nil;
  }

  protected void print(PrintStream s, int depth, int length )
  {
    s.print("null");
  }

  public Pair mapcar(ApplyInterface t)
  {
    return nil;
  }

  public Pair transform(TransformInterface t)
  {
    return nil;
  }

  public String toString()
    {
      return "null";
    }

  // Serialization
  private void readObject(java.io.ObjectInputStream stream)
    throws java.io.IOException, ClassNotFoundException
    {
      throw new java.io.IOException(RB.getStringResource(RB_KEY, "cantDeserializeNil.ex.txt"));
    }


  // Serialization
  private void writeObject(java.io.ObjectOutputStream stream)
    throws java.io.IOException
    {
      throw new java.io.IOException(RB.getStringResource(RB_KEY, "cantSerializeNil.ex.txt"));
    }

}
