// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.io;

import java.io.InputStream;
import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.UnmarshalException;

/**
 * An IOException class which can wrap an underlying exception.
 *
 * @author  Russell Gonsalves
 * @version 1.0 3Dec01
 */

public class ChainedIOException extends IOException implements com.sas.util.ChainedExceptionInterface {
   private Throwable chainedException;

   /**
    * Create a ChainedIOException that contains no other exception
    * or exception message.
    */
   public ChainedIOException() {
   }

   /**
    * Create a ChainedException that is not based on another exception.
    *
    * @param message the text describing the exception.
    */
   public ChainedIOException(String message) {
     super(message);
   }

   /**
    * Create a ChainedException that is not based on another exception.
    *
    * @param message the text describing the exception.
    * @param chainedException the exception this one chains to
    */
   public ChainedIOException(Throwable chainedException, String message) {
     super(message);
     this.chainedException = chainedException;
   }

   /**
    * Get the exception this one chains to.
    */
   public Throwable getTargetException() {
     return chainedException;
   }
 }
