package com.sas.io;

/**
 * Describes a code page encoder which converts text between
 * Unicode and a single byte code stream.
 * @author sasdjb <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @version 1.0
 * @since 1.2
 */

public interface EncoderInterface {

  /**
   * Convert an array of bytes into a String.
   * @param bytes an array of bytes to encode as a String
   * @param offset the byte offset to start encoding. (Ignore the first <var>offset</var> bytes.)
   * @param length the number of bytes to read from the <var>bytes</var> array
   * @return a String constructed from encoding the bytes using this Encoder's current
   *         encoding, or null if there is an IOException.
   */
  public String toString(byte bytes[], int offset, int length);

  /**
   * Convert an array of bytes into a string, using the current encoding.
   * @param bytes an array of bytes to encode as a String
   * @param offset the number to offset
   * @param length the length of the char array
   * @return the char array
   */
  public char[] toCharArray(byte bytes[], int offset, int length);

  /**
   * Convert a substring of a string to a byte array using this Encoder's current encoding.
   * @param string a string to convert to a byte array
   * @param offset the index of the first character in the input which should be encoded.
   * @param length the number of characters to extract from the string
   * @return the encoded bytes
   */
  public byte[] toByteArray(String string, int offset, int length);

  /**
   * Convert region of a character array to a byte array using this Encoder's current encoding.
   * @param string a string to convert to a byte array
   * @param offset the index of the first character in the input which should be encoded.
   * @param length the number of characters to encode
   * @return the encoded bytes
   */
  public byte[] toByteArray(char chars[], int offset, int length);

  /**
   * Provide access to a reusable byte array
   * which may be filled then used as input to the
   * {@link #toString(byte[], int, int)} methods.
   * This provides the user of this Encoder a place
   * to read/write bytes without having to manage them.
   * @param nBytes the minimum number of bytes needed.
   * @return an array of bytes with at least <var>nBytes</var>
   * bytes in it; there may be more.
   */
  public byte[] getByteArray(int nBytes);

  /**
   * Return the name of the character encoding
   * @return the name of the character encoding
   */
  public String getEncoding();

}
