// Copyright (c) 2002 SAS Institute Inc., Cary, NC 27513
package com.sas.io;

import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Map;


/**
 * Provides the capability to write header information to a
 * com.sas.iom.OutputStream IOM OutputStream.
 *
 * @author <a href="mailto:bryan.wolfe@sas.com">Bryan Wolfe</a>
 * @author <a href="mailto:tony.dean@sas.com">Tony Dean</a>
 * @version 1.0
 * @since 1.0
 *
 * @history
 *    28Aug2002 td  Created.
 *
 */
public interface OutputStreamHeaderInterface extends Remote {

   /**
    * Set a header property for the output stream.
    *
    * @param name  header property name.
    * @param value header property value.
    *
    * @throws IOException if an I/O error occurs.
    * @throws IllegalStateException if called after data is written.
    * @throws RemoteException if a network problem occurs.
    */
   public void setHeader(String name, String value) throws IOException, IllegalStateException, RemoteException;

   /**
    * Set header properties for the output stream.
    *
    * @param headers map of header properties.
    *
    * @throws IOException if an I/O error occurs.
    * @throws IllegalStateException if called after data is written.
    * @throws RemoteException if a network problem occurs.
    */
   public void setHeaders(Map headers) throws IOException, IllegalStateException, RemoteException;

}
