package com.sas.io;

import java.io.ByteArrayOutputStream;
//
// History:
//
// Date    Name            Remarks
// 23Aug01 B Beers         S0124519: Split socket driver code into SocketAccessMethod.
//
/**
 * This class extends ByteArrayOutputStream to allow the user to mark
 * locations in the output which can later be patched.
 * @author Biff Beers
 */
public class PatchableByteArrayOutputStream extends ByteArrayOutputStream
{
     public PatchableByteArrayOutputStream()
     {
          super();
          patchList = new int[defaultPatches];
     }

     public PatchableByteArrayOutputStream(int size)
     {
          super(size);
          patchList = new int[defaultPatches];
     }

     public PatchableByteArrayOutputStream(int size,int patches)
     {
          super(size);

          if (patches <= 0)
          {
               patches = defaultPatches;
          }

          patchList = new int[patches];
     }

     public void setPatchLocation()
     {
          patchList[patchLocationCounter++] = count;
     }

     public void skip(int n)
     {
          for (int i=0;i<n;i++)
          {
               write(0);
          }
     }

     public void patch(byte[] b,int off,int len)
     {
          System.arraycopy(b,off,buf,patchList[patchWriteCounter++],len);
     }

     public void reset()
     {
          patchLocationCounter = 0;
          patchWriteCounter = 0;
          super.reset();
     }

     private static final int defaultPatches = 8;
     private int[] patchList = null;
     private int patchLocationCounter = 0;
     private int patchWriteCounter = 0;
}