// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.io;

import java.io.IOException;
import java.io.OutputStream;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Encapsulates a <code>java.io.OutputStream</code> to
 * allow access from a remote virtual machine.
 * <code>RemoteOutputStream</code> implements
 * <code>RemoteOutputStreamInterface</code>.
 * Method signatures and behavior are
 * identical to the encapsulated
 * <code>OutputStream</code> except for the addition of
 * remote exception handling where necessary.
 *
 * @see     com.sas.io.RemoteOutputStreamProxy
 * @author  Bryan Wolfe
 * @version 1.0 09/28/01
 */

public class RemoteOutputStream extends UnicastRemoteObject
    implements RemoteOutputStreamInterface
{
    transient private OutputStream os;

/**
 * Constructs a <code>RemoteOutputStream</code>
 * encapsulating the specified output stream.
 *
 * @param is the stream to be wrapped by this object.
 * @throws RemoteException for any rmi network error.
 */
    public RemoteOutputStream(OutputStream os)
        throws RemoteException
    {
        this.os = os;
    }

    public void write(int b)
        throws IOException
    {
        os.write(b);
    }

    public void write(byte[] b)
        throws IOException
    {
        os.write(b);
    }

    public void write(byte[] b, int off, int len)
        throws IOException
    {
        os.write(b, off, len);
    }

    public void flush()
        throws IOException
    {
        os.flush();
    }

    public void close()
        throws IOException
    {
        os.close();
    }
}
