// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.io;

import java.io.IOException;

/**
 * A proxy wrapper for access to a 
 * <code>java.io.OutputStream</code> on a remote
 * virtual machine.  The class extends and reimplements 
 * <code>java.io.OutputStream<code> on an encapsulated
 * <code>RemoteOutputStreamInterface</code> object.  
 *
 * @see     com.sas.io.RemoteOutputStream
 * @author  Bryan Wolfe
 * @version 1.0 09/28/01
 */

public class RemoteOutputStreamProxy
    extends java.io.OutputStream
{
    transient private RemoteOutputStreamInterface os;
    
/**
 * Constructs a <code>RemoteOutputStreamProxy</code> 
 * encapsulating the specified remote output stream.
 *
 * @param is the stream to be wrapped by this object.
 */
    public RemoteOutputStreamProxy(RemoteOutputStreamInterface os)
    {
        this.os = os;
    }
    
    public void write(int b)
        throws IOException    
    {
        os.write(b);
    }
    
    public void write(byte[] b)
        throws IOException
    {
        os.write(b);
    }
    
    public void write(byte[] b, int off, int len)
        throws IOException    
    {
        os.write(b, off, len);
    }
    
    public void flush()
        throws IOException    
    {
        os.flush();
    }
    
    public void close()
        throws IOException        
    {
        os.close();
    }

}