// Copyright (c) 2002 SAS Institute Inc., Cary, NC 27513
package com.sas.iom;

import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Map;

import com.sas.io.OutputStreamHeaderInterface;


/**
 * Provides the capability to write header information to an
 * {@link com.sas.iom.OutputStream IOM OutputStream}.
 *
 * @author <a href="mailto:bryan.wolfe@sas.com">Bryan Wolfe</a>
 * @author <a href="mailto:tony.dean@sas.com">Tony Dean</a>
 * @version 1.0
 * @since 1.0
 * 
 * @history
 *    28Aug2002 td  Created.
 *
 */
public class OutputStreamHeader extends UnicastRemoteObject
   implements OutputStreamHeaderInterface {


   private OutputStream _outputStream = null;

   public OutputStreamHeader(OutputStream outputStream) throws RemoteException {
      _outputStream = outputStream;
   }


   /**
    *  Set header by delegating to the OutputStream.
    */
   public void setHeader(String name, String value) throws IOException, IllegalStateException, RemoteException {

      if( _outputStream != null )
         _outputStream.setHeader(name, value);
   }
   
   /**
    *  Set headers by delegating to the OutputStream.
    */
   public void setHeaders(Map headers) throws IOException, IllegalStateException, RemoteException {

      if( _outputStream != null )
         _outputStream.setHeaders(headers);      
   }

}
