// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.measures;
import com.sas.text.Message;

/**
 * <code>BaseLength</code> is a non-negative <code>BaseDistance</code>.
 * <code>BaseLength</code> supports the same units of measure as <code>BaseDistance</code>.
 *
 * @author Ryan Norris
 * @version 1.0
 */
public class BaseLength
   extends BaseDistance
{
	static final long serialVersionUID = 1322962397113020376L;

	/**
	 * Construct a <code>Length</code> object with the specified magnitude and unit.  The magnitude for a <code>Length</code>
	 * must be a non-negative value.
	 * @param magnitude the magnitude for the new <code>Length</code> object
	 * @param unit the unit of the new <code>Length</code> object
	 * @exception IllegalArgumentException if the magnitude is a negative value or if the unit value is
	 * not supported
	 */
	public BaseLength(double magnitude, String unit)
	{
		super(magnitude, unit);
		if (magnitude < 0)
		{
			throw new IllegalArgumentException(Message.format(RB.getStringResource("Length.negativeMagnitude.ex.txt"), new Double(magnitude)));
		}
	}

	/**
	 * Constructs a Length object with the specified values.  The magnitude for a Length
	 * must be a non-negative value.
	 * @param text String object containing the magnitude and unit
	 * @exception IllegalArgumentException if the magnitude is a negative value or if the unit value is
	 * not supported
	 */
	public BaseLength(String text)
	{
		super(text);
		if (magnitude < 0)
		{
			throw new IllegalArgumentException(Message.format(RB.getStringResource("Length.negativeMagnitude.ex.txt"), new Double(magnitude)));
		}
	}
}
