// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.measures;

 /**
  * <code>MeasureInterface</code> defines an object which has a magnitude and unit.  Additionally it
  * defines a means of converting a measurement from one unit to another.
  * @author Diane Baucom
  * @author Ryan Norris
  * @version 1.0
  */
public interface MeasureInterface
{
  /**
   * Returns the value of the magnitude property.  This is the numeric value of
   * the unit of measure.
   * @return numeric value of the measurement
   */
  public double getMagnitude();

  /**
   * Returns an equivalent magnitude of the current measurement in terms of the
   * specified unit of measure.  For example, to convert from kilometers
   * to millimeters, you can use the <code>{@link Distance}</code> class:
   * <code>new Distance(1, "km").getMagnitude("mm")</code>
   * @param unit the unit of measure to convert to
   * @return an equivalent magnitude in terms of the specified unit of measure
   */
  public double getMagnitude(Object unit);

  /**
   * Returns the unit of measure of the measurement.  Typically this will be the
   * abbreviated name of the unit property.  For example, <code>"in"</code> for inch and <code>"cm"</code>
   * for centimeter.
   * @return unit of measure of the measurement
   */
  public Object getUnit();

  /**
   * Returns the full name of the unit property.  For example, <code>"inch"</code> and <code>"centimeter"</code>.
   * @return unit of measure of the measurement
   */
  public String getUnitName();
  }
