// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.models;
/**
 * CacheException conveys exceptions occurring in cache access
 * operations. Since a Cache can cache values from 
 * objects which can generate various types of exceptions
 * (remote exceptions, IO exceptions, table exceptions),
 * a CacheException can be used to trap all these variants
 * @author sasdjb <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @version 1.0
 */
public class CacheException
       extends Exception
{
  private final Exception cause;

  /**
   * Construct a CacheException which embeds another exception.
   * @param e an exception caught in an internal cache operation
   */
  public CacheException(Exception e)
    {
      cause = e;
    }

  /**
   * Construct a CacheException with a message string.
   * @param text the cache exception message
   */
  public CacheException(String text)
    {
      super(text);
      cause = null;
    }

  /**
   * Construct a CacheException
   */
  public CacheException()
    {
      super();
      cause = null;
    }

  /**
   * Return the cache message.
   * If the cache embeds another exception, return its message text,
   * else return the string with which the exception was constructed.
   * @return the exception text
   */
  public String getMessage()
    {
      if (cause != null)
        return cause.getMessage();
      else
        return super.getMessage();
    }
}
