// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.models;

/**
 * An interface for accessing a read/write cache.
 * This is a cache which reads values from a secondary
 * cache, performs look ahead buffering of such items,
 * and allows the client to store values back into the cache
 * or to remove items from the cache.
 * @author sasdjb <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @version 1.0
 */

public interface CacheInterface
       extends StaticCacheInterface
{
  /**
   * Removes all items from the cache.
   */
  public void invalidateCache();

  /**
   * removes from the cache the item associated with the key <var>key</var>.
   * @param key the key for the item to remove from the cache.
   * @exception com.sas.models.CacheException if the request cannot
   * be satisfied.
   */
  public void invalidateCache(long key) throws CacheException;

  /**
   * Stores the value <var>value</var> in the cache so that it may be
   * accessed by the key <var>key</var>.  This does <strong>not</strong> write the item
   * through to the secondary cache.
   * @param key the item's key
   * @param value the value to store in the cache; it is associated with
   * the key. If a previous value is in the cache for this key,
   * it is dropped from the cache.
   * @exception com.sas.models.CacheException if the request cannot
   * be satisfied.
   */
  public void storeInCache(long key, Object value) throws CacheException;

  /**
   * Assigns the secondary cache. A previously assigned secondary cache may be
   * discarded, or the cache may cascade its caches.
   * @param cache the secondary cache to assign to this cache.
   * @exception com.sas.models.CacheException if the request cannot
   * be satisfied.
   */
  public void setSecondaryCache(StaticCacheInterface cache) throws CacheException;
}
