package com.sas.net;

 /**
  * Defines an interface to locate resources such as an image (apple.gif) or 
  * sound-file (ring.wav). 
  * @author Drew Anderson
  * @version 1.0
  */
public interface ResourceLocatorInterface
{
	/**
	 * Returns the name of the resource, e.g. "apple.gif" or "c:\sounds\ring.wav". 
	 * @return the path/name to the resource
	 * @see #setResourceName(String)
	 **/
	public String getResourceName();

	/**
	 * Sets the name of the resource, e.g. "apple.gif" or "c:\sounds\ring.wav". 
	 * Depending on the implementation of the ResourceLocatorInterface, the resourceName
	 * could be anything.  
	 * @param name the path/name to the resource
	 * @see #setResourceName(String)
	 **/
	public void setResourceName(String name);

	/** 
	 * Returns a {@link java.net.URL URL} to the resource defined by resourceName. If the 
	 * resource can not be found or if an error occurs in interpreting, null should 
	 * be returned. If when forming the URL an error occurs, a {@link java.net.MalformedURLException MalformedURLException} 
	 * will be thrown. 
	 * @return a URL to the resource, null if it cannot be found
	 * @see #setResourceName(String)
	 * @see java.net.URL
	 * @see java.net.MalformedURLException
	 **/
	public java.net.URL getResource() throws java.net.MalformedURLException;
}