/* $Id$ */
package com.sas.net.brg;

import java.lang.ref.WeakReference;
import java.net.Socket;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Logger;

import org.omg.CORBA.Any;
import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.ARG_OUT;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.Bounds;
import org.omg.CORBA.IMP_LIMIT;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.INV_OBJREF;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.NVList;
import org.omg.CORBA.OBJECT_NOT_EXIST;
import org.omg.CORBA.StringHolder;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.portable.Delegate;
import org.omg.CORBA.portable.ObjectImpl;
import org.omg.CORBA.TypeCodePackage.BadKind;
import com.sas.lang.NonExclusiveLock;
import com.sas.lang.RunnableQueue;
import com.sas.net.brg.orb.ActiveObjectMapKey;
import com.sas.net.brg.orb.BrgOrb;
import com.sas.net.brg.orb.DelegateBase;
import com.sas.net.brg.orb.GenericObject;
import com.sas.net.brg.orb.ProtocolEngineInterface;
import com.sas.net.brg.sam.SocketAccessMethod;
import com.sas.text.Message;
//
// History:
//
// Date    Name            Remarks
// 23Dec97 B Beers         Created.
// 05Feb98 B Beers         Updated for changes in protocol.
// 25Feb98 B Beers         Added debugLevel property.
// 26Feb98 B Beers         Added ObjectInterface.
// 27Feb98 B Beers         Added encryption indicators to protocol.
// 09Apr98 B Beers         Rearranged encryption indicators to allow negotiation.
// 26Jun98 B Beers         Handled OBJECTDESTRUCTOR message.
// 10Sep98 B Beers         Added cryptography support.
// 07Dec98 B Beers         Added event- and request-by-name.
// 06Jan99 B Beers         Moved CommandQueue from com.sas.util.
// 13Apr99 B Beers         S0050819: NLS changes stage 1 - waiting for change day.
// 06May99 B Beers         S0053125: Marshaled input stream exceptions to threads waiting for input.
// 26May99 B Beers         S0053449: Improved object lifecycle management.
// 01Jun99 B Beers         S0055544: Moved string literals into resource file.
// 07Jul99 B Beers         S0050819: NLS changes stage 2 - simple support.
// 09Jul99 B Beers         S0050819: NLS changes stage 3 - full support.
// 05Aug99 B Beers         S0062183: Allow narrow to ConnectionPointContainer.
// 10Aug99 B Beers         S0060349: NLS transcoding optimization.
// 30Aug99 B Beers         S0063163: Map system exceptions.
// 15Dec99 B Beers         S0067137: Ensure client will close when server does.
// 07Jan00 B Beers         S0073387: Prevent NullPointerException when closing.
// 11May00 B Beers         S0084804: Add support for Cp1047.
// 05Jul00 B Beers         S0089446: Disable Nagle algorithm so small messages will be sent promptly.
// 06Sep00 B Beers         S0094267: Enclose thread creation in privileged action blocks.
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 02Mar01 B Beers         S0110700: Mangle objectID correctly for incoming local object.
// 16Mar01 B Beers         S0110897: Improve performance for large parameters.
// 19Apr01 B Beers         S0113357: Fix COMM_FAILURE on release call from garbage collector.
// 01May01 B Beers         S0115551: Insert type code into Any object in parmsToAny.
// 11Jul01 B Beers         S0121077: Fix COMM_FAILURE on close call from garbage collector.
// 26Jul01 B Beers         S0122328: Implement PE_REQS_REQS.
// 03Aug01 B Beers         S0123003: Implement SERVERTRANSCODE reply handling.
// 17Aug01 B Beers         S0124013: Enclose thread.setDaemon in privileged action blocks.
// 23Aug01 B Beers         S0124519: Split socket driver code into SocketAccessMethod.
// 19Sep01 B Beers         S0127146: Implement org.omg.CORBA.Object._non_existent.
// 02Oct01 B Beers         S0127732: Fix Invalid object exception when driving Release while object is being destroyed.
// 12Oct01 B Beers         S0129352: Remove connection sharing logic.
// 15Oct01 B Beers         S0129403: Check class ID for length to prevent hang.
// 10Dec01 B Beers         S0134375: Fix NullPointerException in method calls with a null object reference.
// 11Dec01 B Beers         S0134625: Fix NullPointerException caused when server crashes authenticating client.
// 14Feb02 B Beers         S0139873: Added User Service authorization.
// 15Feb02 B Beers         S0140418: Detected stale connection to shared server.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
// 30Apr02 B Beers         S0149671: Resolve some garbage collection problems.
// 24Oct02 B Beers         S0169574: Added password generation support.
// 19Nov02 B Beers         S0169574: Revise password generation support.
// 11Jul03 B Beers         S0211000: Remove thread create from DelegateImpl.finalize().
// 13Oct03 B Beers         S0149671: Resolve some more garbage collection problems.
// 20Oct03 B Beers         S0223292: Enclose access to remoteObjectTable in synchronized code.
// 11Nov03 B Beers         S0223292: Fix deadlock caused by previous fix.
// 12Nov03 B Beers         S0227109: Remove stub finalizers.
// 16Dec03 B Beers         S0227513: Log unhandled exceptions.
// 29Jan04 B Beers         S0233579: Use logger associated with user that created factory.
/**
 * @author Biff Beers
 */
public class BridgeProtocolEngine implements ProtocolEngineInterface
{
     public BridgeProtocolEngine(BrgOrb orb,AccessMethodInterface accessMethod)
     {
          this.orb = orb;
          this.accessMethod = accessMethod;
          init();
     }

     public BridgeProtocolEngine(BrgOrb orb,Properties properties)
     {
          this.orb = orb;
          accessMethod = new SocketAccessMethod(properties);
          init();
     }

     public BridgeProtocolEngine(BrgOrb orb,Properties properties,Socket socket,int clientID)
     {
          this.orb = orb;
          accessMethod = new SocketAccessMethod(properties,socket,clientID);
          init();
     }

     private void init()
     {
          accessMethod.setProtocolEngine(this);
          clientID = accessMethod.connect();
          remoteObjectTable = new HashMap();
     }

     public Properties getProperties()
     {
          return accessMethod.getProperties();
     }
     
     public void setLogger(Logger logger)
     {
          accessMethod.setLogger(logger);
     }

     public Logger getLogger()
     {
          return accessMethod.getLogger();
     }

     public org.omg.CORBA.Object newRemoteObject(String classID)
     {
          if (classID.length() != 36)
          {
               // {0}: The class ID must contain exactly 36 characters: {1}
               String messageIndex = "brg.B012.ex.msg";
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex,classID);
               throw new INV_OBJREF(message);
          }

          int objectID = accessMethod.newObject(classID);

          if (objectID == 0)
          {
               return null;
          }

          final Integer key = getKey(objectID);
          final GenericObject[] objectWrapper = new GenericObject[1];
          final BridgeProtocolEngine engine = this;

          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    WeakReference ref = (WeakReference)remoteObjectTable.get(key);
                    DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();
                    GenericObject object = null;

                    if (delegate == null)
                    {
                         delegate = new DelegateImpl(orb,engine,key);
                         object = orb.getGenericObject();
                         object._set_delegate(delegate);
                         delegate.setWrapper(object);
                         ref = new WeakReference(delegate);
                         remoteObjectTable.put(key,ref);
                    }
                    else
                    {
                         object = delegate.getWrapper();
                    }

                    delegate.incrementReferenceCount();
                    objectWrapper[0] = object;
               }
          };

          lock.executeExclusive(task);
          return objectWrapper[0];
     }

     public boolean isConnected()
     {
          return accessMethod.isConnected();
     }

     public void generatePassword(String userName,StringHolder qualUserNameHolder,StringHolder genPasswordHolder)
     {
          accessMethod.generatePassword(userName,qualUserNameHolder,genPasswordHolder);
     }

     public void close(boolean waitForCompletion)
     {
          accessMethod.close();
     }

     public int objectToObjectID(final org.omg.CORBA.Object obj)
     {
          if (obj == null)
          {
               return 0;
          }

          Delegate delegate = ((ObjectImpl)obj)._get_delegate();

          if (!(delegate instanceof DelegateBase))
          {
               // {0}: BridgeProtocolEngine cannot handle remote object references using the delegate class {1}.
               String messageIndex = "brg.B002.ex.msg";
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex,delegate.getClass().getName());
               throw new BAD_PARAM(message);
          }

          final DelegateBase delegateBase = (DelegateBase)delegate;
          Integer objectId = (Integer)delegateBase.getEngineInfo(this);
          int oid = 0;

          if (delegate instanceof DelegateImpl)
          {
               // When sending an object ID that represents an object
               // that is hosted by the peer as an input argument
               // of a method or an output parameter on an outcall,
               // set the high order bit of the object ID.
               //----------------------------------------------------
               oid = objectId.intValue() | 0x80000000;
          }
          else
          {
               if (objectId == null)
               {
                    final Integer[] objectIdWrapper = new Integer[1];
                    final BridgeProtocolEngine engine = this;

                    Runnable task = new Runnable()
                    {
                         // do nothing within run() that could lock accessMethod
                         public void run()
                         {
                              if (objectIDCounter > 0x000FFFFF)
                              {
                                   // {0}: The bridge protocol engine cannot support more than {1} local objects in one session.
                                   String messageIndex = "brg.B003.ex.msg";
                                   String messagePattern = RB.getStringResource(messageIndex);
                                   String message = Message.format(messagePattern,messageIndex,Integer.toString(0x000FFFFF));
                                   throw new IMP_LIMIT(message);
                              }

                              Integer ID = getKey(objectIDCounter++|clientID);
                              delegateBase.setEngineInfo(engine,ID);

                              if (localObjectTable == null)
                              {
                                   localObjectTable = new HashMap();
                              }

                              localObjectTable.put(ID,obj);
                              objectIdWrapper[0] = ID;
                         }
                    };

                    lock.executeExclusive(task);
                    objectId = objectIdWrapper[0];
               }

               oid = objectId.intValue();
          }

          return oid;
     }

     public org.omg.CORBA.Object objectIDToObject(int objectID)
     {
          if (objectID == 0)
          {
               return null;
          }

          final ObjectImpl[] objectWrapper = new ObjectImpl[1];

          if (objectID > 0)
          {
               final Integer key = getKey(objectID);
               final BridgeProtocolEngine engine = this;

               Runnable task = new Runnable()
               {
                    // do nothing within run() that could lock accessMethod
                    public void run()
                    {
                         // A remote object
                         WeakReference ref = (WeakReference)remoteObjectTable.get(key);
                         DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();
                         ObjectImpl obj = null;

                         if (delegate == null)
                         {
                              delegate = new DelegateImpl(orb,engine,key);
                              obj = orb.getGenericObject();
                              obj._set_delegate(delegate);
                              delegate.setWrapper((GenericObject)obj);
                              ref = new WeakReference(delegate);
                              remoteObjectTable.put(key,ref);
                         }
                         else
                         {
                              obj = delegate.getWrapper();
                         }

                         objectWrapper[0] = obj;
                    }
               };

               lock.executeExclusive(task);
          }
          else
          {
               // a local object
               final Integer key = getKey(objectID&0x7FFFFFFF);

               Runnable task = new Runnable()
               {
                    // do nothing within run() that could lock accessMethod
                    public void run()
                    {
                         if (localObjectTable == null)
                         {
                              // {0}: Invalid object specified to bridge protocol engine.
                              String messageIndex = "brg.B006.ex.msg";
                              String messagePattern = RB.getStringResource(messageIndex);
                              String message = Message.format(messagePattern,messageIndex);
                              throw new OBJECT_NOT_EXIST(message);
                         }

                         ObjectImpl reference = (ObjectImpl)localObjectTable.get(key);

                         if (reference == null)
                         {
                              // {0}: Invalid object specified to bridge protocol engine.
                              String messageIndex = "brg.B007.ex.msg";
                              String messagePattern = RB.getStringResource(messageIndex);
                              String message = Message.format(messagePattern,messageIndex);
                              throw new OBJECT_NOT_EXIST(message);
                         }

                         objectWrapper[0] = reference;
                    }
               };

               lock.executeNonExclusive(task);
          }

          return objectWrapper[0];
     }

     public int newLocalObject(String classID)
     {
          int l = classID.length();
          byte[] poaID = new byte[l];

          for (int i=0;i<l;i++)
          {
               poaID[i] = (byte)classID.charAt(i);
          }

          ActiveObjectMapKey activeObjectMapKey = orb.getActiveObjectMapKey();
          activeObjectMapKey.setValue(poaID);
          Map activeObjectMap = orb.getActiveObjectMap();
          ObjectImpl reference = null;

          synchronized (activeObjectMap)
          {
               reference = (ObjectImpl)activeObjectMap.get(activeObjectMapKey);
               orb.returnActiveObjectMapKey(activeObjectMapKey);
          }

          if (reference == null)
          {
               // {0}: An unrecognized class ID was specified to the bridge protocol engine: {1}
               String messageIndex = "brg.B008.ex.msg";
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               throw new INV_OBJREF(message);
          }

          DelegateBase delegate = (DelegateBase)reference._get_delegate();
          delegate.incrementReferenceCount();
          return objectToObjectID(reference);
     }

     public void remoteObjectDestructor(final int objectID)
     {
          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    Integer key = getKey(objectID);
                    remoteObjectTable.remove(key);
               }
          };

          lock.executeExclusive(task);
     }

     public boolean interfaceInLocalObject(int objectID,String interfaceID)
     {
          ObjectImpl reference = (ObjectImpl)objectIDToObject(objectID|0x80000000);
          String[] ids = reference._ids();

          if (ids != null)
          {
               interfaceID = interfaceID.toLowerCase();

               for (int i=0;i<ids.length;i++)
               {
                    if (ids[i].toLowerCase().indexOf(interfaceID) >= 0)
                    {
                         return true;
                    }
               }
          }

          return false;
     }

     public void dispatchRequest(final int objectID,String interfaceID,String name,int replyID,ParameterHandleInterface parmHandle)
     {
          ObjectImpl target = (ObjectImpl)objectIDToObject(objectID|0x80000000);

          if (target == null)
          {
               // {0}: Invalid object specified to bridge protocol engine.
               String messageIndex = "brg.B045.ex.msg";
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               Exception ex = new OBJECT_NOT_EXIST(message);
               accessMethod.sendSystemError(objectID,interfaceID,replyID,ex);
          }
          else
          {
               if (name.equals("CastToInterface"))
               {
                    String targetID = interfaceID.toLowerCase();
                    String[] ids = target._ids();
                    boolean found = false;

                    if (ids != null)
                    {
                         for (int i=0;i<ids.length;i++)
                         {
                              if (ids[i].toLowerCase().indexOf(targetID) >= 0)
                              {
                                   found = true;
                                   break;
                              }
                         }
                    }

                    Any result = orb.create_any();
                    result.insert_boolean(found);
                    accessMethod.sendReply(objectID,interfaceID,name,replyID,null,result);
               }
               else if (name.equals("AddRef"))
               {
                    DelegateBase delegate = (DelegateBase)target._get_delegate();
                    int refCount = delegate.incrementReferenceCount();
                    Any result = orb.create_any();
                    result.insert_long(refCount);
                    accessMethod.sendReply(objectID,interfaceID,name,replyID,null,result);
               }
               else if (name.equals("Release"))
               {
                    final DelegateBase delegate = (DelegateBase)target._get_delegate();
                    int refCount = delegate.decrementReferenceCount();
                    Any result = orb.create_any();
                    result.insert_long(refCount);
                    accessMethod.sendReply(objectID,interfaceID,name,replyID,null,result);

                    if (refCount == 0)
                    {
                         Runnable task = new Runnable()
                         {
                              // do nothing within run() that could lock accessMethod
                              public void run()
                              {
                                   Integer key = getKey(objectID);
                                   localObjectTable.remove(key);

                                   if (delegate.isTransient())
                                   {
                                        RunnableQueue queue = delegate.getQueue();
                                        queue.shutDown(false,true);
                                   }
                              }
                         };

                         lock.executeExclusive(task);
                    }
               }
               else
               {
                    DelegateBase delegate = (DelegateBase)target._get_delegate();
                    RunnableQueue queue = delegate.getQueue();
                    ServerRequestImpl serverRequest = getServerRequest();
                    serverRequest.setEngine(this);
                    serverRequest.setAccessMethod(accessMethod);
                    serverRequest.setOperation(name);
                    serverRequest.setParameters(parmHandle);
                    serverRequest.setServant(target);
                    serverRequest.setObjectID(objectID);
                    serverRequest.setInterfaceID(interfaceID);
                    serverRequest.setReplyID(replyID);
                    serverRequest.setOneWay(false);
                    serverRequest.setLogger(accessMethod.getLogger());
                    parmHandle.incrementReferenceCount();
                    queue.enqueueTask(serverRequest);
               }
          }
     }

     public void dispatchEvent(final int objectID,final String eventInterfaceID,String eventName,ParameterHandleInterface parmHandle)
     {
          final Set[] listenerSetWrapper = new Set[1];

          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    WeakReference ref = (WeakReference)remoteObjectTable.get(getKey(objectID));
                    DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                    if (delegate == null)
                    {
                         return;
                    }

                    Map cpTable = delegate.getConnectionPointTable();
                    WeakReference cpRef = (WeakReference)cpTable.get(eventInterfaceID);
                    ConnectionPointDelegateImpl cpDelegate = (cpRef==null)?null:(ConnectionPointDelegateImpl)cpRef.get();

                    if (cpDelegate == null)
                    {
                         return;
                    }

                    // get a clone of the listenerSet so we can
                    // iterate through it outside the critical section.
                    //-------------------------------------------------
                    Map listenerTable = cpDelegate.getListenerTable();
                    Set tempSet = listenerTable.entrySet();
                    Set listenerSet = new HashSet(tempSet.size());
                    listenerSet.addAll(tempSet);
                    listenerSetWrapper[0] = listenerSet;
               }
          };

          lock.executeNonExclusive(task);
          Set listenerSet = listenerSetWrapper[0];

          if (listenerSet == null)
          {
               return;
          }

          Iterator listenerIterator = listenerSet.iterator();

          while (listenerIterator.hasNext())
          {
               ObjectImpl listener = (ObjectImpl)((Map.Entry)listenerIterator.next()).getValue();
               DelegateBase listenerDelegate = (DelegateBase)listener._get_delegate();
               RunnableQueue queue = listenerDelegate.getQueue();
               ServerRequestImpl serverRequest = getServerRequest();
               serverRequest.setEngine(this);
               serverRequest.setAccessMethod(accessMethod);
               serverRequest.setOperation(eventName);
               serverRequest.setParameters(parmHandle);
               serverRequest.setServant(listener);
               serverRequest.setOneWay(true);
               serverRequest.setLogger(accessMethod.getLogger());
               parmHandle.incrementReferenceCount();
               queue.enqueueTask(serverRequest);
          }
     }

     public void accessMethodExit()
     {
          stopDispatchQueues();
     }

     public BrgOrb getORB()
     {
          return orb;
     }

     public static int typeCodeToIomType(TypeCode tc,IntHolder dim)
     {
          TCKind kind = tc.kind();

          switch (kind.value())
          {
               case TCKind._tk_boolean:return IT_BOOLEAN;
               case TCKind._tk_octet:return IT_OCTET;
               case TCKind._tk_short:return IT_SHORT;
               case TCKind._tk_long:return IT_LONG;
               case TCKind._tk_float:return IT_FLOAT;
               case TCKind._tk_double:return IT_DOUBLE;
               case TCKind._tk_string:return IT_STRING;
               case TCKind._tk_enum:return IT_ENUM;
               case TCKind._tk_objref:return IT_OBJECT;

               // The code generated by visibroker for datetime data types
               // is buggy. Sometimes it is a _tk_ulonglong (what it
               // should be) and sometimes it is a _tk_longlong.
               //---------------------------------------------------------
               case TCKind._tk_longlong:
               case TCKind._tk_ulonglong:return IT_DATETIME;

               case TCKind._tk_sequence:
               {
                    dim.value++;
                    TypeCode subType = null;

                    try
                    {
                         subType = tc.content_type();
                    }
                    catch (BadKind e)
                    {
                         // this shouldn't happen
                         String cl = tc.getClass().getName();
                         throw new INTERNAL(cl + ".content_type() threw an exception when it shouldn't have");
                    }

                    return typeCodeToIomType(subType,dim);
               }

               case TCKind._tk_alias:
               {
                    TypeCode subType = null;

                    try
                    {
                         subType = tc.content_type();
                    }
                    catch (BadKind e)
                    {
                         String cl = tc.getClass().getName();
                         throw new INTERNAL(cl + ".content_type() threw an exception when it shouldn't have");
                     }

                    return typeCodeToIomType(subType,dim);
               }

               case TCKind._tk_struct:
               {
                    String name = null;

                    try
                    {
                         name = tc.name();
                    }
                    catch (BadKind e)
                    {
                         String cl = tc.getClass().getName();
                         throw new INTERNAL(cl + ".name() threw an exception when it shouldn't have");
                     }

                    if (name.equals("UUID"))
                    {
                         return IT_UUID;
                    }

                    // otherwise fall through and throw exception
               }

               default:
               {
                    // this shouldn't happen
                    throw new INTERNAL("Unexpected type code");
               }
          }
     }

     // A CORBA datetime value is the number of 100 nanosecond units (a decimicrosecond?)
     // since the beginning of the day on 15Oct1582. A SAS datetime value is the number of
     // seconds since the beginning of the day on 01Jan1960. Neither value implies a
     // particular timezone nor is either value adjusted for daylight savings time.
     // This constant is the number of seconds between midnight on 15Oct1582 and midnight
     // on 01Jan1960. 78 is the number of days between 15Oct1582 and 01Jan1583. 91 is the
     // number of leap days between 01Jan1583 and 01Jan1960.
     //-----------------------------------------------------------------------------------
     private static final double timeConversionFactor = ( (((1960. - 1583.) * 365.) + 78. + 91.) * (24. * 3600.) );

     // Convert a CORBA datetime value to a SAS datetime value.
     //--------------------------------------------------------
     public static double corbaToSas(long corba)
     {
          double corbaInSecs = corba / 10000000.; // Time in seconds since midnight, 15Oct1582
          double sasTime = corbaInSecs - timeConversionFactor; // Time in seconds since midnight, 01Jan1960
          return sasTime;
     }

     public static long sasToCorba(double sas)
     {
          double corbaInSecs = sas + timeConversionFactor; // Time in seconds since midnight, 15Oct1582
          long corbaTime = (long)(corbaInSecs * 10000000.); // Time in deci-micro-seconds since midnight, 15Oct1582
          return corbaTime;
     }

     public static void stringToUuid(String uuid,org.omg.CORBA.portable.OutputStream os)
     {
          os.write_ulong((int)Long.parseLong(uuid.substring(0,8),16));
          os.write_ushort((short)Long.parseLong(uuid.substring(9,13),16));
          os.write_ushort((short)Long.parseLong(uuid.substring(14,18),16));

          for (int i=0;i<2;i++)
          {
               os.write_octet((byte)Long.parseLong(uuid.substring((i*2)+19,(i*2)+21),16));
          }

          for (int i=0;i<6;i++)
          {
               os.write_octet((byte)Long.parseLong(uuid.substring((i*2)+24,(i*2)+26),16));
          }
     }

     public static String uuidToString(org.omg.CORBA.portable.InputStream is)
     {
          StringBuffer uuidBuffer = new StringBuffer();
          dataToString(is.read_ulong(),8,uuidBuffer);
          uuidBuffer.append("-");
          dataToString(((int)is.read_ushort())&0x0000FFFF,4,uuidBuffer);
          uuidBuffer.append("-");
          dataToString(((int)is.read_ushort())&0x0000FFFF,4,uuidBuffer);
          uuidBuffer.append("-");

          for (int i=0;i<2;i++)
          {
               dataToString(((int)is.read_octet())&0x000000FF,2,uuidBuffer);
          }

          uuidBuffer.append("-");

          for (int i=2;i<8;i++)
          {
               dataToString(((int)is.read_octet())&0x000000FF,2,uuidBuffer);
          }

          return uuidBuffer.toString();
     }

     private static void dataToString(int data,int length,StringBuffer hexBuffer)
     {
          String hex = Integer.toHexString(data);

          for (int i=length-hex.length();i>0;i--)
          {
               hexBuffer.append("0");
          }

          hexBuffer.append(hex);
     }

     public static final String CONNECTION_POINT_CONTAINER_IID = "b196b284-bab4-101a-b69c-00aa00241d07";
     public static final int IT_BOOLEAN = 0;
     public static final int IT_BOOLEAN_1D_ARRAY = 1;
     public static final int IT_BOOLEAN_2D_ARRAY = 2;
     public static final int IT_OCTET = 3;
     public static final int IT_OCTET_1D_ARRAY = 4;
     public static final int IT_OCTET_2D_ARRAY = 5;
     public static final int IT_SHORT = 6;
     public static final int IT_SHORT_1D_ARRAY = 7;
     public static final int IT_SHORT_2D_ARRAY = 8;
     public static final int IT_LONG = 9;
     public static final int IT_LONG_1D_ARRAY = 10;
     public static final int IT_LONG_2D_ARRAY = 11;
     public static final int IT_FLOAT = 12;
     public static final int IT_FLOAT_1D_ARRAY = 13;
     public static final int IT_FLOAT_2D_ARRAY = 14;
     public static final int IT_DOUBLE = 15;
     public static final int IT_DOUBLE_1D_ARRAY = 16;
     public static final int IT_DOUBLE_2D_ARRAY = 17;
     public static final int IT_STRING = 18;
     public static final int IT_STRING_1D_ARRAY = 19;
     public static final int IT_STRING_2D_ARRAY = 20;
     public static final int IT_ENUM = 21;
     public static final int IT_ENUM_1D_ARRAY = 22;
     public static final int IT_ENUM_2D_ARRAY = 23;
     public static final int IT_OBJECT = 24;
     //public static final int IT_OBJECT_1D_ARRAY = 25;
     //public static final int IT_OBJECT_2D_ARRAY = 26;
     public static final int IT_DATETIME = 27;
     public static final int IT_DATETIME_1D_ARRAY = 28;
     public static final int IT_DATETIME_2D_ARRAY = 29;
     public static final int IT_UUID = 30;
     public static final int IT_UUID_1D_ARRAY = 31;
     public static final int IT_UUID_2D_ARRAY = 32;
     private BrgOrb orb /* = null */;
     private AccessMethodInterface accessMethod /* = null */;
     private NonExclusiveLock lock = NonExclusiveLock.getInstance();
     private Map remoteObjectTable /* = null */;
     private Map localObjectTable /* = null */;
     private int clientID /* = 0 */;
     private int objectIDCounter /* = 0 */;

     protected boolean interfaceInRemoteObject(Integer objectID,String interfaceID)
     {
          return accessMethod.interfaceInObject(objectID.intValue(),interfaceID);
     }

     protected synchronized boolean doesRemoteObjectExist(Integer objectID)
     {
          return remoteObjectTable.containsKey(objectID);
     }

     protected void requestByName(Integer objectID,String interfaceID,String name,NVList parameters,Any result)
     {
          // Reference counting for outcalls:
          // For local objects, incr ref count for input args for the
          // duration of the call only. Don't worry about local objects
          // received as output args or return values. For remote objects,
          // incr ref count for reference received as output parameters or
          // return values. Don't worry about references sent as input args.
          //----------------------------------------------------------------
          int parameterCount = (parameters == null) ? 0 : parameters.count();

          for (int i=0;i<parameterCount;i++)
          {
               NamedValue nv = null;

               try
               {
                    nv = parameters.item(i);
               }
               catch (Bounds e)
               {
                    // this shouldn't happen
                    StringBuffer msgBuffer = new StringBuffer(parameters.getClass().getName());
                    msgBuffer.append(".item() threw a(n) ");
                    msgBuffer.append(e.getClass().getName());
                    msgBuffer.append(" when it shouldn't have.");
                    throw new INTERNAL(msgBuffer.toString());
               }

               if (nv.flags() != ARG_OUT.value)
               {
                    Any value = nv.value();
                    TypeCode tc = value.type();
                    IntHolder dim = new IntHolder();
                    int iomType = typeCodeToIomType(tc,dim);

                    if (iomType + dim.value == IT_OBJECT)
                    {
                         ObjectImpl obj = (ObjectImpl)value.extract_Object();

                         if (obj != null)
                         {
                              DelegateBase delegate = (DelegateBase)obj._get_delegate();

                              if (!(delegate instanceof DelegateImpl))
                              {
                                   delegate.incrementReferenceCount();
                              }
                         }
                    }
               }
          }

          accessMethod.sendRequest(objectID.intValue(),interfaceID,name,parameters,result);
          Delegate parent = null;

          for (int i=0;i<parameterCount;i++)
          {
               NamedValue nv = null;

               try
               {
                    nv = parameters.item(i);
               }
               catch (Bounds e)
               {
                    // this shouldn't happen
                    StringBuffer msgBuffer = new StringBuffer(parameters.getClass().getName());
                    msgBuffer.append(".item() threw a(n) ");
                    msgBuffer.append(e.getClass().getName());
                    msgBuffer.append(" when it shouldn't have.");
                    throw new INTERNAL(msgBuffer.toString());
               }

               int flags = nv.flags();

               if (flags != ARG_OUT.value)
               {
                    Any value = nv.value();
                    TypeCode tc = value.type();
                    IntHolder dim = new IntHolder();
                    int iomType = typeCodeToIomType(tc,dim);

                    if (iomType + dim.value == IT_OBJECT)
                    {
                         ObjectImpl obj = (ObjectImpl)value.extract_Object();

                         if (obj != null)
                         {
                              DelegateBase delegate = (DelegateBase)obj._get_delegate();

                              if (!(delegate instanceof DelegateImpl))
                              {
                                   delegate.decrementReferenceCount();
                              }
                         }
                    }
               }

               if (flags != ARG_IN.value)
               {
                    Any value = nv.value();
                    TypeCode tc = value.type();
                    IntHolder dim = new IntHolder();
                    int iomType = typeCodeToIomType(tc,dim);

                    if (iomType + dim.value == IT_OBJECT)
                    {
                         ObjectImpl obj = (ObjectImpl)value.extract_Object();

                         if (obj != null)
                         {
                              DelegateBase delegate = (DelegateBase)obj._get_delegate();

                              if (delegate instanceof DelegateImpl)
                              {
                                   delegate.incrementReferenceCount();

                                   // stash a reference to the parent in this delegate to keep the
                                   // garbage collector from collecting the parent while there are
                                   // still outstanding references to the child.
                                   //-------------------------------------------------------------
                                   if (parent == null)
                                   {
                                        ObjectImpl parentObj = (ObjectImpl)objectIDToObject(objectID.intValue());
                                        parent = parentObj._get_delegate();
                                   }

                                   delegate.addParent(parent);
                              }
                         }
                    }
               }
          }

          if (result != null)
          {
               TypeCode tc = result.type();
               IntHolder dim = new IntHolder();
               int iomType = typeCodeToIomType(tc,dim);

               if (iomType + dim.value == IT_OBJECT)
               {
                    ObjectImpl obj = (ObjectImpl)result.extract_Object();

                    if (obj != null)
                    {
                         DelegateBase delegate = (DelegateBase)obj._get_delegate();

                         if (delegate instanceof DelegateImpl)
                         {
                              delegate.incrementReferenceCount();

                              // stash a reference to the parent in this delegate to keep the
                              // garbage collector from collecting the parent while there are
                              // still outstanding references to the child.
                              //-------------------------------------------------------------
                              if (parent == null)
                              {
                                   ObjectImpl parentObj = (ObjectImpl)objectIDToObject(objectID.intValue());
                                   parent = parentObj._get_delegate();
                              }

                              delegate.addParent(parent);
                         }
                    }
               }

          }
     }

     protected boolean initializeEventSupport(final Integer objectID)
     {
          final int[] eventSupportWrapper = {0};

          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    WeakReference ref = (WeakReference)remoteObjectTable.get(objectID);
                    DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                    if (delegate == null)
                    {
                         throw new OBJECT_NOT_EXIST();
                    }

                    eventSupportWrapper[0] = delegate.getEventSupport();
               }
          };

          lock.executeNonExclusive(task);

          if (eventSupportWrapper[0] == DelegateImpl.EVENTS_DUNNO)
          {
               final String[] events = accessMethod.eventsFromObjectID(objectID.intValue());
               eventSupportWrapper[0] = (events == null) ? DelegateImpl.EVENTS_NO : DelegateImpl.EVENTS_YES;

               task = new Runnable()
               {
                    // do nothing within run() that could lock accessMethod
                    public void run()
                    {
                         //start over in case things changed while we were away
                         //----------------------------------------------------
                         WeakReference ref = (WeakReference)remoteObjectTable.get(objectID);
                         DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                         if (delegate == null)
                         {
                              throw new OBJECT_NOT_EXIST();
                         }

                         int tempEventSupport = delegate.getEventSupport();

                         if (tempEventSupport == DelegateImpl.EVENTS_DUNNO)
                         {
                              delegate.setEventSupport(eventSupportWrapper[0]);
                              delegate.setEventList(events);
                         }
                    }
               };

               lock.executeExclusive(task);
          }

          return (eventSupportWrapper[0] == DelegateImpl.EVENTS_YES);
     }

     protected org.omg.CORBA.Object getConnectionPoint(final Integer objectID,final String eventInterfaceID)
     {
          final GenericObject[] cpWrapper = new GenericObject[1];
          final boolean[] supportedWrapper = {true};

          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    WeakReference ref = (WeakReference)remoteObjectTable.get(objectID);
                    DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                    if (delegate == null)
                    {
                         throw new OBJECT_NOT_EXIST();
                    }

                    // now make sure this object supports this event interface
                    String[] eventList = delegate.getEventList();
                    boolean supported = false;

                    for (int i=0;i<eventList.length;i++)
                    {
                         if (eventList[i].indexOf(eventInterfaceID) >= 0)
                         {
                              supported = true;
                              break;
                         }
                    }

                    if (!supported)
                    {
                         supportedWrapper[0] = false;
                         return;
                    }

                    // find the connection point delegate
                    // create a new one if necessary
                    Map cpTable = delegate.getConnectionPointTable();

                    if (cpTable != null)
                    {
                         WeakReference cpRef = (WeakReference)cpTable.get(eventInterfaceID);
                         ConnectionPointDelegateImpl cpDelegate = (cpRef==null)?null:(ConnectionPointDelegateImpl)cpRef.get();

                         if (cpDelegate != null)
                         {
                              cpDelegate.addParent(delegate);
                              cpWrapper[0] = cpDelegate.getWrapper();
                         }
                    }
               }
          };

          lock.executeNonExclusive(task);

          if (!supportedWrapper[0])
          {
               return null;
          }

          if (cpWrapper[0] != null)
          {
               return cpWrapper[0];
          }

          final BridgeProtocolEngine engine = this;

          task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    WeakReference ref = (WeakReference)remoteObjectTable.get(objectID);
                    DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                    if (delegate == null)
                    {
                         throw new OBJECT_NOT_EXIST();
                    }

                    // find the connection point delegate
                    // create a new one if necessary
                    Map cpTable = delegate.getConnectionPointTable();

                    if (cpTable == null)
                    {
                         cpTable = new HashMap();
                         delegate.setConnectionPointTable(cpTable);
                    }

                    WeakReference cpRef = (WeakReference)cpTable.get(eventInterfaceID);
                    ConnectionPointDelegateImpl cpDelegate = (cpRef==null)?null:(ConnectionPointDelegateImpl)cpRef.get();
                    GenericObject cp = null;

                    if (cpDelegate == null)
                    {
                         cpDelegate = new ConnectionPointDelegateImpl(orb,engine,objectID,eventInterfaceID);
                         cp = orb.getGenericObject();
                         cp._set_delegate(cpDelegate);
                         cpDelegate.setWrapper(cp);
                         cpRef = new WeakReference(cpDelegate);
                         cpTable.put(eventInterfaceID,cpRef);
                    }
                    else
                    {
                         cp = cpDelegate.getWrapper();
                    }

                    cpDelegate.addParent(delegate);
                    cpWrapper[0] = cp;
               }
          };

          lock.executeExclusive(task);
          return cpWrapper[0];
     }

     protected int adviseConnectionPoint(final Integer objectID,final String eventInterfaceID,final org.omg.CORBA.Object listener)
     {
          final int[] handle = {0};
          final boolean[] sendAdvise = {false};

          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    WeakReference ref = (WeakReference)remoteObjectTable.get(objectID);
                    DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                    if (delegate == null)
                    {
                         throw new OBJECT_NOT_EXIST();
                    }

                    Map cpTable = delegate.getConnectionPointTable();
                    WeakReference cpRef = (WeakReference)cpTable.get(eventInterfaceID);
                    ConnectionPointDelegateImpl cpDelegate = (cpRef==null)?null:(ConnectionPointDelegateImpl)cpRef.get();

                    if (cpDelegate == null)
                    {
                         // somebody dropped a reference to a CP without calling unadvise
                         ObjectImpl cp = (ObjectImpl)getConnectionPoint(objectID,eventInterfaceID);
                         cpDelegate = (ConnectionPointDelegateImpl)cp._get_delegate();
                    }

                    handle[0] = cpDelegate.getNextHandle();
                    Map listenerTable = cpDelegate.getListenerTable();

                    if (listenerTable.isEmpty())
                    {
                         sendAdvise[0] = true;
                    }

                    DelegateBase listenerDelegate = (DelegateBase)((ObjectImpl)listener)._get_delegate();
                    listenerDelegate.addParent(cpDelegate);
                    int  rc = listenerDelegate.incrementReferenceCount();

                    if (rc == 1 && listenerDelegate.isTransient())
                    {
                         RunnableQueue queue = new RunnableQueue("dispatch queue",1);
                         listenerDelegate.setQueue(queue);
                    }

                    listenerTable.put(getKey(handle[0]),listener);
               }
          };

          lock.executeExclusive(task);

          if (sendAdvise[0])
          {
               accessMethod.advise(objectID.intValue(),eventInterfaceID);
          }

          return handle[0];
     }

     protected void unadviseConnectionPoint(final Integer objectID,final String eventInterfaceID,final int handle)
     {
          final boolean[] sendUnadvise = {false};

          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    WeakReference ref = (WeakReference)remoteObjectTable.get(objectID);
                    DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                    if (delegate == null)
                    {
                         return;
                    }

                    Map cpTable = delegate.getConnectionPointTable();
                    WeakReference cpRef = (WeakReference)cpTable.get(eventInterfaceID);
                    ConnectionPointDelegateImpl cpDelegate = (cpRef==null)?null:(ConnectionPointDelegateImpl)cpRef.get();

                    if (cpDelegate == null)
                    {
                         return;
                    }

                    Map listenerTable = cpDelegate.getListenerTable();
                    ObjectImpl listener = (ObjectImpl)listenerTable.remove(getKey(handle));

                    if (listener != null)
                    {
                         DelegateBase listenerDelegate = (DelegateBase)listener._get_delegate();
                         int rc = listenerDelegate.decrementReferenceCount();

                         if (rc == 0 && listenerDelegate.isTransient())
                         {
                              RunnableQueue queue = listenerDelegate.getQueue();
                              queue.shutDown(false,true);
                         }
                    }

                    if (listenerTable.isEmpty())
                    {
                         sendUnadvise[0] = true;
                    }
               }
          };

          lock.executeExclusive(task);

          if (sendUnadvise[0])
          {
               accessMethod.unadvise(objectID.intValue(),eventInterfaceID);
          }
     }

     private void stopDispatchQueues()
     {
          Runnable task = new Runnable()
          {
               // do nothing within run() that could lock accessMethod
               public void run()
               {
                    if (remoteObjectTable != null)
                    {
                         Set refSet = remoteObjectTable.entrySet();
                         Iterator refIterator = refSet.iterator();

                         while (refIterator.hasNext())
                         {
                              WeakReference ref = (WeakReference)((Map.Entry)refIterator.next()).getValue();
                              DelegateImpl delegate = (ref==null)?null:(DelegateImpl)ref.get();

                              if (delegate == null)
                              {
                                   continue;
                              }

                              Map cpTable = delegate.getConnectionPointTable();

                              if (cpTable == null)
                              {
                                   continue;
                              }

                              Set cpSet = cpTable.entrySet();
                              Iterator cpIterator = cpSet.iterator();

                              while (cpIterator.hasNext())
                              {
                                   WeakReference cpRef = (WeakReference)((Map.Entry)cpIterator.next()).getValue();
                                   ConnectionPointDelegateImpl cpDelegate = (ref==null)?null:(ConnectionPointDelegateImpl)cpRef.get();

                                   if (cpDelegate == null)
                                   {
                                        continue;
                                   }

                                   Map listenerTable = cpDelegate.getListenerTable();
                                   Set listenerSet = listenerTable.entrySet();
                                   Iterator listenerIterator = listenerSet.iterator();

                                   while (listenerIterator.hasNext())
                                   {
                                        ObjectImpl listener = (ObjectImpl)((Map.Entry)listenerIterator.next()).getValue();
                                        DelegateBase listenerDelegate = (DelegateBase)listener._get_delegate();
                                        int rc = listenerDelegate.decrementReferenceCount();

                                        if (rc == 0 && listenerDelegate.isTransient())
                                        {
                                             RunnableQueue queue = listenerDelegate.getQueue();

                                             if (queue != null)
                                             {
                                                  queue.shutDown(false);
                                             }
                                        }
                                   }

                                   listenerTable.clear();
                              }

                              cpTable.clear();
                         }

                         remoteObjectTable.clear();
                    }

                    if (localObjectTable != null)
                    {
                         Set refSet = localObjectTable.entrySet();
                         Iterator refIterator = refSet.iterator();

                         while (refIterator.hasNext())
                         {
                              ObjectImpl obj = (ObjectImpl)((Map.Entry)refIterator.next()).getValue();
                              DelegateBase delegate = (DelegateBase)obj._get_delegate();
                              int rc = delegate.decrementReferenceCount();

                              if (rc == 0 && delegate.isTransient())
                              {
                                   RunnableQueue queue = delegate.getQueue();

                                   if (queue != null)
                                   {
                                        queue.shutDown(false);
                                   }
                              }
                         }

                         localObjectTable.clear();
                    }
               }
          };

          lock.executeExclusive(task);
     }

     protected Integer getKey(int key)
     {
          return new Integer(key);
     }

     private ServerRequestImpl getServerRequest()
     {
          return new ServerRequestImpl();
     }

     protected void returnServerRequest(ServerRequestImpl serverRequest)
     {
     }
}