/* $Id$ */
// Copyright (c) 1999 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg;

import java.util.HashMap;
import java.util.Map;
import org.omg.CORBA.Request;
import com.sas.net.brg.orb.BrgOrb;
import com.sas.net.brg.orb.DelegateBase;
//
// History:
//
// Date    Name            Remarks
// 26May99 B Beers         Created.
// 30Jun99 B Beers         S0057860: Improved object lifecycle management (Part II).
// 01Jul99 B Beers         S0058989: Catch unchecked exceptions in events and outcalls.
// 15Dec99 B Beers         S0067137: Ensure client will close when server does.
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
// 30Apr02 B Beers         S0149671: Resolve some garbage collection problems.
// 12Nov03 B Beers         S0227109: Remove stub finalizers.
//
/**
 * @author Biff Beers
 */
public class ConnectionPointDelegateImpl extends DelegateBase
{
     /**
      * Get a delegate for proxies.
      */
     public ConnectionPointDelegateImpl(BrgOrb orb,BridgeProtocolEngine engine,Integer sourceId,String interfaceId)
     {
          super(orb);
          this.engine = engine;
          this.interfaceId = interfaceId;
          setEngineInfo(engine,sourceId);
          listenerTable = new HashMap();
     }

     public boolean is_a(org.omg.CORBA.Object obj,String repository_id)
     {
          return (repository_id != null && repository_id.toUpperCase().equals("DCE:B196B286-BAB4-101A-B69C-00AA00241D07:1"));
     }

     public Request request(org.omg.CORBA.Object obj,String operation)
     {
          return new ConnectionPointRequestImpl(brgOrb,engine,(Integer)getEngineInfo(engine),interfaceId,obj,operation);
     }

     public boolean equals(Object that)
     {
          if (this == that)
          {
               return true;
          }

          if (that instanceof ConnectionPointDelegateImpl)
          {
               ConnectionPointDelegateImpl thatDelegate = (ConnectionPointDelegateImpl)that;

               if (thatDelegate.getSourceId().equals((Integer)getEngineInfo(engine)) && thatDelegate.getInterfaceId().equals(interfaceId))
               {
                    return true;
               }
          }

          return false;
     }

     private BridgeProtocolEngine engine = null;
     private String interfaceId = null;
     private int handleCounter = 0;
     private Map listenerTable = null;

     protected Integer getSourceId()
     {
          return (Integer)getEngineInfo(engine);
     }

     protected String getInterfaceId()
     {
          return interfaceId;
     }

     protected int getNextHandle()
     {
          return handleCounter++;
     }

     protected Map getListenerTable()
     {
          return listenerTable;
     }
}