// Copyright (c) 1999 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg;

import org.omg.CORBA.Any;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.DynamicImplementation;
import org.omg.CORBA.NVList;
import org.omg.CORBA.portable.Delegate;
import org.omg.CORBA.portable.ObjectImpl;
import com.sas.net.brg.orb.BrgOrb;
import com.sas.net.brg.orb.DelegateBase;
import com.sas.net.brg.orb.RequestBase;
import com.sas.text.Message;

//
// History:
//
// Date    Name            Remarks
// 06Jan99 B Beers         Created.
// 26May99 B Beers         S0053449: Improved object lifecycle management.
// 30Jun99 B Beers         S0057860: Improved object lifecycle management (Part II)
// 15Dec99 B Beers         S0067137: Ensure client will close when server does.
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
//
/**
 * @author Biff Beers
 */
public class ConnectionPointRequestImpl extends RequestBase
{
     public ConnectionPointRequestImpl(BrgOrb orb,BridgeProtocolEngine engine,Integer sourceId,String interfaceId,org.omg.CORBA.Object target,String operation)
     {
          super(orb,target,operation);
          this.engine = engine;
          this.sourceId = sourceId;
          this.interfaceId = interfaceId;
     }

     public void invoke()
     {
          String operation = operation();

          if (operation.equals("Advise"))
          {
               NVList arguments = arguments();
               Any any = null;

               try
               {
                    any = arguments.item(0).value();
               }
               catch (org.omg.CORBA.Bounds e)
               {
                    // {0}: Calls to Advise must have 2 arguments.
                    String messageIndex = "brg.B500.ex.msg";
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    throw new BAD_OPERATION(message);
               }

               org.omg.CORBA.Object obj = any.extract_Object();

               if (!(obj instanceof DynamicImplementation))
               {
                    Delegate delegate = ((ObjectImpl)obj)._get_delegate();

                    if (!(delegate instanceof DelegateBase))
                    {
                         // {0}: BridgeProtocolEngine cannot handle remote object references using the delegate class {1}.
                         String messageIndex = "brg.B501.ex.msg";
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex,delegate.getClass().getName());
                         throw new BAD_PARAM(message);
                    }

                    if (((DelegateBase)delegate).getServant() == null)
                    {
                         // {0}: Event listener objects must implement CORBA Dynamic Skeleton Interface.
                         String messageIndex = "brg.B502.ex.msg";
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex);
                         throw new BAD_PARAM(message);
                    }
               }

               int handle = engine.adviseConnectionPoint(sourceId,interfaceId,obj);

               try
               {
                    any = arguments.item(1).value();
               }
               catch (org.omg.CORBA.Bounds e)
               {
                    // {0}: Calls to Advise must have 2 arguments.
                    String messageIndex = "brg.B503.ex.msg";
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    throw new BAD_OPERATION(message);
               }

               any.insert_ulong(handle);
          }
          else if (operation.equals("Unadvise"))
          {
               NVList arguments = arguments();
               Any any = null;

               try
               {
                    any = arguments.item(0).value();
               }
               catch (org.omg.CORBA.Bounds e)
               {
                    // {0}: Calls to Unadvise must have 1 argument.
                    String messageIndex = "brg.B504.ex.msg";
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    throw new BAD_OPERATION(message);
               }

               int handle = any.extract_ulong();
               engine.unadviseConnectionPoint(sourceId,interfaceId,handle);
          }
          else
          {
               // {0}: {1} is not a valid operation on a connection point.
               String messageIndex = "brg.B505.ex.msg";
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex,operation);
               throw new org.omg.CORBA.BAD_OPERATION(message);
          }
     }

     private BridgeProtocolEngine engine = null;
     private Integer sourceId = null;
     private String interfaceId = null;
}