/* $Id$ */
// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg;

import java.util.Map;

import com.sas.net.brg.orb.BrgOrb;
import com.sas.net.brg.orb.DelegateBase;
import com.sas.net.brg.orb.GenericObject;
//
// History:
//
// Date    Name            Remarks
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 23Aug01 B Beers         S0124519: Split socket driver code into SocketAccessMethod.
// 14Feb02 B Beers         S0139873: Added User Service authorization.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
// 24Oct02 B Beers         S0169574: Added password generation support.
// 19Nov02 B Beers         S0169574: Revise password generation support.
// 11Jul03 B Beers         S0211000: Remove thread create from DelegateImpl.finalize().
// 12Nov03 B Beers         S0227109: Remove stub finalizers.
// 29Jan04 B Beers         S0233579: Use logger associated with user that created factory.
/**
 * @author Biff Beers
 */
public class DelegateImpl extends DelegateBase
{
     protected DelegateImpl(BrgOrb brgOrb,BridgeProtocolEngine engine,Integer objectID)
     {
          super(brgOrb,engine);
          this.engine = engine;
          setEngineInfo(engine,objectID);
     }

     public boolean non_existent(org.omg.CORBA.Object obj)
     {
          return !engine.doesRemoteObjectExist((Integer)getEngineInfo(engine));
     }

     public org.omg.CORBA.Request request(org.omg.CORBA.Object obj,String operation)
     {
          String interfaceID = null;

          if (obj == null || obj instanceof GenericObject)
          {
               interfaceID = "00000000-0000-0000-0000-000000000000";
          }
          else
          {
               String[] dces = ((org.omg.CORBA.portable.ObjectImpl)obj)._ids();
               interfaceID = dces[0].substring(4,40);
          }

          return new RequestImpl(brgOrb,engine,obj,(Integer)getEngineInfo(engine),interfaceID,operation);
     }

     protected static final int EVENTS_DUNNO = 0;
     protected static final int EVENTS_YES = 1;
     protected static final int EVENTS_NO = 2;
     private BridgeProtocolEngine engine /* = null */;
     private String[] eventList /* = null */;
     private int eventSupport = EVENTS_DUNNO;
     private Map cpTable /* = null */;

     protected boolean interfaceInObject(org.omg.CORBA.Object obj,String repository_id)
     {
          if (repository_id == null || repository_id.length() < 40)
          {
               return false;
          }

          String interfaceID = repository_id.substring(4,40);

          if (interfaceID.equals(BridgeProtocolEngine.CONNECTION_POINT_CONTAINER_IID))
          {
               // The caller is trying to narrow to ConnectionPointContainer.
               // Allow it if the remote object supports any event interfaces.
               //-------------------------------------------------------------
               return engine.initializeEventSupport((Integer)getEngineInfo(engine));
          }

          return engine.interfaceInRemoteObject((Integer)getEngineInfo(engine),interfaceID);
     }

     protected String[] getEventList()
     {
          return eventList;
     }

     protected void setEventList(String[] eventList)
     {
          this.eventList = eventList;
     }

     protected int getEventSupport()
     {
          return eventSupport;
     }

     protected void setEventSupport(int eventSupport)
     {
          this.eventSupport = eventSupport;
     }

     protected Map getConnectionPointTable()
     {
          return cpTable;
     }

     protected void setConnectionPointTable(Map cpTable)
     {
          this.cpTable = cpTable;
     }
}