// Copyright (c) 2003 by SAS Institute Inc., Cary, NC 27513

package com.sas.net.brg;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * RB loads and manages the resource bundle for the <code>com.sas.net.brg<code> package.
 * RB has convenience methods which allow other classes to fetch a string resource
 * from the package resource bundle, which is created as
 * a <code>PropertyResourceBundle<code> from <code>com.sas.net.brg.Resources<code>, which is defined
 * in the properties file, <code>Resources.properties<code> (or in a locale-specific
 * version such as <code>Resources_de_DE.properties<code> etc.)
 * @version 1.0
 */
public final class RB
	     extends com.sas.util.RBBase
{
  private static RB rb = new RB();

  /**
   * Construct an instance of a RB which accesses the resources
   * in the resource bundle com.sas.net.brg.Resources.
   * Normally, you won't use this constructor. Instead, use the static methods
   * such as {@link #getStringResource(String key)} to get resources.
   */
  public RB()
  {
    super("Resources");  /*I18nOK:LINE*/ // $NON_NLS-1$
  }

  /**
   * Return a string resource from the default resource bundle via
   * getStringResource(classKey + subKey). Consider using
   * {@link #getStringResource(String)} instead of this method, since this method
   * requires runtime String concatenation and transient object
   * allocation. If the classKey and subKey are static finals or literals, the Java compiler
   * will perform the concatenation for a call such as
   * RB.getStringResource(classKey + subKey)
   * at compile time.
   * <p>
   * If possible, you should use {@link #getStringResource(String key)}
   * in cases where the <var>classKey</var> and <var>subKey</var>
   * are string constants, so the compiler can do compile time string
   * concatenation, rather than forcing this method to concatenate.
   * @param classKey a prefix for a resource key, often the class base name
   * with a trailing ".". For example, the com.sas.util.Alarm class will use
   * the class key "Alarm."
   * @param subKey a resource key. The classKey and subKey are concatenated
   * (classKey + subkey) to yield a key and then a resource is fetched
   * from the resource bundle.
   * @return String a string resource obtained via the key.
   * @exception java.util.MissingResourceException if there is no resource
   * for the string (classKey + subKey)
   */
   static public String getStringResource(String classKey, String subKey)
   {
     return rb._getStringResource(classKey + subKey); /*I18nOK:LINE*/
   }

  /**
   * Return a string resource from the default resource bundle via
   * getStringResource(classKey + subKey) and a specific Locale.
   * Consider using
   * {@link #getStringResource(String, Locale)} instead of this method, since this method
   * requires runtime String concatenation and transient object
   * allocation. If the classKey and subKey are static finals or literals, the Java compiler
   * will perform the concatenation for a call such as
   * RB.getStringResource(classKey + subKey, locale)
   * at compile time.
   * <p>
   * If possible, you should use {@link #getStringResource(String key, Locale locale)}
   * in cases where the <var>classKey</var> and <var>subKey</var>
   * are string constants, so the compiler can do compile time string
   * concatenation, rather than forcing this method to concatenate.
   * @param classKey a prefix for a resource key, often the class base name
   * with a trailing ".". For example, the com.sas.util.Alarm class will use
   * the class key "Alarm."
   * @param subKey a resource key. The classKey and subKey are concatenated
   * (classKey + subkey) to yield a key and then a resource is fetched
   * from the resource bundle.
   * @param locale a locale to use when loading resources.
   * If null, use the {@link Locale#getDefault()}
   * @return String a string resource obtained via the key.
   * @exception java.util.MissingResourceException if there is no resource
   * for the string (classKey + subKey)
   */
   static public String getStringResource(String classKey, String subKey, Locale locale)
   {
     return rb._getStringResource(classKey + subKey, locale); /*I18nOK:LINE*/
   }

  /**
   * Return a string resource from the default resource bundle.
   * @param key a resource key
   * @return String a string resource obtained from the key, namely
   * getResources().getString(key)
   * @exception java.util.MissingResourceException if there is no resource
   * for the key.
   */
   static public String getStringResource(String key)
   {
     return rb._getStringResource(key);
   }

  /**
   * Return a string resource from the default resource bundle and locale.
   * @param key a resource key
   * @param locale a locale to use when loading resources.
   * If null, use the {@link Locale#getDefault()}
   * @return String a string resource obtained from the key, namely
   * getResources().getString(key)
   * @exception java.util.MissingResourceException if there is no resource
   * for the key.
   */
   static public String getStringResource(String key, Locale locale)
   {
     return rb._getStringResource(key, locale);
   }

   /**
    * Get the default resource bundle for this package, which is created as
    * a <code>PropertyResourceBundle</code> from
    * <code>com.sas.net.brg.Resources</code>,  which is defined
    * in the properties file, <em>&lt;package-path&gt;</em>/<code>Resources.properties</code>
    * (or in a locale-specific version such as <code>Resources_de_DE.properties</code> etc.)
    * @return the default resource bundle.
    */
   static public ResourceBundle getResources()
   {
     return rb._getResources();
   }
}
