// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg;

import java.util.ArrayList;
import java.util.List;
import org.omg.CORBA.Any;
import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.ARG_OUT;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.Bounds;
import org.omg.CORBA.DynamicImplementation;
import org.omg.CORBA.Environment;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.NVList;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.UNKNOWN;
import org.omg.CORBA.UnknownUserException;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.TypeCodePackage.BadKind;
import com.sas.iom.SASIOMDefs.GenericErrorHelper;
import com.sas.net.brg.orb.BrgOrb;
import com.sas.net.brg.orb.RequestBase;
import com.sas.text.Message;
//
// History:
//
// Date    Name            Remarks
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 16Mar01 B Beers         S0110897: Improve performance for large parameters.
// 23Aug01 B Beers         S0124519: Split socket driver code into SocketAccessMethod.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
// 18Dec02 B Beers         S0151508: Revise xml error message handling.
// 07Jan03 B Beers         S0151508: Revise xml error message handling. (undo)
// 17Jan03 B Beers         S0177362: Throw GenericError when required.
//
/**
 * @author Biff Beers
 */
public class RequestImpl extends RequestBase
{
     protected RequestImpl(BrgOrb orb,BridgeProtocolEngine engine,org.omg.CORBA.Object target,Integer objectID,String interfaceID,String operation)
     {
          super(orb,target,operation);
          this.engine = engine;
          this.objectID = objectID;
          this.interfaceID = interfaceID;
     }

     public void invoke()
     {
          String operation = operation();

          if (operation.startsWith("_get_") || operation.startsWith("_set_"))
          {
               operation = operation.substring(5);
          }
          else if (operation.equals("FindConnectionPoint"))
          {
               StringBuffer cpidBuffer = new StringBuffer();
               NVList arguments = arguments();
               Any any0 = null;
               Any any1 = null;

               try
               {
                    any0 = arguments.item(0).value();
                    any1 = arguments.item(1).value();
               }
               catch (org.omg.CORBA.Bounds e)
               {
                    // {0}: Calls to FindConnectionPoint must have 2 arguments.
                    String messageIndex = "brg.B100.ex.msg";
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    throw new BAD_OPERATION(message);
               }

               InputStream is = any0.create_input_stream();
               String cpid = BridgeProtocolEngine.uuidToString(is);
               org.omg.CORBA.Object cp = engine.getConnectionPoint(objectID,cpid.toLowerCase());
               org.omg.CORBA.TypeCode tc = any1.type();
               any1.insert_Object(cp,tc);
               return;
          }

          NVList arguments = arguments();
          TypeCode returnType = getReturnType();
          Any result = null;

          if (returnType != null)
          {
               result = return_value();
               result.type(returnType);
          }

          try
          {
               engine.requestByName(objectID,interfaceID,operation,arguments,result);
          }
          catch (UnexpectedUserException e)
          {
               Exception r = null;
               String uuid = e.getUUID();

               if ("00000000-0000-0000-0000-000000000000".equals(uuid) || "".equals(uuid))
               {
                    uuid = GenericErrorHelper.id();
               }

               ExceptionList exceptionList = exceptions();
               final int exceptionCount = exceptionList.count();

               for (int i=0;i<exceptionCount;i++)
               {
                    TypeCode etc = null;
                    String id = null;

                    try
                    {
                         etc = exceptionList.item(i);
                         id = etc.id();
                    }
                    catch (Bounds bd)
                    {
                         // this shouldn't happen
                         String cl = exceptionList.getClass().getName();
                         throw new INTERNAL(cl + ".count() returned an incorrect value");
                    }
                    catch (BadKind bk)
                    {
                         // neither should this
                         String cl = etc.getClass().getName();
                         throw new INTERNAL(cl + ".id() threw an exception when it shouldn't have");
                    }

                    if (id != null && ((id.startsWith("DCE:") && id.length() >= 40 && id.substring(4,40).equalsIgnoreCase(uuid)) || id.equals(uuid)))
                    {
                         Any any = getOrb().create_any();
                         OutputStream os = any.create_output_stream();
                         os.write_string(id);
                         os.write_string(e.getMessage());
                         InputStream is = os.create_input_stream();
                         any.read_value(is,etc);
                         r = new UnknownUserException(any);
                         break;
                    }
               }

               if (r == null)
               {
                    String msg = e.getMessage();
                    r = new UNKNOWN(msg);
               }

               Throwable cause = e.getCause();

               if (cause != null)
               {
                    r.initCause(cause);
               }

               Environment environment = env();
               environment.exception(r);
          }
     }

     private static final char[] digits = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f',};
     private BridgeProtocolEngine engine = null;
     private Integer objectID = null;
     private String interfaceID = null;

     private static void hexToChar(byte[] b,int offset,int length,char[] c)
     {
          for (int i=0;i<length;i++)
          {
               byte bi = b[offset+i];
               int ndx = (bi & 0xF0) >> 4;
               c[i*2] = digits[ndx];
               ndx = bi & 0x0F;
               c[i*2+1] = digits[ndx];
          }
     }
}