/* $Id$ */
// Copyright (c) 1999 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.ARG_OUT;
import org.omg.CORBA.Any;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.Bounds;
import org.omg.CORBA.Context;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.NVList;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.ServerRequest;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.portable.ObjectImpl;

import com.sas.net.brg.orb.DelegateBase;
//
// History:
//
// Date    Name            Remarks
// 06Jan99 B Beers         Created.
// 15Apr99 B Beers         S0051136: Handle requests with >1 parameter.
// 26May99 B Beers         S0053449: Improved object lifecycle management.
// 15Dec99 B Beers         S0067137: Ensure client will close when server does.
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 02May01 B Beers         S0115600: Allow client operation without orb.jar.
// 23Aug01 B Beers         S0124519: Split socket driver code into SocketAccessMethod.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
// 20May02 B Beers         S0149943: Fix threading problem at shutdown time.
// 16Dec03 B Beers         S0227513: Log unhandled exceptions.
//
/**
 * @author Biff Beers
 */
public class ServerRequestImpl extends ServerRequest implements Runnable
{
     protected ServerRequestImpl()
     {
          super();
     }

     public String operation()
     {
          return operation;
     }

     /**
      * @deprecated
      */
     public String op_name()
     {
          return operation();
     }

     public void arguments(NVList args)
     {
          if (nvList != null)
          {
               return;
          }

          nvList = args;
          accessMethod.reconcileParameters(parmHandle,args);

          // track the server's reference count for remote objects
          //------------------------------------------------------
          int parameterCount = (args == null) ? 0 : args.count();

          for (int i=0;i<parameterCount;i++)
          {
               NamedValue nv = null;

               try
               {
                    nv = args.item(i);
               }
               catch (Bounds e)
               {
                    StringBuffer msgBuffer = new StringBuffer(args.getClass().getName());
                    msgBuffer.append(".item() threw a(n) ");
                    msgBuffer.append(e.getClass().getName());
                    msgBuffer.append(" exception when it shouldn't have.");
                    throw new INTERNAL(msgBuffer.toString());
               }

               if (nv.flags() != ARG_OUT.value)
               {
                    Any value = nv.value();
                    TypeCode tc = value.type();
                    IntHolder dim = new IntHolder();
                    int iomType = BridgeProtocolEngine.typeCodeToIomType(tc,dim);

                    if (iomType + dim.value == BridgeProtocolEngine.IT_OBJECT)
                    {
                         ObjectImpl obj = (ObjectImpl)value.extract_Object();
                         DelegateBase delegate = (DelegateBase)obj._get_delegate();

                         if (delegate instanceof DelegateImpl)
                         {
                              delegate.incrementReferenceCount();
                         }
                    }
               }
          }
     }

     /**
      * @deprecated
      */
     public void params(org.omg.CORBA.NVList args)
     {
          arguments(args);
     }

     public void set_result(org.omg.CORBA.Any any)
     {
          result = any;
     }

     /**
      * @deprecated
      */
     public void result(org.omg.CORBA.Any any)
     {
          set_result(any);
     }

     public void set_exception(org.omg.CORBA.Any any)
     {
          userException = any;
     }

     /**
      * @deprecated
      */
     public void except(org.omg.CORBA.Any any)
     {
          set_exception(any);
     }

     public Context ctx()
     {
          return null;
     }

     public void run()
     {
          try
          {
               Throwable systemException = null;

               if (servant instanceof org.omg.CORBA.DynamicImplementation)
               {
                    try
                    {
                         ((org.omg.CORBA.DynamicImplementation)servant).invoke(this);
                    }
                    catch (BAD_OPERATION bo)
                    {
                         String oldOp = operation;
                         String newOp = ((parmHandle.isEmpty())?"_get_":"_set_") + operation;
                         setOperation(newOp);

                         try
                         {
                              ((org.omg.CORBA.DynamicImplementation)servant).invoke(this);
                         }
                         catch (Throwable re)
                         {
                              systemException = re;
                         }
                         finally
                         {
                              setOperation(oldOp);
                         }
                    }
                    catch (Throwable e)
                    {
                         systemException = e;
                    }
               }
               else
               {
                    DelegateBase delegate = (DelegateBase)servant._get_delegate();
                    org.omg.PortableServer.DynamicImplementation srv = (org.omg.PortableServer.DynamicImplementation)delegate.getServant();

                    try
                    {
                         srv.invoke(this);
                    }
                    catch (BAD_OPERATION bo)
                    {
                         String oldOp = operation;
                         String newOp = ((parmHandle.isEmpty())?"_get_":"_set_") + operation;
                         setOperation(newOp);

                         try
                         {
                              srv.invoke(this);
                         }
                         catch (Throwable re)
                         {
                              systemException = re;
                         }
                         finally
                         {
                              setOperation(oldOp);
                         }
                    }
                    catch (Throwable e)
                    {
                         systemException = e;
                    }
               }

               // track the server's reference count for remote objects
               // and bump reference count for our own objects
               //------------------------------------------------------
               int parameterCount = (nvList == null) ? 0 : nvList.count();

               for (int i=0;i<parameterCount;i++)
               {
                    NamedValue nv = null;

                    try
                    {
                         nv = nvList.item(i);
                    }
                    catch (Bounds e)
                    {
                         StringBuffer msgBuffer = new StringBuffer(nvList.getClass().getName());
                         msgBuffer.append(".item() threw a(n) ");
                         msgBuffer.append(e.getClass().getName());
                         msgBuffer.append(" exception when it shouldn't have.");
                         throw new INTERNAL(msgBuffer.toString());
                    }

                    Any value = nv.value();
                    TypeCode tc = value.type();
                    IntHolder dim = new IntHolder();
                    int iomType = BridgeProtocolEngine.typeCodeToIomType(tc,dim);

                    if (iomType + dim.value == BridgeProtocolEngine.IT_OBJECT)
                    {
                         ObjectImpl obj = (ObjectImpl)value.extract_Object();
                         DelegateBase delegate = (DelegateBase)obj._get_delegate();
                         int flags = nv.flags();

                         if (flags != ARG_OUT.value)
                         {
                              if (delegate instanceof DelegateImpl)
                              {
                                   delegate.decrementReferenceCount();
                              }
                         }

                         if (flags != ARG_IN.value)
                         {
                              if (!(delegate instanceof DelegateImpl))
                              {
                                   delegate.incrementReferenceCount();
                              }
                         }
                    }
               }

               if (!oneWay)
               {

                    if (systemException != null)
                    {
                         accessMethod.sendSystemError(objectID,interfaceID,replyID,systemException);
                    }
                    else if (userException != null)
                    {
                         accessMethod.sendUserError(objectID,interfaceID,replyID,userException);
                    }
                    else
                    {
                         accessMethod.sendReply(objectID,interfaceID,operation,replyID,nvList,result);
                    }
               }
               else if (logger != null)
               {
                    if (systemException != null)
                    {
                         logger.log(Level.FINE,"Exception caught while processing event or outcall.",systemException);
                    }
                    else if (userException != null)
                    {
                         logger.log(Level.FINE,"Exception caught while processing event or outcall.",userException);
                    }
               }
          }
          finally
          {
               if (parmHandle.decrementReferenceCount() == 0)
               {
                    accessMethod.releaseParameters(parmHandle);
               }

               engine.returnServerRequest(this);
          }
     }

     private BridgeProtocolEngine engine /* = null */;
     private AccessMethodInterface accessMethod /* = null */;
     private String operation /* = null */;
     private ParameterHandleInterface parmHandle /* = null */;
     private ObjectImpl servant /* = null */;
     private int objectID /* = 0 */;
     private String interfaceID /* = null */;
     private int replyID /* = 0 */;
     private boolean oneWay /* = false */;
     private NVList nvList /* = null */;
     private Any result /* = null */;
     private Any userException /* = null */;
     private Logger logger /* = null */;

     protected void setEngine(BridgeProtocolEngine engine)
     {
          this.engine = engine;
     }

     protected void setAccessMethod(AccessMethodInterface accessMethod)
     {
          this.accessMethod = accessMethod;
     }

     protected void setOperation(String operation)
     {
          this.operation = operation;
     }

     protected void setParameters(ParameterHandleInterface parmHandle)
     {
          this.parmHandle = parmHandle;
     }

     protected void setServant(ObjectImpl servant)
     {
          this.servant = servant;
     }

     protected void setObjectID(int objectID)
     {
          this.objectID = objectID;
     }

     protected void setInterfaceID(String interfaceID)
     {
          this.interfaceID = interfaceID;
     }

     protected void setReplyID(int replyID)
     {
          this.replyID = replyID;
     }

     protected void setOneWay(boolean oneWay)
     {
          this.oneWay = oneWay;
     }
     
     protected void setLogger(Logger logger)
     {
          this.logger = logger;
     }
}