// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg.orb;

import java.util.Hashtable;
//
// History:
//
// Date    Name            Remarks
// 06Jan99 B Beers         Created.
// 06Aug99 B Beers         S0062438: Allow null object references.
// 18Oct99 B Beers         S0066765: Allow long strings.
// 16Mar01 B Beers         S0110897: Improve performance for large parameters.
// 29Jan02 B Beers         S0138065: Fix compiler errors caused by JDK 1.4.
//
/**
 * @author Biff Beers
 */
public class AnyImpl extends org.omg.CORBA.Any
{
     protected static byte[] escape = {(byte)0xBB};
     protected static byte[] doubleEscape = {(byte)0xBB,(byte)0xBB};
     protected static byte[] marker = {0x42,(byte)0xBB,0x42};
     private BrgOrb orb = null;
     private org.omg.CORBA.TypeCode typeCode = null;
     private byte[] value = null;
     private Hashtable objTable = null;
     private Object eObject = null;
     private long eLong = 0L;
     private double eDouble = 0.;
     private boolean eValid = false;

     public AnyImpl(BrgOrb orb)
     {
          this.orb = orb;
     }

     public org.omg.CORBA.TypeCode type()
     {
          return typeCode;
     }

     public void type(org.omg.CORBA.TypeCode typeCode)
     {
          this.typeCode = typeCode;
     }

     public boolean equal(org.omg.CORBA.Any a)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public synchronized void read_value(org.omg.CORBA.portable.InputStream is,org.omg.CORBA.TypeCode t) throws org.omg.CORBA.MARSHAL
     {
          objTable = new Hashtable();

          if (is instanceof InputStreamImpl)
          {
               value = ((InputStreamImpl)is).getBuffer();
               objTable = ((InputStreamImpl)is).getObjTable();
          }
          else
          {
               throw new org.omg.CORBA.NO_IMPLEMENT();
          }

          type(t);
     }

     public synchronized void write_value(org.omg.CORBA.portable.OutputStream os) throws org.omg.CORBA.MARSHAL
     {
          if (eValid)
          {
               switch (typeCode.kind().value())
               {
                    case org.omg.CORBA.TCKind._tk_boolean:
                    {
                         os.write_boolean((eLong!=0L));
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_char:
                    {
                         os.write_char((char)eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_wchar:
                    {
                         os.write_wchar((char)eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_octet:
                    {
                         os.write_octet((byte)eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_short:
                    {
                         os.write_short((short)eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_ushort:
                    {
                         os.write_ushort((short)eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_long:
                    {
                         os.write_long((int)eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_ulong:
                    {
                         os.write_ulong((int)eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_longlong:
                    {
                         os.write_longlong(eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_ulonglong:
                    {
                         os.write_ulonglong(eLong);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_float:
                    {
                         os.write_float((float)eDouble);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_double:
                    {
                         os.write_double(eDouble);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_string:
                    {
                         os.write_string((String)eObject);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_wstring:
                    {
                         os.write_wstring((String)eObject);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_objref:
                    {
                         os.write_Object((org.omg.CORBA.Object)eObject);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_any:
                    {
                         os.write_any((org.omg.CORBA.Any)eObject);
                         break;
                    }

                    case org.omg.CORBA.TCKind._tk_TypeCode:
                    {
                         os.write_TypeCode((org.omg.CORBA.TypeCode)eObject);
                         break;
                    }
               }
          }
          else
          {
               if (os instanceof OutputStreamImpl)
               {
                    ((OutputStreamImpl)os).copyIn(value,objTable);
               }
               else
               {
                    throw new org.omg.CORBA.NO_IMPLEMENT();
               }
          }
     }

     public org.omg.CORBA.portable.OutputStream create_output_stream()
     {
          return orb.create_output_stream();
     }

     public org.omg.CORBA.portable.InputStream create_input_stream()
     {
          org.omg.CORBA.portable.OutputStream outputStream = create_output_stream();
          write_value(outputStream);
          return outputStream.create_input_stream();
     }

     public void insert_Streamable(org.omg.CORBA.portable.Streamable s)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public short extract_short() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_short))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (short)eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_short();
     }

     public void insert_short(short s)
     {
          eLong = s;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
     }

     public int extract_long() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_long))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (int)eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_long();
     }

     public void insert_long(int l)
     {
          eLong = l;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
     }

     public long extract_longlong() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_longlong))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_longlong();
     }

     public void insert_longlong(long l) throws org.omg.CORBA.BAD_OPERATION
     {
          eLong = l;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_longlong);
     }

     public short extract_ushort() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_ushort))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (short)eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_ushort();
     }

     public void insert_ushort(short s)
     {
          eLong = s;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort);
     }

     public int extract_ulong() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_ulong))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (int)eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_ulong();
     }

     public void insert_ulong(int l)
     {
          eLong = l;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulong);
     }

     public long extract_ulonglong() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_ulonglong))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_ulonglong();
     }

     public void insert_ulonglong(long l)
     {
          eLong = l;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulonglong);
     }

     public float extract_float() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_float))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (float)eDouble;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_float();
     }

     public void insert_float(float f)
     {
          eDouble = f;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
     }

     public double extract_double() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_double))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return eDouble;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_double();
     }

     public void insert_double(double d)
     {
          eDouble = d;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_double);
     }

     public boolean extract_boolean() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_boolean))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (eLong != 0L);
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_boolean();
     }

     public void insert_boolean(boolean b)
     {
          eLong = (b)?1L:0L;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_boolean);
     }

     public char extract_char() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_char))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (char)eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_char();
     }

     public void insert_char(char c)
     {
          eLong = c;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char);
     }

     public byte extract_octet() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_octet))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (byte)eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_octet();
     }

     public void insert_octet(byte b)
     {
          eLong = b;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_octet);
     }

     public char extract_wchar() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_wchar))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (char)eLong;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_wchar();
     }

     public void insert_wchar(char c)
     {
          eLong = c;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_wchar);
     }

     public org.omg.CORBA.Any extract_any() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_any))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (org.omg.CORBA.Any)eObject;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_any();
     }

     public void insert_any(org.omg.CORBA.Any a)
     {
          eObject = a;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_any);
     }

     public String extract_string() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_string))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (String)eObject;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_string();
     }

     public void insert_string(String s)
     {
          eObject = s;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
     }

     public String extract_wstring() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_wstring))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid && typeCode.kind().value() == org.omg.CORBA.TCKind._tk_short)
          {
               return (String)eObject;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_wstring();
     }

     public void insert_wstring(String s)
     {
          eObject = s;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_wstring);
     }

     public org.omg.CORBA.Object extract_Object() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_objref))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (org.omg.CORBA.Object)eObject;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_Object();
     }

     public void insert_Object(org.omg.CORBA.Object o)
     {
          insert_Object(o,orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_objref));
     }

     public void insert_Object(org.omg.CORBA.Object o,org.omg.CORBA.TypeCode t)
     {
          eObject = o;
          eValid = true;
          typeCode = t;

          if (o != null)
          {
               try
               {
                    ((org.omg.CORBA.portable.ObjectImpl)o)._get_delegate();
               }
               catch (org.omg.CORBA.SystemException e)
               {
                    DelegateBase delegate = orb.getDelegateBase();
                    delegate.setTransient(true);
                    ((org.omg.CORBA.portable.ObjectImpl)o)._set_delegate(delegate);
               }
          }
     }

     public org.omg.CORBA.TypeCode extract_TypeCode() throws org.omg.CORBA.BAD_OPERATION
     {
          if (typeCode == null || !typeCode.kind().equals(org.omg.CORBA.TCKind.tk_TypeCode))
          {
               throw new org.omg.CORBA.BAD_OPERATION();
          }

          if (eValid)
          {
               return (org.omg.CORBA.TypeCode)eObject;
          }

          org.omg.CORBA.portable.InputStream is = extractCommon();
          return is.read_TypeCode();
     }

     public void insert_TypeCode(org.omg.CORBA.TypeCode t)
     {
          eObject = t;
          eValid = true;
          typeCode = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_TypeCode);
     }

     /**
      * @deprecated by CORBA2.2
      */
     public org.omg.CORBA.Principal extract_Principal() throws org.omg.CORBA.BAD_OPERATION
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     /**
      * @deprecated by CORBA2.2
      */
     public void insert_Principal(org.omg.CORBA.Principal p)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public java.io.Serializable extract_Value() throws org.omg.CORBA.BAD_OPERATION
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public void insert_Value(java.io.Serializable v)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public void insert_Value(java.io.Serializable v,org.omg.CORBA.TypeCode t) throws org.omg.CORBA.MARSHAL
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     private org.omg.CORBA.portable.InputStream extractCommon()
     {
          org.omg.CORBA.portable.OutputStream os = create_output_stream();
          write_value(os);
          return os.create_input_stream();
     }
}