/* $Id$ */
// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg.orb;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.omg.CORBA.BAD_INV_ORDER;
import org.omg.CORBA.Context;
import org.omg.CORBA.ContextList;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.NVList;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.Request;
import org.omg.CORBA.portable.ObjectImpl;
import org.omg.CORBA_2_3.portable.Delegate;

import com.sas.lang.RunnableQueue;
//
// History:
//
// Date    Name            Remarks
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 02May01 B Beers         S0115600: Allow client operation without orb.jar.
// 29Jan02 B Beers         S0138065: Fix compiler errors caused by JDK 1.4.
// 14Feb02 B Beers         S0139873: Added User Service authorization.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
// 30Apr02 B Beers         S0149671: Resolve some garbage collection problems.
// 29Jan04 B Beers         S0233579: Use logger associated with user that created factory.
// 18Feb04 B Beers         S0238566: Clear reference to ConnectionImpl when returning connection.
/**
 * @author Biff Beers
 */
public class DelegateBase extends Delegate
{
     protected DelegateBase(BrgOrb brgOrb)
     {
          this(brgOrb,null);
     }
     
     protected DelegateBase(BrgOrb brgOrb,ProtocolEngineInterface eng)
     {
          this.brgOrb = brgOrb;
          this.eng = eng;
     }

     public org.omg.CORBA.Object duplicate(org.omg.CORBA.Object obj)
     {
          return obj;
     }

     public void release(org.omg.CORBA.Object obj)
     {
     }

     public boolean is_a(org.omg.CORBA.Object obj,String repository_id)
     {
          if (!(obj instanceof GenericObject))
          {
               String[] ids = ((org.omg.CORBA.portable.ObjectImpl)obj)._ids();

               if (ids != null && repository_id != null)
               {
                    for (int i=0;i<ids.length;i++)
                    {
                         if (ids[i].equals(repository_id))
                         {
                              return true;
                         }
                    }
               }
          }

          if (servant != null)
          {
               String[] ids = idsFromServant();
//               String[] ids = ((DynamicImplementation)servant)._all_interfaces(null,null);

               if (ids != null && repository_id != null)
               {
                    for (int i=0;i<ids.length;i++)
                    {
                         if (ids[i].equals(repository_id))
                         {
                              return true;
                         }
                    }
               }
          }

          return interfaceInObject(obj,repository_id);
     }

     public boolean is_equivalent(org.omg.CORBA.Object obj,org.omg.CORBA.Object other)
     {
          if (obj == other)
          {
               return true;
          }

          if (other == null)
          {
               return false;
          }

          Object otherDelegate = null;
          
          try
          {
               otherDelegate = ((ObjectImpl)other)._get_delegate();
          }
          catch (BAD_INV_ORDER e)
          {
               return false;
          }

          return equals(otherDelegate);
     }

     public int hash(org.omg.CORBA.Object obj,int max)
     {
          int h = Math.abs(obj.hashCode());

          if (max != 0 && max != Integer.MAX_VALUE)
          {
               h %= (max + 1);
          }

          return h;
     }

     public org.omg.CORBA.ORB orb(org.omg.CORBA.Object obj)
     {
          return brgOrb;
     }

     public Request request(org.omg.CORBA.Object obj,String operation)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public Request create_request(org.omg.CORBA.Object obj,Context ctx,String operation,NVList arg_list,NamedValue result)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public Request create_request(org.omg.CORBA.Object obj,Context ctx,String operation,NVList arg_list,NamedValue result,ExceptionList exclist,ContextList ctxlist)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public boolean is_local(org.omg.CORBA.Object obj)
     {
          return true;
     }

     public boolean non_existent(org.omg.CORBA.Object obj)
     {
          return false;
     }

     public org.omg.CORBA.Object get_interface_def(org.omg.CORBA.Object obj)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     // public stuff
     //---------------------------------------------------------
     public void setServant(Object servant)
     {
          this.servant = servant;
     }

     public Object getServant()
     {
          return servant;
     }

     public void setObjectID(byte[] objectID)
     {
          this.objectID = objectID;
     }

     public byte[] getObjectID()
     {
          return objectID;
     }

     public synchronized void setEngineInfo(ProtocolEngineInterface engine,Object info)
     {
          if (engineInfoMap == null)
          {
               engineInfoMap = new HashMap();
          }

          engineInfoMap.put(engine,info);
     }

     public synchronized Object getEngineInfo(ProtocolEngineInterface engine)
     {
          return (engineInfoMap==null)?null:engineInfoMap.get(engine);
     }

     public int incrementReferenceCount()
     {
          return ++referenceCount;
     }

     public int decrementReferenceCount()
     {
          return --referenceCount;
     }

     public int getReferenceCount()
     {
          return referenceCount;
     }

     public void setTransient(boolean isTransient)
     {
          this.isTransient = isTransient;
     }

     public boolean isTransient()
     {
          return isTransient;
     }

     public void setWrapper(GenericObject wrapper)
     {
          this.wrapper = wrapper;
     }

     public GenericObject getWrapper()
     {
          return wrapper;
     }

     public void setQueue(RunnableQueue queue)
     {
          this.queue = queue;
     }

     public RunnableQueue getQueue()
     {
          return queue;
     }

     public void setActiveObjectMapKey(ActiveObjectMapKey activeObjectMapKey)
     {
          this.activeObjectMapKey = activeObjectMapKey;
     }

     public ActiveObjectMapKey getActiveObjectMapKey()
     {
          return activeObjectMapKey;
     }

     public void addParent(Object parent)
     {
          parentSet.add(parent);
     }
     
     public void clearParents()
     {
          parentSet.clear();
     }

     public ProtocolEngineInterface getProtocolEngine()
     {
          return eng;
     }

     protected BrgOrb brgOrb /* = null */;
     private Object servant /* = null */;
     private byte[] objectID /* = null */;
     private int referenceCount /* = 0 */;
     private boolean isTransient /* = false */;
     private Map engineInfoMap /* = null */;
     private GenericObject wrapper /* = null */;
     private RunnableQueue queue /* = null */;
     private ActiveObjectMapKey activeObjectMapKey /* = null */;
     private Set parentSet = new HashSet();
     private ProtocolEngineInterface eng /* = null */;

     protected boolean interfaceInObject(org.omg.CORBA.Object obj,String repository_id)
     {
          return false;
     }

     private String[] idsFromServant()
     {
          // use reflection to call this method to avoid loading org.omg.PortableServer.DynamicImplementation
          // unless it is needed. I'm trying to make it possible to run clients without orb.jar in classpath.
          final Object srv = servant;

          return (String[])java.security.AccessController.doPrivileged(new java.security.PrivilegedAction() {
               public Object run()
               {
                    try
                    {
                         Class cls = srv.getClass();
                         Class[] argt = {Class.forName("org.omg.PortableServer.POA"),(new byte[0]).getClass()};
                         java.lang.reflect.Method mth = cls.getDeclaredMethod("_all_interfaces",argt);
                         Object[] argv = {null,null};
                         return mth.invoke(srv,argv);
                    }
                    catch (Exception e)
                    {
                         return null;
                    }
               }
          });
     }
}