// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg.orb;

import java.lang.reflect.Array;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.PushbackInputStream;
import java.util.Hashtable;
import com.sas.io.ReplacementInputStream;

//
// History:
//
// Date    Name            Remarks
// 06Jan99 B Beers         Created.
// 06Aug99 B Beers         S0062438: Allow null object references.
// 27Sep99 B Beers         S0066765: Allow long strings.
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 16Mar01 B Beers         S0110897: Improve performance for large parameters.
// 21Jun02 B Beers         S0156691: Do not cache zero-length arrays in objTable.
//
/**
 * @author Biff Beers
 */
public class InputStreamImpl extends org.omg.CORBA.portable.InputStream
{
     private PushbackInputStream bStream = null;
     private DataInputStream dStream = null;
     private Hashtable objTable = null;
     private byte[] buffer = null;
     private byte[] mkr = null;
     private Object array = null;
     private int index = 0;
     private int count = 0;

     protected InputStreamImpl(byte[] buf)
     {
          this(buf,null);
     }

     protected InputStreamImpl(byte[] buf,Hashtable objTable)
     {
          this.buffer = buf;
          this.objTable = objTable;
          mkr = new byte[4];
     }

     public boolean read_boolean()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_boolean);
          boolean value = false;

          if (array != null)
          {
               value = ((boolean[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    value = dStream.readBoolean();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public char read_char()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_char);
          char value = 0;

          if (array != null)
          {
               value = ((char[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readChar();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public char read_wchar()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_wchar);
          char value = 0;

          if (array != null)
          {
               value = ((char[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readChar();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public byte read_octet()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_octet);
          byte value = 0;

          if (array != null)
          {
               value = ((byte[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readByte();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public short read_short()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_short);
          short value = 0;

          if (array != null)
          {
               value = ((short[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readShort();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public short read_ushort()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_ushort);
          short value = 0;

          if (array != null)
          {
               value = ((short[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readShort();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public int read_long()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_long);
          int value = 0;

          if (array != null)
          {
               value = ((int[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readInt();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public int read_ulong()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_ulong);
          int value = 0;

          if (array != null)
          {
               value = ((int[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readInt();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public long read_longlong()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_longlong);
          long value = 0;

          if (array != null)
          {
               value = ((long[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readLong();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public long read_ulonglong()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_ulonglong);
          long value = 0;

          if (array != null)
          {
               value = ((long[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readLong();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public float read_float()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_float);
          float value = 0;

          if (array != null)
          {
               value = ((float[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readFloat();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public double read_double()
     {
          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_double);
          double value = 0;

          if (array != null)
          {
               value = ((double[])array)[index++];

               if (index == count)
               {
                    array = null;
               }
          }
          else
          {
               try
               {
                    return dStream.readDouble();
               }
               catch (IOException e)
               {
                    throw new org.omg.CORBA.MARSHAL(e.getMessage());
               }
          }

          return value;
     }

     public String read_string()
     {
          checkBuffers();
          int hashCode = 0;

          try
          {
               for (int i=0;i<4;i+=bStream.read(mkr,i,4-i));

               if (mkr[3] != org.omg.CORBA.TCKind._tk_string)
               {
                    throw new org.omg.CORBA.MARSHAL();
               }

               hashCode = dStream.readInt();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }

          Integer key = OutputStreamImpl.getKey(hashCode);
          return (String)objTable.get(key);
     }

     public String read_wstring()
     {
          checkBuffers();
          int hashCode = 0;

          try
          {
               for (int i=0;i<4;i+=bStream.read(mkr,i,4-i));

               if (mkr[3] != org.omg.CORBA.TCKind._tk_wstring)
               {
                    throw new org.omg.CORBA.MARSHAL();
               }

               hashCode = dStream.readInt();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }

          Integer key = OutputStreamImpl.getKey(hashCode);
          return (String)objTable.get(key);
     }

     public void read_boolean_array(boolean[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_boolean();
          }
     }

     public boolean[] extract_boolean_array(int length)
     {
          if (length == 0)
          {
               return new boolean[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_boolean);
          boolean[] value = null;

          if (array != null)
          {
               value = (boolean[])array;
          }
          else
          {
               value = new boolean[length];
               read_boolean_array(value,0,length);
          }

          return value;
     }

     public void read_char_array(char[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_char();
          }
     }

     public char[] extract_char_array(int length)
     {
          if (length == 0)
          {
               return new char[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_char);
          char[] value = null;

          if (array != null)
          {
               value = (char[])array;
          }
          else
          {
               value = new char[length];
               read_char_array(value,0,length);
          }

          return value;
     }

     public void read_wchar_array(char[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_wchar();
          }
     }

     public char[] extract_wchar_array(int length)
     {
          if (length == 0)
          {
               return new char[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_wchar);
          char[] value = null;

          if (array != null)
          {
               value = (char[])array;
          }
          else
          {
               value = new char[length];
               read_char_array(value,0,length);
          }

          return value;
     }

     public void read_octet_array(byte[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_octet();
          }
     }

     public byte[] extract_octet_array(int length)
     {
          if (length == 0)
          {
               return new byte[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_octet);
          byte[] value = null;

          if (array != null)
          {
               value = (byte[])array;
          }
          else
          {
               value = new byte[length];
               read_octet_array(value,0,length);
          }

          return value;
     }

     public void read_short_array(short[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_short();
          }
     }

     public short[] extract_short_array(int length)
     {
          if (length == 0)
          {
               return new short[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_short);
          short[] value = null;

          if (array != null)
          {
               value = (short[])array;
          }
          else
          {
               value = new short[length];
               read_short_array(value,0,length);
          }

          return value;
     }

     public void read_ushort_array(short[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_ushort();
          }
     }

     public short[] extract_ushort_array(int length)
     {
          if (length == 0)
          {
               return new short[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_ushort);
          short[] value = null;

          if (array != null)
          {
               value = (short[])array;
          }
          else
          {
               value = new short[length];
               read_short_array(value,0,length);
          }

          return value;
     }

     public void read_long_array(int[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_long();
          }
     }

     public int[] extract_long_array(int length)
     {
          if (length == 0)
          {
               return new int[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_long);
          int[] value = null;

          if (array != null)
          {
               value = (int[])array;
          }
          else
          {
               value = new int[length];
               read_long_array(value,0,length);
          }

          return value;
     }

     public void read_ulong_array(int[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_ulong();
          }
     }

     public int[] extract_ulong_array(int length)
     {
          if (length == 0)
          {
               return new int[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_ulong);
          int[] value = null;

          if (array != null)
          {
               value = (int[])array;
          }
          else
          {
               value = new int[length];
               read_ulong_array(value,0,length);
          }

          return value;
     }

     public void read_longlong_array(long[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_longlong();
          }
     }

     public long[] extract_longlong_array(int length)
     {
          if (length == 0)
          {
               return new long[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_longlong);
          long[] value = null;

          if (array != null)
          {
               value = (long[])array;
          }
          else
          {
               value = new long[length];
               read_longlong_array(value,0,length);
          }

          return value;
     }

     public void read_ulonglong_array(long[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_ulonglong();
          }
     }

     public long[] extract_ulonglong_array(int length)
     {
          if (length == 0)
          {
               return new long[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_ulonglong);
          long[] value = null;

          if (array != null)
          {
               value = (long[])array;
          }
          else
          {
               value = new long[length];
               read_longlong_array(value,0,length);
          }

          return value;
     }

     public void read_float_array(float[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_float();
          }
     }

     public float[] extract_float_array(int length)
     {
          if (length == 0)
          {
               return new float[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_float);
          float[] value = null;

          if (array != null)
          {
               value = (float[])array;
          }
          else
          {
               value = new float[length];
               read_float_array(value,0,length);
          }

          return value;
     }

     public void read_double_array(double[] value,int offset,int length)
     {
          for (int i=0;i<length;i++)
          {
               value[offset+i] = read_double();
          }
     }

     public double[] extract_double_array(int length)
     {
          if (length == 0)
          {
               return new double[0];
          }

          checkBuffers();
          loadArray(org.omg.CORBA.TCKind._tk_double);
          double[] value = null;

          if (array != null)
          {
               value = (double[])array;
          }
          else
          {
               value = new double[length];
               read_double_array(value,0,length);
          }

          return value;
     }

     public org.omg.CORBA.Object read_Object()
     {
          checkBuffers();
          int hashCode = 0;

          try
          {
               for (int i=0;i<4;i+=bStream.read(mkr,i,4-i));

               if (mkr[3] == org.omg.CORBA.TCKind._tk_null)
               {
                    return null;
               }

               if (mkr[3] != org.omg.CORBA.TCKind._tk_objref)
               {
                    throw new org.omg.CORBA.MARSHAL();
               }

               hashCode = dStream.readInt();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }

          Integer key = OutputStreamImpl.getKey(hashCode);
          return (org.omg.CORBA.Object)objTable.get(key);
     }

     public org.omg.CORBA.TypeCode read_TypeCode()
     {
          checkBuffers();
          int hashCode = 0;

          try
          {
               for (int i=0;i<4;i+=bStream.read(mkr,i,4-i));

               if (mkr[3] != org.omg.CORBA.TCKind._tk_TypeCode)
               {
                    throw new org.omg.CORBA.MARSHAL();
               }

               hashCode = dStream.readInt();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }

          Integer key = OutputStreamImpl.getKey(hashCode);
          return (org.omg.CORBA.TypeCode)objTable.get(key);
     }

     public org.omg.CORBA.Any read_any()
     {
          checkBuffers();
          int hashCode = 0;

          try
          {
               for (int i=0;i<4;i+=bStream.read(mkr,i,4-i));

               if (mkr[3] != org.omg.CORBA.TCKind._tk_any)
               {
                    throw new org.omg.CORBA.MARSHAL();
               }

               hashCode = dStream.readInt();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }

          Integer key = OutputStreamImpl.getKey(hashCode);
          return (org.omg.CORBA.Any)objTable.get(key);
     }

     /**
      * @deprecated by CORBA2.2
      */
     public org.omg.CORBA.Principal read_Principal()
     {
          checkBuffers();
          int hashCode = 0;

          try
          {
               for (int i=0;i<4;i+=bStream.read(mkr,i,4-i));

               if (mkr[3] != org.omg.CORBA.TCKind._tk_Principal)
               {
                    throw new org.omg.CORBA.MARSHAL();
               }

               hashCode = dStream.readInt();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }

          Integer key = OutputStreamImpl.getKey(hashCode);
          return (org.omg.CORBA.Principal)objTable.get(key);
     }

     protected byte[] getBuffer()
     {
          return buffer;
     }

     protected Hashtable getObjTable()
     {
          return objTable;
     }

     private void checkBuffers()
     {
          if (bStream == null)
          {
               bStream = new PushbackInputStream(new ByteArrayInputStream(buffer),4);
               dStream = new DataInputStream(new ReplacementInputStream(bStream,AnyImpl.doubleEscape,AnyImpl.escape));
          }
     }

     private void loadArray(int t)
     {
          if (array == null && objTable != null)
          {
               int i = 0;

               do
               {
                    try
                    {
                         int j = bStream.read(mkr,i,4-i);

                         if (j < 0)
                         {
                              break;
                         }

                         i += j;
                    }
                    catch (IOException ioe)
                    {
                         throw new org.omg.CORBA.MARSHAL(ioe.getMessage());
                    }
               }
               while (i < 4);

               byte[] marker = AnyImpl.marker;

               if (i==4 && mkr[0] == marker[0] && mkr[1] == marker[1] && mkr[2] == marker[2])
               {
                    if (mkr[3] != (byte)t)
                    {
                         throw new org.omg.CORBA.MARSHAL();
                    }

                    int hashCode = 0;

                    try
                    {
                         hashCode = dStream.readInt();
                    }
                    catch (IOException ioe)
                    {
                         throw new org.omg.CORBA.MARSHAL(ioe.getMessage());
                    }

                    Integer key = OutputStreamImpl.getKey(hashCode);
                    array = objTable.get(key);

                    if (array == null)
                    {
                         throw new org.omg.CORBA.MARSHAL();
                    }

                    index = 0;
                    count = Array.getLength(array);
               }
               else
               {
                    try
                    {
                         bStream.unread(mkr,0,i);
                    }
                    catch (IOException ioe)
                    {
                         throw new org.omg.CORBA.MARSHAL(ioe.getMessage());
                    }
               }
          }
     }
}