// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg.orb;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.lang.reflect.Array;
import java.util.Enumeration;
import java.util.Hashtable;
import com.sas.io.ReplacementOutputStream;
//
// History:
//
// Date    Name            Remarks
// 06Jan99 B Beers         Created.
// 29Jan99 B Beers         Allow null strings.
// 26May99 B Beers         S0053449: Improved object lifecycle management.
// 06Aug99 B Beers         S0062438: Allow null object references.
// 27Sep99 B Beers         S0066765: Allow long strings.
// 15Dec99 B Beers         S0067137: Ensure client will close when server does.
// 07Nov00 B Beers         S0102085: Allow null object references again.
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 16Mar01 B Beers         S0110897: Improve performance for large parameters.
// 21Jun02 B Beers         S0156691: Do not cache zero-length arrays in objTable.
// 01Nov02 B Beers         S0170481: Handle collisions in objTable.
//
/**
 * @author Biff Beers
 */
public class OutputStreamImpl extends org.omg.CORBA.portable.OutputStream
{
     private BrgOrb orb = null;
     private ByteArrayOutputStream bStream = null;
     private DataOutputStream dStream = null;
     private Hashtable objTable = null;
     private byte[] buffer = null;
     private Hashtable bufferObjTable = null;

     public OutputStreamImpl(BrgOrb orb)
     {
          this.orb = orb;
          bStream = new ByteArrayOutputStream();
          dStream = new DataOutputStream(new ReplacementOutputStream(bStream,AnyImpl.escape,AnyImpl.doubleEscape));
     }

     public org.omg.CORBA.ORB orb()
     {
          return orb;
     }

     public org.omg.CORBA.portable.InputStream create_input_stream()
     {
          byte[] bArray = (buffer!=null)?buffer:bStream.toByteArray();
          Hashtable oTable = (bufferObjTable!=null)?bufferObjTable:cloneObjTable(objTable);
          InputStreamImpl inputStream = null;

          if (oTable != null)
          {
               inputStream = new InputStreamImpl(bArray,oTable);
          }
          else
          {
               inputStream = new InputStreamImpl(bArray);
          }

          return inputStream;
     }

     public void write_boolean(boolean value)
     {
          try
          {
               checkBuffers();
               dStream.writeBoolean(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_char(char value)
     {
          try
          {
               checkBuffers();
               dStream.writeChar(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_wchar(char value)
     {
          try
          {
               checkBuffers();
               dStream.writeChar(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_octet(byte value)
     {
          try
          {
               checkBuffers();
               dStream.writeByte(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_short(short value)
     {
          try
          {
               checkBuffers();
               dStream.writeShort(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_ushort(short value)
     {
          try
          {
               checkBuffers();
               dStream.writeShort(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_long(int value)
     {
          try
          {
               checkBuffers();
               dStream.writeInt(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_ulong(int value)
     {
          try
          {
               checkBuffers();
               dStream.writeInt(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_longlong(long value)
     {
          try
          {
               checkBuffers();
               dStream.writeLong(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_ulonglong(long value)
     {
          try
          {
               checkBuffers();
               dStream.writeLong(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_float(float value)
     {
          try
          {
               checkBuffers();
               dStream.writeFloat(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_double(double value)
     {
          try
          {
               checkBuffers();
               dStream.writeDouble(value);
               dStream.flush();
          }
          catch (IOException e)
          {
               throw new org.omg.CORBA.MARSHAL(e.getMessage());
          }
     }

     public void write_string(String value)
     {
          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_string);
     }

     public void write_wstring(String value)
     {
          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_wstring);
     }

     public void write_boolean_array(boolean[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          boolean[] obj = new boolean[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_boolean);
     }

     public void insert_boolean_array(boolean[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_boolean);
     }

     public void write_char_array(char[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          char[] obj = new char[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_char);
     }

     public void insert_char_array(char[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_char);
     }

     public void write_wchar_array(char[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          char[] obj = new char[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_wchar);
     }

     public void insert_wchar_array(char[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_wchar);
     }

     public void write_octet_array(byte[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          byte[] obj = new byte[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_octet);
     }

     public void insert_octet_array(byte[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_octet);
     }

     public void write_short_array(short[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          short[] obj = new short[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_short);
     }

     public void insert_short_array(short[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_short);
     }

     public void write_ushort_array(short[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          short[] obj = new short[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_ushort);
     }

     public void insert_ushort_array(short[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_ushort);
     }

     public void write_long_array(int[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          int[] obj = new int[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_long);
     }

     public void insert_long_array(int[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_long);
     }

     public void write_ulong_array(int[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          int[] obj = new int[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_ulong);
     }

     public void insert_ulong_array(int[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_ulong);
     }

     public void write_longlong_array(long[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          long[] obj = new long[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_longlong);
     }

     public void insert_longlong_array(long[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_longlong);
     }

     public void write_ulonglong_array(long[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          long[] obj = new long[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_ulonglong);
     }

     public void insert_ulonglong_array(long[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_ulonglong);
     }

     public void write_float_array(float[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          float[] obj = new float[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_float);
     }

     public void insert_float_array(float[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_float);
     }

     public void write_double_array(double[] value,int offset,int length)
     {
          if (length == 0)
          {
               return;
          }

          checkBuffers();
          double[] obj = new double[length];
          System.arraycopy(value,offset,obj,0,length);
          putObject(obj,org.omg.CORBA.TCKind._tk_double);
     }

     public void insert_double_array(double[] value)
     {
          if (value == null || value.length == 0)
          {
               return;
          }

          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_double);
     }

     public void write_Object(org.omg.CORBA.Object value)
     {
          checkBuffers();
          String key = null;
          int t = 0;

          if (value != null)
          {
               org.omg.CORBA.portable.ObjectImpl objectImpl = (org.omg.CORBA.portable.ObjectImpl)value;

               try
               {
                    objectImpl._get_delegate();
               }
               catch (org.omg.CORBA.SystemException e)
               {
                    DelegateBase delegate = orb.getDelegateBase();
                    delegate.setTransient(true);
                    objectImpl._set_delegate(delegate);
               }

               t = org.omg.CORBA.TCKind._tk_objref;
          }
          else
          {
               t = org.omg.CORBA.TCKind._tk_null;
          }

          putObject(value,t);
     }

     public void write_TypeCode(org.omg.CORBA.TypeCode value)
     {
          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_TypeCode);
     }

     public void write_any(org.omg.CORBA.Any value)
     {
          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_any);
     }

     /**
      * @deprecated by CORBA2.2
      */
     public void write_Principal(org.omg.CORBA.Principal value)
     {
          checkBuffers();
          putObject(value,org.omg.CORBA.TCKind._tk_Principal);
     }

     protected void copyIn(byte[] value,Hashtable ot)
     {
          bufferObjTable = ot;
          buffer = value;
     }

     private void putObject(Object value,int t)
     {
          try
          {
               bStream.write(AnyImpl.marker);
               bStream.write(t);

               if (value != null)
               {
                    int hashCode = value.hashCode();

                    if (objTable == null)
                    {
                         objTable = new Hashtable();
                    }

                    Object key = getKey(hashCode);
                    Object collision = objTable.get(key);
                    boolean store = true;

                    while (collision != null)
                    {
                         if (value == collision || (value instanceof String && value.equals(collision)))
                         {
                              store = false;
                              collision = null;
                         }
                         else
                         {
                              hashCode++;
                              key = getKey(hashCode);
                              collision = objTable.get(key);
                         }
                    }

                    if (store)
                    {
                         objTable.put(key,value);
                    }

                    dStream.writeInt(hashCode);
                    dStream.flush();
               }
          }
          catch (IOException ioe)
          {
               throw new org.omg.CORBA.MARSHAL(ioe.getMessage());
          }
     }

     protected static Integer getKey(int i)
     {
          return new Integer(i);
     }

     private static Hashtable cloneObjTable(Hashtable h)
     {
          if (h == null)
          {
               return null;
          }

          Enumeration keys = h.keys();
          Hashtable h1 = new Hashtable();

          while (keys.hasMoreElements())
          {
               Object key = keys.nextElement();
               Object obj = h.get(key);
               h1.put(key,obj);
          }

          return h1;
     }

     private void checkBuffers()
     {
          if (buffer != null)
          {
               try
               {
                    bStream.write(buffer);
               }
               catch (IOException ioe)
               {
                    throw new org.omg.CORBA.MARSHAL(ioe.getMessage());
               }
               finally
               {
                    buffer = null;
               }
          }

          if (bufferObjTable != null)
          {
               objTable = cloneObjTable(bufferObjTable);
               bufferObjTable = null;
          }
     }

     private static boolean elementsEqual(Object o1,Object o2)
     {
          if (o1 == o2)
          {
               return true;
          }

          if (o1 == null)
          {
               return false;
          }

          if (o1.equals(o2))
          {
               return true;
          }

          // identical arrays a1 and a2 have a1.equals(a2) == false
          if (o2 == null)
          {
               return false;
          }

          String t1 = o1.getClass().getName();
          String t2 = o2.getClass().getName();

          if (!t1.equals(t2))
          {
               return false;
          }

          try
          {
               int l1 = Array.getLength(o1);
               int l2 = Array.getLength(o2);

               if (l1 != l2)
               {
                    return false;
               }

               for (int i=0;i<l1;i++)
               {
                    Object e1 = Array.get(o1,i);
                    Object e2 = Array.get(o2,i);

                    if (e1 == null ? e2 != null : !e1.equals(e2))
                    {
                         return false;
                    }
               }
          }
          catch (IllegalArgumentException iae)
          {
               return false;
          }

          return true;
     }
}