// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg.orb;

import org.omg.CORBA.Any;
import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.ARG_INOUT;
import org.omg.CORBA.ARG_OUT;
import org.omg.CORBA.Context;
import org.omg.CORBA.ContextList;
import org.omg.CORBA.Environment;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.NVList;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Request;
import org.omg.CORBA.TypeCode;
//
// History:
//
// Date    Name            Remarks
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
//
/**
 * @author Biff Beers
 */
public abstract class RequestBase extends Request
{
     protected RequestBase(BrgOrb orb,org.omg.CORBA.Object target,String operation)
     {
          this.orb = orb;
          this.target = target;
          this.operation = operation;
     }

     public org.omg.CORBA.Object target()
     {
          return target;
     }

     public String operation()
     {
          return operation;
     }

     public NVList arguments()
     {
          if (arguments == null)
          {
               arguments = orb.create_list(0);
          }

          return arguments;
     }

     public NamedValue result()
     {
          if (result == null)
          {
               result = orb.create_named_value(null,orb.create_any(),ARG_OUT.value);
          }

          return result;
     }

     public Environment env()
     {
          if (environment == null)
          {
               environment = orb.create_environment();
          }

          return environment;
     }

     public ExceptionList exceptions()
     {
          if (exceptionList == null)
          {
               exceptionList = orb.create_exception_list();
          }

          return exceptionList;
     }

     public ContextList contexts()
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public Context ctx()
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public void ctx(Context c)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public Any add_in_arg()
     {
          NVList argList = arguments();
          Any value = orb.create_any();
          argList.add_value(null,value,ARG_IN.value);
          return value;
     }

     public Any add_named_in_arg(String name)
     {
          NVList argList = arguments();
          Any value = orb.create_any();
          argList.add_value(name,value,ARG_IN.value);
          return value;
     }

     public Any add_inout_arg()
     {
          NVList argList = arguments();
          Any value = orb.create_any();
          argList.add_value(null,value,ARG_INOUT.value);
          return value;
     }

     public Any add_named_inout_arg(String name)
     {
          NVList argList = arguments();
          Any value = orb.create_any();
          argList.add_value(name,value,ARG_INOUT.value);
          return value;
     }

     public Any add_out_arg()
     {
          NVList argList = arguments();
          Any value = orb.create_any();
          argList.add_value(null,value,ARG_OUT.value);
          return value;
     }

     public Any add_named_out_arg(String name)
     {
          NVList argList = arguments();
          Any value = orb.create_any();
          argList.add_value(name,value,ARG_OUT.value);
          return value;
     }

     public void set_return_type(TypeCode tc)
     {
          returnType = tc;
     }

     public Any return_value()
     {
          return result().value();
     }

     public void send_oneway()
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public void send_deferred()
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public boolean poll_response()
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public void get_response()
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     private BrgOrb orb = null;
     private org.omg.CORBA.Object target = null;
     private String operation = null;
     private Environment environment = null;
     private ExceptionList exceptionList = null;
     private NVList arguments = null;
     private NamedValue result = null;
     private TypeCode returnType = null;

     protected TypeCode getReturnType()
     {
          return returnType;
     }

     protected BrgOrb getOrb()
     {
          return orb;
     }
}