// Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513
package com.sas.net.brg.orb;

import org.omg.CORBA.OBJ_ADAPTER;
import org.omg.CORBA.ORB;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.Servant;
import org.omg.PortableServer.POAPackage.ServantNotActive;
import org.omg.PortableServer.POAPackage.WrongPolicy;
import org.omg.PortableServer.portable.Delegate;
//
// History:
//
// Date    Name            Remarks
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
//
/**
 * @author Biff Beers
 */
public class ServantDelegateImpl implements Delegate
{
     protected ServantDelegateImpl(BrgOrb orb,POAImpl poa)
     {
          this.orb = orb;
          this.poa = poa;
     }

     public ORB orb(Servant self)
     {
          return orb;
     }

     public org.omg.CORBA.Object this_object(Servant self)
     {
          try
          {
               return poa.servant_to_reference(self);
          }
          catch (WrongPolicy wp)
          {
               throw new OBJ_ADAPTER();
          }
          catch (ServantNotActive sna)
          {
               throw new OBJ_ADAPTER();
          }
     }

     public POA poa(Servant self)
     {
          return poa;
     }

     public byte[] object_id(Servant self)
     {
          try
          {
               return poa.servant_to_id(self);
          }
          catch (WrongPolicy wp)
          {
               throw new OBJ_ADAPTER();
          }
          catch (ServantNotActive sna)
          {
               throw new OBJ_ADAPTER();
          }
     }

     public POA default_POA(Servant self)
     {
          return poa;
     }

     public boolean is_a(Servant self,String repository_id)
     {
          if (repository_id == null)
          {
               return false;
          }

          String[] ids = self._all_interfaces(poa,object_id(self));

          if (ids != null)
          {
               for (int i=0;i<ids.length;i++)
               {
                    if ((ids[i].startsWith("DCE:") && ids[i].length() >= 40 && ids[i].indexOf(repository_id) >= 0) || ids[i].equals(repository_id))
                    {
                         return true;
                    }
               }
          }

          return false;
     }

     public boolean non_existent(Servant self)
     {
          return false;
     }

     public org.omg.CORBA.Object get_interface_def(Servant self)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     private BrgOrb orb = null;
     private POAImpl poa = null;
     private byte[] objectID = null;

     protected void setObjectID(byte[] objectID)
     {
          this.objectID = objectID;
     }

     protected byte[] getObjectID()
     {
          return objectID;
     }
}