package com.sas.net.brg.orb;

//
// History:
//
// Date    Name            Remarks
// 06Jan99 B Beers         Created.
// 29Jan02 B Beers         S0138065: Fix compiler errors caused by JDK 1.4.
//
/**
 * @author Biff Beers
 */
public class TypeCode extends org.omg.CORBA.TypeCode
{
     /**
      * Constructor for primitive types.
      */
     protected TypeCode(int kind)
     {
          this.kind = kind;

          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_objref:
               {
                    id = "IDL:omg.org/CORBA/Object:1.0";
                    name = "Object";
                    break;
               }
          }
     }

     /**
      * Constructor for struct and except types.
      */
     protected TypeCode(int kind,String id,String name,org.omg.CORBA.StructMember[] structMembers)
     {
          this.kind = kind;
          this.id = id;
          this.name = name;
          memberCount = (structMembers != null) ? structMembers.length : 0;
          memberNames = new String[memberCount];
          memberTypes = new org.omg.CORBA.TypeCode[memberCount];

          for (int i=0;i<memberCount;i++)
          {
               memberNames[i] = structMembers[i].name;
               memberTypes[i] = structMembers[i].type;
          }
     }

     /**
      * Constructor for union types.
      */
     protected TypeCode(String id,String name,org.omg.CORBA.TypeCode discriminator,org.omg.CORBA.UnionMember[] unionMembers)
     {
          this.kind = org.omg.CORBA.TCKind._tk_union;
          this.id = id;
          this.name = name;
          this.discriminator = discriminator;
          defaultIndex = -1;
          memberCount = (unionMembers != null) ? unionMembers.length : 0;
          memberNames = new String[memberCount];
          memberTypes = new org.omg.CORBA.TypeCode[memberCount];
          unionLabels = new org.omg.CORBA.Any[memberCount];

          for (int i=0;i<memberCount;i++)
          {
               memberNames[i] = unionMembers[i].name;
               memberTypes[i] = unionMembers[i].type;
               unionLabels[i] = unionMembers[i].label;

               if (primitiveTypeCodes[org.omg.CORBA.TCKind._tk_octet].equal(unionMembers[i].type))
               {
                    defaultIndex = i;
               }
          }
     }

     /**
      * Constructor for enum types.
      */
     protected TypeCode(String id,String name,String[] enumMembers)
     {
          this.kind = org.omg.CORBA.TCKind._tk_enum;
          this.id = id;
          this.name = name;
          memberCount = (enumMembers != null) ? enumMembers.length : 0;
          memberNames = new String[memberCount];

          for (int i=0;i<memberCount;i++)
          {
               memberNames[i] = enumMembers[i];
          }
     }

     /**
      * Constructor for alias types.
      */
     protected TypeCode(String id,String name,org.omg.CORBA.TypeCode contentType)
     {
          this.kind = org.omg.CORBA.TCKind._tk_alias;
          this.id = id;
          this.name = name;
          this.contentType = contentType;
     }

     /**
      * Constructor for interface types.
      */
     protected TypeCode(String id,String name)
     {
          this.kind = org.omg.CORBA.TCKind._tk_objref;
          this.id = id;
          this.name = name;
     }

     /**
      * Constructor for string and wstring types.
      */
     protected TypeCode(int kind,int length)
     {
          this.kind = kind;
          this.length = length;
     }

     /**
      * Constructor for sequence and array types.
      */
     protected TypeCode(int kind,int length,org.omg.CORBA.TypeCode contentType)
     {
          this.kind = kind;
          this.length = length;
          this.contentType = contentType;
     }

     public boolean equal(org.omg.CORBA.TypeCode tc)
     {
          if (tc == null)
          {
               return false;
          }

          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_null:
               case org.omg.CORBA.TCKind._tk_void:
               case org.omg.CORBA.TCKind._tk_short:
               case org.omg.CORBA.TCKind._tk_long:
               case org.omg.CORBA.TCKind._tk_ushort:
               case org.omg.CORBA.TCKind._tk_ulong:
               case org.omg.CORBA.TCKind._tk_float:
               case org.omg.CORBA.TCKind._tk_double:
               case org.omg.CORBA.TCKind._tk_boolean:
               case org.omg.CORBA.TCKind._tk_char:
               case org.omg.CORBA.TCKind._tk_octet:
               case org.omg.CORBA.TCKind._tk_any:
               case org.omg.CORBA.TCKind._tk_TypeCode:
               case org.omg.CORBA.TCKind._tk_Principal:
               case org.omg.CORBA.TCKind._tk_longlong:
               case org.omg.CORBA.TCKind._tk_ulonglong:
               case org.omg.CORBA.TCKind._tk_longdouble:
               case org.omg.CORBA.TCKind._tk_wchar:
               {
                    int tk = tc.kind().value();

                    switch(tk)
                    {
                         case org.omg.CORBA.TCKind._tk_null:
                         case org.omg.CORBA.TCKind._tk_void:
                         case org.omg.CORBA.TCKind._tk_short:
                         case org.omg.CORBA.TCKind._tk_long:
                         case org.omg.CORBA.TCKind._tk_ushort:
                         case org.omg.CORBA.TCKind._tk_ulong:
                         case org.omg.CORBA.TCKind._tk_float:
                         case org.omg.CORBA.TCKind._tk_double:
                         case org.omg.CORBA.TCKind._tk_boolean:
                         case org.omg.CORBA.TCKind._tk_char:
                         case org.omg.CORBA.TCKind._tk_octet:
                         case org.omg.CORBA.TCKind._tk_any:
                         case org.omg.CORBA.TCKind._tk_TypeCode:
                         case org.omg.CORBA.TCKind._tk_Principal:
                         case org.omg.CORBA.TCKind._tk_longlong:
                         case org.omg.CORBA.TCKind._tk_ulonglong:
                         case org.omg.CORBA.TCKind._tk_longdouble:
                         case org.omg.CORBA.TCKind._tk_wchar:
                         {
                              return (tk == kind);
                         }

                         case org.omg.CORBA.TCKind._tk_alias:
                         {
                              org.omg.CORBA.TypeCode sub = null;

                              try
                              {
                                   sub = tc.content_type();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              return equal(sub);
                         }
                    }

                    break;
               }

               case org.omg.CORBA.TCKind._tk_string:
               case org.omg.CORBA.TCKind._tk_wstring:
               {
                    int tk = tc.kind().value();

                    switch(tk)
                    {
                         case org.omg.CORBA.TCKind._tk_string:
                         case org.omg.CORBA.TCKind._tk_wstring:
                         {
                              if (tk == kind)
                              {
                                   try
                                   {
                                        if (length == tc.length())
                                        {
                                             return true;
                                        }
                                   }
                                   catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                                   {
                                        e.printStackTrace();
                                   }
                              }

                              break;
                         }

                         case org.omg.CORBA.TCKind._tk_alias:
                         {
                              org.omg.CORBA.TypeCode sub = null;

                              try
                              {
                                   sub = tc.content_type();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              return equal(sub);
                         }
                    }

                    break;
               }

               case org.omg.CORBA.TCKind._tk_sequence:
               case org.omg.CORBA.TCKind._tk_array:
               {
                    int tk = tc.kind().value();

                    switch(tk)
                    {
                         case org.omg.CORBA.TCKind._tk_sequence:
                         case org.omg.CORBA.TCKind._tk_array:
                         {
                              if (tk == kind)
                              {
                                   try
                                   {
                                        if (length == tc.length() && equalTypeCodes(contentType,tc.content_type()))
                                        {
                                             return true;
                                        }
                                   }
                                   catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                                   {
                                        e.printStackTrace();
                                   }
                              }

                              break;
                         }

                         case org.omg.CORBA.TCKind._tk_alias:
                         {
                              org.omg.CORBA.TypeCode sub = null;

                              try
                              {
                                   sub = tc.content_type();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              return equal(sub);
                         }
                    }

                    break;
               }

               case org.omg.CORBA.TCKind._tk_objref:
               {
                    int tk = tc.kind().value();

                    switch(tk)
                    {
                         case org.omg.CORBA.TCKind._tk_objref:
                         {
                              try
                              {
                                   if (equalStrings(name,tc.name()) && equalStrings(id,tc.id()))
                                   {
                                        return true;
                                   }
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              break;
                         }

                         case org.omg.CORBA.TCKind._tk_alias:
                         {
                              org.omg.CORBA.TypeCode sub = null;

                              try
                              {
                                   sub = tc.content_type();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              return equal(sub);
                         }
                    }

                    break;
               }

               case org.omg.CORBA.TCKind._tk_struct:
               case org.omg.CORBA.TCKind._tk_except:
               {
                    int tk = tc.kind().value();

                    switch(tk)
                    {
                         case org.omg.CORBA.TCKind._tk_struct:
                         case org.omg.CORBA.TCKind._tk_except:
                         {
                              if (tk == kind)
                              {
                                   try
                                   {
                                        if (equalStrings(name,tc.name()) && equalStrings(id,tc.id()))
                                        {
                                             int size = tc.member_count();

                                             if (size != memberCount)
                                             {
                                                  return false;
                                             }

                                             for (int i=0;i<size;i++)
                                             {
                                                  if (!equalStrings(memberNames[i],tc.member_name(i)))
                                                  {
                                                       return false;
                                                  }

                                                  if (!equalTypeCodes(memberTypes[i],tc.member_type(i)))
                                                  {
                                                       return false;
                                                  }
                                             }

                                             return true;
                                        }
                                   }
                                   catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                                   {
                                        e.printStackTrace();
                                   }
                                   catch (org.omg.CORBA.TypeCodePackage.Bounds e)
                                   {
                                        e.printStackTrace();
                                   }
                              }

                              break;
                         }

                         case org.omg.CORBA.TCKind._tk_alias:
                         {
                              org.omg.CORBA.TypeCode sub = null;

                              try
                              {
                                   sub = tc.content_type();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              return equal(sub);
                         }
                    }

                    break;
               }

               case org.omg.CORBA.TCKind._tk_union:
               {
                    int tk = tc.kind().value();

                    switch(tk)
                    {
                         case org.omg.CORBA.TCKind._tk_union:
                         {
                              try
                              {
                                   if (equalStrings(name,tc.name()) && equalStrings(id,tc.id()) && equalTypeCodes(discriminator,tc.discriminator_type()) && defaultIndex == tc.default_index())
                                   {
                                        int size = tc.member_count();

                                        if (size != memberCount)
                                        {
                                             return false;
                                        }

                                        for (int i=0;i<size;i++)
                                        {
                                             if (!equalStrings(memberNames[i],tc.member_name(i)))
                                             {
                                                  return false;
                                             }

                                             if (!equalTypeCodes(memberTypes[i],tc.member_type(i)))
                                             {
                                                  return false;
                                             }

                                             if (!equalAnys(unionLabels[i],tc.member_label(i)))
                                             {
                                                  return false;
                                             }
                                        }

                                        return true;
                                   }
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                        e.printStackTrace();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.Bounds e)
                              {
                                        e.printStackTrace();
                              }

                              break;
                         }

                         case org.omg.CORBA.TCKind._tk_alias:
                         {
                              org.omg.CORBA.TypeCode sub = null;

                              try
                              {
                                   sub = tc.content_type();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              return equal(sub);
                         }
                    }

                    break;
               }

               case org.omg.CORBA.TCKind._tk_enum:
               {
                    int tk = tc.kind().value();

                    switch(tk)
                    {
                         case org.omg.CORBA.TCKind._tk_enum:
                         {
                              try
                              {
                                   if (equalStrings(name,tc.name()) && equalStrings(id,tc.id()))
                                   {
                                        int size = tc.member_count();

                                        if (size != memberCount)
                                        {
                                             return false;
                                        }

                                        for (int i=0;i<size;i++)
                                        {
                                             if (!equalStrings(memberNames[i],tc.member_name(i)))
                                             {
                                                  return false;
                                             }
                                        }

                                        return true;
                                   }
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                        e.printStackTrace();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.Bounds e)
                              {
                                        e.printStackTrace();
                              }

                              break;
                         }

                         case org.omg.CORBA.TCKind._tk_alias:
                         {
                              org.omg.CORBA.TypeCode sub = null;

                              try
                              {
                                   sub = tc.content_type();
                              }
                              catch (org.omg.CORBA.TypeCodePackage.BadKind e)
                              {
                                   e.printStackTrace();
                              }

                              return equal(sub);
                         }
                    }

                    break;
               }

               case org.omg.CORBA.TCKind._tk_alias:
               {
                    return contentType.equal(tc);
               }
          }

          return false;
     }

     public boolean equivalent(org.omg.CORBA.TypeCode tc)
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public org.omg.CORBA.TypeCode get_compact_typecode()
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public org.omg.CORBA.TCKind kind()
     {
          return org.omg.CORBA.TCKind.from_int(kind);
     }

     public String id() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_objref:
               case org.omg.CORBA.TCKind._tk_struct:
               case org.omg.CORBA.TCKind._tk_union:
               case org.omg.CORBA.TCKind._tk_enum:
               case org.omg.CORBA.TCKind._tk_alias:
               case org.omg.CORBA.TCKind._tk_except:
               {
                    return id;
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public String name() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_objref:
               case org.omg.CORBA.TCKind._tk_struct:
               case org.omg.CORBA.TCKind._tk_union:
               case org.omg.CORBA.TCKind._tk_enum:
               case org.omg.CORBA.TCKind._tk_alias:
               case org.omg.CORBA.TCKind._tk_except:
               {
                    return name;
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public int member_count() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_struct:
               case org.omg.CORBA.TCKind._tk_union:
               case org.omg.CORBA.TCKind._tk_enum:
               case org.omg.CORBA.TCKind._tk_except:
               {
                    return memberCount;
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public String member_name(int index) throws org.omg.CORBA.TypeCodePackage.BadKind,org.omg.CORBA.TypeCodePackage.Bounds
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_struct:
               case org.omg.CORBA.TCKind._tk_union:
               case org.omg.CORBA.TCKind._tk_enum:
               case org.omg.CORBA.TCKind._tk_except:
               {
                    try
                    {
                         return memberNames[index];
                    }
                    catch(ArrayIndexOutOfBoundsException ex)
                    {
                         throw new org.omg.CORBA.TypeCodePackage.Bounds();
                    }
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public org.omg.CORBA.TypeCode member_type(int index) throws org.omg.CORBA.TypeCodePackage.BadKind,org.omg.CORBA.TypeCodePackage.Bounds
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_struct:
               case org.omg.CORBA.TCKind._tk_union:
               case org.omg.CORBA.TCKind._tk_except:
               {
                    try
                    {
                         return memberTypes[index];
                    }
                    catch(ArrayIndexOutOfBoundsException ex)
                    {
                         throw new org.omg.CORBA.TypeCodePackage.Bounds();
                    }
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public org.omg.CORBA.Any member_label(int index) throws org.omg.CORBA.TypeCodePackage.BadKind,org.omg.CORBA.TypeCodePackage.Bounds
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_union:
               {
                    try
                    {
                         return unionLabels[index];
                    }
                    catch(ArrayIndexOutOfBoundsException ex)
                    {
                         throw new org.omg.CORBA.TypeCodePackage.Bounds();
                    }
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public org.omg.CORBA.TypeCode discriminator_type() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_union:
               {
                    return discriminator;
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public int default_index() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_union:
               {
                    return defaultIndex;
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public int length() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_string:
               case org.omg.CORBA.TCKind._tk_sequence:
               case org.omg.CORBA.TCKind._tk_array:
               case org.omg.CORBA.TCKind._tk_wstring:
               {
                    return length;
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }
     }

     public org.omg.CORBA.TypeCode content_type() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          switch(kind)
          {
               case org.omg.CORBA.TCKind._tk_sequence:
               case org.omg.CORBA.TCKind._tk_array:
               case org.omg.CORBA.TCKind._tk_alias:
               {
                    return contentType;
               }

               default:
               {
                    throw new org.omg.CORBA.TypeCodePackage.BadKind();
               }
          }

     }

     public short fixed_digits() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public short fixed_scale() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public short member_visibility(int index) throws org.omg.CORBA.TypeCodePackage.BadKind,org.omg.CORBA.TypeCodePackage.Bounds
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public short type_modifier() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     public org.omg.CORBA.TypeCode concrete_base_type() throws org.omg.CORBA.TypeCodePackage.BadKind
     {
          throw new org.omg.CORBA.NO_IMPLEMENT();
     }

     private static final TypeCode[] primitiveTypeCodes = new TypeCode[28];
     private int kind = 0;
     private String id = null;
     private String name = null;
     private int memberCount = 0;
     private String memberNames[] = null;
     private org.omg.CORBA.TypeCode memberTypes[] = null;
     private org.omg.CORBA.Any unionLabels[] = null;
     private org.omg.CORBA.TypeCode discriminator = null;
     private int defaultIndex = 0;
     private int length = 0;
     private org.omg.CORBA.TypeCode contentType = null;
     private org.omg.CORBA.ORB orb = null;

     static
     {
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_null] = new TypeCode(org.omg.CORBA.TCKind._tk_null);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_void] = new TypeCode(org.omg.CORBA.TCKind._tk_void);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_short] = new TypeCode(org.omg.CORBA.TCKind._tk_short);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_long] = new TypeCode(org.omg.CORBA.TCKind._tk_long);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_ushort] = new TypeCode(org.omg.CORBA.TCKind._tk_ushort);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_ulong] = new TypeCode(org.omg.CORBA.TCKind._tk_ulong);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_float] = new TypeCode(org.omg.CORBA.TCKind._tk_float);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_double] = new TypeCode(org.omg.CORBA.TCKind._tk_double);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_boolean] = new TypeCode(org.omg.CORBA.TCKind._tk_boolean);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_char] = new TypeCode(org.omg.CORBA.TCKind._tk_char);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_octet] = new TypeCode(org.omg.CORBA.TCKind._tk_octet);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_any] = new TypeCode(org.omg.CORBA.TCKind._tk_any);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_TypeCode] = new TypeCode(org.omg.CORBA.TCKind._tk_TypeCode);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_Principal] = new TypeCode(org.omg.CORBA.TCKind._tk_Principal);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_objref] = new TypeCode(org.omg.CORBA.TCKind._tk_objref);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_struct] = null;
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_union] = null;
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_enum] = null;
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_string] = new TypeCode(org.omg.CORBA.TCKind._tk_string);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_sequence] = null;
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_array] = null;
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_alias] = null;
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_except] = null;
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_longlong] = new TypeCode(org.omg.CORBA.TCKind._tk_longlong);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_ulonglong] = new TypeCode(org.omg.CORBA.TCKind._tk_ulonglong);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_longdouble] = new TypeCode(org.omg.CORBA.TCKind._tk_longdouble);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_wchar] = new TypeCode(org.omg.CORBA.TCKind._tk_wchar);
          primitiveTypeCodes[org.omg.CORBA.TCKind._tk_wstring] = new TypeCode(org.omg.CORBA.TCKind._tk_wstring);
     };

     protected static org.omg.CORBA.TypeCode getPrimitiveTypeCode(org.omg.CORBA.TCKind kind)
     {
          return primitiveTypeCodes[kind.value()];
     }

     private static boolean equalStrings(String s1,String s2)
     {
          if (s1 == s2)
          {
               return true;
          }

          if (s1 == null || s2 == null)
          {
               return false;
          }

          return s1.equals(s2);
     }

     private static boolean equalTypeCodes(org.omg.CORBA.TypeCode s1,org.omg.CORBA.TypeCode s2)
     {
          if (s1 == s2)
          {
               return true;
          }

          if (s1 == null || s2 == null)
          {
               return false;
          }

          return s1.equal(s2);
     }

     private static boolean equalAnys(org.omg.CORBA.Any s1,org.omg.CORBA.Any s2)
     {
          if (s1 == s2)
          {
               return true;
          }

          if (s1 == null || s2 == null)
          {
               return false;
          }

          return s1.equal(s2);
     }
}