package com.sas.net.brg.sam;

import java.util.List;
import org.omg.CORBA.Any;
import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.ARG_OUT;
import org.omg.CORBA.Bounds;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.NVList;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import com.sas.net.brg.BridgeProtocolEngine;
//
// History:
//
// Date    Name            Remarks
// 23Dec97 B Beers         Created.
// 23Dec97 B Beers         Adapted for SimpleCommand interface.
// 02Apr98 B Beers         Added support for long arrays.
// 10Sep98 B Beers         Added cryptography support.
// 06Jan99 B Beers         Moved SimpleCommand from com.sas.util.
// 26May99 B Beers         S0053449: Improved object lifecycle management.
// 28Feb01 B Beers         S0110393: Add bind and server flow features.
// 16Mar01 B Beers         S0110897: Improve performance for large parameters.
// 23Aug01 B Beers         S0124519: Split socket driver code into SocketAccessMethod.
// 08Oct01 B Beers         S0128796: Fix NullPointerException sending empty string array.
// 05Apr03 B Beers         S0193189: Blot out password in FINER/FINEST log messages.
//
/**
 * This class is a specialization of Packet for packets sent by the client
 * to the server. It contains code designed to handle outbound packets.
 * @author Biff Beers
 */
public class OutboundPacket extends Packet
{
     protected OutboundPacket(SocketAccessMethod accessMethod)
     {
          super();
          this.accessMethod = accessMethod;
     }

     public String toString()
     {
          String lineSeparator = System.getProperty("line.separator"/*I18NOK:EMS*/);
          StringBuffer buf = new StringBuffer(super.toString());
          int[] dosMaps = getDosMaps();

          if (dosMaps != null)
          {
               List dosParameters = getDosParameters();

               for (int j=0;j<dosMaps.length;j++)
               {
                    Object dosParameter = dosParameters.get(j);
                    buf.append(SocketAccessMethod.dosTypeToString(dosMaps[j]));
                    int l = 0;

                    switch (dosMaps[j])
                    {
                         case SocketAccessMethod.DOS_MP_UCHAR:
                         {
                              String s = (String)dosParameter;

                              if (getType() == Packet.INTERNAL && getMethodId() == SocketAccessMethod.USERPASS && j == 1)
                              {
                                   buf.append("xxxxxxxx"/*I18NOK:EMS*/);
                              }
                              else
                              {
                                   buf.append((s.length()>128)?s.substring(0,128):s);
                              }

                              break;
                         }

                         case SocketAccessMethod.DOS_MP_SHORT:
                         {
                              short[] s = (short[])dosParameter;
                              l = s.length;

                              for (int i=0;i<l&&i<16;i++)
                              {
                                   buf.append(s[i]).append(" "/*I18NOK:EMS*/);
                              }

                              break;
                         }

                         case SocketAccessMethod.DOS_MP_UINT:
                         case SocketAccessMethod.DOS_MP_LONG:
                         case SocketAccessMethod.DOS_MP_ULONG:
                         {
                              int[] s = (int[])dosParameter;
                              l = s.length;

                              for (int i=0;i<l&&i<16;i++)
                              {
                                   buf.append(s[i]).append(" "/*I18NOK:EMS*/);
                              }

                              break;
                         }

                         case SocketAccessMethod.DOS_MP_DOUBLE:
                         {
                              double[] s = (double[])dosParameter;
                              l = s.length;

                              for (int i=0;i<l&&i<16;i++)
                              {
                                   buf.append(s[i]).append(" "/*I18NOK:EMS*/);
                              }

                              break;
                         }

                         case SocketAccessMethod.DOS_MP_BINARY:
                         {
                              byte[] s = (byte[])dosParameter;
                              l = s.length;

                              for (int i=0;i<l&&i<16;i++)
                              {
                                   int b = (s[i] < 0) ? s[i] + 256 : s[i];
                                   buf.append((b<16)?"0":"").append(Integer.toHexString(b)).append(" "); /*I18NOK:LINE*/
                              }

                              break;
                         }
                    }

                    if (l > 16)
                    {
                         buf.append("(").append(l).append(" elements)"/*I18NOK:EMS*/);
                    }

                    buf.append(lineSeparator);
               }

               return buf.toString();
          }

          int parameterCount = getParameterCount();

          if (parameters != null)
          {
               int skipDirection = 0;

               if (getType() == Packet.REPLY)
               {
                    skipDirection = ARG_IN.value;
                    buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UINT));
                    buf.append("0"/*I18NOK:EMS*/).append(lineSeparator);
               }
               else
               {
                    skipDirection = ARG_OUT.value;
               }

               for (int i=0;i<parameterCount;i++)
               {
                    NamedValue nv = getParameterAt(i);

                    if (nv.flags() != skipDirection)
                    {
                         Any value = nv.value();
                         TypeCode tc = value.type();
                         IntHolder dim = new IntHolder();
                         int iomType = BridgeProtocolEngine.typeCodeToIomType(tc,dim);

                         switch (dim.value)
                         {
                              case 0: // scalars
                              {
                                   switch (iomType)
                                   {
                                        case BridgeProtocolEngine.IT_BOOLEAN:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_BINARY));
                                             boolean b = value.extract_boolean();
                                             buf.append((b)?"01":"00"); /*I18NOK:LINE*/
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_OCTET:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_BINARY));
                                             byte o = value.extract_octet();
                                             int b = (o>=0)?o:(o+256);
                                             buf.append((b<16)?"0":"").append(Integer.toHexString(b)); /*I18NOK:LINE*/
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_SHORT:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_SHORT));
                                             buf.append(value.extract_short());
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_LONG:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_LONG));
                                             buf.append(value.extract_long());
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_FLOAT:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));
                                             buf.append(value.extract_float());
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_DOUBLE:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));
                                             buf.append(value.extract_double());
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_STRING:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UCHAR));
                                             String s = value.extract_string();
                                             buf.append((s.length()>128)?s.substring(0,128):s);
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_ENUM:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UINT));
                                             org.omg.CORBA.portable.InputStream is = value.create_input_stream();
                                             buf.append(is.read_long());
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_OBJECT:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UINT));
                                             org.omg.CORBA.Object o = value.extract_Object();
                                             BridgeProtocolEngine engine = accessMethod.getEngine();
                                             buf.append(engine.objectToObjectID(o));
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_UUID:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UCHAR));
                                             org.omg.CORBA.portable.InputStream is = value.create_input_stream();
                                             buf.append(BridgeProtocolEngine.uuidToString(is));
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_DATETIME:
                                        {
                                             buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));
                                             org.omg.CORBA.portable.InputStream is = value.create_input_stream();
                                             long corba = is.read_ulonglong();
                                             double sas = BridgeProtocolEngine.corbaToSas(corba);
                                             buf.append(sas);
                                             break;
                                        }
                                   }

                                   break;
                              }

                              case 1: // 1D arrays
                              {
                                   buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_SHORT));
                                   buf.append(1).append(lineSeparator);
                                   org.omg.CORBA.portable.InputStream is = value.create_input_stream();
                                   int mag0 = is.read_long();
                                   buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_ULONG));
                                   buf.append(mag0);

                                   if (mag0 > 0)
                                   {
                                        switch (iomType)
                                        {
                                             case BridgeProtocolEngine.IT_BOOLEAN:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_BINARY));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       buf.append((is.read_boolean())?"01":"00").append(" "); /*I18NOK:LINE*/
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_OCTET:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_BINARY));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       byte o = is.read_octet();
                                                       int b = (o>=0)?o:(o+256);
                                                       buf.append((b<16)?"0":"").append(Integer.toHexString(b)).append(" "); /*I18NOK:LINE*/
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_SHORT:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_SHORT));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       buf.append(is.read_short()).append(" "); /*I18NOK:LINE*/
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_LONG:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_LONG));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       buf.append(is.read_long()).append(" "); /*I18NOK:LINE*/
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_FLOAT:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       buf.append(is.read_float()).append(" ");
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_DOUBLE:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       buf.append(is.read_double()).append(" ");
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_STRING:
                                             {
                                                  for (int j=0;j<mag0;j++)
                                                  {
                                                       buf.append(lineSeparator);
                                                       buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UCHAR));
                                                       String s = is.read_string();
                                                       buf.append((s.length()>128)?s.substring(0,128):s);
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_DATETIME:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       long o = is.read_ulonglong();
                                                       double sas = BridgeProtocolEngine.corbaToSas(o);
                                                       buf.append(sas).append(" "); /*I18NOK:LINE*/
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_ENUM:
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UINT));

                                                  for (int j=0;j<mag0&&j<16;j++)
                                                  {
                                                       buf.append(is.read_long()).append(" "); /*I18NOK:LINE*/
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_UUID:
                                             {
                                                  for (int j=0;j<mag0;j++)
                                                  {
                                                       buf.append(lineSeparator);
                                                       buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UCHAR));
                                                       buf.append(BridgeProtocolEngine.uuidToString(is));
                                                  }

                                                  break;
                                             }
                                        }

                                        if (mag0 >= 16)
                                        {
                                             buf.append("(").append(Integer.toString(mag0)).append(" elements)"); /*I18NOK:LINE*/
                                        }
                                   }

                                   break;
                              }

                              case 2: // 2D arrays
                              {
                                   buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_SHORT));
                                   buf.append(2).append(lineSeparator);
                                   org.omg.CORBA.portable.InputStream is = value.create_input_stream();
                                   int mag0 = is.read_long();
                                   int mag1 = (mag0 > 0) ? is.read_long() : 0;
                                   buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_ULONG));
                                   buf.append(mag0).append(" ").append(mag1);

                                   switch (iomType)
                                   {
                                        case BridgeProtocolEngine.IT_BOOLEAN:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_BINARY));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append((is.read_boolean())?"01":"00").append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_OCTET:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_BINARY));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       byte o = is.read_octet();
                                                       int b = (o>=0)?o:(o+256);
                                                       buf.append((b<16)?"0":"").append(Integer.toHexString(b)).append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_SHORT:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_SHORT));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append(is.read_short()).append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }


                                        case BridgeProtocolEngine.IT_LONG:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_LONG));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append(is.read_long()).append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_FLOAT:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append(is.read_float()).append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_DOUBLE:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append(is.read_double()).append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_STRING:
                                        {
                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append(lineSeparator);
                                                       buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UCHAR));
                                                       buf.append(is.read_string());
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_ENUM:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UINT));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append(is.read_long()).append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_DATETIME:
                                        {
                                             if (mag0 * mag1 > 0)
                                             {
                                                  buf.append(lineSeparator);
                                                  buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_DOUBLE));
                                             }

                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       long corba = is.read_ulonglong();
                                                       double sas = BridgeProtocolEngine.corbaToSas(corba);
                                                       buf.append(sas).append(" "); /*I18NOK:LINE*/
                                                  }
                                             }

                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_UUID:
                                        {
                                             int count = 0;

                                             for (int j0=0;j0<mag0&&count<16;j0++)
                                             {
                                                  if (j0 != 0)
                                                  {
                                                       is.read_long();
                                                  }

                                                  for (int j1=0;j1<mag1&&count<16;j1++,count++)
                                                  {
                                                       buf.append(lineSeparator);
                                                       buf.append(SocketAccessMethod.dosTypeToString(SocketAccessMethod.DOS_MP_UCHAR));
                                                       buf.append(BridgeProtocolEngine.uuidToString(is));
                                                  }
                                             }

                                             break;
                                        }
                                   }

                                   if (mag0 * mag1 >= 16)
                                   {
                                        buf.append("(").append(Integer.toString(mag0*mag1)).append(" elements)"); /*I18NOK:LINE*/
                                   }

                                   break;
                              }
                         }

                         buf.append(lineSeparator);
                    }
               }
          }

          return buf.toString();
     }

     SocketAccessMethod accessMethod = null;

     protected void setParameters(NVList parameters)
     {
          this.parameters = parameters;
     }

     protected NVList getParameters()
     {
          return parameters;
     }

     protected int getParameterCount()
     {
          return (parameters == null) ? 0 : parameters.count();
     }

     protected NamedValue getParameterAt(int index)
     {
          try
          {
               return parameters.item(index);
          }
          catch (Bounds e)
          {
               throw new IndexOutOfBoundsException(Integer.toString(index));
          }
          catch (NullPointerException n)
          {
               throw new IndexOutOfBoundsException(Integer.toString(index));
          }
     }

     protected void setDosParameters(int[] dosMaps,List dosParameters)
     {
          this.dosParameters = dosParameters;
          this.dosMaps = dosMaps;
     }

     protected int[] getDosMaps()
     {
          return dosMaps;
     }

     protected List getDosParameters()
     {
          return dosParameters;
     }

     private NVList parameters = null;
     private List dosParameters = null;
     private int[] dosMaps = null;
}