/* $Id$ */
package com.sas.net.brg.sam;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.ConnectException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.omg.CORBA.Any;
import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.ARG_OUT;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.Bounds;
import org.omg.CORBA.COMM_FAILURE;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.INITIALIZE;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.INV_OBJREF;
import org.omg.CORBA.MARSHAL;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.NO_IMPLEMENT;
import org.omg.CORBA.NO_MEMORY;
import org.omg.CORBA.NO_PERMISSION;
import org.omg.CORBA.NVList;
import org.omg.CORBA.OBJECT_NOT_EXIST;
import org.omg.CORBA.ORB;
import org.omg.CORBA.StringHolder;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.UNKNOWN;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;
import com.sas.io.Encoder;
import com.sas.io.EncoderInterface;
import com.sas.io.PatchableByteArrayOutputStream;
import com.sas.iom.SASIOMDefs.GenericError;
import com.sas.net.brg.AccessMethodInterface;
import com.sas.net.brg.BridgeProtocolEngine;
import com.sas.net.brg.ParameterHandleInterface;
import com.sas.net.brg.UnexpectedUserException;
import com.sas.net.crypto.CipherInterface;
import com.sas.net.crypto.Crypto;
import com.sas.net.crypto.CryptoException;
import com.sas.net.util.WriterHandler;
import com.sas.net.util.VerySimpleFormatter;
import com.sas.security.BaseSecuritySupport;
import com.sas.services.connection.Server;
import com.sas.text.Message;
//
// History:
//
// Date    Name            Remarks
// 23Aug01 B Beers         S0124519: Split socket driver code into SocketAccessMethod.
// 10Sep01 B Beers         S0126240: Remove case sensitivity in FindConnectionPoint processing.
// 17Sep01 B Beers         S0126555: Implement PE_REQS_REDIRECT.
// 19Sep01 B Beers         S0127027: Fix PE_ENCR_AUTH support.
// 19Sep01 B Beers         S0126981: Send Release call in REQUEST packet.
// 08Oct01 B Beers         S0128796: Fix NullPointerException sending empty string array.
// 12Oct01 B Beers         S0129352: Remove connection sharing logic.
// 18Dec01 B Beers         S0135040: Move encoder assignment code to the right place.
// 11Jan02 B Beers         S0135578: Implement PE_REQS_REDIRECT_2.
// 21Jan02 B Beers         S0108012: Implement PE_BEEN_REDIRECTED.
// 14Feb02 B Beers         S0139873: Added User Service authorization.
// 28Mar02 B Beers         S0145470: Bypass Cp1047Encoder due to confusion about newline/linefeed transcoding.
// 10Apr02 B Beers         S0141718: Remove dependency on netutil.jar.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 14May02 B Beers         S0151508: Handle xml error messages from v9 servers.
// 20May02 B Beers         S0149943: Fix threading problem at shutdown time.
// 29May02 B Beers         S0139445: Logging service integration (happy 9th anniversary Biff & Julie).
// 15Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 24Oct02 B Beers         S0169574: Added password generation support.
// 29Oct02 B Beers         S0156049: Added support for expired credentials.
// 19Nov02 B Beers         S0169574: Revise password generation support.
// 26Nov02 B Beers         S0151508: Revise xml error message handling.
// 02Dec02 B Beers         S0151508: Restore error message handling for V8 servers.
// 02Dec02 B Beers         S0151508: Fix exception type mapping.
// 11Dec02 B Beers         S0169574: Fix bugs in password generation support.
// 17Dec02 B Beers         S0151508: Check for missing text elements in xml error messages.
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
// 14Jan03 B Beers         S0170823: Check for port out of range.
// 17Jan03 B Beers         S0177362: Throw GenericError when required.
// 23Jan03 B Beers         S0156049: Add granularity to authentication errors.
// 31Mar03 B Beers         S0184610: Give threads useful names.
// 04Apr03 B Beers         S0193912: Improve error messages for connection failure.
// 05Apr03 B Beers         S0193189: Blot out password in FINER/FINEST log messages.
// 01May03 B Beers         S0198710: Handle LB timeout.
// 20May03 B Beers         S0202544: Inline readFully calls.
// 25Jun03 B Beers         S0207995: Allow client to negotiate up to server's encrytion level.
// 11Jul03 B Beers         S0211000: Remove thread create from DelegateImpl.finalize().
// 12Nov03 B Beers         S0227109: Remove stub finalizers.
// 16Dec03 B Beers         S0227513: Log unhandled exceptions.
// 29Jan04 B Beers         S0233579: Use logger associated with user that created factory.
// 19May04 B Beers         S0250171: Allow PE_ERROR packets with empty exception lists.
// 02Nov05 B Beers         S0261301: Fix porous isLoggable() guard. 
/**
 * @author Biff Beers
 */
public class SocketAccessMethod implements AccessMethodInterface,Runnable
{
     public SocketAccessMethod(Properties properties)
     {
          this.properties = properties;
          handleDebugLevel();
          client = true;
          init();
     }

     public SocketAccessMethod(Properties properties,Socket s,int clientID)
     {
          this.properties = properties;
          this.socket = s;
          handleDebugLevel();
          client = false;
          this.clientID = clientID;
          init();
     }

     private void init()
     {
          outputBuffer = new PatchableByteArrayOutputStream(1024,2);
          outputFilter = new DataOutputStream(outputBuffer);
          patchBuffer = new ByteArrayOutputStream(4);
          patchFilter = new DataOutputStream(patchBuffer);

          try
          {
               encoder = Encoder.getInstance("ISO8859_1"/*I18NOK:EMS*/);
          }
          catch (UnsupportedEncodingException e)
          {
               // this should only happen if netutil.jar is out of date
               // {0}: com.sas.io.Encoder should support ISO8859_1 even if the Java Virtual Machine doesn''t. You may need to upgrade netutil.jar.
               String messageIndex = "sam.S200.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               INTERNAL internal = new INTERNAL(message);
               internal.initCause(e);
               throw internal;
          }

     }

     public Properties getProperties()
     {
          Properties p = (Properties)properties.clone();

          // There's no point in hiding the user name and password
          // in a connection to an OMR server because the connection
          // could easily be used to look up the info in the server.
          //--------------------------------------------------------
          if (!Server.CLSID_SASOMI.equals(p.getProperty("clsid"/*I18NOK:EMS*/)))
          {
               if (p.containsKey("password"/*I18NOK:EMS*/))
               {
                    p.put("password","xxxxxxxx"); /*I18NOK:LINE*/
               }
          }

          return p;
     }

     public Logger getLogger()
     {
          return logger;
     }
     
     public void setLogger(Logger logger)
     {
          this.logger = logger;
     }
     
     public void setProtocolEngine(BridgeProtocolEngine engine)
     {
          this.engine = engine;
     }

     public int connect()
     {
          replyList = new ArrayList();

          try
          {
               errorDocBuilderFactory = DocumentBuilderFactory.newInstance();
               errorDocBuilder = errorDocBuilderFactory.newDocumentBuilder();
          }
          catch (ParserConfigurationException pce)
          {
               // {0}: An exception was thrown while creating a XML document builder.
               String messageIndex = "sam.S247.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               INITIALIZE initialize = new INITIALIZE(message);
               initialize.initCause(pce);
               throw initialize;
          }

          if (client)
          {
               try
               {
                    establishConnection();
                    state = DISPATCH;
                    startPacketReader();
               }
               catch (RuntimeException r)
               {
                    synchronized (this)
                    {
                         if (socket != null)
                         {
                              try
                              {
                                   socket.close();
                              }
                              catch (IOException ioe)
                              {
                                   logger.log(Level.FINE,"Exception encountered closing socket.",ioe);
                              }

                              socket = null;
                         }

                         if (encoder != null)
                         {
                              Encoder.returnInstance(encoder);
                         }
                    }

                    throw r;
               }
               catch (Error e)
               {
                    synchronized (this)
                    {
                         if (socket != null)
                         {
                              try
                              {
                                   socket.close();
                              }
                              catch (IOException ioe)
                              {
                                   logger.log(Level.FINE,"Exception encountered closing socket.",ioe);
                              }

                              socket = null;
                         }

                         if (encoder != null)
                         {
                              Encoder.returnInstance(encoder);
                              encoder = null;
                         }
                    }

                    throw e;
               }

               try
               {
                    userPass();
               }
               catch (RuntimeException r)
               {
                    close();
                    throw r;
               }
               catch (Error e)
               {
                    close();
                    throw e;
               }
          }
          else
          {
               try
               {
                    openStreams();
                    state = REQUIREMENTS;
                    startPacketReader();
               }
               catch (RuntimeException r)
               {
                    synchronized (this)
                    {
                         if (socket != null)
                         {
                              try
                              {
                                   socket.close();
                              }
                              catch (IOException ioe)
                              {
                                   logger.log(Level.FINE,"Exception encountered closing socket.",ioe);
                              }

                              socket = null;
                         }

                         if (encoder != null)
                         {
                              Encoder.returnInstance(encoder);
                              encoder = null;
                         }
                    }

                    throw r;
               }
               catch (Error e)
               {
                    synchronized (this)
                    {
                         if (socket != null)
                         {
                              try
                              {
                                   socket.close();
                              }
                              catch (IOException ioe)
                              {
                                   logger.log(Level.FINE,"Exception encountered closing socket.",ioe);
                              }

                              socket = null;
                         }

                         if (encoder != null)
                         {
                              Encoder.returnInstance(encoder);
                              encoder = null;
                         }
                    }

                    throw e;
               }
          }

          return clientID;
     }

     public boolean isConnected()
     {
          return (socket != null);
     }

     public int newObject(String classID)
     {
          OutboundPacket out = getOutboundPacket();
          InboundPacket in = null;

          try
          {
               out.setType(Packet.INTERNAL);
               out.setMethodId(NEWOBJECT);
               out.setUuid(classID);
               sendPacket(out);
               in = getReply(out);
               return in.getObjectId();
          }
          catch (UnexpectedUserException e)
          {
               INV_OBJREF inv_objref = new INV_OBJREF(e.getMessage());
               inv_objref.initCause(e);
               throw inv_objref;
          }
          finally
          {
               returnOutboundPacket(out);
               returnInboundPacket(in);
          }
     }

     public boolean interfaceInObject(int objectID,String interfaceID)
     {
          OutboundPacket out = getOutboundPacket();
          InboundPacket in = null;

          try
          {
               out.setType(Packet.INTERNAL);
               out.setObjectId(objectID);
               out.setMethodId(INTERFACEINOBJECT);
               out.setUuid(interfaceID);
               sendPacket(out);
               in = getReply(out);
               List parameters = new ArrayList(1);
               reconcileDosParameters(in,parameters);
               int result = ((int[])parameters.get(0))[0];
               return (result == 0);
          }
          catch (UnexpectedUserException e)
          {
               OBJECT_NOT_EXIST object_not_exist = new OBJECT_NOT_EXIST(e.getMessage());
               object_not_exist.initCause(e);
               throw object_not_exist;
          }
          finally
          {
               returnOutboundPacket(out);
               returnInboundPacket(in);
          }

     }

     public String[] eventsFromObjectID(int objectID)
     {
          OutboundPacket out = getOutboundPacket();
          InboundPacket in = null;

          try
          {
               out.setType(Packet.INTERNAL);
               out.setObjectId(objectID);
               out.setMethodId(EVENTSFROMOBJECTID);
               sendPacket(out);
               in = getReply(out);
               List parameters = new ArrayList(1);
               int[] maps = reconcileDosParameters(in,parameters);
               String[] events = new String[maps.length-1];

               for (int i=1;i<maps.length;i++)
               {
                    events[i-1] = ((String)parameters.get(i)).toLowerCase();
               }

               return events;
          }
          catch (UnexpectedUserException e)
          {
               OBJECT_NOT_EXIST object_not_exist = new OBJECT_NOT_EXIST(e.getMessage());
               object_not_exist.initCause(e);
               throw object_not_exist;
          }
          finally
          {
               returnOutboundPacket(out);
               returnInboundPacket(in);
          }

     }

     public void advise(int objectID,String eventInterfaceID)
     {
          OutboundPacket out = getOutboundPacket();
          InboundPacket in = null;

          try
          {
               out.setType(Packet.INTERNAL);
               out.setObjectId(objectID);
               out.setMethodId(ADVISE);
               out.setUuid(eventInterfaceID);
               sendPacket(out);
               in = getReply(out);
          }
          catch (UnexpectedUserException e)
          {
               OBJECT_NOT_EXIST object_not_exist = new OBJECT_NOT_EXIST(e.getMessage());
               object_not_exist.initCause(e);
               throw object_not_exist;
          }
          finally
          {
               returnOutboundPacket(out);
               returnInboundPacket(in);
          }
     }

     public void unadvise(int objectID,String eventInterfaceID)
     {
          OutboundPacket out = getOutboundPacket();
          InboundPacket in = null;

          try
          {
               out.setType(Packet.INTERNAL);
               out.setObjectId(objectID);
               out.setMethodId(UNADVISE);
               out.setUuid(eventInterfaceID);
               sendPacket(out);
               in = getReply(out);
          }
          catch (UnexpectedUserException e)
          {
               OBJECT_NOT_EXIST object_not_exist = new OBJECT_NOT_EXIST(e.getMessage());
               object_not_exist.initCause(e);
               throw object_not_exist;
          }
          finally
          {
               returnOutboundPacket(out);
               returnInboundPacket(in);
          }

     }

     public void generatePassword(String userName,StringHolder qualUserNameHolder,StringHolder genPasswordHolder)
     {
          List dosParameters = new ArrayList(3);
          int[] dosMaps = new int[1];
          dosMaps[0] = DOS_MP_UCHAR;
          dosParameters.add(userName);
          OutboundPacket out = getOutboundPacket();
          InboundPacket in = null;

          try
          {
               out.setType(Packet.INTERNAL);
               out.setMethodId(GENPASSWORD);
               out.setDosParameters(dosMaps,dosParameters);
               sendPacket(out);
               in = getReply(out);
               dosParameters.clear();
               dosMaps = reconcileDosParameters(in,dosParameters);
               String qualUserName = (String)dosParameters.get(1);
               String genPassword = (String)dosParameters.get(2);

               if (qualUserNameHolder != null)
               {
                    qualUserNameHolder.value = qualUserName;
               }

               if (genPasswordHolder != null)
               {
                    genPasswordHolder.value = genPassword;
               }
          }
          catch (UnexpectedUserException e)
          {
               UNKNOWN unknown = new UNKNOWN(e.getMessage());
               unknown.initCause(e);
               throw unknown;
          }
          finally
          {
               returnOutboundPacket(out);
               returnInboundPacket(in);
          }
     }

     public void sendRequest(int objectID,String interfaceID,String name,NVList parameters,Any result)
     {
          OutboundPacket out = getOutboundPacket();
          out.setObjectId(objectID);
          out.setUuid(interfaceID);
          NVList tempParms = null;
          int parameterCount = (parameters != null) ? parameters.count() : 0;
          ORB orb = engine.getORB();
          tempParms = orb.create_list(parameterCount+2);
          out.setParameters(tempParms);

          if (name.equals("Release"/*I18NOK:EMS*/))
          {
               out.setType(Packet.REQUEST);
               out.setMethodId(2);
          }
          else if (name.equals("AddRef"/*I18NOK:EMS*/))
          {
               out.setType(Packet.REQUEST);
               out.setMethodId(1);
          }
          else if (name.equals("CastToInterface"/*I18NOK:EMS*/))
          {
               out.setType(Packet.REQUEST);
               out.setMethodId(0);
          }
          else
          {
               out.setType(Packet.REQUEST_BY_NAME);
               Any _name = orb.create_any();
               _name.insert_string(name);
               tempParms.add_value("name",_name,ARG_IN.value);
          }

          for (int i=0;i<parameterCount;i++)
          {
               NamedValue nv = null;

               try
               {
                    nv = parameters.item(i);
               }
               catch (Bounds e)
               {
                    // this shouldn't happen
                    String cl = parameters.getClass().getName();
                    throw new INTERNAL(cl + ".count() returned an incorrect value");
               }

               tempParms.add_value(nv.name(),nv.value(),nv.flags());
          }

          sendPacket(out);
          InboundPacket in = null;

          try
          {
               in = getReply(out);
          }
          finally
          {
               returnOutboundPacket(out);
          }

          if (result != null)
          {
               tempParms.add_value("result"/*I18NOK:EMS*/,result,ARG_OUT.value);
          }

          try
          {
               reconcileParameters(in,tempParms);
          }
          finally
          {
               releaseParameters(in);
          }
     }

     public void sendEvent(int objectID,String interfaceID,NVList parameters)
     {
     }

     public void sendReply(int objectID,String interfaceID,String name,int replyID,NVList parameters,Any result)
     {
          OutboundPacket out = getOutboundPacket();
          out.setType(Packet.REPLY);
          out.setObjectId(objectID);
          out.setUuid(interfaceID);
          out.setReplyId(replyID);

          if (name.equals("CastToInterface"/*I18NOK:EMS*/))
          {
               int[] dosMaps = {DOS_MP_UINT};
               List dosParameters = new ArrayList(1);
               dosParameters.add(new int[]{(result.extract_boolean())?0:1});
               out.setDosParameters(dosMaps,dosParameters);
          }
          else
          {
               if (result != null)
               {
                    if (parameters == null)
                    {
                         ORB orb = engine.getORB();
                         parameters = orb.create_list(1);
                    }

                    parameters.add_value("_result"/*I18NOK:EMS*/,result,ARG_OUT.value);
               }

               out.setParameters(parameters);
          }

          sendPacket(out);
          returnOutboundPacket(out);
     }

     public void sendSystemError(int objectID,String interfaceID,int replyID,Throwable ex)
     {
          OutboundPacket out = getOutboundPacket();
          out.setType(Packet.INTERNAL);
          out.setMethodId(ERROR);
          out.setObjectId(objectID);
          out.setUuid(interfaceID);
          out.setReplyId(replyID);
          int[] dosMaps = {DOS_MP_UINT,DOS_MP_UCHAR,DOS_MP_LONG,DOS_MP_UCHAR,DOS_MP_UCHAR};
          List dosParameters = new ArrayList(5);
          int status = 1; //WOIE_FAIL

          if (ex instanceof NO_MEMORY) //WOIE_OUTOFMEMORY
          {
               status = 2;
          }
          else if (ex instanceof NO_IMPLEMENT) //WOIE_NOTIMPL
          {
               status = 3;
          }
          else if (ex instanceof BAD_OPERATION) //WOIE_NOINTERFACE
          {
               status = 4;
          }
          else if (ex instanceof BAD_PARAM) //WOIE_INVALIDARG
          {
               status = 5;
          }
          else if (ex instanceof NO_PERMISSION) //WOIE_ACCESSDENIED
          {
               status = 6;
          }
          else if (ex instanceof OBJECT_NOT_EXIST) //WOIE_NONEXISTENT
          {
               status = 8;
          }

          dosParameters.add(new int[]{status});
          String msg = ex.getMessage();
          dosParameters.add((msg==null)?"":msg);
          dosParameters.add(replyRC);
          dosParameters.add(""/*I18NOK:EMS*/);
          dosParameters.add(""/*I18NOK:EMS*/);
          out.setDosParameters(dosMaps,dosParameters);
          sendPacket(out);
          returnOutboundPacket(out);
     }

     public void sendUserError(int objectID,String interfaceID,int replyID,Any ex)
     {
          OutboundPacket out = getOutboundPacket();
          out.setType(Packet.INTERNAL);
          out.setMethodId(ERROR);
          out.setObjectId(objectID);
          out.setUuid(interfaceID);
          out.setReplyId(replyID);
          int[] dosMaps = {DOS_MP_UINT,DOS_MP_UCHAR,DOS_MP_LONG,DOS_MP_UCHAR,DOS_MP_UCHAR};
          List dosParameters = new ArrayList(5);
          dosParameters.add(userExRC);
          String id = null;

          try
          {
               id = ex.type().id();
          }
          catch (BadKind e)
          {
               // this should never happen
               String cl = ex.type().getClass().getName();
               throw new INTERNAL(cl + ".id() threw an exception when it shouldn't have");
          }

          String msg = null;

          if (id != null && id.startsWith("DCE:"/*I18NOK:EMS*/) && id.length() > 40)
          {
               id = id.substring(4,40);

               try
               {
                    org.omg.CORBA.portable.InputStream is = ex.create_input_stream();
                    is.read_string();
                    msg = is.read_string();
               }
               catch (SystemException se)
               {
               }
          }

          dosParameters.add((msg==null)?""/*I18NOK:EMS*/:msg);
          dosParameters.add(replyRC);
          dosParameters.add((id==null)?""/*I18NOK:EMS*/:id);
          dosParameters.add(""/*I18NOK:EMS*/);
          out.setDosParameters(dosMaps,dosParameters);
          sendPacket(out);
          returnOutboundPacket(out);
     }

     public void reconcileParameters(ParameterHandleInterface parmHandle,NVList parameters)
     {
          InboundPacket in = (InboundPacket)parmHandle;
          byte[] payloadArray = in.getPayload();
          int payloadSize = in.getPayloadSize();
          int wzxMapCount = in.getMapCount();
          int parmOff = wzxMapCount * MapSize;
          ByteArrayInputStream mapBuffer = new ByteArrayInputStream(payloadArray,0,parmOff);
          DataInputStream mapFilter = new DataInputStream(mapBuffer);
          ByteArrayInputStream parmBuffer = new ByteArrayInputStream(payloadArray,parmOff,payloadSize-parmOff);
          DataInputStream parmFilter = new DataInputStream(parmBuffer);
          int skipDirection = (in.getType() == Packet.REPLY) ? ARG_IN.value : ARG_OUT.value;
          int parameterCount = parameters.count();
          IntHolder typeHolder = new IntHolder();
          IntHolder repFactorHolder = new IntHolder();
          BooleanHolder unsignedHolder = new BooleanHolder();
          int wzxMapIndex = 0;
          int wzxType = 0;
          int repFactor = 0;
          boolean unsigned = false;


          try
          {
               // The first wzx map/parm for a REPLY packet is always an unsigned int
               // return code which should always be zero unless this is an ERROR
               // packet. ERROR packets are handled in getReply so we shouldn't see one here.
               // The first wzx map/parm for a REQUEST_BY_NAME or EVENT_BY_NAME packet is
               // always the name of the request or event. Skip it; it's already captured.
               //----------------------------------------------------------------------------
               int type = in.getType();

               if (type == Packet.REPLY)
               {
                    wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                    parmBuffer.skip(4);
                    parmOff += 4;
               }
               else if (type == Packet.REQUEST_BY_NAME || type == Packet.EVENT_BY_NAME)
               {
                    wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                    repFactor = repFactorHolder.value;
                    parmOff += repFactor;
                    parmBuffer.skip(repFactor);
               }

               // Now handle the "real" parameters.
               for (int i=0;i<parameterCount;i++)
               {
                    NamedValue nv = null;

                    try
                    {
                         nv = parameters.item(i);
                    }
                    catch (Bounds e)
                    {
                         // this shouldn't happen
                         String cl = parameters.getClass().getName();
                         throw new INTERNAL(cl + ".count() returned an incorrect value");
                    }

                    if (nv.flags() != skipDirection)
                    {
                         Any any = nv.value();
                         TypeCode tc = any.type();
                         IntHolder dim = new IntHolder();
                         int iomType = BridgeProtocolEngine.typeCodeToIomType(tc,dim);

                         switch (dim.value)
                         {
                              case 0: // scalars
                              {
                                   wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                   switch (iomType)
                                   {
                                        case BridgeProtocolEngine.IT_BOOLEAN:
                                        {
                                             any.insert_boolean(parmBuffer.read() != 0);
                                             parmOff++;
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_OCTET:
                                        {
                                             any.insert_octet(parmFilter.readByte());
                                             parmOff++;
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_SHORT:
                                        {
                                             any.insert_short(parmFilter.readShort());
                                             parmOff += 2;
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_LONG:
                                        {
                                             any.insert_long(parmFilter.readInt());
                                             parmOff += 4;
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_FLOAT:
                                        {
                                             any.insert_float((float)parmFilter.readDouble());
                                             parmOff += 8;
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_DOUBLE:
                                        {
                                             any.insert_double(parmFilter.readDouble());
                                             parmOff += 8;
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_STRING:
                                        {
                                             repFactor = repFactorHolder.value;
                                             String s = encoder.toString(payloadArray,parmOff,repFactor-1);
                                             parmOff += repFactor;
                                             parmBuffer.skip(repFactor);
                                             any.insert_string(s);
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_ENUM:
                                        {
                                             org.omg.CORBA.portable.OutputStream os = any.create_output_stream();
                                             os.write_long(parmFilter.readInt());
                                             parmOff += 4;
                                             any.read_value(os.create_input_stream(),tc);
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_OBJECT:
                                        {
                                             int objectID = parmFilter.readInt();
                                             parmOff += 4;
                                             any.insert_Object(engine.objectIDToObject(objectID),tc);
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_DATETIME:
                                        {
                                             double sas = parmFilter.readDouble();
                                             long corba = BridgeProtocolEngine.sasToCorba(sas);
                                             parmOff += 8;
                                             org.omg.CORBA.portable.OutputStream os = any.create_output_stream();
                                             os.write_ulonglong(corba);
                                             any.read_value(os.create_input_stream(),tc);
                                             break;
                                        }

                                        case BridgeProtocolEngine.IT_UUID:
                                        {
                                             repFactor = repFactorHolder.value;
                                             String s = encoder.toString(payloadArray,parmOff,repFactor-1);
                                             parmOff += repFactor;
                                             parmBuffer.skip(repFactor);
                                             org.omg.CORBA.portable.OutputStream os = any.create_output_stream();
                                             BridgeProtocolEngine.stringToUuid(s,os);
                                             any.read_value(os.create_input_stream(),tc);
                                             break;
                                        }
                                   }

                                   break;
                              }

                              case 1: // 1D arrays
                              {
                                   wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                   parmFilter.readShort();
                                   parmOff += 2;
                                   wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                   int mag0 = parmFilter.readInt();
                                   parmOff += 4;
                                   org.omg.CORBA.portable.OutputStream os = any.create_output_stream();
                                   os.write_long(mag0);

                                   if (mag0 > 0)
                                   {
                                        switch (iomType)
                                        {
                                             case BridgeProtocolEngine.IT_BOOLEAN:
                                             {
                                                  wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       boolean[] a = new boolean[mag0];

                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            a[j] = (parmBuffer.read() != 0);
                                                       }

                                                       ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_boolean_array(a);
                                                  }
                                                  else
                                                  {
                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            os.write_boolean(parmBuffer.read() != 0);
                                                       }
                                                  }

                                                  parmOff += mag0;
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_OCTET:
                                             {
                                                  wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       byte[] a = new byte[mag0];

                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            a[j] = parmFilter.readByte();
                                                       }

                                                       ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_octet_array(a);
                                                  }
                                                  else
                                                  {
                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            os.write_octet(parmFilter.readByte());
                                                       }
                                                  }

                                                  parmOff += mag0;
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_SHORT:
                                             {
                                                  wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       short[] a = new short[mag0];

                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            a[j] = parmFilter.readShort();
                                                       }

                                                       ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_short_array(a);
                                                  }
                                                  else
                                                  {
                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            os.write_short(parmFilter.readShort());
                                                       }
                                                  }

                                                  parmOff += (mag0 * 2);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_LONG:
                                             case BridgeProtocolEngine.IT_ENUM:
                                             {
                                                  wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       int[] a = new int[mag0];

                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            a[j] = parmFilter.readInt();
                                                       }

                                                       ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_long_array(a);
                                                  }
                                                  else
                                                  {
                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            os.write_long(parmFilter.readInt());
                                                       }
                                                  }

                                                  parmOff += (mag0 * 4);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_FLOAT:
                                             {
                                                  wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       float[] a = new float[mag0];

                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            a[j] = (float)parmFilter.readDouble();
                                                       }

                                                       ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_float_array(a);
                                                  }
                                                  else
                                                  {
                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            os.write_float((float)parmFilter.readDouble());
                                                       }
                                                  }

                                                  parmOff += (mag0 * 8);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_DOUBLE:
                                             {
                                                  wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       double[] a = new double[mag0];

                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            a[j] = parmFilter.readDouble();
                                                       }

                                                       ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_double_array(a);
                                                  }
                                                  else
                                                  {
                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            os.write_double(parmFilter.readDouble());
                                                       }
                                                  }

                                                  parmOff += (mag0 * 8);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_STRING:
                                             {
                                                  for (int j=0;j<mag0;j++)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                       repFactor = repFactorHolder.value;
                                                       String s = encoder.toString(payloadArray,parmOff,repFactor-1);
                                                       parmOff += repFactor;
                                                       parmBuffer.skip(repFactor);
                                                       os.write_string(s);
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_DATETIME:
                                             {
                                                  wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       long[] a = new long[mag0];

                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            double sas = parmFilter.readDouble();
                                                            a[j] = BridgeProtocolEngine.sasToCorba(sas);
                                                       }

                                                       ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_ulonglong_array(a);
                                                  }
                                                  else
                                                  {
                                                       for (int j=0;j<mag0;j++)
                                                       {
                                                            double sas = parmFilter.readDouble();
                                                            os.write_ulonglong(BridgeProtocolEngine.sasToCorba(sas));
                                                       }
                                                  }

                                                  parmOff += (mag0 * 8);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_UUID:
                                             {
                                                  for (int j=0;j<mag0;j++)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                       repFactor = repFactorHolder.value;
                                                       String s = encoder.toString(payloadArray,parmOff,repFactor-1);
                                                       parmOff += repFactor;
                                                       parmBuffer.skip(repFactor);
                                                       BridgeProtocolEngine.stringToUuid(s,os);
                                                  }

                                                  break;
                                             }
                                        }
                                   }

                                   any.read_value(os.create_input_stream(),tc);
                                   break;
                              }

                              case 2: // 2D arrays
                              {
                                   wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                   parmFilter.readShort();
                                   parmOff += 2;
                                   wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                   int mag0 = parmFilter.readInt();
                                   int mag1 = parmFilter.readInt();
                                   parmOff += 8;
                                   org.omg.CORBA.portable.OutputStream os = any.create_output_stream();
                                   os.write_long(mag0);

                                   if (mag0 > 0)
                                   {
                                        switch (iomType)
                                        {
                                             case BridgeProtocolEngine.IT_BOOLEAN:
                                             {
                                                  if (mag1 > 0)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                  }

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);
                                                            boolean[] a = new boolean[mag1];

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 a[j1] = (parmBuffer.read() != 0);
                                                            }

                                                            ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_boolean_array(a);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 os.write_boolean(parmBuffer.read() != 0);
                                                            }
                                                       }
                                                  }

                                                  parmOff += (mag0 * mag1);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_OCTET:
                                             {
                                                  if (mag1 > 0)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                  }

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);
                                                            byte[] a = new byte[mag1];

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 a[j1] = parmFilter.readByte();
                                                            }

                                                            ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_octet_array(a);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 os.write_octet(parmFilter.readByte());
                                                            }
                                                       }
                                                  }

                                                  parmOff += (mag0 * mag1);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_SHORT:
                                             {
                                                  if (mag1 > 0)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                  }

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);
                                                            short[] a = new short[mag1];

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 a[j1] = parmFilter.readShort();
                                                            }

                                                            ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_short_array(a);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 os.write_short(parmFilter.readShort());
                                                            }
                                                       }
                                                  }

                                                  parmOff += (mag0 * mag1 * 2);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_LONG:
                                             case BridgeProtocolEngine.IT_ENUM:
                                             {
                                                  if (mag1 > 0)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                  }

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);
                                                            int[] a = new int[mag1];

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 a[j1] = parmFilter.readInt();
                                                            }

                                                            ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_long_array(a);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 os.write_long(parmFilter.readInt());
                                                            }
                                                       }
                                                  }

                                                  parmOff += (mag0 * mag1 * 4);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_FLOAT:
                                             {
                                                  if (mag1 > 0)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                  }

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);
                                                            float[] a = new float[mag1];

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 a[j1] = (float)parmFilter.readDouble();
                                                            }

                                                            ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_float_array(a);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 os.write_float((float)parmFilter.readDouble());
                                                            }
                                                       }
                                                  }

                                                  parmOff += (mag0 * mag1 * 8);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_DOUBLE:
                                             {
                                                  if (mag1 > 0)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                  }

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);
                                                            double[] a = new double[mag1];

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 a[j1] = parmFilter.readDouble();
                                                            }

                                                            ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_double_array(a);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 os.write_double(parmFilter.readDouble());
                                                            }
                                                       }
                                                  }

                                                  parmOff += (mag0 * mag1 * 8);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_STRING:
                                             {
                                                  for (int j0=0;j0<mag0;j0++)
                                                  {
                                                       os.write_long(mag1);

                                                       for (int j1=0;j1<mag1;j1++)
                                                       {
                                                            wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                            repFactor = repFactorHolder.value;
                                                            String s = encoder.toString(payloadArray,parmOff,repFactor-1);
                                                            parmOff += repFactor;
                                                            parmBuffer.skip(repFactor);
                                                            os.write_string(s);
                                                       }
                                                  }

                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_DATETIME:
                                             {
                                                  if (mag1 > 0)
                                                  {
                                                       wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                  }

                                                  if (os instanceof com.sas.net.brg.orb.OutputStreamImpl)
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);
                                                            long[] a = new long[mag1];

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 double sas = parmFilter.readDouble();
                                                                 a[j1] = BridgeProtocolEngine.sasToCorba(sas);
                                                            }

                                                            ((com.sas.net.brg.orb.OutputStreamImpl)os).insert_ulonglong_array(a);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       for (int j0=0;j0<mag0;j0++)
                                                       {
                                                            os.write_long(mag1);

                                                            for (int j1=0;j1<mag1;j1++)
                                                            {
                                                                 double sas = parmFilter.readDouble();
                                                                 os.write_ulonglong(BridgeProtocolEngine.sasToCorba(sas));
                                                            }
                                                       }
                                                  }

                                                  parmOff += (mag0 * mag1 * 8);
                                                  break;
                                             }

                                             case BridgeProtocolEngine.IT_UUID:
                                             {
                                                  for (int j0=0;j0<mag0;j0++)
                                                  {
                                                       os.write_long(mag1);

                                                       for (int j1=0;j1<mag1;j1++)
                                                       {
                                                            wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
                                                            repFactor = repFactorHolder.value;
                                                            String s = encoder.toString(payloadArray,parmOff,repFactor-1);
                                                            parmOff += repFactor;
                                                            parmBuffer.skip(repFactor);
                                                            BridgeProtocolEngine.stringToUuid(s,os);
                                                       }
                                                  }

                                                  break;
                                             }
                                        }
                                   }

                                   any.read_value(os.create_input_stream(),tc);
                                   break;
                              }
                         }
                    }
               }
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while unmarshalling an inbound parameters.
               String messageIndex = "sam.S201.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }
     }

     public void releaseParameters(ParameterHandleInterface parmHandle)
     {
          returnInboundPacket((InboundPacket)parmHandle);
     }

     public synchronized void close()
     {
          if (socket == null)
          {
               return;
          }

          // send a CLOSE packet
          OutboundPacket out = getOutboundPacket();
          out.setType(Packet.INTERNAL);
          out.setMethodId(CLOSE);

          try
          {
               sendPacket(out);
          }
          catch (COMM_FAILURE e)
          {
          }

          try
          {
               socket.close();
          }
          catch (IOException e)
          {
          }

          socket = null;
     }

     public void run()
     {
          SystemException systemException = null;

          while (true)
          {
               try
               {
                    switch (state)
                    {
                         case REQUIREMENTS:
                         {
                              ByteArrayOutputStream byteOutput = new ByteArrayOutputStream(50);
                              byte[] peerReqs = new byte[65];

                              try
                              {
                                   socketInputFilter.readFully(peerReqs,0,33);

                                   if (isLoggable(logger,fineRecord))
                                   {
                                        logger.fine("Read 33 bytes for peer's requirements.");
                                        logger.fine("PE_VERSION: " + Integer.toString(peerReqs[0]));
                                   }
                              }
                              catch (IOException iox)
                              {
                                   // {0}: An exception was thrown while reading the peer requirements vector.
                                   String messageIndex = "sam.S202.ex.msg"/*I18NOK:EMS*/;
                                   String messagePattern = RB.getStringResource(messageIndex);
                                   String message = Message.format(messagePattern,messageIndex);
                                   COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                                   comm_failure.initCause(iox);
                                   throw comm_failure;
                              }

                              if (peerReqs[32] != PE_ENCR_NONE)
                              {
                                   try
                                   {
                                        socketInputFilter.readFully(peerReqs,33,1);
                                        logger.fine("Read length of encryption algorithm list: " + Integer.toString(peerReqs[33]));
                                        socketInputFilter.readFully(peerReqs,34,peerReqs[33]);
                                        logger.fine("Read encryption algorithm list: " + encoder.toString(peerReqs,34,peerReqs[33]));
                                   }
                                   catch (IOException iox)
                                   {
                                        // {0}: An exception was thrown while reading peer encryption requirements.
                                        String messageIndex = "sam.S203.ex.msg"/*I18NOK:EMS*/;
                                        String messagePattern = RB.getStringResource(messageIndex);
                                        String message = Message.format(messagePattern,messageIndex);
                                        COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                                        comm_failure.initCause(iox);
                                        throw comm_failure;
                                   }

                                   byteOutput.write(PE_REQS_FAILED);
                                   byteOutput.write(0);

                                   try
                                   {
                                        byteOutput.writeTo(socketOutput);
                                        socketOutput.flush();
                                   }
                                   catch (IOException iox)
                                   {
                                        // {0}: An exception was thrown while sending encryption requirements: {2}
                                        String messageIndex = "sam.S204.ex.msg"/*I18NOK:EMS*/;
                                        String messagePattern = RB.getStringResource(messageIndex);
                                        String message = Message.format(messagePattern,messageIndex);
                                        COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                                        comm_failure.initCause(iox);
                                        throw comm_failure;
                                   }

                                   break;
                              }

                              if (peerReqs[0] != PE_VERSION)
                              {
                                   // {0}: Unknown protocol version: {1}
                                   String messageIndex = "sam.S205.ex.msg"/*I18NOK:EMS*/;
                                   String messagePattern = RB.getStringResource(messageIndex);
                                   String message = Message.format(messagePattern,messageIndex,Integer.toString(peerReqs[0]));
                                   NO_IMPLEMENT no_implement = new NO_IMPLEMENT(message);
                                   throw no_implement;
                              }

                              byte[] reqVec = getRequirementsVector();

                              if (peerReqs[25] != PE_ROW_MAJOR || !requirementsEqual(reqVec,0,peerReqs,1,reqVec.length))
                              {
                                   if (peerReqs[26] == PE_NO_CAP)
                                   {
                                        // {0}: Neither the client nor the server is capable of marshalling data.
                                        String messageIndex = "sam.S206.ex.msg"/*I18NOK:EMS*/;
                                        String messagePattern = RB.getStringResource(messageIndex);
                                        String message = Message.format(messagePattern,messageIndex);
                                        NO_IMPLEMENT no_implement = new NO_IMPLEMENT(message);
                                        throw no_implement;
                                   }

                                   byteOutput.write(PE_REQS_REQS);
                                   byteOutput.write(reqVec,0,reqVec.length);
                                   byteOutput.write(PE_ROW_MAJOR);
                                   byteOutput.write(PE_NO_CAP);
                                   byteOutput.write(PE_USER);
                                   byteOutput.write(PE_APPL_JDK1_1);
                                   byteOutput.write(PE_PLAT_JAVA);
                                   byteOutput.write(0);
                                   byteOutput.write(0);
                                   byteOutput.write(PE_ENCR_NONE);

                                   if (isLoggable(logger,fineRecord))
                                   {
                                        logger.fine("Sending PE_REQS_REQS: " + Integer.toString(PE_REQS_REQS));
                                        logger.fine("Sending requirements vector, length: " + Integer.toString(reqVec.length));
                                        logger.fine("Sending PE_ROW_MAJOR: " + Integer.toString(PE_ROW_MAJOR));
                                        logger.fine("Sending PE_NO_CAP: " + Integer.toString(PE_NO_CAP));
                                        logger.fine("Sending PE_USER: " + Integer.toString(PE_USER));
                                        logger.fine("Sending PE_APPL_JDK1_1: " + Integer.toString(PE_APPL_JDK1_1));
                                        logger.fine("Sending PE_PLAT_JAVA: " + Integer.toString(PE_PLAT_JAVA));
                                        logger.fine("Sending reserved byte 1: 0");
                                        logger.fine("Sending reserved byte 2: 0");
                                        logger.fine("Sending encryption policy: " + Integer.toString(PE_ENCR_NONE));
                                   }
                              }
                              else
                              {
                                   byteOutput.write(PE_REQS_OK);
                              }

                              try
                              {
                                   byteOutput.writeTo(socketOutput);
                                   socketOutput.flush();
                                   state = AUTHENTICATION;
                              }
                              catch (IOException iox)
                              {
                                   // {0}: An exception was thrown while replying to requirements.
                                   String messageIndex = "sam.S207.ex.msg"/*I18NOK:EMS*/;
                                   String messagePattern = RB.getStringResource(messageIndex);
                                   String message = Message.format(messagePattern,messageIndex);
                                   COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                                   comm_failure.initCause(iox);
                                   throw comm_failure;
                              }

                              setupInputBuffer();
                              break;
                         }

                         case AUTHENTICATION:
                         {
                              InboundPacket in = readPacket();
                              OutboundPacket out = getOutboundPacket();

                              try
                              {
                                   if (isLoggable(logger,finerRecord))
                                   {
                                        logger.finer("Received Packet:\n" + in.toString());
                                   }

                                   List dosParameters = new ArrayList(3);
                                   reconcileDosParameters(in,dosParameters);
                                   String userName = (String)dosParameters.get(0);
                                   String password = (String)dosParameters.get(1);
                                   dosParameters.clear();
                                   out.setType(Packet.INTERNAL);
                                   out.setMethodId(USERPASS);
                                   out.setReplyId(in.getReplyId());
                                   out.setClientId(clientID);
                                   int[] dosMaps = new int[3];
                                   dosMaps[0] = DOS_MP_UINT;
                                   dosParameters.add(new int[]{0});
                                   dosMaps[1] = DOS_MP_BINARY;
                                   dosParameters.add(new byte[]{0x49,0x53,0x4f,0x38,0x38,0x35,0x39,0x5f,0x31,0x00});
                                   dosMaps[2] = DOS_MP_UCHAR;
                                   dosParameters.add(properties.getProperty("docString",""));
                                   out.setDosParameters(dosMaps,dosParameters);
                                   sendPacket(out);
                                   state = DISPATCH;
                              }
                              finally
                              {
                                   returnInboundPacket(in);
                                   returnOutboundPacket(out);
                              }

                              break;
                         }

                         case DISPATCH:
                         {
                              InboundPacket in = readPacket();

                              if (isLoggable(logger,finerRecord))
                              {
                                   String cxNumber = properties.getProperty("cxNumber");
                                   logger.finer("Received Packet on cx#" + cxNumber + ":\n" + in.toString());
                              }

                              synchronized (this)
                              {
                                   packetsToDispatch++;
                                   in.run();
                              }

                              break;
                         }
                    }
               }
               catch (SystemException se)
               {
                    systemException = se;
                    break;
               }
          }

          synchronized (this)
          {
               try
               {
                    long timer = 2000L;

                    while (packetsToDispatch > 0 && timer > 0L)
                    {
                         long start = System.currentTimeMillis();
                         wait(timer);
                         long stop = System.currentTimeMillis();
                         timer -= (stop - start);
                    }
               }
               catch (InterruptedException ie)
               {
               }

               if (packetsToDispatch != 0)
               {
                    logger.fine("Access method going away with " + packetsToDispatch + " packets left to dispatch.");
               }

               logger.finer("Peer going away: " + clientID);

               if (socket != null)
               {
                    try
                    {
                         socket.close();
                    }
                    catch (IOException e)
                    {
                    }

                    socket = null;
               }

               if (encoder != null)
               {
                    Encoder.returnInstance(encoder);
                    encoder = null;
               }

               closeCipher();
          }

          synchronized (replyList)
          {
               // don't lock this while replyList is locked
               fatalException = systemException;
               replyList.notifyAll();
          }

          engine.accessMethodExit();
     }

     private static final int REQUIREMENTS = 0;
     private static final int AUTHENTICATION = 1;
     private static final int DISPATCH = 2;
     private static final int PE_VERSION = 0x69;
     private static final int PE_ROW_MAJOR = 0x00;
     private static final int PE_NO_CAP = 0x00;
     private static final int PE_USER = 0x00;
     private static final int PE_SERVER = 0x01;
     private static final int PE_APPL_JDK1_1 = 0x06;
     private static final int PE_PLAT_JAVA = 0x09;
     private static final int PE_REQS_OK = 0x00;
     private static final int PE_REQS_FAILED = 0x01;
     private static final int PE_REQS_LEVEL = 0x02;
     private static final int PE_REQS_REQS = 0x03;
     private static final int PE_REQS_REDIRECT = 0x04;
     private static final int PE_REQS_VERSION = 0x05;
     private static final int PE_REQS_REDIRECT_2 = 0x06;
     private static final int PE_REQS_LBFAILURE = 0x07;
     private static final int PE_REQS_TIMEOUT = 0x08;
     private static final int PE_REDIRECT_ME = 0x01;
     private static final int PE_BEEN_REDIRECTED = 0x02;
     private static final int PE_ENCR_NONE = 0x00;
     private static final int PE_ENCR_AUTH = 0x01;
     private static final int PE_ENCR_ALL = 0x02;
     private static final int ERROR = 0;
     protected static final int USERPASS = 1;
     private static final int NEWOBJECT = 2;
     private static final int FREEOBJECT = 3;
     private static final int CLOSE = 4;
     private static final int UUIDFROMOBJECTID = 5;
     private static final int ADVISE = 6;
     private static final int UNADVISE = 7;
     private static final int OBJECTDESTRUCTOR = 8;
     private static final int INTERFACEINOBJECT = 9;
     private static final int EVENTSFROMOBJECTID = 10;
     private static final int DEFAULTINTERFACE = 11;
     private static final int SERVERTRANSCODE = 12;
     private static final int GENPASSWORD = 13;
     protected static final int DOS_MP_UCHAR = 2;
     protected static final int DOS_MP_SHORT = 4;
     protected static final int DOS_MP_UINT = 7;
     protected static final int DOS_MP_LONG = 8;
     protected static final int DOS_MP_ULONG = 9;
     protected static final int DOS_MP_DOUBLE = 11;
     protected static final int DOS_MP_BINARY = 28;
     private static final int WZX_REP_MASK       =        524287; /* bits 0-18       */
     private static final int WZX_ATTR_MASK      =   16252928; /* bits 19-23      */
     private static final int WZX_ATTR_UNSIGNED  =    4194304; /* bit 22 (2**22)  */
     private static final int WZX_TYPE_MASK      = 2130706432; /* bits 24-30      */
     private static final int WZX_TYPE_LONG      =   16777216; /* 0000001 (2**24) */
     private static final int WZX_TYPE_SHORT     =   33554432; /* 0000010 (2**25) */
     private static final int WZX_TYPE_INT       =   50331648; /* 0000011         */
     private static final int WZX_TYPE_FLOAT     =   67108864; /* 0000100 (2**26) */
     private static final int WZX_TYPE_DOUBLE    =   83886080; /* 0000101         */
     private static final int WZX_TYPE_CHAR      =  100663296; /* 0000110         */
     private static final int WZX_TYPE_CHAR_B    =  117440512; /* 0000111         */
     private static final int WZX_MAP32XTND      = (int)2147483648L; /* bit 31 (2**31)  */
     private static final int MARS_SPECIAL_STRING = 1;
     private static final int PacketHeaderSize = 64;
     private static final int MapSize = 4;
     private static final int uuidSize = 36;
     private static final int minimumMarsSize = 64;
     private static final int maximumMarsSize = 32767;
     private static final int minimumMapArraySize = 64;
     private static final int maximumMapArraySize = 32767;
     private static final String cryptoPolicyRequired = "required"/*I18NOK:EMS*/;
     private static final String cryptoPolicyOptional = "optional"/*I18NOK:EMS*/;
     private static final String cryptoPolicyNone = "none"/*I18NOK:EMS*/;
     private static final String cryptoPolicyDefault = cryptoPolicyNone;
     private static final String cryptoContentAll = "all"/*I18NOK:EMS*/;
     private static final String cryptoContentAuthentication = "authentication"/*I18NOK:EMS*/;
     private static final String cryptoContentDefault = cryptoContentAll;
     private static final String transcodingPreferenceClient = "client"/*I18NOK:EMS*/;
     private static final String transcodingPreferenceServer = "server"/*I18NOK:EMS*/;
     private static final String transcodingPreferenceDefault = transcodingPreferenceClient;
     private static final LogRecord fineRecord =   new LogRecord(Level.FINE,null);
     private static final LogRecord finerRecord =  new LogRecord(Level.FINER,null);
     private static final LogRecord finestRecord = new LogRecord(Level.FINEST,null);
     private static byte[] requirementsVector = null;
     private static int[] replyRC = {0};
     private static int[] userExRC = {-1};
     private BridgeProtocolEngine engine /* = null */;
     private Properties properties /* = null */;
     private Socket socket /* = null */;
     private OutputStream socketOutput /* = null */;
     private InputStream socketInput /* = null */;
     private DataInputStream socketInputFilter /* = null */;
     private Thread reader /* = null */;
     private List replyList /* = null */;
     private SystemException fatalException /* = null */;
     private DocumentBuilderFactory errorDocBuilderFactory /* = null */;
     private DocumentBuilder errorDocBuilder /* = null */;
     private int state /* = 0 */;
     private Logger logger /* = null */;
     private boolean client = true;
     private int clientID /* = 0 */;
     private int encryptionPolicy /* = 0 */;
     private int transcodingPreferenceFlag /* = 0 */;
     private Crypto crypto /* = null */;
     private CipherInterface cipher /* = null */;
     private int encryptedPacketHeaderSize = PacketHeaderSize;
     private PatchableByteArrayOutputStream outputBuffer /* = null */;
     private DataOutputStream outputFilter /* = null */;
     private int[][] maps /* = null */;
     private Object[] mars /* = null */;
     private ByteArrayOutputStream patchBuffer /* = null */;
     private DataOutputStream patchFilter /* = null */;
     private byte[] headerInputArray /* = null */;
     private ByteArrayInputStream headerInputBuffer /* = null */;
     private DataInputStream headerInputFilter /* = null */;
     private byte[] uuidInputArray /* = null */;
     private EncoderInterface encoder /* = null */;
     private int replyIDCounter /* = 0 */;
     private int packetsToDispatch /* = 0 */;

     private void handleDebugLevel()
     {
          Object logFile = properties.get("logFile"/*I18NOK:EMS*/);

          if (logFile == null)
          {
               logFile = new PrintWriter(System.out);
          }

          if (logFile instanceof Logger)
          {
               logger = (Logger)logFile;
          }
          else if (logFile instanceof Writer)
          {
               VerySimpleFormatter formatter = new VerySimpleFormatter();
               WriterHandler handler = new WriterHandler((Writer)logFile,formatter);
               logger = Logger.getAnonymousLogger();
               logger.addHandler(handler);
               logger.setLevel(Level.OFF);
               logger.setUseParentHandlers(false);
          }
          else if (logFile instanceof String)
          {
               FileHandler handler = null;

               try
               {
                    handler = new FileHandler((String)logFile);
               }
               catch (IOException e)
               {
                    IllegalArgumentException iae = new IllegalArgumentException("An exception was thrown while opening the log file.");
                    iae.initCause(e);
                    throw iae;
               }

               logger = Logger.getAnonymousLogger();
               logger.addHandler(handler);
               logger.setLevel(Level.OFF);
               logger.setUseParentHandlers(false);
          }
          else
          {
               throw new IllegalArgumentException("\"logFile\" must be a String, Writer, or Logger.");
          }

          String dl = properties.getProperty("debugLevel"/*I18NOK:EMS*/);

          if (dl != null && dl.length() > 0)
          {
               int debugLevel = 0;

               try
               {
                    debugLevel = Integer.parseInt(dl);
               }
               catch (NumberFormatException e)
               {
                    IllegalArgumentException iae = new IllegalArgumentException("\"debugLevel\" must be an integer.");
                    iae.initCause(e);
                    throw iae;
               }

               switch (debugLevel)
               {
                    case 1:logger.setLevel(Level.FINE);break;
                    case 2:logger.setLevel(Level.FINER);break;
                    case 3:logger.setLevel(Level.FINEST);break;
               }
          }
     }

     private void startPacketReader()
     {
          reader = BaseSecuritySupport.securitySupport.createThread(this);

          PrivilegedAction action = new PrivilegedAction()
          {
               public Object run()
               {
                    reader.setName("com.sas.net.brg.sam.SocketAccessMethod.inbound_packet_reader"/*I18NOK:EMS*/);
                    reader.setDaemon(true);
                    return null;
               }

          };

          BaseSecuritySupport.securitySupport.doThreadPrivileged(action);
          reader.start();
     }

     private void openStreams()
     {
          try
          {
               socket.setSoLinger(true,1);
          }
          catch (IOException e)
          {
               // {0}: An exception was thrown while setting the SO_LINGER value on the socket.
               String messageIndex = "sam.S208.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               COMM_FAILURE comm_failure = new COMM_FAILURE(message);
               comm_failure.initCause(e);
               throw comm_failure;
          }

          try
          {
               socket.setTcpNoDelay(true);
          }
          catch (IOException e)
          {
               // {0}: An exception was thrown while setting the TCP_NODELAY value on the socket.
               String messageIndex = "sam.S209.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               COMM_FAILURE comm_failure = new COMM_FAILURE(message);
               comm_failure.initCause(e);
               throw comm_failure;
          }

          try
          {
               socketInput = socket.getInputStream();
               socketOutput = socket.getOutputStream();
               socketInputFilter = new DataInputStream(socketInput);
          }
          catch (IOException e)
          {
               // {0}: An exception was thrown while opening streams on the socket.
               String messageIndex = "sam.S210.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               COMM_FAILURE comm_failure = new COMM_FAILURE(message);
               comm_failure.initCause(e);
               throw comm_failure;
          }
     }

     private void establishConnection()
     {
          // get my requirements vector
          byte[] requirementsVector = getRequirementsVector();

          // what's the transcoding preference ?
          //------------------------------------
          String transcodingPreference = properties.getProperty("transcodingPreference"/*I18NOK:EMS*/,transcodingPreferenceDefault);

          if (transcodingPreference.equals(transcodingPreferenceClient))
          {
               transcodingPreferenceFlag = PE_USER;
          }
          else if (transcodingPreference.equals(transcodingPreferenceServer))
          {
               transcodingPreferenceFlag = PE_SERVER;
          }
          else
          {
               // {0}: The value for {1} is invalid: {2}
               String messageIndex = "sam.S213.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex,"transcodingPreference"/*I18NOK:EMS*/,transcodingPreference);
               throw new BAD_PARAM(message);
          }

          // open the socket connection
          String host = properties.getProperty("host"/*I18NOK:EMS*/);
          int port = Integer.parseInt(properties.getProperty("port"/*I18NOK:EMS*/));

          // I loop over the handshaking algorithm until I have sent
          // the server something it can live with. If I run out of
          // possibilities, then I just fail the connection.
          //-----------------------------------------------
          byte requirementsStatus = PE_REQS_FAILED;

          // Send a flag to indicate if this connection has been redirected yet.
          // It shouldn't be redirected more than once.
          //--------------------------------------------------------------------
          int redirectMe = PE_REDIRECT_ME;

          do
          {
               // determine what the encryption policy should be
               String cryptoPolicy = properties.getProperty("encryptionPolicy"/*I18NOK:EMS*/,cryptoPolicyDefault);
               String cryptoContent = properties.getProperty("encryptionContent"/*I18NOK:EMS*/,cryptoContentDefault);
               String cryptoAlgorithms = properties.getProperty("encryptionAlgorithms"/*I18NOK:EMS*/);
               boolean artificialAlgorithms = false;

               if (cryptoPolicy.equals(cryptoPolicyNone))
               {
                    encryptionPolicy = PE_ENCR_NONE;
               }
               else
               {
                    if (!cryptoPolicy.equals(cryptoPolicyRequired) && !cryptoPolicy.equals(cryptoPolicyOptional))
                    {
                         // {0}: The value for {1} is invalid: {2}
                         String messageIndex = "sam.S211.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex,"encryptionPolicy"/*I18NOK:EMS*/,cryptoPolicy);
                         throw new BAD_PARAM(message);
                    }

                    if (cryptoContent.equals(cryptoContentAuthentication))
                    {
                         encryptionPolicy = PE_ENCR_AUTH;
                    }
                    else if (cryptoContent.equals(cryptoContentAll))
                    {
                         encryptionPolicy = PE_ENCR_ALL;
                    }
                    else
                    {
                         // {0}: The value for {1} is invalid: {2}
                         String messageIndex = "sam.S212.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex,"encryptionContent"/*I18NOK:EMS*/,cryptoContent);
                         throw new BAD_PARAM(message);
                    }
               }

               StringTokenizer algTokenizer = null;

               if (cryptoAlgorithms != null && cryptoAlgorithms.length() > 0)
               {
                    String trimmedCryptoAlgorithms = cryptoAlgorithms.trim();
                    algTokenizer = new StringTokenizer(trimmedCryptoAlgorithms,","/*I18NOK:EMS*/);

                    if (algTokenizer.hasMoreTokens() == false)
                    {
                         algTokenizer = null;
                    }
               }

               try
               {
                    socket = new Socket(host,port);
               }
               catch (ConnectException ce)
               {
                    // {0}: A connection to server host {1} could not be established on port {2}. Either
                    // there is no server listening for connections or the server is too busy to accept
                    // any new connections. Make sure that the server has started and that the values
                    // provided for host and port are correct.
                    String messageIndex = "sam.S249.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex,host,Integer.toString(port));
                    COMM_FAILURE comm_failure = new COMM_FAILURE(message,1,CompletionStatus.COMPLETED_NO);
                    comm_failure.initCause(ce);
                    throw comm_failure;
               }
               catch (UnknownHostException uhe)
               {
                    // {0}: The server host name {1} could not be resolved to an IP address. Make sure
                    // that the value provided is correct.
                    String messageIndex = "sam.S250.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex,host);
                    COMM_FAILURE comm_failure = new COMM_FAILURE(message,1,CompletionStatus.COMPLETED_NO);
                    comm_failure.initCause(uhe);
                    throw comm_failure;
               }
               catch (IOException e)
               {
                    // {0}: An exception was thrown while opening the socket.
                    String messageIndex = "sam.S214.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    COMM_FAILURE comm_failure = new COMM_FAILURE(message,1,CompletionStatus.COMPLETED_NO);
                    comm_failure.initCause(e);
                    throw comm_failure;
               }
               catch (IllegalArgumentException iae)
               {
                    // {0}: An exception was thrown while opening the socket.
                    // Make sure the value for host is not null and the value for port is between 0 and 65535.
                    String messageIndex = "sam.S248.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    COMM_FAILURE comm_failure = new COMM_FAILURE(message,1,CompletionStatus.COMPLETED_NO);
                    comm_failure.initCause(iae);
                    throw comm_failure;
               }

               // prepare to read, write, and buffer data
               openStreams();
               ByteArrayOutputStream byteOutput = new ByteArrayOutputStream(50);

               do
               {
                    // send version number
                    byteOutput.write(PE_VERSION);

                    // send our requirements vector
                    byteOutput.write(requirementsVector,0,requirementsVector.length);

                    // send PE_ROW_MAJOR to indicate row-major order on 2d arrays
                    byteOutput.write(PE_ROW_MAJOR);

                    // send PE_NO_CAP to indicate that we cannot do WZX translations
                    byteOutput.write(PE_NO_CAP);

                    // send NLS requirements indicator
                    byteOutput.write(transcodingPreferenceFlag);

                    // send PE_APPL_JDK1_1 to indicate we are running on JDK11 or better
                    byteOutput.write(PE_APPL_JDK1_1);

                    // send PE_PLAT_JAVA to indicate we are running on Java
                    byteOutput.write(PE_PLAT_JAVA);

                    // reserved byte 1
                    byteOutput.write(0);

                    // send PE_REDIRECT_ME if we haven't been redirected yet
                    byteOutput.write(redirectMe);

                    // send byte to indicate encryption policy
                    byteOutput.write(encryptionPolicy);

                    if (isLoggable(logger,fineRecord))
                    {
                         logger.fine("Sent version number: " + Integer.toString(PE_VERSION));
                         logger.fine("Sent requirements vector");
                         logger.fine("Sent 2d array row major order indicator: " + Integer.toString(PE_ROW_MAJOR));
                         logger.fine("Sent WZX capabilities indicator: " + Integer.toString(PE_NO_CAP));
                         String xcode = (transcodingPreferenceFlag==PE_USER) ? "PE_USER" : "PE_SERVER";
                         logger.fine("Sent transcoding preference indicator: " + xcode);
                         logger.fine("Sent application type: " + Integer.toString(PE_APPL_JDK1_1));
                         logger.fine("Sent platform type: " + Integer.toString(PE_PLAT_JAVA));
                         logger.fine("Sent reserved byte: 0");
                         String redir = null;

                         switch (redirectMe)
                         {
                              case PE_REDIRECT_ME:     redir = "PE_REDIRECT_ME";break;
                              case PE_BEEN_REDIRECTED: redir = "PE_BEEN_REDIRECTED";break;
                              default:                 redir = Integer.toString(redirectMe);break;
                         }

                         logger.fine("Sent redirectable indicator: " + redir);
                         logger.fine("Sent encryption policy indicator: " + Integer.toString(encryptionPolicy));
                    }

                    // If the user wants encryption, choose an encryption algorithm from
                    // the list of algorithms, make sure we can support it, and send its
                    // length and name (in ISO8859_1) to the server. If the server can't
                    // support it, then we'll send another name from the list until the
                    // list is exhausted. If there is no list, then we send a name length
                    // of 0, and the server will send a list of its preferred algorithms.
                    //--------------------------------------------------------------------
                    String algName = null;

                    if (encryptionPolicy != PE_ENCR_NONE)
                    {
                         if (algTokenizer != null)
                         {
                              do
                              {
                                   algName = algTokenizer.nextToken().trim();
                                   crypto = Crypto.getCrypto(algName,properties);

                                   if (crypto != null)
                                   {
                                        break;
                                   }
                              }
                              while (algTokenizer.hasMoreTokens());

                              if (crypto == null)
                              {
                                   if (cryptoPolicy.equals(cryptoPolicyOptional))
                                   {
                                        encryptionPolicy = PE_ENCR_NONE;
                                   }
                                   else
                                   {
                                        // {0}: None of the specified encryption algorithms ({1}) are supported by both the client and server.
                                        String messageIndex = "sam.S215.ex.msg"/*I18NOK:EMS*/;
                                        String messagePattern = RB.getStringResource(messageIndex);
                                        String message = Message.format(messagePattern,messageIndex,cryptoAlgorithms);
                                        throw new BAD_PARAM(message);
                                   }
                              }
                         }

                         if (crypto != null)
                         {
                              byte[] algNameBytes = encoder.toByteArray(algName,0,algName.length());
                              byteOutput.write(algNameBytes.length);
                              byteOutput.write(algNameBytes,0,algNameBytes.length);

                              if (isLoggable(logger,fineRecord))
                              {
                                   logger.fine("Sent encryption algorithm name length: " + algNameBytes.length);
                                   logger.fine("Sent encryption algorithm name: " + algName);
                              }
                         }
                         else
                         {
                              byteOutput.write(0);
                              logger.fine("Sent encryption algorithm name length: " + 0);
                         }
                    }

                    // flush the buffer to the socket
                    try
                    {
                         byteOutput.writeTo(socketOutput);
                         socketOutput.flush();
                         byteOutput.reset();
                    }
                    catch (IOException e)
                    {
                         // {0}: An exception was thrown while writing the client requirements to the socket.
                         String messageIndex = "sam.S216.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex);
                         COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                         comm_failure.initCause(e);
                         throw comm_failure;
                    }

                    // read requirements status
                    try
                    {
                         requirementsStatus = socketInputFilter.readByte();
                    }
                    catch (IOException e)
                    {
                         // {0}: An exception was thrown while reading the server''s requirements status from the socket.
                         String messageIndex = "sam.S217.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex);
                         COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                         comm_failure.initCause(e);
                         throw comm_failure;
                    }

                    logger.fine("Received requirements status: " + requirementsStatus);

                    if (requirementsStatus == PE_REQS_OK)
                    {
                         // skip this big if/then/else statement
                    }
                    else if (requirementsStatus == PE_REQS_LEVEL)
                    {
                         try
                         {
                              encryptionPolicy = socketInputFilter.readByte();
                         }
                         catch (IOException ioe)
                         {
                              // {0}: An exception was thrown while reading the server''s required encryption level.
                              String messageIndex = "sam.S253.ex.msg"/*I18NOK:EMS*/;
                              String messagePattern = RB.getStringResource(messageIndex);
                              String message = Message.format(messagePattern,messageIndex);
                              COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                              comm_failure.initCause(ioe);
                              throw comm_failure;
                         }

                         logger.fine("Received server's required encryption level: " + encryptionPolicy);
                         cryptoPolicy = cryptoPolicyRequired; // prevents infinite loop if current policy is optional

                         if (cryptoAlgorithms != null && cryptoAlgorithms.length() > 0)
                         {
                              String trimmedCryptoAlgorithms = cryptoAlgorithms.trim();
                              algTokenizer = new StringTokenizer(trimmedCryptoAlgorithms,","/*I18NOK:EMS*/);

                              if (algTokenizer.hasMoreTokens() == false)
                              {
                                   algTokenizer = null;
                              }
                         }

                         if (algTokenizer == null)
                         {
                              algTokenizer = new StringTokenizer("sasproprietary"/*I18NOK:EMS*/,","/*I18NOK:EMS*/);
                              artificialAlgorithms = true;
                              logger.fine("Synthesizing request for sasproprietary");
                         }
                    }
                    else if (requirementsStatus == PE_REQS_FAILED)
                    {
                         int serverListLen = 0;

                         try
                         {
                              serverListLen = socketInputFilter.readByte();
                         }
                         catch (IOException e)
                         {
                              // {0}: An exception was thrown while reading the encryption algorithm list length from the socket.
                              String messageIndex = "sam.S218.ex.msg"/*I18NOK:EMS*/;
                              String messagePattern = RB.getStringResource(messageIndex);
                              String message = Message.format(messagePattern,messageIndex);
                              COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                              comm_failure.initCause(e);
                              throw comm_failure;
                         }

                         logger.fine("Received encryption algorithm name list length: " + serverListLen);
                         ByteArrayOutputStream tByteStream = new ByteArrayOutputStream(36);
                         byte[] tBuf = new byte[14];
                         int tLen = 0;

                         for (int zzz=0;zzz<serverListLen;zzz++)
                         {
                              if (zzz > 0)
                              {
                                   tByteStream.write((int)',');
                              }

                              try
                              {
                                   tLen = socketInputFilter.readByte();
                              }
                              catch (IOException e)
                              {
                                   // {0}: An exception was thrown while reading an encryption algorithm name''s length from the socket.
                                   String messageIndex = "sam.S219.ex.msg"/*I18NOK:EMS*/;
                                   String messagePattern = RB.getStringResource(messageIndex);
                                   String message = Message.format(messagePattern,messageIndex);
                                   COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                                   comm_failure.initCause(e);
                                   throw comm_failure;
                              }

                              logger.fine("Received encryption algorithm name length: " + tLen);

                              if (tBuf.length < tLen)
                              {
                                   tBuf = new byte[tLen];
                              }

                              try
                              {
                                   //socketInputFilter.readFully(tBuf,0,tLen);
                                   readFully(socketInput,tBuf,0,tLen);
                              }
                              catch (IOException e)
                              {
                                   // {0}: An exception was thrown while reading an encryption algorithm name from the socket.
                                   String messageIndex = "sam.S220.ex.msg"/*I18NOK:EMS*/;
                                   String messagePattern = RB.getStringResource(messageIndex);
                                   String message = Message.format(messagePattern,messageIndex);
                                   COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                                   comm_failure.initCause(e);
                                   throw comm_failure;
                              }

                              if (isLoggable(logger,fineRecord))
                              {
                                   String tStr = encoder.toString(tBuf,0,tLen);
                                   logger.fine("Received encryption algorithm name: " + tStr);
                              }

                              tByteStream.write(tBuf,0,tLen);
                         }

                         if ((algTokenizer == null || artificialAlgorithms) && tByteStream.size() > 0)
                         {
                              artificialAlgorithms = false;
                              byte[] serverListBytes = tByteStream.toByteArray();
                              String serverList = encoder.toString(serverListBytes,0,serverListBytes.length);
                              algTokenizer = new StringTokenizer(serverList.trim(),","/*I18NOK:EMS*/);

                              if (algTokenizer.hasMoreTokens() == false)
                              {
                                   algTokenizer = null;
                              }
                         }

                         if (algTokenizer == null || algTokenizer.hasMoreTokens() == false)
                         {
                              if (cryptoPolicy.equals(cryptoPolicyOptional))
                              {
                                   encryptionPolicy = PE_ENCR_NONE;
                              }
                              else
                              {
                                   // {0}: None of the specified encryption algorithms ({1}) are supported by both the client and server.
                                   String messageIndex = "sam.S221.ex.msg"/*I18NOK:EMS*/;
                                   String messagePattern = RB.getStringResource(messageIndex);
                                   String message = Message.format(messagePattern,messageIndex,cryptoAlgorithms);
                                   throw new BAD_PARAM(message);
                              }
                         }
                    }
                    else if (requirementsStatus == PE_REQS_REDIRECT)
                    {
                         try
                         {
                              StringBuffer hostBuffer = new StringBuffer();

                              for (int zzz=0;zzz<4;zzz++)
                              {
                                   if (zzz > 0)
                                   {
                                        hostBuffer.append('.');
                                   }

                                   hostBuffer.append(socketInputFilter.readUnsignedByte());
                              }

                              host = hostBuffer.toString();
                              port = socketInputFilter.readUnsignedShort();
                              logger.fine("Redirecting to " + host + ":" + port + ".");
                         }
                         catch (IOException ioe)
                         {
                              // {0}: An exception was thrown while reading the redirect host and port from the socket.
                              String messageIndex = "sam.S243.ex.msg"/*I18NOK:EMS*/;
                              String messagePattern = RB.getStringResource(messageIndex);
                              String message = Message.format(messagePattern,messageIndex);
                              COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                              comm_failure.initCause(ioe);
                              throw comm_failure;
                         }

                         try
                         {
                              socket.close();
                         }
                         catch (IOException ioe)
                         {
                              logger.log(Level.FINER,"Ignoring this exception caught while closing redirecting socket.",ioe);
                         }

                         socket = null;
                         crypto = null;
                    }
                    else if (requirementsStatus == PE_REQS_REDIRECT_2)
                    {
                         try
                         {
                              int hostLen = socketInputFilter.readInt();
                              byte[] hostBuf = new byte[hostLen];
                              //socketInputFilter.readFully(hostBuf,0,hostLen);
                              readFully(socketInput,hostBuf,0,hostLen);
                              host = encoder.toString(hostBuf,0,hostLen);
                              port = socketInputFilter.readInt();
                              logger.fine("Redirecting to " + host + ":" + port + ".");
                         }
                         catch (IOException ioe)
                         {
                              // {0}: An exception was thrown while reading the redirect host and port from the socket.
                              String messageIndex = "sam.S244.ex.msg"/*I18NOK:EMS*/;
                              String messagePattern = RB.getStringResource(messageIndex);
                              String message = Message.format(messagePattern,messageIndex);
                              COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                              comm_failure.initCause(ioe);
                              throw comm_failure;
                         }

                         try
                         {
                              socket.close();
                         }
                         catch (IOException ioe)
                         {
                              logger.log(Level.FINER,"Ignoring this exception caught while closing redirecting socket.",ioe);
                         }

                         socket = null;
                         crypto = null;
                         redirectMe = PE_BEEN_REDIRECTED;
                    }
                    else if (requirementsStatus == PE_REQS_LBFAILURE)
                    {
                         // {0}: A problem occurred while connecting to a load balancing spawner. Check the \
                         // spawner log for more details.
                         String messageIndex = "sam.S251.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex);
                         throw new COMM_FAILURE(message);
                    }
                    else if (requirementsStatus == PE_REQS_TIMEOUT)
                    {
                         // {0}: The connection to the load balancing spawner timed out.
                         String messageIndex = "sam.S252.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex);
                         throw new COMM_FAILURE(message,7,CompletionStatus.COMPLETED_NO);
                    }
                    else
                    {
                         // PE_REQS_REQS should never happen because I sent PE_NO_CAP
                         // PE_REQS_VERSION should never happen because I have the lowest protocol version
                         // any other value is unexpected
                         // {0}: Received unexpected response from server: {1}
                         String messageIndex = "sam.S245.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex,Integer.toString(requirementsStatus));
                         throw new INTERNAL(message);
                    }
               }
               while (requirementsStatus == PE_REQS_FAILED || requirementsStatus == PE_REQS_LEVEL);
          }
          while (requirementsStatus == PE_REQS_REDIRECT || requirementsStatus == PE_REQS_REDIRECT_2);

          if (encryptionPolicy != PE_ENCR_NONE)
          {
               try
               {
                    cipher = crypto.openCipher();
                    DataOutputStream dataOutput = new DataOutputStream(socketOutput);
                    cipher.keyExchange(socketInputFilter,dataOutput);
               }
               catch (CryptoException c)
               {
                   //{0}: The encryption key exchange failed: {1}
                    String messageIndex = "sam.S222.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex,c.getMessage());
                    DATA_CONVERSION data_conversion = new DATA_CONVERSION(message);
                    data_conversion.initCause(c);
                    throw data_conversion;
               }
               catch (IOException e)
               {
                    // {0}: An exception was thrown during the encryption key exchange.
                    String messageIndex = "sam.S223.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    COMM_FAILURE comm_failure = new COMM_FAILURE(message);
                    comm_failure.initCause(e);
                    throw comm_failure;
               }
          }

          setupInputBuffer();
     }

     private synchronized static byte[] getRequirementsVector()
     {
          if (requirementsVector == null)
          {
               requirementsVector = new byte[24];
               requirementsVector[0] = 24;
               requirementsVector[1] = 0;
               requirementsVector[2] = 0x10;
               requirementsVector[3] = 0x10;
               requirementsVector[4] = 0x20;
               requirementsVector[5] = 0x20;
               requirementsVector[6] = 0;
               requirementsVector[7] = 0x30;
               requirementsVector[8] = 0;
               requirementsVector[9] = 0;
               requirementsVector[10] = 0x02;
               requirementsVector[11] = (byte)0x80;
               requirementsVector[12] = 0x01;
               requirementsVector[13] = 0;
               requirementsVector[14] = 0;
               requirementsVector[15] = 0;
               requirementsVector[16] = 0;
               requirementsVector[17] = 0;
               requirementsVector[18] = 0;
               requirementsVector[19] = 0;
               requirementsVector[20] = 0;
               requirementsVector[21] = 0;
               requirementsVector[22] = 0x03;
               requirementsVector[23] = 0;
          }

          return requirementsVector;
     }

     private void setupInputBuffer()
     {
          if (cipher != null)
          {
               encryptedPacketHeaderSize = cipher.getCipherTextLength(PacketHeaderSize);
          }

          headerInputArray = new byte[PacketHeaderSize];
          headerInputBuffer = new ByteArrayInputStream(headerInputArray);
          headerInputFilter = new DataInputStream(headerInputBuffer);
          uuidInputArray = new byte[uuidSize];
     }

     private void userPass()
     {
          String userName = properties.getProperty("userName",""); /*I18NOK:LINE*/
          String password = properties.getProperty("password",""); /*I18NOK:LINE*/
          List dosParameters = new ArrayList(3);
          int[] dosMaps = new int[2];
          dosMaps[0] = DOS_MP_UCHAR;
          dosParameters.add(userName);
          dosMaps[1] = DOS_MP_UCHAR;
          dosParameters.add(password);
          OutboundPacket out = getOutboundPacket();
          InboundPacket in = null;

          try
          {
               out.setType(Packet.INTERNAL);
               out.setMethodId(USERPASS);
               out.setDosParameters(dosMaps,dosParameters);
               sendPacket(out);
               in = getReply(out);

               if (encryptionPolicy == PE_ENCR_AUTH)
               {
                    closeCipher();
               }

               clientID = in.getClientId();
               logger.fine("Setting clientID to " + Integer.toString(clientID));
               dosParameters.clear();
               reconcileDosParameters(in,dosParameters);
          }
          catch (UnexpectedUserException e)
          {
               UNKNOWN unknown = new UNKNOWN(e.getMessage());
               unknown.initCause(e);
               throw unknown;
          }
          finally
          {
               returnOutboundPacket(out);
               returnInboundPacket(in);
          }

          byte[] encodingBytes = (byte[])dosParameters.get(1);
          EncoderInterface tempEncoder = null;
          String encodingName = encoder.toString(encodingBytes,0,encodingBytes.length-1);
          logger.fine("Received text encoding: " + encodingName);

          try
          {
               // When connecting to a V8 server with Cp1047 as its
               // native encoding, we expect the open edition flavor,
               // but the server expects the traditional flavor, and
               // that causes confusion over how linefeeds and newlines
               // should be transcoded. The solution is to punt and
               // make the server send ISO8859_1 instead.
               //------------------------------------------------------
               if ("Cp1047"/*I18NOK:EMS*/.equals(encodingName))
               {
                    throw new UnsupportedEncodingException();
               }

               tempEncoder = Encoder.getInstance(encodingName);
          }
          catch (UnsupportedEncodingException e)
          {
               logger.fine("Text encoding is not supported by this virtual machine.");

               if (transcodingPreferenceFlag == PE_USER)
               {
                    logger.fine("Attempting to switch transcoding resposibilities to the server");
                    out = getOutboundPacket();

                    try
                    {
                         out.setType(Packet.INTERNAL);
                         out.setMethodId(SERVERTRANSCODE);
                         sendPacket(out);
                         in = getReply(out);
                         dosParameters.clear();
                         reconcileDosParameters(in,dosParameters);
                    }
                    catch (UnexpectedUserException uue)
                    {
                         UNKNOWN unknown = new UNKNOWN(uue.getMessage());
                         unknown.initCause(uue);
                         throw unknown;
                    }
                    finally
                    {
                         returnOutboundPacket(out);
                         returnInboundPacket(in);
                    }

                    encodingBytes = (byte[])dosParameters.get(1);
                    encodingName = encoder.toString(encodingBytes,0,encodingBytes.length-1);
                    logger.fine("Received text encoding: " + encodingName);

                    try
                    {
                         tempEncoder = Encoder.getInstance(encodingName);
                    }
                    catch (UnsupportedEncodingException u)
                    {
                         // {0}: The client and server were unable to agree on a transport format for character data.
                         String messageIndex = "sam.S224.ex.msg";
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex);
                         throw new INITIALIZE(message);
                    }
               }
               else
               {
                    // {0}: The client and server were unable to agree on a transport format for character data.
                    String messageIndex = "sam.S225.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    throw new INITIALIZE(message);
               }
          }

          synchronized (this)
          {
               Encoder.returnInstance(encoder);
               encoder = tempEncoder;
          }
     }

     private synchronized void sendPacket(OutboundPacket out)
     {
          if (socket == null)
          {
               return;
          }

          if (out.getReplyId() < 0)
          {
               out.setReplyId(replyIDCounter++);

               if (replyIDCounter < 0)
               {
                    replyIDCounter = 0;
               }
          }

          out.setClientId(clientID);

          if (isLoggable(logger,finerRecord))
          {
               String cxNumber = properties.getProperty("cxNumber");
               logger.finer("Sending packet on cx#" + cxNumber + ":\n" + out.toString());
          }

          outputBuffer.reset();

          try
          {
               // write packet header to the output buffer
               outputBuffer.setPatchLocation(); // patch packet size later
               outputBuffer.skip(4);
               outputFilter.writeInt(out.getClientId()); // clientId
               outputFilter.writeInt(out.getObjectId()); // object we're calling method on
               outputFilter.writeInt(out.getMethodId()); // method we're calling
               outputFilter.writeInt(out.getReplyId()); // for matching requests and replies
               outputFilter.writeInt(out.getType()); // packet type
               outputBuffer.setPatchLocation(); // patch map count later
               outputBuffer.skip(4);
               // transcode the UUID
               // this only works if the string encoding maps hex digits to single-byte characters
               String uuid = out.getUuid();
               byte[] uuidBytes = encoder.toByteArray(uuid,0,uuid.length());
               outputFilter.write(uuidBytes); // uuid
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while marshalling an outbound packet header.
               String messageIndex = "sam.S226.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }

          // write maps and parameters to output buffer
          int payloadSize = 0;
          int mapCount = 0;
          int packetSize = PacketHeaderSize;
          int[] dosMaps = out.getDosMaps();

          if (dosMaps != null)
          {
               if (mars == null || mars.length < dosMaps.length || mars.length > maximumMarsSize)
               {
                    int target = Math.max(minimumMarsSize,dosMaps.length);
                    mars = new Object[target];
               }

               List dosParameters = out.getDosParameters();

               for (int i=0;i<dosMaps.length;i++)
               {
                    mapCount += writeMapsFromDOS(dosMaps[i],dosParameters.get(i),i);
               }

               payloadSize += (mapCount * MapSize);

               for (int i=0;i<dosMaps.length;i++)
               {
                    payloadSize += writeParmsFromDOS(dosMaps[i],dosParameters.get(i),i);
               }
          }
          else
          {
               int packetType = out.getType();
               int skipDirection = 0;
               int parameterCount = out.getParameterCount();
               int marsCount = parameterCount;

               if (packetType == Packet.REPLY)
               {
                    skipDirection = ARG_IN.value;
                    marsCount++;
               }
               else
               {
                    skipDirection = ARG_OUT.value;
               }


               if (marsCount > 0)
               {
                    if (mars == null || mars.length < marsCount || mars.length > maximumMarsSize)
                    {
                         int target = Math.max(minimumMarsSize,marsCount);
                         mars = new Object[target];
                    }

                    int marsOff = 0;

                    if (packetType == Packet.REPLY)
                    {
                         mapCount += writeMapsFromDOS(DOS_MP_UINT,replyRC,marsOff++);
                    }

                    // first the parameter descriptors
                    for (int i=0;i<parameterCount;i++)
                    {
                         NamedValue nv = out.getParameterAt(i);

                         if (nv.flags() != skipDirection)
                         {
                              mapCount += writeMapsFromCORBA(nv.value(),marsOff++);
                         }
                    }

                    payloadSize += (mapCount * MapSize);
                    marsOff = 0;

                    if (packetType == Packet.REPLY)
                    {
                         payloadSize += writeParmsFromDOS(DOS_MP_UINT,replyRC,marsOff++);
                    }

                    // then the parameters themselves
                    for (int i=0;i<parameterCount;i++)
                    {
                         NamedValue nv = out.getParameterAt(i);

                         if (nv.flags() != skipDirection)
                         {
                              payloadSize += writeParmsFromCORBA(nv.value(),marsOff++);
                         }
                    }
               }
          }

          // patch the buffer with packet size and map count
          packetSize += payloadSize;

          try
          {
               patchFilter.writeInt(packetSize);
               byte[] patchArray = patchBuffer.toByteArray();
               outputBuffer.patch(patchArray,0,patchArray.length);
               patchBuffer.reset();
               patchFilter.writeInt(mapCount);
               patchArray = patchBuffer.toByteArray();
               outputBuffer.patch(patchArray,0,patchArray.length);
               patchBuffer.reset();
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while patching an outbound packet header.
               String messageIndex = "sam.S227.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }

          if (isLoggable(logger,finestRecord))
          {
               byte[] a = outputBuffer.toByteArray();

               if (out.getType() == Packet.INTERNAL && out.getMethodId() == USERPASS)
               {
                    // USERPASS packet ends with two null-terminated strings. The last string
                    // in the password. Blot everything between the two null characters.
                    int bl = a.length - 2;

                    while (a[bl] != 0)
                    {
                         a[bl] = (byte)'x';
                         bl--;
                    }
               }

               logger.finest("Outgoing data:\n" + printBuffer(a));
          }

          // encrypt if necessary
          if (encryptionPolicy == PE_ENCR_ALL ||
              (encryptionPolicy == PE_ENCR_AUTH && out.getType() == Packet.INTERNAL && out.getMethodId() == USERPASS))
          {
               try
               {
                    byte[] plainText = outputBuffer.toByteArray();
                    outputBuffer.reset();
                    outputFilter.writeInt(encryptedPacketHeaderSize);
                    cipher.encrypt(plainText,0,PacketHeaderSize,outputBuffer);

                    if (payloadSize > 0)
                    {
                         int encryptedPayloadSize = cipher.getCipherTextLength(payloadSize);
                         outputFilter.writeInt(encryptedPayloadSize);
                         cipher.encrypt(plainText,PacketHeaderSize,payloadSize,outputBuffer);
                    }
               }
               catch (CryptoException ce)
               {
                    // {0}: An exception was thrown while encrypting an outbound packet.
                    String messageIndex = "sam.S228.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    DATA_CONVERSION data_conversion = new DATA_CONVERSION(message);
                    data_conversion.initCause(ce);
                    throw data_conversion;
               }
               catch (IOException ioe)
               {
                    // {0}: An exception was thrown while encrypting an outbound packet.
                    String messageIndex = "sam.S229.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    MARSHAL marshal = new MARSHAL(message);
                    marshal.initCause(ioe);
                    throw marshal;
               }
          }

          // spill the output buffer into the socket
          try
          {
               outputBuffer.writeTo(socketOutput);
               socketOutput.flush();
               outputBuffer.reset();
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while writing a packet to the socket.
               String messageIndex = "sam.S230.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               COMM_FAILURE comm_failure = new COMM_FAILURE(message);
               comm_failure.initCause(ioe);
               throw comm_failure;
          }
     }

     private int writeMapsFromCORBA(Any parm,int marsOff)
     {
          TypeCode tc = parm.type();
          IntHolder dim = new IntHolder();
          int iomType = BridgeProtocolEngine.typeCodeToIomType(tc,dim);
          int mapArrayCount = 0;

          switch (dim.value)
          {
               case 0: // scalars
               {
                    ensureMapArrayCapacity(1);

                    switch (iomType)
                    {
                         case BridgeProtocolEngine.IT_BOOLEAN:
                         case BridgeProtocolEngine.IT_OCTET:
                         {
                              maps[0][0] = WZX_TYPE_CHAR_B;
                              maps[0][1] = 1;
                              break;
                         }

                         case BridgeProtocolEngine.IT_SHORT:
                         {
                              maps[0][0] = WZX_TYPE_SHORT;
                              maps[0][1] = 1;
                              break;
                         }

                         case BridgeProtocolEngine.IT_LONG:
                         {
                              maps[0][0] = WZX_TYPE_LONG;
                              maps[0][1] = 1;
                              break;
                         }

                         case BridgeProtocolEngine.IT_FLOAT:
                         case BridgeProtocolEngine.IT_DOUBLE:
                         case BridgeProtocolEngine.IT_DATETIME:
                         {
                              maps[0][0] = WZX_TYPE_DOUBLE;
                              maps[0][1] = 1;
                              break;
                         }

                         case BridgeProtocolEngine.IT_STRING:
                         {
                              maps[0][0] = WZX_TYPE_CHAR | WZX_ATTR_UNSIGNED;
                              String s = parm.extract_string();
                              int l = s.length();
                              byte[] xcode = encoder.toByteArray(s,0,l);
                              mars[marsOff] = xcode;
                              maps[0][1] = xcode.length + 1;
                              break;
                         }

                         case BridgeProtocolEngine.IT_ENUM:
                         case BridgeProtocolEngine.IT_OBJECT:
                         {
                              maps[0][0] = WZX_TYPE_INT | WZX_ATTR_UNSIGNED;
                              maps[0][1] = 1;
                              break;
                         }

                         case BridgeProtocolEngine.IT_UUID:
                         {
                              maps[0][0] = WZX_TYPE_CHAR | WZX_ATTR_UNSIGNED;
                              org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                              String s = BridgeProtocolEngine.uuidToString(is);
                              int l = s.length();
                              byte[] xcode = encoder.toByteArray(s,0,l);
                              mars[marsOff] = xcode;
                              maps[0][1] = xcode.length + 1;
                              break;
                         }

                    }

                    mapArrayCount = 1;
                    break;
               }

               case 1: // 1D arrays
               {
                    ensureMapArrayCapacity(3);
                    maps[0][0] = WZX_TYPE_SHORT;
                    maps[0][1] = 1;
                    org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                    int mag0 = is.read_long();
                    maps[1][0] = WZX_TYPE_LONG | WZX_ATTR_UNSIGNED;
                    maps[1][1] = 1;
                    mapArrayCount = 2;

                    if (mag0 > 0)
                    {
                         switch (iomType)
                         {
                              case BridgeProtocolEngine.IT_BOOLEAN:
                              case BridgeProtocolEngine.IT_OCTET:
                              {
                                   maps[2][0] = WZX_TYPE_CHAR_B;
                                   maps[2][1] = mag0;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_SHORT:
                              {
                                   maps[2][0] = WZX_TYPE_SHORT;
                                   maps[2][1] = mag0;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_LONG:
                              {
                                   maps[2][0] = WZX_TYPE_LONG;
                                   maps[2][1] = mag0;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_FLOAT:
                              case BridgeProtocolEngine.IT_DOUBLE:
                              case BridgeProtocolEngine.IT_DATETIME:
                              {
                                   maps[2][0] = WZX_TYPE_DOUBLE;
                                   maps[2][1] = mag0;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_STRING:
                              {
                                   mapArrayCount += mag0;
                                   ensureMapArrayCapacity(mapArrayCount);
                                   byte[][] xcode = new byte[mag0][];
                                   mars[marsOff] = xcode;

                                   for (int i=0;i<mag0;i++)
                                   {
                                        String s = is.read_string();
                                        int l = s.length();
                                        xcode[i] = encoder.toByteArray(s,0,l);
                                        maps[i+2][0] = WZX_TYPE_CHAR | WZX_ATTR_UNSIGNED;
                                        maps[i+2][1] = xcode[i].length + 1;
                                   }

                                   break;
                              }

                              case BridgeProtocolEngine.IT_ENUM:
                              {
                                   maps[2][0] = WZX_TYPE_INT | WZX_ATTR_UNSIGNED;
                                   maps[2][1] = mag0;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_UUID:
                              {
                                   mapArrayCount += mag0;
                                   ensureMapArrayCapacity(mapArrayCount);
                                   byte[][] xcode = new byte[mag0][];
                                   mars[marsOff] = xcode;

                                   for (int i=0;i<mag0;i++)
                                   {
                                        String s = BridgeProtocolEngine.uuidToString(is);
                                        int l = s.length();
                                        xcode[i] = encoder.toByteArray(s,0,l);
                                        maps[i+2][0] = WZX_TYPE_CHAR | WZX_ATTR_UNSIGNED;
                                        maps[i+2][1] = xcode[i].length + 1;
                                   }

                                   break;
                              }
                         }
                    }

                    break;
               }

               case 2: // 2D arrays
               {
                    ensureMapArrayCapacity(3);
                    maps[0][0] = WZX_TYPE_SHORT;
                    maps[0][1] = 1;
                    org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                    int mag0 = is.read_long();
                    int mag1 = (mag0 > 0) ? is.read_long() : 0;
                    maps[1][0] = WZX_TYPE_LONG | WZX_ATTR_UNSIGNED;
                    maps[1][1] = 2;
                    mapArrayCount = 2;

                    if (mag0 * mag1 > 0)
                    {
                         switch (iomType)
                         {
                              case BridgeProtocolEngine.IT_BOOLEAN:
                              case BridgeProtocolEngine.IT_OCTET:
                              {
                                   maps[2][0] = WZX_TYPE_CHAR_B;
                                   maps[2][1] = mag0 * mag1;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_SHORT:
                              {
                                   maps[2][0] = WZX_TYPE_SHORT;
                                   maps[2][1] = mag0 * mag1;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_LONG:
                              {
                                   maps[2][0] = WZX_TYPE_LONG;
                                   maps[2][1] = mag0 * mag1;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_FLOAT:
                              case BridgeProtocolEngine.IT_DOUBLE:
                              case BridgeProtocolEngine.IT_DATETIME:
                              {
                                   maps[2][0] = WZX_TYPE_DOUBLE;
                                   maps[2][1] = mag0 * mag1;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_STRING:
                              {
                                   mapArrayCount += (mag0 * mag1);
                                   ensureMapArrayCapacity(mapArrayCount);
                                   byte[][][] xcode = new byte[mag0][mag1][];
                                   mars[marsOff] = xcode;
                                   int mapIndex = 2;

                                   for (int i0=0;i0<mag0;i0++)
                                   {
                                        if (i0 != 0)
                                        {
                                             is.read_long();
                                        }

                                        for (int i1=0;i1<mag1;i1++,mapIndex++)
                                        {
                                             String s = is.read_string();
                                             int l = s.length();
                                             xcode[i0][i1] = encoder.toByteArray(s,0,l);
                                             maps[mapIndex][0] = WZX_TYPE_CHAR | WZX_ATTR_UNSIGNED;
                                             maps[mapIndex][1] = xcode[i0][i1].length + 1;
                                        }
                                   }

                                   break;
                              }

                              case BridgeProtocolEngine.IT_ENUM:
                              {
                                   maps[2][0] = WZX_TYPE_INT | WZX_ATTR_UNSIGNED;
                                   maps[2][1] = mag0 * mag1;
                                   mapArrayCount++;
                                   break;
                              }

                              case BridgeProtocolEngine.IT_UUID:
                              {
                                   mapArrayCount += (mag0 * mag1);
                                   ensureMapArrayCapacity(mapArrayCount);
                                   byte[][][] xcode = new byte[mag0][mag1][];
                                   mars[marsOff] = xcode;
                                   int mapIndex = 2;

                                   for (int i0=0;i0<mag0;i0++)
                                   {
                                        if (i0 != 0)
                                        {
                                             is.read_long();
                                        }

                                        for (int i1=0;i1<mag1;i1++,mapIndex++)
                                        {
                                             String s = BridgeProtocolEngine.uuidToString(is);
                                             int l = s.length();
                                             xcode[i0][i1] = encoder.toByteArray(s,0,l);
                                             maps[mapIndex][0] = WZX_TYPE_CHAR | WZX_ATTR_UNSIGNED;
                                             maps[mapIndex][1] = xcode[i0][i1].length + 1;
                                        }
                                   }

                                   break;
                              }
                         }
                    }

                    break;
               }
          }

          return writeMaps(maps,mapArrayCount,outputFilter);
     }

     private int writeMapsFromDOS(int dosMap,Object dosParameter,int marsOff)
     {
          ensureMapArrayCapacity(1);

          switch (dosMap)
          {
               case DOS_MP_UCHAR:
               {
                    String s = (String)dosParameter;
                    byte[] xcode =  encoder.toByteArray(s,0,s.length());
                    mars[marsOff] = xcode;
                    maps[0][0] = WZX_TYPE_CHAR | WZX_ATTR_UNSIGNED;
                    maps[0][1] = xcode.length + 1;
                    break;
               }

               case DOS_MP_SHORT:
               {
                    short[] s = (short[])dosParameter;
                    maps[0][0] = WZX_TYPE_SHORT;
                    maps[0][1] = s.length;
                    break;
               }

               case DOS_MP_UINT:
               {
                    int[] s = (int[])dosParameter;
                    maps[0][0] = WZX_TYPE_INT | WZX_ATTR_UNSIGNED;
                    maps[0][1] = s.length;
                    break;
               }

               case DOS_MP_LONG:
               {
                    int[] s = (int[])dosParameter;
                    maps[0][0] = WZX_TYPE_LONG;
                    maps[0][1] = s.length;
                    break;
               }

               case DOS_MP_ULONG:
               {
                    int[] s = (int[])dosParameter;
                    maps[0][0] = WZX_TYPE_LONG | WZX_ATTR_UNSIGNED;
                    maps[0][1] = s.length;
                    break;
               }

               case DOS_MP_DOUBLE:
               {
                    double[] s = (double[])dosParameter;
                    maps[0][0] = WZX_TYPE_DOUBLE;
                    maps[0][1] = s.length;
                    break;
               }

               case DOS_MP_BINARY:
               {
                    byte[] s = (byte[])dosParameter;
                    maps[0][0] = WZX_TYPE_CHAR_B;
                    maps[0][1] = s.length;
                    break;
               }
          }

          return writeMaps(maps,1,outputFilter);
     }

     private int writeMaps(int[][] mapArray,int mapArrayCount,DataOutputStream outputFilter)
     {
          int mapCount = 0;

          try
          {
               for (int i=0;i<mapArrayCount;i++)
               {
                    int wzxType = mapArray[i][0];
                    int repFactor = mapArray[i][1];
                    int map = 0;

                    if (repFactor <= WZX_REP_MASK)
                    {
                         map = wzxType | repFactor;
                         outputFilter.writeInt(map);
                         mapCount++;
                    }
                    else
                    {
                         map = wzxType | WZX_MAP32XTND;
                         outputFilter.writeInt(map);
                         mapCount++;

                         do
                         {
                              map = wzxType | WZX_REP_MASK | WZX_MAP32XTND;
                              outputFilter.writeInt(map);
                              mapCount++;
                              repFactor -= WZX_REP_MASK;
                         }
                         while (repFactor > WZX_REP_MASK);

                         map = wzxType | repFactor;
                         outputFilter.writeInt(map);
                         mapCount++;
                    }
               }
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while marshalling an outbound packet''s payload descriptor.
               String messageIndex = "sam.S231.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }

          return mapCount;
     }

     private int writeParmsFromCORBA(Any parm,int marsOff)
     {
          int dataSize = 0;

          try
          {
               TypeCode tc = parm.type();
               IntHolder dim = new IntHolder();
               int iomType = BridgeProtocolEngine.typeCodeToIomType(tc,dim);

               switch (iomType + dim.value)
               {
                    case BridgeProtocolEngine.IT_BOOLEAN:
                    {
                         boolean b = parm.extract_boolean();
                         outputBuffer.write((b)?1:0);
                         dataSize = 1;
                         break;
                    }

                    case BridgeProtocolEngine.IT_BOOLEAN_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              outputBuffer.write((is.read_boolean())?1:0);
                         }

                         dataSize += mag0;
                         break;
                    }

                    case BridgeProtocolEngine.IT_BOOLEAN_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         int mag1 = (mag0 > 0) ? is.read_long() : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              if (i0 != 0)
                              {
                                   is.read_long();
                              }

                              for (int i1=0;i1<mag1;i1++)
                              {
                                   outputBuffer.write((is.read_boolean())?1:0);
                              }
                         }

                         dataSize += (mag0 * mag1);
                         break;
                    }

                    case BridgeProtocolEngine.IT_OCTET:
                    {
                         int o = parm.extract_octet();
                         outputBuffer.write(o);
                         dataSize = 1;
                         break;
                    }

                    case BridgeProtocolEngine.IT_OCTET_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              outputBuffer.write(is.read_octet());
                         }

                         dataSize += mag0;
                         break;
                    }

                    case BridgeProtocolEngine.IT_OCTET_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         int mag1 = (mag0 > 0) ? is.read_long() : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              if (i0 != 0)
                              {
                                   is.read_long();
                              }

                              for (int i1=0;i1<mag1;i1++)
                              {
                                   outputBuffer.write(is.read_octet());
                              }
                         }

                         dataSize += (mag0 * mag1);
                         break;
                    }

                    case BridgeProtocolEngine.IT_SHORT:
                    {
                         short s = parm.extract_short();
                         outputFilter.writeShort(s);
                         dataSize = 2;
                         break;
                    }

                    case BridgeProtocolEngine.IT_SHORT_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              outputFilter.writeShort(is.read_short());
                         }

                         dataSize += (mag0 * 2);
                         break;
                    }

                    case BridgeProtocolEngine.IT_SHORT_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         int mag1 = (mag0 > 0) ? is.read_long() : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              if (i0 != 0)
                              {
                                   is.read_long();
                              }

                              for (int i1=0;i1<mag1;i1++)
                              {
                                   outputFilter.writeShort(is.read_short());
                              }
                         }

                         dataSize += (mag0 * mag1 * 2);
                         break;
                    }

                    case BridgeProtocolEngine.IT_LONG:
                    {
                         int n = parm.extract_long();
                         outputFilter.writeInt(n);
                         dataSize = 4;
                         break;
                    }

                    case BridgeProtocolEngine.IT_LONG_1D_ARRAY:
                    case BridgeProtocolEngine.IT_ENUM_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              outputFilter.writeInt(is.read_long());
                         }

                         dataSize += (mag0 * 4);
                         break;
                    }

                    case BridgeProtocolEngine.IT_LONG_2D_ARRAY:
                    case BridgeProtocolEngine.IT_ENUM_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         int mag1 = (mag0 > 0) ? is.read_long() : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              if (i0 != 0)
                              {
                                   is.read_long();
                              }

                              for (int i1=0;i1<mag1;i1++)
                              {
                                   outputFilter.writeInt(is.read_long());
                              }
                         }

                         dataSize += (mag0 * mag1 * 4);
                         break;
                    }

                    case BridgeProtocolEngine.IT_FLOAT:
                    {
                         double d = parm.extract_float();
                         outputFilter.writeDouble(d);
                         dataSize = 8;
                         break;
                    }

                    case BridgeProtocolEngine.IT_FLOAT_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              outputFilter.writeDouble(is.read_float());
                         }

                         dataSize += (mag0 * 8);
                         break;
                    }

                    case BridgeProtocolEngine.IT_FLOAT_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         int mag1 = (mag0 > 0) ? is.read_long() : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              if (i0 != 0)
                              {
                                   is.read_long();
                              }

                              for (int i1=0;i1<mag1;i1++)
                              {
                                   outputFilter.writeDouble(is.read_float());
                              }
                         }

                         dataSize += (mag0 * mag1 * 8);
                         break;
                    }

                    case BridgeProtocolEngine.IT_DOUBLE:
                    {
                         double d = parm.extract_double();
                         outputFilter.writeDouble(d);
                         dataSize = 8;
                         break;
                    }

                    case BridgeProtocolEngine.IT_DOUBLE_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              outputFilter.writeDouble(is.read_double());
                         }

                         dataSize += (mag0 * 8);
                         break;
                    }

                    case BridgeProtocolEngine.IT_DOUBLE_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         int mag1 = (mag0 > 0) ? is.read_long() : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              if (i0 != 0)
                              {
                                   is.read_long();
                              }

                              for (int i1=0;i1<mag1;i1++)
                              {
                                   outputFilter.writeDouble(is.read_double());
                              }
                         }

                         dataSize += (mag0 * mag1 * 8);
                         break;
                    }

                    case BridgeProtocolEngine.IT_STRING:
                    case BridgeProtocolEngine.IT_UUID:
                    {
                         byte[] data = (byte[])mars[marsOff];
                         outputBuffer.write(data);
                         outputBuffer.write(0);
                         dataSize = data.length + 1;
                         mars[marsOff] = null;
                         break;
                    }

                    case BridgeProtocolEngine.IT_STRING_1D_ARRAY:
                    case BridgeProtocolEngine.IT_UUID_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         byte[][] xcode = (byte[][])mars[marsOff];
                         int mag0 = (xcode != null) ? xcode.length : 0;
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              outputBuffer.write(xcode[i]);
                              outputBuffer.write(0);
                              dataSize += (xcode[i].length + 1);
                         }

                         mars[marsOff] = null;
                         break;
                    }

                    case BridgeProtocolEngine.IT_STRING_2D_ARRAY:
                    case BridgeProtocolEngine.IT_UUID_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         byte[][][] xcode = (byte[][][])mars[marsOff];
                         int mag0 = (xcode != null) ? xcode.length : 0;
                         int mag1 = (mag0 > 0) ? xcode[0].length : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              for (int i1=0;i1<mag1;i1++)
                              {
                                   outputBuffer.write(xcode[i0][i1]);
                                   outputBuffer.write(0);
                                   dataSize += (xcode[i0][i1].length + 1);
                              }
                         }

                         mars[marsOff] = null;
                         break;
                    }

                    case BridgeProtocolEngine.IT_ENUM:
                    {
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int e = is.read_long();
                         outputFilter.writeInt(e);
                         dataSize = 4;
                         break;
                    }

                    case BridgeProtocolEngine.IT_OBJECT:
                    {
                         org.omg.CORBA.Object obj = parm.extract_Object();
                         int id = engine.objectToObjectID(obj);
                         outputFilter.writeInt(id);
                         dataSize = 4;
                         break;
                    }

                    case BridgeProtocolEngine.IT_DATETIME:
                    {
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         long corba = is.read_ulonglong();
                         double sas = BridgeProtocolEngine.corbaToSas(corba);
                         outputFilter.writeDouble(sas);
                         dataSize = 8;
                         break;
                    }

                    case BridgeProtocolEngine.IT_DATETIME_1D_ARRAY:
                    {
                         outputFilter.writeShort(1);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         outputFilter.writeInt(mag0);
                         dataSize += 4;

                         for (int i=0;i<mag0;i++)
                         {
                              double sas = BridgeProtocolEngine.corbaToSas(is.read_ulonglong());
                              outputFilter.writeDouble(sas);
                         }

                         dataSize += (mag0 * 8);
                         break;
                    }

                    case BridgeProtocolEngine.IT_DATETIME_2D_ARRAY:
                    {
                         outputFilter.writeShort(2);
                         dataSize += 2;
                         org.omg.CORBA.portable.InputStream is = parm.create_input_stream();
                         int mag0 = is.read_long();
                         int mag1 = (mag0 > 0) ? is.read_long() : 0;
                         outputFilter.writeInt(mag0);
                         outputFilter.writeInt(mag1);
                         dataSize += 8;

                         for (int i0=0;i0<mag0;i0++)
                         {
                              if (i0 != 0)
                              {
                                   is.read_long();
                              }

                              for (int i1=0;i1<mag1;i1++)
                              {
                                   double sas = BridgeProtocolEngine.corbaToSas(is.read_ulonglong());
                                   outputFilter.writeDouble(sas);
                              }
                         }

                         dataSize += (mag0 * mag1 * 8);
                         break;
                    }
               }
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while marshalling an outbound packet''s payload.
               String messageIndex = "sam.S232.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }

          return dataSize;
     }

     private int writeParmsFromDOS(int dosMap,Object dosParameter,int marsOff)
     {
          try
          {
               switch (dosMap)
               {
                    case DOS_MP_UCHAR:
                    {
                         byte[] xcode = (byte[])mars[marsOff];
                         outputBuffer.write(xcode);
                         outputBuffer.write(0);
                         return xcode.length + 1;
                    }

                    case DOS_MP_SHORT:
                    {
                         short[] s = (short[])dosParameter;
                         int l = s.length;

                         for (int i=0;i<l;i++)
                         {
                              outputFilter.writeShort(s[i]);
                         }

                         return (l * 2);
                    }

                    case DOS_MP_UINT:
                    case DOS_MP_LONG:
                    case DOS_MP_ULONG:
                    {
                         int[] s = (int[])dosParameter;
                         int l = s.length;

                         for (int i=0;i<l;i++)
                         {
                              outputFilter.writeInt(s[i]);
                         }

                         return (l * 4);
                    }

                    case DOS_MP_DOUBLE:
                    {
                         double[] s = (double[])dosParameter;
                         int l = s.length;

                         for (int i=0;i<l;i++)
                         {
                              outputFilter.writeDouble(s[i]);
                         }

                         return (l * 8);
                    }

                    case DOS_MP_BINARY:
                    {
                         byte[] s = (byte[])dosParameter;
                         outputBuffer.write(s);
                         return s.length;
                    }
               }
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while marshalling an outbound packet''s payload.
               String messageIndex = "sam.S233.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }

          return 0;
     }

     private void ensureMapArrayCapacity(int capacity)
     {
          if (maps == null || maps.length < capacity || maps.length > maximumMapArraySize)
          {
               int[][] temp = maps;
               int target = Math.max(capacity,minimumMapArraySize);
               maps = new int[target][2];

               // copy the first to elements in case this is a resize for string/uuid arrays
               // minimumMapArraySize must be at least 2
               if (temp != null)
               {
                    maps[0][0] = temp[0][0];
                    maps[0][1] = temp[0][1];
                    maps[1][0] = temp[1][0];
                    maps[1][1] = temp[1][1];
               }
          }
     }

     // doesn't need to be synchronized - only called from dedicated thread
     private InboundPacket readPacket()
     {
          try
          {
               if (encryptionPolicy == PE_ENCR_ALL)
               {
                    socketInputFilter.readInt();
                    cipher.decrypt(socketInputFilter,encryptedPacketHeaderSize,headerInputArray,0);
               }
               else
               {
                    //socketInputFilter.readFully(headerInputArray,0,PacketHeaderSize);
                    readFully(socketInput,headerInputArray,0,PacketHeaderSize);
               }
          }
          catch (CryptoException ce)
          {
               // {0}: An inbound packet header could not be decrypted: {1}
               String messageIndex = "sam.S234.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex,ce.getMessage());
               DATA_CONVERSION data_conversion = new DATA_CONVERSION(message);
               data_conversion.initCause(ce);
               throw data_conversion;
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while reading a packet header from the socket.
               String messageIndex = "sam.S235.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               COMM_FAILURE comm_failure = new COMM_FAILURE(message);
               comm_failure.initCause(ioe);
               throw comm_failure;
          }

          if (isLoggable(logger,finestRecord))
          {
               logger.finest("Incoming data [header]:\n" + printBuffer(headerInputArray));
          }

          headerInputBuffer.reset();
          InboundPacket in = getInboundPacket();
          int packetSize = 0;
          int mapCount = 0;

          // now read the individual header fields from the buffer
          try
          {
               packetSize = headerInputFilter.readInt();
               in.setClientId(headerInputFilter.readInt());
               in.setObjectId(headerInputFilter.readInt());
               in.setMethodId(headerInputFilter.readInt());
               in.setReplyId(headerInputFilter.readInt());
               in.setType(headerInputFilter.readInt());
               mapCount = headerInputFilter.readInt();
               headerInputFilter.readFully(uuidInputArray);
               // transcode the UUID
               // this only works if the string encoding maps hex digits to single-byte characters
               in.setUuid(encoder.toString(uuidInputArray,0,uuidSize));
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while unmarshalling an inbound packet header.
               String messageIndex = "sam.S236.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }

          int payloadSize = packetSize - PacketHeaderSize;

          if (payloadSize <= 0)
          {
               return in;
          }

          in.ensurePayloadCapacity(payloadSize);
          byte[] payloadInputArray = in.getPayload();

          // read from the socket into the buffer
          // decrypt if necessary
          try
          {
               if (encryptionPolicy == PE_ENCR_ALL)
               {
                    int encryptedPayloadSize = socketInputFilter.readInt();
                    cipher.decrypt(socketInputFilter,encryptedPayloadSize,payloadInputArray,0);
               }
               else
               {
                    //socketInputFilter.readFully(payloadInputArray,0,payloadSize);
                    readFully(socketInput,payloadInputArray,0,payloadSize);
               }
          }
          catch (CryptoException ce)
          {
               // {0}: The payload of an inbound packet could not be decrypted: {1}
               String messageIndex = "sam.S237.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex,ce.getMessage());
               DATA_CONVERSION data_conversion = new DATA_CONVERSION(message);
               data_conversion.initCause(ce);
               throw data_conversion;
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while reading a packet from the socket.
               String messageIndex = "sam.S238.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               COMM_FAILURE comm_failure = new COMM_FAILURE(message);
               comm_failure.initCause(ioe);
               throw comm_failure;
          }

          if (isLoggable(logger,finestRecord))
          {
               logger.finest("Incoming data [payload]:\n" + printBuffer(payloadInputArray,payloadSize));
          }

          in.setPayloadSize(payloadSize);
          in.setMapCount(mapCount);
          return in;
     }

     private InboundPacket getReply(OutboundPacket out)
     {
          InboundPacket in = null;

          synchronized (replyList)
          {
               // don't lock this while replyList is locked
               do
               {
                    final int size = replyList.size();

                    if (size > 0)
                    {
                         int target = out.getReplyId();

                         for (int i=0;i<size;i++)
                         {
                              InboundPacket item = (InboundPacket)replyList.get(i);

                              if (item.getReplyId() == target)
                              {
                                   replyList.remove(i);
                                   in = item;
                                   break;
                              }
                         }
                    }

                    if (in == null)
                    {
                         if (fatalException != null)
                         {
                              throw (SystemException)fatalException.fillInStackTrace();
                         }

                         try
                         {
                              replyList.wait();
                         }
                         catch (InterruptedException e)
                         {
                              // {0}: The thread waiting for a reply from the remote host was interrupted.
                              String messageIndex = "sam.S239.ex.msg"/*I18NOK:EMS*/;
                              String messagePattern = RB.getStringResource(messageIndex);
                              String message = Message.format(messagePattern,messageIndex);
                              UNKNOWN unknown = new UNKNOWN(message);
                              unknown.initCause(e);
                              throw unknown;
                         }
                    }
               }
               while (in == null);
          }

          if (in.getType() == Packet.INTERNAL && in.getMethodId() == ERROR)
          {
               try
               {
                    List dosParameters = new ArrayList(5);
                    reconcileDosParameters(in,dosParameters);
                    int errorStatus = ((int[])dosParameters.get(0))[0];
                    String errorXML = (String)dosParameters.get(1);
                    String errorUUID = (String)dosParameters.get(3);
                    SystemException error = parseErrorXML(errorStatus,errorXML,errorUUID);
                    throw error;
               }
               finally
               {
                    returnInboundPacket(in);
               }
          }

          return in;
     }

     protected int[] reconcileDosParameters(InboundPacket in,List dosParameters)
     {
          byte[] payloadArray = in.getPayload();
          int payloadSize = in.getPayloadSize();
          int wzxMapCount = in.getMapCount();
          int[] dosMaps = new int[wzxMapCount];
          int parmOff = wzxMapCount * MapSize;
          ByteArrayInputStream mapBuffer = new ByteArrayInputStream(payloadArray,0,parmOff);
          DataInputStream mapFilter = new DataInputStream(mapBuffer);
          ByteArrayInputStream parmBuffer = new ByteArrayInputStream(payloadArray,parmOff,payloadSize-parmOff);
          DataInputStream parmFilter = new DataInputStream(parmBuffer);
          int wzxMapIndex = 0;
          int dosMapIndex = 0;
          IntHolder typeHolder = new IntHolder();
          IntHolder repFactorHolder = new IntHolder();
          BooleanHolder unsignedHolder = new BooleanHolder();

          while (wzxMapIndex < wzxMapCount)
          {
               wzxMapIndex += readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
               int wzxType = typeHolder.value;
               int repFactor = repFactorHolder.value;
               boolean unsigned = unsignedHolder.value;

               try
               {
                    switch (wzxType)
                    {
                         case WZX_TYPE_CHAR:
                         {
                              dosMaps[dosMapIndex] = DOS_MP_UCHAR;
                              String s = encoder.toString(payloadArray,parmOff,repFactor-1);
                              dosParameters.add(s);
                              parmOff += repFactor;
                              parmBuffer.skip(repFactor);
                              break;
                         }

                         case WZX_TYPE_SHORT:
                         {
                              dosMaps[dosMapIndex] = DOS_MP_SHORT;
                              short[] a = new short[repFactor];

                              for (int j=0;j<repFactor;j++)
                              {
                                   a[j] = parmFilter.readShort();
                              }

                              dosParameters.add(a);
                              parmOff += (repFactor * 2);
                              break;
                         }

                         case WZX_TYPE_INT:
                         {
                              dosMaps[dosMapIndex] = DOS_MP_UINT;
                              int[] a = new int[repFactor];

                              for (int j=0;j<repFactor;j++)
                              {
                                   a[j] = parmFilter.readInt();
                              }

                              dosParameters.add(a);
                              parmOff += (repFactor * 4);
                              break;
                         }

                         case WZX_TYPE_LONG:
                         {
                              if (unsigned)
                              {
                                   dosMaps[dosMapIndex] = DOS_MP_ULONG;
                              }
                              else
                              {
                                   dosMaps[dosMapIndex] = DOS_MP_LONG;
                              }

                              int[] a = new int[repFactor];

                              for (int j=0;j<repFactor;j++)
                              {
                                   a[j] = parmFilter.readInt();
                              }

                              dosParameters.add(a);
                              parmOff += (repFactor * 4);
                              break;
                         }

                         case WZX_TYPE_DOUBLE:
                         {
                              dosMaps[dosMapIndex] = DOS_MP_DOUBLE;
                              double[] a = new double[repFactor];

                              for (int j=0;j<repFactor;j++)
                              {
                                   a[j] = parmFilter.readDouble();
                              }

                              dosParameters.add(a);
                              parmOff += (repFactor * 8);
                              break;
                         }

                         case WZX_TYPE_CHAR_B:
                         {
                              dosMaps[dosMapIndex] = DOS_MP_BINARY;
                              byte[] a = new byte[repFactor];
                              System.arraycopy(payloadArray,parmOff,a,0,repFactor);
                              dosParameters.add(a);
                              parmOff += repFactor;
                              parmBuffer.skip(repFactor);
                              break;
                         }
                    }
               }
               catch (IOException ioe)
               {
                    // {0}: An exception was thrown while unmarshalling an inbound packet''s payload.
                    String messageIndex = "sam.S240.ex.msg"/*I18NOK:EMS*/;
                    String messagePattern = RB.getStringResource(messageIndex);
                    String message = Message.format(messagePattern,messageIndex);
                    MARSHAL marshal = new MARSHAL(message);
                    marshal.initCause(ioe);
                    throw marshal;
               }

               dosMapIndex++;
          }

          return dosMaps;
     }

     private String reconcileName(InboundPacket in)
     {
          byte[] payloadArray = in.getPayload();
          int payloadSize = in.getPayloadSize();
          int wzxMapCount = in.getMapCount();
          int parmOff = wzxMapCount * MapSize;
          ByteArrayInputStream mapBuffer = new ByteArrayInputStream(payloadArray,0,parmOff);
          DataInputStream mapFilter = new DataInputStream(mapBuffer);
          IntHolder typeHolder = new IntHolder();
          IntHolder repFactorHolder = new IntHolder();
          BooleanHolder unsignedHolder = new BooleanHolder();
          readMap(mapFilter,typeHolder,repFactorHolder,unsignedHolder);
          int repFactor = repFactorHolder.value;
          return encoder.toString(payloadArray,parmOff,repFactor-1);
     }

     protected int readMap(DataInputStream mapFilter,IntHolder typeHolder,IntHolder repFactorHolder,BooleanHolder unsignedHolder)
     {
          int wzxMapCount = 0;

          try
          {
               int wzxMap = mapFilter.readInt();
               wzxMapCount++;
               typeHolder.value = wzxMap & WZX_TYPE_MASK;
               unsignedHolder.value = ((wzxMap & WZX_ATTR_UNSIGNED) != 0);
               boolean xtnd = ((wzxMap & WZX_MAP32XTND) != 0);
               int repFactor = 0;

               if (!xtnd)
               {
                    repFactor = wzxMap & WZX_REP_MASK;
               }
               else
               {
                    do
                    {
                         wzxMap = mapFilter.readInt();
                         wzxMapCount++;
                         repFactor += wzxMap & WZX_REP_MASK;
                         xtnd = ((wzxMap & WZX_MAP32XTND) != 0);
                    }
                    while (xtnd);
               }

               repFactorHolder.value = repFactor;
          }
          catch (IOException ioe)
          {
               // {0}: An exception was thrown while unmarshalling an inbound packet''s payload descriptor.
               String messageIndex = "sam.S241.ex.msg"/*I18NOK:EMS*/;
               String messagePattern = RB.getStringResource(messageIndex);
               String message = Message.format(messagePattern,messageIndex);
               MARSHAL marshal = new MARSHAL(message);
               marshal.initCause(ioe);
               throw marshal;
          }

          return wzxMapCount;
     }

     protected void handleReply(InboundPacket in)
     {
          synchronized (replyList)
          {
               // don't lock this while replyList is locked
               replyList.add(in);
               replyList.notifyAll();
          }
     }

     protected void handleInternal(InboundPacket in)
     {
          int methodID = in.getMethodId();

          switch (methodID)
          {
               case ERROR:
               case USERPASS:
               case SERVERTRANSCODE:
               {
                    handleReply(in);
                    break;
               }

               case NEWOBJECT:
               {
                    int mapCount = in.getMapCount();

                    if (mapCount > 0)
                    {
                         handleReply(in);
                    }
                    else
                    {
                         OutboundPacket out = getOutboundPacket();

                         try
                         {
                              String classID = in.getUuid();
                              int replyID = in.getReplyId();
                              int objectID = 0;

                              try
                              {
                                   objectID = engine.newLocalObject(classID);
                              }
                              catch (SystemException se)
                              {
                                   sendSystemError(0,null,replyID,se);
                              }

                              out.setType(Packet.INTERNAL);
                              out.setObjectId(objectID);
                              out.setMethodId(NEWOBJECT);
                              out.setReplyId(replyID);
                              out.setUuid(classID);
                              int[] dosMaps = {DOS_MP_UINT};
                              List dosParameters = new ArrayList(1);
                              dosParameters.add(new int[]{0});
                              out.setDosParameters(dosMaps,dosParameters);
                              sendPacket(out);
                         }
                         finally
                         {
                              returnOutboundPacket(out);
                              returnInboundPacket(in);
                         }
                    }

                    break;
               }

               case CLOSE:
               {
                    synchronized (this)
                    {
                         if (socket != null)
                         {
                              try
                              {
                                   socket.close();
                              }
                              catch (IOException e)
                              {
                              }

                              socket = null;
                         }
                    }

                    returnInboundPacket(in);
                    break;
               }

               case OBJECTDESTRUCTOR:
               {
                    try
                    {
                         int objectID = in.getObjectId();
                         engine.remoteObjectDestructor(objectID);
                    }
                    finally
                    {
                         returnInboundPacket(in);
                    }

                    break;
               }

               case INTERFACEINOBJECT:
               {
                    int mapCount = in.getMapCount();

                    if (mapCount > 0)
                    {
                         handleReply(in);
                    }
                    else
                    {
                         OutboundPacket out = getOutboundPacket();

                         try
                         {
                              int objectID = in.getObjectId();
                              String interfaceID = in.getUuid();
                              int replyID = in.getReplyId();
                              boolean found = false;

                              try
                              {
                                   found = engine.interfaceInLocalObject(objectID,interfaceID);
                              }
                              catch (SystemException se)
                              {
                                   sendSystemError(0,null,replyID,se);
                              }

                              out.setType(Packet.INTERNAL);
                              out.setObjectId(objectID);
                              out.setMethodId(INTERFACEINOBJECT);
                              out.setReplyId(replyID);
                              int[] dosMaps = {DOS_MP_UINT};
                              List dosParameters = new ArrayList(1);
                              dosParameters.add(new int[]{(found)?0:1});
                              out.setDosParameters(dosMaps,dosParameters);
                              sendPacket(out);
                         }
                         finally
                         {
                              returnOutboundPacket(out);
                              returnInboundPacket(in);
                         }
                    }

                    break;
               }

               case ADVISE:
               case UNADVISE:
               case GENPASSWORD:
               case EVENTSFROMOBJECTID:
               {
                    int mapCount = in.getMapCount();

                    if (mapCount > 0 || (mapCount > 1 && methodID == GENPASSWORD))
                    {
                         handleReply(in);
                         break;
                    }
                    else
                    {
                         // fall through to default case
                    }
               }

               default:
               case UUIDFROMOBJECTID:
               case FREEOBJECT:
               case DEFAULTINTERFACE:
               {
                    String classID = in.getUuid();
                    int replyID = in.getReplyId();
                    int objectID = in.getObjectId();
                    Throwable ex = new NO_IMPLEMENT();
                    sendSystemError(objectID,classID,replyID,ex);
                    break;
               }
          }
     }

     protected void handleRequest(InboundPacket in)
     {
          int objectID = in.getObjectId();
          String interfaceID = in.getUuid();
          String name = null;

          if (in.getType() == Packet.REQUEST)
          {
               switch (in.getMethodId())
               {
                    case 0:name="CastToInterface"/*I18NOK:EMS*/;break;
                    case 1:name="AddRef"/*I18NOK:EMS*/;break;
                    case 2:name="Release"/*I18NOK:EMS*/;break;
               }
          }
          else
          {
               name = reconcileName(in);
          }

          int replyID = in.getReplyId();
          engine.dispatchRequest(objectID,interfaceID,name,replyID,in);
     }

     protected void handleEvent(InboundPacket in)
     {
          int objectID = in.getObjectId();
          String eventInterfaceID = in.getUuid();
          String eventName = reconcileName(in);
          engine.dispatchEvent(objectID,eventInterfaceID,eventName,in);
     }

     private static boolean requirementsEqual(byte[] req1,int off1,byte[] req2,int off2,int len)
     {
          for (int i=0;i<len;i++)
          {
               if (i != 12 && req1[i+off1] != req2[i+off2])
               {
                    return false;
               }
          }

          return true;
     }

     private void closeCipher()
     {
          if (cipher != null)
          {
               try
               {
                    cipher.close();
               }
               catch (CryptoException ce)
               {
               }
               finally
               {
                    cipher = null;
               }
          }
     }

     protected BridgeProtocolEngine getEngine()
     {
          return engine;
     }

     private OutboundPacket getOutboundPacket()
     {
          return new OutboundPacket(this);
     }

     private void returnOutboundPacket(OutboundPacket out)
     {
     }

     private InboundPacket getInboundPacket()
     {
          return new InboundPacket(this);
     }

     private void returnInboundPacket(InboundPacket in)
     {
          if (in != null)
          {
               synchronized (this)
               {
                    packetsToDispatch--;

                    if (packetsToDispatch <= 0)
                    {
                         notifyAll();
                    }
               }
          }
     }

     private SystemException parseErrorXML(int errorStatus,String errorXML,String errorUUID)
     {
          Document errorDoc = null;
          String errorMessage = null;
          Throwable cause = null;

          if (errorXML != null)
          {
               synchronized (errorDocBuilder)
               {
                    try
                    {
                         StringReader errorReader = new StringReader(errorXML);
                         InputSource errorSource = new InputSource(errorReader);
                         errorDoc = errorDocBuilder.parse(errorSource);
                    }
                    catch (SAXParseException spe)
                    {
                         // must be a V8 server that doesn't send XML
                         errorMessage = errorXML;
                    }
                    catch (Exception e)
                    {
                         // {0}: An exception was thrown while parsing an XML message from the server.
                         String messageIndex = "sam.S246.ex.msg"/*I18NOK:EMS*/;
                         String messagePattern = RB.getStringResource(messageIndex);
                         String message = Message.format(messagePattern,messageIndex);
                         INTERNAL internal = new INTERNAL(message);
                         internal.initCause(e);
                         throw internal;
                    }
               }

               if (errorDoc != null)
               {
                    Node errorRoot = errorDoc.getDocumentElement();
                    NodeList exceptionElementList = errorRoot.getChildNodes();
                    int exceptionElementCount = exceptionElementList.getLength();

                    for (int i=exceptionElementCount-1;i>0;i--)
                    {
                         Node exceptionElement = exceptionElementList.item(i);
                         errorMessage = extractMessage(exceptionElement);
                         Throwable tempError = new GenericError(errorMessage);

                         if (cause != null)
                         {
                              tempError.initCause(cause);
                         }

                         cause = tempError;
                    }

                    Node exceptionElement = null;
                    
                    if (exceptionElementCount > 0)
                    {
                         exceptionElement = exceptionElementList.item(0);
                    }
                    
                    errorMessage = extractMessage(exceptionElement);
               }
          }

          SystemException error = null;

          if (errorStatus < 0)
          {
               error = new UnexpectedUserException(errorMessage,errorStatus,errorUUID);
          }
          else
          {
               switch (errorStatus)
               {
                    default:
                    case 1: //WOIE_FAIL
                    {
                         error = new UNKNOWN(errorMessage);
                         break;
                    }

                    case 2: // WOIE_OUTOFMEMORY
                    {
                         error = new NO_MEMORY(errorMessage);
                         break;
                    }

                    case 3: // WOIE_NOTIMPL
                    {
                         error = new NO_IMPLEMENT(errorMessage);
                         break;
                    }

                    case 4: // WOIE_NOINTERFACE
                    {
                         error = new BAD_OPERATION(errorMessage);
                         break;
                    }

                    case 5: // WOIE_INVALIDARG
                    {
                         error = new BAD_PARAM(errorMessage);
                         break;
                    }

                    case 6: //  WOIE_ACCESSDENIED
                    {
                         error = new NO_PERMISSION(errorMessage);
                         break;
                    }

                    case 8: // WOIE_NONEXISTENT
                    {
                         error = new OBJECT_NOT_EXIST(errorMessage);
                         break;
                    }

                    case 9: // WOIE_EXPIREDCRED
                    {
                         error = new NO_PERMISSION(errorMessage,3,CompletionStatus.COMPLETED_NO);
                         break;
                    }

                    case 10: // WOIE_INVALIDCRED
                    {
                         error = new NO_PERMISSION(errorMessage,4,CompletionStatus.COMPLETED_NO);
                         break;
                    }
               }
          }

          if (cause != null)
          {
               error.initCause(cause);
          }

          return error;
     }

     private static String extractMessage(Node exceptionElement)
     {
          String blank = ""/*I18nOK:EMS*/;
          
          if (exceptionElement == null)
          {
               return blank;
          }

          Node messageElement = exceptionElement.getFirstChild();

          if (messageElement == null)
          {
               return blank;
          }

          Node textElement = messageElement.getFirstChild();

          if (textElement == null)
          {
               return blank;
          }

          return textElement.getNodeValue();
     }

     protected static String methodIDToString(int methodID)
     {
          switch (methodID)
          {
               case ERROR: return "ERROR";
               case USERPASS: return "USERPASS";
               case NEWOBJECT: return "NEWOBJECT";
               case FREEOBJECT: return "FREEOBJECT";
               case CLOSE: return "CLOSE";
               case UUIDFROMOBJECTID: return "UUIDFROMOBJECTID";
               case ADVISE: return "ADVISE";
               case UNADVISE: return "UNADVISE";
               case OBJECTDESTRUCTOR: return "OBJECTDESTRUCTOR";
               case INTERFACEINOBJECT: return "INTERFACEINOBJECT";
               case EVENTSFROMOBJECTID: return "EVENTSFROMOBJECTID";
               case DEFAULTINTERFACE: return "DEFAULTINTERFACE";
               case SERVERTRANSCODE: return "SERVERTRANSCODE";
               case GENPASSWORD: return "GENPASSWORD";
               default: return "WHO_KNOWS?";
          }
     }

     protected static String dosTypeToString(int dosType)
     {
          switch (dosType)
          {
               case DOS_MP_UCHAR: return "DOS_MP_UCHAR:  ";
               case DOS_MP_SHORT: return "DOS_MP_SHORT:  ";
               case DOS_MP_UINT:  return "DOS_MP_UINT:   ";
               case DOS_MP_LONG:  return "DOS_MP_LONG:   ";
               case DOS_MP_ULONG: return "DOS_MP_ULONG:  ";
               case DOS_MP_DOUBLE:return "DOS_MP_DOUBLE: ";
               case DOS_MP_BINARY:return "DOS_MP_BINARY: ";
          }

          return "UNSUPPORTED DOS TYPE " + dosType;
     }

     private static String printBuffer(byte[] byteArray)
     {
          return printBuffer(byteArray,byteArray.length);
     }

     private static String printBuffer(byte[] byteArray,int length)
     {
          StringBuffer buffer = new StringBuffer(length*5);
          StringBuffer textLine = new StringBuffer("                ");

          buffer.append("; 00000000: ");

          for (int i=0;i<length;i++)
          {
               if (((i % 16) == 0) && i != 0)
               {
                    buffer.append("[").append(textLine).append("]").append("\n; ");
                    hexPrint(i,8,buffer);
                    buffer.append(": ");

                    for (int j=0;j<16; j++)
                    {
                         textLine.setCharAt(j,' ');
                    }
               }

               int b = byteArray[i];

               if (b < 0)
               {
                    b += 256;
               }

               hexPrint(b,2,buffer);
               buffer.append(" ");

               if ((b < 32) || (b > 127))
               {
                    textLine.setCharAt(i%16,'.');
               }
               else
               {
                    textLine.setCharAt(i%16,(char)b);
               }
          }

          if (((length % 16) != 0 ) || (length == 0 ))
          {
               for (int i=0;i<16-(length%16);i++)
               {
                    buffer.append("   ");
               }
          }

          buffer.append("[").append(textLine).append("]");
          return buffer.toString();
     }

     private static void hexPrint(int value,int padding,StringBuffer buffer)
     {
          String hexString = "0123456789ABCDEF";

          for (int i=(padding-1);i>=0;i--)
          {
               buffer.append(hexString.charAt((value>>(i*4))&0xF));
          }
     }

     private static void readFully(InputStream in,byte b[]) throws IOException 
     {
          readFully(in,b,0,b.length);
     }

     private static void readFully(InputStream in,byte b[],int off,int len) throws IOException
     {
          if (len < 0)
          {
               throw new IndexOutOfBoundsException();
          }

          int n = 0;

          while (n < len) 
          {
               int count = in.read(b,off+n,len-n);

               if (count < 0)
               {
                    throw new java.io.EOFException();
               }

               n += count;
          }
     }
     
     private static boolean isLoggable(Logger logger,LogRecord record)
     {
          if (logger == null)
          {
               return false;
          }
          
          Level level = record.getLevel();
          
          if (logger.isLoggable(level))
          {
               Handler[] handlerArr = logger.getHandlers();
               
               for (int i=0;i<handlerArr.length;i++)
               {
                    Handler handler = handlerArr[i];
                    
                    if (handler.isLoggable(record))
                    {
                         return true;
                    }
               }
               
               if (logger.getUseParentHandlers())
               {
                    Logger parent = logger.getParent();
                    
                    if (isLoggable(parent,record))
                    {
                         return true;
                    }
               }
          }
          
          return false;
     }
}