/* $Id: VerySimpleFormatter.java,v 1.1.6.2 2004/01/28 16:17:06 sasbfb Exp $ */
package com.sas.net.util;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;
//
// History:
//
// Date    Name            Remarks
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
// 16Dec03 B Beers         S0227513: Log unhandled exceptions.
// 28Jan04 B Beers         S0233558: Fix NullPointerException.
//
/**
 * A logging formatter that extracts a message from a
 * <CODE>LogRecord</CODE> and places it on a line
 * by itself.
 * @author Biff Beers
 */
public class VerySimpleFormatter extends Formatter
{
     /**
      * Create a VerySimpleFormatter.
      */
     public VerySimpleFormatter()
     {
          this(true);
     }

     /**
      * Create a VerySimpleFormatter.
      */
     public VerySimpleFormatter(boolean insertLineBreaks)
     {
          sw = new StringWriter();
          pw = new PrintWriter(sw);
          sb = sw.getBuffer();
          this.insertLineBreaks = insertLineBreaks;
     }

    /**
     * Format the given log record and return the formatted string.
     * <P>
     * The resulting formatted String is a localized and formatted
     * version of the LogRecord's message field on a line by itself.
     * <P>
     * The <CODE>Formatter.formatMessage()</CODE> convenience method
     * is used to localize and format the message field.
     * @param record the log record to be formatted.
     * @return the formatted log record
     */
     public String format(LogRecord logRecord)
     {
          String message = formatMessage(logRecord);
          Throwable thrown = logRecord.getThrown();
          
          if (thrown == null && !insertLineBreaks)
          {
               return message;
          }

          pw.print(message);
          
          if (thrown != null)
          {
               pw.println();
               thrown.printStackTrace(pw);
          }
          
          if (insertLineBreaks)
          {
               pw.println();
          }

          message = sw.toString();
          sb.setLength(0);
          return message;
     }

     private boolean insertLineBreaks /* = false */;
     private PrintWriter pw /* = null */;
     private StringWriter sw /* = null */;
     private StringBuffer sb /* = null */;
}