/***********************************************************************/
/* Title - MVARow.java                                                 */
/* Author - Dave Mackie, SAS Institute                                 */
/* Date - 5 June 2002                                                  */
/* Purpose - first whack at trying to implement caching in the iom     */
/*           driver.                                                   */
/***********************************************************************/
package com.sas.rio;

import java.util.ArrayList;

public class Cache extends ArrayList
{
    /**
     * extension of ArrayList that does not grow.
     * adding to end past the max size ends up pushing everything 
     * up one. (does that make it a "drop-down list"?
     */
    int m_maxSize = 0;
    
    /**
     * constructor
     * @param maxSize the upper size limit for the cache
     */
    public Cache(int maxSize)
    {
	super(maxSize);
	m_maxSize = maxSize;
    }	
    
    /**
     * add method inserts the given object on the end of the list.
     * @param data the object being inserted
     * @return boolean success/failure of operation
     */
    public boolean add(Object data)
    {
	int _size = this.size();
	
	if (_size < m_maxSize)
	    super.add(data);
	else if (_size == m_maxSize)
	    {
		super.remove(0);
		super.add(data);	
	    }
	return true;			
    }
 
    /**
     * sets the size of the cache on the fly.
     * if the new size is smaller than the old size, then 
     * the first (old size - new size) objects are dropped
     * @param s new size
     */
    public void setSize(int s)
    {
	if (s < super.size())
	    {
	    	
		while (super.size() > s)
		    {
			super.remove(0);
					
		    }	
	    }
	m_maxSize = s;
	
    }	
}
