/* $Id$ */
/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */

package com.sas.rio;

import java.sql.SQLException;
import java.sql.CallableStatement;
import java.sql.Time;
import java.sql.Date;
import java.sql.Timestamp;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Array;
import java.sql.Ref;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;
import java.net.URL;
import java.io.InputStream;
import java.io.Reader;
//
//History Records:
//
// Date          Name                Remarks
// 01/29/99  M Srinivasan   - Created.
// 26Apr2004 damack - S0189758 - additional javadoc
//
/**
 * <P><B>Note:</B> None of the methods in this class are supported because SAS software does not support stored
 * procedures. All methods in this class throw an SQLException.
 *
 * @see MVAConnection#prepareCall
 */
public class MVACallableStatement extends MVAPreparedStatement implements CallableStatement {
    private int vendorCode;
    private String messageIndex;
    String m_errorMsg = null;
    /**
     * Constructs a MVACallableStatement
     * @exception SQLException This exception is required by the interface, but it is never thrown.
     *
     */
    public MVACallableStatement() throws SQLException {
    }

    /**
     *  Registers the OUT parameter in ordinal position parameterIndex to the JDBC type sqlType.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public void registerOutParameter(int parameterIndex, int sqlType)
	    throws SQLException {
	throwException();
    }

    /**
     *  Registers the OUT parameter in ordinal position parameterIndex to the JDBC type sqlType.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public void registerOutParameter(int parameterIndex, int sqlType, int scale)
	    throws SQLException {
	throwException();
    }

    /**
     *  Retrieves whether the last OUT parameter read had the value of SQL NULL.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public boolean wasNull() throws SQLException {
	throwException();
        return false;
    }

    /**
     *  Retrieves the value of the designated JDBC CHAR, VARCHAR, or LONGVARCHAR parameter as a String in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public String getString(int parameterIndex) throws SQLException {
	throwException();
        return null;
    }

    /**
     * Retrieves the value of the designated JDBC BIT parameter as a boolean in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public boolean getBoolean(int parameterIndex) throws SQLException {
	throwException();
        return false;
    }

    /**
     * Retrieves the value of the designated JDBC TINYINT parameter as a byte in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public byte getByte(int parameterIndex) throws SQLException {
	throwException();
        return 0;
    }

    /**
     * Retrieves the value of the designated JDBC SMALLINT parameter as a short in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public short getShort(int parameterIndex) throws SQLException {
	throwException();
        return 0;
    }

    /**
     *  Retrieves the value of the designated JDBC INTEGER parameter as an int in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public int getInt(int parameterIndex) throws SQLException {
	throwException();
        return 0;
    }

    /**
     * Retrieves the value of the designated JDBC BIGINT parameter as a long in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public long getLong(int parameterIndex) throws SQLException {
	throwException();
        return 0;
    }

    /**
     * Retrieves the value of the designated JDBC FLOAT parameter as a float  in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public float getFloat(int parameterIndex) throws SQLException {
	throwException();
        return (float)0.0;
    }

    /**
     * Retrieves the value of the designated JDBC DOUBLE parameter as a double  in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public double getDouble(int parameterIndex) throws SQLException {
	throwException();
        return 0.0;
    }

    /**
     *  Retrieves the value of the designated JDBC NUMERIC parameter as a java.math.BigDecimal object with as many digits to the right of the decimal point as the value contains.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     * @deprecated
     */
    public BigDecimal getBigDecimal(int parameterIndex, int scale) throws SQLException {
	throwException();
        return null;
   }

    /**
     *  Retrieves the value of the designated JDBC BINARY or VARBINARY parameter as an array of byte values in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public byte[] getBytes(int parameterIndex) throws SQLException {
	throwException();
        return null;
    }

    /**
     * Retrieves the value of the designated JDBC DATE parameter as a java.sql.Date object.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public Date getDate(int parameterIndex) throws SQLException {
        throwException();
        return null;
    }

    /**
     *  Retrieves the value of the designated JDBC TIME parameter as a java.sql.Time object.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public Time getTime(int parameterIndex) throws SQLException {
	throwException();
        return null;
    }

    /**
     * Retrieves the value of the designated JDBC TIMESTAMP parameter as a java.sql.Timestamp object.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public Timestamp getTimestamp(int parameterIndex)
	    throws SQLException {
	throwException();
        return null;
    }

    /**
     * Retrieves the value of the designated parameter as an Object in the Java programming language.
     * This method is not supported.
     * @exception SQLException This exception is always thrown.
     */
    public Object getObject(int parameterIndex) throws SQLException {
	throwException();
        return null;
    }

    //--------------------------JDBC 2.0-----------------------------

    /**
     * JDBC 2.0
     *
     * Gets the value of a NUMERIC parameter as a java.math.BigDecimal object.
     * This method is not supported.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @return the parameter value (full precision); if the value is SQL NULL,
     * the result is null
     * @exception SQLException This exception is always thrown.
     * @deprecated
     */
    public BigDecimal getBigDecimal(int parameterIndex) throws SQLException {
	throwException();
        return null;
    }

    /**
     * JDBC 2.0
     *
     * Gets an object representing the value of OUT parameter @i.
     * Use the @map to determine the class from which to construct
     * data of SQL structured and distinct types.
     * This method is not supported.
     *
     * @param i the first parameter is 1, the second is 2, ...
     * @param map the mapping from SQL type names to Java classes
     * @return a java.lang.Object holding the OUT parameter value.
     * @exception SQLException This exception is always thrown.
     */
    public Object getObject(int i, java.util.Map map) throws SQLException {
	throwException();
        return null;
    }

    /**
     * JDBC 2.0
     *
     * Gets a REF(&lt;structured-type&gt;) OUT parameter.
     * This method is not supported.
     *
     * @param i the first parameter is 1, the second is 2, ...
     * @return an object representing data of an SQL REF Type
     * @exception SQLException This exception is always thrown.
     */
    public Ref getRef(int i) throws SQLException {
	throwException();
        return null;
    }

    /**
     * JDBC 2.0
     *
     * Gets a BLOB OUT parameter.
     * This method is not supported.
     *
     * @param i the first parameter is 1, the second is 2, ...
     * @return an object representing a BLOB
     * @exception SQLException This exception is always thrown.
     */
    public Blob getBlob(int i) throws SQLException {
	throwException();
        return null;
     }

    /**
     * JDBC 2.0
     *
     * Gets a CLOB OUT parameter.
     * This method is not supported.
     *
     * @param i the first parameter is 1, the second is 2, ...
     * @return an object representing a CLOB
     * @exception SQLException This exception is always thrown.
     */
    public Clob getClob(int i) throws SQLException {
	throwException();
        return null;
    }

    /**
     * JDBC 2.0
     *
     * Gets an Array OUT parameter.
     * This method is not supported.
     *
     * @param i the first parameter is 1, the second is 2, ...
     * @return an object representing an SQL array
     * @exception SQLException This exception is always thrown.
     */
    public Array getArray(int i) throws SQLException {
	throwException();
        return null;
     }

    /**
     * Gets the value of a SQL DATE parameter as a java.sql.Date object.
     * This method is not supported.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @return the parameter value; if the value is SQL NULL, the result is
     * null.
     * @exception SQLException This exception is always thrown.
     */
    public java.sql.Date getDate(int parameterIndex, Calendar cal)
            throws SQLException {
	throwException();
        return null;
     }

    /**
     * Gets the value of a SQL TIME parameter as a java.sql.Time object.
     * This method is not supported.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @return the parameter value; if the value is SQL NULL, the result is
	 * null
     * @exception SQLException This exception is always thrown.
     */
    public java.sql.Time getTime(int parameterIndex, Calendar cal)
            throws SQLException {
	throwException();
	return null;
    }

    /**
     * Get the value of a SQL TIMESTAMP parameter as a java.sql.Timestamp
     * object.
     * This method is not supported.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @return the parameter value; if the value is SQL NULL, the result is
     * null
     * @exception SQLException This exception is always thrown.
     */
    public java.sql.Timestamp getTimestamp(int parameterIndex, Calendar cal)
            throws SQLException {
	throwException();
         return null;
    }

    private void throwException() throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );
	
    }
    
	/**
	 *  Retrieves the value of the designated JDBC ARRAY parameter as an Array object in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Array getArray(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );
		return null;
	}
	/**
	 *  Retrieves the value of the designated JDBC NUMERIC parameter as a java.math.BigDecimal object with as many digits to the right of the decimal point as the value contains.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public BigDecimal getBigDecimal(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );
		return null;	
	}
	/**
	 *   Retrieves the value of the designated JDBC BLOB parameter as a Blob object in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Blob getBlob(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated JDBC BIT parameter as a boolean in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public boolean getBoolean(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return false;
	}
	/**
	 * Retrieves the value of the designated JDBC TINYINT parameter as a byte in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public byte getByte(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return 0;
	}
	/**
	 *  Retrieves the value of the designated JDBC BINARY or VARBINARY parameter as an array of byte values in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public byte[] getBytes(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return new byte[0];
	}
	/**
	 * Retrieves the value of the designated JDBC CLOB parameter as a Clob object in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Clob getClob(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated JDBC DATE parameter as a java.sql.Date object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Date getDate(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated JDBC DATE parameter as a java.sql.Date object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Date getDate(String parameterName, Calendar cal) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated JDBC DOUBLE parameter as a double  in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public double getDouble(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return (double)0.0;
	}
	/**
	 * Retrieves the value of the designated JDBC FLOAT parameter as a float  in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public float getFloat(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return (float)0.0;
	}
	/**
	 *  Retrieves the value of the designated JDBC INTEGER parameter as an int in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public int getInt(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );
		return (int)0;	
	}
	/**
	 * Retrieves the value of the designated JDBC BIGINT parameter as a long in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public long getLong(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );
		return (long)0;	
	}
	/**
	 * Retrieves the value of the designated parameter as an Object in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Object getObject(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated parameter as an Object in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Object getObject(String parameterName, Map map) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );
		return null;	
	}
	/**
	 * Retrieves the value of the designated JDBC REF(<structured-type>)  parameter as a Ref object in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Ref getRef(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}	
	/**
	 * Retrieves the value of the designated JDBC SMALLINT parameter as a short in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */	
	public short getShort(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return (short)0;
	}
	/**
	 * Retrieves the value of the designated JDBC CHAR, VARCHAR, or LONGVARCHAR parameter as a String in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public String getString(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated JDBC TIME parameter as a java.sql.Time object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Time getTime(String parameterName, Calendar cal) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 *  Retrieves the value of the designated JDBC TIME parameter as a java.sql.Time object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Time getTime(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated JDBC TIMESTAMP parameter as a java.sql.Timestamp object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Timestamp getTimestamp(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of the designated JDBC TIMESTAMP parameter as a java.sql.Timestamp object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public Timestamp getTimestamp(String parameterName, Calendar cal) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of a JDBC DATALINK parameter as a java.net.URL object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public URL getURL(int parameterIndex) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Retrieves the value of a JDBC DATALINK parameter as a java.net.URL object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public URL getURL(String parameterName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
	/**
	 * Registers the OUT parameter in ordinal position parameterIndex to the JDBC type sqlType.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void registerOutParameter(String parameterName, int sqlType) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}	
	/**
	 * Registers the OUT parameter in ordinal position parameterIndex to the JDBC type sqlType.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void registerOutParameter(String parameterName, int sqlType,
										int scale) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Registers the OUT parameter in ordinal position parameterIndex to the JDBC type sqlType.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void registerOutParameter(String parameterName, int sqlType, 
										String typeName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}	
	/**
	 * Sets the designated parameter to the given input stream, which will have the specified number of bytes.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setAsciiStream(String parameterName, InputStream x, int length)
								throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 *  Sets the designated parameter to the given java.math.BigDecimal value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setBigDecimal(String parameterName, BigDecimal x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given input stream, which will have the specified number of bytes.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setBinaryStream(String parameterName, InputStream x, int length)
							throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given Java boolean value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setBoolean(String parameterName, boolean x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given Java byte value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setByte(String parameterName, byte x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given Java array of bytes.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setBytes(String parameterName, byte[] x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 *  Sets the designated parameter to the given Reader  object, which is the given number of characters long.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setCharacterStream(String parameterName, Reader reader, int length)
							throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given java.sql.Date value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setDate(String parameterName, Date x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given java.sql.Date value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setDate(String parameterName, Date x, Calendar cal) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given Java double value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setDouble(String parameterName, double x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 *  Sets the designated parameter to the given Java float value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setFloat(String parameterName, float x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 *  Sets the designated parameter to the given Java int value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setInt(String parameterName, int x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 *  Sets the designated parameter to the given Java long value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setLong(String parameterName, long x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to SQL NULL.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setNull(String parameterName, int sqlType) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to SQL NULL.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setNull(String parameterName, int sqlType, String typeName) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the value of the designated parameter with the given object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setObject(String parameterName, Object x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the value of the designated parameter with the given object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setObject(String parameterName, Object x, int targetSqlType) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the value of the designated parameter with the given object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setObject(String parameterName, Object x, int targetSqlType, int scale) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given Java short value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setShort(String parameterName, short x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	
	/**
	 *  Retrieves the value of the designated JDBC CHAR, VARCHAR, or LONGVARCHAR parameter as a String in the Java programming language.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setString(String parameterName, String x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given java.sql.Time value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setTime(String parameterName, Time x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given java.sql.Time value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setTime(String parameterName, Time x, Calendar cal) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );
	}	
	/**
	 * Sets the designated parameter to the given java.sql.Timestamp value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setTimestamp(String parameterName, Timestamp x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given java.sql.Timestamp value.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setTimestamp(String parameterName, Timestamp x, Calendar cal) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}
	/**
	 * Sets the designated parameter to the given java.net.URL object.
	 * This method is not implemented.
	 * @exception SQLException This exception is always thrown.
	 */
	public void setURL(String parameterName, URL val) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
	}

}

