/* $Id$ */
/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */
package com.sas.rio;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.sql.Types;
import org.omg.CORBA.IntHolder;
//
//History Records:
//
// Date          Name                Remarks
//
// 06/03/99      M Srinivasan       -Change to getDatabaseProductVersion. Get IWorkspace
//                                   from IDataService to get IUtilities.
// 07/26/99      M Srinivasan       -S0061725 :In getTables() call,tabletypes can be NULL even
//                                   if tablenamepattern is not null. So, moved NULL check
//                                   further down.
// 08/24/99      M Srinivasan       -S0064453 :Return artificial resultset for getTypeInfo() via
//                                   MVASqlTypesResultSet.
// 10/25/99      M Srinivasan       -S0068741 :Change driver version to 1.1.
// 12/01/99      M Srinivasan       -S0071124 :Fix to getColumns. Throws NullPointerException when
//                                   table is null.
// 10/27/00      B Beers            -S0100866 :Change driver version to 1.3.
// 12/05/2001    D Mackie           - S0119328 changed the column names for the getTables method s.t. they conform to the JDBC spec.
// 26Apr2004     damack             -S0189758: additional javadoc
// 14Jan2005     damack             -S0278967: added properties object to ctor, so that I can get a local copy,
//                                  and pass it to MVAModifiedResultSet when getColumns and getBestRowIdentifier are
//                                  called.
/**
 * This class provides information about the database as a whole.
 *
 * <P>Many of the methods here return lists of information in ResultSets.
 * You can use the normal ResultSet methods such as getString and getInt
 * to retrieve the data from these ResultSets.  If a given form of
 * metadata is not available, these methods should throw a SQLException.
 *
 * <P>Some of these methods take arguments that are String patterns.  These
 * arguments all have names such as fooPattern.  Within a pattern String, "%"
 * means match any substring of 0 or more characters, and "_" means match
 * any one character. Only metadata entries matching the search pattern
 * are returned. If a search pattern argument is set to a null ref, it means
 * that argument's criteria should be dropped from the search.
 *
 * <P>A SQLException will be thrown if a driver does not support a meta
 * data method.  In the case of methods that return a ResultSet,
 * either a ResultSet (which may be empty) is returned or a
 * SQLException is thrown.
 */
public class MVADatabaseMetaData implements java.sql.DatabaseMetaData
{
    private java.util.Properties m_info;
	private static String driverName = null;
	private MVAConnection connection;
	static int specialColumns = 2;
	static int specialBestRowIdentifier = 3;
	static int specialTypeInfo = 4;
	static int specialIndexInfo = 5;
	private ResultSet resultset = null;
	protected short sv = 0;
	private int vendorCode;
	private String messageIndex;
	public long bookmarkLength = 0;
	public long status = 0;
	public IntHolder bookmarkLengthHolder = null;
	//    private IDataService sasDataService = null;
	//    public IDataSet sasDataSet = null;
	private IntHolder statusHolder = null;
	private XUtil m_xUtil = null;
	private int m_dsHashCode = -1;
	/*
	 * Constructs MVADatabaseMetaData object.
	 * @param connection MVAConnection object
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public MVADatabaseMetaData(MVAConnection connection) throws SQLException
	{
		this.connection = connection;
		m_xUtil = connection.getXUtil();
		//        this.dbmsConnection = connection.dbmsConnection;
		//this.sasDataService = connection.sasDataService;
	}
	public MVADatabaseMetaData(MVAConnection connection, java.util.Properties p) throws SQLException
	{
		this.connection = connection;
		m_xUtil = connection.getXUtil();
		/*
		 * S0278967: copy the properties object that has been passed in.
		 */
		m_info = p;
		//        this.dbmsConnection = connection.dbmsConnection;
		//this.sasDataService = connection.sasDataService;
	}
	//----------------------------------------------------------------------
	// First, a variety of minor information about the target database.
	/**
	 * Determines whether all stored procedures are callable.
	 * @return False. SAS software does not support stored procedures.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean allProceduresAreCallable() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether all accessible tables are selectable.
	 * @return True. All accessable tables are selectable.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean allTablesAreSelectable() throws SQLException
	{
		return true;
	}
	/**
	 * Gets the URL from the MVAConnection object.
	 * @return The URL that is obtained from the MVAConnection object.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getURL() throws SQLException
	{
		return connection.getUrl();
	}
	/**
	 * Gets the user name from the MVAConnection object.
	 * @return The user name that is obtained from the MVAConnection object.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getUserName() throws SQLException
	{
		return connection.getUser();
	}
	/**
	 * Determines whether the connection is read-only.
	 * @return False. SAS software does not support read-only connections.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean isReadOnly() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether NULLs are sorted high.
	 * @return False. NULLs are sorted low.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean nullsAreSortedHigh() throws SQLException
	{
		return false;
	}
	/**
	* Determines whether NULLs are sorted low.
	* @return True. NULLs are sorted low.
	* @exception SQLException This exception is required by the interface, but it is never thrown.
	*
	*/
	public boolean nullsAreSortedLow() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether NULLs are sorted at start.
	 * @return False. NULLs are sorted low
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean nullsAreSortedAtStart() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether NULLs are sorted at end.
	 * @return False. NULLs are sorted low
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean nullsAreSortedAtEnd() throws SQLException
	{
		return false;
	}
	/**
	 * Gets the database product name.
	 * @return The string "SAS".
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getDatabaseProductName() throws SQLException
	{
		return new String("SAS");
	}
	/**
	 * Returns the version of the MVA server.
	 * @return A string that contains the version number from the MVA server.
	 * @exception SQLException This exception is thrown if an error condition is detected or if the
	 * server does not respond.
	  */
	public String getDatabaseProductVersion() throws SQLException
	{
		try
		{
			String s = m_xUtil.getDBMSVersion();
			return s;
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_productVersionError,
				null,
				se);
		}
		return null;
	}
	/**
	 * Gets the driver name.
	 * @return the the driver name.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getDriverName() throws SQLException
	{
		if (driverName == null)
		{
			synchronized (MVADatabaseMetaData.class)
			{
				if (driverName == null)
				{
					StringBuffer nameBuffer =
						new StringBuffer("SAS/Integrated Object Model driver for JDBC, Version ");
					nameBuffer.append(MVADriver.majorVersion).append(
						".").append(
						MVADriver.minorVersion);
					driverName = nameBuffer.append(" Production").toString();
				}
			}
		}
		return driverName;
	}
	/**
	 * Gets the driver version number as a string.
	 * @return String "1.0".
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getDriverVersion() throws SQLException
	{
		return new String(
			MVADriver.majorVersion + "." + MVADriver.minorVersion);
	}
	/**
	 * Gets the driver major version number as a string.
	 * @return 1.
	 *
	 */
	public int getDriverMajorVersion()
	{
		return MVADriver.majorVersion;
	}
	/**
	 * Gets the driver minor version number as a string.
	 * @return 0.
	 *
	 */
	public int getDriverMinorVersion()
	{
		return MVADriver.minorVersion;
	}
	/**
	 * Determines whether the database stores each table in a local file.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean usesLocalFiles() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database uses a file for each table.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean usesLocalFilePerTable() throws SQLException
	{
		return false;
	}
	/**
	* Determines whether the database supports mixed-case, unquoted SQL identifiers.
	* @return False.
	* @exception SQLException This exception is required by the interface, but it is never thrown.
	*
	*/
	public boolean supportsMixedCaseIdentifiers() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database stores uppercase identifiers.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean storesUpperCaseIdentifiers() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database stores lowercase identifiers.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean storesLowerCaseIdentifiers() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database stores mixed-case identifiers.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean storesMixedCaseIdentifiers() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether the database supports mixed-case quoted SQL identifiers.
	 * This method is not JDBC compliant.
	 * @return false.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsMixedCaseQuotedIdentifiers() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database stores uppercase quoted SQL identifiers.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean storesUpperCaseQuotedIdentifiers() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database stores lowercase quoted SQL identifiers.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean storesLowerCaseQuotedIdentifiers() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database stores mixed-case quoted SQL identifiers.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean storesMixedCaseQuotedIdentifiers() throws SQLException
	{
		return false;
	}
	/**
	 * Gets the quote string that is used to surround quoted SQL identifiers.
	 * SAS software does not support quoted identifiers, so a blank is
	 * returned.
	 * NOTE - SAS Software supports quoted ids, but only if the "dquote=ansi" option
	 * is set.  The metadata will only consider what the default is, so no quoting
	 * possible.
	 * This method is not JDBC compliant.
	 * @return A blank string (" ").
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getIdentifierQuoteString() throws SQLException
	{
		return "";
	}
	/**
	 * Gets a comma-delimited list of all database-specific SQL keywords
	 * that are not SQL92 keywords.
	 * @return An empty string (""). No additional keywords are reserved.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getSQLKeywords() throws SQLException
	{
		return "";
	}
	/**
	 * Gets a comma-delimited list of math functions.
	 * @return The string "ABS, ACOS, ASIN, ATAN, CEILING, COS, EXP, FLOOR, LOG, LOG10, MOD, POWER, RAND, ROUND, SIGN, SIN, SQRT, TAN"
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getNumericFunctions() throws SQLException
	{
		return "ABS, ACOS, ASIN, ATAN, CEILING, COS, EXP, FLOOR, LOG, LOG10, MOD, POWER, RAND, ROUND, SIGN, SIN, SQRT, TAN";
	}
	/**
	 * Gets a comma-delimited list of string functions.
	 * @return The string "ASCII, CHAR, CONCAT, LCASE, LEFT, LTRIM, REPEAT, REPLACE, RTRIM, SOUNDEX, SPACE, SUBSTRING, UCASE"
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getStringFunctions() throws SQLException
	{
		return "ASCII, CHAR, CONCAT, LCASE, LEFT, LTRIM, REPEAT, REPLACE, RTRIM, SOUNDEX, SPACE, SUBSTRING, UCASE";
	}
	/**
	 * Gets a comma-delimited list of system functions.
	 * SAS software does not support any system functions, so an
	 * empty string is returned.
	 * @return An empty string ("").
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getSystemFunctions() throws SQLException
	{
		return "";
	}
	/**
	 * Gets a comma-delimited list of time and date functions.
	 * @return The string "CURDATE, CURTIME, DAYNAME, DAYOFMONTH, DAYOFWEEK, DAYOFYEAR, HOUR, MINUTE, MONTH, MONTHNAME, NOW, QUARTER, SECOND, YEAR"
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getTimeDateFunctions() throws SQLException
	{
		return "CURDATE, CURTIME, DAYNAME, DAYOFMONTH, DAYOFWEEK, DAYOFYEAR, HOUR, MINUTE, MONTH, MONTHNAME, NOW, QUARTER, SECOND, YEAR";
	}
	/**
	 * Gets the string that can be used to escape an underscore (_) or a percentage sign (%) in
	 * string based searches such as getTables(). SAS software does not support any escape
	 * characters for schema or table names, so an empty string is returned.
	 * @return An empty string ("").
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getSearchStringEscape() throws SQLException
	{
		return "";
	}
	/**
	 * Gets a string that contains all the "extra" characters that identier names can contain.
	 * SAS software does not support any additional characters for names, so an
	 * empty string is returned.
	 * @return An empty string ("").
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getExtraNameCharacters() throws SQLException
	{
		return "";
	}
	/**
	 * Determines whether ALTER TABLE with an add column is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsAlterTableWithAddColumn() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether ALTER TABLE with a drop column is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsAlterTableWithDropColumn() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether column aliasing is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsColumnAliasing() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether the concatenations between NULL and non-NULL values are NULL.
	 * This method is not SQL-92 compliant.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean nullPlusNonNullIsNull() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the CONVERT function for conversions between SQL types is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsConvert() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the CONVERT function for conversions between the given SQL types from
	 * java.sql.Types is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsConvert(int fromType, int toType)
		throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether table correlation names are supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsTableCorrelationNames() throws SQLException
	{
		return true;
	}
	/**
	 * When table correlation names are supported, determines whether table correlation names must
	 * be different from the table names.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsDifferentTableCorrelationNames() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether expressions are supported in "ORDER BY" lists.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsExpressionsInOrderBy() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether an ORDER BY clause can use columns that are not in the SELECT list.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsOrderByUnrelated() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether some form of the GROUP BY clause is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsGroupBy() throws SQLException
	{
		return true;
	}
	/**
	 * Detrmines whether GROUP BY clause can use columns that are not in the SELECT list.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsGroupByUnrelated() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether a GROUP BY clause can add columns that are not in the SELECT list.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsGroupByBeyondSelect() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether the escape character in LIKE clauses is supported.
	 * This method is not JDBC compliant.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsLikeEscapeClause() throws SQLException
	{
		/*
		 * for versions of SAS prior to 8.2, we do not support the Like escape clause
		 */
		return false;
	}
	/**
	 * Determines whether mulitple ResultSets from a single execute are supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsMultipleResultSets() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether multiple transactions can be open concurrently.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsMultipleTransactions() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether columns can support NULLs.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsNonNullableColumns() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether the ODBC Minimum SQL grammar is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsMinimumSQLGrammar() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether the ODBC Core SQL grammar is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsCoreSQLGrammar() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the ODBC Extended SQL grammar is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsExtendedSQLGrammar() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether ANSI92 entry level SQL grammar is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 */
	public boolean supportsANSI92EntryLevelSQL() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether ANSI92 intermediate SQL grammar is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 */
	public boolean supportsANSI92IntermediateSQL() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether ANSI92 full SQL grammar is supported.
	 * @return False
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 */
	public boolean supportsANSI92FullSQL() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the SQL Integrity Enhancement Facility is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsIntegrityEnhancementFacility() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether some form of outer join is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsOuterJoins() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether full, nested outer joins are supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsFullOuterJoins() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether there is limited support for outer joins.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsLimitedOuterJoins() throws SQLException
	{
		return true;
	}
	/**
	 * Gets the preferred term for "schema".
	 * @return The string "library".
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getSchemaTerm() throws SQLException
	{
		return "library";
	}
	/**
	 * Gets the preferred term for "procedure".
	 * @return An empty string ("").
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getProcedureTerm() throws SQLException
	{
		return "";
	}
	/**
	 * Gets the preferred term for "catalog".
	 * @return An empty string ("").
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getCatalogTerm() throws SQLException
	{
		return "";
	}
	/**
	 * Determines whether a catalog appears at the start of a qualified table name.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean isCatalogAtStart() throws SQLException
	{
		return false;
	}
	/**
	 * Gets the separator that is being used between the catalog name and the table name.
	 * @return An empty string ("").
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public String getCatalogSeparator() throws SQLException
	{
		return "";
	}
	/**
	 * Determines whether a data manipulation statementcaon contain a schema name.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSchemasInDataManipulation() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether a procedure call statement can contain a schema name.
	 * @return False. SAS software does not support stored procedures.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSchemasInProcedureCalls() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether a table definition statement can contain a schema name.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSchemasInTableDefinitions() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether an index definition statement can contain a schema name.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSchemasInIndexDefinitions() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether a privilege definition statement can contain a schema name.
	 * @return False. SAS software does not support privileges.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSchemasInPrivilegeDefinitions() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether a data manipulation statement can contain a catalog name.
	 * @return False. SAS software does not support SQL catalogs.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsCatalogsInDataManipulation() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether a procedure call statement can contain a catalog name.
	 * @return False. SAS software does not support SQL catalogs.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsCatalogsInProcedureCalls() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether a table definition statement can contain a catalog name.
	 * @return False. SAS software does not support SQL catalogs.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsCatalogsInTableDefinitions() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether an index definition statement can contain a catalog name.
	 * @return False. SAS software does not support SQL catalogs.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsCatalogsInIndexDefinitions() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether a privilege definition statement can contain a catalog name.
	 * @return False. SAS software does not support SQL catalogs.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsCatalogsInPrivilegeDefinitions() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether positioned DELETE is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsPositionedDelete() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether positioned UPDATE is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsPositionedUpdate() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether SELECT for UPDATE is supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSelectForUpdate() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether stored procedure calls that use the stored procedure escape
	 * syntax are supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsStoredProcedures() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether subqueries in comparison expressions are supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSubqueriesInComparisons() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether subqueries in exists expressions are supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSubqueriesInExists() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether subqueries in "in" statements are supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSubqueriesInIns() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether subqueries in quantified expressions are supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsSubqueriesInQuantifieds() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether correlated subqueries are supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsCorrelatedSubqueries() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether an SQL UNION is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsUnion() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether an SQL UNION ALL is supported.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsUnionAll() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether cursors can remain open across commits.
	 * @return False. SAS software does not support transactions.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsOpenCursorsAcrossCommit() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether cursors can remain open across rollback.
	 * @return False. SAS software does not support transactions.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsOpenCursorsAcrossRollback() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether statements can remain open across commits.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsOpenStatementsAcrossCommit() throws SQLException
	{
		return true;
	}
	/**
	 * Determines whether statements can remain open across rollbacks.
	 * @return True. SAS software does not support transactions.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsOpenStatementsAcrossRollback() throws SQLException
	{
		return true;
	}
	/**
	 *  Returns the number of hex characters that are allowed in an inline binary literal.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxBinaryLiteralLength() throws SQLException
	{
		return 0;
	}
	/**
	 * Returns the maximum length of a character literal.
	 * @return 32767.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxCharLiteralLength() throws SQLException
	{
		return 32767;
	}
	/**
	 * Returns the maximum length of a column name.
	 * @return 32.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxColumnNameLength() throws SQLException
	{
		return 32;
	}
	/**
	 * Indicates the maximum number of columns that are allowed in a GROUP BY clause.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxColumnsInGroupBy() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the  maximum number of columns that are allowed in an index.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxColumnsInIndex() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the  maximum number of columns that are allowed in an ORDER BY clause.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxColumnsInOrderBy() throws SQLException
	{
		return 0;
	}
	/**
	 * Determines the maximum number of columns that are allowed in a SELECT list.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxColumnsInSelect() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum number of columns that are allowed in a table.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxColumnsInTable() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum number of database connections that can be active concurrently.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxConnections() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum length of a cursor name.
	 * @return 0. SAS software does not support cursors.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxCursorNameLength() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum size of an index (in bytes).
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxIndexLength() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum length of a schema name.
	 * @return 8.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxSchemaNameLength() throws SQLException
	{
		return 8;
	}
	/**
	 * Indicates the maximum length of a procedure name.
	 * @return 0. SAS software does not support stored procedures.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxProcedureNameLength() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum length of a catalog name.
	 * @return 0. SAS software does not support SQL catalogs.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxCatalogNameLength() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum length of a single row.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxRowSize() throws SQLException
	{
		return 0;
	}
	/**
	 * Determines whether getMaxRowSize() includes LONGVARCHAR and LONGVARBINARY
	 * blobs.
	 * @return True.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean doesMaxRowSizeIncludeBlobs() throws SQLException
	{
		return true;
	}
	/**
	 * Indicates the maximum length of an SQL statement.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxStatementLength() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum number of active statements that can be open to this database
	 * concurrently.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxStatements() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum length of a table name.
	 * @return 32.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxTableNameLength() throws SQLException
	{
		return 32;
	}
	/**
	 * Indicates the maximum number of tables in a SELECT list.
	 * @return 0. There is no limit.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxTablesInSelect() throws SQLException
	{
		return 0;
	}
	/**
	 * Indicates the maximum length of a user name.
	 * @return 8.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getMaxUserNameLength() throws SQLException
	{
		return 8;
	}
	/**
	 * Gets the default transaction isolation level for the database.
	 * @return java.sql.Connection.TRANSACTION_NONE
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public int getDefaultTransactionIsolation() throws SQLException
	{
		return Connection.TRANSACTION_NONE;
	}
	/**
	 * Determines whether transactions are supported.
	 * @return False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsTransactions() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether the database supports the transaction isolation level.
	 * @return True if the transaction isolation level is set to java.sql.Connection.TRANSACTION_NONE, else False.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsTransactionIsolationLevel(int level)
		throws SQLException
	{
		return (level == Connection.TRANSACTION_NONE);
	}
	/**
	 * Determines whether both data definition and data manipulation statements
	 * within a transaction are supported.
	 * @return False. SAS software does not support transactions.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsDataDefinitionAndDataManipulationTransactions()
		throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether data manipulation statements within a transaction are
	 * supported.
	 * @return False. SAS software does not support transactions.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean supportsDataManipulationTransactionsOnly()
		throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether a data definition statement within a transaction forces the
	 * transaction to commit.
	 * @return False. SAS software does not support transactions.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean dataDefinitionCausesTransactionCommit() throws SQLException
	{
		return false;
	}
	/**
	 * Determines whether a data definition statement within a transaction is ignored.
	 * @return False. SAS software does not support transactions.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 *
	 */
	public boolean dataDefinitionIgnoredInTransactions() throws SQLException
	{
		return false;
	}
	/**
	 * Get a description of stored procedures available in a catalog. This method is not
	 * supported.
	 * @exception SQLException This exception is always thrown.
	 *
	 */
	public ResultSet getProcedures(
		String catalog,
		String schemaPattern,
		String procedureNamePattern)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVADatabaseMetaData_storedProcsUnsupported);
		return null;
	}
	/**
	 * Gets a description of a catalog's stored procedure parameters
	 * and result columns. This method is not supported.
	 * @exception SQLException This exception is always thrown.
	 *
	 */
	public ResultSet getProcedureColumns(
		String catalog,
		String schemaPattern,
		String procedureNamePattern,
		String columnNamePattern)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVADatabaseMetaData_storedProcsUnsupported);
		return null;
	}
	/**
	 * Gets a description of the tables that are available in a catalog.
	 *
	 * <P>Only table descriptions that match the catalog, schema, table
	 * name, and type criteria are returned.  The rows are ordered by
	 * TABLE_TYPE, TABLE_SCHEM and TABLE_NAME.
	 *
	 * <P>Each table description contains the following columns:
	  * <p>
	 * <table border=1 cellpadding=5>
	  * <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	  * <tr><td>CATALOG</td><td>String</td><td>The table catalog (can be NULL)</td></tr>
	  * <tr><td>SCHEMA</td><td>String</td><td>The table schema (can be NULL)</td></tr>
	  * <tr><td>TABLE</td><td>String</td><td>The table name</td></tr>
	  * <tr><td valign=top>TYPE</td><td valign=top>String</td><td>The table type</td></tr>
	  * <tr><td>COMMENT</td><td>String</td><td>An explanatory comment regarding the table</td></tr>
	 * <tr><td>TABLE_PASS</td><td>String</td><td>A non-standard column returning the password
	  * protection of the table. The returned column values would be one of NULL, READ, WRITE, ALTER
	  * values. A NULL is returned if no passwords are defined for the table. Since this is a
	  * non-standard result, this column must be referenced by name.</td></tr>
	  * </table><p>
	  *
	 * @param catalog A catalog name. An empty string ("") retrieves the rows that do not have a catalog.
	 * @param schemaPattern A schema name pattern. An empty string ("") retrieves the rows that do not
	 * have a schema.
	 * @param tableNamePattern A table name pattern.
	 * @param types A list of table types to include. A value of NULL returns all types.
	 * @return ResultSet in which each row is a table description.
	 * @exception SQLException This exception is thrown if there is no connection to server or if an
	 * IO exception is detected.
	 *
	 */
	public ResultSet getTables(
		String catalog,
		String schemaPattern,
		String tableNamePattern,
		String types[])
		throws SQLException
	{
		int hashCode = -1;
		if (connection.isClosed() == true)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.General_noConnectionError);
		}
		boolean where = false;
		String sql;
		/*
		 * per S0119328, (Web.jdbc), the column names are being changed to conform to jdbc spec.
		 */
		if (connection.getRemarks())
			sql =
				new String("select '' as TABLE_CAT, libname as TABLE_SCHEM, memname as TABLE_NAME, memtype as TABLE_TYPE, memlabel as REMARKS, protect as table_pass, count(*) from dictionary.tables as tables");
		else
			sql =
				new String("select '' as TABLE_CAT, libname as TABLE_SCHEM, memname as TABLE_NAME, memtype as TABLE_TYPE, 'No comments' as REMARKS, protect as table_pass, count(*) from dictionary.tables as tables");
		if ((schemaPattern != null) && (schemaPattern.length() != 0))
		{
			sql =
				sql
					+ new String(
						" where libname like '"
							+ schemaPattern.toUpperCase()
							+ "' ");
			where = true;
		}
		if ((tableNamePattern != null) && (tableNamePattern.length() != 0))
		{
			if (where)
				sql = sql + new String(" and ");
			else
			{
				where = true;
				sql = sql + new String(" where ");
			}
			sql = sql + new String(" memname like '" + tableNamePattern + "' ");
		}
		// types are a comma delimited list, each value in quotes
		if ((types != null) && (types.length != 0))
		{
			//map "TABLE" to "DATA" becose, the database stores the table type as "DATA"
			for (int i = 0; i < types.length; i++)
			{
				if ((types[i] != null)
					&& (types[i].length() != 0)
					&& (types[i].trim().toUpperCase().equals("TABLE")))
				{
					types[i] = "DATA";
				}
			}
			if (where)
				sql = sql + new String(" and ");
			else
			{
				where = true;
				sql = sql + new String(" where ");
			}
			sql =
				sql
					+ new String(
						" memtype in ('" + types[0].toUpperCase() + "' ");
			for (int i = 1; i < types.length; i++)
			{
				if ((types[i] != null) && (types[i].length() != 0))
					sql = sql + ", '" + types[i].toUpperCase() + "' ";
			}
			sql = sql + new String(") ");
		}
		/*
		 * per the JDBC spec, the results are ordered by TABLE_TYPE, TABLE_SCHEM, TABLE_NAME
		 */
		sql = sql + new String(" order by TABLE_TYPE, TABLE_SCHEM, TABLE_NAME");
		try
		{
			// void executeQuery([in]string query,[in]boolean randomAccess,[out] short bookmarkLength,[out] IDataSet aSASDataSet,[out] long status);
			boolean randomAccess = false;
			hashCode = m_xUtil.executeQuery(sql, randomAccess);
			//sasDataSet = dbmsConnection.ExecuteQuery(sql,randomAccess,bookmarkLengthHolder,statusHolder, warningHolder);
		}
		catch (Exception e)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_getTableError,
				null,
				e);
		}
		resultset =
			new MVAModifiedResultSet(
				hashCode,
				m_xUtil,
				MVAModifiedResultSet.TABLES);
		return resultset;
	}
	/**
	 * Gets the schema names available in this database.  The results
	 * are ordered by schema name.
	  * <P>Each row of the result set contains the following columns:
	  * <p>
	  * <table border=1 cellpadding=5>
	  * <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	  * <tr><td>SCHEMA</td><td>String</td><td>The schema name</td></tr>
	  * </table>
	  * @return ResultSet in which each row has a single String column that is a schema name.
	 * @exception SQLException This exception may be thrown.
	 *
	 */
	public ResultSet getSchemas() throws SQLException
	{
		int hashCode = -1;
		if (connection.isClosed() == true)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.General_noConnectionError);
		}
		String sql =
			new String("select unique libname as schema from dictionary.members as members order by schema");
		try
		{
			// void executeQuery([in]string query,[in]boolean randomAccess,[out] short bookmarkLength,[out] IDataSet aSASDataSet,[out] long status);
			boolean randomAccess = false;
			//sasDataSet = dbmsConnection.ExecuteQuery(sql,randomAccess,bookmarkLengthHolder,statusHolder,warningHolder);
			hashCode = m_xUtil.executeQuery(sql, randomAccess);
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_getSchemaError,
				null,
				se);
			/* SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
			if (se instanceof SQLException)
			sqle.setNextException((SQLException)se);
			throw sqle;
			*/
		}
		//resultset = new MVAResultSet(sasDataSet);
		resultset = new MVAResultSet(hashCode, m_xUtil);
		return resultset;
	}
	/**
	 * Gets the catalog names that are available in this database. This method is not supported.
	 * @exception SQLException This exception is always thrown because SAS software does not
	 * support SQL catalogs.
	 */
	public ResultSet getCatalogs() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVADatabaseMetaData_getCatalogsError);
		return null;
	}
	/**
	 * Gets the table types available in this database.  The results
	 * are ordered by table type.
	 *
	 * <P>The table type is:
	  * <p>
	  * <table border=1 cellpadding=5>
	  * <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	  * <tr><td valign=top>TYPE</td><td valign=top>String</td><td>The table type</td></tr>
	  * </table><p>
	 *
	 * @return ResultSet in which each row has a single String column that is a table type.
	 * @exception SQLException This exception may be thrown.
	 *
	 */
	public ResultSet getTableTypes() throws SQLException
	{
		int hashCode = -1;
		if (connection.isClosed() == true)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.General_noConnectionError);
		}
		String sql =
			new String("select unique memtype as type from dictionary.tables as tables order by type");
		try
		{
			// void executeQuery([in]string query,[in]boolean randomAccess,[out] short bookmarkLength,[out] IDataSet aSASDataSet,[out] long status);
			boolean randomAccess = false;
			hashCode = m_xUtil.executeQuery(sql, randomAccess);
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_getTableTypesError,
				null,
				se);
			/* SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
			if (se instanceof SQLException)
			sqle.setNextException((SQLException)se);
			throw sqle;
			*/
		}
		resultset =
			new MVAModifiedResultSet(
				hashCode,
				m_xUtil,
				MVAModifiedResultSet.TABLETYPES);
		return resultset;
	}
	/**
	 * Gets a description of table columns available in a catalog.
	 *
	 * <P>Only column descriptions matching the catalog, schema, table and column name criteria are returned.
	  * The descriptions are ordered by TABLE_SCHEM, TABLE_NAME and ORDINAL_POSITION.
	 *  <P>Each column description contains the following columns:
	 * <p>
	 * <table border=1 cellpadding=5>
	 * <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	 * <tr><td>CATALOG</td><td>String</td><td>A table catalog (can be NULL)</td></tr>
	 * <tr><td>SCHEMA</td><td>String</td><td>A table schema (can be NULL)</td></tr>
	 * <tr><td>TABLE</td><td>String</td><td>A table name</td></tr>
	 * <tr><td>NAME</td><td>String</td><td>A column name</td></tr>
	 * <tr><td>SQLTYPE</td><td>short</td><td>An SQL type from java.sql.Types</td></tr>
	 * <tr><td>TYPE</td><td>String</td><td>A data-source-dependent format name including format width and decimal</td></tr>
	 * <tr><td valign=top>SIZE</td><td valign=top>int</td><td>The column size (For char or date
	 *     types, this is the maximum number of characters; for numeric or decimal types, this is precision.)</td></tr>
	 * <tr><td>EMPTY</td><td>NULL</td><td>Not used</td></tr>
	 * <tr><td>SCALE</td><td>int</td><td>The number of fractional digits</td></tr>
	 * <tr><td>RADIX</td><td>int</td><td>The radix (typically either 10 or 2)</td></tr>
	 * <tr><td valign=top>NULLABLE</td>
	 *     <td valign=top>int</td><td>Constants that state whether NULL is allowed:
	 *   <UL>
	 *    <LI> columnNoNulls - NULL values can possibly not be allowed
	 *    <LI> columnNullable - NULL values are allowed
	 *    <LI> columnNullableUnknown - Nullability is unknown
	 *   </UL>
	 * </td></tr>
	 * <tr><td>COMMENT</td><td>String</td><td>A comment describing the column (can be NULL)</td></tr>
	 * <tr><td>DEFAULT</td><td>String</td><td>The default value (can  be NULL)</td></tr>
	 * <tr><td>EMPTY2</td><td>int</td><td>Not used</td></tr>
	 * <tr><td>EMPTY3</td><td>int</td><td>Not used</td></tr>
	 * <tr><td valign=top>MAX</td><td valign=top>int</td><td>For char types, this is the
	 *       maximum number of bytes in the column</td></tr>
	 * <tr><td>POSITION</td><td>int</td><td>The index of the column in the table
	 *   (starting at 1)</td></tr>
	 * <tr><td valign=top>NOTKNOWN</td><td valign=top>String</td><td>Strings that state whether NULL is allowed:
	 *     <ul>
	 *     <li>"NO" means that the column definitely
	 *      does not allow NULL values
	 *     <li>"YES" means that the column might
	 *     allow NULL values
	 *     <li>An empty string ("") means unknown
	 *     </ul>
	 *      </td></tr>
	 * <p></table>
	 * @param catalog A catalog name. An empty string ("") retrieves the rows that do
	 * not have a catalog.
	 * @param schemaPattern A schema name pattern. An empty string ("") retrieves the
	 * rows that do not have a schema.
	 * @param tableNamePattern A table name pattern. For password protected tables the parameter will be
	 * of the form myTable(pw=XXXX).
	 * @param columnNamePattern A column name pattern.
	 * @exception SQLException This exception may be thrown.
	 *
	 */
	public ResultSet getColumns(
		String catalog,
		String schemaPattern,
		String tableNamePattern,
		String columnNamePattern)
		throws SQLException
	{
		int hashCode = -1;
		if (connection.isClosed() == true)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.General_noConnectionError);
		}
		boolean where = false;
		String sql = null;
		ResultSetMetaData rsmd = null;
		Statement stmt = connection.createStatement();
		if ((tableNamePattern != null)
			&& (tableNamePattern.length() != 0)
			&& (tableNamePattern.indexOf('(') != -1))
		{
			int index = tableNamePattern.indexOf('(');
			sql =
				"select * from "
					+ schemaPattern.trim()
					+ "."
					+ tableNamePattern.trim();
			try
			{
				resultset = stmt.executeQuery(sql);
				rsmd = resultset.getMetaData();
			}
			catch (Exception se)
			{
				MVALocalizedError.throwLocalizedException(
					VendorCode.MVADatabaseMetaData_getColumnsError,
					null,
					se);
				/* SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
				if (se instanceof SQLException)
				    sqle.setNextException((SQLException)se);
				throw sqle;
				*/
			}
			//create values
			Object[][] trow = new Object[rsmd.getColumnCount()][18];
			for (int i = 0; i < rsmd.getColumnCount(); i++)
			{
				trow[i][0] = "";
				trow[i][1] = new String(rsmd.getSchemaName(i + 1));
				trow[i][2] = new String(rsmd.getTableName(i + 1));
				trow[i][3] = new String(rsmd.getColumnName(i + 1));
				trow[i][4] = new Double((double) rsmd.getColumnType(i + 1));
				trow[i][5] = new String(rsmd.getColumnTypeName(i + 1));
				trow[i][6] =
					new Double((double) rsmd.getColumnDisplaySize(i + 1));
				trow[i][7] = null;
				trow[i][8] = new Double((double) rsmd.getScale(i + 1));
				trow[i][9] = new Double((double) 10);
				trow[i][10] = new Double((double) rsmd.isNullable(i + 1));
				trow[i][11] = new String(rsmd.getColumnLabel(i + 1));
				trow[i][12] = null;
				trow[i][13] = null;
				trow[i][14] = null;
				trow[i][15] =
					new Double((double) rsmd.getColumnDisplaySize(i + 1));
				trow[i][16] = new Double((double) i + 1);
				trow[i][17] = "";
			}
			resultset.close();
			stmt.close();
			//create variables. build sql and call preparedescribe again.
			sql =
				"select ' ' as catalog, libname as schema, memname as table, name, 0 as sqltype, format as type, length as size, ' ' as empty, 0 as scale, 10 as radix, 0 as nullable, label as comment, ' ' as default, 0 as empty2, 0 as empty3, length as max, varnum as position, ' ' as notknown, format, type as sastype from dictionary.columns as columns";
			if ((schemaPattern != null) && (schemaPattern.length() != 0))
			{
				sql =
					sql
						+ new String(
							" where libname like '"
								+ schemaPattern.toUpperCase()
								+ "' ");
				where = true;
			}
			tableNamePattern = tableNamePattern.substring(0, index).trim();
			if (where)
				sql = sql + new String(" and ");
			else
			{
				where = true;
				sql = sql + new String(" where ");
			}
			sql = sql + new String(" memname like '" + tableNamePattern + "' ");
			if ((columnNamePattern != null)
				&& (columnNamePattern.length() != 0))
			{
				if (where)
					sql = sql + new String(" and ");
				else
				{
					where = true;
					sql = sql + new String(" where ");
				}
				sql =
					sql + new String(" name like '" + columnNamePattern + "' ");
				where = true;
			}
			// types are a comma delimited list, each value in quotes
			sql = sql + new String(" order by schema, table, position");
			try
			{
				// void executeQuery([in]string query,[in]boolean randomAccess,[out] short bookmarkLength,[out] IDataSet aSASDataSet,[out] long status);
				boolean randomAccess = false;
				hashCode = m_xUtil.executeQuery(sql, randomAccess);
			}
			catch (Exception se)
			{
				MVALocalizedError.throwLocalizedException(
					VendorCode.MVADatabaseMetaData_getColumnsError,
					null,
					se);
				/*
				SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
				if (se instanceof SQLException)
				    sqle.setNextException((SQLException)se);
				throw sqle;
				*/
			}

			resultset =
				new MVAModifiedResultSet(
					hashCode,
					m_xUtil,
					trow,
					MVAModifiedResultSet.PASS);
		}
		else
		{
			sql =
				new String("select ' ' as catalog, libname as schema, memname as table, name, 0 as sqltype, format as type, length as size, ' ' as empty, 0 as scale, 10 as radix, 0 as nullable, label as comment, ' ' as default, 0 as empty2, 0 as empty3, length as max, varnum as position, ' ' as notknown, format, type as sastype from dictionary.columns as columns");
			if ((schemaPattern != null) && (schemaPattern.length() != 0))
			{
				sql =
					sql
						+ new String(
							" where libname like '"
								+ schemaPattern.toUpperCase()
								+ "' ");
				where = true;
			}
			if ((tableNamePattern != null) && (tableNamePattern.length() != 0))
			{
				if (where)
					sql = sql + new String(" and ");
				else
				{
					where = true;
					sql = sql + new String(" where ");
				}
				sql =
					sql
						+ new String(" memname like '" + tableNamePattern + "' ");
			}
			if ((columnNamePattern != null)
				&& (columnNamePattern.length() != 0))
			{
				if (where)
					sql = sql + new String(" and ");
				else
				{
					where = true;
					sql = sql + new String(" where ");
				}
				sql =
					sql + new String(" name like '" + columnNamePattern + "' ");
				where = true;
			}
			// types are a comma delimited list, each value in quotes
			sql = sql + new String(" order by schema, table, position");
			int rstypes[] = new int[20];
			rstypes[0] = Types.VARCHAR;
			rstypes[1] = Types.VARCHAR;
			rstypes[2] = Types.VARCHAR;
			rstypes[3] = Types.VARCHAR;
			rstypes[4] = Types.SMALLINT;
			rstypes[5] = Types.VARCHAR;
			rstypes[6] = Types.INTEGER;
			rstypes[7] = Types.NULL;
			rstypes[8] = Types.INTEGER;
			rstypes[9] = Types.INTEGER;
			rstypes[10] = Types.INTEGER;
			rstypes[11] = Types.VARCHAR;
			rstypes[12] = Types.VARCHAR;
			rstypes[13] = Types.NULL;
			rstypes[14] = Types.INTEGER;
			rstypes[15] = Types.INTEGER;
			rstypes[16] = Types.INTEGER;
			rstypes[17] = Types.VARCHAR;
			rstypes[18] = Types.VARCHAR;
			rstypes[19] = Types.VARCHAR;
			try
			{
				// void executeQuery([in]string query,[in]boolean randomAccess,[out] short bookmarkLength,[out] IDataSet aSASDataSet,[out] long status);
				boolean randomAccess = false;
				hashCode = m_xUtil.executeQuery(sql, randomAccess);
			}
			catch (Exception se)
			{
				MVALocalizedError.throwLocalizedException(
					VendorCode.MVADatabaseMetaData_getColumnsError,
					null,
					se);
				/* SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
				if (se instanceof SQLException)
				    sqle.setNextException((SQLException)se);
				throw sqle;
				*/
			}
/*
 * S0278967: pass properties object so that MVAModifiedResultSet knows to whether to return
 * DOUBLE for all numeric column types;
 */
			resultset =
				new MVAModifiedResultSet(
					hashCode,
					m_xUtil,
					rstypes,
					MVAModifiedResultSet.COLUMNS, m_info);
		}
		return resultset;
	}
	/**
	 * Gets a description of the access rights for a table's columns. This method is not supported.
	 * @exception SQLException This exception is always thrown.
	 */
	public ResultSet getColumnPrivileges(
		String catalog,
		String schema,
		String table,
		String columnNamePattern)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotSupported);
		return null;
	}
	/**
	 * Gets a description of the access rights each table available
	 * in a catalog. This method is not supported.
	 * @exception SQLException This exception is always thrown.
	 */
	public ResultSet getTablePrivileges(
		String catalog,
		String schemaPattern,
		String tableNamePattern)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotSupported);
		return null;
	}
	/**
	 * Gets a description of a table's optimal set of columns that
	  * uniquely identifies a row. Each column description contains the following columns:
	  * <p>
	  *  <table border=1 cellpadding=5>
	  * <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	  * <tr><td>SCOPE</td><td>short</td><td>Actual scope of result:
	  *   <UL>
	  *    <LI> bestRowTemporary - Very temporary, while using row
	  *    <LI> bestRowTransaction - Valid for remainder of current transaction
	  *    <LI> bestRowSession - Valid for remainder of current session
	  *   </UL>
	  * </td></tr>
	  * <tr><td>NAME</td><td>String</td><td>Column name</td</tr>
	  * <tr><td>SQLTYPE</td><td>short</td><td>SQL data type from java.sql.Types</td</tr>
	  * <tr><td>TYPE</td><td>String</td><td>Data source dependent type name</td</tr>
	  * <tr><td>SIZE</td><td>int</td><td>Precision</td</tr>
	  * <tr><td>EMPTY</td><td>int</td><td>Not used</td</tr>
	  * <tr><td>SCALE</td><td>short</td><td>Scale</td</tr>
	  * <tr><td>EMPTY2</td><td>short</td><td>A pseudo column (like an Oracle ROWID)
	  *   <UL>
	  *    <LI> bestRowUnknown - May or may not be pseudo column
	  *    <LI> bestRowNotPseudo - Is NOT a pseudo column
	  *    <LI> bestRowPseudo - Is a pseudo column
	  * </UL>
	  * </td></tr>
	  * </table>
	  * <P>
	 * Note: This method has some performance problems when using multiple queries.
	  *
	 * @param catalog A catalog name. An empty string ("") retrieves the rows that do not
	 * have a catalog.
	 * @param schema A schema name pattern. An empty string ("") retrieves the rows that
	 * do not have a schema.
	 * @param table A table name pattern.
	 * @scope Scope of the result.
	 * @param nullable Whether NULL values are allowed.
	 * @return A ResultSet.
	 * @exception SQLException This exception is thrown if a database error is detected.
	 *
	 */
	public ResultSet getBestRowIdentifier(
		String catalog,
		String schema,
		String table,
		int scope,
		boolean nullable)
		throws SQLException
	{
		int hashCode = -1;
		if (connection.isClosed() == true)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.General_noConnectionError);
		}
		boolean where = false;
		String sql =
			new String("select 2 as scope, indexes.name, 0 as sqltype, col.type as type, col.length as size, 0 as buffer, 0 as scale, 1 as pseudo, col.format as format from dictionary.indexes as indexes,");
		sql = sql + new String(" dictionary.columns as col");
		if ((table == null) || (table.trim().length() == 0))
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_noTableNameError);
		}
		else
		{
			if (where)
				sql = sql + new String(" and ");
			else
			{
				where = true;
				sql = sql + new String(" where ");
			}
		}
		sql =
			sql
				+ new String(
					" indexes.memname = '" + table.trim().toUpperCase() + "' ");
		sql =
			sql
				+ new String(
					" and col.memname = '" + table.trim().toUpperCase() + "' ");
		sql = sql + new String(" and indexes.libname = col.libname");
		sql = sql + new String(" and indexes.name = col.name");
		if ((schema != null) && (schema.length() != 0))
		{
			sql =
				sql
					+ new String(
						" and indexes.libname = '"
							+ schema.trim().toUpperCase()
							+ "' ");
			sql =
				sql
					+ new String(
						" and col.libname = '"
							+ schema.trim().toUpperCase()
							+ "' ");
			where = true;
		}
		if (where)
			sql = sql + new String(" and ");
		else
		{
			where = true;
			sql = sql + new String(" where ");
		}
		sql = sql + new String(" indexes.unique = 'yes'");
		sql = sql + new String(" order by indexes.indxname");
		int rstypes[] = new int[9];
		rstypes[0] = Types.SMALLINT;
		rstypes[1] = Types.VARCHAR;
		rstypes[2] = Types.SMALLINT;
		rstypes[3] = Types.VARCHAR;
		rstypes[4] = Types.INTEGER;
		rstypes[5] = Types.INTEGER;
		rstypes[6] = Types.SMALLINT;
		rstypes[7] = Types.SMALLINT;
		rstypes[8] = Types.VARCHAR;
		try
		{
			// void executeQuery([in]string query,[in]boolean randomAccess,[out] short bookmarkLength,[out] IDataSet aSASDataSet,[out] long status);
			boolean randomAccess = false;
			hashCode = m_xUtil.executeQuery(sql, randomAccess);
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_getBestRowIdentifierError,
				null,
				se);
			/* SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
			if (se instanceof SQLException)
			sqle.setNextException((SQLException)se);
			throw sqle;
			*/
		}
		/*
		 * S0278967: pass in properties object to MVAModifiedResultSet so that it knows
		 * whether to return DOUBLE for all numeric column types
		 */
		resultset =
			new MVAModifiedResultSet(
				hashCode,
				m_xUtil,
				rstypes,
				MVAModifiedResultSet.BESTROW, m_info);
		return resultset;
	}
	/**
	 * Gets a description of the table columns that are automatically
	 * updated when any value in a row is updated.  This method is not supported.
	 * @return An exception is always thrown. No columns are automatically updated.
	 * @exception SQLException This exception is always thrown.
	 *
	 */
	public ResultSet getVersionColumns(
		String catalog,
		String schema,
		String table)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotSupported);
		return null;
	}
	/**
	 * Gets a description of a table's primary key columns.  They
	 * are ordered by COLUMN_NAME. This method has some performance problems
	 * due to multiple queries.
	 *
	 * <P>Each primary key column description has the following columns:
	 * <p>
	 *  <table border=1 cellpadding=5>
	 *  <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	 *  <tr><td>CATALOG</td><td>String</td><td>A table catalog (can be NULL)</td></tr>
	 *  <tr><td>SCHEMA</td><td>String</td><td>A table schema (can be NULL)</td></tr>
	 *  <tr><td>TABLE</td><td>String</td><td>A table name</td></tr>
	 *  <tr><td>NAME</td><td>String</td><td>A column name</td></tr>
	 *  <tr><td>SEQ</td><td>short</td><td>A sequence number within primary key</td></tr>
	 *  <tr><td>INDEX</td><td>String</td><td>A primary key name (can be NULL)</td></tr>
	 *  </table>
	 * @param catalog A catalog name. An empty string ("") retrieves rows that do not
	 * have a catalog.
	 * @param schema A schema name pattern. An empty string ("") retrieves rows that
	 * do not have a schema.
	 * @param table A table name pattern.
	 * @return A ResultSet.
	 * @exception SQLException This exception is thrown if a database-access error is detected.
	 *
	 */
	public ResultSet getPrimaryKeys(
		String catalog,
		String schema,
		String table)
		throws SQLException
	{
		int hashCode = -1;
		if (connection.isClosed() == true)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.General_noConnectionError);
		}
		boolean where = false;
		String sql =
			new String("select '' as catalog, libname as schema, memname as table, name, indxpos as seq, indxname as index from dictionary.indexes as indexes");
		if ((schema != null) && (schema.length() != 0))
		{
			sql =
				sql
					+ new String(
						" where libname = '"
							+ schema.trim().toUpperCase()
							+ "' ");
			where = true;
		}
		if ((table == null) || (table.trim().length() == 0))
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_noTableNameError);
		}
		else
		{
			if (where)
				sql = sql + new String(" and ");
			else
			{
				where = true;
				sql = sql + new String(" where ");
			}
			sql =
				sql
					+ new String(
						" memname = '" + table.trim().toUpperCase() + "' ");
		}
		if (where)
			sql = sql + new String(" and ");
		else
		{
			where = true;
			sql = sql + new String(" where ");
		}
		sql = sql + new String(" unique = 'yes'");
		//sql = sql + new String(" order by name");
		int rstypes[] = new int[6];
		rstypes[0] = Types.VARCHAR;
		rstypes[1] = Types.VARCHAR;
		rstypes[2] = Types.VARCHAR;
		rstypes[3] = Types.VARCHAR;
		rstypes[4] = Types.SMALLINT;
		rstypes[5] = Types.VARCHAR;
		try
		{
			boolean randomAccess = false;
			hashCode = m_xUtil.executeQuery(sql, randomAccess);
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_getPrimaryKeysError,
				null,
				se);
			/* SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
			if (se instanceof SQLException)
			sqle.setNextException((SQLException)se);
			throw sqle;
			*/
		}
		resultset =
			new MVAModifiedResultSet(
				hashCode,
				m_xUtil,
				rstypes,
				MVAModifiedResultSet.PRIMARYKEYS);
		return resultset;
	}
	/**
	 * Gets a description of the primary key columns that are
	 * referenced by a table's foreign key columns. This method is not supported.
	 * @exception SQLException This exception is always thrown.
	 *
	 */
	public ResultSet getImportedKeys(
		String catalog,
		String schema,
		String table)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVADatabaseMetaData_foreignKeysError);
		return null;
	}
	/**
	 * Gets a description of a foreign key columns that reference a
	 * table's primary key columns. This method is not supported.
	 * @exception SQLException This exception is always thrown.
	 *
	 */
	public ResultSet getExportedKeys(
		String catalog,
		String schema,
		String table)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVADatabaseMetaData_foreignKeysError);
		return null;
	}
	/**
	 * Gets a description of the foreign key columns in the foreign key
	 * table that reference the primary key columns of the primary key table. This method is not supported.
	 * @exception SQLException This exception is always thrown.
	 *
	 */
	public ResultSet getCrossReference(
		String primaryCatalog,
		String primarySchema,
		String primaryTable,
		String foreignCatalog,
		String foreignSchema,
		String foreignTable)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVADatabaseMetaData_foreignKeysError);
		return null;
	}
	/**
	 * Gets a description of all the standard SQL types supported by
	 * this database. They are ordered by DATA_TYPE and then by how
	 * closely the data type maps to the corresponding JDBC SQL type.
	 * <p>
	  * Each type description has the following columns:
	 * <p>
	  *  <table border=1 cellpadding=5>
	 *  <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	  *  <tr><td>TYPE_NAME</td><td>String</td><td>Type name</td></tr>
	  *  <tr><td>DATA_TYPE</td><td>short</td><td>SQL data type from java.sql.Types</td></tr>
	  *  <tr><td>PRECISION</td><td>int</td><td>maximum precision</td></tr>
	  *  <tr><td>LITERAL_PREFIX</td><td>String</td><td>prefix used to quote a literal (may be null)</td></tr>
	  *  <tr><td>LITERAL_SUFFIX</td><td>String</td><td>suffix used to quote a literal (may be null)</td></tr>
	  *  <tr><td>CREATE_PARAMS</td><td>String</td><td>parameters used in creating the type (may be null)</td></tr>
	  *  <tr><td>NULLABLE</td><td>short</td><td>can you use NULL for this type ?
	 *         <UL>
	 *              <LI> typeNullable - allows NULL values
	 *              <LI> typeNullableUnknown - nullability unknown
	 *      </UL>
	 *  </td></tr>
	  *  <tr><td>CASE_SENSITIVE</td><td>boolean</td><td>is it case sensitive ?</td></tr>
	 *  <tr><td>SEARCHABLE</td><td>short</td><td>can you use "WHERE" based on this type:
	 *         <UL>
	 *              <LI> typePredNone - No support
	 *              <LI> typePredChar - Only supported with WHERE .. LIKE
	 *              <LI> typePredBasic - Supported except for WHERE .. LIKE
	 *              <LI> typeSearchable - Supported for all WHERE ..
	 *      </UL>
	 *  </td></tr>
	  *  <tr><td>UNSIGNED_ATTRIBUTE</td><td>boolean</td><td>is it unsigned ?</td></tr>
	  *  <tr><td>FIXED_PREC_SCALE</td><td>boolean</td><td>can it be a money value ?</td></tr>
	  *  <tr><td>AUTO_INCREMENT</td><td>boolean</td><td>can it be used for an auto-increment value ?</td></tr>
	  *  <tr><td>LOCAL_TYPE_NAME</td><td>String</td><td>Localized version of type name (may be null)</td></tr>
	  *  <tr><td>MINIMUM_SCALE</td><td>short</td><td>minimum scale supported</td></tr>
	  *  <tr><td>MAXIMUM_SCALE</td><td>short</td><td>maximum scale supported</td></tr>
	  *  <tr><td>SQL_DATA_TYPE</td><td>int</td><td>unused</td></tr>
	  *  <tr><td>SQL_DATETIME_SUB</td><td>int</td><td>unused</td></tr>
	  *  <tr><td>NUM_PREC_RADIX</td><td>int</td><td>usually 2 or 10</td></tr>
	 *  </table>
	 *
	 * @return ResultSet in which each row is an SQL type definition.
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
	 */
	public ResultSet getTypeInfo() throws SQLException
	{
		resultset = new MVASqlTypesResultSet();
		return resultset;
	}
	/**
	 * Gets a description of a table's indices and statistics. Each index column description contains the following columns:
	 * <p>
	 *  <table border=1 cellpadding=5>
	 *  <tr><th>Column name</th><th>Data type</th><th>Description</th></tr>
	 *  <tr><td>CATALOG</td><td>String</td><td>A table catalog (can be NULL)</td></tr>
	 *  <tr><td>SCHEMA</td><td>String</td><td>A table schema (can be NULL)</td></tr>
	 *  <tr><td>TABLE</td><td>String</td><td>A table name</td></tr>
	 *  <tr><td>NONUNIQ</td><td>boolean</td><td>Whether index values can be non-unique</td></tr>
	 *  <tr><td>QUAL</td><td>String</td><td>An index catalog (can be NULL)</td></tr>
	 *  <tr><td>INDEX</td><td>String</td><td>An index name</td></tr>
	 *  <tr><td>TYPE</td><td>short</td><td>An index type:
	 *    <UL>
	 *     <LI> tableIndexStatistic - Table statistics returned in conjuction with a table's index descriptions
	 *     <LI> tableIndexClustered - A clustered index
	 *     <LI> tableIndexHashed - A hashed index
	 *     <LI> tableIndexOther - Some other style of index
	 *    </UL>
	 *  </td></tr>
	 *  <tr><td>POS</td><td>short</td><td>Column sequence number within a index</td></tr>
	 *  <tr><td>NAME</td><td>String</td><td>Column name (can be NULL)</td></tr>
	 *  <tr><td>SEQ</td><td>String</td><td>Column sort sequence (can be NULL):
	 *    <UL>
	 *     <LI> "A" - Ascending
	 *     <LI> "D" - Descending
	 * </ul>
	 * </td></tr>
	 *  <tr><td>CARDINAL</td><td>int</td><td>Number of rows in a table or unique values in the index</td></tr>
	 *  <tr><td>PAGES</td><td>int</td><td>Number of pages used for the table or for the current index</td></tr>
	 *  <tr><td>FILTER</td><td>String</td><td>Filter condition, if any (can be NULL)</td></tr>
	 *  </table>
	 *
	 * @param catalog A catalog name. An empty string ("") retrieves rows that do not
	 * have a catalog.
	 * @param schema A schema name pattern. An empty string ("") retrieves rows that do
	 * not have a schema name.
	 * @param table A table name.
	 * @param unique When True, return only indices for unique values;
	 *     when False, return indices regardless of whether unique or not.
	 * @param approximate When True, result is allowed to reflect approximate
	 *     or out of data values; when False, results are requested to be
	 *     accurate.
	 * @return A ResultSet.
	 * @exception SQLException This exception is thrown if a database-access error is detected.
	 *
	 */
	public ResultSet getIndexInfo(
		String catalog,
		String schema,
		String table,
		boolean unique,
		boolean approximate)
		throws SQLException
	{
		int hashCode = -1;
		if (connection.isClosed() == true)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.General_noConnectionError);
		}
		String uniq = null;
		boolean where = false;
		String sql =
			new String("select '' as catalog, libname as schema, memname as table, unique ne 'yes' as nonuniq,");
		sql =
			sql
				+ new String(
					" '' as qual,indxname as index,"
						+ tableIndexClustered
						+ " as type,indxpos as pos, name, 'A' as seq, 0 as cardinal, 0 as pages, '' as filter from dictionary.indexes as indexes");
		if ((schema != null) && (schema.length() != 0))
		{
			sql =
				sql
					+ new String(
						" where libname = '" + schema.toUpperCase() + "' ");
			where = true;
		}
		if ((table == null) || (table.trim().length() == 0))
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_noTableNameError);
		}
		else
		{
			if (where)
				sql = sql + new String(" and ");
			else
			{
				where = true;
				sql = sql + new String(" where ");
			}
			sql = sql + new String(" memname = '" + table.toUpperCase() + "' ");
		}
		if (unique)
		{
			uniq = new String("yes");
			if (where)
				sql = sql + new String(" and ");
			else
			{
				where = true;
				sql = sql + new String(" where ");
			}
			sql = sql + new String(" unique = '" + uniq + "' ");
		}
		int rstypes[] = new int[13];
		rstypes[0] = Types.VARCHAR;
		rstypes[1] = Types.VARCHAR;
		rstypes[2] = Types.VARCHAR;
		rstypes[3] = Types.BIT;
		rstypes[4] = Types.VARCHAR;
		rstypes[5] = Types.VARCHAR;
		rstypes[6] = Types.SMALLINT;
		rstypes[7] = Types.SMALLINT;
		rstypes[8] = Types.VARCHAR;
		rstypes[9] = Types.VARCHAR;
		rstypes[10] = Types.INTEGER;
		rstypes[11] = Types.INTEGER;
		rstypes[12] = Types.VARCHAR;
		try
		{
			boolean randomAccess = false;
			hashCode = m_xUtil.executeQuery(sql, randomAccess);
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVADatabaseMetaData_getIndexInfoError,
				null,
				se);
			/* SQLException sqle = new SQLException(errorMsg, null, VendorCode.VC_UNKNOWNERROR);
			if (se instanceof SQLException)
			sqle.setNextException((SQLException)se);
			throw sqle;
			*/
		}
		resultset =
			new MVAModifiedResultSet(
				hashCode,
				m_xUtil,
				rstypes,
				MVAModifiedResultSet.INDEX);
		return resultset;
	}
	//--------------------------JDBC 2.0-----------------------------
	/**
	 * JDBC 2.0
	 *
	 * Determines if the database supports the given result set type.
	 *
	 * @param type The type defined in java.sql.ResultSet
	 * @return True if the type is TYPE_SCROLL_INSENSITIVE or TYPE_FORWARD_ONLY, else, returns False.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 * @see MVAConnection
	 */
	public boolean supportsResultSetType(int type) throws SQLException
	{
		if ((type == ResultSet.TYPE_SCROLL_INSENSITIVE)
			|| (type == ResultSet.TYPE_FORWARD_ONLY))
			return true;
		else
			return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines if the database supports the concurrency type in combination
	 * with the given result set type.
	 *
	 * @param type The type defined in java.sql.ResultSet.
	 * @param concurrency type The concurrency type defined in java.sql.ResultSet.
	 * @return True if the resultset type is TYPE_SCROLL_INSENSITIVE or TYPE_FORWARD_ONLY and if the
	  * concurrency type is CONCUR_UPDATABLE or CONCUR_READ_ONLY, else returns False.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 * @see MVAConnection
	 */
	public boolean supportsResultSetConcurrency(int type, int concurrency)
		throws SQLException
	{
		if (((type == ResultSet.TYPE_SCROLL_INSENSITIVE)
			|| (type == ResultSet.TYPE_FORWARD_ONLY))
			&& ((concurrency == ResultSet.CONCUR_UPDATABLE)
				|| (concurrency == ResultSet.CONCUR_READ_ONLY)))
			return true;
		else
			return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether a result set's own changes are visible.
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return True. Own changes are visible.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean ownUpdatesAreVisible(int type) throws SQLException
	{
		//TODO: why is the input type required ?
		return true;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether a result set's own changes are visible.
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return True. Own changes are visible.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean ownDeletesAreVisible(int type) throws SQLException
	{
		//TODO: why is the input type required ?
		return true;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether a result set's own changes are visible.
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return True. Own changes are visible.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean ownInsertsAreVisible(int type) throws SQLException
	{
		//TODO: why is the input type required ?
		return true;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether changes made by others are visible.
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return False. Changes made by others are not visible.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean othersUpdatesAreVisible(int type) throws SQLException
	{
		//TODO: check this ?
		return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether changes made by others are visible.
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return False. Changes made by others are not visible.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean othersDeletesAreVisible(int type) throws SQLException
	{
		//TODO: check this ?
		return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether changes made by others are visible.
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return False. Changes made by others are not visible.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean othersInsertsAreVisible(int type) throws SQLException
	{
		return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether or not a visible row update can be detected by
	 * calling ResultSet.rowUpdated().
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return False. Updates are not detected.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean updatesAreDetected(int type) throws SQLException
	{
		return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether or not a visible row delete can be detected by
	 * calling ResultSet.rowDeleted().  If deletesAreDetected()
	 * returns false, then deleted rows are removed from the result set.
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return False. Deletes are not detected.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean deletesAreDetected(int type) throws SQLException
	{
		return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines whether or not a visible row insert can be detected
	 * by calling ResultSet.rowInserted().
	 *
	 * @param result set type. The type defined in java.sql.ResultSet.
	 * @return False. Inserts are not detected.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean insertsAreDetected(int type) throws SQLException
	{
		return false;
	}
	/**
	 * JDBC 2.0
	 *
	 * Determines if the database supports batch updates.
	 *
	 * @return True. Batch updates are supported as sequential queuing of update calls.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public boolean supportsBatchUpdates() throws SQLException
	{
		return true;
	}
	/**
	 * JDBC 2.0
	 *
	 * Gets a description of the user-defined types defined in a particular
	 * schema.  Schema specific UDTs may have type JAVA_OBJECT, STRUCT,
	 * or DISTINCT.
	 *
	 * @param catalog a catalog name; "" retrieves those without a
	 * catalog; null means drop catalog name from the selection criteria
	 * @param schemaPattern a schema name pattern; "" retrieves those
	 * without a schema
	 * @param typeNamePattern a type name pattern; may be a fully qualified
	 * name
	 * @param types a list of user-named types to include (JAVA_OBJECT,
	 * STRUCT, or DISTINCT); null returns all types
	 * @return An empty ResultSet object.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public ResultSet getUDTs(
		String catalog,
		String schemaPattern,
		String typeNamePattern,
		int[] types)
		throws SQLException
	{
		return new MVAResultSet();
	}
	/**
	 * JDBC 2.0
	 *
	 * Gets the connection that produced this metadata object.
	 *
	 * @return An MVAConnection object.
	 * @exception SQLException This exception is required by the interface but it is never thrown.
	 */
	public Connection getConnection() throws SQLException
	{
		return connection;
	}
	/**
	 * Retrieves a description of the given attribute of the given type for a user-defined type (UDT) that is available in the given schema and catalog.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public ResultSet getAttributes(
		String catalog,
		String schemaPattern,
		String typeNamePattern,
		String attributeNamePattern)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return null;
	}
	/**
	 * Retrieves the major version number of the underlying database.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public int getDatabaseMajorVersion() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return 0;
	}
	/**
	 * Retrieves the minor version number of the underlying database.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public int getDatabaseMinorVersion() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return 0;
	}
	/**
	 * Retrieves the major JDBC version number for this driver.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public int getJDBCMajorVersion() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return 0;
	}
	/**
	 * Retrieves the minor JDBC version number for this driver.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public int getJDBCMinorVersion() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return 0;
	}
	/**
	 * Retrieves the default holdability of this ResultSet  object.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public int getResultSetHoldability() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return 0;
	}
	/**
	 * Indicates whether the SQLSTATE returned by SQLException.getSQLState  is X/Open (now known as Open Group) SQL CLI or SQL99.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public int getSQLStateType() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return 0;
	}
	/**
	 * Retrieves a description of the table hierarchies defined in a particular schema in this database.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public ResultSet getSuperTables(
		String catalog,
		String schemaPattern,
		String tableNamePattern)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return null;
	}
	/**
	 * Retrieves a description of the user-defined type (UDT) hierarchies defined in a particular schema in this database. 
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public ResultSet getSuperTypes(
		String catalog,
		String schemaPattern,
		String typeNamePattern)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return null;
	}
	/**
	 * Indicates whether updates made to a LOB are made on a copy or directly to the LOB.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public boolean locatorsUpdateCopy() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return false;
	}
	/**
	 * Retrieves whether auto-generated keys can be retrieved after a statement has been executed.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public boolean supportsGetGeneratedKeys() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return false;
	}
	/**
	 * Retrieves whether it is possible to have multiple ResultSet objects returned from a CallableStatement object simultaneously.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public boolean supportsMultipleOpenResults() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return false;
	}
	/**
	 * Retrieves whether this database supports named parameters to callable statements.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public boolean supportsNamedParameters() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return false;
	}
	/**
	 * Retrieves whether this database supports the given result set holdability.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public boolean supportsResultSetHoldability(int holdability)
		throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return false;
	}
	/**
	 * Retrieves whether this database supports savepoints.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public boolean supportsSavepoints() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return false;
	}
	/**
	* Retrieves whether this database supports statement pooling.
	* Empty method implementation for JDK1.4 compliance.
	* @exception SQLException This exception is always thrown to indicate the method is not implemented.
	*/
	public boolean supportsStatementPooling() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.General_methodNotImplemented);
		return false;
	}
}
