/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */
package com.sas.rio;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;
import java.util.MissingResourceException;
import java.sql.SQLException;
/*
*History Records:
*
* Date          Name                Remarks
* 06/25/98  M Srinivasan   - Created.
*
* 11/19/2001 Mackie, D.            Added several routines for using
*                                   vendor codes as the key.
* 05/29/2003 Mackie, D.            Changed throwLocalizedException, throwLocalizedBatchException and
*                                  throwLocalizedRIOException to take the integer argument and look up
*                                  the corresponding String key in VendorCodes.errorStrings.
*                                  Plus, changed getString(int) to do the same lookup, and added a 
*                                  getString(int, Object[]) method, similar to getString(String, Object[])
*/

public class MVALocalizedError {
    static ResourceBundle myResources = null;
    static String resourceBundle = "com.sas.rio.Resources";

    public MVALocalizedError() {
    }
    public static void display(Throwable error, String messageIndex) {
        try {
            myResources = ResourceBundle.getBundle(resourceBundle);
        }
        catch(MissingResourceException e) {
            error.printStackTrace(System.err);
            return;
        }

        String message = null;
        try {
            message = myResources.getString(messageIndex);
        }
        catch(MissingResourceException e) {
            ;
        }
        if(message == null) {
            error.printStackTrace(System.err);
            return;
        }
        Object[] messageArgs = {messageIndex};
        System.out.println(MessageFormat.format(message,messageArgs));
    }
    //public static String getStringResource(Throwable error, String messageIndex) throws MissingResourceException {
    //    String message = null;
    //    try {
    //        myResources = ResourceBundle.getBundle(resourceBundle);
    //        message = myResources.getString(messageIndex);
    //    }
    //    catch(MissingResourceException e) {
    //        message = error.getMessage();
    //    }
    //    return message;
    //}
    public static String getLocalizedError(String defaultString, Object[] args) {
        if(args == null || args.length == 0)
            return defaultString;
        for(int i=0;i<args.length;i++) {
            if(args[i] == null)
                args[i] = "";
        }
        String message = null;
        try {
            myResources = ResourceBundle.getBundle(resourceBundle);
            message = myResources.getString(args[0].toString());
        }
        catch(MissingResourceException e) {
            message = defaultString;
        }
        return MessageFormat.format(message, args);
    }

    /**
     * 11/19/2001 - damack: added the code below.
     * Added code below in response to M. Soltys' request that exceptions be
     * made more easily localizable.
     */

     /**
      * getString(int) calls thru to getString(String)
      */
    public static String getString(int k)
    {
      return getString(VendorCode.errorStrings[k], null);
    }

    /**
     * getString(String) calls thru to getString(String, Object[])
     */
    public static String getString(String key)
    {
      return getString(key, null);
    }

    /**
     * getString(String, Object[]) gets the string corresponding to the
     * key from the resource bundle, and if the Object array (args) is not
     * null, then the args are used to format the string.
     * and then said string is returned.
     */
    public static String getString(String key, Object[] args)
    {
      String message = null;
      try
        {
          myResources = ResourceBundle.getBundle(resourceBundle);
          message = myResources.getString(key);
        }
      catch (MissingResourceException mre)
        {
          // d'oh!
          mre.printStackTrace();
        }
      if (args != null && args.length > 0)
        return MessageFormat.format(message, args);

      return message;

    }
    public static String getString(int k, Object[] args)
    {
 
      return getString(VendorCode.errorStrings[k], args);
 

    }


    /**
     * throwLocalizedException(int) calls throwLocalizedException(int, Object[])
     */
    public static void throwLocalizedException(int vc) throws SQLException
    {
      throwLocalizedException(vc, null, null);
    }

    public static void throwLocalizedException(int vc, Object[] args)
                                                throws SQLException
    {
      throwLocalizedException(vc, args, null);
    }
    /**
     * rather than having the exceptions thrown in the code, this routine (or
     * one of the others below) is called to do the throwing. The idea being
     * that the caller just provides an error code, which also corresponds to
     * the string that is the exception message.
     */
    public static void throwLocalizedException(int vc, Object[] args,
                                                  Exception next)
                                                  throws SQLException
    {
      String message = null;
    String key = VendorCode.errorStrings[vc];
      try
        {
          myResources = ResourceBundle.getBundle(resourceBundle);
          message = myResources.getString(key);
        }
      catch (MissingResourceException mre)
        {
          // d'oh!
          mre.printStackTrace();
        }
      StringBuffer doh = new StringBuffer();
      String msg = null;
      if (args != null && args.length > 0)
        {
          msg = MessageFormat.format(message, args);
          doh.append(msg);
        }
      else
        doh.append(message);

      doh.append(" ");
      SQLException se = null;

      if (next != null)
        {
            if (next.getMessage() != null)
              doh.append(next.getMessage());
            else
              doh.append(next.toString());
        }
     se = new SQLException(doh.toString(), null, vc);

      if (next != null)
        {
          if (next instanceof SQLException)
            se.setNextException((SQLException)next);

        }
      throw se;

    }
    public static void throwLocalizedRIOException(int vc) throws RIOException
    {
      throwLocalizedRIOException(vc, null);
    }

    public static void throwLocalizedRIOException(int vc, Object[] args) throws RIOException
    {
      String message = null;
      String key = VendorCode.errorStrings[vc];
      try
        {
          myResources = ResourceBundle.getBundle(resourceBundle);
          message = myResources.getString(key);
        }
      catch (MissingResourceException mre)
        {
          // d'oh!
          mre.printStackTrace();
        }
      String msg = null;
      if (args != null && args.length > 0)
        msg = MessageFormat.format(message, args);
      RIOException se = null;
      if (msg != null)
          se = new RIOException(msg);
      else
          se = new RIOException(message);

      throw se;

    }

   public static void throwLocalizedBatchException(int vc) throws SQLException
    {
      throwLocalizedBatchException(vc, null);
    }

    public static void throwLocalizedBatchException(int vc, Object[] args) throws SQLException
    {
      String message = null;
      try
        {
          myResources = ResourceBundle.getBundle(resourceBundle);
          message = myResources.getString(Integer.toString(vc));
        }
      catch (MissingResourceException mre)
        {
          // d'oh!
          mre.printStackTrace();
        }
      String msg = null;
      if (args != null && args.length > 0)
        msg = MessageFormat.format(message, args);
      SQLException se = null;
      if (msg != null)
          se = new SQLException(msg, null, vc);
      else
          se = new SQLException(message, null, vc);

      throw se;

    }
}
