/* $Id$ */
/*
 * Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513 Unpublished- All
 * Rights Reserved.
 */
package com.sas.rio;

//import com.sas.iom.SAS.IDataSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.sql.DatabaseMetaData;
import java.util.Properties;
//
//History Records:
//
// Date Name Remarks
// 11/17/98 M Srinivasan - Created.
// 07/27/99 M Srinivasan - S0061725 :getTables() will return atleast 1 row due
// to count(*)
//                              in the query. So, returned false in next() if the
//                              7th column is 0.
// 02/06/01 R Gonsalves - S0101924: Fix NaN check for INDEX.
// 02/08/01 B Beers - S0101924: Fix NaN check for PRIMARYKEYS.
// 07Feb2003 damack - S0183030: fix issue with extra columns showing up.
// 14Jan2005 damack - S0278967: added a properties object to ctor, in order to get coercenumerictypes
//                              value.
//
public class MVAModifiedResultSet extends MVAResultSet
{
    static final int NONE = 0;

    static final int COLUMNS = 1;

    static final int BESTROW = 2;

    static final int PRIMARYKEYS = 3;

    static final int INDEX = 4;

    static final int PASS = 5;

    static final int TABLES = 6;

    static final int TABLETYPES = 7;

    int specialHandling = MVAModifiedResultSet.NONE;

    short count = 1;

    Object[][] trow = null;

    String m_errorMsg = null;
    boolean m_coerceNumericTypes = true;
    /**
     * Constructs an MVAModifiedResultSet object.
     * 
     * @param sasDataSet
     *            A SASDataSetInterface object.
     * @param specHandling
     *            Indicates whether any special manipulation of columns is
     *            required.
     * @exception SQLException
     *                This exception is thrown if input is null.
     */
    public MVAModifiedResultSet(/** IDataSet sasDataSet */
    int hashCode, XUtil xUtil, int specHandling) throws SQLException
    {
        super(hashCode, xUtil);
        specialHandling = specHandling;
        switch (specialHandling)
        {
            case COLUMNS:
                m_columnCount = 18;
                // m_rowCache.setColumnCount(m_columnCount);
                break;
            case PASS:
                m_columnCount = 18;
                //m_rowCache.setColumnCount(m_columnCount);
                break;
            case BESTROW:
                m_columnCount = 8;
                //m_rowCache.setColumnCount(m_columnCount);
                break;
            case PRIMARYKEYS:
                m_columnCount = 6;
                //m_rowCache.setColumnCount(m_columnCount);
                break;
            case INDEX:
                m_columnCount = 13;
                //m_rowCache.setColumnCount(m_columnCount);
                break;
            case TABLES:
                m_columnCount = 6;
                //m_rowCache.setColumnCount(m_columnCount);
                break;
            case TABLETYPES:
                m_columnCount = 1;
                //m_rowCache.setColumnCount(m_columnCount);
                break;
            case NONE:
            default:
                break;
        }
    }
    public MVAModifiedResultSet(
            int hashCode, XUtil xUtil, int specHandling, Properties p) throws SQLException
            {
                super(hashCode, xUtil);
                if (p != null)
                {
                    String ntc = (String)p.get("coercenumerictypes");
                    if (ntc != null && ntc.length() > 0 && ntc.equalsIgnoreCase("false"))
                        m_coerceNumericTypes = false;
                }
                specialHandling = specHandling;
                switch (specialHandling)
                {
                    case COLUMNS:
                        m_columnCount = 18;
                        // m_rowCache.setColumnCount(m_columnCount);
                        break;
                    case PASS:
                        m_columnCount = 18;
                        //m_rowCache.setColumnCount(m_columnCount);
                        break;
                    case BESTROW:
                        m_columnCount = 8;
                        //m_rowCache.setColumnCount(m_columnCount);
                        break;
                    case PRIMARYKEYS:
                        m_columnCount = 6;
                        //m_rowCache.setColumnCount(m_columnCount);
                        break;
                    case INDEX:
                        m_columnCount = 13;
                        //m_rowCache.setColumnCount(m_columnCount);
                        break;
                    case TABLES:
                        m_columnCount = 6;
                        //m_rowCache.setColumnCount(m_columnCount);
                        break;
                    case TABLETYPES:
                        m_columnCount = 1;
                        //m_rowCache.setColumnCount(m_columnCount);
                        break;
                    case NONE:
                    default:
                        break;
                }
            }
    /**
     * Constructs an MVAModifiedResultSet object.
     * 
     * @param sasDataSet
     *            A SASDataSetInterface object.
     * @param types
     *            An array specifying the SQL Types of each column
     * @param specHandling
     *            Indicates whether any special manipulation of columns is
     *            required.
     * @exception SQLException
     *                This exception is thrown if input is null.
     */
    public MVAModifiedResultSet(int hashCode, XUtil xUtil, int types[],
            int specHandling) throws SQLException
    {
        this(hashCode, xUtil, specHandling);
        int b = m_columnCount;
        if (specHandling == BESTROW)
            b++;
        else if (specHandling == COLUMNS)
            b = types.length;
        for (int i = 0; i < b; i++)
        {
            m_rowCache.setColumnType(i + 1, types[i]);
        }
    }
    public MVAModifiedResultSet(int hashCode, XUtil xUtil, int types[],
            int specHandling, Properties p) throws SQLException
    {
        this(hashCode, xUtil, specHandling, p);
        int b = m_columnCount;
        if (specHandling == BESTROW)
            b++;
        else if (specHandling == COLUMNS)
            b = types.length;
        for (int i = 0; i < b; i++)
        {
            m_rowCache.setColumnType(i + 1, types[i]);
        }
    }

    /**
     * Constructs an MVAModifiedResultSet object.
     * 
     * @param sasDataSet
     *            A SASDataSetInterface object.
     * @param types
     *            An array specifying the SQL Types of each column
     * @param trow
     *            An object array of artificially constructed values
     * @param specHandling
     *            Indicates whether any special manipulation of columns is
     *            required.
     * @exception SQLException
     *                This exception is thrown if input is null.
     */
    public MVAModifiedResultSet(int hashCode, XUtil xUtil, Object[][] trow,
            int specHandling) throws SQLException
    {
        this(hashCode, xUtil, specHandling);
        this.trow = trow;
    }

    /**
     * Proceeds to the next row in the ResultSet.
     * 
     * @return True if the next row is valid and False if not valid.
     * @exception SQLException
     *                This exception is thrown if an exception is detected.
     */
    public synchronized boolean next() throws SQLException
    {
        /**
         * if(maxReached == true) { return false; }
         */
        if ((m_statement != null) && (m_statement.getMaxRows() != 0)
                && (m_rowCache.getActual() == m_statement.getMaxRows()))
        {
            //maxReached = true;
            close();
            return false;
        }
        if (specialHandling == MVAModifiedResultSet.PASS)
        {
            return super.next();
        }

        //System.out.println("flags : " + flags + "rows : " + rows + " blocknum
        // : " + blocknum);
        try
        {
            /**
             * yet another defect fix, this time S0138285.
             */
            /**
             * if (specialHandling == TABLETYPES) { boolean cr =
             * Mitchfetch(flags, bindKey, 1, rowsOffset); if (!cr) { maxReached =
             * true; m_eof = true; maxRows = m_currentRowNumber; close(); return
             * false; } } else { currentRow =
             * fetch(flags,bindKey,rows,rowsOffset); if(currentRow == null) {
             * maxReached = true; m_eof = true; maxRows = m_currentRowNumber;
             * close(); return false; }
             *  }
             */
            //currentRow = fetch(flags,bindKey,rows,rowsOffset);
            //if(currentRow == null) {
            /**
             * after D. Burriss reported a problem with this version of next(),
             * I changed the fetch from the above commented call of fetch to the
             * below call to mitch fetch. (currentRow gets set in Mitchfetch)
             * 1/17/2002 (damack) S0121549
             */
            /**
             * m_bof = false; m_currentRowNumber++;
             */
            if (!m_rowCache.next())
                return false;
        }
        catch (Exception se)
        {
            MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAModifiedResultSet_nextError, null, se);

        }
        String s = null;
        switch (specialHandling)
        {
            case MVAModifiedResultSet.TABLETYPES:
                s = ((String) m_rowCache.getData(1)).trim();

                //		currentRow[0] = currentRow[0].toString().trim();
                if (s.toUpperCase().equals("DATA"))
                {
                    m_rowCache.setData(1, "TABLE");
                }
                break;
            case MVAModifiedResultSet.TABLES:
                Double d = (Double) m_rowCache.getData(7);

                if (d.intValue() == 0)
                {
                    try
                    {
                        close();
                    }
                    catch (Exception se)
                    {
                        MVALocalizedError.throwLocalizedException(
                                VendorCode.MVAModifiedResultSet_nextError,
                                null, se);
                    }
                    return false;
                }
                s = ((String) m_rowCache.getData(4)).trim();
                //                currentRow[3] = currentRow[3].toString().trim();
                if (s.toUpperCase().equals("DATA"))
                {
                    m_rowCache.setData(4, "TABLE");
                }
                String p1 = null;
                String p2 = null;
                String p3 = null;
                String priv = null;
                String protect = ((String) m_rowCache.getData(6)).trim()
                        .toUpperCase();
                if (protect.equals("---") || protect.equals(""))
                {
                    m_rowCache.setData(6, null);
                }
                else
                {
                    if (protect.substring(0, 1).equals("R"))
                    {
                        p1 = "READ";
                    }
                    if (protect.substring(1, 2).equals("W"))
                    {
                        p2 = "WRITE";
                    }
                    if (protect.substring(2, 3).equals("A"))
                    {
                        p3 = "ALTER";
                    }
                    if (p1 != null)
                    {
                        priv = p1;
                    }
                    if (p2 != null)
                    {
                        if (priv != null)
                            priv = priv + "," + p2;
                        else
                            priv = p2;
                    }
                    if (p3 != null)
                    {
                        if (priv != null)
                            priv = priv + "," + p3;
                        else
                            priv = p3;
                    }
                    m_rowCache.setData(6, priv);
                }
                break;
            case MVAModifiedResultSet.COLUMNS:
                String sastype = (String) m_rowCache.getData(20);
                if (sastype.equals("char"))
                {
                    m_rowCache.setData(5, new Double(Types.VARCHAR));
                }
                else
                {
                    String tempFormat = (String) m_rowCache.getData(19);
                    int subtype = Sqpformat.intepretFormat(tempFormat);
                    /*
                     * S0278967: if type coercion is on, then return the column type of integer
                     */
                    if (m_coerceNumericTypes)
                    {
                    switch (subtype)
                    {
                        case MVAResultSet.INTEGER:
                            m_rowCache.setData(5, new Double(Types.INTEGER));
                            break;
                        case MVAResultSet.DATETIME:
                            m_rowCache.setData(5, new Double(Types.TIMESTAMP));
                            break;
                        case MVAResultSet.DATE:
                            m_rowCache.setData(5, new Double(Types.DATE));
                            break;
                        case MVAResultSet.TIME:
                            m_rowCache.setData(5, new Double(Types.TIME));
                            break;
                        default:
                            m_rowCache.setData(5, new Double(Types.DOUBLE));
                            break;
                    }
                    }
                    else
                    {
                        /*
                         * S0278967: if type coercion is off, then return double for numeric columns
                         */
                        switch (subtype)
                        {
                            case MVAResultSet.DATETIME:
                                m_rowCache.setData(5, new Double(Types.TIMESTAMP));
                                break;
                            case MVAResultSet.DATE:
                                m_rowCache.setData(5, new Double(Types.DATE));
                                break;
                            case MVAResultSet.TIME:
                                m_rowCache.setData(5, new Double(Types.TIME));
                                break;
                            default:
                                m_rowCache.setData(5, new Double(Types.DOUBLE));
                                break;
                        }   
                    }
                }
                //currentRow[5] =
                // getSQLTypeString((int)(Double.valueOf(currentRow[4].toString()).doubleValue()));
                m_rowCache.setData(8, null);

                if ((int) (Double.valueOf(m_rowCache.getData(5).toString())
                        .doubleValue()) != Types.VARCHAR)
                {
                    m_rowCache.setData(11, new Double(
                            DatabaseMetaData.columnNullable));
                    m_rowCache.setData(18, "YES");
                }
                else
                {
                    m_rowCache.setData(11, new Double(
                            DatabaseMetaData.columnNoNulls));
                    m_rowCache.setData(18, "NO");
                }

                m_rowCache.setData(13, null);
                m_rowCache.setData(14, null);
                m_rowCache.setData(15, null);

                break;
            case MVAModifiedResultSet.BESTROW:
                sastype = (String) m_rowCache.getData(4);
                if (sastype.equals("char"))
                {
                    m_rowCache.setData(3, new Double(Types.VARCHAR));
                }
                else
                {
                    String tempFormat = (String) m_rowCache.getData(9);
                    int subtype = Sqpformat.intepretFormat(tempFormat);
                    /*
                     * S0278967: if type coercion is on, then return INTEGER as appropriate
                     */
                    if (m_coerceNumericTypes)
                    {
                    switch (subtype)
                    {
                        case MVAResultSet.INTEGER:
                            m_rowCache.setData(3, new Double(Types.INTEGER));
                            break;
                        case MVAResultSet.DATETIME:
                            m_rowCache.setData(3, new Double(Types.TIMESTAMP));
                            break;
                        case MVAResultSet.DATE:
                            m_rowCache.setData(3, new Double(Types.DATE));
                            break;
                        case MVAResultSet.TIME:
                            m_rowCache.setData(3, new Double(Types.TIME));
                            break;
                        default:
                            m_rowCache.setData(3, new Double(Types.DOUBLE));
                            break;
                    }
                    }
                    else
                    {
                        /*
                         * S0278967: if tyep coercion is off, then return DOUBLE for numerics.
                         */
                        switch (subtype)
                        {
                            case MVAResultSet.DATETIME:
                                m_rowCache.setData(3, new Double(Types.TIMESTAMP));
                                break;
                            case MVAResultSet.DATE:
                                m_rowCache.setData(3, new Double(Types.DATE));
                                break;
                            case MVAResultSet.TIME:
                                m_rowCache.setData(3, new Double(Types.TIME));
                                break;
                            default:
                                m_rowCache.setData(3, new Double(Types.DOUBLE));
                                break;
                        }   
                    }
                }
                m_rowCache.setData(4, getSQLTypeString((int) (Double
                        .valueOf(m_rowCache.getData(3).toString())
                        .doubleValue())));
                break;
            case MVAModifiedResultSet.PRIMARYKEYS:
                Double w = (Double) m_rowCache.getData(5);
                if (w.isNaN())
                {
                    m_rowCache.setData(5, new Double(0));
                }
                else
                {
                    m_rowCache.setData(5, new Double(count++));
                }
                break;
            case MVAModifiedResultSet.INDEX:
                if (((Double) m_rowCache.getData(8)).isNaN())
                {
                    m_rowCache.setData(8, new Double(0));
                }
                else
                {
                    m_rowCache.setData(8, new Double(count++));
                }
                break;
            case MVAModifiedResultSet.NONE:
            default:
                break;
        }
        return true;
    }

    /**
     * Gets an MVAResultSetMetaData object describing this object.
     * 
     * @return An MVAResultSetMetaData object
     * @exception SQLException
     *                This exception is thrown if an error condition is
     *                detected.
     */
    public ResultSetMetaData getMetaData() throws SQLException
    {
        ResultSetMetaData rsmetadata;
        // S0183030: pass a column count to the metadata ctor, so we don't show
        //           hidden columns.
        rsmetadata = new MVAResultSetMetaData(this, m_columnCount);
        return rsmetadata;
    }

    /**
     * Gets the SQLType String.
     */
    String getSQLTypeString(int type)
    {
        switch (type)
        {
            case (Types.BIT):
                return new String("BIT");
            case (Types.TINYINT):
                return new String("TINYINT");
            case (Types.SMALLINT):
                return new String("SMALLINT");
            case (Types.INTEGER):
                return new String("INTEGER");
            case (Types.BIGINT):
                return new String("BIGINT");
            case (Types.FLOAT):
                return new String("FLOAT");
            case (Types.REAL):
                return new String("REAL");
            case (Types.DOUBLE):
                return new String("DOUBLE");
            case (Types.NUMERIC):
                return new String("NUMERIC");
            case (Types.DECIMAL):
                return new String("DECIMAL");
            case (Types.CHAR):
                return new String("CHAR");
            case (Types.VARCHAR):
                return new String("VARCHAR");
            case (Types.LONGVARCHAR):
                return new String("LONGVARCHAR");
            case (Types.DATE):
                return new String("DATE");
            case (Types.TIME):
                return new String("TIME");
            case (Types.TIMESTAMP):
                return new String("TIMESTAMP");
            case (Types.BINARY):
                return new String("BINARY");
            case (Types.VARBINARY):
                return new String("VARBINARY");
            case (Types.LONGVARBINARY):
                return new String("LONGVARBINARY");
            case (Types.NULL):
                return new String("NULL");
            default:
                return new String("");
        }
    }

}