/* $Id$ */
/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */
package com.sas.rio;

import java.math.BigDecimal;
import java.sql.ParameterMetaData;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.sql.SQLException;
import java.sql.DataTruncation;
import java.sql.Ref;
import java.sql.Clob;
import java.sql.Blob;
import java.sql.Array;
import java.util.Calendar;
import java.net.URL;
//
//History Records:
//
// Date          Name                Remarks
// 01/29/99  M Srinivasan   - Created.
//
// 26Apr2004 damack - S0189758 - added javadoc comments
// 12Apr2005 damack - S0287684 - fixed problem with String setNull (setNull was using an
//                               empty string to represent a null, and it needs to use an
//                               empty string wrapped in single quotes (ie, '')
  public class MVAPreparedStatement extends MVAStatement implements java.sql.PreparedStatement
  {
    private String sqlquery;
    private String subsquery;
    private String[] parameters;
    private boolean nullValue[];
    private int parmCount;
    private boolean retval;
    private int vendorCode;
    private String messageIndex;
      String m_errorMsg = null;
    /*
     * Constructs a MVAPreparedStatement object.
     *
     * @param none used only by MVACallableStatement
     */
    public MVAPreparedStatement()
    {
    }

    /*
     * Constructs a MVAPreparedStatement object.
     *
	 * @param inconnection MVAConnection object
     * @param sql input sql statement
     * @param props input connection properties.
     */
    public MVAPreparedStatement(MVAConnection inconnection, String sql, java.util.Properties props)
    {
      super(inconnection, props);
      parmCount = 0;
      int b = 0;
      int inquotes = -1;

      if ((sql == null) || (sql.length() == 0))
        sqlquery = "";
      else
        sqlquery = sql;

      int length = sql.length();
      while (b < length) {
        if ((sql.charAt(b) == '\'') || (sql.charAt(b) == '\"')) {
          if (inquotes == -1)
            inquotes = sql.charAt(b);
          else if (inquotes == sql.charAt(b))
            inquotes = -1;
        }
        if ((sql.charAt(b) == '?') && (inquotes == -1) ) {
          parmCount++;
        }
        b++;
      }

      parameters = new String[parmCount];
      nullValue = new boolean[parmCount];
      for (int i = 0; i < parmCount; i++)
        nullValue[i] = false;
    }

    public MVAPreparedStatement(MVAConnection inconnection, String sql)
    {
      this(inconnection, sql, null);
    }
    /**
     * Executes a prepared SQL query and returns the ResultSet.
     *
	 * @return An MVAResultSet object that contains the data that is produced by the query.
     * @exception SQLException This exception is thrown if an error condition is detected.
     */
    public synchronized ResultSet executeQuery() throws SQLException
    {
      substituteParameters();
      super.executeQuery(subsquery);
      return super.snresultset;
    }

    /**
     * Executes an SQL INSERT, UPDATE or DELETE statement.
     *
	 * @return An int that contains the number of rows affected.
     * @exception SQLException This exception is thrown if an error condition is detected.
     */
    public synchronized int executeUpdate() throws SQLException
    {
      substituteParameters();
      return super.executeUpdate(subsquery);
    }

    /**
     * Sets a parameter to SQL NULL.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param sqlType An SQL type code that is defined by java.sql.Types.
     * @exception SQLException This exception is thrown if a database error is detected or if
     * the sqlType is not FLOAT, DOUBLE, INTEGER, TIME, DATE or TIMESTAMP. If the sqlType is
     * CHAR, VARCHAR or LONGVARCHAR, an exception will not be thrown, and the value in the database
     * will be set to a blank.
     * SAS software does not allow other types to be NULL.
     */
    public synchronized void setNull(int parameterIndex, int sqlType) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if ((sqlType == Types.FLOAT) || (sqlType == Types.DOUBLE) ||
          (sqlType == Types.INTEGER) || (sqlType == Types.TIME) ||
          (sqlType == Types.DATE) || (sqlType == Types.TIMESTAMP)) {

        String tempString = ".";
        parameters[parameterIndex -1] = tempString;
      } 
      /* 
       * S0287684: fixed setNull by wrapping the blank in single quotes, so that an empty string
       * is properly put in the sql going to sas.  Also, added the Types.CHAR to the sql types
       * covered by this case, since it seems wrong to allow the other two char types and not
       * CHAR.
       * Also, modified the javadoc above to include a mention of CHAR in the part about
       * nulls getting turned into blanks.
       */ 
      else if (sqlType == Types.VARCHAR || sqlType == Types.LONGVARCHAR || sqlType == Types.CHAR)
        {
          parameters[parameterIndex - 1] =  " '' ";
        }
        else
       {
	  MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_setNullError );

      }
      nullValue[parameterIndex -1] = true;
    }

    /**
     * Sets a parameter to a Java Boolean value. This method is not supported.
     *
     * @exception SQLException This exception is always thrown because SAS software
     * does not support the SQL type BIT.
     */
    public synchronized void setBoolean(int parameterIndex, boolean x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeBITUnsupported );

    }

    /**
     * Sets a parameter to a Java byte value. This method is not supported.
     *
	 * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type TINYINT.
     */
    public synchronized void setByte(int parameterIndex, byte x) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeTINYINTUnsupported );

    }

    /**
     * Sets a parameter to a Java short value. This method is not supported.
     *
	 * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type SMALLINT.
     */
    public synchronized void setShort(int parameterIndex, short x) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeSMALLINTUnsupported );

    }

    /**
     * Sets a parameter to a Java int value.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x An int value.
     * @exception SQLException This exception is thrown if parameterIndex is not valid.
     */
    public synchronized void setInt(int parameterIndex, int x) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	  MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      Integer temp = new Integer(x);
      String tempString = temp.toString();
      parameters[parameterIndex -1] = tempString;
   }

    /**
     * Sets a parameter to a Java long value. This method is not supported.
     *
	 * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type BIGINT.
     */
    public synchronized void setLong(int parameterIndex, long x) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeBigIntUnsupported );

    }

    /**
     * Sets a parameter to a Java float value.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x float value.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public synchronized void setFloat(int parameterIndex, float x) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
        MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == Float.NaN)
        setNull(parameterIndex, Types.FLOAT);
      else
        {
          Float temp = new Float(x);
          String tempString = temp.toString();
          parameters[parameterIndex -1] = tempString;
        }
    }

    /**
     * Sets a parameter to a Java double value.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x double value.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public synchronized void setDouble(int parameterIndex, double x) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
       MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == Double.NaN)
        setNull(parameterIndex, Types.DOUBLE);
      else
        {
          Double temp = new Double(x);
          String tempString = temp.toString();
          parameters[parameterIndex -1] = tempString;
        }
    }

    /**
     * Sets a parameter to a Java BigDecimal value. This method is not supported.
     *
     * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type Numeric.
     */
    public synchronized void setBigDecimal(int parameterIndex, BigDecimal x) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeNumericUnsupported );

    }

    /**
     * Sets a parameter to a Java string value.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x String value.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public synchronized void setString(int parameterIndex, String x) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == null)
      {
        setNull(parameterIndex, Types.LONGVARCHAR);
        return;
      }
      /*
       * per S0106249, we now wrap the passed in string in single quotes.
       * in order to avoid breaking S0119330, we'll have to escape the
       * single quote.
       */
       else
        {
          if (x.indexOf("'") > -1)
            {
              boolean done = false;
              String esc = x;
              int index = 0;
              while (!done)
                {

                  String y = esc.substring(0, esc.indexOf("'", index));
                  String z = esc.substring(esc.indexOf("'", index), esc.length());
                  index = esc.indexOf("'", index);
                  esc = y + "'" + z;

                  index+=2;
                  if (esc.indexOf("'", index) < 0) done = true;
                }

              parameters[parameterIndex - 1] = new String("'" + esc + "'");
            }
          else
            parameters[parameterIndex - 1] = new String("'" + x + "'");

        }

    }

    /**
     * Sets a parameter to a Java byte array value. This method is not supported.
     *
	 * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type BINARY.
     */
    public synchronized void setBytes(int parameterIndex, byte x[]) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeBINARYUnsupported );

    }

    /**
     * Sets a parameter to a Java java.sql.Date value.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x java.sql.Date value.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public synchronized void setDate(int parameterIndex, java.sql.Date x) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	  MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == null)
      {
        setNull(parameterIndex, Types.DATE);
        return;
      }
      else
      {
        Calendar cal = Calendar.getInstance();
        cal.setTime(x);
        int day = cal.get(Calendar.DAY_OF_MONTH);
        int mon = cal.get(Calendar.MONTH) + 1;
        int year = cal.get(Calendar.YEAR);
        String tempString = new String("'" + day + MVAStatement.months[mon] + year + "'d ");
        parameters[parameterIndex -1] = tempString;
      }
    }

    /**
     * Sets a parameter to a Java java.sql.Time value.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x java.sql.Time value.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
  public synchronized void setTime(int parameterIndex, java.sql.Time x) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == null)
      {
        setNull(parameterIndex, Types.TIME);
        return;
      }
      else
      {
        String tempString = new String("'" + x.toString() + "'t ");
        parameters[parameterIndex -1] = tempString;
      }
    }

    /**
     * Sets a parameter to a Java java.sql.Timestamp value.
     *
	 * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x java.sql.Timestamp value.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public synchronized void setTimestamp(int parameterIndex, java.sql.Timestamp x) throws SQLException
    {
      String nanostring = null;
      String zeroes = "000000000";
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == null)
      {
        setNull(parameterIndex, Types.TIMESTAMP);
        return;
      }
      else
      {
          Calendar cal = Calendar.getInstance();
          cal.setTime(x);
          int day = cal.get(Calendar.DAY_OF_MONTH);
          int mon = cal.get(Calendar.MONTH) + 1;
          int year = cal.get(Calendar.YEAR);
          int hour = cal.get(Calendar.HOUR_OF_DAY);
          int minute = cal.get(Calendar.MINUTE);
          int seconds = cal.get(Calendar.SECOND);
          int nanos = x.getNanos();
          if (nanos == 0)
             nanostring = "0";
          else {
             Integer n = new Integer(nanos);
             nanostring = n.toString();
             int len = nanostring.length();
             nanostring = new String(zeroes.substring(0,9-len) + nanostring);
          }
          String tempString = new String("'" + day + MVAStatement.months[mon] + year + ":" +
                                       hour + ":" + minute + ":" + seconds + "." + nanostring + "'dt ");

              parameters[parameterIndex -1] = tempString;
      }
    }

    /**
     * Sets a parameter to a Java java.io.InputStream.
     *
	 * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type LONGVARCHAR.
     */
    public synchronized void setAsciiStream(int parameterIndex, java.io.InputStream x, int length) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeLONGVARCHARUnsupported );

//TODO   temporarily commented out until compliance tests are received
//      if ((parameterIndex > parmCount) || (parameterIndex < 1))
//          throw new SQLException(VendorCode.MVAPreparedStatement exception: setAsciiStream. Invalid parameterIndex." );

//      byte input[] = new byte[length];
//      try {
//        x.read(input, 0, length);
//      } catch (IOException ioe) {
//        throw new SQLException(VendorCode.SQL Exception: MVAPreparedStatement.setAsciiStream: " + ioe.getMessage() );
//      }
//      String tempString = new String(input,0);
//      parameters.setElementAt(tempString, parameterIndex);
    }

    /**
     * Sets a parameter to a Java java.io.InputStream. This method is not supported.
     *
	 * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type LONGVARCHAR.
     * @deprecated
     */
    public synchronized void setUnicodeStream(int parameterIndex, java.io.InputStream x, int length) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeLONGVARCHARUnsupported );


//   TODO temporarily commented out until compliance tests are received
//      if ((parameterIndex > parmCount) || (parameterIndex < 1))
//        throw new SQLException(VendorCode.SQL Exception: MVAPreparedStatement. setUnicodeStream: Invalid parameterIndex." );

//      byte input[] = new byte[length];
//      try {
//        x.read(input, 0, length);
//      } catch (IOException ioe) {
//        throw new SQLException(VendorCode.SQL Exception: MVAPreparedStatement.setInputStream: " + ioe.getMessage() );
//      }
//      String tempString = new String(input,0);
//      parameters.setElementAt(tempString, parameterIndex);
    }

    /**
     * Sets a parameter to a Java java.io.InputStream. This method is not supported.
     *
	 * @exception SQLException This exception is always thrown because SAS software
     * does not support SQL type LONGVARBINARY.
     */
    public synchronized void setBinaryStream(int parameterIndex, java.io.InputStream x, int length) throws SQLException
    {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_sqlTypeLONGVARBINARYUnsupported );


//      TODO temporarily commented out until compliance tests are received
//      if ((parameterIndex > parmCount) || (parameterIndex < 1))
//        throw new SQLException(VendorCode.SQL Exception: MVAPreparedStatement.setBinaryStream: Invalid parameterIndex." );

//      byte input[] = new byte[length];
//      try {
//        x.read(input, 0, length);
//      } catch (IOException ioe) {
//        throw new SQLException(VendorCode.ySQL Exception: MVAPreparedStatement.setBinaryStream: " + ioe.getMessage() );
//      }
//      String tempString = new String(input,0);
//      parameters.setElementAt(tempString, parameterIndex);
  }

    /**
     * Clears all the parameters.
     *
	 * @exception SQLException This exception is required by the interface, but it is never thrown.
     */
    public synchronized void clearParameters() throws SQLException
    {
      parameters = new String[parmCount];
      for (int i = 0; i < parmCount; i++)
        nullValue[i] = false;
    }

    /**
     * Sets a parameter to the input type.
     *
     * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x An input object.
     * @param sqlType The target SQL type for the object.
     * @param scale This parameter is ignored. SAS software does not support Decimal or Numeric  data
     * types.
     * @exception SQLException This exception may be thrown.
     */
    public synchronized void setObject(int parameterIndex, Object x, int sqlType, int scale) throws SQLException
    {
        setObject(parameterIndex, x, sqlType);
    }

    /**
     * Sets a parameter to the default type.
     *
     * @param parameterIndex The first parameter is 1, the second parameter is 2, and so on.
     * @param x An input object.
     * @exception SQLException This exception may be thrown.
     */
  public synchronized void setObject(int parameterIndex, Object x) throws SQLException
  {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == null) {
         setObject(parameterIndex, x, Types.LONGVARCHAR);
         return;
      }

      if (x instanceof java.sql.Date) {
          setDate(parameterIndex,(java.sql.Date)x);
          return;
      }

      if (x instanceof java.sql.Time) {
          setTime(parameterIndex,(java.sql.Time)x);
          return;
      }

      if (x instanceof java.sql.Timestamp) {
          setTimestamp(parameterIndex,(java.sql.Timestamp)x);
          return;
      }

      if ((x instanceof java.lang.String) || (x instanceof java.lang.Character)) {
          setString(parameterIndex,x.toString());
          return;
      }

      if ((x instanceof java.lang.Integer) || (x instanceof java.lang.Float) || (x instanceof java.lang.Double)) {
         String tempString = x.toString();
         parameters[parameterIndex -1] = tempString;
         return;
      }
      MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_unknownObjectType );

  }

    /**
     * Sets a parameter to the input type.
     *
     * @param parameterIndex The first parameter is 1, the second is 2, and so on.
     * @param x An input object.
     * @param sqlType The target SQL type for the object.
     * @exception SQLException This exception may be thrown.
     */
    public synchronized void setObject(int parameterIndex, Object x, int sqlType) throws SQLException
    {
      if ((parameterIndex > parmCount) || (parameterIndex < 1)) {
	MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_invalidParamIndexError );

      }
      if (x == null) {
         setNull(parameterIndex, sqlType);
         return;
      }


      switch(sqlType) {

        case Types.INTEGER:
        case Types.FLOAT:
        case Types.DOUBLE:
          if ((x instanceof java.lang.Integer) || (x instanceof java.lang.Float) || (x instanceof java.lang.Double)
             || (x instanceof java.lang.String) || (x instanceof java.lang.Character)) {
             String tempString = x.toString();
             parameters[parameterIndex -1] = tempString;
          } else {
	      MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_unknownObjectType );

          }
          break;
        case Types.CHAR:
        case Types.VARCHAR:
        case Types.LONGVARCHAR:
          if ((x instanceof java.lang.Integer) || (x instanceof java.lang.Float) || (x instanceof java.lang.Double)
             || (x instanceof java.lang.String) || (x instanceof java.lang.Character)
             || (x instanceof java.sql.Date) || (x instanceof java.sql.Time) || (x instanceof java.sql.Timestamp)) {
             setString(parameterIndex, x.toString());
          } else {
	       MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_unknownObjectType );

          }
          break;
        case Types.DATE:
          java.sql.Date tDate = null;
          if (x instanceof java.sql.Timestamp) {
             String s = x.toString();
             tDate = java.sql.Date.valueOf(s.substring(0,10));
          } else if (x instanceof java.lang.String) {
             tDate = java.sql.Date.valueOf(x.toString());
          } else if (x instanceof java.sql.Date) {
             tDate = (java.sql.Date)x;
          } else {
	       MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_unknownObjectType );

          }
          setDate(parameterIndex, tDate);
          break;
        case Types.TIME:
          java.sql.Time tTime = null;
          if (x instanceof java.sql.Timestamp) {
             String s = x.toString();
             tTime = java.sql.Time.valueOf(s.substring(11,19));
          } else if (x instanceof java.lang.String) {
             tTime = java.sql.Time.valueOf(x.toString());
          } else if (x instanceof java.sql.Time) {
             tTime = (java.sql.Time)x;
          } else {
	      MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_unknownObjectType );

          }
          setTime(parameterIndex, tTime);
          break;
        case Types.TIMESTAMP:
          java.sql.Timestamp tStamp = null;
          if (x instanceof java.sql.Timestamp) {
             tStamp = (java.sql.Timestamp)x;
          } else if (x instanceof java.lang.String) {
             tStamp = java.sql.Timestamp.valueOf(x.toString());
          } else if (x instanceof java.sql.Date) {
             long lTime = ((java.sql.Date)x).getTime();
             tStamp = new java.sql.Timestamp(lTime);
          } else {
	       MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_unknownObjectType );

          }
          setTimestamp(parameterIndex, tStamp);
          break;
        case Types.BINARY:
        case Types.VARBINARY:
        case Types.LONGVARBINARY:
        case Types.BIT:
        case Types.TINYINT:
        case Types.SMALLINT:
        case Types.BIGINT:
        case Types.REAL:
        case Types.NUMERIC:
        case Types.DECIMAL:
        default:
	    MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_unknownObjectType );

      }
      return;
    }

    /**
     * Executes a query.
     *
	 * @return True if the result is a ResultSet; False if it is an integer.
     * @exception SQLException This exception may be thrown.
     * @see MVAStatement#execute
     */
    public synchronized boolean execute() throws SQLException
    {
      substituteParameters();
      return super.execute(subsquery);
    }

    //--------------------------JDBC 2.0-----------------------------

    /**
     * JDBC 2.0
     *
     * Adds a set of parameters to the batch.
     * This method is not yet supported.
     *
     * @exception SQLException This exception is always thrown.
     * @see MVAStatement#addBatch
     */
    public void addBatch() throws SQLException {
        MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

    }

    /**
     * JDBC 2.0
     *
     * Sets a parameter to java.io.Reader object.
     * This method is not yet supported.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x The java reader object which contains the UNICODE data.
     * @param length The number of characters in the stream.
     * @exception SQLException This exception is always thrown
     */
    public void setCharacterStream(int parameterIndex,
       			    java.io.Reader reader,
			        int length) throws SQLException {
        MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

	}

    /**
     * JDBC 2.0
     *
     * Sets a parameter to a REF(&lt;structured-type&gt;) object.
     * This method is not supported.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x An object representing data of an SQL REF Type .
     * @exception SQLException This exception is always thrown as SAS software does not support REF SQL type.
     */
    public void setRef (int parameterIndex, Ref x) throws SQLException {
        MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

    }

    /**
     * JDBC 2.0
     *
     * Sets a parameter to a BLOB object.
     * This method is not supported.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x An object representing a BLOB.
     * @exception SQLException This exception is always thrown as SAS software does not support BLOB SQL type.
     */
    public void setBlob (int parameterIndex, Blob x) throws SQLException {
        MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

	}

    /**
     * JDBC 2.0
     *
     * Sets a parameter to a CLOB object.
     * This method is not supported.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x An object representing a CLOB.
     * @exception SQLException This exception is always thrown as SAS software does not support CLOB SQL type.
     */
    public void setClob (int parameterIndex, Clob x) throws SQLException {
        MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

	}

    /**
     * JDBC 2.0
     *
     * Sets a parameter to an Array object.
     * This method is not supported.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x An object representing an SQL array.
     * @exception SQLException This exception is always thrown as SAS software does not support Array SQL type.
     */
    public void setArray (int parameterIndex, Array x) throws SQLException {
        MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

    }

    /**
     * JDBC 2.0
     *
     * Gets the number, types and properties of a ResultSet's columns as a ResultSetMetaData object.
	 *
     * @return The description of a ResultSet's columns as a ResultSetMetaData object.
     * @exception SQLException This exception is thrown if a database error occurs or if the result set is NULL.
     */
    public ResultSetMetaData getMetaData() throws SQLException {
        // might need a try/catch block here...
        substituteParameters();

        retval = super.execute(subsquery);

        ResultSet rs = getResultSet();
        try {
            ResultSetMetaData rsmd = rs.getMetaData();
            return rsmd;
        }
        catch(SQLException e) {
	     MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_getMetaDataError , null, e);

        }
        return null;
	}

    /**
     * JDBC 2.0
     *
     * Sets a parameter to a java.sql.Date value.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x The parameter value, a java.sql.Date object.
     * @param cal A Calendar object. Ignored if cal is NULL.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public void setDate(int parameterIndex, java.sql.Date x, Calendar cal)
	        throws SQLException {
        if(cal == null) {
            setDate(parameterIndex,x);
        }
        else {
          if (x == null)
            {
              setNull(parameterIndex, Types.DATE);
              return;
            }
            else
            {
              java.sql.Date tdate = new java.sql.Date(x.getTime() + cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET));
              setDate(parameterIndex,tdate);
            }
          }
	}

    /**
     * JDBC 2.0
     *
     * Sets a parameter to a java.sql.Time value.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x The parameter value, a java.sql.Time object.
     * @param cal A Calendar object. Ignored if cal is NULL.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public void setTime(int parameterIndex, java.sql.Time x, Calendar cal)
	        throws SQLException {
        if(cal == null) {
            setTime(parameterIndex,x);
        }
        else {
          if (x == null)
          {
            setNull(parameterIndex, Types.TIME);
            return;
          }
          else
          {
            java.sql.Time ttime = new java.sql.Time(x.getTime() + cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET));
            setTime(parameterIndex,ttime);
          }
        }
	}

    /**
     * JDBC 2.0
     *
	 * Sets a parameter to a java.sql.Timestamp value.
     *
     * @param parameterIndex The parameter index. The first parameter is 1, the second is 2, and so on.
     * @param x The parameter value, a java.sql.Timestamp value.
     * @param cal A Calendar object. Ignored if cal is NULL.
     * @exception SQLException This exception is thrown if parameter index is not valid.
     */
    public void setTimestamp(int parameterIndex, java.sql.Timestamp x, Calendar cal)
	        throws SQLException {
        if(cal == null) {
            setTimestamp(parameterIndex,x);
        }
        else {
          if (x == null)
          {
            setNull(parameterIndex, Types.TIMESTAMP);
            return;
          }
          else
          {
            java.sql.Timestamp ttimestamp = new java.sql.Timestamp(x.getTime() + cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET));
            setTimestamp(parameterIndex,ttimestamp);
          }
        }
	}

    /**
     * JDBC 2.0
     *
     * Sets a parameter to SQL NULL.
     * This method is not yet supported.
     *
	 * @param parameterIndex The parameter index. The first parameter is 1, the second parameter is 2, and so on.
     * @param sqlType An SQL type code that is defined by java.sql.Types.
     * @param x An input String.
     * @exception SQLException This exception is always thrown.
     */
    public synchronized void setNull(int parameterIndex, int sqlType, String x) throws SQLException {
	 MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

	}

    /**
     * JDBC 2.0
     *
     * This method is not yet supported.
     *
	 * @param parameterIndex The parameter index. The first parameter is 1, the second parameter is 2, and so on.
     * @param sqlType An SQL type code that is defined by java.sql.Types.
     * @param x An input String.
     * @exception SQLException This exception is always thrown.
     */
    public synchronized void registerOutParameter(int parameterIndex, int sqlType, String x) throws SQLException {
        MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotSupported );

	}

    private synchronized void substituteParameters() throws SQLException
    {
      subsquery = new String();
// BFB 08Apr98 String tempString = new String();
      String parm;
      int b = 0;
      int e = 0;
      int inquotes = -1;
      int length = sqlquery.length();
      for (int i = 0; i < parmCount; i++) {
        parm = parameters[i];
        if (parm == null) {
            Object[] args = {  new Integer(++i) };
	   MVALocalizedError.throwLocalizedException(VendorCode.MVAPreparedStatement_substituteParamError, args ); //, String.valueOf(++i));

        }
        // if length is greater than maxFieldSize, check if double quotes are
        // present. if there are no double quotes, throw datatruncation exception.
        // if there are double quotes, if length is greater than maxFieldSize+2, then,
        // throw datatruncation exception.
        if (parm.length() > (super.maxFieldSize)) {
          if (parm.charAt(0) == '\"' || parm.charAt(0) == '\'') {
             if (parm.length() > (super.maxFieldSize+2)) {
                throw new DataTruncation(i+1, true, false, parm.length(), super.maxFieldSize);
             }
          } else {
             throw new DataTruncation(i+1, true, false, parm.length(), super.maxFieldSize);
          }
        }
        while (e < length) {
          if ((sqlquery.charAt(e) == '\'') || (sqlquery.charAt(e) == '\"')) {
            if (inquotes == -1)
              inquotes = sqlquery.charAt(e);
            else if (inquotes == sqlquery.charAt(e))
              inquotes = -1;
            }
          if ((sqlquery.charAt(e) == '?') && (inquotes == -1) ) {
            subsquery = subsquery + sqlquery.substring(b,e) + parm;
            b = e + 1;
            e++;
            break;
          }
          e++;
        }
      }
      if (b < length)
        subsquery = subsquery + sqlquery.substring(b);
    }
    
    /**
     * Retrieves the number, types and properties of this PreparedStatement object's parameters.
	 * Empty method implementation for JDK1.4 compliance.
	 * @exception SQLException Always thrown to indicate that this method is not implemented.
	 */
	public ParameterMetaData getParameterMetaData() throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );	
		return null;
	}
    /**
     * Sets the designated parameter to the given java.net.URL value.
     * Empty method implementation for JDK1.4 compliance.
     * @exception SQLException Always thrown to indicate that this method is not implemented.
     */
	public void setURL(int parameterIndex, URL x) throws SQLException
	{
		MVALocalizedError.throwLocalizedException(VendorCode.General_methodNotImplemented );		
	}

}
