/* $Id$ */
/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */
/*
Mitchel Soltys
I'm in the process or reorganizing a little. Initially the changes will be
primarily organizational in order to understand the code more easily.
Start at the bottom of the file, since this will be a bottom up ;)
reorganization.
 */
/*
Holders
It appears that, when using CORBA, to access pass-by-reference arguments to C methods,
you create a holder and then pass it into the function. The data can then be
accessed via, holder.value.
 */
package com.sas.rio;
import java.math.BigDecimal;
import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Calendar;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Ref;
import java.sql.Array;
import com.sas.iom.SAS.IDataSetPackage.BindKeys;

import com.sas.iom.JDBCConstants;
//
//History Records:
//
// Date          Name                Remarks
// 05/03/99 M Srinivasan   - Fix to S0052769. Fixed scrolling problems.
// 06/10/99 M Srinivasan   - S0056456: Added SequentialAccess to flags parameter in
//                           OpenDataSet if user requests sequential access.
//                           The default for OpenDataSet is random access.
// 06/11/99 M Srinivasan   - S0056715: ResultSet.getDouble() should return double value
//                           of DATE/TIME/TIMESTAMP objects instead of throwing exception
//                           since these are stored internally in SAS as double values.
// 07/09/99 M Srinivasan   - Fix to updateXXX methods to handle inserts to new table.
//                           Fix to insertRow. Removed call to LockRecord.
// 07/27/99 M Srinivasan   - S0061725 : Fix initialization of blocknum problem in constructor
//                           MVAResultSet(sasDataSet).
// 08/03/99 M Srinivasan   - S0061395 : RIOColumnType -> BaseType change (IOM Hierarchy Change).
// 08/19/99 M Srinivasan   - S0063704 : Record output methods in RIO/IOM not accounting for
//                           fuzzing and missing values in all environments. Interface change to
//                           updateRecords() and insertRecords() to send additional values -
//                           namely, missingNumericValues, changedNumericValues and changedCharacterValues.
//                           The changed* flags indicate if a column value has been updated by the client.
//                           The missingNumericValue indicate the missing value to be updated to.
//                           Had to keep track of which columns were updated whenever a update* method is
//                           called.
//                           Also, fixed null constructor to set eof/bof/maxreached flags to true to indicate
//                           empty resultset.
// 08/24/99 M Srinivasan    -S0064453 : array, columnCount,sortedBy made protected as MVASqlTypesResultSet extends
//                           this class.
// 08/30/99 M Srinivasan    -S0062489 : Look for BegOfFile (RIO change) while scrolling backwards (readRecords).
//                          -Reset blocknum at the end of afterLast() and beforeFirst() calls and commented closing
//						     ResultSet when EOF or BOF is reached. This will enable scrolling backwards/forwards.
//                          -Reset blocknum after updateRecords() and addRecords() due to RIO behavior.
// 09/07/99 M Srinivasan    -S0065400 : Fix for array out of bounds. The cached array index should remain less than
//                           fetchsize.
//							-Also fix rs.first() and rs.last() problems.
//                          -Add code for optimistic locking. compareOK() method implemented.
// 10/12/99 M Srinivasan    -S0065400 : Improvement to optimistic locking. When updateRow() fails with an exception due to
//                           data inconsistency, users can call refreshRow() to redo the update.
// 10/25/99 M Srinivasan    -S0068652 : Return default ResultSet type and ResultSetConcurrency type when getType() or
//                           getConcurrencyType() is called from a ResultSet object obtained via DBMD calls. This is
//                           much better than throwing NullPointerException even though there is no Statement associated
//                           to the ResultSet.
// 10/29/99 M Srinivasan    -S0068901 : Fix to passing null elements in String[] arrays. Brgorb is more stricter and
//                           requires all input parameters to have non-null values.
// 11/10/99 M Srinivasan    -S0065788 : Fix scrolling problems.
// 12/06/99 M Srinivasan    -S0070764 : Fix problem calling absolute() twice.
//  02Feb01 R Gonsalves     S0101990: RESULTSET.GETBYTE() THROWS NUMBERFORMATEXCEPTION FOR VALID VALUES
/*
 * 7/10/2002 D Mackie       - S0158451: reimplemented caching.  rearranged some of the code.
 * 12/12/2002 D Mackie      - S0142734: rmi implementation.
 * 12/23/2002 D Mackie      - S0176479: added code to check for the rs being closed prior
 *                                      to performing any action.  If closed, throw an exception.
 * 05Feb2003 damack         - S0182464: added calls to throwGenericExceptions in updateXXXX methods to throw
 *                                      an exception if an update is attempted on an invalid row.
 * 18Mar2003 damack         - S0190151: added code to relative to check the cursor position.  if the 
 *                                      cursor is off the resultset (beforefirst/afterlast), then we 
 *                                      throw an exception.  This brings us into closer compliance with
 *                                      the API spec. 
 * 06May2003 damack         -S0196790: tweaked the getXXXX routines to cut down on object allocs.
 * 15Jul2003 damack         -S0211519: changed timestampToDouble to not use "x.getNanos() / 1000000"
 * 28Jul2003 damack         -S0213132: removed code from get<Primitive> methods that set a warning
 *                                     when a missing value was retrieved.  This change affects:
 *                                     getShort, getInt, getLong, getFloat, getDouble
 * 14Apr2004 damack         -S0217898: corrected glitch with fix of DST issue.  We were using cal.get(DSTOFFSET)
 *                                     when adjusting for the dst offset, in the expectation that this would give
 *                                     give us the correct offset for that timezone.  Unfortunately, didn't quite
 *                                     realize that the value returned depended on our calendar at a given moment
 * 									   in time(!).  Have changed to use a hard coded constant (3600000 = 3600 * 1000). 
 * 26Apr2004 damack         -S0189758: javadoc for 1.4 methods.
 * 14Jan2005 damack         -S0278967: add properties object to calls to MVARowCache (from ctors which take a properties object)
 *                                     in order to get coercenumerictypes to the cache for init-ing.
 * 14Jan2005 damack         -S0282468: add a calcJavaDate method, called from getDate/Time/Timestamp(index, cal) so that
 *                                     the proper timezone is used to calculate the offset from sas epoch.
 * */
/**
 * <P>A ResultSet provides access to a table of data.  A ResultSet
 * object is usually generated by executing a Statement.
 *
 * <P>A ResultSet maintains a cursor pointing to its current row of
 * data.  Initially the cursor is positioned before the first row.
 * The 'next' method moves the cursor to the next row.
 *
 * <P>The getXXX methods retrieve column values for the current
 * row.  You can retrieve values either using the index number of the
 * column, or by using the name of the column.  In general using the
 * column index will be more efficient.  Columns are numbered from 1.
 *
 * <P>For maximum portability, ResultSet columns within each row should be
 * read in left-to-right order and each column should be read only once.
 *
 * <P>For the getXXX methods, the JDBC driver attempts to convert the
 * underlying data to the specified Java type and returns a suitable
 * Java value.  See the JDBC specification for allowable mappings
 * from SQL types to Java types with the ResultSet.getXXX methods.
 *
 * <P>Column names used as input to getXXX methods are case
 * insensitive.  When performing a getXXX using a column name, if
 * several columns have the same name, then the value of the first
 * matching column will be returned. The column name option is
 * designed to be used when column names are used in the SQL
 * query. For columns that are NOT explicitly named in the query, it
 * is best to use column numbers. If column names were used there is
 * no way for the programmer to guarantee that they actually refer to
 * the intended columns.
 *
/** <P>A ResultSet is automatically closed by the Statement that
 * generated it when that Statement is closed, re-executed, or is used
 * to retrieve the next result from a sequence of multiple results.
 *
 * <P>The number, types and properties of a ResultSet's columns are
 * provided by the ResulSetMetaData object returned by the getMetaData
 * method.
 *
 * @see Statement#executeQuery
 * @see Statement#getResultSet
 * @see ResultSetMetaData
 */
public class MVAResultSet implements java.sql.ResultSet
{
	final static long MSECS_PER_HOUR = 3600000; 
    boolean m_isClosed = true;
    static double JAVA_DATE_EPOCH = 3653;
    //no. of days between 01jan60 and 01jan70
    static long MAX_SASTIME = 86400; //no of seconds in a day
    static double JAVA_DATETIME_EPOCH = JAVA_DATE_EPOCH * MAX_SASTIME * 1000.0;
    //3653 days in milliseconds
    //    boolean date;
    //boolean time;
    String m_errorMsg = null;
    public static final int MAYBE_INTEGER = 5;
    public static final int INTEGER = 4;
    public static final int DATETIME = 3;
    public static final int TIME = 2;
    public static final int DATE = 1;
    public static final int NOSUBTYPE = 0;
    public static final int CHARACTER = 2;
    public static final int NUMERIC = 1;
    static final int MAXLEN = 32767;
    private java.util.Properties m_info = null;
    protected int maxFieldSize = 32767;
    protected int m_fetchSize = MVAConnection.XHRSQPRECS;
    protected SQLWarning m_warnings = null;
    private boolean m_isUpdatable = false;
    protected int bindKey = BindKeys._BindKeysUnformatted;
    private boolean m_getTableFlag = false;
    /*
     * TRANSITION STUFF:
     */
    protected MVAStatement m_statement = null;
    private int m_direction = FETCH_FORWARD;
    private int m_maxFieldSize = 32767;
    protected MVARowCache m_rowCache = null;
    //private IDataSet m_sasDataSet = null;
    protected boolean m_wasNull = false;
    protected int m_columnCount = 0;
    protected int m_maxRows = 0;
    /*
     * Constructs a ResultSet. One generated through communication
     * with the MVA server.
     */
    public MVAResultSet() throws SQLException
    {
        m_rowCache = new MVARowCache();
        /*
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    public MVAResultSet(java.util.Properties p) throws SQLException
    {
        m_info = p;
        String ntc = null;
        if (m_info != null)
            ntc = (String)m_info.get("coercenumerictypes");

        if (ntc != null && ntc.length() > 0 && ntc.equalsIgnoreCase("false"))
            m_rowCache = new MVARowCache(false);
        else 
            m_rowCache = new MVARowCache();
        /*
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    /*
     * Constructs a ResultSet. One generated through communication
     * with the MVA server.
     * @param sasDataSet The SASDataSet object that generated the result set.
     */
    /*
       public MVAResultSet(IDataSet sasDataSet) throws SQLException
       {
       m_sasDataSet = sasDataSet;
       m_rowCache = new MVARowCache(m_sasDataSet);
       }
    */
    public MVAResultSet(int hashCode, XUtil xUtil) throws SQLException
    {
        m_rowCache = new MVARowCache(hashCode, xUtil);
        /**
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    /*
       public MVAResultSet(IDataSet sasDataSet, java.util.Properties p)
       throws SQLException
       {
       m_info = p;
       m_sasDataSet = sasDataSet;
       m_rowCache = new MVARowCache(m_sasDataSet);
       }
    */
    public MVAResultSet(int hashCode, XUtil xUtil, java.util.Properties p)
        throws SQLException
    {
        m_info = p;
        String ntc = null;
        if (m_info != null)
            ntc = (String)m_info.get("coercenumerictypes");
        if (ntc != null && ntc.length() > 0 && ntc.equalsIgnoreCase("false"))
            m_rowCache = new MVARowCache(hashCode, xUtil, false);
        else 
            m_rowCache = new MVARowCache(hashCode, xUtil);
        
        /**
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    /*
     * Constructs a ResultSet. One generated through communication
     * with the MVA server.
     * @param stmt The Statement object that is used to create/open the table.
     * @param tableName The name of the table.
     * @param tableType The type of the table, "DATA", or "VIEW" or NULL.
     * @param password The READ, WRITE or ALTER password of the table if any.
     * @exception SQLException This exception is thrown if an error condition is detected.
     */
    public MVAResultSet(
        MVAStatement stmt,
        String tableName,
        String tableType,
        String[] password)
        throws SQLException
    {
        /**
            this.stmt = stmt;
            this.tableName = tableName;
            this.tableType = tableType;
            this.password = password;
            this.fetchSize = stmt.fetchSize;
            this.blocknum = stmt.fetchSize - 1;
            this.sasLibref = stmt.sasLibref;
            this.maxFieldSize = stmt.maxFieldSize;
            this.m_isUpdatable = true;
            this.options = stmt.options;
            openDataSet();
            InitChangedValues();
            m_getTableFlag = true;
        */
        this(stmt, tableName, tableType, password, null);
        /**
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    public MVAResultSet(
        MVAStatement stmt,
        String tableName,
        String tableType,
        String[] password,
        java.util.Properties p)
        throws SQLException
    {
        m_info = p;
        String bK = null;
        String ntc = null;
        if (m_info != null)
        {
            bK = m_info.getProperty(JDBCConstants.APPLY_FORMATS.toLowerCase());
            ntc = m_info.getProperty("coercenumerictypes");
        }
        if (bK != null && bK.length() > 0)
            bindKey = parseBindKey(bK);
        m_statement = stmt;
        /**
            this.tableName = tableName;
            this.tableType = tableType;
            this.password = password;
            this.fetchSize = stmt.fetchSize;
            this.blocknum = stmt.fetchSize - 1;
            this.sasLibref = stmt.sasLibref;
            this.maxFieldSize = stmt.maxFieldSize;
            this.m_isUpdatable = true;
            this.options = stmt.options;
            openDataSet();
            InitChangedValues();
        */
        m_getTableFlag = true;
        m_direction = stmt.direction;
        m_maxFieldSize = stmt.maxFieldSize;
        //m_sasDataSet = stmt.sasDataSet;
        if (stmt.getResultSetConcurrency() == ResultSet.CONCUR_UPDATABLE)
            m_isUpdatable = true;
        else
            m_isUpdatable = false;
        //    m_isUpdatable = true;
        
        if (ntc != null && ntc.length() > 0 && ntc.equalsIgnoreCase("false"))
            m_rowCache = new MVARowCache(m_statement.getXUtil(),
                    					m_statement, 
                    					tableName, 
                    					password, 
                    					false);
        else
            m_rowCache =
            new MVARowCache(
                m_statement.getXUtil(),
                m_statement,
                tableName,
                password);
        if (stmt.fetchSize > 0)
        {
            m_fetchSize = stmt.fetchSize;
            m_rowCache.setCacheSize(m_fetchSize);
        }
        m_rowCache.setRSType(m_statement.getResultSetType());
        m_columnCount = m_rowCache.getColumnCount();
        m_rowCache.setBindkey(bindKey);
        if (stmt.getMaxRows() > 0)
            m_rowCache.setMaxRows(stmt.getMaxRows());
        /**
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    /*
     * Constructs a ResultSet. One generated through communication
     * with the MVA server.
     * @param stmt The MVAStatement object that generated the result set.
     * @exception SQLException This exception is thrown if an error condition is detected.
     */
    public MVAResultSet(MVAStatement stmt, int hashCode) throws SQLException
    {
        /**
            this.stmt = stmt;
            this.fetchSize = stmt.fetchSize;
            this.blocknum = stmt.fetchSize - 1;
            this.direction = stmt.direction;
            this.maxFieldSize = stmt.maxFieldSize;
            this.sasDataSet = stmt.sasDataSet;
            InitChangedValues();
        */
        this(stmt, null, hashCode);
        /**
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    public MVAResultSet(
        MVAStatement stmt,
        java.util.Properties p,
        int hashCode)
        throws SQLException
    {
        m_info = p;
        String bK = null;
        String ntc = null;
        if (m_info != null)
        {
            bK = m_info.getProperty(JDBCConstants.APPLY_FORMATS.toLowerCase());
            ntc = m_info.getProperty("coercenumerictypes");
        }
       if (bK != null && bK.length() > 0)
            bindKey = parseBindKey(bK);
        m_statement = stmt;
        /** don't worry about bind key support right now.
         * concentrate on the basics 
         */
        m_direction = stmt.direction;
        m_maxFieldSize = stmt.maxFieldSize;
        //m_sasDataSet = stmt.sasDataSet;
        if (ntc != null && ntc.length() > 0 && ntc.equalsIgnoreCase("false"))
            m_rowCache = new MVARowCache(hashCode, stmt.getXUtil(), false);
        else
            m_rowCache = new MVARowCache(hashCode, stmt.getXUtil());
        m_rowCache.setRSType(m_statement.getResultSetType());
        if (stmt.fetchSize > 0)
        {
            m_fetchSize = stmt.fetchSize;
            m_rowCache.setCacheSize(m_fetchSize);
        }
        m_columnCount = m_rowCache.getColumnCount();
        m_rowCache.setBindkey(bindKey);
        if (stmt.getMaxRows() > 0)
            m_rowCache.setMaxRows(stmt.getMaxRows());

        /**
         * if we make it to the end, we can safely assume the result set is open.
         */
        m_isClosed = false;
    }
    /*
      private void InitChangedValues() throws SQLException
      {
       
        changedValues = new boolean[1][m_rowCache.getColumnCount()];
        for(int i=0;i<m_rowCache.getColumnCount();i++) {
          changedValues[0][i] = false;
        }
        beforeFirst();
      }
    */
    protected int getBindKey()
    {
        return bindKey;
    }
    /**************************************************************************************/
    /* POSITIONING METHODS:                                                               */
    /**************************************************************************************/
    /**
      * JDBC 2.0
      * Moves to an absolute row number in the result set.
      *
      * <p>If row is positive, moves to an absolute row with respect to the
      * beginning of the result set.  The first row is row 1, the second
      * is row 2, and so on.
      *
      * <p>If row is negative, moves to an absolute row position with respect to
      * the end of result set.  For example, calling absolute(-1) positions the
      * cursor on the last row, absolute(-2) indicates the next-to-last
      * row, and so on.
      *
      * <p>An attempt to position the cursor beyond the first/last row in
      * the result set, leaves the cursor before/after the first/last
      * row, respectively.
      *
      * <p>Note: Calling absolute(1) is the same as calling first().
      * Calling absolute(-1) is the same as calling last().
      *
      * @param rowsOffset The number of rows to move to.
      * @return True if on the result set, False if off.
      * @exception SQLException This exception is thrown if a server error is detected or
      * rowsOffset is 0, or result set type is TYPE_FORWARD_ONLY.
      */
    public synchronized boolean absolute(int rowIndex) throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRSType);
        else if (rowIndex == 0)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRowNumber);
        return m_rowCache.absolute(rowIndex);
    }
    /**
     * JDBC 2.0
     * Moves a relative number of rows, either positive or negative.
     * Attempting to move beyond the first/last row in the
     * result set positions the cursor before/after the
     * the first/last row. Calling relative(0) is valid, but does
    * not change the cursor position.
    *
    * <p>Note: Calling relative(1) is different than calling next()
    * since is makes sense to call next() when there is no current row,
    * for example, when the cursor is positioned before the first row
    * or after the last row of the result set.
    *
    * @param rowsOffset The number of rows to move to.
    * @return True if on a row, False otherwise.
    * @exception SQLException This exception is thrown if a server error is detected or there
    * is no current row, or result set type is TYPE_FORWARD_ONLY.
    */
    public synchronized boolean relative(int rowOffset) throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRSType);
        /*
         * S0190151: check cursor position and throw an exception if not
         * on a valid row
         */
        if (m_rowCache.getAfterLast() || m_rowCache.getBeforeFirst())
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRowPosition);
        return m_rowCache.relative(rowOffset);
    }
    /**
     * JDBC 2.0
     * 
     * Moves to the front of the result set, just before the
     * first row. Has no effect if the result set contains no rows.
     *
     * @exception SQLException This exception is thrown if the result set type is TYPE_FORWARD_ONLY.
     */
    public synchronized void beforeFirst() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRSType);
        if (m_rowCache.getBeforeFirst())
            return;
        m_rowCache.absolute(1);
        m_rowCache.previous();
    }
    /**
     * JDBC 2.0
     * 
     * Moves to the end of the result set, just after the last
     * row.  Has no effect if the result set contains no rows.
     *
     * @exception SQLException This exception is thrown if the result set type is TYPE_FORWARD_ONLY.
     */
    public synchronized void afterLast() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRSType);
        if (m_rowCache.getAfterLast())
            return;
        m_rowCache.absolute(-1);
        m_rowCache.next();
    }
    /**
     * JDBC 2.0
     * 
     * Moves to the first row in the result set.
     *
     * @return True if on a valid row, False if no rows in the result set.
     * @exception SQLException This exception is thrown if a server error is detected or if the
     * result set type is TYPE_FORWARD_ONLY.
     */
    public synchronized boolean first() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRSType);
        return m_rowCache.absolute(1);
    }
    /**
     * JDBC 2.0
     * 
     * Moves to the last row in the result set.
     *
     * @return True if on a valid row, False if no rows in the result set.
     * @exception SQLException This exception is thrown if an MvaException is detected or
     * result set type is TYPE_FORWARD_ONLY.
     */
    public synchronized boolean last() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRSType);
        return m_rowCache.absolute(-1);
    }
    /**
     * JDBC 2.0
     * 
     * Determines if the cursor is before the first row in the result set.
     *
     * @return True if before the first row, False otherwise. Returns
     * false when the result set contains no rows.
     * @exception SQLException This exception is required by the interface but it is never thrown.
     */
    public synchronized boolean isBeforeFirst() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cantPerformForwardOnly);
        }
        return m_rowCache.getBeforeFirst();
    }
    /**
     * JDBC 2.0
     *  Determines if the cursor is after the last row in the result set.
     * NOTE: Indicates EOF (status flag from last ReadRecords())
     *
     * @return True if after the last row, False otherwise.  Returns
     * false when the result set contains no rows.
     * @exception SQLException This exception is required by the interface but it is never thrown.
     */
    public synchronized boolean isAfterLast() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cantPerformForwardOnly);
        }
        return m_rowCache.getAfterLast();
    }
    /** 
     * JDBC 2.0
     *  Indicates whether the cursor is on the first row of this <code>ResultSet</code> object.
     * @return boolean - true if cursor in on first row, false if not.
     * @exception SQLException - If a data access error occurs.
     */
    public boolean isFirst() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cantPerformForwardOnly);
        }
        return m_rowCache.getIsFirst();
    }
    /** 
     * JDBC 2.0
     *  Indicates whether the cursor is on the last row of this <code>ResultSet</code> object.
     * @return boolean - true if cursor in on last row, false if not.
     * @exception SQLException - If a data access error occurs.
     */
    public boolean isLast() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cantPerformForwardOnly);
        }
        return m_rowCache.getIsLast();
    }
    /**
     * Proceeds to the next row in the ResultSet.
     * @return True if the next row is valid and False if not valid.
     * @exception SQLException This exception is thrown if a server error is detected.
     */
    public synchronized boolean next() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return m_rowCache.next();
    }
    /**
     * JDBC 2.0
     *
     * Moves to the previous row in the result set.
     *
     * <p>Note: previous() is not the same as relative(-1) since it
     * makes sense to call previous() when there is no current row.
     *
     * @return True if on a valid row, False if off the result set.
     * @exception SQLException This exception is thrown if a server error is detected or
     * result set type is TYPE_FORWARD_ONLY.
     */
    public synchronized boolean previous() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (getType() == ResultSet.TYPE_FORWARD_ONLY)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRSType);
        return m_rowCache.previous();
    }
    /*****************************************************************************************/
    /**
     * Closes the ResultSet.
     *
     * @exception SQLException This exception is thrown if a server error is detected.
     */
    public synchronized void close() throws SQLException
    {
        if (m_rowCache != null)
            m_rowCache.close();
        m_rowCache = null;
        m_isClosed = true;
    }
    /**
     * Determines whether the given column in the current row holds an SQL NULL. This
     * method always returns False for SQL VARCHAR types.
     * Note that you must first call getXXX on a column to try to read
     * its value and then call wasNull() to find if the value was
     * the SQL NULL.
     * @return True if the last column that is read contains a NULL value.
     * @exception SQLException This exception is thrown if the current row is not valid.
     */
    public synchronized boolean wasNull() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return m_wasNull;
    }
    //======================================================================
    // Methods for accessing results by column index
    //======================================================================
    /**
     * Gets the value of a column in the current row as a Java String.
     *
     * @param columnIndex The first column is 1, the second is 2, and so on.
     * @return The column value. If the value is SQL NULL, the result is null.
     * @exception SQLException This exception is thrown if the column value is not valid.
     */
    public synchronized String getString(int columnIndex) throws SQLException
    {
        String tempString;
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (o == null || m_rowCache.getIsMissing(columnIndex))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        if (type != Types.VARCHAR && bindKey != BindKeys._BindKeysFormatted)
        {
            double value = Double.valueOf(o.toString()).doubleValue();
            long javadate;
            switch (type)
            {
                case Types.DATE :
                    javadate = calcJavaDate(value, true, false);
                    tempString =
                        toSQLString(javadate, true, false, m_maxFieldSize);
                    break;
                case Types.TIME :
                    javadate = calcJavaDate(value, false, true);
                    tempString =
                        toSQLString(javadate, false, true, m_maxFieldSize);
                    break;
                case Types.TIMESTAMP :
                    javadate = calcJavaDate(value, true, true);
                    tempString =
                        toSQLString(javadate, true, true, m_maxFieldSize);
                    break;
                default :
                    if (type == Types.BIT)
                    {
                        if (value == 0)
                            tempString = "FALSE";
                        else
                            tempString = "TRUE";
                    }
                    else
                    {
                        if (type == Types.SMALLINT || type == Types.INTEGER)
                            tempString = "" + (int) value;
                        else
                            tempString = "" + value;
                        //"NaN"
                    }
                    break;
            }
        }
        //Assume its a VARCHAR
        //else if(type == BaseType._BaseTypeCharacter) {
        else
        {
            tempString = o.toString();
        }
        if (tempString.length() > m_maxFieldSize)
        {
            tempString = tempString.substring(0, m_maxFieldSize);
        }
        return tempString;
    }
    /**
     * Gets the value of a column in the current row as a Java boolean.
     *
     * @param columnIndex The first column is 1, the second is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is FALSE.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a boolean value.
     */
    public synchronized boolean getBoolean(int columnIndex) throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToBoolean);
        }
        if ((o == null) || m_rowCache.getIsMissing(columnIndex))
        {
            m_wasNull = true;
            return false;
        }
        else
            m_wasNull = false;
        /*
         * add bindkey stuff here.  not sure how to handle booleans, though, so
         * i need to do research.
         */
        if (type == Types.BIT)
        {
            if ((double) (Double.parseDouble(o.toString().trim()))
                == 0)
                return false;
            else
                return true;
        }
        if (type == Types.VARCHAR)
        {
            String ctemp = o.toString().trim();
            Boolean temp = new Boolean(ctemp);
            try
            {
                return temp.booleanValue();
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            if ((double) (Double.parseDouble(o.toString().trim()))
                == 0)
                return false;
            else
                return true;
        }
        return false;
    }
    /**
     * Gets the value of a column in the current row as a Java byte.
     *
     * @param columnIndex The first column is 1, the second is 2, and so on.
     * @return The column value. If the value is SQL NULL, then result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to byte representation.
     */
    public synchronized byte getByte(int columnIndex) throws SQLException
    {
        // Rag: I don't see the purpose of singleByte being an array as opposed to a scalar of type byte.
        byte singleByte[] = new byte[1];
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToByte);
        }
        if ((o == null) || m_rowCache.getIsMissing(columnIndex))
        {
            m_wasNull = true;
            return singleByte[0];
        }
        else
            m_wasNull = false;
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            if (temp.length() == 0)
                return singleByte[0];
            else
            {
                try
                {
                    return (byte) (Byte.valueOf(temp).byteValue());
                }
                catch (NumberFormatException e)
                {
                    MVALocalizedError.throwLocalizedException(
                        VendorCode.MVAResultSet_numberFormatError,
                        null,
                        e);
                }
            }
        }
        else
        {
            // Rag: I feel we can use my new commented out statement (below) as all numeric values are stored as Doubles.
            //      However, at this point I'm not comfortable enough with the code to do it even though with this change S0101990 works
            // singleByte[0] = ((Double)currentRow[columnIndex-1]).byteValue();
            singleByte[0] = Double.valueOf(o.toString().trim()).byteValue();
        }
        return singleByte[0];
    }
    /**
     * Gets the value of a column in the current row as a Java short.
     *
     * @param columnIndex The first column is 1, the second is 2, and so on.
     * @return The column value. If the value is SQL NULL, then result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to byte representation.
     */
    public synchronized short getShort(int columnIndex) throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToShort);
        }
        if ((o == null) || m_rowCache.getIsMissing(columnIndex))
        {
            m_wasNull = true;
            return (short) 0;
        }
        else
            m_wasNull = false;
        /**
         * for booleans, is it ok to do the below?
         */
        if (type == Types.BIT)
        {
            if (Boolean.valueOf(o.toString().trim()).booleanValue() == true)
                return (short) 1;
            else
                return (short) 0;
        }
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                return (short) (Integer.parseInt(temp));
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            return (short) (Double.parseDouble(o.toString().trim()));
        }
        return 0;
    }
    /**
     * Gets the value of a column in the current row as a Java int.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid int value.
     *
     */
    public synchronized int getInt(int columnIndex) throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToInt);
        }
        if ((o == null) || m_rowCache.getIsMissing(columnIndex))
        {
            m_wasNull = true;
            return 0;
        }
        else
            m_wasNull = false;
        /**
         * for booleans, is it ok to do what's going on below?
         */
        if (type == Types.BIT)
        {
            if (Boolean.valueOf(o.toString().trim()).booleanValue() == true)
                return 1;
            else
                return 0;
        }
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                return (int) (Integer.parseInt(temp));
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            //is it ok to convert from double to int ?? I think so.
            return (int) (Double.parseDouble(o.toString().trim()));
        }
        return 0;
    }
    /**
     * Gets the value of a column in the current row as a Java long.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid long value.
     */
    public synchronized long getLong(int columnIndex) throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToLong);
        }
        if ((o == null) || m_rowCache.getIsMissing(columnIndex))
        {
            m_wasNull = true;
            return (long) 0;
        }
        else
            m_wasNull = false;
        /**
         * for booleans, since we're doing a .toString(), I don't think
         * we have any problems with formatting
         */
        if (type == Types.BIT)
        {
            if (Boolean.valueOf(o.toString().trim()).booleanValue() == true)
                return (long) 1;
            else
                return (long) 0;
        }
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                return (long) (Long.parseLong(temp));
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            //is it ok to convert from double to short ?? I think so.
            return (long) (Double.parseDouble(o.toString().trim()));
        }
        return 0;
    }
    /**
     * Gets the value of a column in the current row as a Java float.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid float value.
     */
    public synchronized float getFloat(int columnIndex) throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToFloat);
        }
        if (o == null || m_rowCache.getIsMissing(columnIndex))
        {
            m_wasNull = true;
            return (float) 0;
        }
        else
            m_wasNull = false;
        /**
         * again, i don't think we have a problem with boolean formatting
         */
        if (type == Types.BIT)
        {
            if (Boolean.valueOf(o.toString().trim()).booleanValue() == true)
                return (float) 1;
            else
                return (float) 0;
        }
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                return (float) (Float.parseFloat(temp));
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            //is it ok to convert from double to float ?? I think so.
            return (float) (Double.parseDouble(o.toString().trim()));
        }
        return (float) 0.0;
    }
    /**
     * Gets the value of a column in the current row as a Java double.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid double value.
     */
    public synchronized double getDouble(int columnIndex) throws SQLException
    {
        throwGenericExceptions(columnIndex);
        
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        //double value = 0.0;
        //if (o != null) value = ((Number)o).doubleValue();
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex)))
            
        {
            m_wasNull = true;
            return (double) 0;
        }
        else
            m_wasNull = false;
        /**
         * just be aware of bools
         */
        if (type == Types.BIT)
        {
            boolean b = Boolean.valueOf(o.toString().trim()).booleanValue();    
            if (b == true)
                return (double) 1;
            else
            {
                return (double) 0;
            }
        }
        
        if (type == Types.VARCHAR)
        {
            String temp = (String)o; //o.toString().trim();
            try
            {
                double v = Double.parseDouble(temp);
                return v;
                //return (double) (Double.valueOf(temp).doubleValue());
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            if (bindKey == BindKeys._BindKeysUnformatted)
                return ((Number)o).doubleValue();   
            else
                return Double.parseDouble(((String)o).trim()); 
            //return Double.parseDouble(o.toString().trim());
            // return (double) (Double.valueOf(o.toString().trim()).doubleValue());
        }
        return 0.0;
    }
    /**
     * Gets the value of a column in the current row as a Java BigDecimal.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid BigDecimal value.
     * @deprecated
     */
    public synchronized BigDecimal getBigDecimal(int columnIndex, int scale)
        throws SQLException
    {
        double tdouble = 0.0;
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToNumeric);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        if (type == Types.BIT)
        {
            if (Boolean.valueOf(o.toString().trim()).booleanValue() == true)
                tdouble = (double) 1;
            else
                tdouble = (double) 0;
        }
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                tdouble = Double.parseDouble(temp);
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            tdouble = Double.parseDouble(o.toString().trim());
        }
        return new BigDecimal(tdouble);
    }
    /**
     * Gets the value of a column in the current row as a Java byte array. This method
     * is not supported.
     * @exception SQLException This exception is always thrown because SAS software
     * does not support conversions to binary representation.
     */
    public synchronized byte[] getBytes(int columnIndex) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.MVAResultSet_getBytesError);
        return null;
    }
    /**
     * Gets the value of a column in the current row as a Java java.sql.Date.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Date value.
     */
    public synchronized java.sql.Date getDate(int columnIndex)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (!((type == Types.DATE)
            || (type == Types.VARCHAR)
            || (type == Types.TIMESTAMP)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToDate);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        /**
         * if bindKey is "formatted", then the date is going to be in string form.
         */
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                return java.sql.Date.valueOf(temp);
            }
            catch (IllegalArgumentException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            double value = Double.parseDouble(o.toString());
            long javadate;
            if (type == Types.DATE)
            {
                javadate = calcJavaDate(value, true, false);
                return ((java.sql.Date) toSQLType(javadate, true, false));
            }
            else
            {
                javadate = calcJavaDate(value, true, true);
                java.sql.Timestamp tstamp =
                    (java.sql.Timestamp) toSQLType(javadate, true, true);
                return (new java.sql.Date(tstamp.getTime()));
            }
        }
        return null;
    }
    /**
     * Gets the value of a column in the current row as a Java java.sql.Time.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Time value.
     */
    public synchronized java.sql.Time getTime(int columnIndex)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (!((type == Types.TIME)
            || (type == Types.VARCHAR)
            || (type == Types.TIMESTAMP)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToTime);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        /**
         * much like the date, i'm guessing that the time would be in
         * a string format
         */
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                return java.sql.Time.valueOf(temp);
            }
            catch (IllegalArgumentException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            double value = Double.parseDouble(o.toString().trim());
            long javadate = calcJavaDate(value, false, true);
            return ((java.sql.Time) toSQLType(javadate, false, true));
        }
        return null;
    }
    /**
     * Gets the value of a column in the current row as a Java java.sql.Timestamp.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Timestamp value.
     */
    public synchronized java.sql.Timestamp getTimestamp(int columnIndex)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (!((type == Types.DATE)
            || (type == Types.VARCHAR)
            || (type == Types.TIMESTAMP)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToTimestamp);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        /**
         * damack - ditto the date/time deal for timestamp.
         */
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                return java.sql.Timestamp.valueOf(temp);
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            double value = Double.parseDouble(o.toString());
            long javadate;
            if (type == Types.DATE)
            {
                javadate = calcJavaDate(value, true, false);
                java.sql.Date tdate =
                    (java.sql.Date) toSQLType(javadate, true, false);
                return (new java.sql.Timestamp(tdate.getTime()));
            }
            else
            {
                javadate = calcJavaDate(value, true, true);
                return ((java.sql.Timestamp) toSQLType(javadate, true, true));
            }
        }
        return null;
    }
    /**
     * Gets the value of a column in the current row as a Java InputStream.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Type.VARCHAR value. SAS software
     * does not support other conversions.
     *
     */
    public synchronized java.io.InputStream getAsciiStream(int columnIndex)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (type != Types.VARCHAR)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToInputStream);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        String temp = o.toString().trim();
        byte[] outbytes = new byte[temp.length()];
        outbytes = temp.getBytes();
        return (java.io.InputStream) new ByteArrayInputStream(outbytes);
    }
    /**
     * Gets the value of a column in the current row as a Java InputStream.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Type.VARCHAR value. SAS software
     * does not support other conversions.
     * @deprecated
     */
    public synchronized java.io.InputStream getUnicodeStream(int columnIndex)
        throws SQLException
    {
        return getAsciiStream(columnIndex);
    }
    /**
     * Gets the value of a column in the current row as a Java InputStream.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Type.VARCHAR value. SAS software
     * does not support other conversions.
     */
    public synchronized java.io.InputStream getBinaryStream(int columnIndex)
        throws SQLException
    {
        return getAsciiStream(columnIndex);
    }
    //======================================================================
    // Methods for accessing results by column name
    //======================================================================
    /**
     * Gets the value of a column in the current row as a Java String.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid.
     */
    public synchronized String getString(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getString(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java boolean.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is False.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a boolean value.
     */
    public synchronized boolean getBoolean(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getBoolean(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java byte.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted to byte representation.
     */
    public synchronized byte getByte(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getByte(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java short.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid short value.
     */
    public synchronized short getShort(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getShort(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java int.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid int value.
     */
    public synchronized int getInt(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getInt(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java long.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid long value.
     */
    public synchronized long getLong(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getLong(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java float.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid float value.
     */
    public synchronized float getFloat(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getFloat(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java double.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is 0.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid double value.
     */
    public synchronized double getDouble(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getDouble(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java BigDecimal.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid BigDecimal value.
     * @deprecated
     */
    public synchronized BigDecimal getBigDecimal(String columnName, int scale)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getBigDecimal(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java byte array. This method is
     * not supported.
     * @exception SQLException This exception is always thrown becaue SAS software does not
     * support conversions to binary representation.
     */
    public synchronized byte[] getBytes(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getBytes(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java java.sql.Date.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid java.sql.Date value.
     */
    public synchronized java.sql.Date getDate(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getDate(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java java.sql.Time.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid java.sql.Time value.
     */
    public synchronized java.sql.Time getTime(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getTime(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java java.sql.Timestamp.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid java.sql.Timestamp value.
     */
    public synchronized java.sql.Timestamp getTimestamp(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getTimestamp(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java InputStream.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * value is not java.sql.Type.VARCHAR. No other conversions are supported.
     */
    public synchronized java.io.InputStream getAsciiStream(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getAsciiStream(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java InputStream.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * value is not java.sql.Type.VARCHAR. No other conversions are supported.
     * @deprecated
     */
    public synchronized java.io.InputStream getUnicodeStream(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getUnicodeStream(columnIndex));
    }
    /**
     * Gets the value of a column in the current row as a Java InputStream.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * value is not java.sql.Type.VARCHAR. No other conversions are supported.
     *
     */
    public synchronized java.io.InputStream getBinaryStream(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getBinaryStream(columnIndex));
    }
    /*****************************************************************************/
    //=====================================================================
    // Advanced features:
    //=====================================================================
    /**
     * Gets the SQLWarnings that are associated with this row.
     *
     * @return SQLWarning. All warnings for the current row.
     * @exception SQLException This exception is required by the interface, but it is never thrown.
     */
    public synchronized SQLWarning getWarnings() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return m_warnings;
    }
    /**
     * After this call getWarnings returns null until a new warning is
     * reported for this ResultSet.
     *
     * @exception SQLException This exception is required by the interface, but it is never thrown.
     */
    public synchronized void clearWarnings() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        m_warnings = null;
    }
    /**
     * Gets the name of the SQL cursor used by this ResultSet.
     *
       *
     * @return the ResultSet's SQL cursor name
     * @exception SQLException if a database-access error occurs.
     */
    public synchronized String getCursorName() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
  
        return null;
    }
    /**
     * Gets a ResultSetMetaData object that is associated with this ResultSet.
     *
     * @return A MVAResultSetMetaData object.
     * @exception SQLException This exception is thrown if an error condition is detected.
     */
    public synchronized ResultSetMetaData getMetaData() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return new MVAResultSetMetaData(this);
    }
    /**
     * Gets the value of a column in its default Java representation.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return The column value as an Object. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized Object getObject(int columnIndex) throws SQLException
    {

        throwGenericExceptions(columnIndex);
        
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        if (type != Types.VARCHAR && bindKey != BindKeys._BindKeysFormatted)
            {
                // if it's not VARCHAR, then it's a double internally.
                double value =  ((Number)o).doubleValue();
                // S0196790: above line WAS this below. 
                //Double.parseDouble(o.toString().trim());
                // The new way to get value is significantly faster, due to object alloc costs.
                //double value = Double.valueOf(o.toString()).doubleValue();
                long javadate;
                switch (type)
                    {
                    case Types.DATE :
                        javadate = calcJavaDate(value, true, false);
                        return toSQLType(javadate, true, false);
                    case Types.TIME :
                        javadate = calcJavaDate(value, false, true);
                        return toSQLType(javadate, false, true);
                    case Types.TIMESTAMP :
                        javadate = calcJavaDate(value, true, true);
                        return toSQLType(javadate, true, true);
                    default :
                        switch (type)
                            {
                            case Types.BIT :
                                Boolean btemp;
                                if (value == 0)
                                    btemp = Boolean.FALSE;
                                else
                                    btemp = Boolean.TRUE;
                                return (Object) btemp;
                            case Types.INTEGER :
                            case Types.SMALLINT :
                                Integer itemp = new Integer((int) value);
                                return (Object) itemp;
                            default :
                                //Double temp = new Double(value);
                                //return (Object) temp;
                                return o;
                            }
                    }
            }
        //Assume its a VARCHAR
        //else if(type == BaseType._BaseTypeCharacter) {
        else
        {
     
            return o;
            //return (Object) o.toString();
        }
    }    
    /**
     * Gets the value of a column in its default Java representation.
     *
     * @param columnName The name of the column.
     * @return The column value as an Object. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized Object getObject(String columnName) throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getObject(columnIndex));
    }
    //----------------------------------------------------------------
    /**
     * Maps a Resultset column name to a ResultSet column index.
     *
     * @param columnName The name of the column.
     * @return The column index.
     * @exception SQLException This exception is thrown if the column name is not valid.
     */
    public synchronized int findColumn(String columnName) throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (columnName == null)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_findColumnError);
        }
        int columnNumber = m_rowCache.getColumnIndex(columnName.trim());
        if (columnNumber < 1)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_findColumnError);
        }
        return columnNumber;
    }
    //--------------------------JDBC 2.0-----------------------------------
    //---------------------------------------------------------------------
    // Getter's and Setter's
    //---------------------------------------------------------------------
    /**
     * JDBC 2.0
     *
     * Gets the value of a column in the current row as a java.io.Reader.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return A java.io.Reader Object. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized java.io.Reader getCharacterStream(int columnIndex)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (type != Types.VARCHAR)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToReader);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        String temp = o.toString().trim();
        byte[] outbytes = new byte[temp.length()];
        outbytes = temp.getBytes();
        return (java.io.Reader) new InputStreamReader(
            (java.io.InputStream) new ByteArrayInputStream(outbytes));
    }
    /**
     * JDBC 2.0
     *
     * Gets the value of a column in the current row as a java.io.Reader.
     *
     * @param columnName The name of the column.
     * @return A java.io.Reader Object. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized java.io.Reader getCharacterStream(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return getCharacterStream(columnIndex);
    }
    /**
     * JDBC 2.0
     *
     * Gets the value of a column in the current row as a java.math.BigDecimal
     * object.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return A BigDecimal object.If the value is SQL NULL, the result is NULL.
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized BigDecimal getBigDecimal(int columnIndex)
        throws SQLException
    {
        return getBigDecimal(columnIndex, 0);
    }
    /**
     * JDBC 2.0
     *
     * Gets the value of a column in the current row as a java.math.BigDecimal
     * object.
     *
     * @param columnName The name of the column.
     * @return A BigDecimal object.If the value is SQL NULL, the result is NULL.
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized BigDecimal getBigDecimal(String columnName)
        throws SQLException
    {
        int columnIndex = findColumn(columnName);
        return (getBigDecimal(columnIndex, 0));
    }
    /**
     * JDBC 2.0
     *
     * Gets the current row number.  The first row is number 1, the
     * second number 2, and so on.
     *
     * @return The current row number.Return 0 if there is no current row.
     * @exception SQLException This exception is required by the interface but it is never thrown.
     */
    public synchronized int getRow() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return m_rowCache.getActual();
    }
    /**
     * JDBC 2.0
     *
     * Give a hint as to the direction in which the rows in a result set
     * will be processed. The hint applies only to result sets created
     * using this Statement object.  The default value is
     * ResultSet.FETCH_FORWARD.
     *
     * @param direction The initial direction for processing rows.
     * @exception SQLException This exception is thrown if the direction
     * is not one of ResultSet.FETCH_FORWARD, ResultSet.FETCH_REVERSE, or
     * ResultSet.FETCH_UNKNOWN.
     */
    public void setFetchDirection(int direction) throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if ((direction != ResultSet.FETCH_FORWARD)
            && (direction != ResultSet.FETCH_REVERSE)
            && (direction != ResultSet.FETCH_UNKNOWN))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidFetchDirection);
        }
        m_direction = direction;
    }
    /**
     * JDBC 2.0
     *
     * Gets the result set fetch direction.
     *
     * @return The result set fetch direction. The default is ResultSet.FETCH_FORWARD.
     * @exception SQLException This exception is required by the interface but it is never thrown.
     */
    public int getFetchDirection() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return m_direction;
    }
    /**
     * JDBC 2.0
     *
     * Give the JDBC driver a hint as to the number of rows that should
     * be fetched from the database when more rows are needed.
     * If the value specified is zero, then the hint is ignored. The default value is zero.
       *
     * @param rows The number of rows to fetch.
     * @exception SQLException This exception is thrown if the
     * condition 0 <= rows <= stmt.getMaxRows() is not satisfied.
     */
    public void setFetchSize(int rows) throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (rows == 0)
        {
            m_fetchSize = MVAConnection.XHRSQPRECS;
            m_rowCache.setCacheSize(m_fetchSize);
            return;
        }
        if ((rows < 0)
            || ((m_statement.getMaxRows() != 0)
                && (rows > m_statement.getMaxRows())))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidFetchSize);
        }
        m_fetchSize = rows;
        m_rowCache.setCacheSize(m_fetchSize);
    }
    /**
     * JDBC 2.0
     *
     * Gets the fetch size.
     * @return An integer value indicating the number of rows fetched for this result set.
     * @exception SQLException This exception is required by the interface but it is never thrown.
     */
    public int getFetchSize() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return m_fetchSize;
    }
    /**
     * JDBC 2.0
     *
     * Gets the result set type. The type is determined based
     * on the statement that created the result set.
     * @return The result set type, TYPE_SCROLL_INSENSITIVE or TYPE_FORWARD_ONLY based on the Statement object.
     * @exception SQLException This exception is required by the interface but it is never thrown.
     */
    public synchronized int getType() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (m_statement != null)
        {
            return m_statement.getResultSetType();
        }
        else
        {
            return ResultSet.TYPE_FORWARD_ONLY;
        }
    }
    /**
     * JDBC 2.0
     *
     * Gets the concurrency of this result set.  The concurrency
     * used is determined by the statement that created the result set.
     *
     * @return The concurrency type, CONCUR_UPDATABLE or CONCUR_READ_ONLY based on the Statement object.
     * @exception SQLException This exception is required by the interface but it is never thrown.
     */
    public synchronized int getConcurrency() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (m_statement != null)
        {
            if (m_statement.getResultSetConcurrency()
                == ResultSet.CONCUR_UPDATABLE
                && m_getTableFlag)
                return ResultSet.CONCUR_UPDATABLE;
            else
            {
                SQLWarning w =
                    new SQLWarning(
                        MVALocalizedError.getString(
                            VendorCode.MVAResultSet_concurrencyWarning),
                        null,
                        VendorCode.MVAResultSet_concurrencyWarning);
                if (m_warnings == null)
                    m_warnings = w;
                else
                    m_warnings.setNextWarning(w);
                return ResultSet.CONCUR_READ_ONLY;
            }
        }
        else
        {
            return ResultSet.CONCUR_READ_ONLY;
        }
    }
    //---------------------------------------------------------------------
    // Updates
    //---------------------------------------------------------------------
    /**
     * JDBC 2.0
     *
     * Determine if the current row has been updated.  This method is not
     * supported.
     * @exception SQLException This exception is always thrown because SAS software
     * does not detect row updated.
     *
     * @see MVADatabaseMetaData#updatesAreDetected
     */
    public synchronized boolean rowUpdated() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        MVALocalizedError.throwLocalizedException(
            VendorCode.MVAResultSet_rowUpdatedError);
        return false;
    }
    /**
     * JDBC 2.0
     *
     * Determines if the current row has been inserted.  This method is not
     * supported.
     * @exception SQLException This exception is always thrown because SAS software
     * does not detect row inserts.
     *
     * @see MVADatabaseMetaData#insertsAreDetected
     */
    public synchronized boolean rowInserted() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        MVALocalizedError.throwLocalizedException(
            VendorCode.MVAResultSet_rowInsertedError);
        return false;
    }
    /**
     * JDBC 2.0
     *
     * Determines if this row has been deleted.  This method is not
     * supported.
     * @exception SQLException This exception is always thrown because SAS software
     * does not detect row deletes.
     *
     * @see MVADatabaseMetaData#deletesAreDetected
     */
    public synchronized boolean rowDeleted() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        MVALocalizedError.throwLocalizedException(
            VendorCode.MVAResultSet_rowDeletedError);
        return false;
    }
    /**
     * JDBC 2.0
     *
     * Give a nullable column a null value.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateNull(int columnIndex) throws SQLException
    {
        throwGenericExceptions(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        try
        {
            if (m_rowCache.getColumnType(columnIndex) != Types.VARCHAR)
            {
                m_rowCache.updateData(columnIndex, new Double(Double.NaN));
            }
            else
            {
                m_rowCache.updateData(columnIndex, null);
            }
        }
        catch (Exception e)
        {
            Object[] messageArg = { e.getClass().getName(), e.getMessage()};
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_updateNullError,
                messageArg);
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a boolean value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateBoolean(int columnIndex, boolean x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToBoolean);
        }
        if (type == Types.BIT)
        {
            if (x == true)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
        }
        if (type == Types.VARCHAR)
        {
            //TODO: Do we support updateBoolean for a Character column ?????
            Boolean tbool = new Boolean(x);
            m_rowCache.updateData(columnIndex, tbool.toString());
        }
        else
        {
            //TODO: Do we need special handling here ...
            if (x == true)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a byte value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateByte(int columnIndex, byte x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        int type = m_rowCache.getColumnType(columnIndex);
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToByte);
        }
        if (type == Types.BIT)
        {
            if (x == 1)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else if (x == 0)
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
            else
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_cannotConvertToByte);
            }
        }
        Byte tbyte = new Byte(x);
        String temp = tbyte.toString();
        if (type == Types.VARCHAR)
        {
            if (temp.length() == 0)
                m_rowCache.updateData(columnIndex, null);
            else
                m_rowCache.updateData(columnIndex, temp);
        }
        else
        {
            m_rowCache.updateData(columnIndex, Double.valueOf(temp));
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a short value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateShort(int columnIndex, short x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);

        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToShort);
        }
        if (type == Types.BIT)
        {
            if (x == 0)
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
            else if (x == 1)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_cannotConvertToShort);
            }
        }
        else
        {
            Short tshort = new Short(x);
            String temp = tshort.toString();
            if (type == Types.VARCHAR)
            {
                if (temp.length() == 0)
                    m_rowCache.updateData(columnIndex, null);
                else
                    m_rowCache.updateData(columnIndex, temp);
            }
            else
            {
                m_rowCache.updateData(columnIndex, Double.valueOf(temp));
            }
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an integer value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateInt(int columnIndex, int x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);

        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToInt);
        }
        if (type == Types.BIT)
        {
            if (x == 0)
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
            else if (x == 1)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_cannotConvertToInt);
            }
        }
        else
        {
            Integer tint = new Integer(x);
            String temp = tint.toString();
            if (type == Types.VARCHAR)
            {
                if (temp.length() == 0)
                    m_rowCache.updateData(columnIndex, null);
                else
                    m_rowCache.updateData(columnIndex, temp);
            }
            else
            {
                m_rowCache.updateData(columnIndex, Double.valueOf(temp));
            }
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a long value.
     * Method not yet supported.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is always thrown.
     */
    public synchronized void updateLong(int columnIndex, long x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotSupported);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a float value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateFloat(int columnIndex, float x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        
        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }
        
        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToFloat);
        }
        if (type == Types.BIT)
        {
            if (x == 0)
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
            else if (x == 1)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_cannotConvertToFloat);
            }
        }
        else
        {
            Float tfloat = new Float(x);
            String temp = tfloat.toString();
            if (type == Types.VARCHAR)
            {
                if (temp.length() == 0)
                    m_rowCache.updateData(columnIndex, null);
                else
                    m_rowCache.updateData(columnIndex, temp);
            }
            else
            {
                m_rowCache.updateData(columnIndex, Double.valueOf(temp));
            }
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a Double value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateDouble(int columnIndex, double x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        
        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }
        
        /*
        	if((type == Types.DATE) || (type == Types.TIME) || (type == Types.TIMESTAMP)) 
        	    {
        		MVALocalizedError.throwLocalizedException(VendorCode.MVAResultSet_cannotConvertToDouble );
        		
        	    }
        */
        if (type == Types.BIT)
        {
            if (x == 0)
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
            else if (x == 1)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_cannotConvertToDouble);
            }
        }
        else
        {
            Double tdouble = new Double(x);
            String temp = tdouble.toString();
            if (type == Types.VARCHAR)
            {
                if (temp.length() == 0)
                    m_rowCache.updateData(columnIndex, null);
                else
                    m_rowCache.updateData(columnIndex, temp);
            }
            else
            {
                m_rowCache.updateData(columnIndex, tdouble);
            }
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a BigDecimal value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateBigDecimal(int columnIndex, BigDecimal x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if ((type == Types.DATE)
            || (type == Types.TIME)
            || (type == Types.TIMESTAMP))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToBigDecimal);
        }
        if (x == null)
            {
                updateNull(columnIndex);
                return;   
            }
        if (type == Types.BIT)
        {
            if (x.doubleValue() == 0)
                m_rowCache.updateData(columnIndex, Double.valueOf("0"));
            else if (x.doubleValue() == 1)
                m_rowCache.updateData(columnIndex, Double.valueOf("1"));
            else
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_cannotConvertToBigDecimal);
            }
        }
        else
        {
            Double tdouble = new Double(x.doubleValue());
            String temp = x.toString();
            if (type == Types.VARCHAR)
            {
                if (temp.length() == 0)
                    m_rowCache.updateData(columnIndex, null);
                else
                    m_rowCache.updateData(columnIndex, temp);
            }
            else
            {
                m_rowCache.updateData(columnIndex, tdouble);
            }
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a String value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateString(int columnIndex, String x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);

        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if (x == null) 
            {
                updateNull(columnIndex);
                return;
            }
        else if (type != Types.VARCHAR)
        {
            switch (type)
            {
                case Types.DATE :
                    m_rowCache.updateData(columnIndex, Date.valueOf(x.trim()));
                    break;
                case Types.TIME :
                    m_rowCache.updateData(columnIndex, Time.valueOf(x.trim()));
                    break;
                case Types.TIMESTAMP :
                    m_rowCache.updateData(
                        columnIndex,
                        Timestamp.valueOf(x.trim()));
                    break;
                default :
                    if (type == Types.BIT)
                    {
                        if (x.trim() == "1")
                            m_rowCache.updateData(
                                columnIndex,
                                Double.valueOf(x.trim()));
                        else if (x.trim() == "0")
                            m_rowCache.updateData(
                                columnIndex,
                                Double.valueOf(x.trim()));
                        else
                        {
                            MVALocalizedError.throwLocalizedException(
                                VendorCode.MVAResultSet_invalidInputString);
                        }
                    }
                    else
                    {
                        m_rowCache.updateData(
                            columnIndex,
                            Double.valueOf(x.trim()));
                        //"NaN"
                    }
                    break;
            }
        }
        else
        {
            m_rowCache.updateData(columnIndex, x);
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a byte array value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateBytes(int columnIndex, byte x[])
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        updateString(columnIndex, new String(x));
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a Date value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateDate(int columnIndex, java.sql.Date x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);

        if (x == null)
        {
            updateNull(columnIndex);
            return;
        }
        double value = dateToDouble(x);
        int type = m_rowCache.getColumnType(columnIndex);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if (!((type == Types.DATE)
            || (type == Types.TIMESTAMP)
            || (type == Types.VARCHAR)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToDate);
        }
        switch (type)
        {
            case Types.DATE :
                m_rowCache.updateData(columnIndex, new Double(value));
                break;
            case Types.TIMESTAMP :
                m_rowCache.updateData(columnIndex, new Double(value));
                break;
            default :
                m_rowCache.updateData(columnIndex, x.toString());
                break;
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a Time value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateTime(int columnIndex, java.sql.Time x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        if (x == null)
        {
            updateNull(columnIndex);
            return;   
        }
        int type = m_rowCache.getColumnType(columnIndex);
        double value = timeToDouble(x);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if (!((type == Types.TIME)
            || (type == Types.TIMESTAMP)
            || (type == Types.VARCHAR)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToTime);
        }
        switch (type)
        {
            case Types.TIME :
                m_rowCache.updateData(columnIndex, new Double(value));
                break;
            case Types.TIMESTAMP :
                m_rowCache.updateData(columnIndex, new Double(value));
                break;
            default :
                m_rowCache.updateData(columnIndex, x.toString());
                break;
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a Timestamp value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateTimestamp(
        int columnIndex,
        java.sql.Timestamp x)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);

        int type = m_rowCache.getColumnType(columnIndex);
        if (x == null)
        {
            updateNull(columnIndex);
            return;   
        }
        double value = timestampToDouble(x);
        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if (!((type == Types.DATE)
            || (type == Types.TIMESTAMP)
            || (type == Types.VARCHAR)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToTimestamp);
        }
        switch (type)
        {
            case Types.DATE :
                m_rowCache.updateData(columnIndex, new Double(value));
                break;
            case Types.TIMESTAMP :
                m_rowCache.updateData(columnIndex, new Double(value));
                break;
            default :
                m_rowCache.updateData(columnIndex, x.toString());
                break;
        }
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an ascii stream value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @param length the length of the stream
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateAsciiStream(
        int columnIndex,
        java.io.InputStream x,
        int length)
        throws SQLException
    {
         throwGenericExceptions(columnIndex);        
        if (x == null) 
            {
                updateNull(columnIndex);
                return;   
            }
        char[] value = new char[length];
        InputStreamReader reader = new InputStreamReader(x);
        try
        {
            reader.read(value);
        }
        catch (IOException e)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_updateAsciiStreamFailedError,
                null,
                e);
        }
        updateString(columnIndex, new String(value));
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a binary stream value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @param length the length of the stream
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateBinaryStream(
        int columnIndex,
        java.io.InputStream x,
        int length)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (x == null)
            {
                updateNull(columnIndex);
                return;   
            }
        byte[] value = new byte[length];
        try
        {
            x.read(value);
        }
        catch (IOException e)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_updateBinaryStreamFailedError,
                null,
                e);
        }
        updateBytes(columnIndex, value);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a character stream value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @param length the length of the stream
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateCharacterStream(
        int columnIndex,
        java.io.Reader x,
        int length)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);

        if (x == null) 
        {
            updateNull(columnIndex);
            return;
        }    
        char[] value = new char[length];
        try
        {
            x.read(value);
        }
        catch (IOException e)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_updateCharStreamFailedError,
                null,
                e);
        }
        updateString(columnIndex, new String(value));
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an Object value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @param scale For java.sql.Types.DECIMAL or java.sql.Types.NUMERIC types
     *  this is the number of digits after the decimal.  For all other
     *  types this value will be ignored.
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateObject(int columnIndex, Object x, int scale)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);

        if (m_isUpdatable == false)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        }

        if (x == null)
        {
            updateNull(columnIndex);
            return;
        }
        if (x instanceof java.sql.Date)
        {
            updateDate(columnIndex, (java.sql.Date) x);
            return;
        }
        if (x instanceof java.sql.Time)
        {
            updateTime(columnIndex, (java.sql.Time) x);
            return;
        }
        if (x instanceof java.sql.Timestamp)
        {
            updateTimestamp(columnIndex, (java.sql.Timestamp) x);
            return;
        }
        if ((x instanceof java.lang.String)
            || (x instanceof java.lang.Character))
        {
            updateString(columnIndex, x.toString());
            return;
        }
        if (x instanceof java.lang.Integer)
        {
            updateInt(columnIndex, ((java.lang.Integer) x).intValue());
            return;
        }
        if (x instanceof java.lang.Float)
        {
            updateFloat(columnIndex, ((java.lang.Float) x).floatValue());
            return;
        }
        if (x instanceof java.lang.Double)
        {
            updateDouble(columnIndex, ((java.lang.Double) x).doubleValue());
            return;
        }
        MVALocalizedError.throwLocalizedException(
            VendorCode.MVAResultSet_unknownObjectType);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an Object value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @param x the new column value
     * @exception SQLException This exception is thrown if the column index is not valid.
     */
    public synchronized void updateObject(int columnIndex, Object x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        updateObject(columnIndex, x, 0);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a null value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateNull(String columnName) throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateNull(columnIndex);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a boolean value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateBoolean(String columnName, boolean x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateBoolean(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a byte value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateByte(String columnName, byte x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateByte(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a short value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateShort(String columnName, short x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateShort(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an integer value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateInt(String columnName, int x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateInt(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a long value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateLong(String columnName, long x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateLong(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a float value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateFloat(String columnName, float x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateFloat(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a double value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateDouble(String columnName, double x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateDouble(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a BigDecimal value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateBigDecimal(String columnName, BigDecimal x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateBigDecimal(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a String value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateString(String columnName, String x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateString(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a byte array value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateBytes(String columnName, byte x[])
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateBytes(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a Date value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateDate(String columnName, java.sql.Date x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateDate(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a Time value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateTime(String columnName, java.sql.Time x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateTime(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a Timestamp value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateTimestamp(
        String columnName,
        java.sql.Timestamp x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateTimestamp(columnIndex, x);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an ascii stream value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @param length of the stream
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateAsciiStream(
        String columnName,
        java.io.InputStream x,
        int length)
        throws SQLException
    {

        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateAsciiStream(columnIndex, x, length);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a binary stream value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @param length of the stream
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateBinaryStream(
        String columnName,
        java.io.InputStream x,
        int length)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateBinaryStream(columnIndex, x, length);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with a character stream value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @param length of the stream
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateCharacterStream(
        String columnName,
        java.io.Reader reader,
        int length)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateCharacterStream(columnIndex, reader, length);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an Object value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @param scale For java.sql.Types.DECIMAL or java.sql.Types.NUMERIC types
     *  this is the number of digits after the decimal.  For all other
     *  types this value will be ignored.
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateObject(
        String columnName,
        Object x,
        int scale)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateObject(columnIndex, x, scale);
    }
    /**
     * JDBC 2.0
     *
     * Update a column with an Object value.
     *
     * The updateXXX() methods are used to update column values in the
     * current row, or the insert row.  The updateXXX() methods do not
     * update the underlying database, instead the updateRow() or insertRow()
     * methods are called to update the database.
     *
     * @param columnName the name of the column
     * @param x the new column value
     * @exception SQLException if a database-access error occurs
     */
    public synchronized void updateObject(String columnName, Object x)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        updateObject(columnIndex, x);
    }
    /**********************************************************************************/
    /* <REPLACE THIS BLOCK>                                                           */
    /**
     * JDBC 2.0
     *
     * Insert the contents of the insert row into the result set and
     * the database.  Must be on the insert row when this method is called.
     *
     * NOTE: SAS software does not support inserts, though inserts might be
     * implemented as append.
     * @exception SQLException This exception is always thrown.
     *
     * This function is available only on ResultSets generated from MVAStatement.getTable()
     */
    public synchronized void insertRow() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (!m_isUpdatable)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        /** todo: bindkey stuff */
        m_rowCache.insertRow();
    }
    /* </replace> ? */
    /**
     * JDBC 2.0
     *
     * Update the underlying database with the new contents of the
     * current row.  Cannot be called when on the insert row.
     *
     * @exception SQLException if a database-access error occurs, or
     * if called when on the insert row
     *
     * This function is available only on ResultSets generated from MVAStatement.getTable()
     */
    public synchronized void updateRow() throws SQLException
    {
        if (m_rowCache.getAfterLast() || m_rowCache.getBeforeFirst())
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRowPosition);   
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (!m_isUpdatable)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notUpdatableResultSetError);
        /** todo: bindkey stuff */
        m_rowCache.updateRow();
    }
    /**
     * JDBC 2.0
     *
     * Delete the current row from the result set and the underlying
     * database.  Cannot be called when on the insert row.
     *
     * @exception SQLException if a database-access error occurs, or if
     * called when on the insert row.
     */
    public synchronized void deleteRow() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (!m_isUpdatable)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notDeletableResultSetError);
        m_rowCache.deleteRow();
    }
    /**
     * JDBC 2.0
     *
     * Refresh the value of the current row with its current value in
     * the database.  Cannot be called when on the insert row.
     *
     * The refreshRow() method provides a way for an application to
     * explicitly tell the JDBC driver to refetch a row(s) from the
     * database.  An application may want to call refreshRow() when
     * caching or prefetching is being done by the JDBC driver to
     * fetch the latest value of a row from the database.  The JDBC driver
     * may actually refresh multiple rows at once if the fetch size is
     * greater than one.
     *
     * All values are refetched subject to the transaction isolation
     * level and cursor sensitivity.  If refreshRow() is called after
     * calling updateXXX(), but before calling updateRow() then the
     * updates made to the row are lost.  Calling refreshRow() frequently
     * will likely slow performance.
     *
     * @exception SQLException This exception is thrown if an MvaException is detected.
     */
    public synchronized void refreshRow() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        m_rowCache.refresh();
    }
    /**
     * JDBC 2.0
     *
     * The cancelRowUpdates() method may be called after calling an
     * updateXXX() method(s) and before calling updateRow() to rollback
     * the updates made to a row.  If no updates have been made or
     * updateRow() has already been called, then this method has no
     * effect.
     *
     * @exception SQLException if a database-access error occurs, or if
     * called when on the insert row.
     *
     */
    public synchronized void cancelRowUpdates() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (m_isUpdatable)
        {
            m_rowCache.cancelUpdate();
        }
    }
    /**
     * JDBC 2.0
     *
     * Move to the insert row.  The current cursor position is
     * remembered while the cursor is positioned on the insert row.
     *
     * The insert row is a special row associated with an updatable
     * result set.  It is essentially a buffer where a new row may
     * be constructed by calling the updateXXX() methods prior to
     * inserting the row into the result set.
     *
     * Only the updateXXX(), getXXX(), and insertRow() methods may be
     * called when the cursor is on the insert row.  All of the columns in
     * a result set must be given a value each time this method is
     * called before calling insertRow().  UpdateXXX()must be called before
     * getXXX() on a column.
     *
     * NOTE: SAS software does not support inserts, though inserts might be
     * implemented as append.
     *
     * @exception SQLException This exception is always thrown.
     */
    public synchronized void moveToInsertRow() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (!m_isUpdatable)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_unableToMoveToInsertRowError);
        m_rowCache.moveToInsertRow();
    }
    /**
     * JDBC 2.0
     *
     * Move the cursor to the remembered cursor position, usually the
     * current row.  Has no effect unless the cursor is on the insert
     * row.
     *
     * @exception SQLException if a database-access error occurs,
     * or the result set is not updatable
     */
    public synchronized void moveToCurrentRow() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (!m_isUpdatable)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_unableToMoveToCurrentRowError);
        m_rowCache.moveToCurrentRow();
        return;
    }
    /**
     * JDBC 2.0
     *
     * Return the Statement that produced the ResultSet.
     *
     * @return the Statment that produced the result set, or
     * null if the result was produced some other way.
     * @exception SQLException if a database-access error occurs
     */
    public synchronized Statement getStatement() throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        return m_statement;
    }
    /**
     * JDBC 2.0
     *
     * Returns the value of column i as a Java object.  Use the 
     * @map to determine the class from which to construct data of
     * SQL structured and distinct types.
     *
     * @param i the first column is 1, the second is 2, ...
     * @param map the mapping from SQL type names to Java classes
     * @return an object representing the SQL value
     */
    public Object getObject(int i, java.util.Map map) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get a REF(&lt;structured-type&gt;) column.
     *
     * @param i the first column is 1, the second is 2, ...
     * @return an object representing data of an SQL REF type
     */
    public Ref getRef(int i) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get a BLOB column.
     *
     * @param i the first column is 1, the second is 2, ...
     * @return an object representing a BLOB
     */
    public Blob getBlob(int i) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get a CLOB column.
     *
     * @param i the first column is 1, the second is 2, ...
     * @return an object representing a CLOB
     */
    public Clob getClob(int i) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get an array column.
     *
     * @param i the first column is 1, the second is 2, ...
     * @return an object representing an SQL array
     */
    public Array getArray(int i) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Returns the value of column i as a Java object.  Use the
     * map to determine the class from which to construct data of
     * SQL structured and distinct types.
     *
     * @param colName the column name
     * @param map the mapping from SQL type names to Java classes
     * @return an object representing the SQL value
     */
    public Object getObject(String colName, java.util.Map map)
        throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get a REF(&lt;structured-type&gt;) column.
     *
     * @param colName the column name
     * @return an object representing data of an SQL REF type
     */
    public Ref getRef(String colName) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get a BLOB column.
     *
     * @param colName the column name
     * @return an object representing a BLOB
     */
    public Blob getBlob(String colName) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get a CLOB column.
     *
     * @param colName the column name
     * @return an object representing a CLOB
     */
    public Clob getClob(String colName) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Get an array column.
     *
     * @param colName the column name
     * @return an object representing an SQL array
     */
    public Array getArray(String colName) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * JDBC 2.0
     *
     * Gets the value of a column in the current row as a java.sql.Date
     * object.  Use the calendar to construct an appropriate millisecond
     * value for the Date, if the underlying database doesn't store
     * timezone information.
     *
     * @param columnIndex The first column is 1, the second is 2, and so on.
     * @param cal The calendar to use in constructing the date. Ignored if cal is NULL.
     * @return The column value. If the value is SQL NULL, then the result is null.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Date value.
     */
    public synchronized java.sql.Date getDate(int columnIndex, Calendar cal)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (!((type == Types.DATE)
            || (type == Types.VARCHAR)
            || (type == Types.TIMESTAMP)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToDate);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        /**
         * if bindKey is "formatted", then the date is going to be in string form.
         */
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                if (cal == null)
                    return java.sql.Date.valueOf(temp);
                else 
                {
                    Date tempDate = java.sql.Date.valueOf(temp);
                    return new Date(tempDate.getTime() + cal.getTimeZone().getOffset(tempDate.getTime()));
                }
            }
            catch (IllegalArgumentException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            double value = Double.parseDouble(o.toString());
            long javadate;
            if (cal == null)
                cal = Calendar.getInstance();
            if (type == Types.DATE)
            {
                javadate = calcJavaDate(value, true, false, cal);
                return ((java.sql.Date) toSQLType(javadate, true, false));
            }
            else
            {
                javadate = calcJavaDate(value, true, true, cal);
                java.sql.Timestamp tstamp =
                    (java.sql.Timestamp) toSQLType(javadate, true, true);
                return (new java.sql.Date(tstamp.getTime()));
            }
        }
        return null;
  
    }
    /**
     * Gets the value of a column in the current row as a java.sql.Date
     * object. Use the calendar to construct an appropriate millisecond
     * value for the Date, if the underlying database doesn't store
     * timezone information.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @param cal The calendar to use in constructing the date. Ignored if cal is NULL.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid java.sql.Date value.
     */
    public synchronized java.sql.Date getDate(String columnName, Calendar cal)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        return getDate(columnIndex, cal);
    }
    /**
     * Gets the value of a column in the current row as a java.sql.Time
     * object. Use the calendar to construct an appropriate millisecond
     * value for the Time, if the underlying database doesn't store
     * timezone information.
     *
     * @param columnIndex The first column is 1, the second is 2, and so on.
     * @param cal The calendar to use in constructing the date. Ignored if cal is NULL.
     * @return The column value. If the value is SQL NULL, then the result is null.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Time value.
     */
    public synchronized java.sql.Time getTime(int columnIndex, Calendar cal)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (!((type == Types.TIME)
            || (type == Types.VARCHAR)
            || (type == Types.TIMESTAMP)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToTime);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        /**
         * much like the date, i'm guessing that the time would be in
         * a string format
         */
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                if (cal == null)
                    return java.sql.Time.valueOf(temp);
                else
                {
                    java.sql.Time tempTime = Time.valueOf(temp);
                    return new java.sql.Time(tempTime.getTime() + cal.getTimeZone().getOffset(tempTime.getTime()));
                }
            }
            catch (IllegalArgumentException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            if (cal == null) cal = Calendar.getInstance();
            double value = Double.parseDouble(o.toString().trim());
            long javadate = calcJavaDate(value, false, true, cal);
            return ((java.sql.Time) toSQLType(javadate, false, true));
        }
        return null;

    }
    /**
     * Gets the value of a column in the current row as a java.sql.Time
     * object. Use the calendar to construct an appropriate millisecond
     * value for the Time, if the underlying database doesn't store
     * timezone information.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @param cal The calendar to use in constructing the date. Ignored if cal is NULL.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid java.sql.Time value.
     */
    public synchronized java.sql.Time getTime(String columnName, Calendar cal)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        return (getTime(columnIndex, cal));
    }
    /**
     * Gets the value of a column in the current row as a java.sql.Timestamp
     * object. Use the calendar to construct an appropriate millisecond
     * value for the Timestamp, if the underlying database doesn't store
     * timezone information.
     *
     * @param columnIndex The first column is 1, the second is 2, and so on.
     * @param cal The calendar to use in constructing the date. Ignored if cal is NULL.
     * @return The column value. If the value is SQL NULL, then the result is null.
     * @exception SQLException This exception is thrown if the column value is not valid or
     * the SQL type cannot be converted to a valid java.sql.Timestamp value.
     */
    public synchronized java.sql.Timestamp getTimestamp(
        int columnIndex,
        Calendar cal)
        throws SQLException
    {
        throwGenericExceptions(columnIndex);
        Object o = m_rowCache.getData(columnIndex);
        int type = m_rowCache.getColumnType(columnIndex);
        if (!((type == Types.DATE)
            || (type == Types.VARCHAR)
            || (type == Types.TIMESTAMP)))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_cannotConvertToTimestamp);
        }
        if ((o == null) || (m_rowCache.getIsMissing(columnIndex) == true))
        {
            m_wasNull = true;
            return null;
        }
        else
            m_wasNull = false;
        /**
         * damack - ditto the date/time deal for timestamp.
         */
        if (type == Types.VARCHAR)
        {
            String temp = o.toString().trim();
            try
            {
                if (cal == null)
                    return java.sql.Timestamp.valueOf(temp);
                else
                {
                    java.sql.Timestamp tempTS = java.sql.Timestamp.valueOf(temp);
                    return new Timestamp(tempTS.getTime() + cal.getTimeZone().getOffset(tempTS.getTime()));
                }
            }
            catch (NumberFormatException e)
            {
                MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAResultSet_numberFormatError,
                    null,
                    e);
            }
        }
        else
        {
            double value = Double.parseDouble(o.toString());
            long javadate;
            if (cal == null) cal = Calendar.getInstance();
            if (type == Types.DATE)
            {
                javadate = calcJavaDate(value, true, false, cal);
                java.sql.Date tdate =
                    (java.sql.Date) toSQLType(javadate, true, false);
                return (new java.sql.Timestamp(tdate.getTime()));
            }
            else
            {
                javadate = calcJavaDate(value, true, true, cal);
                return ((java.sql.Timestamp) toSQLType(javadate, true, true));
            }
        }
        return null;
 
    }
    /**
     * Gets the value of a column in the current row as a java.sql.Timestamp
     * object. Use the calendar to construct an appropriate millisecond
     * value for the Timestamp, if the underlying database doesn't store
     * timezone information.
     *
     * @param columnName The name of the column that is returned by the ResultSetMetaData object.
     * @param cal The calendar to use in constructing the date. Ignored if cal is NULL.
     * @return The column value. If the value is SQL NULL, then the result is NULL.
     * @exception SQLException This exception is thrown if the column value is not valid or the
     * SQL type cannot be converted into a valid java.sql.Timestamp value.
     */
    public synchronized java.sql.Timestamp getTimestamp(
        String columnName,
        Calendar cal)
        throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        int columnIndex = findColumn(columnName);
        return (getTimestamp(columnIndex, cal));
    }
/*************************************************************************************/
    /**
     * Calculate the milliseconds from sas double.
     *
     * @return The java date/time/timestamp value in milliseconds.
     */
    private long calcJavaDate(double value, boolean date, boolean time)
    {
        double sasdate = value;
        long javadate = 0;
        if (date && time)
        {
            sasdate = sasdate * 1000.0 - JAVA_DATETIME_EPOCH;
            //sas returns no. of seconds
        }
        else if (date)
        {
            sasdate = (sasdate * MAX_SASTIME * 1000.0) - JAVA_DATETIME_EPOCH;
            //sas returns no. of days
        }
        else if (time)
        {
            sasdate = sasdate * 1000.0 - JAVA_DATETIME_EPOCH;
            //sas returns no. of seconds
        }
        Calendar tcal = Calendar.getInstance();
        tcal.set(1970, 0, 1, 0, 0, 0);
        tcal.set(Calendar.MILLISECOND, 0);
        javadate = (long) (Math.floor(sasdate + tcal.getTime().getTime()));
        //adjust to GMT
        Timestamp tstamp = new Timestamp(javadate);
        Calendar cal = Calendar.getInstance();
        if (cal.getTimeZone().inDaylightTime(tstamp))
        {
            javadate -= MSECS_PER_HOUR; // subtract one hour's worth of milliseconds.
        }
        return javadate;
    }
    private long calcJavaDate(double value, boolean date, boolean time, Calendar cal)
    {
        double sasdate = value;
        long javadate = 0;
        if (date && time)
        {
            sasdate = sasdate * 1000.0 - JAVA_DATETIME_EPOCH;
            //sas returns no. of seconds
        }
        else if (date)
        {
            sasdate = (sasdate * MAX_SASTIME * 1000.0) - JAVA_DATETIME_EPOCH;
            //sas returns no. of days
        }
        else if (time)
        {
            sasdate = sasdate * 1000.0 - JAVA_DATETIME_EPOCH;
            //sas returns no. of seconds
        }
//        Calendar tcal = Calendar.getInstance();
        cal.set(1970, 0, 1, 0, 0, 0);
        cal.set(Calendar.MILLISECOND, 0);
        javadate = (long) (Math.floor(sasdate + cal.getTime().getTime()));
        //adjust to GMT
        Timestamp tstamp = new Timestamp(javadate);
        //Calendar cal = Calendar.getInstance();
        if (cal.getTimeZone().inDaylightTime(tstamp))
        {
            javadate -= MSECS_PER_HOUR; // subtract one hour's worth of milliseconds.
        }
        return javadate;
    }
    /**
     * Convert date value to a string representation of its SQL type.
     *
     * @return A string value of field, NULL if the contents is a missing value.
     */
    private String toSQLString(
        long javadate,
        boolean date,
        boolean time,
        int len)
    {
        String dateString = null;
        if (date && time)
        {
            Timestamp tstamp = new Timestamp(javadate);
            dateString = tstamp.toString();
        }
        else if (date)
        {
            Date tdate = new Date(javadate);
            dateString = tdate.toString();
        }
        else
        {
            Time ttime = new Time(javadate);
            dateString = ttime.toString();
        }
        if (dateString.length() > len)
        {
            dateString = dateString.substring(0, len);
        }
        return dateString;
    }
    /**
     * Obtain date value to the SQL type associated with this object.
     *
     * @return Default Java representation of the object's SQL type.
     */
    private Object toSQLType(long javadate, boolean date, boolean time)
    {
        if (date && time)
        {
            return ((Object) new Timestamp(javadate));
        }
        else if (date)
        {
            return ((Object) new Date(javadate));
        }
        else
        {
            return ((Object) new Time(javadate));
        }
    }

    /*
     * Make sure data is consistent.
     */
    /////////////////////////////////////////////////////////////////////////////////////////
    //
    //                          MITCHEL'S OPERATIONS
    //
    //
    // Initially I tried to have a simply model where we knew the size of the result set.
    // However, that cabability doesn't seem to be present for SQLRowsets, plus if the
    // ResultSet is transparent, the dataset could be changing so the size could be constantly
    // changing. Plus JDBC doesn't provide a "RowCount" method, so I'm switching to a different
    // paradigm.
    //
    // In the new approach I use negative values to signify a conceptual position of the
    // cursor m_currentRowIndex, since the ResultSet could be any size, but will never be
    // negatively indexed.
    /////////////////////////////////////////////////////////////////////////////////////////
    //NON-CACHED FETCHING
    //0 to N-1. Special values include BEFORE_FIRST and AFTER_LAST
    int m_currentRowIndex;
    int m_numRows = 0;
    /*
     *     protected void NullifyRow()
    {
    currentRow = null;
    saveRow = null;
    }
    */
    /**
     * Calculate sas double value of Java Date.
     */
    private double dateToDouble(java.sql.Date x)
    {
        /*
         * S0217898: it appears that adjusting the time value
         * from java to SAS and then adjusting for timezone is a bad idea,
         * since this leads to an erroneous time from which to determine the
         * timezone.  Fix is to adjust for timezone and THEN tack on the adjustment
         * for SAS' epoch.
         */
        double value = x.getTime();// + JAVA_DATETIME_EPOCH;
        value = adjustForTimeZone(value);
        value += JAVA_DATETIME_EPOCH;
        return value / (1000 * MAX_SASTIME);
    }
    /**
     * calculate sas double value of Java Timestamp.
     * The value returned is in seconds.
     */
    /*
     * S0211519: this was found by Karl Moss.  
     * SUNs Javadoc erroneously states that getNanos()/1000000 should be
     * added to the getTime() value.  This is already done in SUN's 
     * Timestamp.getTime() method.  
     */
    private double timestampToDouble(java.sql.Timestamp x)
    {
        /*
         * S0217898: it appears that adjusting the time value
         * from java to SAS and then adjusting for timezone is a bad idea,
         * since this leads to an erroneous time from which to determine the
         * timezone.  Fix is to adjust for timezone and THEN tack on the adjustment
         * for SAS' epoch.
         */
        double value = x.getTime();// + JAVA_DATETIME_EPOCH;
            
        value = adjustForTimeZone(value);
        value += JAVA_DATETIME_EPOCH;
        return value / 1000;
    }
    /**
     * Calculate sas double value of Java Time.
     * The value returned is in seconds.
     */
    private double timeToDouble(java.sql.Time x)
    {
        double value = x.getTime();
        value = adjustForTimeZone(value);
        return value / 1000;
    }
    /**
     * Adjust for TimeZone (GMT) and DST. Used for update.
     */
    private double adjustForTimeZone(double value)
    {
        Calendar tcal = Calendar.getInstance();
        
        tcal.set(1970, 0, 1, 0, 0, 0);
        tcal.set(Calendar.MILLISECOND, 0);
         
        value = (long) (Math.floor(value - tcal.getTime().getTime()));
        //adjust from GMT
        Timestamp tstamp = new Timestamp((long) value);       
        
        
        Calendar cal = Calendar.getInstance();
        if (cal.getTimeZone().inDaylightTime(tstamp))
        {
            value += MSECS_PER_HOUR; // add on an hour's worth of milliseconds
        }
        return value;
    }
    /*
     * damack - private functions that i find to be useful.
     */
    private int parseBindKey(String bK)
    {
        if (bK.equals("true") || bK.equals("True"))
            return BindKeys._BindKeysFormatted;
        else if (bK.equals("false") || bK.equals("False"))
            return BindKeys._BindKeysUnformatted;
        else // default value
            return BindKeys._BindKeysUnformatted;
    }
    private void throwGenericExceptions(int columnIndex) throws SQLException
    {
        if (m_isClosed)
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_notOpenError);
        if (m_rowCache.getAfterLast() || m_rowCache.getBeforeFirst())
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSet_invalidRowPosition);
        if ((columnIndex < 1) || (columnIndex > m_rowCache.getColumnCount()))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
    }
    protected MVARowCache getCache()
    {
        return m_rowCache;
    }
    /**
     * Retrieves the value of the designated column in the current row of this ResultSet object as a java.net.URL  object in the Java programming language.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
    public java.net.URL getURL(int columnIndex) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * Retrieves the value of the designated column in the current row of this ResultSet object as a java.net.URL  object in the Java programming language.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public java.net.URL getURL(String columnName) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
        return null;
    }
    /**
     * Updates the designated column with a java.sql.Array value.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateArray(int columnIndex, Array x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
    /**
     * Updates the designated column with a java.sql.Array value.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateArray(String columnName, Array x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
    /**
     * Updates the designated column with a java.sql.Blob value.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateBlob(int columnIndex, Blob x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
    /**
     * Updates the designated column with a java.sql.Blob value.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateBlob(String columnName, Blob x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
    /**
     * Updates the designated column with a java.sql.Clob value.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateClob(int columnIndex, Clob x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
    /**
     * Updates the designated column with a java.sql.Clob value.
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateClob(String columnName, Clob x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
    /**
     * Updates the designated column with a java.sql.Ref value
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateRef(int columnIndex, Ref x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
    /**
     * Updates the designated column with a java.sql.Ref value
    * Empty method implementation for JDK1.4 compliance.
    * @exception SQLException Always thrown, to indicate that this method is not implemented.
    */
     public void updateRef(String columnName, Ref x) throws SQLException
    {
        MVALocalizedError.throwLocalizedException(
            VendorCode.General_methodNotImplemented);
    }
}
