/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 * 07Feb2003 damack S0183030: added constructor that takes a column count.
 *                            used by MVAModifiedResultSet.
 */
package com.sas.rio;
import java.sql.SQLException;
import java.sql.Types;
import java.sql.ResultSetMetaData;
import com.sas.iom.SAS.IDataSet;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Date;
import com.sas.iom.SAS.IDataSetPackage.BindKeys;
/**
 * A ResultSetMetaData object can be used to find out about the types
 * and properties of the columns in a ResultSet.
 */
public class MVAResultSetMetaData implements RIOResultSetMetaDataInterface
{
    String m_errorMsg = null;
    private int m_numberColumns = 0;
    private int maxFieldSize = 32767;
    private int vendorCode;
    private String messageIndex;
    protected MVAResultSet rs = null;
    private IDataSet sasDataSet = null;
    private String[] m_columnNames = null;
    private int[] m_columnLengths = null;
    private int[] m_columnTypes = null;
    // private int[] columnSubTypes = null;
    private int[] m_columnSqlTypes = null;
    private String[] m_columnLabels = null;
    private String[] m_formatNames = null;
    private short[] m_formatWidths = null;
    private short[] m_formatDecimals = null;
    private short[] m_formatLengths = null;
    private String[] m_informatNames = null;
    private short[] m_informatWidths = null;
    private short[] m_informatDecimals = null;
    private int bindKey = BindKeys._BindKeysUnformatted;
    /*
     * Constructs an MVAResultSetMetaData object.
     * @param stmt The MVAResultSet object.
     * @exception SQLException This exception is thrown if an MVAException is detected.
     */
    public MVAResultSetMetaData(MVAResultSet rs) throws SQLException
    {
        this(rs, null);
    }
    public MVAResultSetMetaData(MVAResultSet rs, int cc) throws SQLException
    {
        if (rs == null)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSetMetaData_noResultSetError);
        }
        MVARowCache mrc = rs.getCache();
        int i = 0;
        this.rs = rs;
        this.maxFieldSize = rs.maxFieldSize;
        m_numberColumns = cc;
        m_columnNames = new String[m_numberColumns];
        this.m_columnTypes = new int[m_numberColumns];
        this.m_columnLengths = new int[m_numberColumns];
        this.m_columnLabels = new String[m_numberColumns];
        this.m_formatNames = new String[m_numberColumns];
        this.m_formatWidths = new short[m_numberColumns];
        this.m_formatDecimals = new short[m_numberColumns];
        this.m_formatLengths = new short[m_numberColumns];
        this.m_informatNames = new String[m_numberColumns];
        this.m_informatWidths = new short[m_numberColumns];
        this.m_informatDecimals = new short[m_numberColumns];
        this.m_columnSqlTypes = new int[m_numberColumns];
        for (i = 0; i < m_numberColumns; i++)
        {
            m_columnNames[i] = mrc.getColumnName(i + 1);
            this.m_columnTypes[i] = mrc.getColumnType(i + 1);
            this.m_formatNames[i] = mrc.getColumnFormat(i + 1);
            this.m_formatWidths[i] = mrc.getFormatWidth(i + 1);
            this.m_formatDecimals[i] = mrc.getFormatDecimal(i + 1);
            this.m_formatLengths[i] = mrc.getFormatLength(i + 1);
            this.m_informatNames[i] = mrc.getColumnInformat(i + 1);
            this.m_informatWidths[i] = mrc.getInformatWidth(i + 1);
            this.m_informatDecimals[i] = mrc.getInformatDecimal(i + 1);
            this.m_columnSqlTypes[i] = mrc.getColumnType(i + 1);
            this.m_columnLabels[i] = mrc.getColumnLabel(i + 1);
            this.m_columnLengths[i] = mrc.getColumnLength(i + 1);
        }
    
    
        this.bindKey = rs.getBindKey();
    }
    /*
     * Constructs an MVAResultSetMetaData object.
     * @param rs an MVAResultSet object.
     * @param columnSqlTypes an int array of SQLTypes used for modified
     * MVAResultSets generated as a result of some MVADatabaseMetaData
     * methods.
     * @exceptihon SQLException This exception is thrown if an error condition is detected.
     */
    public MVAResultSetMetaData(MVAResultSet rs, int[] columnSqlTypes)
        throws SQLException
    {
        if (rs == null)
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.MVAResultSetMetaData_noResultSetError);
        }
        MVARowCache mrc = rs.getCache();
        int i = 0;
        this.rs = rs;
        this.maxFieldSize = rs.maxFieldSize;
        m_numberColumns = mrc.getColumnCount();
        m_columnNames = new String[m_numberColumns];
        this.m_columnTypes = new int[m_numberColumns];
        this.m_columnLengths = new int[m_numberColumns];
        this.m_columnLabels = new String[m_numberColumns];
        this.m_formatNames = new String[m_numberColumns];
        this.m_formatWidths = new short[m_numberColumns];
        this.m_formatDecimals = new short[m_numberColumns];
        this.m_formatLengths = new short[m_numberColumns];
        this.m_informatNames = new String[m_numberColumns];
        this.m_informatWidths = new short[m_numberColumns];
        this.m_informatDecimals = new short[m_numberColumns];
        this.m_columnSqlTypes = new int[m_numberColumns];
        for (i = 0; i < m_numberColumns; i++)
        {
            m_columnNames[i] = mrc.getColumnName(i + 1);
            this.m_columnTypes[i] = mrc.getColumnType(i + 1);
            this.m_formatNames[i] = mrc.getColumnFormat(i + 1);
            this.m_formatWidths[i] = mrc.getFormatWidth(i + 1);
            this.m_formatDecimals[i] = mrc.getFormatDecimal(i + 1);
            this.m_formatLengths[i] = mrc.getFormatLength(i + 1);
            this.m_informatNames[i] = mrc.getColumnInformat(i + 1);
            this.m_informatWidths[i] = mrc.getInformatWidth(i + 1);
            this.m_informatDecimals[i] = mrc.getInformatDecimal(i + 1);
            this.m_columnSqlTypes[i] = mrc.getColumnType(i + 1);
            this.m_columnLabels[i] = mrc.getColumnLabel(i + 1);
            this.m_columnLengths[i] = mrc.getColumnLength(i + 1);
        }
        /**
           if(columnSqlTypes != null)
           {
           this.columnSqlTypes = columnSqlTypes;
           }
           
           this.columnSubTypes = rs.columnSubTypes;
        */
        /** TODO resolve the bind key issues.
        this.bindKey = rs.bindKey;
        */
        this.bindKey = rs.getBindKey();
    }
    /**
     * Gets the number of columns in the ResultSet.
     * @return Number of columns.
     * @exception SQLException This exception is required by the interface, but it is never thrown.
     */
    public int getColumnCount() throws SQLException
    {
        return m_numberColumns;
    }
    /**
     * Determines whether a column is automatically numbered.
     * @param column The column index.
     * @return False.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isAutoIncrement(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return false;
    }
    /**
     * Determines whether case matters in a column.
     * @param column The column index.
     * @return True if the java.sql.Type is VARCHAR; otherwise False.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isCaseSensitive(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        if (m_columnSqlTypes[column - 1] == Types.VARCHAR)
            return true;
        else
            return false;
    }
    /**
     * Determines whether a column can be used in a WHERE clause.
     * @param column The column index.
     * @return True.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isSearchable(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return true;
    }
    /**
     * Determines whether a column is a cash value.
     * @param column The column index.
     * @return False.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isCurrency(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return false;
    }
    /**
     * Determines whether you can put a NULL in this column.
     * @param column The column index.
     * @return ResultSetMetaData.columnNoNulls or ResultSetMetaData.columnNullable.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public int isNullable(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        if (m_columnSqlTypes[column - 1] == Types.VARCHAR)
            return ResultSetMetaData.columnNoNulls;
        else
            return ResultSetMetaData.columnNullable;
    }
    /**
     * Determines whether the value in a column is a signed number.
     * @param column The column index.
     * @return False if java.sql.Type.VARCHAR or BIT; otherwise True.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isSigned(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        if ((m_columnSqlTypes[column - 1] == Types.VARCHAR)
            || (m_columnSqlTypes[column - 1] == Types.BIT))
            return false;
        else
            return true;
    }
    /**
     * Gets the normal maximum width of the column (in characters).
     * @param column The column index.
     * @return Maximum width of the column.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public int getColumnDisplaySize(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        switch (m_columnSqlTypes[column - 1])
        {
            case Types.CHAR :
            case Types.VARCHAR :
                if (m_columnLengths[column - 1] > maxFieldSize)
                    return maxFieldSize;
                else
                    return m_columnLengths[column - 1];
            case Types.BIT :
                return 1;
            case Types.TINYINT :
                return 4;
            case Types.SMALLINT :
                return 6;
            case Types.INTEGER :
                return 11;
            case Types.BIGINT :
                return 209;
            case Types.REAL :
                return 13;
            case Types.FLOAT :
            case Types.DOUBLE :
                return 22;
            case Types.DATE :
                return 10;
            case Types.TIME :
                return 8;
            case Types.TIMESTAMP :
                return 21;
            default :
                return 0;
        }
    }
    /**
     * Gets the suggested column title for use in printouts and displays.
     * @param column The column index.
     * @return The column label.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public String getColumnLabel(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return m_columnLabels[column - 1];
    }
    /**
     * Gets the column name.
     * @param column The column index.
     * @return The column name.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public String getColumnName(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return m_columnNames[column - 1];
    }
    /**
     * Gets the schema name for the table that the column is in.
     * SQL pass-thru does not surface any information about which underlying
     * table/tables a particular ResultSet is derived from.
     * @param column The column index.
     * @return An empty string (""). The information is not available.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public String getSchemaName(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return "";
    }
    /**
     * Gets the maximum number of decimal digits that can be in the column.
     * @param column The column index.
     * @return For numeric and date columns, the maximum number of decimal digits; for character columns,
     * the maximum length.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public int getPrecision(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        switch (m_columnSqlTypes[column - 1])
        {
            case Types.CHAR :
            case Types.VARCHAR :
                return m_columnLengths[column - 1];
            case Types.BIT :
                return 1;
            case Types.TINYINT :
                return 3;
            case Types.SMALLINT :
                return 5;
            case Types.INTEGER :
                return 10;
            case Types.BIGINT :
                return 19;
            case Types.REAL :
                return 7;
            case Types.FLOAT :
            case Types.DOUBLE :
                return 15;
            case Types.DATE :
                return 10;
            case Types.TIME :
                return 8;
            case Types.TIMESTAMP :
                return 21;
            default :
                return 0;
        }
    }
    /**
     * Gets the number of digits in the column that are to right of the decimal.
     * @param column The column index.
     * @return 0. The information is not available.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public int getScale(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return 0;
    }
    /**
     * Gets the number of digits in the column that are to right of the decimal.
     * NOTE SQL pass-thru does not surface any information about which underlying
     * table/tables a particular ResultSet was derived from.
     * @param column The column index.
     * @return 0. The information is not available.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public String getTableName(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        /**
        if(rs.tableName == null)
            return "";
        else
            return rs.tableName;
        */
        return "";
    }
    /**
     * Gets the catalog name of the table that the column is in.
     * @param column The column index.
     * @return An empty string (""). The information is not available.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public String getCatalogName(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return "";
    }
    /**
     * Gets the SQL type for the column.
     * @param column The column index.
     * @return java.sql.Type for the column.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     * @see Types
     */
    public int getColumnType(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        if (bindKey != BindKeys._BindKeysFormatted)
            return m_columnSqlTypes[column - 1];
        else
            return java.sql.Types.VARCHAR;
        //return m_columnTypes[column - 1];
    }
    /**
     * Gets the data-source-specific type name for the column.
     * @param column The column index.
     * @return String The format name of the column including format width and decimal.
     * An empty string("") is returned if no formats are defined for the column.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public String getColumnTypeName(int column) throws SQLException
    {
        /**
         * NOTE: There is some question as to whether the return value of this method should be the
         * actual SAS base type or the format. The existing code is returning the format, but that
         * may need to change in the future.
         */
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        String formatName = m_formatNames[column - 1].trim();
        if (m_formatWidths[column - 1] != 0)
        {
            formatName += m_formatWidths[column - 1];
        }
        if (!formatName.equals(""))
        {
            formatName += ".";
        }
        if (m_formatDecimals[column - 1] != 0)
        {
            formatName += m_formatDecimals[column - 1];
        }
        return formatName;
    }
    /**
     * Determines whether a column is read-only.
     * @param column The column index.
     * @return False.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isReadOnly(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return false;
    }
    /**
     * Determines whether it is at all possible to successfully write to the column.
     * @param column The column index.
     * @return True.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isWritable(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return true;
    }
    /**
     * Determines whether you can successfully write to the column.
     * @param column The column index.
     * @return False.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    public boolean isDefinitelyWritable(int column) throws SQLException
    {
        if ((column < 1) || (column > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        return false;
    }
    //--------------------------JDBC 2.0-----------------------------------
    /**
     * JDBC 2.0
     *
     * Gets the fully qualified name of the Java class whose instances are
     * manufactured if ResultSet.getObject() is called to retrieve a value
     * from the column. ResultSet.getObject() may return a subClass of the
     * class returned by this method.
     * @param column The column index.
     * @return String version of the class name.
     * @exception SQLException This exception is thrown if the value for column is not valid.
     */
    /*The following code is a cut and paste variation of MVAResultSet.getObject.
    One of those infamous, get it done now things. However, in general, it seems that
    it might be a good idea to have a generalize type mapping class to map between
    SAS and SQL types.
    Functions to consider are
      MVAResultSet.getColumnInformation
      MVAResultSet.toSQLType
      All MVAResultSet get operations
    We could hardcode the expected return value, but I feel that getting the
    name from the class is more robust and more inline with Java style.
    */
    public String getColumnClassName(int columnIndex) throws SQLException
    {
        Object columnObject;
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedException(
                VendorCode.General_invalidColumnValue);
        }
        if (bindKey == BindKeys._BindKeysFormatted)
            columnObject = new String("");
        else if (m_columnTypes[columnIndex - 1] != Types.VARCHAR)
        {
            switch (m_columnTypes[columnIndex - 1])
            {
                case Types.DATE :
                    columnObject = new Date(0);
                    break;
                case Types.TIME :
                    columnObject = new Time(0);
                    break;
                case Types.TIMESTAMP :
                    columnObject = new Timestamp(0);
                    break;
                case Types.BIT :
                    columnObject = new Boolean(true);
                    break;
                case Types.INTEGER :
                case Types.SMALLINT :
                    columnObject = new Integer(0);
                    break;
                default :
                    columnObject = new Double(0.0);
                    break;
            }
        }
        //Assume its a VARCHAR
        //else if(type == BaseType._BaseTypeCharacter) {
        else
        {
            columnObject = new String("");
        }
        return columnObject.getClass().getName();
    }
    /**
     * RIO
     *
     * Gets the format name of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return A String containing the format name of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public String getColumnFormatName(int columnIndex) throws RIOException
    {
        // RIO??
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        return m_formatNames[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the format names of the columns in the ResultSet.
     *
     * @return An array of Strings containing the format names of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public String[] getColumnFormatNames() throws RIOException
    {
        return m_formatNames;
    }
    /**
     * RIO
     *
     * Gets the format length of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the format length of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public int getColumnFormatLength(int columnIndex) throws RIOException
    {
        // RIO??
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        return m_formatLengths[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the format lengths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the format lengths of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public int[] getColumnFormatLengths() throws RIOException
    {
        int[] tFormatLengths = null;
        for (int i = 0; i < m_formatLengths.length; i++)
            tFormatLengths[i] = m_formatLengths[i];
        return tFormatLengths;
    }
    /**
     * RIO
     *
     * Gets the format width of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the format width of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public int getColumnFormatWidth(int columnIndex) throws RIOException
    {
        // RIO??
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        return m_formatWidths[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the format widths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the format widths of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public int[] getColumnFormatWidths() throws RIOException
    {
        int[] tFormatWidths = null;
        for (int i = 0; i < m_formatWidths.length; i++)
            tFormatWidths[i] = m_formatWidths[i];
        return tFormatWidths;
    }
    /**
     * RIO
     *
     * Gets the format decimal of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the format decimal of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public int getColumnFormatDecimal(int columnIndex) throws RIOException
    {
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            // RIO??
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        return m_formatDecimals[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the format lengths of the columns in the ResultSet.
     *
     * @return An array of int values containing the format lengths of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public int[] getColumnFormatDecimals() throws RIOException
    {
        int[] tFormatDecimals = null;
        for (int i = 0; i < m_formatDecimals.length; i++)
            tFormatDecimals[i] = m_formatDecimals[i];
        return tFormatDecimals;
    }
    /**
     * RIO
     *
     * Gets the informat name of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return A String containing the informat name of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public String getColumnInformatName(int columnIndex) throws RIOException
    {
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            // RIO??
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        return m_informatNames[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the informat names of the columns in the ResultSet.
     *
     * @return An array of String values containing the informat names of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public String[] getColumnInformatNames() throws RIOException
    {
        return m_informatNames;
    }
    /**
     * RIO
     *
     * Gets the informat length of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the informat length of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public int getColumnInformatLength(int columnIndex) throws RIOException
    {
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            // RIO??
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        //TODO: informat length is not returned.
        return m_formatLengths[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the informat lengths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the informat lengths of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public int[] getColumnInformatLengths() throws RIOException
    {
        //TODO: informat length is not returned.
        int[] tInformatLengths = null;
        for (int i = 0; i < m_formatLengths.length; i++)
            tInformatLengths[i] = m_formatLengths[i];
        return tInformatLengths;
    }
    /**
     * RIO
     *
     * Gets the informat width of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the informat width of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public int getColumnInformatWidth(int columnIndex) throws RIOException
    {
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            // RIO??
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        return m_informatWidths[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the informat widths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the informat widths of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public int[] getColumnInformatWidths() throws RIOException
    {
        int[] tInformatWidths = null;
        for (int i = 0; i < m_informatWidths.length; i++)
            tInformatWidths[i] = m_informatWidths[i];
        return tInformatWidths;
    }
    /**
     * RIO
     *
     * Gets the informat decimal of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the informat decimal of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    public int getColumnInformatDecimal(int columnIndex) throws RIOException
    {
        if ((columnIndex < 1) || (columnIndex > m_numberColumns))
        {
            // RIO??
            MVALocalizedError.throwLocalizedRIOException(
                VendorCode.General_invalidColumnValue);
        }
        return m_informatDecimals[columnIndex - 1];
    }
    /**
     * RIO
     *
     * Gets the informat decimals of the columns in the ResultSet.
     *
     * @return An array of integer values containing the informat decimals of the columns.
     * @exception RIOException This exception is required by the interface but it is never thrown.
     */
    public int[] getColumnInformatDecimals() throws RIOException
    {
        int[] tInformatDecimals = null;
        for (int i = 0; i < m_informatDecimals.length; i++)
            tInformatDecimals[i] = m_informatDecimals[i];
        return tInformatDecimals;
    }
}
