/* $Id$ */
/* Copyright (c)1999 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */

package com.sas.rio;

import java.util.Hashtable;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import java.sql.Types;

/*
 *
 * @MVASqlTypesResultSet implements java.sql.ResultSet implements
 * <P>This class creates an artificial result set for
 * DatabaseMetaData.getTypeInfo()
 *
 */
//
//History Records:
//
// Date          Name                Remarks
// 08/20/99  M Srinivasn    - Created.
// 08/24/99  M Srinivasan   - S0064453 - Need artificial resultset for dbmd.getTypeInfo.
// 10/25/99  M Srinivasan   - S0066781 - Fix to artificial resultset so that rs.getBoolean() will work.
// 08/02/05  D Mackie       - S0308479 - uppercased the column names so that we're mimicing the behavior of the 
//                                       regular cache's init method (which uppercases the column names)
//                                       also got rid of build warnings.
//
  public class MVASqlTypesResultSet extends MVAResultSet {
    private int currentRowNumber = 0;
    private int rowsFetched = 0;
    private boolean maxReached = false;
    private boolean eof = false;
    private boolean bof = false;
    private int maxRows = 0;
    private String cursorName;
    private Hashtable namesToColumns;

    /**
     * Construct an artificial ResultSet; one not generated through
     * communication with the MVA server.
     * @exception SQLException if a RIOException was detected
     */
    public MVASqlTypesResultSet() throws SQLException {
        super();
       	m_columnCount = 18;
        m_rowCache = new MVARowCache(18);
        createVars();
        createValues();
       // super.currentRow = null;
        //super.cursorName = "";
        //missingValuesCurrentRow = new boolean[columnCount];
        //for(int i=0;i<columnCount;i++) {
        //    missingValuesCurrentRow[i] = false;
       // }
        //String names[] = new String[18];
        //super.namesToColumns = new Hashtable();
        //for(int i=0;i<columnNames.length; i++) {
         //   super.namesToColumns.put(columnNames[i].trim().toUpperCase(), new Integer(i+1));
        //}
    }
    /**
     * Proceed to the next row in the result set.
     * @return true if the next row is valid, false if not
     * @exception SQLException never really thrown, required by interface
     */
    public synchronized boolean next() throws SQLException
    {
      	return m_rowCache.next();
    }
    /**
     * Closes the result set.
     * @exception SQLException never really thrown, required by interface
     */
    public synchronized void close() throws SQLException
    {
    }
    /**
     * Create artificial variables list
     *
     */
    private void createVars() {
    	
	
        m_rowCache.setColumnName(1,"TYPE_NAME");
        m_rowCache.setColumnName(2,"DATA_TYPE");
        m_rowCache.setColumnName(3,"PRECISION");
        m_rowCache.setColumnName(4, "LITERAL_PREFIX");
	m_rowCache.setColumnName(5, "LITERAL_SUFFIX");
	m_rowCache.setColumnName(6, "CREATE_PARAMS");
	m_rowCache.setColumnName(7, "NULLABLE");
        m_rowCache.setColumnName(8, "CASE_SENSITIVE");
	m_rowCache.setColumnName(9, "SEARCHABLE");
	m_rowCache.setColumnName(10, "UNSIGNED_ATTRIBUTE");
        m_rowCache.setColumnName(11, "FIXED_PREC_SCALE");
	m_rowCache.setColumnName(12, "AUTO_INCREMENT");
	m_rowCache.setColumnName(13, "LOCAL_TYPE_NAME");
	m_rowCache.setColumnName(14, "MINIMUM_SCALE");
	m_rowCache.setColumnName(15, "MAXIMUM_SCALE");
	m_rowCache.setColumnName(16, "SQL_DATA_TYPE");
	m_rowCache.setColumnName(17, "SQL_DATETIME_SUB");
	m_rowCache.setColumnName(18, "NUM_PREC_RADIX");
	

	
        m_rowCache.setColumnType(1, Types.VARCHAR);
        m_rowCache.setColumnType(2, Types.SMALLINT);
        m_rowCache.setColumnType(3, Types.INTEGER);
	m_rowCache.setColumnType(4, Types.VARCHAR);
        m_rowCache.setColumnType(5, Types.VARCHAR);
        m_rowCache.setColumnType(6, Types.VARCHAR);
	m_rowCache.setColumnType(7, Types.SMALLINT);
	m_rowCache.setColumnType(8, Types.BIT);
	m_rowCache.setColumnType(9, Types.SMALLINT);
	m_rowCache.setColumnType(10, Types.BIT);
	m_rowCache.setColumnType(11, Types.BIT);
	m_rowCache.setColumnType(12, Types.BIT);
	m_rowCache.setColumnType(13, Types.VARCHAR);
	m_rowCache.setColumnType(14, Types.SMALLINT);
	m_rowCache.setColumnType(15, Types.SMALLINT);
	m_rowCache.setColumnType(16, Types.INTEGER);
	m_rowCache.setColumnType(17, Types.INTEGER);
	m_rowCache.setColumnType(18, Types.INTEGER);


        m_rowCache.setColumnLength(1, 10);
        m_rowCache.setColumnLength(2, 8);
        m_rowCache.setColumnLength(3, 8);
	m_rowCache.setColumnLength(4, 1);
        m_rowCache.setColumnLength(5, 1);
        m_rowCache.setColumnLength(6, 1);
	m_rowCache.setColumnLength(7, 8);
	m_rowCache.setColumnLength(8, 8);
	m_rowCache.setColumnLength(9, 8);
	m_rowCache.setColumnLength(10, 8);
	m_rowCache.setColumnLength(11, 8);
	m_rowCache.setColumnLength(12, 8);
	m_rowCache.setColumnLength(13, 5);
	m_rowCache.setColumnLength(14, 8);
	m_rowCache.setColumnLength(15, 8);
	m_rowCache.setColumnLength(16, 8);
	m_rowCache.setColumnLength(17, 8);
	m_rowCache.setColumnLength(18, 8);
	
        m_rowCache.setColumnLabel(1, "SQL Type Name");
        m_rowCache.setColumnLabel(2, "SQL Type");
        m_rowCache.setColumnLabel(3, "Precision");
	m_rowCache.setColumnLabel(4, "Prefix");
        m_rowCache.setColumnLabel(5, "Suffix");
        m_rowCache.setColumnLabel(6, "Parameters");
	m_rowCache.setColumnLabel(7, "Nullable");
	m_rowCache.setColumnLabel(8, "Case Sensitive");
	m_rowCache.setColumnLabel(9, "Where");
	m_rowCache.setColumnLabel(10, "Unsigned");
	m_rowCache.setColumnLabel(11, "Money");
	m_rowCache.setColumnLabel(12, "Auto-increment");
	m_rowCache.setColumnLabel(13, "Local Version Name");
	m_rowCache.setColumnLabel(14, "Minimum Scale");
	m_rowCache.setColumnLabel(15, "Maximum Scale");
	m_rowCache.setColumnLabel(16, "Data Type");
	m_rowCache.setColumnLabel(17, "DateTime");
	m_rowCache.setColumnLabel(18, "Radix");


	//      sortedBy = new byte[18];
        for(int i=1;i<=18;i++) 
	    {
		m_rowCache.setFormatLength(i, (short)0);
		m_rowCache.setFormatName(i, "");
		m_rowCache.setFormatWidth(i, (short)0);
		m_rowCache.setFormatDecimal(i, (short)0);
		m_rowCache.setInformatName(i, "");
		m_rowCache.setInformatWidth(i, (short)0);
		m_rowCache.setInformatDecimal(i, (short)0);
		//sortedBy[i] = (byte)i;
	    }
    }
    /**
     * Create artificial values list
     *
     */
    private void createValues() {
        Object[][] array = new Object[8][18];

		// set String representation of type
        array[0][0] = "CHAR";
      	array[1][0] = "INTEGER";
    	array[2][0] = "FLOAT";
    	array[3][0] = "DOUBLE PRECISION";
      	array[4][0] = "VARCHAR";
      	array[5][0] = "DATE";
      	array[6][0] = "TIME";
      	array[7][0] = "TIMESTAMP";

		// set numeric representation of type
        /*
		array[0][1] = new Double((double)Types.CHAR);
      	array[1][1] = new Double((double)Types.INTEGER);
      	array[2][1] = new Double((double)Types.FLOAT);
      	array[3][1] = new Double((double)Types.DOUBLE);
      	array[4][1] = new Double((double)Types.VARCHAR);
      	array[5][1] = new Double((double)Types.DATE);
      	array[6][1] = new Double((double)Types.TIME);
      	array[7][1] = new Double((double)Types.TIMESTAMP);
        */
        array[0][1] = new Short((short)Types.CHAR);
      	array[1][1] = new Short((short)Types.INTEGER);
      	array[2][1] = new Short((short)Types.FLOAT);
      	array[3][1] = new Short((short)Types.DOUBLE);
      	array[4][1] = new Short((short)Types.VARCHAR);
      	array[5][1] = new Short((short)Types.DATE);
      	array[6][1] = new Short((short)Types.TIME);
      	array[7][1] = new Short((short)Types.TIMESTAMP);


        //set maximum precision
        array[0][2] = new Integer(32767);
      	array[1][2] = new Integer(10);
      	array[2][2] = new Integer(15);
      	array[3][2] = new Integer(15);
      	array[4][2] = new Integer(32767);
      	array[5][2] = new Integer(10);
      	array[6][2] = new Integer(8);
      	array[7][2] = new Integer(21);

        // set literal prefix, suffix and parameters
        for(int i=0;i<8;i++) {
            array[i][3] = "";
            array[i][4] = "";
            array[i][5] = "";
		}

        // set can you set null
        array[0][6] = new Short((short)DatabaseMetaData.typeNoNulls);
      	array[1][6] = new Short((short)DatabaseMetaData.typeNullable);
      	array[2][6] = new Short((short)DatabaseMetaData.typeNullable);
      	array[3][6] = new Short((short)DatabaseMetaData.typeNullable);
      	array[4][6] = new Short((short)DatabaseMetaData.typeNoNulls);
      	array[5][6] = new Short((short)DatabaseMetaData.typeNullable);
      	array[6][6] = new Short((short)DatabaseMetaData.typeNullable);
      	array[7][6] = new Short((short)DatabaseMetaData.typeNullable);

        // set case sensitive?
        array[0][7] = new Double((double)1);
      	array[1][7] = new Double((double)0);
    	array[2][7] = new Double((double)0);
      	array[3][7] = new Double((double)0);
      	array[4][7] = new Double((double)1);
      	array[5][7] = new Double((double)0);
      	array[6][7] = new Double((double)0);
      	array[7][7] = new Double((double)0);

        // set searchable
        for(int i=0;i<8;i++) {
        	array[i][8] = new Short((short)DatabaseMetaData.typeSearchable);
        }

        // set unsigned
        array[0][9] = new Double((double)1);
      	array[1][9] = new Double((double)0);
    	array[2][9] = new Double((double)0);
      	array[3][9] = new Double((double)0);
      	array[4][9] = new Double((double)1);
      	array[5][9] = new Double((double)0);
      	array[6][9] = new Double((double)0);
      	array[7][9] = new Double((double)0);

        // set money value
        for(int i=0;i<8;i++) {
        	array[i][10] = new Double((double)0);
        }

        // set auto-increment
        for(int i=0;i<8;i++) {
        	array[i][11] = new Double((double)0);
        }

        // set local version of name
        for(int i=0;i<8;i++) {
        	array[i][12] = "";
        }

        // set minimum scale
        for(int i=0;i<8;i++) {
        	array[i][13] = new Short((short) 0);
        }

        // set maximum scale
        for(int i=0;i<8;i++) {
        	array[i][14] = new Short((short) 0);
        }

        // set data type
        for(int i=0;i<8;i++) {
        	array[i][15] = new Integer(0);
        }

        // set datetime
        for(int i=0;i<8;i++) {
        	array[i][16] = new Integer(0);
        }

        // set radix
        for(int i=0;i<8;i++) {
        	array[i][17] = new Integer(10);
        }

	m_rowCache.replaceData(array);
    }

}
