 /* $Id$ */
/*
 * Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513 Unpublished- All
 * Rights Reserved.
 */
package com.sas.rio;

import java.util.Hashtable;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.BatchUpdateException;
import java.util.StringTokenizer;
import java.util.Vector;
import org.omg.CORBA.IntHolder;

//
//History Records:
//
// Date Name Remarks
// 05/04/99 M Srinivasan - Fix to S0052966. Supported multiple librefs in
// Connection
//                           properties by adding libref name to getTable(), createTable(),
//                           and deleteTable() methods of RIOStatementInterface.
// 06/10/99 M Srinivasan - S0056456: Code change to getTable(). Options object
// is replaced with
//                           options string. Column object removed from getTable().
// 07/09/99 M Srinivasan - Fix to createTable. Close the dataset after calling
// createDataSet. This
//                           will eliminate library in use problem while deassigning it later.
// 10/29/99 M Srinivasan -S0068901 : Fix to passing null elements in String[]
// arrays. Brgorb is more stricter and
//                           requires all input parameters to have non-null values.
// 11/07/00 B Beers - S0102084 : Eliminate extraneous assignment in execute().
// 12/03/2001 D Mackie - S0106244 : fixed problem in sasify where "eq ." was
// munged into "eq."
// 02/07/2003 damack - S0183008 : fix issue in execute where a call to getTable
// (returning a resultset)
//                                      got followed by a call the MVAResultSet ctor, blowing away the hash code
//                                      required for cleanup
// 26Apr2004 damack - S0189758 : javadoc.
// 28Apr2004 damack - S0236919: changed execute and executeQuery so that "select
// * from table" uses
//                                     the WORK lib when it goes to getTable (instead of null, which causes
//                                     getTable to fail.
// 13May2004 damack - S0235627: fix sasify so that quoted portions of the select
// stmt are not munged
//                                     for dots.
//
// 25Jul2005 damack - S0308525: add code to close() s.t. connection.statementClosed is 
//                              called to tell the connection that the statement closed..
//                              (perf imp. for DI)
// This class is used when SQL has to be converted to an acceptable SAS format.
class funcEntry
{
    static final int none = 0;

    static final int changeConcat = 1;

    static final int changeLeft = 2;

    static final int changeLTrim = 3;

    static final int changeRepeat = 4;

    static final int changeSpace = 5;

    static final int changePower = 6;

    static final int changeRand = 7;

    static final int changeRound = 8;

    static final int changeDayname = 9;

    static final int changeDayofyear = 10;

    static final int changeMonthname = 11;

    public String sasFn;

    public int specialHandling;

    funcEntry(String sasFn, int specialHandling)
    {
        if (sasFn != null)
            this.sasFn = sasFn;
        else
            this.sasFn = null;
        this.specialHandling = specialHandling;
    }
}
/**
 * <P>
 * A Statement object is used for executing a static SQL statement and obtaining
 * the results produced by it.
 * 
 * <P>
 * Only one ResultSet per Statement can be open at any point in time. Therefore,
 * if the reading of one ResultSet is interleaved with the reading of another,
 * each must have been generated by different Statements. All statement execute
 * methods implicitly close a statment's current ResultSet if an open one
 * exists.
 * 
 * @see Connection#createStatement
 * @see ResultSet
 */

public class MVAStatement implements RIOStatementInterface
{
    private static final int SASIFY_DEFAULT = 0;

    private static final int SASIFY_V91 = 1;

    private static final int SASIFY_NONE = 2;

    private int m_sasifyTableNames;

    private String m_errorMsg = null;

    private static final int d = 1;

    private static final int t = 2;

    private static final int ts = 3;

    private static final int fn = 4;

    private static final int oj = 5;

    private static final int escape = 6;

    protected static String months[] = new String[13];

    private static Hashtable sasfns = new Hashtable();

    protected int maxFieldSize = 32767;

    private int maxRows = 0;

    private int timeout = 0;

    private int updateCount = -1;

    protected MVAResultSet snresultset = null;

    private MVAConnection connection;

    private SQLWarning warnings = null;

    protected boolean describe;

    private int vendorCode;

    private String messageIndex;

    protected int fetchSize = MVAConnection.XHRSQPRECS;

    public int direction = MVAResultSet.FETCH_FORWARD;

    private int resultSetType;

    private int resultSetConcurrency;

    protected String options;

    private Vector m_batchList = new Vector();

    private java.util.Properties info = null;

    // executeQuery parameters
    private boolean randomAccess = false;

    public long bookmarkLength = 0;

    public long status = 0;

    public IntHolder bookmarkLengthHolder = null;

    //    public IDataSet sasDataSet = null;
    //public IDataSetHolder sasDataSetHolder = null;
    public IntHolder statusHolder = null;

    // private IDBMSConnection dbmsConnection = null;
    //protected IDataService sasDataService = null;
    //public ILibref sasLibref = null;
    protected String libref = null;

    private XUtil m_xUtil = null;
    static
    {
        funcEntry entry = new funcEntry("RANK", 0);
        sasfns.put("ASCII", entry);
        sasfns.put("ascii", entry);
        entry = new funcEntry("BYTE", 0);
        sasfns.put("CHAR", entry);
        sasfns.put("char", entry);
        entry = new funcEntry("", funcEntry.changeConcat);
        sasfns.put("CONCAT", entry);
        sasfns.put("concat", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("DIFFERENCE", entry);
        sasfns.put("difference", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("INSERT", entry);
        sasfns.put("insert", entry);
        entry = new funcEntry("LOWCASE", 0);
        sasfns.put("LCASE", entry);
        sasfns.put("lcase", entry);
        entry = new funcEntry("SUBSTR", funcEntry.changeLeft);
        sasfns.put("LEFT", entry);
        sasfns.put("left", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("LENGTH", entry);
        sasfns.put("length", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("LOCATE", entry);
        sasfns.put("locate", entry);
        entry = new funcEntry("TRIM(LEFT", funcEntry.changeLTrim);
        sasfns.put("LTRIM", entry);
        sasfns.put("ltrim", entry);
        entry = new funcEntry("REPEAT", funcEntry.changeRepeat);
        sasfns.put("REPEAT", entry);
        sasfns.put("repeat", entry);
        entry = new funcEntry("TRANWRD", 0);
        sasfns.put("REPLACE", entry);
        sasfns.put("replace", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("RIGHT", entry);
        sasfns.put("right", entry);
        entry = new funcEntry("TRIMN", 0);
        sasfns.put("RTRIM", entry);
        sasfns.put("rtrim", entry);
        entry = new funcEntry("SOUNDEX", 0);
        sasfns.put("SOUNDEX", entry);
        sasfns.put("soundex", entry);
        entry = new funcEntry("REPEAT", funcEntry.changeSpace);
        sasfns.put("SPACE", entry);
        sasfns.put("space", entry);
        entry = new funcEntry("SUBSTR", 0);
        sasfns.put("SUBSTRING", entry);
        sasfns.put("substring", entry);
        entry = new funcEntry("UPCASE", 0);
        sasfns.put("UCASE", entry);
        sasfns.put("ucase", entry);
        entry = new funcEntry("ABS", 0);
        sasfns.put("ABS", entry);
        sasfns.put("abs", entry);
        entry = new funcEntry("ARCOS", 0);
        sasfns.put("ACOS", entry);
        sasfns.put("acos", entry);
        entry = new funcEntry("ARSIN", 0);
        sasfns.put("ASIN", entry);
        sasfns.put("asin", entry);
        entry = new funcEntry("ATAN", 0);
        sasfns.put("ATAN", entry);
        sasfns.put("atan", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("ATAN2", entry);
        sasfns.put("atan2", entry);
        entry = new funcEntry("CEIL", 0);
        sasfns.put("CEILING", entry);
        sasfns.put("ceiling", entry);
        entry = new funcEntry("COS", 0);
        sasfns.put("COS", entry);
        sasfns.put("cos", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("COT", entry);
        sasfns.put("cot", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("DEGREES", entry);
        sasfns.put("degrees", entry);
        entry = new funcEntry("EXP", 0);
        sasfns.put("EXP", entry);
        sasfns.put("exp", entry);
        entry = new funcEntry("FLOOR", 0);
        sasfns.put("FLOOR", entry);
        sasfns.put("floor", entry);
        entry = new funcEntry("LOG", 0);
        sasfns.put("LOG", entry);
        sasfns.put("log", entry);
        entry = new funcEntry("LOG10", 0);
        sasfns.put("LOG10", entry);
        sasfns.put("log10", entry);
        entry = new funcEntry("MOD", 0);
        sasfns.put("MOD", entry);
        sasfns.put("mod", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("PI", entry);
        sasfns.put("pi", entry);
        entry = new funcEntry("  (", funcEntry.changePower);
        sasfns.put("POWER", entry);
        sasfns.put("power", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("RADIANS", entry);
        sasfns.put("radians", entry);
        entry = new funcEntry("RANUNI", funcEntry.changeRand);
        sasfns.put("RAND", entry);
        sasfns.put("rand", entry);
        entry = new funcEntry("ROUND", funcEntry.changeRound);
        sasfns.put("ROUND", entry);
        sasfns.put("round", entry);
        entry = new funcEntry("SIGN", 0);
        sasfns.put("SIGN", entry);
        sasfns.put("sign", entry);
        entry = new funcEntry("SIN", 0);
        sasfns.put("SIN", entry);
        sasfns.put("sin", entry);
        entry = new funcEntry("SQRT", 0);
        sasfns.put("SQRT", entry);
        sasfns.put("sqrt", entry);
        entry = new funcEntry("TAN", 0);
        sasfns.put("TAN", entry);
        sasfns.put("tan", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("TRUNCATE", entry);
        sasfns.put("truncate", entry);
        entry = new funcEntry("DATE", 0);
        sasfns.put("CURDATE", entry);
        sasfns.put("curdate", entry);
        entry = new funcEntry("TIME", 0);
        sasfns.put("CURTIME", entry);
        sasfns.put("curtime", entry);
        entry = new funcEntry("LEFT(PUT", funcEntry.changeDayname);
        sasfns.put("DAYNAME", entry);
        sasfns.put("dayname", entry);
        entry = new funcEntry("DAY", 0);
        sasfns.put("DAYOFMONTH", entry);
        sasfns.put("dayofmonth", entry);
        entry = new funcEntry("WEEKDAY", 0);
        sasfns.put("DAYOFWEEK", entry);
        sasfns.put("dayofweek", entry);
        entry = new funcEntry("MOD(JULDATE", funcEntry.changeDayofyear);
        sasfns.put("DAYOFYEAR", entry);
        sasfns.put("dayofyear", entry);
        entry = new funcEntry("HOUR", 0);
        sasfns.put("HOUR", entry);
        sasfns.put("hour", entry);
        entry = new funcEntry("MINUTE", 0);
        sasfns.put("MINUTE", entry);
        sasfns.put("minute", entry);
        entry = new funcEntry("MONTH", 0);
        sasfns.put("MONTH", entry);
        sasfns.put("month", entry);
        entry = new funcEntry("LEFT(PUT", funcEntry.changeMonthname);
        sasfns.put("MONTHNAME", entry);
        sasfns.put("monthname", entry);
        entry = new funcEntry("DATETIME", 0);
        sasfns.put("NOW", entry);
        sasfns.put("now", entry);
        entry = new funcEntry("QTR", 0);
        sasfns.put("QUARTER", entry);
        sasfns.put("quarter", entry);
        entry = new funcEntry("SECOND", 0);
        sasfns.put("SECOND", entry);
        sasfns.put("second", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("TIMESTAMPADD", entry);
        sasfns.put("timestampadd", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("TIMESTAMPDIFF", entry);
        sasfns.put("timestampdiff", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("WEEK", entry);
        sasfns.put("week", entry);
        entry = new funcEntry("YEAR", 0);
        sasfns.put("YEAR", entry);
        sasfns.put("year", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("DATABASE", entry);
        sasfns.put("database", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("IFNULL", entry);
        sasfns.put("ifnull", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("USER", entry);
        sasfns.put("user", entry);
        entry = new funcEntry(null, 0);
        sasfns.put("CONVERT", entry);
        sasfns.put("convert", entry);
        months[0] = "";
        months[1] = "JAN";
        months[2] = "FEB";
        months[3] = "MAR";
        months[4] = "APR";
        months[5] = "MAY";
        months[6] = "JUN";
        months[7] = "JUL";
        months[8] = "AUG";
        months[9] = "SEP";
        months[10] = "OCT";
        months[11] = "NOV";
        months[12] = "DEC";
    }

    // Constructs a MVAStatement object for MVACallableStatement
    public MVAStatement()
    {
    }

    public MVAStatement(MVAConnection connection)
    {
        this(connection, null);
    }

    // Constructs an MVAStatement object.
    // @param connection the MVAConnection object associated with this object
    public MVAStatement(MVAConnection connection, java.util.Properties props)
    {
        this.connection = connection;
        //  this.sasDataService = connection.sasDataService;
        //  this.dbmsConnection = connection.dbmsConnection;
        m_xUtil = connection.getXUtil();
        this.resultSetType = connection.resultSetType;
        this.resultSetConcurrency = connection.resultSetConcurrency;
        if (resultSetType == ResultSet.TYPE_SCROLL_INSENSITIVE)
            randomAccess = true;
        else if (resultSetType == ResultSet.TYPE_FORWARD_ONLY)
            randomAccess = false;
        info = props;
        if (info != null)
        {
            String temp = (String) info.get("sasifytablenames".toLowerCase());
            if (temp != null)
            {
                if (temp.equalsIgnoreCase("none"))
                    m_sasifyTableNames = SASIFY_NONE;
                else if (temp.equalsIgnoreCase("v91"))
                    m_sasifyTableNames = SASIFY_V91;
                else if (temp.equalsIgnoreCase("default"))
                    m_sasifyTableNames = SASIFY_DEFAULT;
            }
        }
    }

    /**
     * Execute a SQL statement that returns a single ResultSet.
     * 
     * @param query
     *            Typically this is a static SQL SELECT statement.
     * @return An MVAResultSet object that contains the data produced by the
     *         query.
     * @exception SQLException
     *                This exception is thrown if an MVAException is detected.
     */
    public ResultSet executeQuery(String query) throws SQLException
    {
        int hashCode = -1;
        boolean gt = true;
        /**
         * fix for S0102148
         */
        if (snresultset != null)
        {
            snresultset.close();
            snresultset = null;
        }
        updateCount = -1;
        // TODO: need to work on warnings...
        clearWarnings();
        SQLWarning warning;
        if (connection.isClosed() == true)
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.General_noConnectionError);
        }
        // Submit the Query
        if (query != null)
        {
            if (gt)
            {
                String libref = "WORK"; // default the libref to "WORK". that
                // way, if none is specified in
                // the query, we use work. (S0236919)
                String memname = null;
                StringTokenizer st = new StringTokenizer(query);
                /* verrrry primitive */
                int index = 0;
                while (st.hasMoreTokens() && gt)
                {
                    String s = st.nextToken();
                    switch (index)
                    {
                        case 0: // select
                            if (!s.equalsIgnoreCase("select"))
                                gt = false;
                            break;
                        case 1: // *
                            if (!s.equals("*"))
                                gt = false;
                            break;
                        case 2: // from
                            if (!s.equalsIgnoreCase("from"))
                                gt = false;
                            break;
                        case 3: // <table.member>
                        {
                            StringTokenizer poo = new StringTokenizer(s, ".");
                            while (poo.countTokens() > 2)
                                poo.nextToken();
                            if (poo.countTokens() == 1)
                                memname = poo.nextToken();
                            else
                            {
                                libref = poo.nextToken();
                                memname = poo.nextToken();
                            }
                        }
                            break;
                        default: // if this is called, bail out of attempt
                            gt = false;
                            break;
                    }
                    index++;
                }
                if (gt)
                {
                    try
                    {
                        // stripping off the semi-colon just for convenience
                        // sake
                        // (so if you pass "select * from get.table;", you still
                        // have
                        // the performance boost from getTable.
                        if (memname.charAt(memname.length() - 1) == ';')
                        {
                            String t = memname;
                            memname = t.substring(0, t.length() - 1);
                        }

                        snresultset = (MVAResultSet) getTable(libref, memname);
                    }
                    catch (Exception re)
                    {
                        // rather than barfing, just try again the regular way.
                        snresultset = null;
                        gt = false;
                    }
                }
                else
                {
                    warning = new SQLWarning(
                            MVALocalizedError
                                    .getString(VendorCode.MVAStatement_rsNotUpdatableWarning));
                    warnings.setNextWarning(warning);
                }
            }
            if (!gt)
            {
                try
                {
                    String tempQuery = sasify(query);
                    String temp = tempQuery.toUpperCase();
                    temp = temp.trim();
                    if (temp.indexOf("SELECT ") != 0)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_querySelectError);
                    }
                    hashCode = m_xUtil.executeQuery(tempQuery, randomAccess);
                    snresultset = new MVAResultSet(this, info, hashCode);
                }
                catch (Exception e)
                {
                    MVALocalizedError.throwLocalizedException(
                            VendorCode.MVAStatement_genericQueryError, null, e);
                }
            }
        }
        else
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.MVAStatement_genericResultSetError);
        }
        return snresultset;
    }

    /**
     * Executes an SQL INSERT, UPDATE or DELETE statement.
     * 
     * @param sql
     *            A SQL INSERT, UPDATE or DELETE statement.
     * @return The row count for INSERT, UPDATE or DELETE.
     * @exception SQLException
     *                This exception is thrown if an exception is detected.
     */
    public int executeUpdate(String sql) throws SQLException
    {
        clearWarnings();
        /**
         * fix for S0102148
         */
        if (snresultset != null)
            snresultset.close();
        snresultset = null;
        updateCount = 0;
        if (connection.isClosed() == true)
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.General_noConnectionError);
        }
        if ((sql == null) || (sql.length() == 0))
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.MVAStatement_noStatementError);
        }
        try
        {
            String tempSql = sasify(sql);
            String temp = tempSql.toUpperCase();
            temp = temp.trim();
            if (temp.indexOf("SELECT ") == 0)
            {
                MVALocalizedError
                        .throwLocalizedException(VendorCode.MVAStatement_noSelectInUpdateError);
            }
            updateCount = m_xUtil.executeUpdate(tempSql);
            // TODO warnings?
        }
        catch (Exception e)
        {
            MVALocalizedError.throwLocalizedException(
                    VendorCode.MVAStatement_executeUpdateError, null, e);
        }
        return updateCount;
    }

    /**
     * In many cases, it is desirable to immediately release a Statements's
     * database and JDBC resources instead of waiting for this to happen when it
     * is automatically closed; the close method provides this immediate
     * release.
     * 
     * <P>
     * <B>Note: </B> A Statement is automatically closed when it is garbage
     * collected. When a Statement is closed, its current ResultSet, if one
     * exists, is also closed.
     * 
     * @exception SQLException
     *                This exception is thrown if an error condition is
     *                detected.
     */
    public void close() throws SQLException
    {
        try
        {
            /**
             * if (sasLibref != null) { sasLibref._release(); sasLibref = null; }
             */
            m_xUtil.releaseLibref();
            // TODO m_xUtil.releaseLibref()
            if (snresultset != null)
            {
                snresultset.close();
                snresultset = null;
            }
        }
        catch (Exception e)
        {
            snresultset = null;
        }
	finally
        {
        /* 
         * S0308525: added call to statementClosed here to ensure that when a 
         * statement is closed, it removes itself from the connection's list of statements.
         * I've put the call in the finally block so that even if we get an exception in release
         * or rs.close(), we still remove the statement from the con list. 
         * Changing call to connection to be "statementClosed", which will
         * signal the connection that it is free to dispose of the statement (or whatever else it
         * wants to do )
         */
	  connection.statementClosed(this);
        }
    }

    //----------------------------------------------------------------------
    /**
     * The maxFieldSize limit (in bytes) is the maximum amount of data returned
     * for any column value; it only applies to BINARY, VARBINARY,
     * LONGVARBINARY, CHAR, VARCHAR, and LONGVARCHAR columns. If the limit is
     * exceeded, the excess data is silently discarded.
     * 
     * @return The current max column size limit. The default limit is 32767.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     */
    public int getMaxFieldSize() throws SQLException
    {
        return maxFieldSize;
    }

    /**
     * The maxFieldSize limit (in bytes) is set to limit the size of data that
     * can be returned for any column value; it only applies to BINARY,
     * VARBINARY, LONGVARBINARY, CHAR, VARCHAR, and LONGVARCHAR fields. If the
     * limit is exceeded, the excess data is silently discarded. For maximum
     * portability use values greater than 256.
     * 
     * @param max
     *            The new max column size limit. Zero means unlimited.
     * @exception SQLException
     *                This exception is thrown if the max field size is less
     *                than 0.
     */
    public void setMaxFieldSize(int max) throws SQLException
    {
        if (max < 0)
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.MVAStatement_maxFieldsError);
        }
        this.maxFieldSize = max;
    }

    /**
     * The maxRows limit is the maximum number of rows that a ResultSet can
     * contain. If the limit is exceeded, the excess rows are silently dropped.
     * There is no default limit.
     * 
     * @return The current max row limit. Zero means unlimited.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     */
    public int getMaxRows() throws SQLException
    {
        return maxRows;
    }

    /**
     * The maxRows limit is set to limit the number of rows that any ResultSet
     * can contain. If the limit is exceeded, the excess rows are silently
     * dropped.
     * 
     * @param max
     *            The new max rows limit. Zero means unlimited.
     * @exception SQLException
     *                This exception is thrown if the max rows is less than 0.
     */
    public void setMaxRows(int max) throws SQLException
    {
        if (max < 0)
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.MVAStatement_maxRowsError);
        }
        this.maxRows = max;
        if ((maxRows < fetchSize) && (maxRows != 0))
        {
            fetchSize = maxRows;
        }
    }

    /**
     * If escape scanning is on (the default), the driver will do escape
     * substitution before sending the SQL to the database.
     * 
     * SAS software does not support escape processing.
     * 
     * @param enable
     *            The value for this parameter must be False.
     * @exception SQLException
     *                This exception is thrown if value of enable is True.
     */
    public void setEscapeProcessing(boolean enable) throws SQLException
    {
        if (enable == true)
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.General_methodNotSupported);
        }
    }

    /**
     * Gets the query timeout limit. The queryTimeout limit is the number of
     * seconds the driver will wait for a Statement to execute. If the limit is
     * exceeded, a SQLException is thrown.
     * 
     * @return 0. Time out is currently not supported. Zero means unlimited.
     * @exception SQLException
     *                if a database-access error occurs.
     */
    public int getQueryTimeout() throws SQLException
    {
        return 0;
    }

    /**
     * Sets the query timeout limit. The queryTimeout limit is the number of
     * seconds the driver will wait for a Statement to execute. If the limit is
     * exceeded, a SQLException is thrown.
     * 
     * @param seconds
     *            The value of this parameter must be 0. Timeout is currently
     *            not supported.
     * @exception SQLException
     *                This exception is thrown if value of seconds is not equal
     *                to 0.
     */
    public void setQueryTimeout(int seconds) throws SQLException
    {
        if (seconds != 0)
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.General_methodNotSupported);
        }
    }

    /**
     * Cancel can be used by one thread to cancel a statement that is being
     * executed by another thread.
     * 
     * @exception SQLException
     *                This exception may be thrown.
     */
    public void cancel() throws SQLException
    {
        close();
    }

    /**
     * Gets the warnings for this statement.
     * 
     * @return An SQLWarning.
     * @exception SQLException
     *                This exception is required by the interface, but it is
     *                never thrown.
     */
    public SQLWarning getWarnings() throws SQLException
    {
        return warnings;
    }

    /**
     * Clears the warnings for this Statement.
     * 
     * @exception SQLException
     *                This exception is required by the interface, but it is
     *                never thrown.
     */
    public void clearWarnings() throws SQLException
    {
        warnings = null;
        warnings = new SQLWarning();
    }

    /**
     * Sets the cursor name. Cursors are not supported by SAS software so the
     * cursor name will be ignored.
     * 
     * @param name
     *            The new cursor name.
     * @exception SQLException
     *                This exception is required by the interface, but it is
     *                never thrown.
     */
    public void setCursorName(String name) throws SQLException
    {
        return;
    }

    //----------------------- Multiple Results --------------------------
    /**
     * Execute a SQL statement that may return multiple results. Under some
     * (uncommon) situations a single SQL statement may return multiple result
     * sets and/or update counts. Normally you can ignore this, unless you're
     * executing a stored procedure that you know may return multiple results,
     * or unless you're dynamically executing an unknown SQL string. The
     * "execute", "getMoreResults", "getResultSet" and "getUpdateCount" methods
     * let you navigate through multiple results.
     * 
     * The "execute" method executes a SQL statement and indicates the form of
     * the first result. You can then use getResultSet or getUpdateCount to
     * retrieve the result, and getMoreResults to move to any subsequent
     * result(s).
     * 
     * @param sql
     *            An SQL statement
     * @return True if the next result is a ResultSet. False if it is an update
     *         count or there are no more results
     * @exception SQLException
     *                This exception is thrown if an MvaException is detected.
     * @see #getResultSet
     * @see #getUpdateCount
     * @see #getMoreResults
     */
    public boolean execute(String sql) throws SQLException
    {
        clearWarnings();
        SQLWarning warning;
        /**
         * fix for S0102148
         */
        if (snresultset != null)
            snresultset.close();
        snresultset = null;
        updateCount = -1;
        if (connection.isClosed() == true)
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.General_noConnectionError);
        }
        // Submit the Query
        if (sql != null)
        {
            int hashCode = -1;
            try
            {
                String tempSql = sasify(sql);
                String temp = tempSql.toUpperCase();
                temp = temp.trim();
                if (temp.indexOf("SELECT ") != 0)
                {
                    //not a select statement
                    updateCount = m_xUtil.executeUpdate(tempSql);
                    // get the warnings?
                    return false;
                }
                else
                {
                    boolean gt = true;
                    if (gt)
                    {
                        String libref = "WORK"; // S0236919: set the libref to a
                        // default of "WORK"
                        // that way, if none is picked up in the query, we use
                        // that.
                        String memname = null;
                        StringTokenizer st = new StringTokenizer(sql);
                        /* verrrry primitive */
                        int index = 0;
                        while (st.hasMoreTokens() && gt)
                        {
                            String s = st.nextToken();
                            switch (index)
                            {
                                case 0: // select
                                    if (!s.equalsIgnoreCase("select"))
                                        gt = false;
                                    break;
                                case 1: // *
                                    if (!s.equals("*"))
                                        gt = false;
                                    break;
                                case 2: // from
                                    if (!s.equalsIgnoreCase("from"))
                                        gt = false;
                                    break;
                                case 3: // <table.member>
                                {
                                    StringTokenizer poo = new StringTokenizer(
                                            s, ".");
                                    while (poo.countTokens() > 2)
                                        poo.nextToken();
                                    if (poo.countTokens() == 1)
                                        memname = poo.nextToken();
                                    else
                                    {
                                        libref = poo.nextToken();
                                        memname = poo.nextToken();
                                    }
                                }
                                    break;
                                default: // if this is called, bail out of
                                    // attempt
                                    gt = false;
                                    break;
                            }
                            index++;
                        }
                        if (gt)
                        {
                            try
                            {
                                // stripping off the semi-colon just for
                                // convenience sake
                                // (so if you pass "select * from get.table;",
                                // you still have
                                // the performance boost from getTable.
                                if (memname.charAt(memname.length() - 1) == ';')
                                {
                                    String t = memname;
                                    memname = t.substring(0, t.length() - 1);
                                }

                                //todo
                                snresultset = (MVAResultSet) getTable(libref,
                                        memname);
                            }
                            catch (RIOException re)
                            {
                                gt = false;
                                snresultset = null;
                            }
                        }
                        else
                        {
                            warning = new SQLWarning(
                                    MVALocalizedError
                                            .getString(VendorCode.MVAStatement_rsNotUpdatableWarning));
                            warnings.setNextWarning(warning);
                        }
                    }
                    if (!gt)
                    {
                        hashCode = m_xUtil.executeQuery(tempSql, randomAccess);
                        /** moved the below call here to fix S0183008 */
                        snresultset = new MVAResultSet(this, info, hashCode);
                    }
                }
            }
            catch (Exception e)
            {
                MVALocalizedError.throwLocalizedException(
                        VendorCode.MVAStatement_executeError, null, e);
            }
        }
        else
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.MVAStatement_resultSetError);
        }
        return true;
    }

    /**
     * Gets the current result as a ResultSet. It should only be called once per
     * result.
     * 
     * @return The current result as a ResultSet. NULL if the result is an
     *         update count or there are no more results.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     * @see #execute
     */
    public ResultSet getResultSet() throws SQLException
    {
        return snresultset;
    }

    /**
     * Gets the current result as an update count.If the result is a ResultSet
     * or there are no more results, -1 is returned. It should only be called
     * once per result.
     * 
     * @return The current result as an update count. -1 if it is a ResultSet or
     *         there are no more results.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     * @see #execute
     */
    public int getUpdateCount() throws SQLException
    {
        return updateCount;
    }

    /**
     * Moves to a Statement's next result. It returns true if this result is a
     * ResultSet. getMoreResults also implicitly closes any current ResultSet
     * obtained with getResultSet.
     * 
     * There are no more results when (!getMoreResults() && (getUpdateCount() ==
     * -1)
     * 
     * @return False. Multiple result sets are not supported.
     * @exception SQLException
     *                This exception may be thrown.
     * @see #execute
     */
    public boolean getMoreResults() throws SQLException
    {
        if (snresultset != null)
        {
            snresultset.close();
            snresultset = null;
        }
        return false;
    }

    //--------------------------JDBC 2.0-----------------------------
    /**
     * JDBC 2.0
     * 
     * Give a hint as to the direction in which the rows in a result set will be
     * processed. The hint applies only to result sets created using this
     * Statement object. The default value is ResultSet.FETCH_FORWARD.
     * 
     * @param direction
     *            The initial direction for processing rows.
     * @exception SQLException
     *                This exception is thrown if the direction is not one of
     *                ResultSet.FETCH_FORWARD, ResultSet.FETCH_REVERSE, or
     *                ResultSet.FETCH_UNKNOWN.
     */
    public void setFetchDirection(int direction) throws SQLException
    {
        if ((direction != ResultSet.FETCH_FORWARD)
                && (direction != ResultSet.FETCH_REVERSE)
                && (direction != ResultSet.FETCH_UNKNOWN))
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.MVAStatement_fetchDirectionError);
        }
        this.direction = direction;
    }

    /**
     * JDBC 2.0
     * 
     * Gets the result set fetch direction.
     * 
     * @return The result set fetch direction. The default is
     *         ResultSet.FETCH_FORWARD.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     */
    public int getFetchDirection() throws SQLException
    {
        return direction;
    }

    /**
     * JDBC 2.0
     * 
     * Give the JDBC driver a hint as to the number of rows that should be
     * fetched from the database when more rows are needed. The number of rows
     * specified only affects result sets created using this statement. If the
     * value specified is zero, then the hint is ignored. The default value is
     * zero.
     * 
     * @param rows
     *            The number of rows to fetch.
     * @exception SQLException
     *                This exception is thrown if the condition 0 <= rows <=
     *                this.getMaxRows() is not satisfied.
     */
    public void setFetchSize(int rows) throws SQLException
    {
        if (rows == 0)
        {
            this.fetchSize = MVAConnection.XHRSQPRECS;
            return;
        }
        if ((rows < 0)
                || ((this.getMaxRows() != 0) && (rows > this.getMaxRows())))
        {
            MVALocalizedError
                    .throwLocalizedException(VendorCode.MVAStatement_fetchSizeError);
        }
        this.fetchSize = rows;
    }

    /**
     * JDBC 2.0
     * 
     * Gets the fetch size.
     * 
     * @return An integer value indicating the number of rows fetched for this
     *         result set.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     */
    public int getFetchSize() throws SQLException
    {
        return fetchSize;
    }

    /**
     * JDBC 2.0
     * 
     * Gets the result set concurrency type.
     * 
     * @return The result set concurrency type, CONCUR_UPDATABLE or
     *         CONCUR_READ_ONLY.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     */
    public int getResultSetConcurrency() throws SQLException
    {
        return resultSetConcurrency;
    }

    /**
     * JDBC 2.0
     * 
     * Gets the result set type.
     * 
     * @return The result set type, TYPE_SCROLL_INSENSITIVE or
     *         TYPE_FORWARD_ONLY. Changes made while the result set is open are
     *         not visible.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     */
    public int getResultSetType() throws SQLException
    {
        return resultSetType;
    }

    /**
     * JDBC 2.0
     * 
     * Adds a SQL command to the current batch of commmands for the statement.
     * This method is optional.
     * 
     * @param sql
     *            typically this is a static SQL INSERT or UPDATE statement
     * @exception SQLException
     *                if a database-access error occurs, or the driver does not
     *                support batch statements
     */
    public void addBatch(String sql) throws SQLException
    {
        m_batchList.add(sql);
    }

    /**
     * JDBC 2.0
     * 
     * Make the set of commands in the current batch empty. This method is
     * optional.
     * 
     * @exception SQLException
     *                if a database-access error occurs, or the driver does not
     *                support batch statements
     */
    public void clearBatch() throws SQLException
    {
        m_batchList = new Vector();
    }

    /**
     * JDBC 2.0
     * 
     * Submit a batch of commands to the database for execution. This method is
     * optional.
     * 
     * @return an array of update counts containing one element for each command
     *         in the batch. The array is ordered according to the order in
     *         which commands were inserted into the batch
     * @exception SQLException
     *                if a database-access error occurs, or the driver does not
     *                support batch statements
     */
    public int[] executeBatch() throws SQLException, BatchUpdateException
    {
        /**
         * req: 1. continue executing in the event of a statement failure. 2.
         * throw exception (if necessary) at end.) 3. if you get a connection
         * error (or any other "critical" sql error), then bail immediately.
         * NOTE: According to the JDBC API Tutorial, you throw an SQLException
         * only if you have a data access error or the driver doesn't support
         * Batch. Otherwise, if a statement fails or returns something other
         * than an update count, throw a BatchUpdateException.
         */
        int listSize = m_batchList.size();
        int[] updateCounts = new int[listSize];
        int i;
        int uc = 0;
        boolean throwFlag = false;
        BatchUpdateException bue = null;
        for (i = 0; i < listSize; i++)
        {
            /**
             * check the connection. If bad, bail.
             */
            if (connection.isClosed())
            {
                MVALocalizedError
                        .throwLocalizedException(VendorCode.General_noConnectionError);
            }
            /**
             * check for select first, so we don't waste time calling down. if
             * bad, mark it so we can throw the exception at the end, but
             * continue processing.
             */
            String sql = (String) m_batchList.elementAt(i);
            String tempSql = sasify(sql);
            String temp = tempSql.toUpperCase();
            temp = temp.trim();
            if (temp.indexOf("SELECT ") == 0)
            {
                throwFlag = true;
                uc = -3;
            }
            else
            {
                /**
                 * try sending the statement to executeUpdate. executeUpdate
                 * calls ExecuteStatement in the IDBMSConnection class. Any
                 * error condition in that results in an exception (it appears).
                 */
                try
                {
                    uc = executeUpdate((String) m_batchList.elementAt(i));
                }
                /**
                 * Looking at the code in executeUpdate, it looks like the only
                 * exception thrown is SQLException. But, just to be safe, catch
                 * Exception as well.
                 */
                catch (SQLException se)
                {
                    if (se.getErrorCode() == 208) // connection error. bail.
                        throw se;
                    else
                    {
                        uc = -3;
                        throwFlag = true;
                    }
                }
                catch (Exception e)
                {
                    uc = -3;
                    throwFlag = true;
                }
            }
            /**
             * store whatever the update count value (uc) is in updateCount.
             */
            updateCounts[i] = uc;
        }
        /**
         * if throwFlag is true, then we need to throw a BatchUpdateException.
         * Basically, if any non-fatal error occured.
         */
        if (throwFlag)
        {
            /**
             * Object[] messageArg = { "MVA.S500.ex.txt" };
             */
            String bueMsg = MVALocalizedError
                    .getString(VendorCode.MVAStatement_batchUpdateException);
            bue = new BatchUpdateException(bueMsg, null,
                    VendorCode.VC_UNKNOWNERROR,
                    // vendor code is meaningless.
                    updateCounts);
            throw bue;
        }
        return updateCounts;
    }

    /**
     * JDBC 2.0
     * 
     * Return the Connection that produced the Statement.
     * 
     * @return the Connection object.
     * @exception SQLException
     *                This exception is required by the interface but it is
     *                never thrown.
     */
    public Connection getConnection() throws SQLException
    {
        return connection;
    }

    /**
     * RIO
     * 
     * Creates a new table with the name specified in the tableName String. The
     * columns to be created and their attributes are passed through the
     * columnObject parameter. Any SAS options to be sent with the create table
     * will be passed through the optionsObject parameter.
     * 
     * @param libref
     *            The libref name. Cannot be NULL.
     * @param tableName
     *            The name of the table to be created. Table name cannot be
     *            NULL.
     * @param columnObject
     *            A Column Object array containing column names and attributes.
     *            Column object cannot be NULL.
     * @param options
     *            A String of applicaaple SAS dataset options. May be NULL.
     * @exception RIOException
     *                This exception may be thrown.
     */
    public void createTable(String libref, String tableName,
            Column[] columnObject, String options) throws RIOException
    {
        boolean success = false;
        try
        {
            success = m_xUtil.createTable(libref, tableName, columnObject,
                    options);
        }
        catch (Exception e)
        {
            Object[] args =
            {
                e.getMessage()
            };
            MVALocalizedError.throwLocalizedRIOException(
                    VendorCode.MVAStatement_createTableError, args);
        }
    }

    /**
     * RIO
     * 
     * Gets an MVAResultSet object for the given table name.By default all the
     * columns and all rows are selected.
     * 
     * @param libref
     *            The libref name. Cannot be NULL.
     * @param tableName
     *            The name of the table. Table name cannot be NULL. By default,
     *            the driver looks for a dataset to open, if one doesn't exist
     *            it looks for a view to open. If both exist, it opens a
     *            dataset.
     * @return An MVAResultSet object. All columns and all rows are selected.
     * @exception RIOException
     *                This exception may be thrown.
     */
    public ResultSet getTable(String libref, String tableName)
            throws RIOException
    {
        return getTable(libref, tableName, null, null, null);
    }

    /**
     * RIO
     * 
     * Gets an MVAResultSet object for the given table name.
     * 
     * @param libref
     *            The libref name. Cannot be NULL.
     * @param tableName
     *            The name of the table to be opened.
     * @param tableType
     *            The type of the table to be opened.It could be "DATA" or
     *            "VIEW". If NULL, the driver looks for a dataset to open, if
     *            one doesn't exist it looks for a view to open. If both exist,
     *            it opens a dataset. This parameter is currently being ignored
     *            and defaulted to NULL.
     * @param password
     *            A String array of 3 elements specifying READ, WRITE and ALTER
     *            passwords if any. This parameter can be NULL. The first
     *            element is the READ password, the second the WRITE password
     *            and the third the ALTER password.
     * @param options
     *            A String whose contents can be any SAS dataset options
     *            appropriate to the underlying engine and library. This
     *            includes the WHERE clause. options can be NULL.
     * @return An MVAResultSet object. The ResultSet could be a subset of the
     *         actual set of columns and rows.
     * @exception RIOException
     *                This exception may be thrown.
     */
    public ResultSet getTable(String libref, String tableName,
            String tableType, String[] password, String options)
            throws RIOException
    {
        //TODO: call project columns and handle options...
        // TODO RioExceptions?
        this.libref = libref;
        this.options = options;
        if (libref == null || libref.length() == 0)
        {
            MVALocalizedError
                    .throwLocalizedRIOException(VendorCode.MVAStatement_nullLibrefError);
        }
        try
        {
            return new MVAResultSet(this, tableName, tableType, password, info);
        }
        catch (SQLException e)
        {
            // why did they even bother with "RIOExceptions"???
            Object[] args =
            {
                e.getMessage()
            };
            MVALocalizedError.throwLocalizedRIOException(
                    VendorCode.MVAStatement_genericResultSetError2, args);
        }
        return null;
    }

    /**
     * Deletes the table with the name specified in the tableName String.
     * 
     * @param libref
     *            The libref name. Cannot be NULL.
     * @param tableName
     *            The name of the table to be deleted. By default, the tableName
     *            dataset will be deleted.
     * @exception RIOException
     *                This exception may be thrown.
     */
    public void deleteTable(String libref, String tableName)
            throws RIOException
    {
        deleteTable(libref, tableName, null, null);
    }

    /**
     * RIO
     * 
     * Deletes the table for the specified tableType with the name specified in
     * the tableName String.
     * 
     * @param libref
     *            The libref name. Cannot be NULL.
     * @param tableName
     *            The name of the table to be deleted.
     * @param tableType
     *            The type of the table. It could be "DATA" or "VIEW" or any
     *            table type available in the database. If NULL, the tableName
     *            dataset will be deleted.
     * @param alterPassword
     *            The alter password.
     * @exception RIOException
     *                This exception may be thrown.
     */
    public void deleteTable(String libref, String tableName, String tableType,
            String alterPassword) throws RIOException
    {
        // TODO RioExceptions?
        this.libref = libref;
        if (libref == null || libref.length() == 0)
        {
            MVALocalizedError
                    .throwLocalizedRIOException(VendorCode.MVAStatement_nullLibrefError);
        }
        try
        {
            m_xUtil.deleteTable(libref, tableName, tableType, alterPassword);
        }
        catch (Exception e)
        {
            Object[] args =
            {
                e.getMessage()
            };
            MVALocalizedError.throwLocalizedRIOException(
                    VendorCode.MVAStatement_deleteFailed, args);
        }
    }

 

    /**
     * Sasify statements.
     * S0235627 (+ S0266879) - changed sasify to switch based on a connection property.
     * defaults to running code written for fix (see sasifyNone)
     */
    private String sasify(String stmt) throws SQLException
    {
        switch (m_sasifyTableNames)
        {
            case SASIFY_V91: /* "V91" */
                return sasifyV91(stmt);
            case SASIFY_NONE: /* "none" */
                return sasifyNone(stmt);
            case SASIFY_DEFAULT: /* "default" */
            default:
                return sasifyDefault(stmt);
        }
    }

    /*
     * sasifyDefault is the "default" sasify behavior. This is the most current version of the 
     * function, and includes the changes for defects S0266879 and S0235627
     */
    private String sasifyDefault(String stmt) throws SQLException
    {
        // TODO a perfect example of something where vendor codes would be good.
        Integer temp = new Integer(0);
        int monthIndex = 0;
        String sub1 = "";
        String sub2 = "";
        String sub3 = "";
        String temps = "";
        int b = 0;
        int c = 0;
        int e = 0;
        int i, k, l;
        int nested, endloc; // for processing escape clauses
        int type = 0;
        stmt = stmt.trim();
        // Translate escape and shorthand Functions, Calls, Date, Time and
        // Timestamp literals
        // to SAS literals
        // Look for escape strings
        // find begin of escape string (must be after beginning)
        while ((l = findLastString(stmt, "{")) > 0)
        {
            nested = 0;
            i = 0;
            int length;
            int inquotes = -1;
            ///////////////////////////////////////
            // find begin & end of escape string //
            ///////////////////////////////////////
            // NOTE: Escape sequences can be nested so we need to blank
            //       the appropriate one so we don't kill processing of
            //       an inner escape sequence.
            //       e.g. select {fn DAYNAME( {d '1996-01-01'} )} from junk
            b = l;
            endloc = findFirstString(stmt, "}", b);
            if (endloc == -1)
            {
                MVALocalizedError
                        .throwLocalizedException(VendorCode.MVAStatement_malformedStatement);
            }
            e = endloc;
            // Break into three substrings
            sub1 = stmt.substring(0, b);
            sub1 = sub1.trim();
            sub2 = stmt.substring(b + 1, e);
            sub2 = sub2.trim();
            if ((e + 1) < stmt.length())
            {
                sub3 = stmt.substring(e + 1);
                sub3 = sub3.trim();
            }
            else
            {
                sub3 = "";
            }
            temps = sub2.toLowerCase();
            type = 0;
            if ((temps.charAt(0) == 't') && (temps.charAt(1) == 's'))
                type = MVAStatement.ts; // look for ts
            if ((temps.charAt(0) == 'f') && (temps.charAt(1) == 'n'))
                type = MVAStatement.fn; // look for fn
            if ((temps.charAt(0) == 't') && (temps.charAt(1) != 's'))
                type = MVAStatement.t; // look for time
            if (temps.charAt(0) == 'd')
                type = MVAStatement.d; // look for date
            if ((temps.charAt(0) == 'o') && (temps.charAt(1) == 'j'))
                type = MVAStatement.oj; // look for date
            if (temps.startsWith("escape ", 0))
                type = MVAStatement.escape;
            // convert the ODBC literal data to SAS literal data
            // individual character moves are messier than wsprintf and strncpy
            // but much faster
            switch (type)
            {
                case MVAStatement.t:
                    // time = t'hh:mm:ss' -> 'hh:mm:ss't (11 to 11)
                    // trim leading t and all leading and trailing blanks
                    sub2 = sub2.substring(1);
                    sub2 = sub2.trim();
                    if (sub2.length() < 10)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTime);
                    }
                    if (sub2.charAt(0) != '\'' || sub2.charAt(9) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTime);
                    }
                    stmt = new String(sub1 + " " + sub2.substring(0) + "t "
                            + sub3);
                    break;
                case MVAStatement.d:
                    // date = d'yyyy-mm-dd' -> 'ddmmmyyyy'd (13 to 12)
                    // trim leading d and all leading and trailing blanks
                    sub2 = sub2.substring(1);
                    sub2 = sub2.trim();
                    if (sub2.length() < 12)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    if (sub2.charAt(0) != '\'' || sub2.charAt(11) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    temp = new Integer(sub2.substring(6, 8));
                    monthIndex = temp.intValue();
                    if ((monthIndex < 1) || (monthIndex > 12))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    stmt = new String(sub1 + " '" + sub2.substring(9, 11)
                            + months[monthIndex] + sub2.substring(1, 5) + "'d "
                            + sub3);
                    break;
                case MVAStatement.ts:
                    // timestamp = ts'yyyy-mm-dd hh:mm:ss[.f..f]' ->
                    // 'ddmmmyyyy:hh:mm:ss[.f..f]'dt
                    // trim leading ts and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    if (sub2.length() < 21)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    if (sub2.charAt(0) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    k = sub2.indexOf('\'', 1); // find trailing quote
                    if (k < 20)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    temp = new Integer(sub2.substring(6, 8));
                    monthIndex = temp.intValue();
                    if ((monthIndex < 1) || (monthIndex > 12))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    stmt = new String(sub1 + " '" + sub2.substring(9, 11)
                            + months[monthIndex] + sub2.substring(1, 5) + ":"
                            + sub2.substring(12) + "dt " + sub3);
                    break;
                case MVAStatement.oj:
                    // 'oj table LEFT OUTER JOIN table on condition' -> 'table
                    // left JOIN b on condition'
                    // trim leading ts and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    temps = sub2.toUpperCase();
                    b = findFirstString(temps, "OUTER", 0);
                    if (b == -1)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedOuterJoin);
                    }
                    stmt = new String(sub1 + " " + sub2.substring(0, b)
                            + sub2.substring(b + 5) + " " + sub3);
                    break;
                case MVAStatement.escape: // escape character processing
                    // find character
                    b = sub2.indexOf('\'');
                    if ((b == -1) || (sub2.length() < b + 2)
                            || (sub2.charAt(b + 2) != '\''))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedEscape);
                    }
                    char escapeChar = sub2.charAt(b + 1);
                    temps = new String(sub1 + " " + sub3);
                    String newStmt = new String();
                    length = temps.length();
                    i = 0;
                    int curEscape = -2;
                    inquotes = -1;
                    while (i < length)
                    {
                        if ((temps.charAt(i) == '\'')
                                || (temps.charAt(i) == '\"'))
                        {
                            if (inquotes == -1)
                                inquotes = temps.charAt(i);
                            else if (inquotes == temps.charAt(i))
                                inquotes = -1;
                        }
                        if ((inquotes != -1) && (temps.charAt(i) == escapeChar)
                                && (i != curEscape + 1))
                        {
                            curEscape = i;
                        }
                        else
                        {
                            newStmt = newStmt + temps.charAt(i);
                        }
                        i++;
                    }
                    stmt = newStmt.trim();
                    break;
                case MVAStatement.fn: // fn function
                    // trim leading fn and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    b = findFirstString(sub2, "(", 0);
                    if (b == -1)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                    }
                    temps = sub2.substring(0, b);
                    temps = temps.toUpperCase();
                    funcEntry tempfunc = (funcEntry) sasfns.get(temps);
                    if ((tempfunc == null) || (tempfunc.sasFn == null))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_undefinedFunction);
                    }
                    sub2 = new String(tempfunc.sasFn + sub2.substring(b));
                    if (tempfunc.specialHandling != 0)
                    {
                        switch (tempfunc.specialHandling)
                        {
                            case funcEntry.changeConcat:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                i = b + 1;
                                c = -1;
                                inquotes = -1;
                                nested = 0;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                String temp1 = new String(sub2.substring(b + 1,
                                        c)
                                        + " || ");
                                b = findLastString(sub2.substring(c + 1), ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                b = findLastString(sub2, ")");
                                String temp2 = new String(sub2.substring(c + 1,
                                        b));
                                if (temp2.equals(""))
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                sub2 = new String(temp1 + temp2);
                                break;
                            case funcEntry.changeLeft:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -1;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedLeftFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + ",1"
                                        + sub2.substring(c));
                                break;
                            case funcEntry.changeLTrim:
                                sub2 = new String(sub2 + ")");
                                break;
                            case funcEntry.changeRepeat:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = 0;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ')')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedRepeatFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + "-1"
                                        + sub2.substring(c));
                                break;
                            case funcEntry.changeSpace:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedSpaceFunction);
                                }
                                sub2 = new String(sub2.substring(0, b + 1)
                                        + "' ',(" + sub2.substring(b + 1)
                                        + "-1)");
                                break;
                            case funcEntry.changePower:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -2;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedPowerFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + ")**("
                                        + sub2.substring(c + 1) + ")");
                                break;
                            case funcEntry.changeRand:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                c = findLastString(sub2, ")");
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                if ((c - b) == 1)
                                    sub2 = new String(sub2.substring(0, b + 1)
                                            + "0" + sub2.substring(b + 1));
                                break;
                            case funcEntry.changeRound:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -1;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedRoundFunction);
                                }
                                sub2 = new String(sub2.substring(0, c + 1)
                                        + "10**-(" + sub2.substring(c + 1)
                                        + ")");
                                break;
                            case funcEntry.changeDayname:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedDayNameFunction);
                                }
                                sub2 = new String(sub2.substring(0, b)
                                        + ", DowName.)" + sub2.substring(b));
                                break;
                            case funcEntry.changeDayofyear:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedDayOfYearFunction);
                                }
                                sub2 = new String(sub2 + ", 1000)");
                                break;
                            case funcEntry.changeMonthname:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedChangeMonthName);
                                }
                                sub2 = new String(sub2.substring(0, b)
                                        + ", MonName.)" + sub2.substring(b));
                                break;
                            default:
                                MVALocalizedError
                                        .throwLocalizedException(VendorCode.MVAStatement_undefinedFunction);
                        }
                    }
                    stmt = new String(sub1 + " " + sub2 + " " + sub3);
                    stmt = stmt.trim();
                    break;
                default: // other = bad syntax and let SAS complain
                    return stmt;
            } // end of switch to particular type of escape sequence
        } // while particular escape string found
        // Now, look for Create statements
        boolean scanStmt = false;
        temps = stmt.toUpperCase();
        StringTokenizer st = new StringTokenizer(temps, " ");
        String token;
        if (st.hasMoreTokens())
        {
            token = st.nextToken();
            if (token.equals("CREATE") && st.hasMoreTokens())
            {
                token = st.nextToken();
                if (token.equals("TABLE"))
                {
                    scanStmt = true;
                }
            }
            else
            {
                if (token.equals("ALTER") && st.hasMoreTokens())
                {
                    token = st.nextToken();
                    if (token.equals("TABLE"))
                    {
                        while (st.hasMoreTokens())
                        {
                            token = st.nextToken();
                            if ((token.equals("ADD") || token.equals("MODIFY"))
                                    && st.hasMoreTokens())
                            {
                                scanStmt = true;
                                token = st.nextToken();
                                if (token.equals("COLUMN"))
                                {
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
        /*
         * Look for three part names: schema.table.column and truncate to
         * table.column
         */
        i = b = 0;
        c = -1;
        int state = 0;
        int dots = 0;
        int dot1 = 0;
        String tempStmt;
        int inquotes = -1;
        int edge = 0;
        // BFB 08Apr98 int length = 0;
        // damack - 12/03/2001 - S0106244
        if (stmt.indexOf(".") != -1)
        {
            /*
             * S0235627 fix: still scan for dots and trim spaces, but use a
             * tokenizer to omit quoted portions of statement from consideration
             * for "squeezing" out spaces around dots.
             */
            /*
             * First, create a tokenizer for the stmt, spec-ing single and
             * double quotes as the delimiters. Also, create a buffer, and some
             * flags.
             */
            StringTokenizer sqlParse = new StringTokenizer(stmt, "'\"", true);
            StringBuffer mungedStmt = new StringBuffer();
            boolean quoted = false;
            boolean isDouble = false;
            boolean isSingle = false;

            while (sqlParse.hasMoreTokens())
            {
                String x = sqlParse.nextToken();
                /*
                 * if the token we get is a single or double quote, we need to
                 * do special handling
                 */
                if (x.equals("'") || x.equals("\""))
                {
                    /*
                     * if we're not in the quoted state and we've hit a quote,
                     * we need to flip.
                     */
                    if (quoted == false)
                    {
                        /*
                         * if the token is a single quote, then we set the
                         * "isSingle" flag true, and explicitly set "isDouble"
                         * to false. (more to avoid confusion about what state
                         * we're in later)
                         */
                        if (x.equals("'"))
                        {
                            isSingle = true;
                            quoted = true;
                        }
                        /*
                         * do the opposite here.
                         */
                        else
                        {
                            isDouble = true;
                            quoted = true;
                        }
                    }
                    else
                    // quoted = true
                    {
                        /*
                         * if we're in a quoted state already, then we have to
                         * check the token to see if it's a match for the
                         * starting quote. hence, if we're "isSingle" or
                         * "isDouble", we'll check to see if the token matches,
                         * and if so, we'll turn off the flag, and the quoted
                         * flag.
                         */
                        if (isSingle)
                        {
                            if (x.equals("'"))
                            {
                                isSingle = false;
                                quoted = false;
                            }
                        }
                        else
                        {
                            if (x.equals("\""))
                            {
                                isDouble = false;
                                quoted = false;
                            }
                        }
                    }
                    //quoted = !quoted;
                }
                i = 0;
                if (quoted == false)
                {
                    while ((i = x.indexOf(" .", i)) != -1)
                    {
                        int dotidx = x.indexOf(".", i);
                        if ((dotidx + 1) >= x.length()
                                || x.charAt(dotidx + 1) == ' '
                                || x.charAt(dotidx + 1) == '\n'
                                || x.charAt(dotidx + 1) == '\t'
                                || x.charAt(dotidx + 1) == '\f'
                                || x.charAt(dotidx + 1) == '\r')
                            i++; // no op
                        else
                        {

                            tempStmt = x;
                            x = tempStmt.substring(0, i)
                                    + tempStmt.substring(i + 1);
                        }
                    }
                }
                mungedStmt.append(x);
            }
            /*
             * if anything went into the buffer, then change stmt by copying
             * mungedStmt in.
             */
            if (mungedStmt.length() > 0)
                stmt = mungedStmt.toString();
            i = 0;
            char curChar = ' ';
            while (i < stmt.length())
            {
                curChar = stmt.charAt(i);
                if ((curChar == '\'') || (curChar == '\"'))
                {
                    if (inquotes == -1)
                        inquotes = curChar;
                    else if (inquotes == curChar)
                        inquotes = -1;
                }
                if (((Character.isLetterOrDigit(curChar)) || (curChar == '_') || (curChar == '.'))
                        && (inquotes == -1))
                {
                    if (state == 0)
                    {
                        dots = 0;
                        edge = i;
                        state = 1;
                    }
                    else
                    {
                        if (curChar == '.')
                        {
                            dots++;
                            if (dots == 1)
                                dot1 = i;
                        }
                    }
                }
                else
                {
                    if (state == 1)
                    {
                        if (dots > 1)
                        {
                            tempStmt = stmt;
                            stmt = tempStmt.substring(0, edge)
                                    + tempStmt.substring(dot1 + 1);
                            state = 0;
                            i = 0;
                        }
                        else
                        {
                            state = 0;
                        }
                    }
                }
                i++;
            }
            if (state == 1)
            {
                if (dots > 1)
                {
                    tempStmt = stmt;
                    stmt = tempStmt.substring(0, edge)
                            + tempStmt.substring(dot1 + 1);
                    state = 0;
                    i = 0;
                }
            }
        }

        if (scanStmt == false)
        {
            return stmt;
        }
        // Need to scan the statement looking for column types that must be
        // converted to SAS types
        // types and add appropriate format statements
        // Replace SQL types with SAS types
        // SQL Type = INTEGER
        b = 0;
        while (((i = findFirstString(stmt, "INTEGER", b)) != -1)
                || ((i = findFirstString(stmt, "integer", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 7))
                    && ((stmt.charAt(b + 7) == ' ')
                            || (stmt.charAt(b + 7) == ',') || (stmt
                            .charAt(b + 7) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 "
                        + stmt.substring(b + 7));
                b = b + 7;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 7)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 ");
                b = b + 7;
            }
            else
            {
                b = b + 7;
            }
        }
        // SQL Type = LONG
        b = 0;
        while (((i = findFirstString(stmt, "LONG", b)) != -1)
                || ((i = findFirstString(stmt, "long", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = TIME
        b = 0;
        while (((i = findFirstString(stmt, "TIME", b)) != -1)
                || ((i = findFirstString(stmt, "time", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=time8. "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=time8. ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = DATE
        b = 0;
        while (((i = findFirstString(stmt, "DATE", b)) != -1)
                || ((i = findFirstString(stmt, "date", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=date9. "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=date9. ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = TIMESTAMP
        b = 0;
        while (((i = findFirstString(stmt, "TIMESTAMP", b)) != -1)
                || ((i = findFirstString(stmt, "timestamp", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 9))
                    && ((stmt.charAt(b + 9) == ' ')
                            || (stmt.charAt(b + 9) == ',') || (stmt
                            .charAt(b + 9) == ')')))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19.2 " + stmt.substring(b + 9));
                b = b + 9;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 9)))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19. ");
                b = b + 9;
            }
            else
            {
                b = b + 9;
            }
        }
        // SQL Type = datetime
        /**
         * S0164788 fix
         */
        b = 0;
        while (((i = findFirstString(stmt, "DATETIME", b)) != -1)
                || ((i = findFirstString(stmt, "datetime", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 8))
                    && ((stmt.charAt(b + 8) == ' ')
                            || (stmt.charAt(b + 8) == ',') || (stmt
                            .charAt(b + 8) == ')')))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19.2 " + stmt.substring(b + 8));
                b = b + 8;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 8)))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19. ");
                b = b + 8;
            }
            else
            {
                b = b + 8;
            }
        }
        // Finished with modifiying SQL
        return stmt;
    }

    /*
     * sasifyNone is a version of sasify which does not do the "3-level to 2-level table names" parsing.
     * So, no looking for " .", and no dropping of part of a table name
     */
    private String sasifyNone(String stmt) throws SQLException
    {
        // TODO a perfect example of something where vendor codes would be good.
        Integer temp = new Integer(0);
        int monthIndex = 0;
        String sub1 = "";
        String sub2 = "";
        String sub3 = "";
        String temps = "";
        int b = 0;
        int c = 0;
        int e = 0;
        int i, k, l;
        int nested, endloc; // for processing escape clauses
        int type = 0;
        stmt = stmt.trim();
        // Translate escape and shorthand Functions, Calls, Date, Time and
        // Timestamp literals
        // to SAS literals
        // Look for escape strings
        // find begin of escape string (must be after beginning)
        while ((l = findLastString(stmt, "{")) > 0)
        {
            nested = 0;
            i = 0;
            int length;
            int inquotes = -1;
            ///////////////////////////////////////
            // find begin & end of escape string //
            ///////////////////////////////////////
            // NOTE: Escape sequences can be nested so we need to blank
            //       the appropriate one so we don't kill processing of
            //       an inner escape sequence.
            //       e.g. select {fn DAYNAME( {d '1996-01-01'} )} from junk
            b = l;
            endloc = findFirstString(stmt, "}", b);
            if (endloc == -1)
            {
                MVALocalizedError
                        .throwLocalizedException(VendorCode.MVAStatement_malformedStatement);
            }
            e = endloc;
            // Break into three substrings
            sub1 = stmt.substring(0, b);
            sub1 = sub1.trim();
            sub2 = stmt.substring(b + 1, e);
            sub2 = sub2.trim();
            if ((e + 1) < stmt.length())
            {
                sub3 = stmt.substring(e + 1);
                sub3 = sub3.trim();
            }
            else
            {
                sub3 = "";
            }
            temps = sub2.toLowerCase();
            type = 0;
            if ((temps.charAt(0) == 't') && (temps.charAt(1) == 's'))
                type = MVAStatement.ts; // look for ts
            if ((temps.charAt(0) == 'f') && (temps.charAt(1) == 'n'))
                type = MVAStatement.fn; // look for fn
            if ((temps.charAt(0) == 't') && (temps.charAt(1) != 's'))
                type = MVAStatement.t; // look for time
            if (temps.charAt(0) == 'd')
                type = MVAStatement.d; // look for date
            if ((temps.charAt(0) == 'o') && (temps.charAt(1) == 'j'))
                type = MVAStatement.oj; // look for date
            if (temps.startsWith("escape ", 0))
                type = MVAStatement.escape;
            // convert the ODBC literal data to SAS literal data
            // individual character moves are messier than wsprintf and strncpy
            // but much faster
            switch (type)
            {
                case MVAStatement.t:
                    // time = t'hh:mm:ss' -> 'hh:mm:ss't (11 to 11)
                    // trim leading t and all leading and trailing blanks
                    sub2 = sub2.substring(1);
                    sub2 = sub2.trim();
                    if (sub2.length() < 10)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTime);
                    }
                    if (sub2.charAt(0) != '\'' || sub2.charAt(9) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTime);
                    }
                    stmt = new String(sub1 + " " + sub2.substring(0) + "t "
                            + sub3);
                    break;
                case MVAStatement.d:
                    // date = d'yyyy-mm-dd' -> 'ddmmmyyyy'd (13 to 12)
                    // trim leading d and all leading and trailing blanks
                    sub2 = sub2.substring(1);
                    sub2 = sub2.trim();
                    if (sub2.length() < 12)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    if (sub2.charAt(0) != '\'' || sub2.charAt(11) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    temp = new Integer(sub2.substring(6, 8));
                    monthIndex = temp.intValue();
                    if ((monthIndex < 1) || (monthIndex > 12))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    stmt = new String(sub1 + " '" + sub2.substring(9, 11)
                            + months[monthIndex] + sub2.substring(1, 5) + "'d "
                            + sub3);
                    break;
                case MVAStatement.ts:
                    // timestamp = ts'yyyy-mm-dd hh:mm:ss[.f..f]' ->
                    // 'ddmmmyyyy:hh:mm:ss[.f..f]'dt
                    // trim leading ts and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    if (sub2.length() < 21)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    if (sub2.charAt(0) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    k = sub2.indexOf('\'', 1); // find trailing quote
                    if (k < 20)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    temp = new Integer(sub2.substring(6, 8));
                    monthIndex = temp.intValue();
                    if ((monthIndex < 1) || (monthIndex > 12))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    stmt = new String(sub1 + " '" + sub2.substring(9, 11)
                            + months[monthIndex] + sub2.substring(1, 5) + ":"
                            + sub2.substring(12) + "dt " + sub3);
                    break;
                case MVAStatement.oj:
                    // 'oj table LEFT OUTER JOIN table on condition' -> 'table
                    // left JOIN b on condition'
                    // trim leading ts and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    temps = sub2.toUpperCase();
                    b = findFirstString(temps, "OUTER", 0);
                    if (b == -1)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedOuterJoin);
                    }
                    stmt = new String(sub1 + " " + sub2.substring(0, b)
                            + sub2.substring(b + 5) + " " + sub3);
                    break;
                case MVAStatement.escape: // escape character processing
                    // find character
                    b = sub2.indexOf('\'');
                    if ((b == -1) || (sub2.length() < b + 2)
                            || (sub2.charAt(b + 2) != '\''))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedEscape);
                    }
                    char escapeChar = sub2.charAt(b + 1);
                    temps = new String(sub1 + " " + sub3);
                    String newStmt = new String();
                    length = temps.length();
                    i = 0;
                    int curEscape = -2;
                    inquotes = -1;
                    while (i < length)
                    {
                        if ((temps.charAt(i) == '\'')
                                || (temps.charAt(i) == '\"'))
                        {
                            if (inquotes == -1)
                                inquotes = temps.charAt(i);
                            else if (inquotes == temps.charAt(i))
                                inquotes = -1;
                        }
                        if ((inquotes != -1) && (temps.charAt(i) == escapeChar)
                                && (i != curEscape + 1))
                        {
                            curEscape = i;
                        }
                        else
                        {
                            newStmt = newStmt + temps.charAt(i);
                        }
                        i++;
                    }
                    stmt = newStmt.trim();
                    break;
                case MVAStatement.fn: // fn function
                    // trim leading fn and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    b = findFirstString(sub2, "(", 0);
                    if (b == -1)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                    }
                    temps = sub2.substring(0, b);
                    temps = temps.toUpperCase();
                    funcEntry tempfunc = (funcEntry) sasfns.get(temps);
                    if ((tempfunc == null) || (tempfunc.sasFn == null))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_undefinedFunction);
                    }
                    sub2 = new String(tempfunc.sasFn + sub2.substring(b));
                    if (tempfunc.specialHandling != 0)
                    {
                        switch (tempfunc.specialHandling)
                        {
                            case funcEntry.changeConcat:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                i = b + 1;
                                c = -1;
                                inquotes = -1;
                                nested = 0;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                String temp1 = new String(sub2.substring(b + 1,
                                        c)
                                        + " || ");
                                b = findLastString(sub2.substring(c + 1), ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                b = findLastString(sub2, ")");
                                String temp2 = new String(sub2.substring(c + 1,
                                        b));
                                if (temp2.equals(""))
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                sub2 = new String(temp1 + temp2);
                                break;
                            case funcEntry.changeLeft:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -1;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedLeftFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + ",1"
                                        + sub2.substring(c));
                                break;
                            case funcEntry.changeLTrim:
                                sub2 = new String(sub2 + ")");
                                break;
                            case funcEntry.changeRepeat:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = 0;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ')')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedRepeatFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + "-1"
                                        + sub2.substring(c));
                                break;
                            case funcEntry.changeSpace:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedSpaceFunction);
                                }
                                sub2 = new String(sub2.substring(0, b + 1)
                                        + "' ',(" + sub2.substring(b + 1)
                                        + "-1)");
                                break;
                            case funcEntry.changePower:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -2;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedPowerFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + ")**("
                                        + sub2.substring(c + 1) + ")");
                                break;
                            case funcEntry.changeRand:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                c = findLastString(sub2, ")");
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                if ((c - b) == 1)
                                    sub2 = new String(sub2.substring(0, b + 1)
                                            + "0" + sub2.substring(b + 1));
                                break;
                            case funcEntry.changeRound:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -1;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedRoundFunction);
                                }
                                sub2 = new String(sub2.substring(0, c + 1)
                                        + "10**-(" + sub2.substring(c + 1)
                                        + ")");
                                break;
                            case funcEntry.changeDayname:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedDayNameFunction);
                                }
                                sub2 = new String(sub2.substring(0, b)
                                        + ", DowName.)" + sub2.substring(b));
                                break;
                            case funcEntry.changeDayofyear:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedDayOfYearFunction);
                                }
                                sub2 = new String(sub2 + ", 1000)");
                                break;
                            case funcEntry.changeMonthname:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedChangeMonthName);
                                }
                                sub2 = new String(sub2.substring(0, b)
                                        + ", MonName.)" + sub2.substring(b));
                                break;
                            default:
                                MVALocalizedError
                                        .throwLocalizedException(VendorCode.MVAStatement_undefinedFunction);
                        }
                    }
                    stmt = new String(sub1 + " " + sub2 + " " + sub3);
                    stmt = stmt.trim();
                    break;
                default: // other = bad syntax and let SAS complain
                    return stmt;
            } // end of switch to particular type of escape sequence
        } // while particular escape string found
        // Now, look for Create statements
        boolean scanStmt = false;
        temps = stmt.toUpperCase();
        StringTokenizer st = new StringTokenizer(temps, " ");
        String token;
        if (st.hasMoreTokens())
        {
            token = st.nextToken();
            if (token.equals("CREATE") && st.hasMoreTokens())
            {
                token = st.nextToken();
                if (token.equals("TABLE"))
                {
                    scanStmt = true;
                }
            }
            else
            {
                if (token.equals("ALTER") && st.hasMoreTokens())
                {
                    token = st.nextToken();
                    if (token.equals("TABLE"))
                    {
                        while (st.hasMoreTokens())
                        {
                            token = st.nextToken();
                            if ((token.equals("ADD") || token.equals("MODIFY"))
                                    && st.hasMoreTokens())
                            {
                                scanStmt = true;
                                token = st.nextToken();
                                if (token.equals("COLUMN"))
                                {
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }

        if (scanStmt == false)
        {
            return stmt;
        }
        // Need to scan the statement looking for column types that must be
        // converted to SAS types
        // types and add appropriate format statements
        // Replace SQL types with SAS types
        // SQL Type = INTEGER
        b = 0;
        while (((i = findFirstString(stmt, "INTEGER", b)) != -1)
                || ((i = findFirstString(stmt, "integer", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 7))
                    && ((stmt.charAt(b + 7) == ' ')
                            || (stmt.charAt(b + 7) == ',') || (stmt
                            .charAt(b + 7) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 "
                        + stmt.substring(b + 7));
                b = b + 7;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 7)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 ");
                b = b + 7;
            }
            else
            {
                b = b + 7;
            }
        }
        // SQL Type = LONG
        b = 0;
        while (((i = findFirstString(stmt, "LONG", b)) != -1)
                || ((i = findFirstString(stmt, "long", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = TIME
        b = 0;
        while (((i = findFirstString(stmt, "TIME", b)) != -1)
                || ((i = findFirstString(stmt, "time", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=time8. "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=time8. ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = DATE
        b = 0;
        while (((i = findFirstString(stmt, "DATE", b)) != -1)
                || ((i = findFirstString(stmt, "date", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=date9. "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=date9. ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = TIMESTAMP
        b = 0;
        while (((i = findFirstString(stmt, "TIMESTAMP", b)) != -1)
                || ((i = findFirstString(stmt, "timestamp", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 9))
                    && ((stmt.charAt(b + 9) == ' ')
                            || (stmt.charAt(b + 9) == ',') || (stmt
                            .charAt(b + 9) == ')')))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19.2 " + stmt.substring(b + 9));
                b = b + 9;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 9)))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19. ");
                b = b + 9;
            }
            else
            {
                b = b + 9;
            }
        }
        // SQL Type = datetime
        /**
         * S0164788 fix
         */
        b = 0;
        while (((i = findFirstString(stmt, "DATETIME", b)) != -1)
                || ((i = findFirstString(stmt, "datetime", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 8))
                    && ((stmt.charAt(b + 8) == ' ')
                            || (stmt.charAt(b + 8) == ',') || (stmt
                            .charAt(b + 8) == ')')))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19.2 " + stmt.substring(b + 8));
                b = b + 8;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 8)))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19. ");
                b = b + 8;
            }
            else
            {
                b = b + 8;
            }
        }
        // Finished with modifiying SQL
        return stmt;
    }

    /* 
     * sasifyV91 is the version of sasify that was in existence at ship time of 9.1
     */
    private String sasifyV91(String stmt) throws SQLException
    {
        // TODO a perfect example of something where vendor codes would be good.
        Integer temp = new Integer(0);
        int monthIndex = 0;
        String sub1 = "";
        String sub2 = "";
        String sub3 = "";
        String temps = "";
        int b = 0;
        int c = 0;
        int e = 0;
        int i, k, l;
        int nested, endloc; // for processing escape clauses
        int type = 0;
        stmt = stmt.trim();
        // Translate escape and shorthand Functions, Calls, Date, Time and
        // Timestamp literals
        // to SAS literals
        // Look for escape strings
        // find begin of escape string (must be after beginning)
        while ((l = findLastString(stmt, "{")) > 0)
        {
            nested = 0;
            i = 0;
            int length;
            int inquotes = -1;
            ///////////////////////////////////////
            // find begin & end of escape string //
            ///////////////////////////////////////
            // NOTE: Escape sequences can be nested so we need to blank
            //       the appropriate one so we don't kill processing of
            //       an inner escape sequence.
            //       e.g. select {fn DAYNAME( {d '1996-01-01'} )} from junk
            b = l;
            endloc = findFirstString(stmt, "}", b);
            if (endloc == -1)
            {
                MVALocalizedError
                        .throwLocalizedException(VendorCode.MVAStatement_malformedStatement);
            }
            e = endloc;
            // Break into three substrings
            sub1 = stmt.substring(0, b);
            sub1 = sub1.trim();
            sub2 = stmt.substring(b + 1, e);
            sub2 = sub2.trim();
            if ((e + 1) < stmt.length())
            {
                sub3 = stmt.substring(e + 1);
                sub3 = sub3.trim();
            }
            else
            {
                sub3 = "";
            }
            temps = sub2.toLowerCase();
            type = 0;
            if ((temps.charAt(0) == 't') && (temps.charAt(1) == 's'))
                type = MVAStatement.ts; // look for ts
            if ((temps.charAt(0) == 'f') && (temps.charAt(1) == 'n'))
                type = MVAStatement.fn; // look for fn
            if ((temps.charAt(0) == 't') && (temps.charAt(1) != 's'))
                type = MVAStatement.t; // look for time
            if (temps.charAt(0) == 'd')
                type = MVAStatement.d; // look for date
            if ((temps.charAt(0) == 'o') && (temps.charAt(1) == 'j'))
                type = MVAStatement.oj; // look for date
            if (temps.startsWith("escape ", 0))
                type = MVAStatement.escape;
            // convert the ODBC literal data to SAS literal data
            // individual character moves are messier than wsprintf and strncpy
            // but much faster
            switch (type)
            {
                case MVAStatement.t:
                    // time = t'hh:mm:ss' -> 'hh:mm:ss't (11 to 11)
                    // trim leading t and all leading and trailing blanks
                    sub2 = sub2.substring(1);
                    sub2 = sub2.trim();
                    if (sub2.length() < 10)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTime);
                    }
                    if (sub2.charAt(0) != '\'' || sub2.charAt(9) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTime);
                    }
                    stmt = new String(sub1 + " " + sub2.substring(0) + "t "
                            + sub3);
                    break;
                case MVAStatement.d:
                    // date = d'yyyy-mm-dd' -> 'ddmmmyyyy'd (13 to 12)
                    // trim leading d and all leading and trailing blanks
                    sub2 = sub2.substring(1);
                    sub2 = sub2.trim();
                    if (sub2.length() < 12)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    if (sub2.charAt(0) != '\'' || sub2.charAt(11) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    temp = new Integer(sub2.substring(6, 8));
                    monthIndex = temp.intValue();
                    if ((monthIndex < 1) || (monthIndex > 12))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedDate);
                    }
                    stmt = new String(sub1 + " '" + sub2.substring(9, 11)
                            + months[monthIndex] + sub2.substring(1, 5) + "'d "
                            + sub3);
                    break;
                case MVAStatement.ts:
                    // timestamp = ts'yyyy-mm-dd hh:mm:ss[.f..f]' ->
                    // 'ddmmmyyyy:hh:mm:ss[.f..f]'dt
                    // trim leading ts and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    if (sub2.length() < 21)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    if (sub2.charAt(0) != '\'')
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    k = sub2.indexOf('\'', 1); // find trailing quote
                    if (k < 20)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    temp = new Integer(sub2.substring(6, 8));
                    monthIndex = temp.intValue();
                    if ((monthIndex < 1) || (monthIndex > 12))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedTimeStamp);
                    }
                    stmt = new String(sub1 + " '" + sub2.substring(9, 11)
                            + months[monthIndex] + sub2.substring(1, 5) + ":"
                            + sub2.substring(12) + "dt " + sub3);
                    break;
                case MVAStatement.oj:
                    // 'oj table LEFT OUTER JOIN table on condition' -> 'table
                    // left JOIN b on condition'
                    // trim leading ts and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    temps = sub2.toUpperCase();
                    b = findFirstString(temps, "OUTER", 0);
                    if (b == -1)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedOuterJoin);
                    }
                    stmt = new String(sub1 + " " + sub2.substring(0, b)
                            + sub2.substring(b + 5) + " " + sub3);
                    break;
                case MVAStatement.escape: // escape character processing
                    // find character
                    b = sub2.indexOf('\'');
                    if ((b == -1) || (sub2.length() < b + 2)
                            || (sub2.charAt(b + 2) != '\''))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedEscape);
                    }
                    char escapeChar = sub2.charAt(b + 1);
                    temps = new String(sub1 + " " + sub3);
                    String newStmt = new String();
                    length = temps.length();
                    i = 0;
                    int curEscape = -2;
                    inquotes = -1;
                    while (i < length)
                    {
                        if ((temps.charAt(i) == '\'')
                                || (temps.charAt(i) == '\"'))
                        {
                            if (inquotes == -1)
                                inquotes = temps.charAt(i);
                            else if (inquotes == temps.charAt(i))
                                inquotes = -1;
                        }
                        if ((inquotes != -1) && (temps.charAt(i) == escapeChar)
                                && (i != curEscape + 1))
                        {
                            curEscape = i;
                        }
                        else
                        {
                            newStmt = newStmt + temps.charAt(i);
                        }
                        i++;
                    }
                    stmt = newStmt.trim();
                    break;
                case MVAStatement.fn: // fn function
                    // trim leading fn and all leading and trailing blanks
                    sub2 = sub2.substring(2);
                    sub2 = sub2.trim();
                    b = findFirstString(sub2, "(", 0);
                    if (b == -1)
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                    }
                    temps = sub2.substring(0, b);
                    temps = temps.toUpperCase();
                    funcEntry tempfunc = (funcEntry) sasfns.get(temps);
                    if ((tempfunc == null) || (tempfunc.sasFn == null))
                    {
                        MVALocalizedError
                                .throwLocalizedException(VendorCode.MVAStatement_undefinedFunction);
                    }
                    sub2 = new String(tempfunc.sasFn + sub2.substring(b));
                    if (tempfunc.specialHandling != 0)
                    {
                        switch (tempfunc.specialHandling)
                        {
                            case funcEntry.changeConcat:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                i = b + 1;
                                c = -1;
                                inquotes = -1;
                                nested = 0;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                String temp1 = new String(sub2.substring(b + 1,
                                        c)
                                        + " || ");
                                b = findLastString(sub2.substring(c + 1), ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                b = findLastString(sub2, ")");
                                String temp2 = new String(sub2.substring(c + 1,
                                        b));
                                if (temp2.equals(""))
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedConcatFunction);
                                }
                                sub2 = new String(temp1 + temp2);
                                break;
                            case funcEntry.changeLeft:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -1;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedLeftFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + ",1"
                                        + sub2.substring(c));
                                break;
                            case funcEntry.changeLTrim:
                                sub2 = new String(sub2 + ")");
                                break;
                            case funcEntry.changeRepeat:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = 0;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ')')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedRepeatFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + "-1"
                                        + sub2.substring(c));
                                break;
                            case funcEntry.changeSpace:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedSpaceFunction);
                                }
                                sub2 = new String(sub2.substring(0, b + 1)
                                        + "' ',(" + sub2.substring(b + 1)
                                        + "-1)");
                                break;
                            case funcEntry.changePower:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -2;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedPowerFunction);
                                }
                                sub2 = new String(sub2.substring(0, c) + ")**("
                                        + sub2.substring(c + 1) + ")");
                                break;
                            case funcEntry.changeRand:
                                b = findFirstString(sub2, "(", 0);
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                c = findLastString(sub2, ")");
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedFunction);
                                }
                                if ((c - b) == 1)
                                    sub2 = new String(sub2.substring(0, b + 1)
                                            + "0" + sub2.substring(b + 1));
                                break;
                            case funcEntry.changeRound:
                                i = b = 0;
                                c = -1;
                                inquotes = -1;
                                nested = -1;
                                length = sub2.length();
                                while (i < length)
                                {
                                    if (sub2.charAt(i) == '(')
                                        nested++;
                                    if (sub2.charAt(i) == ')')
                                        nested--;
                                    if ((sub2.charAt(i) == '\'')
                                            || (sub2.charAt(i) == '\"'))
                                    {
                                        if (inquotes == -1)
                                            inquotes = sub2.charAt(i);
                                        else if (inquotes == sub2.charAt(i))
                                            inquotes = -1;
                                    }
                                    if ((sub2.charAt(i) == ',')
                                            && (inquotes == -1)
                                            && (nested == 0))
                                    {
                                        c = i;
                                        break;
                                    }
                                    else
                                        i++;
                                }
                                if (c == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedRoundFunction);
                                }
                                sub2 = new String(sub2.substring(0, c + 1)
                                        + "10**-(" + sub2.substring(c + 1)
                                        + ")");
                                break;
                            case funcEntry.changeDayname:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedDayNameFunction);
                                }
                                sub2 = new String(sub2.substring(0, b)
                                        + ", DowName.)" + sub2.substring(b));
                                break;
                            case funcEntry.changeDayofyear:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedDayOfYearFunction);
                                }
                                sub2 = new String(sub2 + ", 1000)");
                                break;
                            case funcEntry.changeMonthname:
                                b = findLastString(sub2, ")");
                                if (b == -1)
                                {
                                    MVALocalizedError
                                            .throwLocalizedException(VendorCode.MVAStatement_malformedChangeMonthName);
                                }
                                sub2 = new String(sub2.substring(0, b)
                                        + ", MonName.)" + sub2.substring(b));
                                break;
                            default:
                                MVALocalizedError
                                        .throwLocalizedException(VendorCode.MVAStatement_undefinedFunction);
                        }
                    }
                    stmt = new String(sub1 + " " + sub2 + " " + sub3);
                    stmt = stmt.trim();
                    break;
                default: // other = bad syntax and let SAS complain
                    return stmt;
            } // end of switch to particular type of escape sequence
        } // while particular escape string found
        // Now, look for Create statements
        boolean scanStmt = false;
        temps = stmt.toUpperCase();
        StringTokenizer st = new StringTokenizer(temps, " ");
        String token;
        if (st.hasMoreTokens())
        {
            token = st.nextToken();
            if (token.equals("CREATE") && st.hasMoreTokens())
            {
                token = st.nextToken();
                if (token.equals("TABLE"))
                {
                    scanStmt = true;
                }
            }
            else
            {
                if (token.equals("ALTER") && st.hasMoreTokens())
                {
                    token = st.nextToken();
                    if (token.equals("TABLE"))
                    {
                        while (st.hasMoreTokens())
                        {
                            token = st.nextToken();
                            if ((token.equals("ADD") || token.equals("MODIFY"))
                                    && st.hasMoreTokens())
                            {
                                scanStmt = true;
                                token = st.nextToken();
                                if (token.equals("COLUMN"))
                                {
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
        /*
         * Look for three part names: schema.table.column and truncate to
         * table.column
         */
        i = b = 0;
        c = -1;
        int state = 0;
        int dots = 0;
        int dot1 = 0;
        String tempStmt;
        int inquotes = -1;
        int edge = 0;
        // BFB 08Apr98 int length = 0;
        // damack - 12/03/2001 - S0106244
        if (stmt.indexOf(".") != -1)
        {
            while ((i = stmt.indexOf(" .", i)) != -1)
            {
                int dotidx = stmt.indexOf(".", i);
                if ((dotidx + 1) >= stmt.length()
                        || stmt.charAt(dotidx + 1) == ' ')
                    i++; // no op
                else
                {
                    tempStmt = stmt;
                    stmt = tempStmt.substring(0, i) + tempStmt.substring(i + 1);
                }
            }
            i = 0;
            char curChar = ' ';
            while (i < stmt.length())
            {
                curChar = stmt.charAt(i);
                if ((curChar == '\'') || (curChar == '\"'))
                {
                    if (inquotes == -1)
                        inquotes = curChar;
                    else if (inquotes == curChar)
                        inquotes = -1;
                }
                if (((Character.isLetterOrDigit(curChar)) || (curChar == '_') || (curChar == '.'))
                        && (inquotes == -1))
                {
                    if (state == 0)
                    {
                        dots = 0;
                        edge = i;
                        state = 1;
                    }
                    else
                    {
                        if (curChar == '.')
                        {
                            dots++;
                            if (dots == 1)
                                dot1 = i;
                        }
                    }
                }
                else
                {
                    if (state == 1)
                    {
                        if (dots > 1)
                        {
                            tempStmt = stmt;
                            stmt = tempStmt.substring(0, edge)
                                    + tempStmt.substring(dot1 + 1);
                            state = 0;
                            i = 0;
                        }
                        else
                        {
                            state = 0;
                        }
                    }
                }
                i++;
            }
            if (state == 1)
            {
                if (dots > 1)
                {
                    tempStmt = stmt;
                    stmt = tempStmt.substring(0, edge)
                            + tempStmt.substring(dot1 + 1);
                    state = 0;
                    i = 0;
                }
            }
        }
        if (scanStmt == false)
        {
            return stmt;
        }
        // Need to scan the statement looking for column types that must be
        // converted to SAS types
        // types and add appropriate format statements
        // Replace SQL types with SAS types
        // SQL Type = INTEGER
        b = 0;
        while (((i = findFirstString(stmt, "INTEGER", b)) != -1)
                || ((i = findFirstString(stmt, "integer", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 7))
                    && ((stmt.charAt(b + 7) == ' ')
                            || (stmt.charAt(b + 7) == ',') || (stmt
                            .charAt(b + 7) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 "
                        + stmt.substring(b + 7));
                b = b + 7;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 7)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 ");
                b = b + 7;
            }
            else
            {
                b = b + 7;
            }
        }
        // SQL Type = LONG
        b = 0;
        while (((i = findFirstString(stmt, "LONG", b)) != -1)
                || ((i = findFirstString(stmt, "long", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=11.0 ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = TIME
        b = 0;
        while (((i = findFirstString(stmt, "TIME", b)) != -1)
                || ((i = findFirstString(stmt, "time", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=time8. "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=time8. ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = DATE
        b = 0;
        while (((i = findFirstString(stmt, "DATE", b)) != -1)
                || ((i = findFirstString(stmt, "date", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 4))
                    && ((stmt.charAt(b + 4) == ' ')
                            || (stmt.charAt(b + 4) == ',') || (stmt
                            .charAt(b + 4) == ')')))
            {
                stmt = new String(stmt.substring(0, b) + " num format=date9. "
                        + stmt.substring(b + 4));
                b = b + 4;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 4)))
            {
                stmt = new String(stmt.substring(0, b) + " num format=date9. ");
                b = b + 4;
            }
            else
            {
                b = b + 4;
            }
        }
        // SQL Type = TIMESTAMP
        b = 0;
        while (((i = findFirstString(stmt, "TIMESTAMP", b)) != -1)
                || ((i = findFirstString(stmt, "timestamp", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 9))
                    && ((stmt.charAt(b + 9) == ' ')
                            || (stmt.charAt(b + 9) == ',') || (stmt
                            .charAt(b + 9) == ')')))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19.2 " + stmt.substring(b + 9));
                b = b + 9;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 9)))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19. ");
                b = b + 9;
            }
            else
            {
                b = b + 9;
            }
        }
        // SQL Type = datetime
        /**
         * S0164788 fix
         */
        b = 0;
        while (((i = findFirstString(stmt, "DATETIME", b)) != -1)
                || ((i = findFirstString(stmt, "datetime", b)) != -1))
        {
            b = i;
            if (((stmt.charAt(b - 1) == ' ') || (stmt.charAt(b - 1) == '('))
                    && (stmt.length() > (b + 8))
                    && ((stmt.charAt(b + 8) == ' ')
                            || (stmt.charAt(b + 8) == ',') || (stmt
                            .charAt(b + 8) == ')')))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19.2 " + stmt.substring(b + 8));
                b = b + 8;
            }
            else if ((stmt.charAt(b - 1) == ' ') && (stmt.length() == (b + 8)))
            {
                stmt = new String(stmt.substring(0, b)
                        + " num format=datetime19. ");
                b = b + 8;
            }
            else
            {
                b = b + 8;
            }
        }
        // Finished with modifiying SQL
        return stmt;
    }

    /*
     * This method exists only to ignore values within quotes, otherwise look
     * for the token starting at the beginning of the string
     */
    protected int findFirstString(String stmt, String token, int offset)
    {
        int curoffset = offset;
        boolean singleQuote = false;
        boolean doubleQuote = false;
        int len = stmt.length();
        int tlen = token.length() - 1;
        while (curoffset < len)
        {
            if (stmt.charAt(curoffset) == '\'')
            {
                if (singleQuote == true)
                    singleQuote = false;
                else
                    singleQuote = true;
            }
            if (stmt.charAt(curoffset) == '\"')
            {
                if (doubleQuote == true)
                    doubleQuote = false;
                else
                    doubleQuote = true;
            }
            if ((!singleQuote) && (!doubleQuote)
                    && (stmt.charAt(curoffset) == token.charAt(0)))
            {
                int j = 1;
                while (j <= tlen)
                {
                    if ((curoffset + j) > (len - 1))
                        return -1;
                    if (stmt.charAt(curoffset + j) != token.charAt(j))
                        break;
                    j++;
                }
                if (j >= tlen)
                {
                    return curoffset;
                }
            }
            curoffset++;
        }
        return -1;
    }

    /*
     * This method exists only to ignore values within quotes, otherwise look
     * for the token starting at the end of the string
     */
    private int findLastString(String stmt, String token)
    {
        int curoffset = stmt.length() - 1;
        boolean singleQuote = false;
        boolean doubleQuote = false;
        // BFB 08Apr98 int len = stmt.length();
        int tlen = token.length() - 1;
        while (curoffset >= 0)
        {
            if (stmt.charAt(curoffset) == '\'')
            {
                if (singleQuote == true)
                    singleQuote = false;
                else
                    singleQuote = true;
            }
            if (stmt.charAt(curoffset) == '\"')
            {
                if (doubleQuote == true)
                    doubleQuote = false;
                else
                    doubleQuote = true;
            }
            if ((!singleQuote) && (!doubleQuote)
                    && (stmt.charAt(curoffset) == token.charAt(tlen)))
            {
                int j = 1;
                while (j < tlen)
                {
                    if ((curoffset - j) < 0)
                        return -1;
                    if (stmt.charAt(curoffset - j) != token.charAt(tlen - j))
                        break;
                    j++;
                }
                if (j >= tlen)
                {
                    return (curoffset - tlen);
                }
            }
            curoffset--;
        }
        return -1;
    }

    public XUtil getXUtil()
    {
        return m_xUtil;
    }

    /**
     * Executes the given SQL statement, which may return multiple results, and
     * signals the driver that any auto-generated keys should be made available
     * for retrieval. The driver will ignore this signal if the SQL statement is
     * not an INSERT statement. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public int executeUpdate(String sql, int autoGeneratedKeys)
            throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return 0;
    }

    /**
     * Executes the given SQL statement and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval. The driver will ignore the array if the SQL statement is
     * not an INSERT statement. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public int executeUpdate(String sql, int[] columnIndexes)
            throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return 0;
    }

    /**
     * Executes the given SQL statement and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval. The driver will ignore the array if the SQL statement is
     * not an INSERT statement. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public int executeUpdate(String sql, String[] columnNames)
            throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return 0;
    }

    /**
     * Executes the given SQL statement and signals the driver with the given
     * flag about whether the auto-generated keys produced by this Statement
     * object should be made available for retrieval. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public boolean execute(String sql, int autoGeneratedKeys)
            throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return false;
    }

    /**
     * Executes the given SQL statement and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval. The driver will ignore the array if the SQL statement is
     * not an INSERT statement. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public boolean execute(String sql, int[] columnIndexes) throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return false;
    }

    /**
     * Executes the given SQL statement and signals the driver that the
     * auto-generated keys indicated in the given array should be made available
     * for retrieval. The driver will ignore the array if the SQL statement is
     * not an INSERT statement. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public boolean execute(String sql, String[] columnNames)
            throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return false;
    }

    /**
     * Retrieves any auto-generated keys created as a result of executing this
     * Statement object. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public ResultSet getGeneratedKeys() throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return null;
    }

    /**
     * Moves to this Statement object's next result, deals with any current
     * ResultSet object(s) according to the instructions specified by the given
     * flag, and returns true if the next result is a ResultSet object. Method
     * not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public boolean getMoreResults(int current) throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return false;
    }

    /**
     * Retrieves the result set holdability for ResultSet objects generated by
     * this Statement object. Method not implemented.
     * 
     * @exception SQLException
     *                Always thrown, to indicate that this method is not
     *                implemented.
     */
    public int getResultSetHoldability() throws SQLException
    {
        MVALocalizedError
                .throwLocalizedException(VendorCode.General_methodNotImplemented);
        return 0;
    }
}
