/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */
package com.sas.rio;

/**
 * <P>This is the Options class which defines the SAS options like By clause, 
 * while creating a table. This class also defines the SQL options like OrderBy 
 * clause and Where clause.
 * This class is used to pass the SAS and SQL options as arguments for table
 * creation and table access, but more than that is also used to disect the
 * SQL statement in case we want to look at the SQL statement that caused a
 * problem. This acts like an archive for SQL queries.
 */
public class Options {
    private String byText;
    private String orderByText;
    private String whereText;
    private String optionsText;

    /**
     * Constructs an Options object.
     */
    public Options() {
    }
    /**
     * Sets options string to be passed in while creating a table. The options can be
     * any dataset options that are appropriate given the underlying library and engine.
     * The options string may be NULL.
     *
     * @param text An input String of options. Options may be NULL.
     * @param appendFlag A boolean flag to append or not append options text.
     * @exception RIOException This exception is never thrown.
     */
    public void setOptions(String text, boolean appendFlag) throws RIOException {
        if((text != null) && (text.length() != 0)) {
            if(appendFlag == true)
                optionsText = optionsText + new String(", " + text.trim());
            else
                optionsText = text;
        }
    }
    /**
     * Sets BY statement text.
     *
     * @param text An input String of BY statement text.
     * @param appendFlag A boolean flag to append or not append text.
     * @exception RIOException This exception is never thrown.
     */
    public void setByStatement(String text, boolean appendFlag) throws RIOException {
        if((text != null) && (text.length() != 0)) {
            if(appendFlag == true)
                byText = byText + new String(", " + text.trim());
            else
                byText = text;
        }
    }
    /**
     * Sets ORDER BY statement text.
     *
     * @param text An input String of ORDER BY statement text.
     * @param appendFlag A boolean flag to append or not append text.
     * @exception RIOException This exception is never thrown.
     */
    public void setOrderByStatement(String text, boolean appendFlag) throws RIOException {
        if((text != null) && (text.length() != 0)) {
            if(appendFlag == true)
                orderByText = orderByText + new String(", " + text.trim());
            else
                orderByText = text;
        }
    }
    /**
     * Sets WHERE clause text.
     *
     * @param text An input String of WHERE clause statement text.
     * @param appendFlag boolean flag to append or not append text.
     * @exception RIOException This exception is never thrown.
     */
    public void setWhereClause(String text, boolean appendFlag) throws RIOException {
        if((text != null) && (text.length() != 0)) {
            if (appendFlag == true)
                whereText = whereText + new String(" and " + text.trim());
            else
                whereText = text;
        }
    }
    /**
     * Gets an MVAResultSet object containing rows of a by-group.
     * Note: Method not yet implemented.
     *
     * @return MVAResultSet object containing rows of a by-group.
     * @exception RIOException This exception is thrown if a RIOException is detected.
     */
    public MVAResultSet getByGroup() throws RIOException {
        throw new RIOException("Method not yet implemented.");
    }
    /**
     * Gets the ORDER BY statement text.
     *
     * @return String The ORDER BY statement text.
     * @exception RIOException This exception is never thrown.
     */
    public String getOrderByStatement() throws RIOException {
        return orderByText;
    }
    /**
     * Gets the SAS BY statement text.
     *
     * @return String The SAS BY statement text.
     * @exception RIOException This exception is never thrown.
     */
    public String getByStatement() throws RIOException {
        return byText;
    }
    /**
     * Gets the Where clause statement text.
     *
     * @return String The Where clause statement text.
     * @exception RIOException This exception is never thrown.
     */
    public String getWhereClause() throws RIOException {
        return whereText;
    }
    /**
     * Gets the options text.
     *
     * @return String The options text.
     * @exception RIOException This exception is never thrown.
     */
    public String getOptions() throws RIOException {
        return optionsText;
    }
}



