/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */

package com.sas.rio;

import java.sql.ResultSetMetaData;


/**
 * <P>This is the interface definition of RIOResultSetMetaDataInterface.
 * This interface extends the java.sql.ResultSetMetaData interface functionality.
 */
public interface RIOResultSetMetaDataInterface extends java.sql.ResultSetMetaData {

    /**
     * Gets the format name of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return A String containing the format name of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    String getColumnFormatName(int columnIndex) throws RIOException;

    /**
     * Gets the format names of the columns in the ResultSet.
     *
     * @return An array of Strings containing the format names of the columns.
     * @exception RIOException This exception may be thrown.
     */
    String[] getColumnFormatNames() throws RIOException;

    /**
     * Gets the format length of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the format length of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    int getColumnFormatLength(int columnIndex) throws RIOException;

    /**
     * Gets the format lengths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the format lengths of the columns.
     * @exception RIOException This exception may be thrown.
     */
    int[] getColumnFormatLengths() throws RIOException;

    /**
     * Gets the format width of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the format width of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    int getColumnFormatWidth(int columnIndex) throws RIOException;

    /**
     * Gets the format widths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the format widths of the columns.
     * @exception RIOException This exception may be thrown.
     */
    int[] getColumnFormatWidths() throws RIOException;

    /**
     * Gets the format decimal of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the format decimal of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    int getColumnFormatDecimal(int columnIndex) throws RIOException;

    /**
     * Gets the format lengths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the format lengths of the columns.
     * @exception RIOException This exception may be thrown.
     */
    int[] getColumnFormatDecimals() throws RIOException;

    /**
     * Gets the informat name of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return A String containing the informat name of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    String getColumnInformatName(int columnIndex) throws RIOException;

    /**
     * Gets the informat names of the columns in the ResultSet.
     *
     * @return An array of String values containing the informat names of the columns.
     * @exception RIOException This exception may be thrown.
     */
    String[] getColumnInformatNames() throws RIOException;

    /**
     * Gets the informat length of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer value containing the informat length of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    int getColumnInformatLength(int columnIndex) throws RIOException;
    
    /**
     * Gets the informat lengths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the informat lengths of the columns.
     * @exception RIOException This exception may be thrown.
     */
    int[] getColumnInformatLengths() throws RIOException;

    /**
     * Gets the informat width of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer containing the informat width of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    int getColumnInformatWidth(int columnIndex) throws RIOException;

    /**
     * Gets the informat widths of the columns in the ResultSet.
     *
     * @return An array of integer values containing the informat widths of the columns.
     * @exception RIOException This exception may be thrown.
     */
    int[] getColumnInformatWidths() throws RIOException;

    /**
     * Gets the informat decimal of the column.
     *
     * @param columnIndex The first column is 1, the second column is 2, and so on.
     * @return An integer containing the informat decimal of the column.
     * @exception RIOException This exception is thrown if the column index is not valid.
     */
    int getColumnInformatDecimal(int columnIndex) throws RIOException;

    /**
     * Gets the informat decimals of the columns in the ResultSet.
     *
     * @return An array of integer values containing the informat decimals of the columns.
     * @exception RIOException This exception may be thrown.
     */
    int[] getColumnInformatDecimals() throws RIOException;

}
