/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */

package com.sas.rio;

import java.sql.Statement;
import java.sql.ResultSet;

//
//History Records:
//
// Date          Name                Remarks
// 05/04/99 M Srinivasan   - Fix to S0052966. Supported multiple librefs in Connection
//                           properties by adding libref name to getTable(), createTable(),
//                           and deleteTable() methods.
// 06/08/99 M Srinivasan   - S0056456: Options object is now an options String whose contents can be
//                           any SAS dataset options. Made this change to getTable.
//                         - Get rid of Column object. OpenDataSet does not support subsetting columns.
/**
 * <P>This is the interface definition of RIOStatementInterface.
 * This interface extends the java.sql.Statement interface functionality.
 */
public interface RIOStatementInterface extends java.sql.Statement {
    /**
     * Creates a new table with the name specified in the tableName String.
     * The columns to be created and their attributes are passed through the
     * columnObject parameter. Any SAS options to be sent with the create
     * table will be passed through the options parameter.
     *
     * @param libref The libref name. Cannot be NULL.
     * @param tableName The name of the table to be created. Table name cannot be NULL.
     * @param columnObject A Column Object array containing column names and attributes. Column object
     * cannot be NULL.
     * @param options A String of applicable SAS dataset options. May be NULL.
     * @exception RIOException This exception may be thrown.
     */
    void createTable(String libref, String tableName, Column[] columnObject, String options) throws RIOException;

    /**
     * Gets an MVAResultSet object for the given table name. By default all the
     * columns and all rows are selected.
     *
     * @param libref The libref name. Cannot be NULL.
     * @param tableName The name of the table to be opened. Table name cannot be NULL.
	 * By default, the driver looks for a dataset to open, if one doesn't exist it looks for a
	 * view to open. If both exist, it opens a dataset.
     * @return An MVAResultSet object. All columns and all rows are selected.
     * @exception RIOException This exception may be thrown.
     */
    ResultSet getTable(String libref, String tableName) throws RIOException;

    /**
     * Gets an MVAResultSet object for the given table name.
     *
     * @param libref The libref name. Cannot be NULL.
     * @param tableName The name of the table to be opened.
     * @param tableType The type of the table to be opened.It could be "DATA" or "VIEW".
     * If NULL, the driver looks for a dataset to open, if one doesn't exist it looks for a
	 * view to open. If both exist, it opens a dataset. This parameter is currently being ignored and
	 * defaulted to NULL.
     * @param password A String array of 3 elements specifying READ, WRITE and ALTER passwords if any.
	 * This parameter can be NULL. The first element is the READ password, the second the WRITE password
     * and the third the ALTER password.
     * @param options A String whose contents can be any SAS dataset options appropriate to the
	 * underlying engine and library. This includes the WHERE clause. options can be NULL.
     * @return An MVAResultSet object. The ResultSet could be a subset of the actual set of columns and rows.
     * @exception RIOException This exception may be thrown.
     */
    ResultSet getTable(String libref, String tableName, String tableType, String[] password, String options) throws RIOException;

	/**
     * Deletes the table with the name specified in the tableName String.
     *
     * @param libref The libref name. Cannot be NULL.
     * @param tableName The name of the table to be deleted. By default,
	 * the tableName dataset will be deleted.
     * @exception RIOException This exception may be thrown.
     */
    void deleteTable(String libref, String tableName) throws RIOException;

	/**
     * Deletes the table for the specified tableType with the name specified in the tableName String.
     *
     * @param libref The libref name. Cannot be NULL.
     * @param tableName The name of the table to be deleted.
     * @param tableType The type of the table. It could be "DATA" or "VIEW" or any table type
	 * available in the database. If NULL, the tableName dataset will be deleted.
     * @param alterPassword The alter password.
     * @exception RIOException This exception may be thrown.
     */
    void deleteTable(String libref, String tableName, String tableType, String alterPassword) throws RIOException;

}
