/* $Id: RIOUtil.java,v 1.14.4.2 2004/04/30 19:07:23 damack Exp $ */
/**
 * NAME - RIOUtil
 * DATE - 7/31/2002
 * AUTHOR - Dave Mackie, SAS Institute
 * PURPOSE - abstract out connection details from the IOM JDBC Driver,
 *           to make the driver more portable across servers.
 * one RIOUtil object per connection, please....
 * 12/17/2002 - S0142734: fixing a data integrity issue discovered by Randy W.
 *                        the missing values code in addRows was correctly accounting
 *                        for a missing value in the "formatted" case, but it was not 
 *                        advancing the index counter for char values.  this lead to 
 *                        the column values being "pushed" to the right, meaning that the 
 *                        cache was corrupted and data integrity was compromised(bad)
 * 
 * 03/24/2003 - S0191393: fix issue with passing nulls in updateRow.
 * 03/27/2003 - S0192553: during cleanup from failed getDBMSConnection call, possible NPE.
 * 04/21/2003 - S0195329: add a columntypes parm to updateRow.  Modified updateRow to handle 
 *                        case when bindkey is FORMATTED. (This means that data that we get from
 *                        the server is formatted, so the old assumption that a 'double' was a numeric
 *                        column (which I guess would still hold true) and a 'string' was a varchar
 *                        was unsound). 
 * 05/06/2003 - S0196790: changed readRecords to use DataHolder to return data instead of an Object[][].
 *                        (perf improvement)
 * 06/03/2003 - S0205046: changed getWorkspace to lowercase the WorkspaceConstants.ENCRYPTION* property
 *                        keys before attempting to look them up.
 * 12/22/2003 - S0230433: changed disconnect() to manually close the orb, instead of just closing the 
 *                        workspace object.
 * 30Apr2004  - S0207156: changed assignLibrefs so that bad librefs don't cause trailing librefs to fail. 
 */
package com.sas.rio;
import java.rmi.RemoteException;
import com.sas.iom.SAS.IDataSet;
import com.sas.iom.SAS.ILibref;
import com.sas.iom.SAS.IDBMSConnection;
import com.sas.iom.SAS.IDataService;
import com.sas.iom.SAS.IWorkspace;
import com.sas.iom.SAS.IWorkspaceHelper;
import com.sas.iom.SAS.ISASAutomation;
import com.sas.iom.SAS.ISASAutomationHelper;
import com.sas.iom.WorkspaceConstants;
import com.sas.iom.SASIOMDefs.DoubleSeqHolder;
import org.omg.CORBA.IntHolder;
import com.sas.iom.SASIOMDefs.LongSeqHolder;
import com.sas.iom.SASIOMDefs.ShortSeqHolder;
import org.omg.CORBA.StringHolder;
import com.sas.iom.SASIOMDefs.StringSeqHolder;
import com.sas.iom.SASIOMDefs.OctetSeqHolder;
import com.sas.iom.SASIOMDefs.VariableArray2dOfStringHolder;
import com.sas.iom.SASIOMDefs.VariableArray2dOfDoubleHolder;
import com.sas.iom.SASIOMDefs.VariableArray2dOfOctetHolder;
import java.sql.SQLWarning;
import java.sql.SQLException;
//import com.sas.iom.SAS.BaseType;
import com.sas.iom.SAS.NotMissing;
import com.sas.iom.SAS.RIOStatus;
import com.sas.iom.SAS.NumericMissingDot;
import com.sas.iom.SAS.IDataSetPackage.BindKeys;
import com.sas.iom.SASIOMDefs.DateTimeHolder;
import java.sql.ResultSet;
import com.sas.iom.JDBCConstants;
import com.sas.iom.LDAPConstants;
import java.util.Vector;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.HashMap;
import java.sql.Types;
public class RIOUtil implements XUtil
{
	int DEFAULT_CAPACITY = 64; // share has a max of 64 result sets, so....
	IDataSet m_dataSet = null;
	IDBMSConnection m_connection = null;
	SQLWarning warning = null;
	IDataService m_dataService = null;
	IWorkspace m_workspace = null;
	ILibref m_libref = null;
	ISASAutomation m_automation = null;
	HashMap dataMap = null;
	int EOF = -1;
	int BOF = -2;
	int AOK = -3;
	int m_status = AOK;
	boolean m_connected = false;
	Vector m_librefList = null;
	String[] libnames = null;
	Properties m_info = null;
	SQLWarning warnings = null;
	boolean m_fromSSI = false;
	boolean m_fromDataService = false;
	byte[] m_bookmarkStorage = null;
	/** 
	 * discussion of the various Ixxxx classes should go here.
	 */
	/***************************************************************************/
	/**
	 * constructor 1: takes properties and url.
	 */
	public RIOUtil(String url, java.util.Properties p) throws SQLException
	{
		dataMap = new HashMap(DEFAULT_CAPACITY);
		m_info = p;
		try
		{
			m_workspace = getWorkspace(m_info);
			m_dataService = m_workspace.DataService();
			m_connection = getDBMSConnection(m_dataService);
			//executeUndoPolicyNoneStatement(p); //?
			assignLibrefs(
				m_info.getProperty(JDBCConstants.LIBREFS.toLowerCase()));
			m_connected = true;
		}
		catch (Exception e)
		{
			if (m_connection != null)
			{
				m_connection._release();
				m_connection = null;
			}
			if (m_dataService != null)
			{
				if (m_librefList != null) // S0192553
				{
					for (int x = 0; x < m_librefList.size(); x++)
					{
						try
						{
							m_dataService.DeassignLibref(libnames[x]);
						}
						catch (Exception ex)
						{
						}
					}
				}
				m_dataService._release();
				m_dataService = null;
			}
			ILibref sasLibref;
			if (m_librefList != null)
			{
				for (int x = 0; x < m_librefList.size(); x++)
				{
					sasLibref = (ILibref) m_librefList.elementAt(0);
					if (sasLibref != null)
					{
						sasLibref._release();
						sasLibref = null;
					}
				}
			}
			if (m_workspace != null)
			{
				m_workspace._release();
				m_workspace = null;
			}
			if (m_automation != null)
			{
				m_automation._release();
				m_automation = null;
			}
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAConnection_error,
				null,
				e);
		}
	}
	public RIOUtil(IDataService ids, java.util.Properties p)
		throws SQLException
	{
		m_fromSSI = true;
		m_info = p;
		dataMap = new HashMap(DEFAULT_CAPACITY);
		m_dataService = ids;
		try
		{
			m_connection = getDBMSConnection(m_dataService);
			assignLibrefs(
				m_info.getProperty(JDBCConstants.LIBREFS.toLowerCase()));
			m_connected = true;
			m_fromDataService = true;
		}
		catch (Exception e)
		{
			if (m_connection != null)
			{
				m_connection._release();
				m_connection = null;
			}
			if (m_dataService != null)
			{
				if (m_librefList != null) // S0192553
				{
					for (int x = 0; x < m_librefList.size(); x++)
					{
						try
						{
							m_dataService.DeassignLibref(libnames[x]);
						}
						catch (Exception ex)
						{
						}
					}
				}
				m_dataService._release();
				m_dataService = null;
			}
			ILibref sasLibref;
			if (m_librefList != null)
			{
				for (int x = 0; x < m_librefList.size(); x++)
				{
					sasLibref = (ILibref) m_librefList.elementAt(0);
					if (sasLibref != null)
					{
						sasLibref._release();
						sasLibref = null;
					}
				}
			}
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAConnection_error,
				null,
				e);
		}
	}
	public RIOUtil(IWorkspace iw, Properties p) throws SQLException
	{
		m_fromSSI = true;
		m_info = p;
		dataMap = new HashMap(DEFAULT_CAPACITY);
		m_workspace = iw;
		try
		{
			m_dataService = m_workspace.DataService();
			m_connection = getDBMSConnection(m_dataService);
			assignLibrefs(
				m_info.getProperty(JDBCConstants.LIBREFS.toLowerCase()));
			m_connected = true;
			m_fromDataService = true;
		}
		catch (Exception e)
		{
			if (m_connection != null)
			{
				m_connection._release();
				m_connection = null;
			}
			if (m_dataService != null)
			{
				if (m_librefList != null) // S0192553
				{
					for (int x = 0; x < m_librefList.size(); x++)
					{
						try
						{
							m_dataService.DeassignLibref(libnames[x]);
						}
						catch (Exception ex)
						{
						}
					}
				}
				m_dataService._release();
				m_dataService = null;
			}
			ILibref sasLibref;
			if (m_librefList != null)
			{
				for (int x = 0; x < m_librefList.size(); x++)
				{
					sasLibref = (ILibref) m_librefList.elementAt(0);
					if (sasLibref != null)
					{
						sasLibref._release();
						sasLibref = null;
					}
				}
			}
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAConnection_error,
				null,
				e);
		}
	}
	public boolean isConnected() throws RemoteException
	{
		return m_connected;
	}
	public SQLWarning getWarning() throws RemoteException /** never thrown */
	{
		return warning;
	}
	/**
	 * Q: is it faster to return objects or primitives in RMI?
	 */
	public String getDBMSVersion() throws RemoteException
	{
		try
		{
			IWorkspace sasWorkspace = m_dataService.Parent();
			com.sas.iom.SAS.IUtilities sasUtilities = sasWorkspace.Utilities();
			com.sas.iom.SAS.IHostSystem hostSystem = sasUtilities.HostSystem();
			StringSeqHolder systemInfoHolder = new StringSeqHolder();
			VariableArray2dOfStringHolder machineInfoHolder =
				new VariableArray2dOfStringHolder();
			hostSystem.GetInfo(systemInfoHolder, machineInfoHolder);
			hostSystem._release();
			sasUtilities._release();
			sasWorkspace._release();
			String[] systemInfo = systemInfoHolder.value;
			String sasVersion = systemInfo[0];
			return sasVersion;
		}
		catch (com.sas.iom.SASIOMDefs.GenericError ge)
		{
			return null;
		}
	}
	/**
	 * Statement methods
	 */
	public int executeUpdate(String sql) throws RemoteException, SQLException
	{
		int updateCount = 0;
		warning = null;
		if ((sql == null) || (sql.length() == 0))
		{

			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAStatement_noStatementError);
		}
		try
		{
			IntHolder statusHolder = new IntHolder();
			StringHolder warningHolder = new StringHolder();
			m_connection.ExecuteStatement(sql, statusHolder, warningHolder);
			updateCount = (int) statusHolder.value;
			String wtxt = warningHolder.value;
			/**
			 * fix for S0102147
			 */
			if (wtxt != null && wtxt.length() > 0)
			{
				warning = new SQLWarning(wtxt);
			}
		}
		catch (Exception e)
		{
			/**
			 * rethrow the exception so that the caller can catch it
			 */
			throw new SQLException(e.toString());
		}
		return updateCount;
	}
	public int executeQuery(String sql, boolean randomAccess)
		throws RemoteException, SQLException
	{
		int updateCount = -1;
		warning = null;
		// Submit the Query
		if (sql != null)
		{
			try
			{
				// note, this is just to get it to compile
				IntHolder statusHolder = new IntHolder();
				IntHolder bookmarkLengthHolder = new IntHolder();
				StringHolder warningHolder = new StringHolder();
				m_dataSet =
					m_connection.ExecuteQuery(
						sql,
						randomAccess,
						bookmarkLengthHolder,
						statusHolder,
						warningHolder);
				//int bookmarkLength = bookmarkLengthHolder.value;
				String wtxt = warningHolder.value;
				/**
				 * fix for S0102147
				 */
				if (wtxt != null && wtxt.length() > 0)
				{
					warning = new SQLWarning(wtxt);
				}
			}
			catch (Exception e)
			{
				throw new SQLException(e.toString());
			}
		}
		else
		{
			/** 
			 * error or exception?
			 */
		}
		if (m_dataSet != null)
		{
			Integer k = new Integer(m_dataSet.hashCode());
			dataMap.put(k, m_dataSet);
			return k.intValue();
		}
		else
			return -1;
	}
	public void deleteTable(
		String libref,
		String tableName,
		String tableType,
		String alterPassword)
		throws RemoteException, SQLException
	{
		if (libref == null || libref.length() == 0)
		{
			//MVALocalizedError.throwLocalizedRIOException(VendorCode.MVAStatement_nullLibrefError );
		}
		try
		{
			m_libref = useLibref(libref);
			if (tableName == null)
				tableName = "";
			if (tableType == null)
				tableType = "";
			if (alterPassword == null)
				alterPassword = "";
			m_libref.DeleteMember(tableName, tableType, alterPassword);
		}
		catch (Exception e)
		{
			throw new SQLException(e.toString());
		}

	}
	public boolean createTable(
		String libref,
		String tableName,
		Column[] columns,
		String options)
		throws RemoteException, SQLException
	{
		IDataSet dataSet = null;
		/**
		 * some sanity checks
		 */
		if (libref == null || libref.length() == 0)
		{
			// TODO: RioExceptions?
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAStatement_nullLibrefError);
		}
		if (tableName == null || tableName.length() == 0)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAStatement_nullTableNameError);
		}
		if (columns == null)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAStatement_nullArrayError);
		}
		int len = columns.length;
		String[] columnNames = new String[len];
		String[] columnLabels = new String[len];
		int[] columnTypes = new int[len];
		int[] columnLengths = new int[len];
		String[] columnFormatNames = new String[len];
		short[] columnFormatWidths = new short[len];
		short[] columnFormatDecimals = new short[len];
		String[] columnInformatNames = new String[len];
		short[] columnInformatWidths = new short[len];
		short[] columnInformatDecimals = new short[len];
		for (int i = 0; i < len; i++)
		{
			if (columns[i] != null)
			{
				try
				{
					columnNames[i] = columns[i].getColumnName();
					columnLabels[i] = columns[i].getColumnLabel();
					columnTypes[i] = columns[i].getColumnType();
					columnLengths[i] = columns[i].getMaxLength();
					columnFormatNames[i] = columns[i].getColumnFormatName();
					columnFormatWidths[i] =
						(short) (columns[i].getColumnFormatWidth());
					columnFormatDecimals[i] =
						(short) (columns[i].getColumnFormatDecimal());
					columnInformatNames[i] = columns[i].getColumnInformatName();
					columnInformatWidths[i] =
						(short) (columns[i].getColumnInformatWidth());
					columnInformatDecimals[i] =
						(short) (columns[i].getColumnInformatDecimal());
				}
				catch (RIOException re)
				{
					MVALocalizedError.throwLocalizedException(
						re.getErrorCode(),
						new Object[] { re.getMessage()});
				}
			}
		}
		IntHolder bookmarkLengthHolder = new IntHolder();
		//flag to set SASSASLibraryExclusiveMemberAccess and SASSASLibraryReadOnlyAccess
		int flags = 0;
		//TODO: need to use options
		String dataSetLabel = "";
		if (options == null)
			options = "";
		try
		{
			m_libref = useLibref(libref);
			dataSet =
				m_libref.CreateDataSet(
					flags,
					tableName,
					options,
					dataSetLabel,
					columnNames,
					columnLabels,
					columnTypes,
					columnLengths,
					columnFormatNames,
					columnFormatWidths,
					columnFormatDecimals,
					columnInformatNames,
					columnInformatWidths,
					columnInformatDecimals,
					bookmarkLengthHolder);
		}
		catch (Exception e)
		{
			throw new SQLException(e.toString());
		}
		try
		{
			if (dataSet != null)
			{
				dataSet.Close();
				dataSet._release();
				dataSet = null;
			}
		}
		catch (Exception e)
		{
			dataSet = null;
		}
		return true;
		/**
		 * need to figure out what we're going to do about the problem of multiple datasets.
		 * this is muy importante.  if we get stuck only being able to have a single dataset
		 * open at a time, then we're eff-ewe-sea-kay-ed.
		 */
		/**
		 * maybe have a "hashcode" or something to identify each result set in a hashmap
		 * or arraylist or some such, and then when you construct another result set, you'll
		 * pass that hash code along with the XUtil object.
		 */
		/**
		 * hmmmm
		 */
	}
	/**
	 * connection method.  
	 * for now, let's cut this out of the XUtil interface.
	 * Having an object will imply connectedness.
	 * I'll add a method to test connection, and another to close the connection
	 * later.
	 */
	/**
	   public boolean connect(String url, java.util.Properties p) throws RemoteException
	   {
	   }
	*/
	/**
	 * ResultSet methods
	 */
	public boolean insertRow(
		int hashCode,
		Object[] data,
		int bindKey,
		int[] columnTypes)
		throws RemoteException, SQLException
	{
		/**
		 * S0195329: ignore the bindkey passed in and use "innerbindkey" set to unformatted.
		 */
		int innerbindKey = BindKeys._BindKeysUnformatted;
		/** S0195329 */
		int j;
		m_dataSet = (IDataSet) dataMap.get(new Integer(hashCode));
		if (m_dataSet == null)
			return false;
		try
		{
			int numericValuesCount = 0;
			int characterValuesCount = 0;
			for (int i = 0; i < data.length; i++)
			{
				if (columnTypes[i] != Types.VARCHAR)
					numericValuesCount++;
				else
					characterValuesCount++;
			}
			double[][] numericValuesForInsert =
				new double[1][numericValuesCount];
			String[][] characterValuesForInsert =
				new String[1][characterValuesCount];
			byte[][] missingNumericValuesForInsert =
				new byte[1][numericValuesCount];
			j = 0;
			for (int i = 0; i < data.length; i++)
			{
				if (columnTypes[i] != Types.VARCHAR)
				{
					numericValuesForInsert[0][j] =
						((Double) data[i]).doubleValue();
					if (Double.isNaN(numericValuesForInsert[0][j]))
					{
						missingNumericValuesForInsert[0][j] =
							(byte) NumericMissingDot.value;
					}
					else
					{
						missingNumericValuesForInsert[0][j] =
							(byte) NotMissing.value;
					}
					j++;
				}
			}
			j = 0;
			for (int i = 0; i < data.length; i++)
			{
				if (columnTypes[i] == Types.VARCHAR)
				{
					if (data[i] == null)
					{
						characterValuesForInsert[0][j] = "";
					}
					else
					{
						characterValuesForInsert[0][j] = data[i].toString();
					}
					j++;
				}
			}
			IntHolder statusHolder = new IntHolder();
			OctetSeqHolder bookmarksInsertHolder = new OctetSeqHolder();
			m_dataSet.AddRecords(
				innerbindKey,
				numericValuesForInsert,
				missingNumericValuesForInsert,
				characterValuesForInsert,
				bookmarksInsertHolder,
				statusHolder);
			int status = statusHolder.value;
			if (status != RIOStatus._RIOStatusRIO_S_OK)
			{
				switch (status)
				{
					case RIOStatus._RIOStatusRIO_E_RecordLocked :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_recordLockedError);
					case RIOStatus._RIOStatusRIO_E_InvalidBookmark :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_recordBookmarkError);
					case RIOStatus._RIOStatusRIO_E_RecDeleted :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_recordDeletedError);
					case RIOStatus._RIOStatusRIO_E_Failure :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_insertServerError);
				}
			}
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAResultSet_insertFailedError,
				null,
				se);
		}
		return true;
	}
	public boolean updateRow(
		int hashCode,
		Object[] data,
		byte[] bookmark,
		int bindKey,
		int[] columnTypes)
		throws RemoteException, SQLException
	{
		/** 
		 * S0195329: set an innerbindkey to unformatted.
		 */
		int innerbindKey = BindKeys._BindKeysUnformatted;
		/** S0195329 */
		int flags = 0;
		byte[] bookmarksUpdate = null;
		int j;
		String[] characterValuesFromLockRecords = null;
		double[] numericValuesFromLockRecords = null;
		m_dataSet = (IDataSet) dataMap.get(new Integer(hashCode));
		if (m_dataSet == null)
			return false;
		try
		{
			/** I'm guessing that m_cursor == blocknum */
			bookmarksUpdate = bookmark;
			StringSeqHolder characterValuesHolder = new StringSeqHolder();
			DoubleSeqHolder numericValuesHolder = new DoubleSeqHolder();
			OctetSeqHolder missingNumericValuesHolder = new OctetSeqHolder();
			m_dataSet.LockRecord(
				flags,
				innerbindKey,
				bookmarksUpdate,
				characterValuesHolder,
				numericValuesHolder,
				missingNumericValuesHolder);
			/**
			 * lockRow is NEVER getting checked, so there's no reason
			 * to go thru the rigomarole below to build it.
			 */
			characterValuesFromLockRecords = characterValuesHolder.value;
			numericValuesFromLockRecords = numericValuesHolder.value;
			/**   
			   int tempIndex = 0;
			   Object[] lockRow = new Object[getColumnCount()];
			   
			   tempIndex = 0;
			   for(int varIndex=0;varIndex<getColumnCount();varIndex++) 
			   {
			   if(getColumnType(varIndex + 1) != Types.VARCHAR) 
			   {
			   lockRow[varIndex] = new Double(numericValuesFromLockRecords[tempIndex++]);
			   }
			   }
			   
			   tempIndex = 0;
			   for(int varIndex=0;varIndex<getColumnCount();varIndex++) 
			   {
			   if(getColumnType(varIndex + 1) == Types.VARCHAR) 
			   {
			   lockRow[varIndex] = (Object)characterValuesFromLockRecords[tempIndex++];
			   }
			   }
			*/
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAResultSet_lockRecordsFailedError,
				null,
				se);
		}
		try
		{
			IntHolder methodStatusHolder = null;
			double[][] numericValuesForUpdate =
				new double[1][numericValuesFromLockRecords.length];
			String[][] characterValuesForUpdate =
				new String[1][Math.max(
					characterValuesFromLockRecords.length,
					1)];
			byte[][] missingNumericValuesForUpdate =
				new byte[1][numericValuesFromLockRecords.length];
			byte[][] changedNumericValuesForUpdate = new byte[1][0];
			byte[][] changedCharacterValuesForUpdate = new byte[1][0];
			j = 0;
			for (int i = 0; i < data.length; i++)
			{
				//if (data[i] instanceof Double)
				/**
				 * S0195329: use the columntypes array to determine whether we're storing
				 * a numeric or character, rather than relying on the object type to indicate
				 * something to us. 
				 */
				if (columnTypes[i] != Types.VARCHAR)					/**
					 * S0195329
					 */
				{
					/**
					 * S0195329: due to the possibility of formatted data coming over that could
					 * be 'missing', we go thru this process of converting the data[] to string (using
					 * toString) and then checking it and doing a parseDouble prior to storage.
					 * I may change this a bit to check if the bindkey passed in is formatted.
					 * I'm trying to keep things simple right now.
					 */
					double tdv;
					if (bindKey == BindKeys._BindKeysFormatted)
					{
						String tv = data[i].toString();
						if (tv.length() == 0 || tv.trim().equals(""))
							tdv = Double.NaN;
						else
							tdv = Double.parseDouble(tv);
					}
					else
					{
						/** 
						 * if the bindkey isn't formatted, then the data in data[] is going to be the
						 * proper instance/type
						 */
						tdv = ((Double) data[i]).doubleValue();
					}
					numericValuesForUpdate[0][j] = tdv;
					/** S0195329 */
					if (Double.isNaN(numericValuesForUpdate[0][j]))
					{
						missingNumericValuesForUpdate[0][j] =
							(byte) NumericMissingDot.value;
					}
					else
					{
						missingNumericValuesForUpdate[0][j] =
							(byte) NotMissing.value;
					}
					j++;
				}
			}
			j = 0;
			for (int i = 0; i < data.length; i++)
			{
				//if (!(data[i] instanceof Double)) // instanceof String? 
				/** 
				 * S0195329: again, check the column type instead of the type in the object array
				 */
				if (columnTypes[i] == Types.VARCHAR)					/** S0195329 */
				{
					if (data[i] == null)
					{
						/**
						 * S0191393: if data is null, then we need to send an empty string
						 * to the server (corba doesn't allow nulls to be passed except under
						 * very specific circumstances
						 */
						characterValuesForUpdate[0][j] = "";
					}
					else
					{
						characterValuesForUpdate[0][j] = data[i].toString();
					}
					j++;
				}
			}
			if (characterValuesFromLockRecords.length == 0)
				characterValuesForUpdate[0][0] = "";
			methodStatusHolder = new IntHolder();
			LongSeqHolder statusHolder = new LongSeqHolder();
			m_dataSet.UpdateRecords(
				bookmarksUpdate,
				innerbindKey,
				changedNumericValuesForUpdate,
				numericValuesForUpdate,
				missingNumericValuesForUpdate,
				changedCharacterValuesForUpdate,
				characterValuesForUpdate,
				methodStatusHolder,
				statusHolder);
			int[] status = statusHolder.value;
			if (methodStatusHolder.value != RIOStatus._RIOStatusRIO_S_OK)
			{
				switch (status[0])
				{
					case RIOStatus._RIOStatusRIO_E_RecordLocked :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_updateFailedRecordLocked);
					case RIOStatus._RIOStatusRIO_E_InvalidBookmark :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_updateFailedBadBookmark);
					case RIOStatus._RIOStatusRIO_E_RecDeleted :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_updateFailedRecordDeleted);
					case RIOStatus._RIOStatusRIO_E_Failure :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_updateFailedServerError);
				}
			}
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAResultSet_updateFailed,
				null,
				se);
		}
		// nuke thne updaterow.
		//	m_updateRow.setBookmark(bookmarksUpdate);
		//	m_rows.set(m_cursor, m_updateRow);
		//	m_updateRow = null;    
		return true;
	}
	public boolean deleteRow(int hashCode, byte[] bookmark)
		throws RemoteException, SQLException
	{
		int flags = 0;
		int bindKey = BindKeys._BindKeysUnformatted;
		byte[] bookmarksDelete = null;
		m_dataSet = (IDataSet) dataMap.get(new Integer(hashCode));
		if (m_dataSet == null)
			return false;
		try
		{
			bookmarksDelete = bookmark;
			StringSeqHolder characterValuesHolder = new StringSeqHolder();
			DoubleSeqHolder numericValuesHolder = new DoubleSeqHolder();
			OctetSeqHolder missingNumericValuesHolder = new OctetSeqHolder();
			m_dataSet.LockRecord(
				flags,
				bindKey,
				bookmarksDelete,
				characterValuesHolder,
				numericValuesHolder,
				missingNumericValuesHolder);
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAResultSet_lockRecordsFailed,
				null,
				se);
		}
		try
		{
			IntHolder methodStatusHolder = new IntHolder();
			LongSeqHolder statusHolder = new LongSeqHolder();
			m_dataSet.DeleteRecords(
				bookmarksDelete,
				methodStatusHolder,
				statusHolder);
			int[] status = statusHolder.value;
			if (methodStatusHolder.value != RIOStatus._RIOStatusRIO_S_OK)
			{
				switch (status[0])
				{
					case RIOStatus._RIOStatusRIO_E_RecordLocked :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_deleteFailedRecordLocked);
					case RIOStatus._RIOStatusRIO_E_InvalidBookmark :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_deleteFailedBadBookmark);
					case RIOStatus._RIOStatusRIO_E_RecDeleted :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_deleteFailedRecordDeleted);
					case RIOStatus._RIOStatusRIO_E_Failure :
						MVALocalizedError.throwLocalizedException(
							VendorCode.MVAResultSet_deleteFailedServerError);
				}
			}
		}
		catch (Exception se)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAResultSet_deleteFailed,
				null,
				se);
		}
		return true;
	}
	public Object[][] getColumnInfo(int hashCode)
		throws RemoteException, SQLException
	{
		m_dataSet = (IDataSet) dataMap.get(new Integer(hashCode));
		if (m_dataSet == null)
			return null;
		boolean[] flags = new boolean[0];
		/** 
		 * using "holders" is how CORBA handles pass-by-ref args
		 */
		String[] columnNames = null;
		int[] columnTypes = null;
		int[] columnLengths = null;
		String[] columnLabels = null;
		String[] formatNames = null;
		short[] formatWidths = null;
		short[] formatDecimals = null;
		short[] formatLengths = null;
		String[] informatNames = null;
		short[] informatWidths = null;
		short[] informatDecimals = null;
		IntHolder numberColumnsHolder = new IntHolder();
		int numberColumns = 0;
		StringSeqHolder columnNamesHolder = new StringSeqHolder();
		//String[] columnNames = null;
		LongSeqHolder columnTypesHolder = new LongSeqHolder();
		//int[] columnTypes = null;
		LongSeqHolder columnLengthsHolder = new LongSeqHolder();
		StringSeqHolder columnLabelsHolder = new StringSeqHolder();
		StringSeqHolder formatNamesHolder = new StringSeqHolder();
		ShortSeqHolder formatWidthsHolder = new ShortSeqHolder();
		ShortSeqHolder formatDecimalsHolder = new ShortSeqHolder();
		ShortSeqHolder formatLengthsHolder = new ShortSeqHolder();
		StringSeqHolder informatNamesHolder = new StringSeqHolder();
		ShortSeqHolder informatWidthsHolder = new ShortSeqHolder();
		ShortSeqHolder informatDecimalsHolder = new ShortSeqHolder();
		OctetSeqHolder sortedByHolder = new OctetSeqHolder();
		/**
		 * ok, let's get the column info for the data set.
		 */
		try
		{
			m_dataSet.GetColumnDefs(
				flags,
				numberColumnsHolder,
				columnNamesHolder,
				columnTypesHolder,
				columnLengthsHolder,
				columnLabelsHolder,
				formatNamesHolder,
				formatWidthsHolder,
				formatDecimalsHolder,
				formatLengthsHolder,
				informatNamesHolder,
				informatWidthsHolder,
				informatDecimalsHolder,
				sortedByHolder);
		}
		catch (Exception se)
		{
			throw new SQLException(se.toString());
		}
		/**
		 * now get the stuff in the holders, and assign it to the
		 * correct variables.
		 */
		numberColumns = numberColumnsHolder.value;
		//		columnCount = numberColumns;
		//		m_columnCount = columnCount;
		columnNames = columnNamesHolder.value;
		columnTypes = columnTypesHolder.value;
		columnLengths = columnLengthsHolder.value;
		columnLabels = columnLabelsHolder.value;
		formatNames = formatNamesHolder.value;
		formatWidths = formatWidthsHolder.value;
		formatDecimals = formatDecimalsHolder.value;
		formatLengths = formatLengthsHolder.value;
		informatNames = informatNamesHolder.value;
		informatWidths = informatWidthsHolder.value;
		informatDecimals = informatDecimalsHolder.value;
		Object[][] rval = new Object[11][numberColumns];
		for (int i = 0; i < numberColumns; i++)
		{
			rval[0][i] = columnNames[i];
			rval[1][i] = new Integer(columnTypes[i]);
			rval[2][i] = new Integer(columnLengths[i]);
			rval[3][i] = columnLabels[i];
			rval[4][i] = formatNames[i];
			rval[5][i] = new Short(formatWidths[i]);
			rval[6][i] = new Short(formatDecimals[i]);
			rval[7][i] = new Short(formatLengths[i]);
			rval[8][i] = informatNames[i];
			rval[9][i] = new Short(informatWidths[i]);
			rval[10][i] = new Short(informatDecimals[i]);
		}
		return rval;
	}
	public int openDataSet(
		String libref,
		String tableName,
		String[] password,
		String options,
		int concurFlag,
		int typeFlag)
		throws RemoteException, SQLException
	{
		m_dataSet = null;
		int flags = 0;
		if (concurFlag == ResultSet.CONCUR_READ_ONLY)
		{
			flags = ILibref.ReadOnlyAccess;
		}
		if (typeFlag == ResultSet.TYPE_FORWARD_ONLY)
		{
			flags = flags | ILibref.SequentialAccess;
		}
		String[] openPass = new String[3];
		if ((password != null) && (password.length != 0))
		{
			for (int i = 0; i < password.length; i++)
			{
				openPass[i] = password[i];
			}
		}
		for (int xxx = 0; xxx < openPass.length; xxx++)
		{
			if (openPass[xxx] == null)
			{
				openPass[xxx] = "";
			}
		}
		if (options == null)
		{
			options = "";
		}
		//        int bookmarkLength = 0;
		StringHolder dataSetLabelHolder = new StringHolder();
		StringHolder dataSetTypeHolder = new StringHolder();
		DateTimeHolder dateCreatedHolder = new DateTimeHolder();
		DateTimeHolder dateModifiedHolder = new DateTimeHolder();
		IntHolder recordLengthHolder = new IntHolder();
		StringHolder compressionRoutineHolder = new StringHolder();
		IntHolder bookmarkLengthHolder = new IntHolder();
		IntHolder logRecordCountHolder = new IntHolder();
		IntHolder phyRecordCountHolder = new IntHolder();
		IntHolder attributesHolder = new IntHolder();
		try
		{
			releaseLibref();
			m_libref = useLibref(libref);
			m_dataSet =
				m_libref.OpenDataSet(
					flags,
					tableName,
					options,
					openPass,
					dataSetLabelHolder,
					dataSetTypeHolder,
					dateCreatedHolder,
					dateModifiedHolder,
					recordLengthHolder,
					compressionRoutineHolder,
					bookmarkLengthHolder,
					logRecordCountHolder,
					phyRecordCountHolder,
					attributesHolder);
			/**
				 * dataSetLabel = dataSetLabelHolder.value;
				 *	dataSetType = dataSetTypeHolder.value;
				 *	dateCreated = dateCreatedHolder.value;
				 *	dateModified = dateModifiedHolder.value;
				 *	recordLength = recordLengthHolder.value;
				 *	compressionRoutine = compressionRoutineHolder.value;
				 *	//TODO: use DataSetAttrIndexed from this attributes.
				 *	attributes = attributesHolder.value;
			 */
			//bookmarkLength = bookmarkLengthHolder.value;
			/**
				 * logRecordCount = logRecordCountHolder.value;
				 * phyRecordCount = phyRecordCountHolder.value;
				 */
		}
		catch (Exception e)
		{
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAResultSet_openDataSetError,
				null,
				e);
		}
		if (m_dataSet == null)
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAResultSet_openDataSetError);
		Integer k = new Integer(m_dataSet.hashCode());
		dataMap.put(k, m_dataSet);
		return k.intValue();
	}
	private ILibref useLibref(String libref)
		throws RIOException // I'll probably get rid of the
	// 'throws RIOException' part.
	{
		try
		{
			return m_dataService.UseLibref(libref);
		}
		catch (Exception e)
		{
			Object[] args = { e.getMessage()};
			MVALocalizedError.throwLocalizedRIOException(
				VendorCode.MVAStatement_useLibrefError,
				args);
		}
		return null;
	}
	/**
	 * Gets IWorkspace object.
	 */
	private IWorkspace getWorkspace(java.util.Properties info)
		throws SQLException
	{
		IWorkspace sasWorkspace = null;
		try
		{
			Properties op = new Properties();
			op.put(
				WorkspaceConstants.ORBCLASSPROPERTY,
				WorkspaceConstants.DEFAULTORBCLASS);
			op.put(
				WorkspaceConstants.ORBSINGLETONCLASSPROPERTY,
				WorkspaceConstants.DEFAULTORBSINGLETONCLASS);
			org.omg.CORBA.ORB orb1 =
				org.omg.CORBA.ORB.init((String[]) null, op);
			op = new Properties();
			op.put(
				WorkspaceConstants.HOST,
				info.getProperty(WorkspaceConstants.HOST));
			op.put(
				WorkspaceConstants.PORT,
				info.getProperty(WorkspaceConstants.PORT));
			op.put(
				WorkspaceConstants.USERNAME,
				info.getProperty(WorkspaceConstants.USERNAME));
			op.put(
				WorkspaceConstants.PASSWORD,
				info.getProperty(WorkspaceConstants.PASSWORD));
			String debugLevel = info.getProperty(WorkspaceConstants.DEBUGLEVEL);
			if (debugLevel != null)
			{
				op.put(WorkspaceConstants.DEBUGLEVEL, debugLevel);
			}
			String sasRequiredEncryptionLevel =
				info.getProperty(LDAPConstants.SASREQUIREDENCRYPTIONLEVEL);
			/*
			 * begin S0205046 fix 
			 */
			String encryptionPolicy =
				info.getProperty(
					WorkspaceConstants.ENCRYPTIONPOLICY.toLowerCase());
			String encryptionContent =
				info.getProperty(
					WorkspaceConstants.ENCRYPTIONCONTENT.toLowerCase());
			String encryptionAlgorithms =
				info.getProperty(
					WorkspaceConstants.ENCRYPTIONALGORITHMS.toLowerCase());
			/*
			 * end S0205046 fix
			 */
			if ((encryptionAlgorithms != null)
				&& (encryptionAlgorithms.length() != 0))
			{
				op.put(
					WorkspaceConstants.ENCRYPTIONALGORITHMS,
					encryptionAlgorithms);
			}
			if ((encryptionPolicy != null) && (encryptionPolicy.length() != 0))
			{
				op.put(WorkspaceConstants.ENCRYPTIONPOLICY, encryptionPolicy);
				if ((encryptionContent != null)
					&& (encryptionContent.length() != 0))
				{
					op.put(
						WorkspaceConstants.ENCRYPTIONCONTENT,
						encryptionContent);
				}
			}
			else
			{
				if ((sasRequiredEncryptionLevel != null)
					&& (sasRequiredEncryptionLevel.length() != 0))
				{
					if (sasRequiredEncryptionLevel
						.equalsIgnoreCase("authentication"))
					{
						op.put(WorkspaceConstants.ENCRYPTIONPOLICY, "required");
						if ((encryptionContent != null)
							&& (encryptionContent.length() != 0))
						{
							op.put(
								WorkspaceConstants.ENCRYPTIONCONTENT,
								encryptionContent);
						}
						else
						{
							op.put(
								WorkspaceConstants.ENCRYPTIONCONTENT,
								"authentication");
						}
					}
					else if (
						sasRequiredEncryptionLevel.equalsIgnoreCase("all"))
					{
						op.put(WorkspaceConstants.ENCRYPTIONPOLICY, "required");
						if ((encryptionContent != null)
							&& (encryptionContent.length() != 0))
						{
							op.put(
								WorkspaceConstants.ENCRYPTIONCONTENT,
								encryptionContent);
						}
						else
						{
							op.put(WorkspaceConstants.ENCRYPTIONCONTENT, "all");
						}
					}
				}
			}
			org.omg.CORBA.Object object =
				orb1.string_to_object(propertiesToUrl(op));
			try
			{
				m_automation = ISASAutomationHelper.narrow(object);
			}
			catch (org.omg.CORBA.BAD_PARAM x)
			{
			}
			if (m_automation != null)
			{
				sasWorkspace = m_automation.GetWorkspaceObject();
			}
			else
			{
				sasWorkspace = IWorkspaceHelper.narrow(object);
			}
		}
		catch (Exception e)
		{
			if (e instanceof org.omg.CORBA.COMM_FAILURE
				&& ((org.omg.CORBA.COMM_FAILURE) e).minor == 1)
			{
				// sub in s273, passing in server, port, etc.
				Object[] args =
					{
						info.getProperty(WorkspaceConstants.HOST),
						info.getProperty(WorkspaceConstants.PORT),
						e.getMessage()};
				MVALocalizedError.throwLocalizedException(
					VendorCode.MVAConnection_workspaceConnectionError,
					args);
			}
			else
				MVALocalizedError.throwLocalizedException(
					VendorCode.MVAConnection_workspaceError,
					null,
					e);
		}
		return sasWorkspace;
	}
	/**
	* Gets IDBMSConnection object from IDataService object.
	*/
	private IDBMSConnection getDBMSConnection(IDataService sasDataService)
		throws SQLException
	{
		IDBMSConnection dbmsConnection = null;
		try
		{
			IntHolder statusHolder = new IntHolder();
			StringHolder warningHolder = new StringHolder();
			//GET DBMS PROPERTIES
			String database =
				m_info.getProperty(JDBCConstants.DBMS.toLowerCase());
			//HACK TO FIX THE FACT THAT OLDER USERS CALL CONNECTION
			//DIRECTLY AND NOT THROUGH THE DRIVER
			if (database == null)
			{
				database = new String("SQLVIEW");
			}
			String databaseOptions =
				m_info.getProperty(JDBCConstants.DBMS_OPTIONS.toLowerCase());
			if (databaseOptions == null)
			{
				databaseOptions = new String("");
			}
			dbmsConnection =
				sasDataService.ConnectToDBMS(
					database,
					databaseOptions,
					statusHolder,
					warningHolder);
			if (dbmsConnection == null)
			{
				// must have missed this one!
				MVALocalizedError.throwLocalizedException(
					VendorCode.MVAConnection_nullDbmsConnection);
			}
			if ((warningHolder.value != null)
				&& (warningHolder.value.length() != 0))
			{
				SQLWarning warning =
					new SQLWarning(warningHolder.value, null, 58);
				if (warnings == null)
					warnings = warning;
				else
					warnings.setNextWarning(warning);
			}
		}
		catch (Exception e)
		{
			// TEST OF THE NEW EXCEPTION HANDLING
			//      ThrowGeneralSQLException(VendorCode.getDBMSConnection",58,e);
			MVALocalizedError.throwLocalizedException(
				VendorCode.MVAConnection_error,
				null,
				e);
			/* SQLException se = new SQLException(error, null, VendorCode.VC_UNKNOWNERROR);
			 if (e instanceof SQLException)
			   se.setNextException((SQLException)e);
			 throw se;
			*/
		}
		return dbmsConnection;
	}
	/**
	* Assigns librefs.
	*/
	private Vector assignLibrefs(String librefs) throws SQLException
	{
		String warningMsg = null;
		SQLWarning warning = null;
		m_librefList = new Vector();
		if (librefs == null)
		{
			return m_librefList;
		}
		String[] paths = new String[10];
		String[] libnames = new String[10];
		String delim = ";";
		int i = 0;
		StringTokenizer stColon = new StringTokenizer(librefs, delim);
		while (stColon.hasMoreTokens())
		{
			try
			{
				String libSegment = stColon.nextToken();
				if (libSegment.indexOf('(') < 0 && libSegment.indexOf(')') < 0)
				
				{
					delim = "'\"";
					//the libpath is a String like this --> "lib1 'xxxx'; lib2 'yyyy'"
					StringTokenizer stSpaces =
						new StringTokenizer(/*stColon.nextToken() */ libSegment, delim);
					if (stSpaces.hasMoreTokens())
					{
						String tlib = stSpaces.nextToken().trim();
						String tpath = stSpaces.nextToken().trim();
						ILibref sasLibref =
							m_dataService.AssignLibref(
								tlib, 
								"",
								tpath,
								"");
						m_librefList.addElement(sasLibref);
						libnames[i] = tlib;
						paths[i] = tpath;
					}
				}
				else // special processing required.
					{
					StringTokenizer stSpaces = 
						new StringTokenizer(libSegment, " )", true);
					if (stSpaces.hasMoreTokens())
					{
						String tlib = stSpaces.nextToken().trim();
						String tpath = stSpaces.nextToken(")").trim(); 
						tpath = tpath + stSpaces.nextToken();

						ILibref sasLibref =
							m_dataService.AssignLibref(
								tlib, 
								"",
								tpath,
								"");
						m_librefList.addElement(sasLibref);
						libnames[i] = tlib;
						paths[i] = tpath;
					}
				}
				i++;
			}
			catch (com.sas.iom.SAS.InvalidPName e1)
			{
				warningMsg =
					MVALocalizedError.getString(
						VendorCode.MVAConnection_invalidPnameWarning);
				
				
					SQLWarning sqw = new SQLWarning(
						warningMsg,
						null,
						VendorCode.MVAConnection_invalidPnameWarning);
				if (warning == null ) warning = sqw;
				else warning.setNextWarning(sqw);
			}
			catch (com.sas.iom.SAS.NoAccessMethod e2)
			{
				warningMsg =
					MVALocalizedError.getString(
						VendorCode.MVAConnection_noAccessMethodWarning);
				SQLWarning sqw =
					new SQLWarning(
						warningMsg,
						null,
						VendorCode.MVAConnection_noAccessMethodWarning);
				if (warning == null ) warning = sqw;
								else warning.setNextWarning(sqw);
			}
			catch (com.sas.iom.SAS.InvalidLName e3)
			{
				warningMsg =
					MVALocalizedError.getString(
						VendorCode.MVAConnection_invalidLNameWarning);
				SQLWarning sqw =
					new SQLWarning(
						warningMsg,
						null,
						VendorCode.MVAConnection_invalidLNameWarning);
				if (warning == null ) warning = sqw;
								else warning.setNextWarning(sqw);
			}
			catch (com.sas.iom.SAS.IDataServicePackage.NoLibrary e4)
			{
				warningMsg =
					MVALocalizedError.getString(
						VendorCode.MVAConnection_noLibraryWarning);
				SQLWarning sqw =
					new SQLWarning(
						warningMsg,
						null,
						VendorCode.MVAConnection_noLibraryWarning);
				if (warning == null ) warning = sqw;
								else warning.setNextWarning(sqw);
			}
			catch (com.sas.iom.SAS.LNameNoAssign e5)
			{
				warningMsg =
					MVALocalizedError.getString(
						VendorCode.MVAConnection_lnameNotAssignedWarning);
				SQLWarning sqw =
					new SQLWarning(
						warningMsg,
						null,
						VendorCode.MVAConnection_lnameNotAssignedWarning);
				if (warning == null ) warning = sqw;
								else warning.setNextWarning(sqw);
			}
			catch (com.sas.iom.SAS.IDataServicePackage.InvalidEngine e6)
			{
				warningMsg =
					MVALocalizedError.getString(
						VendorCode.MVAConnection_invalidEngineWarning);
				SQLWarning sqw =
					new SQLWarning(
						warningMsg,
						null,
						VendorCode.MVAConnection_invalidEngineWarning);
				if (warning == null ) warning = sqw;
								else warning.setNextWarning(sqw);
			}
			catch (Exception e)
			{
				MVALocalizedError.throwLocalizedException(
					VendorCode.MVAConnection_assignLibrefError,
					null,
					e);
			}
		}
		this.libnames = libnames;
	
	// TODO: vendor codes!
	if (warning != null)
	{
		if (warnings == null)
		{
			warnings = warning;
		}
		else
		{
			warnings.setNextWarning(warning);
		}
	}
	return m_librefList;
}
/**
* Converts Properties object to url to get stringifiedObjectReference.
*/
private static String propertiesToUrl(Properties properties)
{
	StringBuffer buffer = new StringBuffer("bridge://");
	buffer.append(properties.getProperty(WorkspaceConstants.HOST));
	buffer.append(":");
	buffer.append(properties.getProperty(WorkspaceConstants.PORT));
	buffer.append("/");
	buffer.append(WorkspaceConstants.SASAUTOMATIONID);
	StringBuffer query = null;
	Enumeration propertyNames = properties.propertyNames();
	while (propertyNames.hasMoreElements())
	{
		String propertyName = (String) propertyNames.nextElement();
		if (!propertyName.equals(WorkspaceConstants.HOST)
			&& !propertyName.equals(WorkspaceConstants.PORT))
		{
			if (query == null)
			{
				query = new StringBuffer();
			}
			else
			{
				query.append("&");
			}
			query.append(propertyName);
			query.append("=");
			query.append(properties.getProperty(propertyName));
		}
	}
	if (query != null)
	{
		buffer.append("?");
		buffer.append(query.toString());
	}
	return buffer.toString();
}
private Object[][] addRows(
	String[][] characterValues,
	double[][] numericValues,
	byte[][] missingNumericValues,
	byte[] bookmarks,
	boolean direction,
	int bindKey,
	int[] columnTypes)
{
	// direction true = forward, false=back
	int cindex = 0;
	int nindex = 0;
	int cc = 0;
	cc = columnTypes.length;
	Object[][] data = new Object[characterValues.length][];
	/**	
			int blen = 0;
			if (m_bookmarkLength == 0)
			    blen = bookmarks.length / characterValues.length;
			else 
				blen = m_bookmarkLength;
	*/
	if (direction)
	{
		for (int i = 0; i < characterValues.length; i++)
		{
			data[i] = new Object[cc];
			/**			
							int s = i * blen;
							int e = s + blen;
							byte[] b = new byte[blen];
							int k = 0;
							int j = s;
							for (j = s, k = 0; j < e; j++, k++)
							{
			
							    b[k] = bookmarks[j];
			//				    System.out.print("b["+k+"]:" + b[k] + "|");
							}
			//				System.out.println("");
							row.setBookmark(b);
			*/
			cindex = nindex = 0;
			int k = 0;
			if (bindKey == BindKeys._BindKeysUnformatted)
			{
				for (k = 0; k < cc; k++)
				{
					switch (columnTypes[k])
					{
						case Types.VARCHAR :
							data[i][k] = characterValues[i][cindex++];
							//row.setMissing(k, new Boolean(false));
							break;
							/** this better never happen. You need to vette the 
							 * types prior to the read records call to sub in the correct
							 * type
							 *
							case Types.NULL:
								//int at = getActualType(k + 1);
								
								switch(at)
								{
									case Types.VARCHAR:
										row.add(k, characterValues[i][cindex++]);
										row.setMissing(k, new Boolean(false));
										break;
										default:
										row.add(k, new Double(numericValues[i][nindex]));
										if (missingNumericValues[i][nindex] == NotMissing.value)
											row.setMissing(k, new Boolean(false));
										else
											row.setMissing(k, new Boolean(true));
							
										nindex++;
										break;
								}
								break;
								*/
						default :
							if (missingNumericValues[i][nindex]
								== NotMissing.value)
								data[i][k] =
									new Double(numericValues[i][nindex]);
							else
								data[i][k] = new Double(Double.NaN);
							nindex++;
							break;
					}
				}
			}
			else // bindkeys._bindkeysformatted
				{
				for (k = 0; k < cc; k++)
				{
					switch (columnTypes[k])
					{
						case Types.VARCHAR :
							data[i][k] = characterValues[i][cindex++];
							break;
						default :
							if (missingNumericValues[i][nindex]
								== NotMissing.value)
								data[i][k] = characterValues[i][cindex];
							else
								data[i][k] = new Double(Double.NaN);
							/** S0142734: data integrity fix */
							cindex++;
							nindex++;
					}
					//row.setMissing(k, new Boolean(false));	
				}
			}
			//m_rows.add(row);			
		}
	}
	else
	{
		// counting down.
		int z = 0;
		int i = 0;
		for (i = characterValues.length - 1, z = 0; i >= 0; i--, z++)
		{
			//				MVARow row = new MVARow(m_columnCount);
			/**			
							int s = i * blen;//z
							int e = s + blen;
							byte[] b = new byte[blen];
							int k = 0;
							int j = s;
							for (j = s, k = 0; j < e; j++, k++)
							{
							    b[k] = bookmarks[j];
								//System.out.print("b["+k+"]:" + b[k] + "|");
							}
							//System.out.println("");
							row.setBookmark(b);
			*/
			cindex = nindex = 0;
			data[i] = new Object[cc];
			int k = 0;
			if (bindKey == BindKeys._BindKeysUnformatted)
			{
				for (k = 0; k < cc; k++)
				{
					switch (columnTypes[k])
					{
						case Types.VARCHAR :
							data[i][k] = characterValues[i][cindex++];
							//row.setMissing(k, new Boolean(false));
							break;
							/** see above, about what to do with NULLs
							case Types.NULL:
								int at = getActualType(k + 1);
								switch(at)
								{
									case Types.VARCHAR:
										row.add(k, characterValues[i][cindex++]);
										row.setMissing(k, new Boolean(false));
										break;
										default:
										row.add(k, new Double(numericValues[i][nindex]));
										if (missingNumericValues[i][nindex] == NotMissing.value)
											row.setMissing(k, new Boolean(false));
										else
											row.setMissing(k, new Boolean(true));
							
										nindex++;
										break;
								}
								break;
								*/
						default :
							if (missingNumericValues[i][nindex]
								== NotMissing.value)
								data[i][k] =
									new Double(numericValues[i][nindex]);
							else
								data[i][k] = new Double(Double.NaN);
							nindex++;
							break;
					}
				}
			}
			else // bindkeys._bindkeysformatted
				{
				for (k = 0; k < cc; k++)
				{
					switch (columnTypes[k])
					{
						case Types.VARCHAR :
							data[i][k] = characterValues[i][cindex++];
							break;
						default :
							if (missingNumericValues[i][nindex]
								== NotMissing.value)
								data[i][k] = characterValues[i][cindex];
							else
								data[i][k] = new Double(Double.NaN);
							/** S0142734: data integrity fix */
							cindex++;
							nindex++;
					}
				}
			}
		}
	}
	return data;
}
public int getStatus() throws RemoteException
{
	return m_status;
}
public void close(int hashCode) throws RemoteException
{
	IDataSet dataSet = (IDataSet) dataMap.remove(new Integer(hashCode));
	if (dataSet == null)
		return;
	try
	{
		if (dataSet != null)
		{
			dataSet.Close();
			dataSet._release();
			dataSet = null;
		}
	}
	catch (Exception e)
	{
		dataSet = null;
	}
}
public void disconnect() throws RemoteException, SQLException
{
	try
	{
		if (m_connection != null)
		{
			m_connection.Disconnect();
			m_connection._release();
			m_connection = null;
		}
		if (m_dataService != null)
		{
			/*
			 * note, there is NO reason why this is here.  Never, ever turn it on.
			 * The only reason I'm including it is as legacy stuff to look at for 9.2        
			for (int i = 0; i < m_librefList.size(); i++)
			    {
			        m_dataService.DeassignLibref(libnames[i]);
			        
			    }
			*/
			if (m_fromDataService == false)
			{
				m_dataService._release();
				m_dataService = null;
			}
		}
		ILibref sasLibref;
		for (int i = 0; i < m_librefList.size(); i++)
		{
			sasLibref = (ILibref) m_librefList.elementAt(0);
			if (sasLibref != null)
			{
				sasLibref._release();
				sasLibref = null;
			}
		}
		if (m_fromSSI == false)
		{
			if (m_workspace != null)
			{
				/* S0230433: changed to code to get orb and shutdown as below */
				org.omg.CORBA.ORB orb =
					((org.omg.CORBA.portable.ObjectImpl) m_workspace)._orb();
				orb.shutdown(true);
				m_workspace = null;
			}
			if (m_automation != null)
			{
				m_automation._release();
				m_automation = null;
			}
		}
	}
	catch (Exception e)
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVAConnection_closeError,
			null,
			e);
	}
}
public int countRecords(int hashCode) throws RemoteException, SQLException
{
	m_dataSet = (IDataSet) dataMap.get(new Integer(hashCode));
	if (m_dataSet == null)
		return 0;
	IntHolder logicalCount = new IntHolder();
	IntHolder physCount = new IntHolder();
	try
	{
		m_dataSet.GetRecordCounts(logicalCount, physCount);
	}
	catch (Exception e)
	{
		throw new SQLException(e.toString());
	}
	return logicalCount.value;
}
public void releaseLibref() throws RemoteException
{
	try
	{
		if (m_libref != null)
		{
			m_libref._release();
			m_libref = null;
		}
	}
	catch (Exception e)
	{
		throw new RemoteException(e.toString());
	}
}
/** S0181889  method to get bookmarks */
public byte[] getBookmarkArray() throws RemoteException
{
	return m_bookmarkStorage;
}
public DataHolder readRecords(
	int hashCode,
	int flags,
	int bindKey,
	int rows,
	int rowOffset,
	byte[] pos,
	int[] columnTypes)
	throws RemoteException, SQLException
{
	DataHolder data = null;
	boolean forward = true;
	int o_status = 0;
	if (rows < 0)
		forward = false;
	m_dataSet = (IDataSet) dataMap.get(new Integer(hashCode));
	if (m_dataSet == null)
		return null;
	byte[] positionBookmark = null;
	if (pos == null)
		positionBookmark = new byte[0];
	else
		positionBookmark = pos;
	//CREATE HOLDERS TO ACCESS PASS BY REFERENCE PARAMETERS
	VariableArray2dOfStringHolder characterValuesHolder =
		new VariableArray2dOfStringHolder();
	String[][] characterValues = null;
	VariableArray2dOfDoubleHolder numericValuesHolder =
		new VariableArray2dOfDoubleHolder();
	double[][] numericValues = null;
	VariableArray2dOfOctetHolder missingNumericValuesHolder =
		new VariableArray2dOfOctetHolder();
	byte[][] missingNumericValues = null;
	OctetSeqHolder bookmarksHolder = new OctetSeqHolder();
	IntHolder statusHolder = new IntHolder();
	//int status = 0;
	byte[] bm = null;
	boolean aok = false;
	try
	{
		//CALL RIO READ RECORDS METHOD
		m_dataSet.ReadRecords(
			flags,
			bindKey,
			positionBookmark,
			rows,
			rowOffset,
			characterValuesHolder,
			numericValuesHolder,
			missingNumericValuesHolder,
			bookmarksHolder,
			statusHolder);
		//STORE DATA IN HOMOGENEOUS ARRAYS
		//might be able to do a conversion without loss of efficiency
		//since this is probably a copy anyway
		characterValues = characterValuesHolder.value;
		numericValues = numericValuesHolder.value;
		missingNumericValues = missingNumericValuesHolder.value;
		bm = bookmarksHolder.value;
		o_status = statusHolder.value;
		aok = true;
	}
	catch (Exception e)
	{
		MVALocalizedError.throwLocalizedException(
			VendorCode.MVAResultSet_readRecordsFailed,
			null,
			e);
		return null; // well something bad obviously happened.
	}
	if (aok)
	{
		if (o_status == RIOStatus._RIOStatusRIO_W_BegOfFile)
			m_status = BOF;
		if (o_status == RIOStatus._RIOStatusRIO_W_EndOfFile)
			m_status = EOF;
		else
			m_status = AOK;
		if (characterValues.length == 0 & numericValues.length == 0)
		{
			return null; // [0][0]?
		}
		else			/**
			     * need to take addRows, and munge it to give back an Object[][] array, which 
			     * we'll then return to the caller.
			     * (in the normal front-to-back order, regardless of direction)
			     */
			{
			data = new DataHolder();
			data.characterValues = characterValues;
			data.numericValues = numericValues;
			data.missingNumericValues = missingNumericValues;
			data.bookmarkArray = bookmarksHolder.value;
			data.status = m_status;
		}
	}
	return data; //??		
}
}
