/* Copyright (c)1998 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */

package com.sas.rio;

import java.io.DataInputStream;
import java.io.PrintStream;
import java.io.IOException;
import java.util.Hashtable;
import java.sql.Types;

/**
 * This class is used to contain information
 * about the data format.
 */
public class Sqpformat
{
    static Hashtable formats;
    static String[]  prefixes;
    static Hashtable suffixes;

    static {
      formats = new Hashtable();
      formats.put("DATE", new Integer(MVAResultSet.DATE));
      formats.put("DATETIME", new Integer(MVAResultSet.DATETIME));
      formats.put("DAY", new Integer(MVAResultSet.DATE));
      formats.put("DDMMYY", new Integer(MVAResultSet.DATE));
      formats.put("DOWNAME", new Integer(MVAResultSet.DATE));
      formats.put("HHMM", new Integer(MVAResultSet.TIME));
      formats.put("HOUR", new Integer(MVAResultSet.TIME));
      formats.put("JULDAY", new Integer( MVAResultSet.DATE));
      formats.put("JULIAN", new Integer(MVAResultSet.DATE));
      formats.put("MINGUO", new Integer(MVAResultSet.DATE));
      formats.put("MMDDYY", new Integer(MVAResultSet.DATE));
      formats.put("MMSS", new Integer(MVAResultSet.TIME));
      formats.put("MMYY", new Integer(MVAResultSet.DATE));
      formats.put("MONNAME", new Integer(MVAResultSet.DATE));
      formats.put("MONTH", new Integer(MVAResultSet.DATE));
      formats.put("MONYY", new Integer(MVAResultSet.DATE));
      formats.put("NENGO", new Integer(MVAResultSet.DATE));
      formats.put("QTR", new Integer(MVAResultSet.DATE));
      formats.put("QTRR", new Integer(MVAResultSet.DATE));
      formats.put("TIME", new Integer(MVAResultSet.TIME));
      formats.put("TOD", new Integer(MVAResultSet.DATETIME));
      formats.put("WEEKDATE", new Integer(MVAResultSet.DATE));
      formats.put("WEEKDATX", new Integer(MVAResultSet.DATE));
      formats.put("WEEKDAY", new Integer(MVAResultSet.DATE));
      formats.put("WORDDATE", new Integer(MVAResultSet.DATE));
      formats.put("WORDDATX", new Integer(MVAResultSet.DATE));
      formats.put("YEAR", new Integer(MVAResultSet.DATE));
      formats.put("YYMM", new Integer(MVAResultSet.DATE));
      formats.put("YYMMDD", new Integer(MVAResultSet.DATE));
      formats.put("YYMON", new Integer(MVAResultSet.DATE));
      formats.put("YYQ", new Integer(MVAResultSet.DATE));
      formats.put("YYQR", new Integer(MVAResultSet.DATE));

      formats.put("BINARY", new Integer(MVAResultSet.INTEGER));
      formats.put("COMMA", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("COMMAX", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("DOLLAR", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("DOLLARX", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("HEX", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("IB", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("OCTAL", new Integer(MVAResultSet.INTEGER));
      formats.put("PIB", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("ROMAN", new Integer(MVAResultSet.INTEGER));
      formats.put("SSN", new Integer(MVAResultSet.INTEGER));
      formats.put("Z", new Integer(MVAResultSet.MAYBE_INTEGER));
      formats.put("F", new Integer(MVAResultSet.MAYBE_INTEGER));

      /**
       * S0217637 - additional format/type support 
       */
      
      formats.put("DTYEAR", new Integer(MVAResultSet.DATETIME));
      formats.put("DTWKDATX", new Integer(MVAResultSet.DATETIME));
      formats.put("DTYYQC", new Integer(MVAResultSet.DATETIME));
      formats.put("DATEAMPM", new Integer(MVAResultSet.DATETIME));
      formats.put("TIMEAMPM", new Integer(MVAResultSet.TIME));
      
      /* end S0217637 */
      prefixes = new String[13];
      prefixes[0] = new String("DAN");
      prefixes[1] = new String("DES");
      prefixes[2] = new String("DEU");
      prefixes[3] = new String("ESP");
      prefixes[4] = new String("EUR");
      prefixes[5] = new String("FIN");
      prefixes[6] = new String("FRA");
      prefixes[7] = new String("FRS");
      prefixes[8] = new String("ITA");
      prefixes[9] = new String("NLD");
      prefixes[10] = new String("NOR");
      prefixes[11] = new String("PTG");
      prefixes[12] = new String("SVE");

      suffixes = new Hashtable();
      suffixes.put("DFDD", new Integer(MVAResultSet.DATE));
      suffixes.put("DFDE", new Integer(MVAResultSet.DATE));
      suffixes.put("DFDN", new Integer(MVAResultSet.DATE));
      suffixes.put("DFDT", new Integer(MVAResultSet.DATETIME));
      suffixes.put("DFDWN", new Integer(MVAResultSet.DATE));
      suffixes.put("DFMN", new Integer(MVAResultSet.DATE));
      suffixes.put("DFMY", new Integer(MVAResultSet.DATE));
      suffixes.put("DFWDX", new Integer(MVAResultSet.DATE));
      suffixes.put("DFWKX", new Integer(MVAResultSet.DATE));
   }

   int name_len;
   String name;
   int  wid, ndec;
   boolean v6 = true;
   private String messageIndex;

   //TODO: getColumnInformation returns len,wid,dec in int. But, we need only shorts. are we overkilling ????
   public Sqpformat(String formatName,int formatLength,int formatWidth, int formatDecimal) {
    this.name = formatName;
    this.name_len = formatLength;
    this.wid = formatWidth;
    this.ndec = formatDecimal;
   }

   public Sqpformat(boolean v6) {
     this.v6 = v6;
   }

   public Sqpformat() {
   }

    /**
     * Get the type of the data.
     * @param inname format name
     * @return int data type, based on format
     */
    public int findFormat(String inname)
    {
      int subtype;
      String tstring = inname.trim();
      if (tstring.length() == 0) {
        if (ndec == 0 && wid !=0) {
           return MVAResultSet.INTEGER;
        } else {
          return MVAResultSet.NOSUBTYPE;
        }
      }
      Integer temp;
      temp = (Integer) formats.get(tstring);
      if (temp != null)
        subtype = temp.intValue();
      else
        subtype = 0;
      if (subtype == MVAResultSet.MAYBE_INTEGER) {
        if (inname.indexOf("HEX") != -1) {
          if (wid != 16)
            return MVAResultSet.INTEGER;
          else
            return MVAResultSet.NOSUBTYPE;
        }
        if (ndec == 0)
          return MVAResultSet.INTEGER;
      }

      if (subtype != 0)
        return subtype;

      // Look for more formats
      subtype = getIntl(inname);
      /* S0217637 */

      if (subtype == 0)
        subtype = checkForSep(inname);

      return subtype;
    }

    /**
     * Interpret the format returned by dictionary.columns.
     * @param inname format name
     * @return int data type, based on format
     */
    static public int intepretFormat(String inname)
    {
      inname = inname.trim();
      int length = inname.length();
      if (length == 0)
        return 0;
      char tchar[] = inname.toCharArray();
      if (Character.isDigit(tchar[0])) {
        if (tchar[length-1] == '.') {
           return MVAResultSet.INTEGER;
        } else {
           return 0;
        }
      }
      if (inname.indexOf("HEX") != -1) {
        if (inname.indexOf("16") == -1)
            return MVAResultSet.INTEGER;
          else
            return 0;
      }

      int i = length - 1;
      while ( i >= 0) {
        if ((tchar[i] == '.') || Character.isDigit(tchar[i])) {
          tchar[i] = ' ';
          i--;
          continue;
        } else {
          break;
        }
      }
      String tstring = new String(tchar);
      tstring = tstring.trim();
      Integer temp;
      temp = (Integer) formats.get(tstring);
      if (temp != null) {
        int subtype = temp.intValue();
        if ((subtype == MVAResultSet.MAYBE_INTEGER) && (inname.charAt(length-1) == '.'))
          subtype = MVAResultSet.INTEGER;
        return subtype;
      }
      // Look for more formats
      int st;
      st = getIntl(tstring);
      /* S0217637 */

      if (st == 0)
        st = checkForSep(tstring);

      return st;
    }
    /**
     * Determine if there is a prefix or suffix on the
     * format.
     * @param format format of data
     * @return type of data
     */
    static public int getIntl(String format) {
        /**
         * S0217636 - remove this length check
         * pass 2 - put the length check back in, but changed it to 
         * check for lengths < 7, since none of the i18n formats in the
         * hashtable are less than 7
         */
      if (format.length() < 7)
        return 0;
      /*
       * get the tentative prefix value
       */
      String temp = format.substring(0,3);
      int i;
      Integer subtype;
      for (i = 0; i < prefixes.length; i++) {
        /*
         * look in the prefix array for a match
         */
         
        if (prefixes[i].equals(temp)) {
          /**
           * S0217636 - and change the substring call below from 3,5 to 3, length of string
           *            to get all of the remaining format info.
           */
          String suffix = format.substring(3,format.length());
          subtype = (Integer) suffixes.get(suffix);
          if (subtype != null)
            return(subtype.intValue());
          else
            return 0;
        }
      }
      return 0;
    }
    /*
     * S0217637 fix
     */
    /*
     * This method checks for "separator formats".
     * These are the seven formats that have either
     * B, C, S, N, or P at the end, indicating the
     * appropriate separator character to use when
     * formatting.
     */
    static private int checkForSep(String inname)
    {
        Integer st;
        String temp = inname.substring(0, inname.length() - 1);
        char c = inname.charAt(inname.length() - 1);
        st = (Integer)formats.get(temp);
        int likely = 0;
        if (st != null)
            likely = st.intValue();
        if (likely != 0)
        {
            switch(c)
            {
                case 'B':
                    if (temp.equals("MMDDYY") || temp.equals("DDMMYY") ||
                        temp.equals("YYMMDD"))
                            return likely;
                    else
                        return 0; // invalid format, return default.
                case 'C':
                case 'D':
                case 'N':
                case 'P':
                case 'S':
                    if (temp.equals("MMDDYY") || temp.equals("DDMMYY") ||
                        temp.equals("YYMMDD") || temp.equals("MMYY") ||
                        temp.equals("YYQ") || temp.equals("YYMM") || temp.equals("YYQR"))
                        return likely;
                    else
                        return 0;
                default:
                    return 0;   
            }   
        }
        return likely;           
    }

}

