/* $Id$ */
package com.sas.rio;

/*
 * NAME: VendorCode.java
 * PURPOSE: Repository for "vendor codes" that are passed to SQLException, and also
 *          map to the errorStrings array, which stores the message keys.
 * AUTHOR: Dave Mackie, SAS Institute
 * HISTORY:
 * 05/29/2003: D Mackie  S0203447: renumbered the MVA* vendor codes to go from 0 to 215.
 *                                 This numbering corresponds with the errorStrings array, which contains
 *                                 the proper string keys for getting messages from the Resources.properties file.
 * 14Jan2005: damack     S0278967: added a vendor code and text tag for property description for coercenumerictypes
 */
/*
 * NOTE:
 * Messages added to the Resources.properties file require that the key go at the end of 
 * the errorStrings array, and that the number corresponding to the index of the new message be
 * added at the bottom of the vendor codes list.  New codes should map directly to the index of their
 * key.
 * Any questions should be directed to Dave Mackie (Dave.Mackie@sas.com) x13808
 */
public interface VendorCode
{
    // VC_NOCONNECTION goes with General.noConnectionError.txt
    public static final int VC_NOCONNECTION = 1;
    // VC_CTORERROR goes with the *.constructorError.txt
    public static final int VC_CTORERROR = 2;
    // VC_NEXTERROR goes with *.nextError.txt
    public static final int VC_NEXTERROR = 3;
    // VC_NOSUPMETHOD goes with General.methodNotSupported.txt
    public static final int VC_NOSUPMETHOD = 4;
    /**
     * MVAStatement error codes:
     */
    // QUERYNOTSELECT goes with MVAStatement.queryNotSelect.txt
    // ("query not a select statement")
    public static final int VC_QUERYNOTSELECT = 5;
    // VC_UNKNOWNERROR == "unknown * error", like WXEIOERR
    // ie, "<operation> failed for an unknown reason".  the catchall
    static final int VC_UNKNOWNERROR = 6;
    // VC_RESULTSETERROR - MVAStatement.genericResultSetError.txt
    static final int VC_RESULTSETERROR = 7;
    // VC_NOSTATMENT == MVAStatement.noStatementError.txt
    static final int VC_NOSTATEMENT = 8;
    // can't select in an update, matches MVAStatment.noSelectInUpdate.txt
    static final int VC_ERRSELECTINUPDATE = 9;
    // MVAStatement.maxFieldsError.txt
    static final int VC_MAXFIELDSERR = 10;
    static final int VC_MAXROWSERR = 11;
    static final int VC_FETCHDIRERR = 12;
    static final int VC_FETCHSIZEERR = 13;
    static final int VC_MALSTATEMENT = 14;
    static final int VC_MALTIME = 15;
    static final int VC_MALDATE = 16;
    static final int VC_MALTIMESTAMP = 17;
    static final int VC_MALOUTERJOIN = 18;
    static final int VC_MALESCAPE = 19;
    static final int VC_MALFUNC = 20;
    static final int VC_MALCONCATFUNC = 21;
    static final int VC_MALLEFTFUNC = 22;
    static final int VC_MALREPEATFUNC = 23;
    static final int VC_MALSPACEFUNC = 24;
    static final int VC_MALPOWERFUNC = 25;
    static final int VC_MALROUNDFUNC = 26;
    static final int VC_MALDAYNAMEFUNC = 27;
    static final int VC_MALDAYYEARFUNC = 28;
    static final int VC_MALCHANGEMNTHNAMEFUNC = 29;
    static final int VC_UNDEFFUNC = 30;
    static final int VC_GETFAILED = 31;
    // NE suffix means that error was a "NamingException"
    static final int VC_GETFAILEDNE = 32;
    static final int VC_MISSINGLDAPENTRY = 33;
    static final int VC_INVALIDLDAPENTRY = 34;
    // again, the NE means that it was originally a NamingException
    static final int VC_AUTHUSERERRORNE = 35;
    // MVAConnection codes:
    // couldn't get the workspace
    static final int VC_NOWORKSPACE = 36;
    // transactions not supported
    static final int VC_TRANSACTIONSNOSUP = 37;
    // no server specified
    static final int VC_NOSERVERSPEC = 38;
    // bad port spec. (too short)
    static final int VC_INVALIDPORTSPEC = 39;
    // dbms connection is null.
    static final int VC_NULLDBMSCONNECTION = 40;
    // MVAConnection warnings (VC_W*)
    static final int VC_WINVALIDPNAME = 41;
    static final int VC_WNOACCESSMETH = 42;
    static final int VC_WINVALIDLNAME = 43;
    static final int VC_WNOLIB = 44;
    static final int VC_WLNAMENOTASS = 45;
    static final int VC_WINVALIDENG = 46;
    static final int VC_ASSLIBREFERR = 47;
    // MVAPreparedStatment:
    static final int VC_TYPENOTSUP = 48;
    static final int VC_INVALIDINDEX = 49;
    static final int VC_INVALIDTYPE = 50;
    static final int VC_UNKNOWNTYPE = 51;
    static final int VC_NULLPARMERR = 52;
    // MVAResultSet
    static final int VC_CANNOTCONVERT = 53;
    static final int VC_INVALIDROWPOS = 54;
    static final int VC_NUMFORMATERR = 55;
    static final int VC_NOSUCHCOLUMN = 56;
    static final int VC_PREVERROR = 57;
    static final int VC_INVALIDROWNUM = 58;
    static final int VC_TYPEFORWARDONLY = 59;
    static final int VC_RSNOTUPDATABLE = 60;
    static final int VC_INVALIDINPUT = 61;
    static final int VC_RECORDLOCKED = 62;
    static final int VC_BOOKMARKCORRUPT = 63;
    static final int VC_RECORDDELETED = 64;
    static final int VC_SERVERERROR = 65;
    static final int VC_BADDATA = 66;
    static final int VC_LOCKFAILED = 67;
    static final int VC_READFAILED = 68;
    // MVADatabaseMetaData:
    static final int VC_NOSUPSTOREDPROCS = 69;
    static final int VC_NOSUPCATALOGS = 70;
    static final int VC_NOTABLENAME = 71;
    static final int VC_NOSUPFOREIGNKEYS = 72;
    static final int VC_INVALIDCOLVAL = 73;
    /**
     * new school vendor codes:
     */
    /* begin */
    static final int General_invalidColumnType                         = 0;
    static final int General_methodNotSupported                         = 1;
    static final int General_noConnectionError                          = 2;
    static final int General_invalidColumnValue                         = 3;
    static final int General_methodNotImplemented                       = 4;
    static final int Column_setColumnNameError                          = 5;
    static final int Column_setMaxLengthError                           = 6;
    static final int LDAPColumnsResultSet_constructorError              = 7;
    static final int LDAPColumnsResultSet_nextError                     = 8;
    static final int LDAPDatabaseMetaData_constructorError              = 9;
    static final int LDAPDatabaseMetaData_getTableTypesError            = 10;
    static final int LDAPDatabaseMetaData_getTablesError                = 11;
    static final int LDAPDatabaseMetaData_getSchemasError               = 12;
    static final int LDAPDatabaseMetaData_getColumnsError               = 13;
    static final int LDAPDatabaseMetaData_getSASColumnsError            = 14;
    static final int LDAPDatabaseMetaData_getSASTablesError             = 15;
    static final int LDAPDatabaseMetaData_getContextDataError           = 16;
    static final int LDAPDatabaseMetaData_getSASTablednError            = 17 ;
    static final int LDAPDatabaseMetaData_getSASLibrariesError          = 18 ;
    static final int LDAPDatabaseMetaData_getSASLibrarydnError          = 19;
    static final int LDAPDatabaseMetaData_getSASLibrefError             = 20;
    static final int LDAPDatabaseMetaData_missingLDAPEntryError         = 21;
    static final int LDAPDatabaseMetaData_invalidLDAPEntryError         = 22;
    static final int LDAPDatabaseMetaData_authenticateUserError         = 23;
    static final int LDAPSchemaResultSet_constructorError               = 24;
    static final int LDAPSchemaResultSet_nextError                      = 25;
    static final int LDAPTablesResultSet_constructorError               = 26;
    static final int LDAPTablesResultSet_nextError                      = 27;
    static final int LDAPTableTypesResultSet_constructorError           = 28;
    static final int LDAPTableTypesResultSet_nextError                  = 29;
    static final int MVAConnection_generalSQLExceptionChained           = 30;
    static final int MVAConnection_generalSQLException                  = 31;
    static final int MVAConnection_error                                = 32;
    static final int MVAConnection_workspaceError                       = 33;
    static final int MVAConnection_createStatementError                 = 34;
    static final int MVAConnection_prepareStatementError                = 35;
    static final int MVAConnection_transactionError                     = 36;
    static final int MVAConnection_closeError                           = 37;
    static final int MVAConnection_getMetadataError                     = 38;
    static final int MVAConnection_noTransactionsError                  = 39;
    static final int MVAConnection_urlMustSpecifyServer                 = 40;
    static final int MVAConnection_nullDbmsConnection                   = 41;
    static final int MVAConnection_executeUndoPolicyError               = 42;
    static final int MVAConnection_invalidPnameWarning                  = 43;
    static final int MVAConnection_noAccessMethodWarning                = 44;
    static final int MVAConnection_invalidLNameWarning                  = 45;
    static final int MVAConnection_noLibraryWarning                     = 46;
    static final int MVAConnection_lnameNotAssignedWarning              = 47;
    static final int MVAConnection_invalidEngineWarning                 = 48;
    static final int MVAConnection_assignLibrefError                    = 49;
    static final int MVADatabaseMetaData_productVersionError            = 50;
    static final int MVADatabaseMetaData_storedProcsUnsupported         = 51;
    static final int MVADatabaseMetaData_getTableError                  = 52;
    static final int MVADatabaseMetaData_getSchemaError                 = 53;
    static final int MVADatabaseMetaData_getCatalogsError               = 54;
    static final int MVADatabaseMetaData_getTableTypesError             = 55;
    static final int MVADatabaseMetaData_getColumnsError                = 56;
    static final int MVADatabaseMetaData_noTableNameError               = 57;
    static final int MVADatabaseMetaData_getBestRowIdentifierError      = 58;
    static final int MVADatabaseMetaData_getPrimaryKeysError            = 59;
    static final int MVADatabaseMetaData_foreignKeysError               = 60;
    static final int MVADatabaseMetaData_getIndexInfoError              = 61;
    static final int MVAModifiedResultSet_nextError                     = 62;
    static final int MVAPreparedStatement_invalidParamIndexError        = 63;
    static final int MVAPreparedStatement_setNullError                  = 64;
    static final int MVAPreparedStatement_sqlTypeBITUnsupported         = 65;
    static final int MVAPreparedStatement_sqlTypeTINYINTUnsupported     = 66;
    static final int MVAPreparedStatement_sqlTypeSMALLINTUnsupported    = 67;
    static final int MVAPreparedStatement_sqlTypeBigIntUnsupported      = 68;
    static final int MVAPreparedStatement_sqlTypeNumericUnsupported     = 69;
    static final int MVAPreparedStatement_sqlTypeBINARYUnsupported      = 70;
    static final int MVAPreparedStatement_sqlTypeLONGVARCHARUnsupported = 71;
    static final int MVAPreparedStatement_unknownObjectType             = 72;
    static final int MVAPreparedStatement_getMetaDataError              = 73;
    static final int MVAPreparedStatement_substituteParamError          = 74;
    static final int MVAResultSet_invalidRowPosition                    = 75;
    static final int MVAResultSet_cannotConvertToBoolean                = 76;
    static final int MVAResultSet_cannotConvertToByte                   = 77;
    static final int MVAResultSet_numberFormatError                     = 78;
    static final int MVAResultSet_cannotConvertToShort                  = 79;
    static final int MVAResultSet_cannotConvertToInt                    = 80;
    static final int MVAResultSet_cannotConvertToLong                   = 81;
    static final int MVAResultSet_cannotConvertToFloat                  = 82;
    static final int MVAResultSet_cannotConvertToNumeric                = 83;
    static final int MVAResultSet_getBytesError                         = 84;
    static final int MVAResultSet_cannotConvertToDate                   = 85;
    static final int MVAResultSet_cannotConvertToTime                   = 86;
    static final int MVAResultSet_cannotConvertToTimestamp              = 87;
    static final int MVAResultSet_cannotConvertToInputStream            = 88;
    static final int MVAResultSet_findColumnError                       = 89 ;
    static final int MVAResultSet_cannotConvertToReader                 = 90;
    static final int MVAResultSet_nextError                             = 91;
    static final int MVAResultSet_previousError                         = 92;
    static final int MVAResultSet_readRecordsFailed                     = 93;
    static final int MVAResultSet_isLastError                           = 94;
    static final int MVAResultSet_invalidRowNumber                      = 95;
    static final int MVAResultSet_absoluteRowError                      = 96;
    static final int MVAResultSet_absoluteError                         = 97;
    static final int MVAResultSet_invalidFetchDirection                 = 98;
    static final int MVAResultSet_invalidFetchSize                      = 99;
    static final int MVAResultSet_rowUpdatedError                       = 100;
    static final int MVAResultSet_rowInsertedError                      = 101;
    static final int MVAResultSet_rowDeletedError                       = 102;
    static final int MVAResultSet_notUpdatableResultSetError            = 103;
    static final int MVAResultSet_cannotConvertToDouble                 = 104;
    static final int MVAResultSet_cannotConvertToBigDecimal             = 105;
    static final int MVAResultSet_invalidInputString                    = 106;
    static final int MVAResultSet_updateAsciiStreamFailedError          = 107;
    static final int MVAResultSet_updateBinaryStreamFailedError         = 108;
    static final int MVAResultSet_updateCharStreamFailedError           = 109 ;
    static final int MVAResultSet_unknownObjectType                     = 110;
    static final int MVAResultSet_recordLockedError                     = 111;
    static final int MVAResultSet_recordBookmarkError                   = 112;
    static final int MVAResultSet_recordDeletedError                    = 113;
    static final int MVAResultSet_insertServerError                     = 114;
    static final int MVAResultSet_insertFailedError                     = 115;
    static final int MVAResultSet_lockFailedBadData                     = 116;
    static final int MVAResultSet_lockRecordsFailedError                = 117;
    static final int MVAResultSet_updateFailedRecordLocked              = 118;
    static final int MVAResultSet_updateFailedBadBookmark               = 119;
    static final int MVAResultSet_updateFailedRecordDeleted             = 120;
    static final int MVAResultSet_updateFailedServerError               = 121;
    static final int MVAResultSet_updateFailed                          = 122;
    static final int MVAResultSet_notDeletableResultSetError            = 123;
    static final int MVAResultSet_lockRecordsFailed                     = 124;
    static final int MVAResultSet_deleteFailedRecordLocked              = 125;
    static final int MVAResultSet_deleteFailedBadBookmark               = 126;
    static final int MVAResultSet_deleteFailedRecordDeleted             = 127;
    static final int MVAResultSet_deleteFailedServerError               = 128;
    static final int MVAResultSet_deleteFailed                          = 129;
    static final int MVAResultSet_refreshRowError                       = 130;
    static final int MVAResultSet_insertsNotSupported                   = 131;
    static final int MVAResultSet_fetchsaveError                        = 132;
    static final int MVAResultSet_fetchRefreshError                     = 133;
    static final int MVAResultSet_getColumnInformationError             = 134;
    static final int MVAResultSet_openDataSetError                      = 135;
    static final int MVAResultSetMetaData_noResultSetError              = 136;
    static final int MVAStatement_querySelectError                      = 137;
    static final int MVAStatement_genericQueryError                     = 138;
    static final int MVAStatement_genericResultSetError                 = 139;
    static final int MVAStatement_noStatementError                      = 140;
    static final int MVAStatement_noSelectInUpdateError                 = 141;
    static final int MVAStatement_executeUpdateError                    = 142;
    static final int MVAStatement_maxFieldsError                        = 143;
    static final int MVAStatement_maxRowsError                          = 144;
    static final int MVAStatement_executeError                          = 145;
    static final int MVAStatement_resultSetError                        = 146;
    static final int MVAStatement_fetchDirectionError                   = 147;
    static final int MVAStatement_fetchSizeError                        = 148;
    static final int MVAStatement_batchUpdateException                  = 149;
    static final int MVAStatement_nullLibrefError                       = 150;
    static final int MVAStatement_nullTableNameError                    = 151;
    static final int MVAStatement_nullArrayError                        = 152;
    static final int MVAStatement_createTableError                      = 153;
    static final int MVAStatement_genericResultSetError2                = 154;
    static final int MVAStatement_deleteFailed                          = 155;
    static final int MVAStatement_useLibrefError                        = 156;
    static final int MVAStatement_malformedStatement                    = 157;
    static final int MVAStatement_malformedTime                         = 158;
    static final int MVAStatement_malformedDate                         = 159;
    static final int MVAStatement_malformedTimeStamp                    = 160;
    static final int MVAStatement_malformedOuterJoin                    = 161;
    static final int MVAStatement_malformedEscape                       = 162;
    static final int MVAStatement_malformedFunction                     = 163;
    static final int MVAStatement_undefinedFunction                     = 164;
    static final int MVAStatement_malformedConcatFunction               = 165;
    static final int MVAStatement_malformedLeftFunction                 = 166;
    static final int MVAStatement_malformedRepeatFunction               = 167;
    static final int MVAStatement_malformedSpaceFunction                = 168;
    static final int MVAStatement_malformedPowerFunction                = 169;
    static final int MVAStatement_malformedDayNameFunction              = 170;
    static final int MVAStatement_malformedDayOfYearFunction            = 171;
    static final int MVAStatement_malformedChangeMonthName              = 172;
    static final int MVAStatement_malformedRoundFunction                = 173;
    static final int MVAResultSet_updateNullError                       = 174;
    static final int MVAConnection_scrollTypesWarning                   = 175;
    static final int MVAConnection_concurrencyTypesWarning              = 176;
    static final int MVAConnection_workspaceConnectionError             = 177;
    static final int MVAConnection_rsTypeWarning                        = 178;
    static final int MVAResultSet_concurrencyWarning                    = 179;
    static final int MVAResultSet_missingByteWarning                    = 180;
    static final int MVAResultSet_missingDoubleWarning                  = 181;
    static final int MVAResultSet_missingFloatWarning                   = 182;
    static final int MVAResultSet_missingIntWarning                     = 183;
    static final int MVAResultSet_missingLongWarning                    = 184;
    static final int MVAResultSet_missingShortWarning                   = 185;
    static final int MVAPreparedStatement_sqlTypeLONGVARBINARYUnsupported = 186;
    static final int MVAResultSet_invalidRSType                         = 187;
    static final int MVAResultSet_isBeforeFirstError                    = 188;
    static final int MVAResultSet_isAfterLastError                      = 189;
    static final int MVAResultSet_notOnInsertRowError                   = 190;
    static final int MVAStatement_rsNotUpdatableWarning                 = 191;
    static final int MVAResultSet_insertRowUpdateError                  = 192;
    static final int MVAResultSet_unableToMoveToInsertRowError          = 193;
    static final int MVAResultSet_unableToMoveToCurrentRowError         = 194;
    static final int MVAResultSet_cantPerformForwardOnly                = 195;
    static final int MVAResultSet_notOpenError                          = 196;
    static final int MVAResultSet_insertFailedNoDataError               = 197;
    
    /************************************************************/
    /* codes for property descriptions, see S0203447            */
    /************************************************************/
    static final int DESC_USER_NAME               = 198;
    static final int DESC_PASSWORD                = 199;
    static final int DESC_DBMS                    = 200;
    static final int DESC_DBMS_OPTIONS            = 201;
    static final int DESC_REMARKS                 = 202;
    static final int DESC_UNDO_POLICY_NONE        = 203;
    static final int DESC_LIBREFS                 = 204;
    static final int DESC_ENCRYPTIONPOLICY        = 205;
    static final int DESC_ENCRYPTIONCONTENT       = 206;
    static final int DESC_ENCRYPTIONALGORITHMS    = 207;
    static final int DESC_LDAP_DBMD               = 208;
    static final int DESC_PROVIDER_URL            = 209;
    static final int DESC_SECURITY_AUTHENTICATION = 210;
    static final int DESC_BASE_CONTEXT            = 211;
    static final int DESC_SECURITY_PRINCIPAL      = 212;
    static final int DESC_SECURITY_CREDENTIALS    = 213;
    static final int DESC_APPLY_FORMATS           = 214;
    static final int DESC_RMIURL                  = 215;
    static final int DESC_SASIFYTABLENAMES        = 216;
    static final int DESC_COERCENUMERICTYPES      = 217;
    /**************************************************************************************************/
    static final String[] errorStrings = { 
        /* General_invalidColumnType */ 
        "General.invalidColumnType.txt",
        /*General_methodNotSupported */ 
        "General.methodNotSupported.txt",
        /* General_noConnectionError */ 
        "General.noConnectionError.txt",
        /*General_invalidColumnValue */ 
        "General.invalidColumnValue.txt",
        /*General_methodNotImplemented */ 
        "General.methodNotImplemented.txt",
        /*Column_setColumnNameError */ 
        "Column.setColumnNameError.txt",
        /*Column_setMaxLengthError */ 
        "Column.setMaxLengthError.txt",
        /*LDAPColumnsResultSet_constructorError */ 
        "LDAPColumnsResultSet.constructorError.txt",
        /*LDAPColumnsResultSet_nextError */ 
        "LDAPColumnsResultSet.nextError.txt",
        /*LDAPDatabaseMetaData_constructorError */ 
        "LDAPDatabaseMetaData.constructorError.txt",
        /*LDAPDatabaseMetaData_getTableTypesError */ 
        "LDAPDatabaseMetaData.getTableTypesError.txt",
        /*LDAPDatabaseMetaData_getTablesError */ 
        "LDAPDatabaseMetaData.getTablesError.txt",
        /*LDAPDatabaseMetaData_getSchemasError */ 
        "LDAPDatabaseMetaData.getSchemasError.txt",
        /*LDAPDatabaseMetaData_getColumnsError */ 
        "LDAPDatabaseMetaData.getColumnsError.txt",
        /*LDAPDatabaseMetaData_getSASColumnsError */ 
        "LDAPDatabaseMetaData.getSASColumnsError.txt",
        /*LDAPDatabaseMetaData_getSASTablesError */ 
        "LDAPDatabaseMetaData.getSASTablesError.txt",
        /*LDAPDatabaseMetaData_getContextDataError */ 
        "LDAPDatabaseMetaData.getContextDataError.txt",
        /*LDAPDatabaseMetaData_getSASTablednError */ 
        "LDAPDatabaseMetaData.getSASTablednError.txt",
        /*LDAPDatabaseMetaData_getSASLibrariesError */ 
        "LDAPDatabaseMetaData.getSASLibrariesError.txt",
        /*LDAPDatabaseMetaData_getSASLibrarydnError */ 
        "LDAPDatabaseMetaData.getSASLibrarydnError.txt",
        /*LDAPDatabaseMetaData_getSASLibrefError */ 
        "LDAPDatabaseMetaData.getSASLibrefError.txt",
        /*LDAPDatabaseMetaData_missingLDAPEntryError */ 
        "LDAPDatabaseMetaData.missingLDAPEntryError.txt",
        /*LDAPDatabaseMetaData_invalidLDAPEntryError */ 
        "LDAPDatabaseMetaData.invalidLDAPEntryError.txt",
        /*LDAPDatabaseMetaData_authenticateUserError */ 
        "LDAPDatabaseMetaData.authenticateUserError.txt",
        /*LDAPSchemaResultSet_constructorError */ 
        "LDAPSchemaResultSet.constructorError.txt",
        /*LDAPSchemaResultSet_nextError */ 
        "LDAPSchemaResultSet.nextError.txt",
        /*LDAPTablesResultSet_constructorError */ 
        "LDAPTablesResultSet.constructorError.txt",
        /*LDAPTablesResultSet_nextError */ 
        "LDAPTablesResultSet.nextError.txt",
        /*LDAPTableTypesResultSet_constructorError */ 
        "LDAPTableTypesResultSet.constructorError.txt",
        /*LDAPTableTypesResultSet_nextError */ 
        "LDAPTableTypesResultSet.nextError.txt",
        /*MVAConnection_generalSQLExceptionChained */ 
        "MVAConnection.generalSQLExceptionChained.fmt",
        
        /*MVAConnection_generalSQLException */ 
        "MVAConnection.generalSQLException.fmt",
        /*MVAConnection_error */ 
        "MVAConnection.error.txt",
        /*MVAConnection_workspaceError */ 
        "MVAConnection.workspaceError.txt",
        /*MVAConnection_createStatementError */ 
        "MVAConnection.createStatementError.txt",
        /*MVAConnection_prepareStatementError */ 
        "MVAConnection.prepareStatementError.txt",
        /*MVAConnection_transactionError */ 
        "MVAConnection.transactionError.txt",
        /*MVAConnection_closeError */ 
        "MVAConnection.closeError.txt",
        /*MVAConnection_getMetadataError */ 
        "MVAConnection.getMetadataError.txt",
        /*MVAConnection_noTransactionsError */ 
        "MVAConnection.noTransactionsError.txt",
        /*MVAConnection_urlMustSpecifyServer */ 
        "MVAConnection.urlMustSpecifyServer.txt",
        /*MVAConnection_nullDbmsConnection */ 
        "MVAConnection.nullDbmsConnection.txt",
        /*MVAConnection_executeUndoPolicyError */ 
        "MVAConnection.executeUndoPolicyError.txt",
        /*MVAConnection_invalidPnameWarning */ 
        "MVAConnection.invalidPnameWarning.txt",
        /*MVAConnection_noAccessMethodWarning */ 
        "MVAConnection.noAccessMethodWarning.txt",
        /* MVAConnection_invalidLNameWarning */ 
        "MVAConnection.invalidLNameWarning.txt",
        /*MVAConnection_noLibraryWarning */ 
        "MVAConnection.noLibraryWarning.txt",
        /*MVAConnection_lnameNotAssignedWarning */ 
        "MVAConnection.lnameNotAssignedWarning.txt",
        /*MVAConnection_invalidEngineWarning */ 
        "MVAConnection.invalidEngineWarning.txt",
        /*MVAConnection_assignLibrefError */ 
        "MVAConnection.assignLibrefError.txt",
        /*MVADatabaseMetaData_productVersionError */ 
        "MVADatabaseMetaData.productVersionError.txt",
        /*MVADatabaseMetaData_storedProcsUnsupported */ 
        "MVADatabaseMetaData.storedProcsUnsupported.txt",
        /*MVADatabaseMetaData_getTableError */ 
        "MVADatabaseMetaData.getTableError.txt",
        /*MVADatabaseMetaData_getSchemaError */ 
        "MVADatabaseMetaData.getSchemaError.txt",
        /*MVADatabaseMetaData_getCatalogsError */ 
        "MVADatabaseMetaData.getCatalogsError.txt",
        /*MVADatabaseMetaData_getTableTypesError */ 
        "MVADatabaseMetaData.getTableTypesError.txt",
        /*MVADatabaseMetaData_getColumnsError */ 
        "MVADatabaseMetaData.getColumnsError.txt",
        /*MVADatabaseMetaData_noTableNameError */ 
        "MVADatabaseMetaData.noTableNameError.txt",
        /*MVADatabaseMetaData_getBestRowIdentifierError */ 
        "MVADatabaseMetaData.getBestRowIdentifierError.txt",
        /*MVADatabaseMetaData_getPrimaryKeysError */ 
        "MVADatabaseMetaData.getPrimaryKeysError.txt",
        /*MVADatabaseMetaData_foreignKeysError */ 
        "MVADatabaseMetaData.foreignKeysError.txt",
        /*MVADatabaseMetaData_getIndexInfoError */ 
        "MVADatabaseMetaData.getIndexInfoError.txt",
        /*MVAModifiedResultSet_nextError */ 
        "MVAModifiedResultSet.nextError.txt",
        /*MVAPreparedStatement_invalidParamIndexError */ 
        "MVAPreparedStatement.invalidParamIndexError.txt",
        /*MVAPreparedStatement_setNullError */ 
        "MVAPreparedStatement.setNullError.txt",
        /*MVAPreparedStatement_sqlTypeBITUnsupported */ 
        "MVAPreparedStatement.sqlTypeBITUnsupported.txt",
        /*MVAPreparedStatement_sqlTypeTINYINTUnsupported */ 
        "MVAPreparedStatement.sqlTypeTINYINTUnsupported.txt",
        /*MVAPreparedStatement_sqlTypeSMALLINTUnsupported */ 
        "MVAPreparedStatement.sqlTypeSMALLINTUnsupported.txt",
        /*MVAPreparedStatement_sqlTypeBigIntUnsupported */ 
        "MVAPreparedStatement.sqlTypeBigIntUnsupported.txt",
        /*MVAPreparedStatement_sqlTypeNumericUnsupported */ 
        "MVAPreparedStatement.sqlTypeNumericUnsupported.txt",
        /*MVAPreparedStatement_sqlTypeBINARYUnsupported */ 
        "MVAPreparedStatement.sqlTypeBINARYUnsupported.txt",
        /*MVAPreparedStatement_sqlTypeLONGVARCHARUnsupported */ 
        "MVAPreparedStatement.sqlTypeLONGVARCHARUnsupported.txt",
        /*MVAPreparedStatement_unknownObjectType */ 
        "MVAPreparedStatement.unknownObjectType.txt",
        /*MVAPreparedStatement_getMetaDataError */ 
        "MVAPreparedStatement.getMetaDataError.txt",
        
        /*  MVAPreparedStatement_substituteParamError */ 
        "MVAPreparedStatement.substituteParamError.fmt",
        /*  MVAResultSet_invalidRowPosition */ 
        "MVAResultSet.invalidRowPosition.txt",
        /*MVAResultSet_cannotConvertToBoolean */ 
        "MVAResultSet.cannotConvertToBoolean.txt",
        /*MVAResultSet_cannotConvertToByte */ 
        "MVAResultSet.cannotConvertToByte.txt",
        /*MVAResultSet_numberFormatError */ 
        "MVAResultSet.numberFormatError.txt",
        /*MVAResultSet_cannotConvertToShort */ 
        "MVAResultSet.cannotConvertToShort.txt",
        /*MVAResultSet_cannotConvertToInt */ 
        "MVAResultSet.cannotConvertToInt.txt",
        /*MVAResultSet_cannotConvertToLong */ 
        "MVAResultSet.cannotConvertToLong.txt",
        /*MVAResultSet_cannotConvertToFloat */ 
        "MVAResultSet.cannotConvertToFloat.txt",
        /*MVAResultSet_cannotConvertToNumeric */ 
        "MVAResultSet.cannotConvertToNumeric.txt",
        /*MVAResultSet_getBytesError */ 
        "MVAResultSet.getBytesError.txt",
        /*MVAResultSet_cannotConvertToDate */ 
        "MVAResultSet.cannotConvertToDate.txt",
        /*MVAResultSet_cannotConvertToTime */ 
        "MVAResultSet.cannotConvertToTime.txt",
        /*MVAResultSet_cannotConvertToTimestamp */ 
        "MVAResultSet.cannotConvertToTimestamp.txt",
        /*MVAResultSet_cannotConvertToInputStream */ 
        "MVAResultSet.cannotConvertToInputStream.txt",
        /*MVAResultSet_findColumnError */ 
        "MVAResultSet.findColumnError.txt",
        /*MVAResultSet_cannotConvertToReader */ 
        "MVAResultSet.cannotConvertToReader.txt",
        /*MVAResultSet_nextError */ 
        "MVAResultSet.nextError.txt",
        /*MVAResultSet_previousError */ 
        "MVAResultSet.previousError.txt",
        /*MVAResultSet_readRecordsFailed */ 
        "MVAResultSet.readRecordsFailed.txt",
        /*MVAResultSet_isLastError */ 
        "MVAResultSet.isLastError.txt",
        /*MVAResultSet_invalidRowNumber */ 
        "MVAResultSet.invalidRowNumber.txt",
        /*MVAResultSet_absoluteRowError */ 
        "MVAResultSet.absoluteRowError.txt",
        /*MVAResultSet_absoluteError */ 
        "MVAResultSet.absoluteError.txt",
        /*MVAResultSet_invalidFetchDirection */ 
        "MVAResultSet.invalidFetchDirection.txt",
        /*MVAResultSet_invalidFetchSize */ 
        "MVAResultSet.invalidFetchSize.txt",
        /*MVAResultSet_rowUpdatedError */ 
        "MVAResultSet.rowUpdatedError.txt",
        /*MVAResultSet_rowInsertedError */ 
        "MVAResultSet.rowInsertedError.txt",
        /*MVAResultSet_rowDeletedError */ 
        "MVAResultSet.rowDeletedError.txt",
        /*MVAResultSet_notUpdatableResultSetError */ 
        "MVAResultSet.notUpdatableResultSetError.txt",
        /*MVAResultSet_cannotConvertToDouble */
        "MVAResultSet.cannotConvertToDouble.txt",
        /*MVAResultSet_cannotConvertToBigDecimal */ 
        "MVAResultSet.cannotConvertToBigDecimal.txt",
        /*MVAResultSet_invalidInputString */ 
        "MVAResultSet.invalidInputString.txt",
        /*MVAResultSet_updateAsciiStreamFailedError */ 
        "MVAResultSet.updateAsciiStreamFailedError.txt",
        /*MVAResultSet_updateBinaryStreamFailedError */ 
        "MVAResultSet.updateBinaryStreamFailedError.txt",
        /*MVAResultSet_updateCharStreamFailedError */ 
        "MVAResultSet.updateCharStreamFailedError.txt",
        /*MVAResultSet_unknownObjectType */ 
        "MVAResultSet.unknownObjectType.txt",
        /*MVAResultSet_recordLockedError */ 
        "MVAResultSet.recordLockedError.txt",
        /* MVAResultSet_recordBookmarkError */ 
        "MVAResultSet.recordBookmarkError.txt",
        /* MVAResultSet_recordDeletedError */
        "MVAResultSet.recordDeletedError.txt",
        /* MVAResultSet_insertServerError */ 
        "MVAResultSet.insertServerError.txt",
        /* MVAResultSet_insertFailedError */ 
        "MVAResultSet.insertFailedError.txt",
        /* MVAResultSet_lockFailedBadData */ 
        "MVAResultSet.lockFailedBadData.txt",
        /* MVAResultSet_lockRecordsFailedError */ 
        "MVAResultSet.lockRecordsFailedError.txt",
        /* MVAResultSet_updateFailedRecordLocked */ 
        "MVAResultSet.updateFailedRecordLocked.txt",
        /* MVAResultSet_updateFailedBadBookmark */ 
        "MVAResultSet.updateFailedBadBookmark.txt",
        /* MVAResultSet_updateFailedRecordDeleted */ 
        "MVAResultSet.updateFailedRecordDeleted.txt",
        /* MVAResultSet_updateFailedServerError */ 
        "MVAResultSet.updateFailedServerError.txt",
        /* MVAResultSet_updateFailed */ 
        "MVAResultSet.updateFailed.txt",
        /* MVAResultSet_notDeletableResultSetError */ 
        "MVAResultSet.notDeletableResultSetError.txt",
        /* MVAResultSet_lockRecordsFailed */ 
        "MVAResultSet.lockRecordsFailed.txt",
        /* MVAResultSet_deleteFailedRecordLocked */ 
        "MVAResultSet.deleteFailedRecordLocked.txt",
        /* MVAResultSet_deleteFailedBadBookmark */ 
        "MVAResultSet.deleteFailedBadBookmark.txt",
        /* MVAResultSet_deleteFailedRecordDeleted */ 
        "MVAResultSet.deleteFailedRecordDeleted.txt",
        /* MVAResultSet_deleteFailedServerError */ 
        "MVAResultSet.deleteFailedServerError.txt",
        /* MVAResultSet_deleteFailed */
        "MVAResultSet.deleteFailed.txt",
        /* MVAResultSet_refreshRowError */ 
        "MVAResultSet.refreshRowError.txt",
        /* MVAResultSet_insertsNotSupported */ 
        "MVAResultSet.insertsNotSupported.txt",
        /* MVAResultSet_fetchsaveError */ 
        "MVAResultSet.fetchSaveError.txt",
        /* MVAResultSet_fetchRefreshError */ 
        "MVAResultSet.fetchRefreshError.txt",
        /* MVAResultSet_getColumnInformationError */ 
        "MVAResultSet.getColumnInformationError.txt",
        /* MVAResultSet_openDataSetError */ 
        "MVAResultSet.openDataSetError.txt",
        /* MVAResultSetMetaData_noResultSetError */ 
        "MVAResultSetMetaData.noResultSetError.txt",
        /* MVAStatement_querySelectError */ 
        "MVAStatement.querySelectError.txt",
        /* MVAStatement_genericQueryError */ 
        "MVAStatement.genericQueryError.txt",
        /* MVAStatement_genericResultSetError */ 
        "MVAStatement.genericResultSetError.txt",
        /* MVAStatement_noStatementError */ 
        "MVAStatement.noStatementError.txt",
        /* MVAStatement_noSelectInUpdateError */ 
        "MVAStatement.noSelectInUpdateError.txt",
        /* MVAStatement_executeUpdateError */ 
        "MVAStatement.executeUpdateError.txt",
        /* MVAStatement_maxFieldsError */ 
        "MVAStatement.maxFieldsError.txt",
        /* MVAStatement_maxRowsError */ 
        "MVAStatement.maxRowsError.txt",
        /* MVAStatement_executeError */ 
        "MVAStatement.executeError.txt",
        /* MVAStatement_resultSetError */ 
        "MVAStatement.resultSetError.txt",
        /* MVAStatement_fetchDirectionError */
        "MVAStatement.fetchDirectionError.txt",
        /* MVAStatement_fetchSizeError */ 
        "MVAStatement.fetchSizeError.txt",
        /* MVAStatement_batchUpdateException */
        "MVAStatement.batchUpdateException.txt",
        /* MVAStatement_nullLibrefError */ 
        "MVAStatement.nullLibrefError.txt",
        /* MVAStatement_nullTableNameError */ 
        "MVAStatement.nullTableNameError.txt",
        /* MVAStatement_nullArrayError */ 
        "MVAStatement.nullArrayError.txt",
        /* MVAStatement_createTableError */ 
        "MVAStatement.createTableError.fmt", 
        /* MVAStatement_genericResultSetError2 */ 
        "MVAStatement.genericResultSetError2.fmt", 
        /* MVAStatement_deleteFailed */ 
        "MVAStatement.deleteFailed.fmt", 
        /* MVAStatement_useLibrefError */ 
        "MVAStatement.useLibrefError.fmt", 
        /* MVAStatement_malformedStatement */ 
        "MVAStatement.malformedStatement.txt",
        /* MVAStatement_malformedTime */ 
        "MVAStatement.malformedTime.txt",
        /* MVAStatement_malformedDate */ 
        "MVAStatement.malformedDate.txt",
        /* MVAStatement_malformedTimeStamp */ 
        "MVAStatement.malformedTimeStamp.txt",
        /* MVAStatement_malformedOuterJoin */ 
        "MVAStatement.malformedOuterJoin.txt",
        /* MVAStatement_malformedEscape */ 
        "MVAStatement.malformedEscape.txt",
        /* MVAStatement_malformedFunction */ 
        "MVAStatement.malformedFunction.txt",
        /* MVAStatement_undefinedFunction */ 
        "MVAStatement.undefinedFunction.txt",
        /* MVAStatement_malformedConcatFunction */ 
        "MVAStatement.malformedConcatFunction.txt",
       /* MVAStatement_malformedLeftFunction */ 
        "MVAStatement.malformedLeftFunction.txt",
        /* MVAStatement_malformedRepeatFunction */ 
        "MVAStatement.malformedRepeatFunction.txt",
        /* MVAStatement_malformedSpaceFunction */ 
        "MVAStatement.malformedSpaceFunction.txt",
        /* MVAStatement_malformedPowerFunction */
        "MVAStatement.malformedPowerFunction.txt",
        /*  MVAStatement_malformedDayNameFunction */ 
        "MVAStatement.malformedDayNameFunction.txt",
        /*  MVAStatement_malformedDayOfYearFunction */ 
        "MVAStatement.malformedDayOfYearFunction.txt",
        /*  MVAStatement_malformedChangeMonthName */ 
        "MVAStatement.malformedChangeMonthName.txt",
        /*  MVAStatement_malformedRoundFunction */ 
        "MVAStatement.malformedRoundFunction.txt",

        /*  MVAResultSet_updateNullError */ 
        "MVAResultSet.updateNullError.fmt",
        /*  MVAConnection_scrollTypesWarning */ 
        "MVAConnection.scrollTypesWarning.txt",
        /*  MVAConnection_concurrencyTypesWarning */ 
        "MVAConnection.concurrencyTypesWarning.txt",
        /*  MVAConnection_workspaceConnectionError */ 
        "MVAConnection.workspaceConnectionError.fmt",
        /*  MVAConnection_rsTypeWarning */ 
        "MVAConnection.rsTypeWarning.txt",
        /*  MVAResultSet_concurrencyWarning */ 
        "MVAResultSet.concurrencyWarning.txt",
        /*  MVAResultSet_missingByteWarning */ 
        "MVAResultSet.missingByteWarning.txt",
        /*  MVAResultSet_missingDoubleWarning */ 
        "MVAResultSet.missingDoubleWarning.txt",
        /*  MVAResultSet_missingFloatWarning */ 
        "MVAResultSet.missingFloatWarning.txt",
        /*  MVAResultSet_missingIntWarning */ 
        "MVAResultSet.missingIntWarning.txt",
        /*  MVAResultSet_missingLongWarning */ 
        "MVAResultSet.missingLongWarning.txt",
        /*  MVAResultSet_missingShortWarning */ 
        "MVAResultSet.missingShortWarning.txt",
        /*  MVAPreparedStatement_sqlTypeLONGVARBINARYUnsupported */ 
        "MVAPreparedStatement.sqlTypeLONGVARBINARYUnsupported.txt",
        /*  MVAResultSet_invalidRSType */ 
        "MVAResultSet.invalidRSType.txt",
        /*  MVAResultSet_isBeforeFirstError */ 
        "MVAResultSet.isBeforeFirstError.txt",
        /*  MVAResultSet_isAfterLastError */ 
        "MVAResultSet.isAfterLastError.txt",
        /*  MVAResultSet_notOnInsertRowError */ 
        "MVAResultSet.notOnInsertRowError.txt",
        /*  MVAStatement_rsNotUpdatableWarning */ 
        "MVAStatement.rsNotUpdatableWarning.txt",
        /*  MVAResultSet_insertRowUpdateError */ 
        "MVAResultSet.insertRowUpdateError.txt",
        /*  MVAResultSet_unableToMoveToInsertRowError */ 
        "MVAResultSet.unableToMoveToInsertRowError.txt",
        /*  MVAResultSet_unableToMoveToCurrentRowError */ 
        "MVAResultSet.unableToMoveToCurrentRowError.txt",
        /*  MVAResultSet_cantPerformForwardOnly */ 
        "MVAResultSet.cantPerformForwardOnly.txt",
        /*  MVAResultSet_notOpenError */ 
        "MVAResultSet.notOpenError.txt",
        /*  MVAResultSet_insertFailedNoDataError */ 
        "MVAResultSet.insertFailedNoDataError.txt",
        
        /************************************************************/
        /* codes for property descriptions, see S0203447            */
        /************************************************************/
        /*  DESC_USER_NAME               */ 
        "Desc.User_Name.txt",
        /*  DESC_PASSWORD                */ 
        "Desc.Password.txt",
        /*  DESC_DBMS                    */ 
        "Desc.Dbms.txt",
        /*  DESC_DBMS_OPTIONS            */ 
        "Desc.Dbms_Options.txt",
        /*  DESC_REMARKS                 */ 
        "Desc.Remarks.txt",
        /*  DESC_UNDO_POLICY_NONE        */ 
        "Desc.Undo_Policy_None.txt",
        /*  DESC_LIBREFS                 */ 
        "Desc.Librefs.txt",
        /*  DESC_ENCRYPTIONPOLICY        */ 
        "Desc.EncryptionPolicy.txt",
        /*  DESC_ENCRYPTIONCONTENT       */ 
        "Desc.EncryptionContent.txt",
        /*  DESC_ENCRYPTIONALGORITHMS    */ 
        "Desc.EncryptionAlgorithms.txt",
        /*  DESC_LDAP_DBMD               */ 
        "Desc.Ldap_Dbmd.txt",
        /*  DESC_PROVIDER_URL            */ 
        "Desc.Provider_Url.txt",
        /*  DESC_SECURITY_AUTHENTICATION */ 
        "Desc.Security_Authentication.txt",
        /*  DESC_BASE_CONTEXT            */ 
        "Desc.Base_Context.txt",
        /*  DESC_SECURITY_PRINCIPAL      */ 
        "Desc.Security_Principal.txt", 
        /*  DESC_SECURITY_CREDENTIALS    */ 
        "Desc.Security_Credentials.fmt",
        /*  DESC_APPLY_FORMATS           */ 
        "Desc.Apply_Formats.txt",
        /*  DESC_RMIURL                  */ 
        "Desc.RmiUrl.txt",
        /* DESC_SASIFYTABLENAMES         */
        "Desc.SasifyTableNames.txt",
        "Desc.CoerceNumericTypes.txt"
        
    };
}
