/* Copyright (c) 2000 by SAS Institute Inc., Cary, NC 27513 */
package com.sas.security;

// PermissionCombiner splices in a set of protection domains
// to allow a known secruity context to be created.
public class PermissionCombiner
	implements java.security.DomainCombiner
{
	java.security.ProtectionDomain[] domains;

	PermissionCombiner(java.security.ProtectionDomain[] d)
	{
		domains = d;
	}

	// Defined in java.security.DomainCombiner
	public java.security.ProtectionDomain[] combine(java.security.ProtectionDomain[] current, java.security.ProtectionDomain[] assigned)
	{
		java.util.Vector s = new java.util.Vector();

		// take the current protection domain and check it for the protectionDomains
		// we want to splice in. Add them if they need to be added.
		if (current != null)
		{
			for (int i=0;i<current.length;++i)
			{
				java.security.ProtectionDomain pd = current[i];

				if (!implies(pd, domains))
				{
					pd = cloneAndAddPermissions(pd);
				}

				s.addElement(pd);
			}
		}

		// take the assigned protection domain and check it for the protectionDomains
		// we want to splice in. Add them if they need to be added.
		if (assigned != null)
		{
			for (int i=0;i<assigned.length;++i)
			{
				java.security.ProtectionDomain pd = assigned[i];

				if (!s.contains(pd))
				{
					if (!implies(pd, domains))
					{
						pd = cloneAndAddPermissions(pd);
					}

					s.addElement(pd);
				}
			}
		}

		// create the new protectionDomain
		java.security.ProtectionDomain[] combined = new java.security.ProtectionDomain[s.size()];
		s.copyInto(combined);
		return combined;
	}

	// return true iff ALL permissions are implied
	private boolean implies(java.security.ProtectionDomain pd, java.security.ProtectionDomain[] testDomains)
	{
		int i;
		boolean flag = false;
		for(i = 0;i< testDomains.length;++i)
		{
			java.util.Enumeration p = testDomains[i].getPermissions().elements();
			while (p.hasMoreElements())
			{
				flag = true; // this is the fall through flag
				java.security.Permission d = (java.security.Permission)p.nextElement();
				if (!pd.implies(d))
					return false;
			}
		}
		return flag;
	}

	private java.security.ProtectionDomain cloneAndAddPermissions(java.security.ProtectionDomain protectionDomain)
	{
		java.security.PermissionCollection newPermissions = new java.security.Permissions();
		java.security.PermissionCollection permissionCollection = protectionDomain.getPermissions();
		if (permissionCollection != null)
		{
			java.util.Enumeration perms = permissionCollection.elements();

			while (perms.hasMoreElements())
			{
				newPermissions.add((java.security.Permission)perms.nextElement());
			}
		}

		int i;
		for(i = 0;i< domains.length;++i)
		{
			java.util.Enumeration p = domains[i].getPermissions().elements();
			while (p.hasMoreElements())
			{
				java.security.Permission d = (java.security.Permission)p.nextElement();
				if (!protectionDomain.implies(d))
					newPermissions.add(d);
			}
		}
		return new java.security.ProtectionDomain(protectionDomain.getCodeSource(),newPermissions);
	}

}
