// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

/**
  * StackChecker
  *
  * Checks the stack for a given method
  */

package com.sas.security;

public class StackChecker extends java.io.Writer
{
  protected java.util.Vector m_stack;
  
  public void write(char cbuf[], int off, int len) throws java.io.IOException
    {
      if (m_stack == null) {
        m_stack = new java.util.Vector();
      }
      m_stack.addElement(new String(cbuf));
    }

  public void flush()
    {
    }

  public void close()
    {
    }

  public boolean isOnStack(Object o, String methodName)
    {
      return isOnStack(o.getClass(), methodName);
    }

  public boolean isOnStack(Class c, String methodName)
    {
      return isOnStack(c, methodName, 2);
    }

  // Looks on the current stack for the given method in the given class.
  // If it appears in the stack at least the number of given times, true
  // is returned.
  
  public boolean isOnStack(Class c, String methodName, int atLeastNTimes)
    {
      boolean rc = false;

      // Null class can't be on the stack
      
      if (c == null) {
        return false;
      }

      // Clear the old stack

      if (m_stack != null) {
        m_stack.removeAllElements();
        m_stack = null;
      }
      
      // Create a throwable object so we can get the stack trace

      Throwable t = new Throwable();

      // Create a new PrintWriter object

      java.io.PrintWriter pw = new java.io.PrintWriter(this);

      // Get the stack trace

      t.printStackTrace(pw);

      // Format the full method name

      String fullName = c.getName() + "." + methodName;

      // Find the number of times the method is on the stack
      
      int n = find(fullName);

      if (n >= atLeastNTimes) {
        rc = true;
      }
      return rc;
    }

  // Count the number of times a method is found on the stack
  
  protected int find(String name)
    {
      int n = 0;

      if (m_stack != null) {
        for (int i = 0; i < m_stack.size(); i++) {
          String s = (String) m_stack.elementAt(i);
          if (s != null) {

            // If the given method name is on the stack, increment our counter
            
            if (s.indexOf(name) >= 0) {
              n++;
            }
          }
        }
      }
      return n;
    }
  
}
