package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 26Nov01 B Beers         S0133039: Created.
// 08Jan02 B Beers         S0135798: Add shutdownAfter/runForever feature.
// 16Jan02 B Beers         S0136628: Fixed one-off problem for reactivationLimit.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 30Apr02 B Beers         S0149671: Resolve some garbage collection problems.
// 13May02 B Beers         S0149086: Reinstate advanced connection management.
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 26Nov02 B Beers         S0151508: Revise xml error message handling.
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
//
/**
 * Used internally by the connection service to maintain information about factory-managed connections.
 * @author Biff Beers
 */
public final class ConnectionEnvelope
{
     private static int counter /* = 0 */;
     private org.omg.CORBA.Object rootObject /* = null */;
     private org.omg.CORBA.Object userObject /* = null */;
     private Credential userCredential /* = null */;
     private Credential ownerCredential /* = null */;
     private Server server /* = null */;
     private int serverIndex /* = 0 */;
     private int clientCount /* = 0 */;
     private PuddleEnvelope puddleEnvelope /* = null */;
     private int puddleIndex /* = 0 */;
     private int cxNumber = counter++;
     private int clientNumber /* = 0 */;
     private int clusterIndex /* = 0 */;
     private boolean bogus /* = false */;
     private Throwable cause /* = null */;
     private int reactivationCount /* = 0 */;
     private long shutdownTime = Long.MAX_VALUE;

     protected void setRootObject(org.omg.CORBA.Object rootObject)
     {
          this.rootObject = rootObject;
     }

     protected org.omg.CORBA.Object getRootObject()
     {
          return rootObject;
     }

     protected void setUserObject(org.omg.CORBA.Object userObject)
     {
          this.userObject = userObject;
     }

     protected org.omg.CORBA.Object getUserObject()
     {
          return (userObject != null) ? userObject : rootObject;
     }

     protected void setUserCredential(Credential userCredential)
     {
          this.userCredential = userCredential;
     }

     protected Credential getUserCredential()
     {
          return userCredential;
     }

     protected void setOwnerCredential(Credential ownerCredential)
     {
          this.ownerCredential = ownerCredential;
     }

     protected Credential getOwnerCredential()
     {
          return ownerCredential;
     }

     protected void setServer(Server server)
     {
          this.server = server;
     }

     protected Server getServer()
     {
          return server;
     }

     protected void setServerIndex(int serverIndex)
     {
          this.serverIndex = serverIndex;
     }

     protected int getServerIndex()
     {
          return serverIndex;
     }

     protected void incrClientCount()
     {
          clientCount++;
     }

     protected void decrClientCount()
     {
          clientCount--;
     }

     protected int getClientCount()
     {
          return clientCount;
     }

     protected void setPuddleEnvelope(PuddleEnvelope puddleEnvelope)
     {
          this.puddleEnvelope = puddleEnvelope;
     }

     protected PuddleEnvelope getPuddleEnvelope()
     {
          return puddleEnvelope;
     }

     protected void setPuddleIndex(int puddleIndex)
     {
          this.puddleIndex = puddleIndex;
     }

     protected int getPuddleIndex()
     {
          return puddleIndex;
     }

     protected int getConnectionNumber()
     {
          return cxNumber;
     }

     protected int getNextClientNumber()
     {
          return clientNumber++;
     }

     protected void setClusterIndex(int clusterIndex)
     {
          this.clusterIndex = clusterIndex;
     }

     protected int getClusterIndex()
     {
          return clusterIndex;
     }

     protected void setBogus(boolean bogus,Throwable cause)
     {
          this.bogus = bogus;
          this.cause = cause;
     }

     protected boolean isBogus()
     {
          return bogus;
     }

     protected Throwable getCause()
     {
          return cause;
     }

     protected void incrReactivationCount()
     {
          reactivationCount++;
     }

     protected int getReactivationCount()
     {
          return reactivationCount;
     }

     protected void setShutdownTime(long shutdownTime)
     {
          this.shutdownTime = shutdownTime;
     }

     protected long getShutdownTime()
     {
          return shutdownTime;
     }
}