package com.sas.services.connection;

import java.util.logging.Logger;
//
// History:
//
// Date    Name            Remarks
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 13May02 B Beers         S0149086: Reinstate advanced connection management.
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
// 17Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring (phase 2).
//
/**
 * The administrator's interface to a connection factory. This interface
 * allows an adminstrator to manipulate a factory in ways that may
 * affect all factory users. These manipulations include shutting the factory
 * down and destroying the factory.
 * @author Biff Beers
 */
public interface ConnectionFactoryAdminInterface
{
     /**
      * Get the logger used by this factory
      * @return the logger used by this factory
      */
     Logger getLogger();

     /**
      * Invalidate the factory, destroy all idle connections, and destroy
      * other connections as they are returned to the factory.
      */
     void shutdown();

     /**
      * Invalidate the factory and destroy all connections. Even connections
      * that are allocated to users at the time of the call are destroyed.
      */
     void destroy();
}