package com.sas.services.connection;

import java.util.logging.Logger;
//
// History:
//
// Date    Name            Remarks
// 26Nov01 B Beers         S0133039: Created.
// 03Dec01 B Beers         S0133638: Add JDNI configuration.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 03May02 B Beers         S0149086: Removed obsolete methods.
// 07May02 B Beers         S0150375: Added Info Service configuration.
// 13May02 B Beers         S0149086: Reinstate advanced connection management.
// 29May02 B Beers         S0139445: Logging service integration (happy 9th anniversary Biff & Julie).
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
// 17Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring (phase 2).
// 21Feb03 B Beers         S0185859: Change handling of factory name to prevent factory proliferation.
// 28Feb03 B Beers         S0185859: Change handling of factory name to prevent factory proliferation (phase 2).
// 14Mar03 B Beers         S0183680: Shut down factory when connection to repository is lost.
// 15May03 B Beers         S0200797: Add API to control application monitoring.
//
/**
 * The base class for all connection factory configurations. A connection factory
 * configuration can be presented to the 
 * {@link com.sas.services.connection.ConnectionFactoryManager connection factory manager}
 * to obtain a {@link com.sas.services.connection.ConnectionFactoryInterface connection factory}
 * with the desire configuration.
 * @author Biff Beers
 */
public abstract class ConnectionFactoryConfiguration implements Cloneable
{
     public boolean equals(Object that)
     {
          if (this == that)
          {
               return true;
          }

          if (that == null || !getClass().getName().equals(that.getClass().getName()))
          {
               return false;
          }

          return true;
     }

     public int hashCode()
     {
          int hc = getClass().getName().hashCode();
          return hc;
     }

     protected static ConnectionFactoryKernel getKernelFromDelegate(ConnectionFactoryManager manager,ConnectionFactoryConfiguration original,Logger logger,ConnectionFactoryConfiguration delegate) throws ConnectionFactoryException
     {
          return delegate.getKernel(manager,original,logger);
     }

     protected ConnectionFactoryKernel getKernel(ConnectionFactoryManager manager,Logger logger) throws ConnectionFactoryException
     {
          return getKernel(manager,this,logger);
     }

     protected abstract ConnectionFactoryKernel getKernel(ConnectionFactoryManager manager,ConnectionFactoryConfiguration cxfConfig,Logger logger) throws ConnectionFactoryException;

     protected static void setMonitor(ConnectionFactoryKernel kernel,Logger monitor)
     {
          kernel.setMonitor(monitor);
     }

     /**
      * Provides the configuration with a chance to clean up any resources
      * it may be holding for a kernel.
      * @param kernel the kernel that has been destroyed
      */
     protected void kernelDestroyed(ConnectionFactoryKernel kernel)
     {
     }

     private String loggerName = null;
}