package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
//
/**
 * A class used to notify factory users of important events such as factory shutdown.
 * @author Biff Beers
 */
public class ConnectionFactoryEvent
{
     protected ConnectionFactoryEvent(int type)
     {
          this(type,null);
     }

     protected ConnectionFactoryEvent(int type,Object info)
     {
          this.type = type;
          this.info = info;
     }

     /**
      * Get the event type.
      * @return the event type
      * @see #TYPE_SHUTDOWN
      * @see #TYPE_DESTROYED
      * @see #TYPE_FATAL_EXCEPTION
      */
     public int getType()
     {
          return type;
     }

     /**
      * Get the event parameter (if any).
      * Some event types encapsulate additional information. The following
      * list contains details.
      * <P>
      * <DL>
      * <DT><STRONG><CODE>TYPE_SHUTDOWN</CODE></STRONG><DD><CODE>null</CODE>
      * <DT><STRONG><CODE>TYPE_DESTROYED</CODE></STRONG><DD><CODE>null</CODE>
      * <DT><STRONG><CODE>TYPE_FATAL_EXCEPTION</CODE></STRONG>
      *  <DD><CODE>{@link com.sas.services.connection.FatalConnectionFactoryException}</CODE>
      * </DL>
      * @return the event parameter or <CODE>null</CODE>
      */
     public Object getInfo()
     {
          return info;
     }

     /**
      * Event type indicating factory shutdown.
      * @see #getType()
      */
     public static final int TYPE_SHUTDOWN = 0;

     /**
      * Event type indicating factory destruction.
      * @see #getType()
      */
     public static final int TYPE_DESTROYED = 1;

     /**
      * Event type indicating a fatal error which
      * renders the factory unusable.
      * @see #getType()
      */
     public static final int TYPE_FATAL_EXCEPTION = 2;
     private int type = 0;
     private Object info = null;
}