/* $Id$ */
package com.sas.services.connection;

import com.sas.util.ChainedException;
import com.sas.util.ChainedExceptionInterface;
//
// History:
//
// Date    Name            Remarks
// 26Nov01 B Beers         S0133039: Created.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 26Nov02 B Beers         S0151508: Revise xml error message handling.
// 02Dec02 B Beers         S0151508: Minimize impact of removing ChainedException.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * The base class for all connection service exceptions. This
 * exception may wrap lower level exceptions.
 * @author Biff Beers
 */
public class ConnectionFactoryException extends Exception implements ChainedExceptionInterface
{
     /**
      * No-arg constructor.
      */
     public ConnectionFactoryException()
     {
          super();
     }

     /**
      * Construct a connection factory exception with a message.
      * @param msg a message providing further details about the exception
      */
     public ConnectionFactoryException(String msg)
     {
          super(msg);
     }

     /**
      * Construct a connection factory exception with a nested exception.
      * @param nestedException the exception that caused this exception
      */
     public ConnectionFactoryException(Throwable nestedException)
     {
          super(nestedException);
     }

     /**
      * Construct a connection factory exception with a message and a nested exception.
      * @param msg a message providing further details about the exception
      * @param nestedException the exception that caused this exception
      */
     public ConnectionFactoryException(String msg,Throwable nestedException)
     {
          super(msg,nestedException);
     }

     public Throwable getTargetException()
     {
          return getCause();
     }

     /**
      * @see com.sas.util.ChainedException#getRootException()
      */
     public Throwable getRootException()
     {
          return ChainedException.getRootException(this);
     }
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = -404996839163508393L;
}