/* $Id$ */
package com.sas.services.connection;

import java.util.List;
import java.util.logging.Logger;
//
// History:
//
// Date    Name            Remarks
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 15Jul02 B Beers         S0158652: Fail gracefully for null user context.
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
// 17Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring (phase 2).
// 29Jan04 B Beers         S0233579: Use logger associated with user that created factory.
/**
 * Base class for all connection factories.
 * @author Biff Beers
 */
public abstract class ConnectionFactoryShell implements ConnectionFactoryShellInterface
{
     public void addConnectionFactoryEventListener(ConnectionFactoryEventListener listener)
     {
          getKernel().addConnectionFactoryEventListener(listener);
     }

     public void removeConnectionFactoryEventListener(ConnectionFactoryEventListener listener)
     {
          getKernel().removeConnectionFactoryEventListener(listener);
     }

     public List getDomains()
     {
          return getKernel().getDomains();
     }

     public ConnectionInterface getConnection(Credential userLogin) throws ConnectionFactoryException
     {
          return getConnection(userLogin,0L,null);
     }

     public ConnectionInterface getConnection(Credential userLogin,long waitMilliseconds) throws ConnectionFactoryException
     {
          return getConnection(userLogin,waitMilliseconds,null);
     }

     public ConnectionFactoryAdminInterface getAdminInterface(Credential adminLogin) throws ConnectionFactoryException
     {
          return getKernel().getAdminInterface(adminLogin);
     }

     private ConnectionFactoryKernel kernel = null;

     protected ConnectionInterface getConnection(Credential userLogin,long waitMilliseconds,Logger cxLogger) throws ConnectionFactoryException
     {
          ConnectionInterface cx = kernel.getConnection(userLogin,waitMilliseconds,cxLogger);
          return cx;
     }
     
     protected void setKernel(ConnectionFactoryKernel kernel)
     {
          this.kernel = kernel;
     }

     protected ConnectionFactoryKernel getKernel()
     {
          return kernel;
     }
}