package com.sas.services.connection;

import java.util.List;
//
// History:
//
// Date    Name            Remarks
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
//
/**
 * Methods common to all connection factory interfaces.
 * @author Biff Beers
 */
public interface ConnectionFactoryShellInterface
{
     /**
      * Add a listener for connection factory events.
      * @param listener a listener for connection factory events
      */
     void addConnectionFactoryEventListener(ConnectionFactoryEventListener listener);

     /**
      * Remove a listener for connection factory events.
      * @param listener a listener for connection factory events
      */
     void removeConnectionFactoryEventListener(ConnectionFactoryEventListener listener);

     /**
      * Get the list of authentication domains supported by this factory. The domains are listed in order
      * of increasing estimated time to connect.
      * @return the list of authentication domains supported by this factory
      */
     List getDomains();

     /**
      * Get a connection to a server. If the factory has allocated all the connections it is allowed
      * to manage to users
      * when this call is made, then this call will block until another user returns a connection
      * to the factory's pool.
      * @param login credentials for someone allowed to use this factory
      * @return a connection
      * @throws com.sas.services.connection.ConnectionFactoryException if the factory cannot create a
      * connection to satisfy this request or if the request times out or if the factory has been shutdown
      */
     ConnectionInterface getConnection(Credential login) throws ConnectionFactoryException;

     /**
      * Get a connection to a server. If the factory has allocated all the connections it is
      * allowed to manage to users
      * when this call is made, then this call will behave as instructed by the value of
      * <PARAM>waitMilliseconds</PARAM>.
      * @param login credentials for someone allowed to use this factory
      * @param waitMilliseconds a flag indicating how the call should behave if the factory has allocated
      * all the connections it is allowed to manage to other users.
      * <P>
      * <DL>
      * <DT><CODE>&lt;0</CODE><DD>throw an exception immediately
      * <DT><CODE>0</CODE><DD>wait until another user returns a connection
      * to the factory's pool
      * <DT><CODE>&gt;0</CODE><DD>wait up to <PARAM>waitMilliseconds</PARAM> milliseconds for another user
      * to return a connection to the factory's pool and throw an exception if a connection is not returned
      * in the alloted time.
      * @return a connection
      * @throws com.sas.services.connection.ConnectionFactoryException if the factory cannot create a
      * connection to satisfy this request or if the request times out or if the factory has been shutdown
      */
     ConnectionInterface getConnection(Credential login,long waitMilliseconds) throws ConnectionFactoryException;

     /**
      * Get the factory administrator's interface. The administrator's interface allows you to manipulate factory logging
      * and to shutdown or destroy the factory. Actions performed on the administrator's interface may impact other
      * users of this factory.
      * @param login credentials for someone allowed to administer this factory or <CODE>null</CODE>
      * @return the factory administrator's interface
      */
     ConnectionFactoryAdminInterface getAdminInterface(Credential adminLogin) throws ConnectionFactoryException;
}