package com.sas.services.connection;

import java.util.ArrayList;
import java.util.List;
//
// History:
//
// Date    Name            Remarks
// 14Feb02 B Beers         S0139873: Added User Service authorization.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
// 14Jan03 B Beers         S0177764: Handle null PrintWriter.
// 17Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring (phase 2).
//
/**
 * Internal utilities used by the connection service.
 * @author Biff Beers
 */
public class ConnectionFactoryUtil
{
     private ConnectionFactoryUtil()
     {
     }

     protected static Server[] copyServerArray(Server[] in)
     {
          if (in == null)
          {
               return null;
          }

          final int len = in.length;
          Server[] out = (Server[])in.clone();

          for (int i=0;i<len;i++)
          {
               out[i] = (Server)in[i].clone();
          }

          return out;
     }

     protected static Cluster[] copyClusterArray(Cluster[] in)
     {
          if (in == null)
          {
               return null;
          }

          final int len = in.length;
          Cluster[] out = (Cluster[])in.clone();

          for (int i=0;i<len;i++)
          {
               out[i] = (Cluster)in[i].clone();
          }

          return out;
     }

     protected static Puddle[] copyPuddleArray(Puddle[] in)
     {
          if (in == null)
          {
               return null;
          }

          final int len = in.length;
          Puddle[] out = (Puddle[])in.clone();

          for (int i=0;i<len;i++)
          {
               out[i] = (Puddle)in[i].clone();
          }

          return out;
     }

     protected static List copyList(List in)
     {
          if (in == null)
          {
               return null;
          }

          List out = new ArrayList(in.size());
          out.addAll(in);
          return out;
     }
}