package com.sas.services.connection;

import org.omg.CORBA.StringHolder;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.portable.Delegate;
import org.omg.CORBA.portable.ObjectImpl;
import com.sas.net.brg.DelegateImpl;
import com.sas.net.brg.orb.ProtocolEngineInterface;
//
// History:
//
// Date    Name            Remarks
// 26Nov01 B Beers         S0133039: Created.
// 14Feb02 B Beers         S0139873: Added User Service authorization.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 30Apr02 B Beers         S0149671: Resolve some garbage collection problems.
// 19Nov02 B Beers         S0169574: Revise password generation support.
//
/**
 * Simple implementation of the connection handle.
 * @author Biff Beers
 */
public class ConnectionImpl implements ConnectionInterface
{
     protected ConnectionImpl(ConnectionEnvelope cxEnv,ConnectionFactoryKernel kernel,int clientNumber)
     {
          this.cxEnv = cxEnv;
          this.kernel = kernel;
          this.clientNumber = clientNumber;
     }

     public synchronized org.omg.CORBA.Object getObject()
     {
          checkHandle();
          return cxEnv.getUserObject();
     }

     public synchronized void generatePassword(String userName,StringHolder qualUserNameHolder,StringHolder genPasswordHolder) throws ConnectionFactoryException
     {
          checkHandle();
          ObjectImpl objImpl = (ObjectImpl)cxEnv.getUserObject();
          Delegate del = objImpl._get_delegate();

          if (del == null || !(del instanceof DelegateImpl))
          {
               throw new ConnectionFactoryException("!!!");
          }

          DelegateImpl delImpl = (DelegateImpl)del;
          ProtocolEngineInterface engine = delImpl.getProtocolEngine();

          try
          {
               engine.generatePassword(userName,qualUserNameHolder,genPasswordHolder);
          }
          catch (SystemException se)
          {
               throw new ConnectionFactoryException(se);
          }
     }

     public synchronized void close()
     {
          if (cxEnv == null)
          {
               return;
          }

          kernel.reactivateConnection(cxEnv,clientNumber,ConnectionFactoryKernel.REACT_CX_CLOSED);
          cxEnv = null;
     }

     private ConnectionEnvelope cxEnv /* = null */;
     private ConnectionFactoryKernel kernel /* = null */;
     private int clientNumber /* = 0 */;

     protected void finalize() throws Throwable
     {
          if (cxEnv == null)
          {
               return;
          }

          kernel.reactivateConnection(cxEnv,clientNumber,ConnectionFactoryKernel.REACT_CX_GC);
          cxEnv = null;
     }

     private void checkHandle()
     {
          if (cxEnv == null)
          {
               throw new IllegalStateException("This connection handle has been closed.");
          }
     }
}