package com.sas.services.connection;

import org.omg.CORBA.StringHolder;
//
// History:
//
// Date    Name            Remarks
// 26Nov01 B Beers         S0133039: Created.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 19Nov02 B Beers         S0169574: Revise password generation support.
//
/**
 * The factory-managed connection handle.
 * @author Biff Beers
 */
public interface ConnectionInterface
{
     /**
      * Get the remote object reference to the toplevel object on the IOM server.
      * This connection to the server has been allocated to the caller by the
      * connection factory, and every call to this method returns
      * the same object reference until {@link #close() close()} is
      * called. After the call to {@link #close() close()}, any call
      * to this method will throw an <CODE>IllegalStateException</CODE>.
      * @return the remote object reference to the toplevel object on the IOM server
      */
     public org.omg.CORBA.Object getObject();

     /**
      * Generate a password for a user name. The server will generate a password for
      * the user name in a generated password domain. The output of this method
      * is the generated password and the user name qualified with the generated
      * password domain. The qualified user name and generated password can then
      * be used to connect to the server that generated the password.
      * @param userName the user name for which to generate a password
      * @param qualUserNameHolder (OUTPUT) a wrapper which will, after the method
      * call, contain the user name qualified with the generated password domain
      * @param genPasswordHolder (OUTPUT) a wrapper which will, after the method
      * call, contain the generated password for the user name.
      * @throws com.sas.services.connection.ConnectionFactoryException if the server
      * does not support password generation or if the current client does not
      * have permission to generate passwords
      */
     public void generatePassword(String userName,StringHolder qualUserNameHolder,StringHolder genPasswordHolder) throws ConnectionFactoryException;

     /**
      * Return the connection to the allocating factory's connection pool.
      * Multiple calls to this method have no effect.
      */
     public void close();
}