/* $Id: ConnectionTimeoutException.java,v 1.2.18.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 01May03 B Beers         S0198710: Handle LB timeout.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * The connection factory throws a timeout exception when a connection request
 * could not be completed before an administrative time limit expired. Load
 * balancing spawners cause timeout exceptions if their load does not fall
 * below a certain level within a certain amount of time.
 * <P>
 * After a timeout exception has been thrown, a future connection
 * request might succeed. In
 * other words, a timeout exception does not imply a server failure.
 * @author Biff Beers
 */
public class ConnectionTimeoutException extends ConnectionFactoryException
{
     /**
      * No-arg constructor.
      */
     public ConnectionTimeoutException()
     {
          super();
     }

     /**
      * Construct a connection factory exception with a message.
      * @param msg a message providing further details about the exception
      */
     public ConnectionTimeoutException(String msg)
     {
          super(msg);
     }

     /**
      * Construct a connection factory exception with a nested exception.
      * @param nestedException the exception that caused this exception
      */
     public ConnectionTimeoutException(Throwable nestedException)
     {
          super(nestedException);
     }

     /**
      * Construct a connection factory exception with a message and a nested exception.
      * @param msg a message providing further details about the exception
      * @param nestedException the exception that caused this exception
      */
     public ConnectionTimeoutException(String msg,Throwable nestedException)
     {
          super(msg,nestedException);
     }
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = -473815733449247235L;
}