/* $Id: Credential.java,v 1.2.8.1.14.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;

import java.io.Serializable;
//
// History:
//
// Date    Name            Remarks
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 15Oct02 B Beers         S0168666: Provide implementation for hashCode.
// 18Feb04 B Beers         S0238566: Clear reference to ConnectionImpl when returning connection.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * A base class for credentials that can be used to authenticate
 * and authorize use of a connection factory.
 * @author Biff Beers
 */
public abstract class Credential implements Cloneable,Serializable
{
     /**
      * Construct a credential object.
      * @param domain the authentication domain
      * @see #setDomain
      */
     protected Credential(String domain)
     {
          this();
          setDomain(domain);
     }

     /**
      * Construct a credential object.
      */
     protected Credential()
     {
     }

     /**
      * Set the authentication domain. The domain is a label that the factory
      * uses to group servers and credentials. When connecting through a factory, the
      * factory user must present a credential object with a domain that matches
      * the domain of at least one server managed by the factory.
      * <P>
      * Empty string and <CODE>null</CODE> are considered to be equivalent domains.
      * The default value is empty string.
      * @param domain the authentication domain
      */
     public void setDomain(String domain)
     {
          this.domain = (domain != null) ? domain : "";
     }

     /**
      * Get the authentication domain.
      * @return the authentication domain
      * @see #setDomain
      */
     public String getDomain()
     {
          return domain;
     }

     public boolean equals(Object that)
     {
          if (this == that)
          {
               return true;
          }

          if (that == null || !getClass().getName().equals(that.getClass().getName()))
          {
               return false;
          }

          Credential thatCredential = (Credential)that;
          String thatDomain = thatCredential.getDomain();

          if (!domain.equals(thatDomain))
          {
               return false;
          }

          return true;
     }

     public int hashCode()
     {
          int hc = getClass().getName().hashCode();
          hc ^= domain.hashCode();
          return hc;
     }

     public Object clone()
     {
          try
          {
               return super.clone();
          }
          catch (CloneNotSupportedException e)
          {
               throw new InternalError("A Cloneable object threw CloneNotSupportedException.");
          }
     }
     
     public String toString()
     {
          return "domain=\"" + domain + "\"";/*I18nOK:LINE*/
     }

     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = -4997394505379717834L;
     private String domain = "";
}
