/* $Id: FailoverCluster.java,v 1.1.22.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * A failover cluster of IOM servers. A failover
 * cluster is a set of redundant servers that is
 * configured such that, if one server fails,
 * another can take it's place. For the connection
 * factory, the failover feature is only employed at
 * the time a connection is made. If a connection
 * attempt to one server fails, another server in
 * the cluster is tried. If an existing connection
 * fails unexpectedly, there is no attempt to move
 * the connection to another server.
 * @author Biff Beers
 */
public final class FailoverCluster extends Cluster
{
     /**
      * Construct a failover cluster with one server.
      * This constructor is a shortcut for the following more
      * complicated code:
      * <PRE>
      * FailoverCluster(new Server[]{server})
      * </PRE>
      * @param server the server to add to the cluster
      */
     public FailoverCluster(Server server)
     {
          this(new Server[] {server});
     }

     /**
      * Construct a failover cluster of IOM servers.
      * @param servers the servers to add to the cluster.
      */
     public FailoverCluster(Server[] servers)
     {
          super(servers);
     }

     /**
      * Get the cluster type. This method always
      * returns {@link Cluster#TYPE_FAILOVER Cluster.TYPE_FAILOVER}.
      * @return the cluster type
      * @see Cluster#TYPE_FAILOVER
      */
     public int getClusterType()
     {
          return Cluster.TYPE_FAILOVER;
     }
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = 1010344660318392014L;

     protected int computeMaxClients()
     {
          Server[] servers = getServerArray();
          int min = Integer.MAX_VALUE;

          for (int i=0;i<servers.length;i++)
          {
               min = Math.min(min,servers[i].getMaxClients());
          }

          return min;
     }
}