/* $Id: InvalidCredentialsException.java,v 1.1.20.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 29Oct02 B Beers         S0156049: Added support for expired credentials.
// 23Jan02 B Beers         S0156049: Add granularity to authentication errors.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * The connection factory throws an invalid credentials exception when a
 * connection request cannot be granted due to invalid or missing credentials.
 * Examples of invalid credentials include unknown or missing user names
 * and passwords.
 * @author Biff Beers
 */
public class InvalidCredentialsException extends LoginException
{
     /**
      * No-arg constructor.
      */
     public InvalidCredentialsException()
     {
          super();
     }

     /**
      * Construct a connection factory exception with a message.
      * @param msg a message providing further details about the exception
      */
     public InvalidCredentialsException(String msg)
     {
          super(msg);
     }

     /**
      * Construct a connection factory exception with a nested exception.
      * @param nestedException the exception that caused this exception
      */
     public InvalidCredentialsException(Throwable nestedException)
     {
          super(nestedException);
     }

     /**
      * Construct a connection factory exception with a message and a nested exception.
      * @param msg a message providing further details about the exception
      * @param nestedException the exception that caused this exception
      */
     public InvalidCredentialsException(String msg,Throwable nestedException)
     {
          super(msg,nestedException);
     }
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = 7936073234507049029L;
}