/* $Id: LoadBalancingCluster.java,v 1.1.22.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * A load balancing cluster of IOM servers. The load balancing feature
 * is implemented with a IOM Object Spawner that monitors the
 * load of a set of redundant servers and directs requests for new
 * connections to the least loaded server.
 * @author Biff Beers
 */
public final class LoadBalancingCluster extends Cluster
{
     /**
      * Construct a load balancing cluster with one server.
      * This constructor is a shortcut for the following more
      * complicated code:
      * <PRE>
      * LoadBalancingCluster(new Server[]{server})
      * </PRE>
      * @param server the server to add to the cluster
      */
     public LoadBalancingCluster(Server server)
     {
          this(new Server[] {server});
     }

     /**
      * Construct a load balancing cluster cluster of IOM servers.
      * @param servers the servers to add to the cluster.
      */
     public LoadBalancingCluster(Server[] servers)
     {
          super(servers);
     }

     /**
      * Get the cluster type. This method always
      * returns {@link Cluster#TYPE_LOAD_BALANCING Cluster.TYPE_LOAD_BALANCING}.
      * @return the cluster type
      * @see Cluster#TYPE_LOAD_BALANCING
      */
     public int getClusterType()
     {
          return Cluster.TYPE_LOAD_BALANCING;
     }
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = -6540622077802844816L;

     protected int computeMaxClients()
     {
          Server[] servers = getServerArray();
          long maxClients = 0L;

          for (int i=0;i<servers.length;i++)
          {
               maxClients += servers[i].getMaxClients();
          }

          return (maxClients <= Integer.MAX_VALUE) ? (int)maxClients : Integer.MAX_VALUE;
     }
}