/* $Id$ */
package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 23Jan02 B Beers         S0156049: Add granularity to authentication errors.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * The connection factory throws a login exception when a connection request
 * cannot be granted because the connecting client cannot be authenticated
 * or authorized. After a login exception has been thrown, a future connection
 * request accompanied by different or updated credentials might succeed. In
 * other words, a login exception does not imply a server failure.
 * @author Biff Beers
 */
public class LoginException extends ConnectionFactoryException
{
     /**
      * No-arg constructor.
      */
     public LoginException()
     {
          super();
     }

     /**
      * Construct a connection factory exception with a message.
      * @param msg a message providing further details about the exception
      */
     public LoginException(String msg)
     {
          super(msg);
     }

     /**
      * Construct a connection factory exception with a nested exception.
      * @param nestedException the exception that caused this exception
      */
     public LoginException(Throwable nestedException)
     {
          super(nestedException);
     }

     /**
      * Construct a connection factory exception with a message and a nested exception.
      * @param msg a message providing further details about the exception
      * @param nestedException the exception that caused this exception
      */
     public LoginException(String msg,Throwable nestedException)
     {
          super(msg,nestedException);
     }
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = -2422477752945441005L;
}