/* $Id: MonitorRecord.java,v 1.2.18.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.LogRecord;
//
// History:
//
// Date    Name            Remarks
// 10Jan03 B Beers         S0177764: Use java.util.logging package and application monitoring.
// 15May03 B Beers         S0200797: Add API to control application monitoring.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * The log record for messages sent to an application monitor.
 * This class is used internally by the <CODE>ConnectionFactoryManager</CODE>.
 * @author Biff Beers
 */
public class MonitorRecord extends LogRecord
{
     public MonitorRecord()
     {
          super(Level.FINEST,null);
          setType(TYPE_SERVER_STATUS);
          setPuddleIndex(-1);
          setIdle(0);
     }

     public void setType(String type)
     {
          contents.put(KEY_TYPE,type);
     }

     public void setFactoryName(String factoryName)
     {
          contents.put(KEY_FACTORY_NAME,factoryName);
     }

     public void setPuddleIndex(int puddleIndex)
     {
          contents.put(KEY_PUDDLE_INDEX,Integer.toString(puddleIndex));
     }

     public void setClusterIndex(int clusterIndex)
     {
          contents.put(KEY_CLUSTER_INDEX,Integer.toString(clusterIndex));
     }

     public void setServerIndex(int serverIndex)
     {
          contents.put(KEY_SERVER_INDEX,Integer.toString(serverIndex));
     }

     public void setServerName(String serverName)
     {
          contents.put(KEY_SERVER_NAME,serverName);
     }

     public void setConnections(int connections)
     {
          contents.put(KEY_CONNECTIONS,Integer.toString(connections));
     }

     public void setIdle(int idle)
     {
          contents.put(KEY_IDLE,Integer.toString(idle));
     }

     public void setActive(int active)
     {
          contents.put(KEY_ACTIVE,Integer.toString(active));
     }

     public void setCause(String cause)
     {
          contents.put(KEY_CAUSE,cause);
     }

     public Map getContents()
     {
          return contents;
     }

     public static final String KEY_TYPE = "TYPE";
     public static final String KEY_FACTORY_NAME = "FACTORY_NAME";
     public static final String KEY_PUDDLE_INDEX = "PUDDLE_INDEX";
     public static final String KEY_CLUSTER_INDEX = "CLUSTER_INDEX";
     public static final String KEY_SERVER_INDEX = "SERVER_INDEX";
     public static final String KEY_SERVER_NAME = "SERVER_NAME";
     public static final String KEY_PORT = "PORT";
     public static final String KEY_CONNECTIONS = "CONNECTIONS";
     public static final String KEY_IDLE = "IDLE";
     public static final String KEY_ACTIVE = "ACTIVE";
     public static final String KEY_CAUSE = "CAUSE";
     public static final String TYPE_SERVER_STATUS = "server_status";
     public static final String CAUSE_REQUEST = "request";
     public static final String CAUSE_RETURN = "return";
     public static final String CAUSE_GC = "garbage_collection";
     public static final String CAUSE_TIMEOUT = "timeout";
     public static final String CAUSE_MIN_AVAIL = "min_avail";
     public static final String CAUSE_MIN_SIZE = "min_size";
     public static final String CAUSE_SHUTDOWN = "shutdown";
     public static final String CAUSE_DESTROY = "destroy";
     public static final String CAUSE_REFRESH = "refresh";
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = 3215877166949527102L;
     private Map contents = new HashMap();
}