/* $Id: PasswordCredential.java,v 1.1.8.1.14.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 26Nov01 B Beers         S0133039: Created.
// 14Feb02 B Beers         S0139873: Added User Service authorization.
// 26Apr02 B Beers         S0149086: Revise the connection factory interface.
// 29May02 B Beers         S0139445: Logging service integration (happy 9th anniversary Biff & Julie).
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 18Feb04 B Beers         S0238566: Clear reference to ConnectionImpl when returning connection.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * Login credentials for an IOM server.
 * @author Biff Beers
 */
public class PasswordCredential extends Credential
{
     /**
      * Construct login credentials.
      * @param userName the user name
      * @param password the password
      * @param domain the authentication domain. The authentication domain
      * indicates the context in which this user name and password are valid.
      * Servers that require user name and password also have domains, and
      * this domain must match the server's domain if this login is valid on
      * the server. <CODE>null</CODE> and empty (<CODE>""</CODE>) domains are equivalent.
      */
     public PasswordCredential(String userName,String password,String domain)
     {
          super(domain);
          setUserName(userName);
          setPassword(password);
     }

     /**
      * Construct login credentials. The authentication domain
      * will be "" (empty string).
      * @param userName the user name
      * @param password the password
      */
     public PasswordCredential(String userName,String password)
     {
          super();
          setUserName(userName);
          setPassword(password);
     }

     /**
      * Set the user name.
      * @param userName the user name
      */
     public void setUserName(String userName)
     {
          this.userName = (userName != null) ? userName : "";
     }

     /**
      * Get the user name.
      * @return the user name
      */
     public String getUserName()
     {
          return userName;
     }

     /**
      * Set the password.
      * @param password the password
      */
     public void setPassword(String password)
     {
          this.password = (password != null) ? password : "";
     }

     /**
      * Get the password.
      * @return the password
      */
     public String getPassword()
     {
          return password;
     }

     public boolean equals(Object that)
     {
          if (!super.equals(that))
          {
               return false;
          }

          PasswordCredential thatPasswordCredential = (PasswordCredential)that;
          String thatUserName = thatPasswordCredential.getUserName();

          if (!userName.equals(thatUserName))
          {
               return false;
          }

          String thatPassword = thatPasswordCredential.getPassword();

          if (!password.equals(thatPassword))
          {
               return false;
          }

          return true;
     }

     public int hashCode()
     {
          int hc = super.hashCode();
          hc ^= userName.hashCode();
          hc ^= password.hashCode();
          return hc;
     }

     public String toString()
     {
          return "user name=\"" + userName + "\" " + super.toString();/*I18nOK:LINE*/
     }
     
     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = -7129081732710136136L;
     private String userName = "";
     private String password = "";
}