/* $Id: TCPIPServer.java,v 1.1.22.1 2005/10/11 17:14:12 sasbfb Exp $ */
package com.sas.services.connection;
//
// History:
//
// Date    Name            Remarks
// 11Oct02 B Beers         S0168379: Generalize connection service for other servers.
// 11Oct05 B Beers         S0324947: Added serialVersionUID.
/**
 * A base class to describe IOM servers that communicate with clients
 * using a TCP/IP application protocol.
 * @author Biff Beers
 */
public abstract class TCPIPServer extends Server
{
     public TCPIPServer(String classID,String host,int port)
     {
          super(classID);
          setHost(host);
          setPort(port);
     }

     /**
      * Set the host.
      * @param host the host
      */
     public void setHost(String host)
     {
          if (host == null)
          {
               throw new IllegalArgumentException("\"host\" cannot be null.");
          }

          origHost = host;
          normHost = host.toLowerCase();
     }

     /**
      * Get the host.
      * @return the host
      */
     public String getHost()
     {
          return origHost;
     }

     /**
      * Get the normalized host.
      * The normalized host is all lowercase.
      * @return the normalized host
      */
     public String getNormalizedHost()
     {
          return normHost;
     }

     /**
      * Set the port.
      * @param port the port
      */
     public void setPort(int port)
     {
          if (port <= 0)
          {
               throw new IllegalArgumentException("\"port\" cannot be less than 1.");
          }

          this.port = port;
     }

     /**
      * Get the port.
      * @return the port
      */
     public int getPort()
     {
          return port;
     }

     public boolean equals(Object that)
     {
          if (!super.equals(that))
          {
               return false;
          }

          TCPIPServer thatServer = (TCPIPServer)that;
          String thatNormHost = thatServer.getNormalizedHost();

          if (!normHost.equals(thatNormHost))
          {
               return false;
          }

          int thatPort = thatServer.getPort();

          if (port != thatPort)
          {
               return false;
          }

          return true;
     }

     public int hashCode()
     {
          int hc = super.hashCode();
          hc ^= normHost.hashCode();
          hc ^= port;
          return hc;
     }

     /**
      * Serial version ID. 
      * This value should be changed if this version is not compatible with the previous version.
      */
     private static final long serialVersionUID = 4763548047827393826L;
     private String origHost = null;
     private String normHost = null;
     private int port = 0;
}