// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.services.webdav;

import java.rmi.RemoteException;
import java.util.List;
import java.util.Map;

import com.sas.services.logging.LoggerInterface;

/**
 * This interface represents the operations expected on a DAV collection. Collections can
 * contain other collections and/or resources (known as items).
 * <p>
 * @author  Rob Owen
 * @version 1.0 15-Nov-2001
 */
public interface CollectionInterface extends ResourceInterface
{
    /**
     * Create a new collection in the existing collection
     */
    public CollectionInterface createCollection(String relpath) throws DAVException, RemoteException;
    public CollectionInterface createCollection(String relpath, LoggerInterface logger) throws DAVException, RemoteException;

    /**
     * Create a new collection in the existing collection
     */
    public CollectionInterface createCollection(String relpath, String userName, String pwd) throws DAVException, RemoteException;
    public CollectionInterface createCollection(String relpath, String userName, String pwd, LoggerInterface logger) throws DAVException, RemoteException;

    /**
     * Create a new resource in the existing collection
     */
    public ResourceInterface createResource(String relpath) throws DAVException, RemoteException;
    public ResourceInterface createResource(String relpath, LoggerInterface logger) throws DAVException, RemoteException;

    /**
     * Create a new resource in the existing collection
     */
    public ResourceInterface createResource(String relpath, String userName, String pwd) throws DAVException, RemoteException;
    public ResourceInterface createResource(String relpath, String userName, String pwd, LoggerInterface logger) throws DAVException, RemoteException;


    /**
     * Get a list of the immediate items (depth=1) in the collection
     */
    public List getChildren() throws DAVException, RemoteException;

    /**
     * Get all the properties of a collection and its members in as few server requests as possible.
     */
    public Map getChildrenProperties() throws DAVException, RemoteException;

}