/* $Id$ */
// Copyright (c) 2002 SAS Institute Inc., Cary, NC 27513
package com.sas.services.webdav;

import com.sas.services.util.Constants;
/**
 * A convenience class to collect all the constants used in the DAV environment.
 * <p>
 *
 * @author Rob Owen
 * @version 1.0 27-Jan-2002
 */

public class DAVConstants {

    private DAVConstants() {
        // Prevent instantiation
    }

    /**
     * SAS DAVClient identifier used in the User-Agent HTTP header field
     */
    public static final String SASCLIENT_STRING        = "SAS Institute DAVClient/1.0";  /*I18nOK:EMS*/

    /**
     * Offset used for user area
     */
    public static final String USER_PROFILES_PATH      = "Users/";  /*I18nOK:EMS*/
    /**
     * Name of schema folder
     */
    public static final String SCHEMA_PATH             = "$$Schema$$";  /*I18nOK:EMS*/
    /**
     * SAS resourcetype property name (cf. DAV resourcetype property name)
     */
    public static final String SASDAV_RESOURCETYPE     = "{"+Constants.NS_PUBLISH_WEBDAV_URI+"}type";  /*I18nOK:LINE*/
    /**
     * SAS resourcetype property name (cf. DAV resourcetype property name)
     */
    public static final String SASDAV_DESCRIPTION     = "{"+Constants.NS_PUBLISH_WEBDAV_URI+"}description";  /*I18nOK:LINE*/
    /**
     * SAS keyword property name (in the sasdav namespace)
     */
    public static final String SASDAV_SASKEYWORD = "{"+Constants.NS_PUBLISH_WEBDAV_URI+"}saskeyword";  /*I18nOK:LINE*/

    /**
     * SAS resourcetype property name as XML 1.0
     */
    public static final String SASDAV_XML_RESOURCETYPE = "<sas:type xmlns:sas=\""+Constants.NS_PUBLISH_WEBDAV_URI+"\"/>"; /*I18nOK:LINE*/
    /**
     * SAS package property name. Is equal to "package" for a package
     */
    public static final String SASDAV_PACKAGETYPE      = "{"+Constants.NS_PUBLISH_WEBDAV_URI+"}type";  /*I18nOK:LINE*/
    /**
     * SAS Folder manager class name property
     */
    public static final String FOLDER_MANAGER_PROPERTY = "{"+Constants.NS_PUBLISH_WEBDAV_URI+"}FolderManagerClass"; /*I18nOK:LINE*/
    /**
     * SAS property list property name
     */
    public static final String SCHEMA_PROPLIST_PROP    = "{"+Constants.NS_WEBDAV_URI+"}proplist"; /*I18nOK:LINE*/
    /**
     * Namespace to be used with properties defined in a profile
     */
    public static final String PROFILE_NAMESPACE       = "{}";  /*I18nOK:EMS*/

    /**
     * Value returned by Slide for {DAV:}resourcetype when resource is a collection
     */
    public static final String DAV_COLLECTION          = "COLLECTION";  /*I18nOK:EMS*/

    // DAV properties (RFC2518)
    /**
     * DAV resourcetype property name in {namespace}propertyname format
     */
    public static final String DAV_RESOURCETYPE        = "{DAV:}resourcetype";  /*I18nOK:EMS*/
    /**
     * DAV creationdate property name in {namespace}propertyname format
     */
    public static final String DAV_CREATIONDATE        = "{DAV:}creationdate";  /*I18nOK:EMS*/
    /**
     * DAV displayname property name in {namespace}propertyname format
     */
    public static final String DAV_DISPLAYNAME         = "{DAV:}displayname";  /*I18nOK:EMS*/
    /**
     * DAV getcontentlanguage property name in {namespace}propertyname format
     */
    public static final String DAV_GETCONTENTLANGUAGE  = "{DAV:}getcontentlanguage";  /*I18nOK:EMS*/
    /**
     * DAV getcontentlength property name in {namespace}propertyname format
     */
    public static final String DAV_GETCONTENTLENGTH    = "{DAV:}getcontentlength";  /*I18nOK:EMS*/
    /**
     * DAV getcontenttype property name in {namespace}propertyname format
     */
    public static final String DAV_GETCONTENTTYPE      = "{DAV:}getcontenttype";  /*I18nOK:EMS*/
    /**
     * DAV getetag property name in {namespace}propertyname format
     */
    public static final String DAV_GETETAG             = "{DAV:}getetag";  /*I18nOK:EMS*/
    /**
     * DAV lastmodified property name in {namespace}propertyname format
     */
    public static final String DAV_GETLASTMODIFIED     = "{DAV:}getlastmodified";  /*I18nOK:EMS*/
    /**
     * DAV lockdiscovery property name in {namespace}propertyname format
     */
    public static final String DAV_LOCKDISCOVERY       = "{DAV:}lockdiscovery";  /*I18nOK:EMS*/
    /**
     * DAV supportedlock property name in {namespace}propertyname format
     */
    public static final String DAV_SUPPORTEDLOCK       = "{DAV:}supportedlock";  /*I18nOK:EMS*/
    
	public static final String DAV_ISCOLLECTION        = "{DAV:}iscollection";  /*I18nOK:EMS*/
	public static final String DAV_IS_COLLECTION       = "{DAV:}is-collection";  /*I18nOK:EMS*/
	public static final String DAV_ACL                 = "{DAV:}acl";   /*I18nOK:EMS*/
	public static final String DAV_                    = "{DAV:}";      /*I18nOK:EMS*/
	public static final String DAV_ALL_PRIVILEGES      = "{DAV:}all";   /*I18nOK:EMS*/
	public static final String DAV_READ_PRIVILEGE      = "{DAV:}read";  /*I18nOK:EMS*/
    public static final String DAV_WRITE_PRIVILEGE     = "{DAV:}write";  /*I18nOK:EMS*/
    public static final String DAV_OWNER               = "{DAV:}owner"; /*I18nOK:EMS*/
    public static final String DAV_XYTHOS_WRITE_PRIVILEGE          = "{http://www.xythos.com/namespaces/StorageServer/acl/}write";
    public static final String DAV_XYTHOS_DELETE_PRIVILEGE         = "{http://www.xythos.com/namespaces/StorageServer/acl/}delete";
    public static final String DAV_XYTHOS_PERMISSIONS_PRIVILEGE    = "{http://www.xythos.com/namespaces/StorageServer/acl/}permissions";
    public static final String DAV_XYTHOS_INHERIT_READ_PRIVILEGE   = "{http://www.xythos.com/namespaces/StorageServer/acl/}inherit-read";
    public static final String DAV_XYTHOS_INHERIT_WRITE_PRIVILEGE  = "{http://www.xythos.com/namespaces/StorageServer/acl/}inherit-write";
    public static final String DAV_XYTHOS_INHERIT_DELETE_PRIVILEGE = "{http://www.xythos.com/namespaces/StorageServer/acl/}inherit-delete";
    public static final String DAV_XYTHOS_INHERIT_PERMISSIONS_PRIVILEGE = "{http://www.xythos.com/namespaces/StorageServer/acl/}inherit-permissions";
    
    /**
     * Constants used for DAV basic search
     */
    public static final String ALLPROP       = "allprop";                  /*I18nOK:EMS*/
    public static final String AND           = "and";                      /*I18nOK:EMS*/
    public static final String ASCENDING     = "ascending";                /*I18nOK:EMS*/
    public static final String BASICSEARCH   = "basicsearch";              /*I18nOK:EMS*/
    public static final String CASESENSITIVE = "casesensitive";            /*I18nOK:EMS*/
    public static final String CONTAINS      = "contains";                 /*I18nOK:EMS*/
    public static final String DEPTH         = "depth";                    /*I18nOK:EMS*/
    public static final String DESCENDING    = "descending";               /*I18nOK:EMS*/
    public static final String EQ            = "eq";                       /*I18nOK:EMS*/
    public static final String FROM          = "from";                     /*I18nOK:EMS*/
    public static final String GT            = "gt";                       /*I18nOK:EMS*/
    public static final String GTE           = "gte";                      /*I18nOK:EMS*/
    public static final String HREF          = "href";                     /*I18nOK:EMS*/
    public static final String INFINITY      = "infinity";                 /*I18nOK:EMS*/
    public static final String ISCOLLECTION  = "iscollection";             /*I18nOK:EMS*/
	public static final String IS_COLLECTION = "is-collection";            /*I18nOK:EMS*/
    public static final String ISDEFINED     = "isdefined";                /*I18nOK:EMS*/
    public static final String ISPRINCIPAL   = "is-principal";             /*I18nOK:EMS*/
    public static final String LIKE          = "like";                     /*I18nOK:EMS*/
    public static final String LIMIT         = "limit";                    /*I18nOK:EMS*/
    public static final String LITERAL       = "literal";                  /*I18nOK:EMS*/
    public static final String LT            = "lt";                       /*I18nOK:EMS*/
    public static final String LTE           = "lte";                      /*I18nOK:EMS*/
    public static final String NOT           = "not";                      /*I18nOK:EMS*/
    public static final String OR            = "or";                       /*I18nOK:EMS*/
    public static final String ORDER         = "order";                    /*I18nOK:EMS*/
    public static final String ORDERBY       = "orderby";                  /*I18nOK:EMS*/
    public static final String PROP          = "prop";                     /*I18nOK:EMS*/
    public static final String RESOURCETYPE  = "resourcetype";             /*I18nOK:EMS*/
    public static final String RESULTSET     = "resultset";                /*I18nOK:EMS*/
    public static final String SCOPE         = "scope";                    /*I18nOK:EMS*/
    public static final String SELECT        = "select";                   /*I18nOK:EMS*/
    public static final String WHERE         = "where";                    /*I18nOK:EMS*/
    
    /**
     * Constants used for ACL
     */
    public static final String E_ACL             = "acl";                  /*I18nOK:EMS*/
    public static final String E_ACE             = "ace";                  /*I18nOK:EMS*/
    public static final String E_ALL             = "all";                  /*I18nOK:EMS*/
    public static final String E_AUTHENTICATED   = "authenticated";        /*I18nOK:EMS*/
    public static final String E_UNAUTHENTICATED = "unauthenticated";      /*I18nOK:EMS*/
    public static final String E_PROPERTY        = "property";             /*I18nOK:EMS*/
    public static final String E_GRANT           = "grant";                /*I18nOK:EMS*/
    public static final String E_DENY            = "deny";                 /*I18nOK:EMS*/
    public static final String E_PRIVILEGE       = "privilege";            /*I18nOK:EMS*/
    public static final String E_PROTECTED       = "protected";            /*I18nOK:EMS*/
    public static final String E_INHERITED       = "inherited";            /*I18nOK:EMS*/
    public static final String E_HREF            = "href";                 /*I18nOK:EMS*/
    public static final String E_INVERT          = "invert";               /*I18nOK:EMS*/
    public static final String E_PRINCIPAL       = "principal";            /*I18nOK:EMS*/
    public static final String NS_DAV_URI        = "DAV:";                 /*I18nOK:EMS*/
    
    /**
     * HTTP Header string identifying a Xythos server
     */
    public static final String XYTHOS_WFS_VERSION_HDR = "Xythos-WFS-Version"; /*I18nOK:EMS*/
    
    /**
     * The default content character encoding charset 
     */
    public static final String DEFAULT_CONTENT_CHARSET = "ISO-8859-1"; /*I18nOK:EMS*/
    
    /**
     * Debug level 
     */
    public static final int DEFAULT_DEBUG_TRACE = 10;
}