/* $Id$ */
// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.services.webdav;

/**
 * DAVException
 *
 * Wrapper for any exception that might occur in the DAV resource code. At the moment this is a chained
 * exception, but once we have the information from the originating exception, there may be little point
 * in keeping a copy.
 *
 * @author Rob Owen
 * @version 1.0 10-Sep-2001
 */
public class DAVException extends Exception
{

   /**
    * Serial version UID. This value should remain unchanged until one
    * desires to make previous serialized versions no longer compatible with
    * the current version.
    */
   private static final long serialVersionUID = 3170530015347844157L;

   /**
     * A status code associated with this exception. If the originating exception is not an instance of
     * HttpException then this will be -1, otherwise this is a valid Http return code
     */
    private int httpStatusCode = -1;

    /**
     * The underlying exception that created this exception.
     */
    private Exception _exception;

    /**
     * Default constructor
     */
    public DAVException()
    {
        super();
    }

    /**
     * Constructor for specifying an exception message
     *
     * @param msg Message for the exception
     */
    public DAVException(final String msg)
    {
        this(msg, null);
    }

    /**
     * Constructor for specifying an exception message and the underlying exception
     *
     * @param msg Message for the exception
     * @param exception The underlying exception
     */
    public DAVException(final String msg, final Exception exception)
    {
        super(msg);
        if (exception instanceof org.apache.commons.httpclient.HttpException)
           this.httpStatusCode = ((org.apache.commons.httpclient.HttpException)exception).getStatusCode();
        _exception = exception;
    }

    /**
     * Return the underlying exception. This will be null if the exception wasn't specified
     */
    public final Exception getException()
    {
        return _exception;
    }

    public int getStatusCode()
    {
        return httpStatusCode;
    }
}