/* $Id$ */
// Copyright (c) 2001-2004 SAS Institute Inc., Cary, NC 27513
// All Rights Reserved.

package com.sas.services.webdav;

import com.sas.services.logging.LoggerInterface;
import java.util.Hashtable;

/**
 * This class is an example implementation of a ConnectonInterceptor. This implementation is geared towards
 * writing out all the information that flows through the connection interface and so it can be used for
 * debugging purposes. Setting a debug value on DAVResource will cause DAVResource to create an instance
 * of this class for use by the HttpClient.
 *
 * @author  <A HREF="mailto:Rob.Owen@sas.com">Rob Owen</A>
 * @version 1.1.3 03/23/04
 * @since 1.0
 */
public class DebugConnectionInterceptor implements org.apache.commons.httpclient.ConnectionInterceptor
{
    LoggerInterface _logger = null;
    public DebugConnectionInterceptor() {
    }
    
    public DebugConnectionInterceptor( LoggerInterface _logger ) {
        this._logger = _logger;
    }
    
    /** 
     * Helper method to write log messages. If a logger exists, then use the logger,
     * otherwise just use System.out
     * @param logLine Line to be logged
     */
    private void writeLog( String logLine) {
        if ( _logger != null ) {
            if ( _logger.isDebugEnabled() ) _logger.debug( logLine, DAVResource.class.getName() );
        } else System.out.println( logLine );
    }
    
    
    /**
     * Connect
     */
    public void connect()
    {
        writeLog( "[Connect]" /*I18nOK:EMS*/ );
    }


    /**
     * Disconnect.
     */
    public void disconnect()
    {
        writeLog( "[Disconnect]" /*I18nOK:EMS*/ );
    }


    /**
     * Retry.
     *
     * @return boolean true if a retry should be attempted
     */
    public boolean retry(int status)
    {
        writeLog( "[Retry, status="+status+"]" ); /*I18nOK:LINE*/
        return false;
    }


    /**
     * Received an informational status code.
     *
     * @return boolean true if a retry should be attempted
     */
    public boolean info(int status, Hashtable headers)
    {
        writeLog( "[Info, status="+status+"]" ); /*I18nOK:LINE*/
        return false;
    }


    /**
     * Unexpected error.
     *
     * @param status Status code; can be equal to -1 if status code is not
     * known
     * @param e Underlying exception; can be null
     * @return boolean true if processing of the request should be stopped
     */
    public boolean error(int status, Exception e)
    {
        writeLog( "[Error status="+status+" exception="+e.getMessage()+"]" ); /*I18nOK:LINE*/
        return false;
    }


    /**
     * Sent request.
     */
    public void sentRequest()
    {
        writeLog( "[Sent request]" /*I18nOK:EMS*/ );
    }


    /**
     * Received response.
     */
    public void receivedResponse()
    {
        writeLog( "[Received response]" /*I18nOK:EMS*/ );
    }


    /**
     * Received expectation.
     */
    public void receivedExpectation()
    {
        writeLog( "[Received expectation]" /*I18nOK:EMS*/ );
    }


    /**
     * Required authentication.
     */
    public void requiredAuthentication()
    {
        writeLog( "[Require authentication]" /*I18nOK:EMS*/);
    }


    /**
     * Authenticate.
     */
    public void authenticate()
    {
        writeLog( "[Authenticate]" /*I18nOK:EMS*/);
    }

}