// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.services.webdav;
import com.sas.services.logging.LoggerInterface;
/**
 * This class is an example implementation of a StreamInterceptor. This implementation is geared towards
 * writing out all the information that flows through the stream interface and so it can be used for
 * debugging purposes. Setting a debug value on DAVResource will cause DAVResource to create an instance
 * of this class for use by the HttpClient.
 *
 * @author  Rob Owen
 * @version 1.0 20-Sep-2001
 *
 */
public class DebugStreamInterceptor implements org.apache.commons.httpclient.StreamInterceptor
{
    LoggerInterface _logger = null;
    public DebugStreamInterceptor() {
    }
    
    public DebugStreamInterceptor( LoggerInterface _logger ) {
        this._logger = _logger;
    }

    /** 
     * Helper method to write log messages. If a logger exists, then use the logger,
     * otherwise just use System.out
     * @param logLine Line to be logged
     */
    private void writeLog( String logLine) {
        if ( _logger != null ) {
	        if ( _logger.isDebugEnabled( DAVResource.class.getName() ) ) _logger.debug( logLine, DAVResource.class.getName() );
        } else System.out.println( logLine );
    }
    
    /**
     * Log the contents being written.
     */
    public void bytesWrite(byte[] b, int data, int count)
    {
        // writeLog( "### "+count+" bytes written" ); /*I18nOK:LINE*/
        if (count > 0)
        {
            if (b == null) writeLog( "### "+data ); /*I18nOK:LINE*/
            else {
            	String str = new String(b, data, count);
            	str = str.replace( '\r', ' ' ); /*I18nOK:LINE*/
				str = str.replace( '\n', ' ' ); /*I18nOK:LINE*/
            	if ( str.startsWith("[") && str.endsWith("]") ) writeLog( str );   /*I18nOK:LINE*/
            	else writeLog( "> "+str );   /*I18nOK:LINE*/
            }
        }
    }


    /**
     * Log the contents being read.
     */
    public void bytesRead(byte[] b, int data, int count)
    {
        // writeLog("### "+count+" bytes read"); /*I18nOK:LINE*/
        if (count > 0)
        {
            if (b == null) writeLog( "### "+data ); /*I18nOK:LINE*/
            else writeLog( "< "+new String(b, data, count) ); /*I18nOK:LINE*/
        }
    }

}