/* $Id: NamespaceMap.java,v 1.1.22.2 2005/10/12 14:59:28 domelz Exp $ */
package com.sas.services.webdav;

/**
 * Implements a specific map from prefixes to namespace URI's. Prefixes
 * will automatically be generated and a record maintained of the
 * namespace URI associated with it. One may also specify an explicit prefix 
 * to namespace URI mapping to use.
 * 
 * @author  Rob Owen
 * @version 1.1 20-Jan-2003
 * @since   1.1
 */
public class NamespaceMap extends java.util.HashMap {
    
   /**
    * Serial version UID. This value should remain unchanged until one
    * desires to make previous serialized versions no longer compatible with
    * the current version.
    */
   private static final long serialVersionUID = 3970489135130944123L;

   /**
     * Sets a specific mapping from the prefix to the namespace URI. This would
     * be used if there was a preferred prefix (eg. predefined).
     * @param prefix The preferred prefix
     * @param namespaceURI The namespace URI
     * @return String Returns the prefix mapped to the namespace URI
     */public String addNSPrefix( String prefix, String namespaceURI ) {
        String nsprefix = (String)get( namespaceURI );
        if ( nsprefix == null ) {
            put( namespaceURI, prefix );
            nsprefix = prefix;
        }
        return nsprefix;
    }
             
    /** 
     * Returns a prefix that can be used for the given namespaceURI. If a
     * previous mapping did not exist then a new prefix is created for this 
     * namespace URI and returned.
     * @param namespaceURI The namespace URI for which a prefix is required
     * @return String The prefix to be used for the specified namespace URI
     */
    public String getPrefixforURI( String namespaceURI ) {
        String nsprefix = (String)get( namespaceURI );
        if ( nsprefix == null ) {
            nsprefix = "Z"+size();
            put( namespaceURI, nsprefix );
        }
        return nsprefix;
    }
    

}
