/* $Id$ */
// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.services.webdav;

import java.io.InputStream;
import java.net.URL;
import java.rmi.RemoteException;

/**
 * Represents an item in a collection
 */
/**
 * This interface represents the operations that can be carried out on a non-collection resource or item.
 * <p>
 * @author  Rob Owen
 * @version 1.0 15-Nov-2001
 */
public interface ResourceInterface extends java.rmi.Remote
{
    /**
     * Get the URL for this resource
     */
    public String getURL() throws DAVException, RemoteException;

    /**
     * Get the name for this resource
     */
    public String getName() throws DAVException, RemoteException;

    /**
     * Delete the item
     */
    public void delete() throws DAVException, RemoteException;

    /**
     * Lock the item
     */
    public void lock() throws DAVException, RemoteException;

    /**
     * Unlock the item
     */
    public void unlock() throws DAVException, RemoteException;

    /**
     * Copy the item to destination
     */
    public ResourceInterface copy(String destination) throws DAVException, RemoteException;

    /**
     * Move an item to destination
     */
    public ResourceInterface move(String destination) throws DAVException, RemoteException;

    /**
     * Compare two items - are they equal
     */
    public boolean equals(ResourceInterface e2) throws DAVException, RemoteException;

    /**
     * Remove properties from an item
     *
     * @param pMap PropertyMap with all the property names to be removed from the item
     */
    public void removeProperties(PropertyMap pMap) throws DAVException, RemoteException;

    /**
     * Set the properties of an item
     *
     * @param pMap PropertyMap with all the properties to be set on the item
     */
    public void setProperties(PropertyMap pMap) throws DAVException, RemoteException;

    /**
     * Get the properties of an item in a property map
     *
     * @return PropertyMap properties for the item
     *
     */
    public PropertyMap getProperties() throws DAVException, RemoteException;

    /**
     * Set the contents of the element
     */
    public void setContents(byte[] input) throws DAVException, RemoteException;

    /**
     * Set the contents of the element
     */
    public void setContents( URL url ) throws DAVException, RemoteException;

    /**
     * Set the contents of the element from an input stream. For remote deployment 
     * the InputStream should be wrapped in an {@link com.sas.io.InputStreamAdapter InputStreamAdapter}
     */
    public void setContents( InputStream is ) throws DAVException, RemoteException;

    /**
     * Get the contents of the element
     */
    public InputStream getContents() throws DAVException, RemoteException;
    
    /**
     * Returns whether the resource exists or not
     */
    public boolean exists() throws DAVException, RemoteException;

    /**
     * Close the underlying DAVResource
     */
    public void close() throws DAVException, RemoteException;

    /**
     * Returns whether this Resource object encapsulates a shared resource
     * @return boolean Whether this Resource encapsulates a shared resource
     */
    boolean isSharedResource() throws DAVException, RemoteException;

    /**
     * Retrieves the shared resource
     * @return SharedResourceInterface The shared resource that this object encapsulates
     */
    SharedResourceInterface getSharedResource() throws DAVException, RemoteException;
}