/* $Id: SharedResourceInterface.java,v 1.1.4.3 2005/10/07 17:33:53 domelz Exp $ */
// Copyright (c) 2002-2004 SAS Institute Inc., Cary, NC 27513
// All Rights Reserved.
package com.sas.services.webdav;
import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * This interface is for a shared resource that represents a DAVResource. 
 * <p>
 * @author  Rob Owen
 * @version 1.0 27-Feb-05
 * @since   1.0
 */
public interface SharedResourceInterface extends Remote {
    
	/**
	 * Retrieves the underlying DAVResource. An attempt is made to access the resource using the mutex. Once
	 * the retrieval is successful, no one else can gain access to the underlying DAVResource until it is 
	 * released. See @see #releaseConnection(DAVResourceInterface).  
	 * @return DAVResourceInterface The underlying shared resource that is not remoteable
     * @throws RemoteException in the case of a network error
	 */
    DAVResourceInterface getConnection() throws RemoteException;
    
	/**
	 * Releases the underlying DAVResource.  
	 * @param r The underlying shared resource that was previously retrieved
     * @throws RemoteException in the case of a network error
	 */
    void releaseConnection(DAVResourceInterface r) throws RemoteException;
    
	/**
	 * Releases the underlying DAVResource.  
     * @throws RemoteException in the case of a network error
	 */
    void releaseConnection() throws RemoteException;

    /**
	 * Close the underlying DAVResource.  
     * @throws RemoteException in the case of a network error
	 */
    void close() throws RemoteException;

	/**
	 * Calls the Collection.createCollection method for this instance and returns a remote collection. 
	 * @see Collection#createCollection(SharedResourceInterface, CollectionInterface, String)
	 * @param c 
	 * @param name  
     * @throws RemoteException in the case of a network error
	 */
    CollectionInterface createCollection(CollectionInterface c, String name) throws RemoteException;

    /**
	 * Calls the Collection.createResource method for this instance and returns a remote resource. 
	 * @see Collection#createResource(SharedResourceInterface, CollectionInterface, String)
	 * @param c 
	 * @param name  
     * @throws RemoteException in the case of a network error
	 */
    ResourceInterface createResource(CollectionInterface c, String name) throws RemoteException;

    /**
	 * Calls the ResourceFactory.createCollection method for this instance and returns a remote collection. 
	 * @see ResourceFactory#createCollection(SharedResourceInterface, String)
	 * @param resource 
	 * @param urlString  
     * @throws RemoteException in the case of a network error
	 */
    CollectionInterface createCollectionRF(ResourceInterface resource, String urlString) throws RemoteException;

    /**
	 * Calls the ResourceFactory.bindResource method for this instance and returns either a remote collection
	 * or a remote resource. 
	 * @see ResourceFactory#bindResource(SharedResourceInterface, String)
	 * @param path
     * @throws RemoteException in the case of a network error
	 */
    ResourceInterface bindResource(String path) throws RemoteException;

    /**
	 * Calls the ResourceFactory.bindResource method for this instance and returns a remote collection. 
	 * @see ResourceFactory#bindResource(SharedResourceInterface, String, PropertyMap)
	 * @param path 
	 * @param p  
     * @throws RemoteException in the case of a network error
	 */
    ResourceInterface bindResource(String path, PropertyMap p) throws RemoteException;

}
