/* $Id$ */
// Copyright (c) 2001 SAS Institute Inc., Cary, NC 27513
package com.sas.services.webdav;

import org.apache.webdav.lib.Property;
import java.io.InputStream;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
/**
 * A convenience class to bring together a number of utilties.
 *
 * @author Rob Owen
 * @version 1.0 10-Sep-2001
 */
public class Utils
{
    static char[] hexChars = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
    static char[] translateChars = {'.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    ' ','!','"','#','$','%','&','.','(',')','*','+',',','-','.','/',  /*I18nOK:LINE*/
                                    '0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','?',  /*I18nOK:LINE*/
                                    '@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',  /*I18nOK:LINE*/
                                    'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^','_', /*I18nOK:LINE*/
                                    '`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',  /*I18nOK:LINE*/
                                    'p','q','r','s','t','u','v','w','x','y','z','{','|','}','~','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',  /*I18nOK:LINE*/
                                    '.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.'}; /*I18nOK:LINE*/

    private static final String CDATA_START = "<![CDATA[";    /*I18nOK:EMS*/
    private static final String CDATA_END   = "]]>";          /*I18nOK:EMS*/
    private static final String CDATA_END_ESCAPED = "]]&gt;"; /*I18nOK:EMS*/

    /**
     * Convert a byte array to a hex representation that is easy to visualise.
     *
     * @param chars The byte array to convert to hex display
     * @return String A string with the multiline hex display output.
     */
    public static String bytesToHex(byte[] chars)
    {
        char[] prtChars = new char[16];

        StringBuffer buffer = new StringBuffer();
        buffer.append(' '); /*I18nOK:EMS*/
        for (int i=0; i<chars.length; i++)
        {
           if (i%16 == 0)
           {
              if (i > 0) {
                  buffer.append("  "+String.valueOf(prtChars)); /*I18nOK:COS*/
                  buffer.append("\n ");
              }
              buffer.append(intToHex(i, 6)+"  ");
              for (int j=0; j < 16; j++) prtChars[j] = ' ';  /*I18nOK:EMS */
              // counter = 0;
           }
           if (i%4 == 0) buffer.append(' '); /*I18nOK:EMS */
           buffer.append(hexChars[(chars[i] >> 4) & 0x0f]);
           buffer.append(hexChars[chars[i] & 0x0f]);
           prtChars[i%16] = translateChars[chars[i]<0 ? chars[i]+256 : chars[i]];
        }
        int charsOnLine = chars.length%16;
        if (charsOnLine == 0) charsOnLine = 16;
        for (int i=charsOnLine; i<16; i++) {
            buffer.append(' '); /*I18nOK:EMS */
            buffer.append(' '); /*I18nOK:EMS */
        }
        if (charsOnLine < 5)  buffer.append(' '); /*I18nOK:EMS */
        if (charsOnLine < 9)  buffer.append(' '); /*I18nOK:EMS */
        if (charsOnLine < 13) buffer.append(' '); /*I18nOK:EMS */
        buffer.append("  "+String.valueOf(prtChars)+"\n");
        return new String(buffer);

    }
    
    /**
     * Convert a stream of bytes to hex.
     * @param byteStream A stream containing the bytes to convert
     * @return String Hex representation of chars
     */
    public static String byteStreamToHex( InputStream byteStream )
    {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        int c;
        try {
            while ( (c=byteStream.read()) != -1 ) {
                baos.write( c );
            }
        } catch ( IOException ioe ) {
            // baos contains the stream so far
        }
        return bytesToHex( baos.toByteArray() );
    }


    /**
     * Convert a set of characters to hex.
     * @param chars A string containing the characters to convert
     * @return String Hex representation of chars
     */
    public static String stringToHex( String chars )
    {
        StringBuffer result = new StringBuffer();
        for (int i=0; i<chars.length(); i++)
        {
            result.append(intToHex(chars.charAt(i),2));
        }
        return new String(result);
    }


    /**
     * Convert an int to printable hex format (eg 33 = x'21') padding if necessary.
     *
     * @param number The number to be converted
     * @param width The width of the resulting string
     * @return String A string of width "width" containing the hex representation of the number "number"
     */
    public static String intToHex (int number,  int width)
    {
        int aNibble;
        char[] result = new char[width];
        for (int i=0; i < width; i++)
        {
            aNibble = number & 0x0f;
            number  = number >> 4;
            result[width-i-1] = hexChars[aNibble];
        }
        return new String(result);
    }

    /**
     * Compare two url strings. A trailing slash is not significant and so this method checks
     * to see whether the two urls are equal with trailing slashes.
     *
     * @param url1 The first url string
     * @param url2 The second url string
     * @return boolean true if the urls represent the same resource, false otherwise
     */
    public static boolean isSameUrl ( String url1, String url2 )
    {
        if (url1.equals(url2)) return true;
        String _url1 = url1;
        String _url2 = url2;
        if (_url1.endsWith("/")) _url1 = _url1.substring(0, _url1.length()-1);
        if (_url2.endsWith("/")) _url2 = _url2.substring(0, _url2.length()-1);
        return _url1.equals(_url2);
    }

    /**
     * A utility method to check through a set of properties and determine if it is a collection
     * or not. When using depth=1 or depth=infinity, it is possible to get this information without
     * having to bind to the resource itself. Use the value for the "{DAV:}resourcetype" property to
     * determine whether this is a collection or not.
     *
     * @param properties The property map for this resource.
     */
    public static boolean isCollection( java.util.Map properties )
    {
        String resourcetype = (String) properties.get( DAVConstants.DAV_RESOURCETYPE );
        return ((resourcetype != null) && resourcetype.equals( DAVConstants.DAV_COLLECTION ));
    }

    /**
     * A utility method to convert an array of WebDAV Property objects to a property map.
     *
     * @param props An array of Property objects
     * @return PropertyMap (maps property names to values). The property names in this case use
     * the format "{namespace}propertyname".
     */
    public static PropertyMap convertPropertyArray( Property[] props )
    {
        PropertyMap _properties = new PropertyMap();
        for (int i = 0; i < props.length; i++) {
            if (props[i].getNamespaceURI() != null) {
                _properties.put("{" + props[i].getNamespaceURI() + "}" + props[i].getLocalName(), props[i].getPropertyAsString());
            } else {
                _properties.put("{}" + props[i].getLocalName(), props[i].getPropertyAsString());
            }
        }
        return _properties;
    }

    /**
     * Get the relative path for a given url. This is the last segment of the name after the last forward
     * slash, unless there is a forward slash at the end of the url. In this case it is the name inbetween
     * the two slashes.
     *
     * @param urlString A string representing a url
     * @return String The relative path, or last segment in the url
     *
     */
     public static String getRelativePath( String urlString ) {

        String _relPath;
        int ptr = urlString.lastIndexOf('/');
        if (ptr < 0) _relPath = urlString;

        // If there is a slash at the end of the url, then we have found the wrong one
        else if (urlString.endsWith("/")) {
            int ptr2 = urlString.lastIndexOf('/', ptr-1);
            _relPath = urlString.substring(ptr2+1, ptr);
        } else _relPath = urlString.substring(ptr+1);
        return _relPath;
     }

    /**
     * Append a segment to a path.
     *
     * Concatenate the segment to the path with a slash inbetween. This routine takes into
     * account all the nasties, including blanks before or after the slashes, empty segment
     * or path, muliple slashes, and missing slashes.
     * <p>
     * @param path The path (or url)
     * @param segment The segment to be added to the path
     * @return String The correctly created new path
     *
     */
    public static String addSegment( String path, String segment ) {
        if ( path == null && segment == null ) return "";
        if ( path == null ) return segment.trim();
        if (segment == null ) return path.trim();
        StringBuffer tmp = new StringBuffer( path.trim() );
        if( (tmp.length() == 0) || (tmp.charAt(tmp.length()-1) != '/')) tmp.append("/");
        String tmpSegment = segment.trim();
        if( tmpSegment.startsWith("/")) {
            if( tmpSegment.length() > 1 )tmp.append( tmpSegment.substring( 1 ) );
        } else tmp.append( tmpSegment );
        return tmp.toString();/*I18nOK:LSM*/
    }

    /**
     * Encapsulate the text in a CDATA section in case there is any markup in text. If
     * text already a CDATA section then just return it. 
     * @param text The text to be encapsulated
     * @return The CDATA section encapsulating the text
     */
    public static String CDATA(String text) {
        if ( text == null ) return "";
        if ( !text.startsWith(CDATA_START) ) {
            if ( text.indexOf(CDATA_END) > -1 ) {
                text = text.replaceAll(CDATA_END, CDATA_END_ESCAPED);
            }
            return CDATA_START + text + CDATA_END;
        }
        return text;
    }

    /**
     * Escape the three characters "<", "&", ">", which cannot appear in literal form 
     * except as markup (unless they are in a CDATA section).
     * @param text The text to be escaped
     * @return The escaped text
     */
    public static String XMLESCAPE(String text) {
        if ( text == null ) return "";
        if ( text.startsWith(CDATA_START) && text.endsWith(CDATA_END)) return text;
        text = text.replaceAll("&","&amp;");
        text = text.replaceAll("<","&lt;");
        text = text.replaceAll(">","&gt;");
        return text;
    }

    /**
     * Strip off leading forward slash characters
     * @param buffer Initial string from which to remove leading slashes
     * @return String The input string with leading slashes removed
     **/
    public static String stripLeadingSlashes(String buffer) {
        StringBuffer _buffer = new StringBuffer(buffer);
        while ( _buffer.length() > 0 && _buffer.charAt(0) == '/') _buffer.deleteCharAt(0);
        return _buffer.toString(); /*I18nOK:LSM*/
    }
}
