/* $Id: DAVTreeNode.java,v 1.1.2.1.14.2 2005/10/11 16:05:41 domelz Exp $ */
//Copyright (c) 2001-2004 SAS Institute Inc., Cary, NC 27513
//All Rights Reserved.
package com.sas.services.webdav.tools;

import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import javax.swing.tree.DefaultMutableTreeNode;
import com.sas.services.webdav.DAVConstants;

/**
 * Inner class representing a node in the webDAV system
 * @author Rob Owen
 * @version 1.1.3 10/11/05
 * @since 1.0
 */
public class DAVTreeNode extends DefaultMutableTreeNode {

   private static final long serialVersionUID = 4051045263622026040L;
   protected String resName;
    transient protected TreeMap DAVprops;
    protected boolean isCollection;
    protected String name;
    protected String fullName;
    protected boolean populated;
    protected boolean interim;

    public DAVTreeNode(String url, boolean isCollection) {
        super(url);
        this.isCollection = isCollection;
        fullName = url;
        String _url = fullName.trim();
        // Determine the relative path for this url ... this is the piece after the last slash in the url
        int ptr = _url.lastIndexOf('/');
        // Need special handling for collections
        if (_url.charAt(_url.length() - 1) != '/') {
            // If it is a collection and the url doesn't end with a slash then we have found the
            // right slash, but lets add a slash to the url to create the right format.
            if (isCollection) _url = _url + "/";
        } else {
            // If it does already end in a slash then we have just found that as the last slash
            // in the url, search from one back from this trailing slash
            ptr = _url.lastIndexOf('/', ptr - 1);
        }
        String relpath = _url.substring(ptr + 1);
        if (relpath.length() == 0) relpath = _url;
        resName = relpath;
    }

    public boolean isLeaf() {
        return !isCollection;
    }

    public boolean getAllowsChildren() {
        return isCollection;
    }

    public String toString() {
        return resName;
    }
    
   public boolean populateCollections(DAVTree tree, boolean descend) {
       boolean addedNodes = false;
       // Do this only once
       if (!populated) {
           if (interim) {
               // we've had a quick look here before
               // remove the dummy node that we added last time
               removeAllChildren();
               interim = false;
           }
           if ( !isCollection ) return addedNodes; 
           Iterator _children = tree.getChildrenList(fullName).entrySet().iterator(); // Get
           // list
           // of
           // contents
           // Process the collections
           while (_children.hasNext()) {
               Map.Entry child = (Map.Entry) _children.next();
               String _name = (String) child.getKey();
               String resourcetype = (String) child.getValue();
               // DAVResource d = getDAVResource( name );
               DAVTreeNode node = new DAVTreeNode(_name, resourcetype.equals(DAVConstants.DAV_COLLECTION));
               this.add(node);
               // if ( node.getAllowsChildren() && descend ) {
               //     node.populateCollections( false );
               // }
               addedNodes = true;
               if (!descend) {
                   // Only add one node if not descending
                   break;
               }
           }
           // If we are scanning to get all the collections, or if we found no
           // subdirectories, there is no reason to look at this directory again,
           // so set populated to true. Otherwise, we set interim so that we can
           // look again in the future if we need to.
           if (descend || !addedNodes) {
               populated = true;
           } else {
               interim = true;
           }
       }
       return addedNodes;
   }
}
