/* $Id$ */
//Copyright (c) 2001-2004 SAS Institute Inc., Cary, NC 27513
//All Rights Reserved.
package com.sas.services.webdav.tools;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.font.TextAttribute;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.AttributedString;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;
import java.io.InputStream;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeSelectionModel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.swing.SwingConstants;

import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import javax.swing.ImageIcon;
import com.sas.text.Message;

/**
 * Displays XML in a JTree
 * @author  Rob Owen
 * @version 1.0 12-Dec-2001
 */

public class DOMTree extends JTree {

   private static final long serialVersionUID = 3616728283719481398L;
   private ImageIcon plusImage = new ImageIcon(this.getClass().getClassLoader().getResource("com/sas/services/webdav/tools/treeplus.gif"));
    private ImageIcon minusImage = new ImageIcon(this.getClass().getClassLoader().getResource("com/sas/services/webdav/tools/treeminus.gif"));
    private static JFrame frame;
    private Hashtable prefixes = new Hashtable();
    private boolean _showDetails = false;

    /** Constants used for JAXP 1.2 */
    static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
    static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
    /** All output will use this encoding */
    static final String outputEncoding = "UTF-8";

    private static final int FRAME_WIDTH = 720;
    private static final int FRAME_HEIGHT = 580;

    private static int treeIndent;
    private static final Color COLOR_FRAME_BACKGROUND = new Color( 220, 240, 255 );
    private static final Color COLOR_DELIMITER = Color.BLUE;
    private static final Color COLOR_TEXT      = Color.BLACK;
    private static final Color COLOR_NAME      = new Color(153, 0, 0);
	private static final Color COLOR_DEFAULT   = Color.BLUE;
	private static final Color COLOR_NS        = Color.RED;
	private static final Font  FONT_DEFAULT    = new Font( "SanSerif", Font.PLAIN, 11 );
	private static final Font  FONT_DEFAULT_BOLD = new Font( "SanSerif", Font.BOLD, 11 );

    public DOMTree() {

		try {
			UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
		} catch (Exception e) {
		    // just look the default then
		}
		
		// We have no tree listener but this looks more natrual
		getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		setShowsRootHandles(false);
		putClientProperty("JTree.lineStyle", /*I18nOK:EMS*/ "None" /*I18nOK:EMS*/ );

		CustomBasicRenderer btcr = new CustomBasicRenderer();
		setCellRenderer(btcr);

		BasicTreeUI basicTreeUI = (BasicTreeUI)getUI();
		basicTreeUI.setLeftChildIndent(15);
		basicTreeUI.setRightChildIndent(0);
		basicTreeUI.setExpandedIcon(minusImage);
		basicTreeUI.setCollapsedIcon(plusImage);
		treeIndent = basicTreeUI.getRightChildIndent() + basicTreeUI.getLeftChildIndent();
		treeIndent = 15;

		// If this were editable it would result in too much code
		// for this demo.
		setEditable(false);
		// setBackground( COLOR_FRAME_BACKGROUND );
    }
    
    
    public DOMTree(Node root, boolean showDetails) {
		this();
		_showDetails = showDetails;
		setRoot(root);

    }
    
    public DOMTree( InputStream is ) {
        this();
        _showDetails = false;
        setRoot( parseXML(is) );
    }
    
	public void setRoot(Node root) {
		// Take a DOM and convert it to a Tree model for the JTree
		prefixes.clear();
		DefaultMutableTreeNode top = createTreeNode(root, _showDetails);
		Iterator items = prefixes.entrySet().iterator();
		String nsString = "";
		while (items.hasNext()) {
			Map.Entry me = (Map.Entry)items.next();
			nsString = " xmlns:" + me.getKey() + "=\"" + me.getValue() + "\" " + nsString; /*I18nOK:LINE*/
		}
		AttributedText at = new AttributedText( FONT_DEFAULT, COLOR_DEFAULT );
		at.setText( getNodeType(root)+": " + nsString ); /*I18nOK:LINE*/
		top.setUserObject( at ); /*I18nOK:LINE*/
		DefaultTreeModel dtModel = new DefaultTreeModel(top);
		setModel(dtModel);
	}


    /**
    
    This takes a DOM Node and recurses through the children until each one is
    added to a DefaultMutableTreeNode. This can then be used by the JTree as a
    tree model. The second parameter can be used to provide more visual detail
    for debugging.
    
    */
    protected DefaultMutableTreeNode createTreeNode(Node root, boolean showDetails) {
        DefaultMutableTreeNode dmtNode = null;
        // String nodeText = null;

        String type = getNodeType(root);
        String name = root.getNodeName();
        String value = root.getNodeValue();
        boolean elementFinished = false;
        AttributedText at = new AttributedText( FONT_DEFAULT, COLOR_DEFAULT ) ;

        if (showDetails) {
            at.setText( '[' + type + "] --> " + name + " = " + value );
        } else {
            // Special case for TEXT_NODE, others are similar but not catered for here.
            if (root instanceof Element) {
                String prefix = root.getPrefix();
                String ns = root.getNamespaceURI();
                String localName = root.getLocalName();
                if (prefix != null && ns != null) {
                    String usedNS = (String)prefixes.get(prefix);
                    if (usedNS == null) {
                        prefixes.put(prefix, ns);
                        // Seems like the xmlns is an attribute on the node
                        at.setText( "<", COLOR_DELIMITER );
                        at.addText( name, COLOR_NAME );
                    } else {
                        at.setText( "<", COLOR_DELIMITER );
                        at.addText( prefix + ':' + localName, COLOR_NAME );
					}
                } else {
                    at.setText( "<", COLOR_DELIMITER );
                    at.addText( name, COLOR_NAME );
				}
            } else {
                at.setText( (root.getNodeType() == Node.TEXT_NODE) ? value : name, COLOR_NAME );
			}
            if (root instanceof Document)
                at.setText( "<?xml version=\"1.0\" ?>" );
            if (root instanceof Comment)
                at.setText( "<!--" + ((Comment)root).getData() + "-->" );

        }

        // Display the attributes if there are any
        NamedNodeMap attribs = root.getAttributes();
        if (attribs != null) {
            for (int i = 0; i < attribs.getLength(); i++) {
                Node attNode = attribs.item(i);
                String attName = attNode.getNodeName().trim();
                String attValue = attNode.getNodeValue().trim();

                if (attValue != null) {
                    if (attValue.length() > 0) {
                        if ( attName.startsWith("xmlns") ) at.addText( ' ' + attName, COLOR_NS );
                        else at.addText( ' ' + attName, COLOR_NAME );
                        at.addText( "=\"", COLOR_DELIMITER );
						if ( attName.startsWith("xmlns") ) at.addText( attValue, COLOR_NS, FONT_DEFAULT_BOLD );
						else at.addText( attValue, COLOR_TEXT, FONT_DEFAULT_BOLD );
                        at.addText( "\"", COLOR_DELIMITER );
                    }
                }
            }
        }

        elementFinished = false;
        if (root instanceof Element) {
            if (root.hasChildNodes()) {
                at.addText( ">", COLOR_DELIMITER );
                if (root.getFirstChild() == root.getLastChild() && root.getFirstChild() instanceof Text) {
                    at.addText( root.getFirstChild().getNodeValue(), COLOR_TEXT, FONT_DEFAULT_BOLD );
                    at.addText( "</", COLOR_DELIMITER );
                    at.addText( name, COLOR_NAME );
                    at.addText( ">", COLOR_DELIMITER );
                    elementFinished = true;
                }
            } else {
                at.addText( "/>", COLOR_DELIMITER );
                elementFinished = true;
            }
        }
        
        // at.setText(nodeText);

        dmtNode = new DefaultMutableTreeNode();
        dmtNode.setUserObject( at );

        if (!elementFinished) {
            // If there are any children and they are non-null then recurse...
            if (root.hasChildNodes()) {
                NodeList childNodes = root.getChildNodes();
                if (childNodes != null) {
                    for (int k = 0; k < childNodes.getLength(); k++) {
                        Node nd = childNodes.item(k);
                        if (nd != null) {
                            // A special case could be made for each Node type.
                            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                                dmtNode.add(createTreeNode(nd, showDetails));
                            }

                            // Filter out comment nodes
                            if (nd.getNodeType() == Node.COMMENT_NODE)
                                continue;

                            // This is the default
                            String data = nd.getNodeValue();
                            if (data != null) {
                                data = data.trim();
                                if (!data.equals("\n") && !data.equals("\r\n") && data.length() > 0) {
                                    dmtNode.add(createTreeNode(nd, showDetails));
                                }
                            }
                        }
                    }

                    // A special case could be made for each Node type.
                    if (root.getNodeType() == Node.ELEMENT_NODE) {
                    	at = new AttributedText( FONT_DEFAULT, COLOR_DEFAULT );
                    	at.setText( "</", COLOR_DELIMITER );
                    	at.addText( root.getNodeName(), COLOR_NAME );
                    	at.addText( ">", COLOR_DELIMITER );
                        dmtNode.add(new DefaultMutableTreeNode(at));
                    }
                }
            }
        }

        return dmtNode;
    }

    /**
    
    This simple method returns a displayable string given a NodeType
    
    */
    public String getNodeType(Node node) {
        String type;

        switch (node.getNodeType()) {
            case Node.ELEMENT_NODE :
                {
                    type = "Element" /*I18nOK:EMS*/;
                    break;
                }
            case Node.ATTRIBUTE_NODE :
                {
                    type = "Attribute" /*I18nOK:EMS*/;
                    break;
                }
            case Node.TEXT_NODE :
                {
                    type = "Text" /*I18nOK:EMS*/;
                    break;
                }
            case Node.CDATA_SECTION_NODE :
                {
                    type = "CData section" /*I18nOK:EMS*/;
                    break;
                }
            case Node.ENTITY_REFERENCE_NODE :
                {
                    type = "Entity reference" /*I18nOK:EMS*/;
                    break;
                }
            case Node.ENTITY_NODE :
                {
                    type = "Entity" /*I18nOK:EMS*/;
                    break;
                }
            case Node.PROCESSING_INSTRUCTION_NODE :
                {
                    type = "Processing instruction" /*I18nOK:EMS*/;
                    break;
                }
            case Node.COMMENT_NODE :
                {
                    type = "Comment" /*I18nOK:EMS*/;
                    break;
                }
            case Node.DOCUMENT_NODE :
                {
                    type = "Document" /*I18nOK:EMS*/;
                    break;
                }
            case Node.DOCUMENT_TYPE_NODE :
                {
                    type = "Document type" /*I18nOK:EMS*/;
                    break;
                }
            case Node.DOCUMENT_FRAGMENT_NODE :
                {
                    type = "Document fragment" /*I18nOK:EMS*/;
                    break;
                }
            case Node.NOTATION_NODE :
                {
                    type = "Notation" /*I18nOK:EMS*/;
                    break;
                }
            default :
                {
                    type = "Unknown, contact Sun!" /*I18nOK:EMS*/;
                    break;
                }
        }
        return type;
    }

    /**
    
    A common point of exit...
    
    */

    private static void exit() {
        //System.out.println("Graceful exit");
        //System.exit(0);
    }

    public static void helpInfo() {
        System.out.println(RB.getStringResource("DOMTree.cmd1.txt"));
        System.out.println(RB.getStringResource("DOMTree.cmd2.txt"));
        System.out.println(RB.getStringResource("DOMTree.cmd3.txt"));
        System.out.println(RB.getStringResource("DOMTree.cmd4.txt"));
        System.exit(8);
    }

    public static void main(String[] args) throws Exception {
        String schemaFile = null;
        String xmlFile = null;
        DOMTree tree = null;
        boolean validate = false;
        if (args.length < 1)
            helpInfo();

		try {
			UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
		} catch (Exception e) {
		    // just look like the default then
		}
        //try
        //{
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("-xsd")) {
                validate = true;
            } else if (args[i].equals("-xsdf")) {
                if (i == args.length - 1) {
                    helpInfo();
                }
                validate = true;
                schemaFile = args[++i];
            } else
                xmlFile = args[i];
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        if (validate) {
            dbf.setValidating(true);
            try {
                dbf.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
            } catch (IllegalArgumentException x) {
                // This can happen if the parser does not support JAXP 1.2
                System.err.println(Message.format(RB.getResources(), "DOMTree.dbattrerr1.fmt.txt", JAXP_SCHEMA_LANGUAGE));
                System.err.println(RB.getStringResource("DOMTree.dbattrerr2.txt"));
                System.exit(1);
            }
        }

        // Set the schema source, if any.  See the JAXP 1.2 maintenance
        // update specification for more complex usages of this feature.
        if (schemaFile != null) {
            dbf.setAttribute(JAXP_SCHEMA_SOURCE, new File(schemaFile));
        }

        DocumentBuilder db = dbf.newDocumentBuilder();

        // Set ErrorHandler before parsing
        OutputStreamWriter errorWriter = new OutputStreamWriter(System.err, outputEncoding);
        db.setErrorHandler(new MyErrorHandler(new PrintWriter(errorWriter, true)));

        try {
        if (xmlFile != null) {
            Document doc = db.parse(new FileInputStream(xmlFile));
            tree = new DOMTree(doc, false);
            } else {
                System.out.println( RB.getStringResource("DOMTree.filenotexist.txt") );
            }
        } finally {
            if ( errorWriter != null ) {
                try {
                    errorWriter.close();
                } catch (IOException ioe) {
                    // ignore
                }
            }
        }

		// Create a frame to "hold" the panel
		frame = new JFrame(RB.getStringResource("DOMTree.application.txt"));
		// Create a new JScrollPane but override one of the methods.
		JScrollPane jScroll = new JScrollPane(tree)
		 {
         private static final long serialVersionUID = 3762534533580403249L;

         // This keeps the scrollpane a reasonable size
			public Dimension getPreferredSize() {
				return new Dimension(frame.getWidth() - 20 /*FRAME_WIDTH-20*/
				, frame.getHeight() - 40 /*FRAME_HEIGHT-40*/
				);
			}
		};

		frame.getContentPane().add( jScroll );

		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension dim = toolkit.getScreenSize();
		int screenHeight = dim.height;
		int screenWidth = dim.width;

		// This should display a WIDTH x HEIGHT sized Frame in the middle of the screen
		frame.setBounds((screenWidth - FRAME_WIDTH) / 2, (screenHeight - FRAME_HEIGHT) / 2, FRAME_WIDTH, FRAME_HEIGHT);

		//frame.setBackground( COLOR_FRAME_BACKGROUND );
		// frame.getContentPane().setLayout(new BorderLayout());

		// Add a WindowListener so that we can close the window
		WindowListener wndCloser = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				exit();
			}
		};
		frame.addWindowListener(wndCloser);

		frame.validate();
		frame.setVisible(true);
    }

    // Error handler to report errors and warnings
    private static class MyErrorHandler implements ErrorHandler {
        /** Error handler output goes here */
        private PrintWriter out;

        MyErrorHandler(PrintWriter out) {
            this.out = out;
        }

        /**
         * Returns a string describing parse exception details
         */
        private String getParseExceptionInfo(SAXParseException spe) {
            String systemId = spe.getSystemId();
            if (systemId == null) {
                systemId = "null";
            }
            String info = Message.format(RB.getResources(), "DOMTree.parsererr.fmt.txt" /*I18nOK:EMS*/
            , systemId, new Integer(spe.getLineNumber()), spe.getMessage());
            return info;
        }

        public void warning(SAXParseException spe) throws SAXException {
            out.println(Message.format(RB.getResources(), "DOMTree.warning.fmt.txt" /*I18nOK:EMS*/
            , getParseExceptionInfo(spe)));
        }

        public void error(SAXParseException spe) throws SAXException {
            String message = Message.format(RB.getResources(), "DOMTree.error.fmt.txt" /*I18nOK:EMS*/
            , getParseExceptionInfo(spe));
            throw new SAXException(message);
        }

        public void fatalError(SAXParseException spe) throws SAXException {
            String message = Message.format(RB.getResources(), "DOMTree.fatalerror.fmt.txt" /*I18nOK:EMS*/
            , getParseExceptionInfo(spe));
            throw new SAXException(message);
        }
    }

    // This class is a custom renderer based on TreeCellRenderer
    protected class CustomBasicRenderer implements TreeCellRenderer {
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            // Allow the original renderer to set up the label
            //            Component c = super.getTreeCellRendererComponent(tree, value, selected,
            //            													expanded, leaf, row,
            //            													hasFocus);
            AttributedText at = null;
            Object o = ((DefaultMutableTreeNode)value).getUserObject();
            if ( !(o instanceof AttributedText) ) {
            	at = new AttributedText( FONT_DEFAULT, COLOR_DEFAULT );
            	at.setText( o.toString() );
            } else at = (AttributedText)o;
            at.setIsLeaf( leaf );
            Component c = new MyTreeLabel( at );
            c.setBackground( COLOR_FRAME_BACKGROUND );

            // Now change the icon and text position
            if (c instanceof JLabel) {
                JLabel l = (JLabel)c;
                l.setHorizontalAlignment(SwingConstants.LEFT);
                l.setVerticalAlignment(SwingConstants.CENTER);
                l.setVerticalTextPosition(SwingConstants.CENTER);
                l.setHorizontalTextPosition(SwingConstants.LEFT);

                // Set the font
                int type = Font.PLAIN;
                l.setFont(new Font("SanSerif", type, 11));
                // l.setIcon( null );
            }

            return c;
        }
    }

    protected class MyTreeLabel extends JLabel {

      private static final long serialVersionUID = 3689916162842571318L;
      private AttributedText attributedLabel;
        public MyTreeLabel() {
            // private constructor
        }

        public MyTreeLabel(AttributedText attributedLabel) {
            super();
            this.attributedLabel = attributedLabel;
            setBackground(Color.WHITE);
        }

        public Dimension getPreferredSize() {
            return new Dimension(attributedLabel.getAttributedString().getIterator().getEndIndex() * 9, 15);
        }

        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            int offset1 = 0;
            int offset2 = treeIndent;
            int offset = offset2;
            if ( attributedLabel.isClosingElement() ) {
                offset = offset1;
            }
            g.drawString(attributedLabel.getAttributedString().getIterator(), offset, 10);
            
        }
    }
    
	public class AttributedText {
	
		private Font defaultFont   = null;
		private Color defaultColor = null;
		private StringBuffer text = new StringBuffer();
		private Vector colorAttr = new Vector(40);
		private Vector fontAttr = new Vector(40);
		private boolean isLeaf = false;
	
		public AttributedText() {
		    // private constructor
		}
		public AttributedText( Font font, Color color ) {
			defaultFont = font;
			defaultColor = color;
		}
	
		public void addText( String str ) {
			text.append( str );
			for ( int i=0; i<str.length(); i++ ) {
				colorAttr.add( null );
				fontAttr.add( null );
			}
		}
	
		public void addText( String str, Color color ) {
			text.append( str );
			for ( int i=0; i<str.length(); i++ ) {
				colorAttr.add( color );
				fontAttr.add( null );
			}
		}
	
		public void addText( String str, Color color, Font font ) {
			text.append( str );
			for ( int i=0; i<str.length(); i++ ) {
				colorAttr.add( color );
				fontAttr.add( font );
			}
		}
	
		public void setText( String str ) {
			text.setLength( 0 );
			colorAttr.setSize( 0 );
			text.append( str );
			for ( int i=0; i<str.length(); i++ ) {
				colorAttr.add( null );
				fontAttr.add( null );
			}
		}
	
		public void setText( String str, Color color ) {
			text.setLength( 0 );
			colorAttr.setSize( 0 );
			text.append( str );
			for ( int i=0; i<str.length(); i++ ) {
				colorAttr.add( color );
				fontAttr.add( null );
			}
		}

		public void setText( String str, Color color, Font font ) {
			text.setLength( 0 );
			colorAttr.setSize( 0 );
			text.append( str );
			for ( int i=0; i<str.length(); i++ ) {
				colorAttr.add( color );
				fontAttr.add( font );
			}
		}

		public boolean isClosingElement() {
			String s = text.toString(); /*I18nOK:LSM*/
			return s.startsWith("</");
		}
		
		public void setIsLeaf( boolean isLeaf ) {
			this.isLeaf = isLeaf;
		}
		
		public boolean isLeaf() {
			return isLeaf;
		}
		
		public AttributedString getAttributedString() {
			AttributedString as = new AttributedString( text.toString() ); /*I18nOK:LSM*/
			as.addAttribute( TextAttribute.FONT, defaultFont );
			as.addAttribute( TextAttribute.FOREGROUND, defaultColor );
			Color lastColor = null;
			Font lastFont = null;
			int lastColorIndex = 0;
			int lastFontIndex = 0;
			for ( int i=0; i<text.length(); i++ ) {
				if ( colorAttr.get(i) != lastColor ) {
					if ( i > 0 && lastColor != null ) {
						as.addAttribute( TextAttribute.FOREGROUND, lastColor, lastColorIndex, i );
					}
					lastColorIndex = i;
					lastColor = (Color)colorAttr.get(i);
				}
				if ( fontAttr.get(i) != lastFont ) {
					if ( i > 0 && lastFont != null ) {
						as.addAttribute( TextAttribute.FONT, lastFont, lastFontIndex, i );
					}
					lastFontIndex = i;
					lastFont = (Font)fontAttr.get(i);
				}
			
			}
			if ( lastColor != null && lastColorIndex < text.length() ) as.addAttribute( TextAttribute.FOREGROUND, lastColor, lastColorIndex, text.length() );
			if ( lastFont != null && lastFontIndex < text.length() ) as.addAttribute( TextAttribute.FONT, lastFont, lastFontIndex, text.length() );
			return as;
		}
	}

	private Document parseXML( InputStream is ) {

		Document document = null;

		try {
			final DocumentBuilderFactory documentBuilderFactory = 
				DocumentBuilderFactory.newInstance();
			documentBuilderFactory.setNamespaceAware(true);
			final DocumentBuilder documentBuilder = 
				documentBuilderFactory.newDocumentBuilder();

			document = documentBuilder.parse(is);


		}
		catch (Exception e) {
		    e.printStackTrace();
		}

		return document;

	}   
}
