/*
 * Copyright (c) 2007 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm;

import java.rmi.RemoteException;

import org.apache.log4j.Logger;

import com.sas.solutions.brm.errors.BRMError;
import com.sas.solutions.brm.errors.BRMInternalError;
import com.sas.solutions.brm.errors.BRMUserError;

public abstract class TaskRunner
{
    private static final Logger LOG = Logger.getLogger(TaskRunner.class);

    BRMController controller;
    String host;
    
    public TaskRunner(BRMController controller, String host)
    {
        this.controller = controller;
        this.host = host;
    }

    protected BRMAgentInterface getAgent()
    {
        BRMAgentInterface agent = this.controller.getRemoteAgent(this.host);
        return agent;
    }

    /**
     * Shut down component
     */
    public void stop(String component)
    {
        try
        {
            getAgent().setComponent(component);
            if (!(getAgent().stopServices()))
            {
                throw new BRMError("Could not stop component " + component +
                        " on host " + this.host);
            }
        }
        catch (BRMUserError e)
        {
            throw new BRMError(e);
        }
        catch (BRMInternalError e)
        {
            throw new BRMError(e);
        }
        catch(RemoteException e)
        {
            throw new BRMError(e);
        }
    }
    
    /**
     * Run backup or restore, depending on this instance's task type
     */
    public void runTask(String component)
    {
        try
        {
            getAgent().setComponent(component);
            if (!runCurrentTask())
            {
                throw new BRMError("Error for component " + component
                        + " on host " + this.host);
            }
        }
        catch (BRMUserError e)
        {
            throw new BRMError(e);
        }
        catch (BRMInternalError e)
        {
            throw new BRMError(e);
        }
        catch(RemoteException e)
        {
            throw new BRMError(e);
        }
    }

    /**
     * Restart component
     */
    public void start(String component)
    {
        try
        {
            getAgent().setComponent(component);
            if (!(getAgent().startServices()))
            {
                throw new BRMError("Could not start component " + component +
                        " on host " + this.host);
            }
        }
        catch (BRMUserError e)
        {
            throw new BRMError(e);
        }
        catch (BRMInternalError e)
        {
            throw new BRMError(e);
        }
        catch(RemoteException e)
        {
            throw new BRMError(e);
        }
    }

    /**
     * Shut down the TaskRunner
     */
    public void close()
    {
    }

    /**
     * Run backup or restore, depending on the type of task this is
     * @return true iff successful
     * @throws BRMUserError
     * @throws BRMInternalError
     * @throws RemoteException
     */
    public abstract boolean runCurrentTask()
        throws BRMUserError, BRMInternalError, RemoteException;
}
