package com.sas.solutions.brm.logging;

import java.io.File;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.BuildListener;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;

public class BRMBuildListener implements BuildListener
{
    private static final Logger LOG = Logger.getLogger("remote.agentAnt");
    String component;
    
    public BRMBuildListener(String component) {
        this.component=component;
    }

    public void buildStarted(BuildEvent event) {
        LOG.info("Build started for component:"+ component );

    }

    public void buildFinished(BuildEvent event) {
        LOG.info("Build finished for component:"+ component );

    }

    public void targetStarted(BuildEvent event) {
        LOG.info(constructMessageFront(event)+" target started");

    }

    public void targetFinished(BuildEvent event) {
        LOG.info(constructMessageFront(event)+" target finished");

    }

    public void taskStarted(BuildEvent event) {
        LOG.debug(constructMessageFront(event)+" task started");

    }

    public void taskFinished(BuildEvent event) {
        LOG.debug(constructMessageFront(event)+" task finished");

    }

    public void messageLogged(BuildEvent event)
    {
        Level level = logLevelFromPriorityInt(event.getPriority());
        StringBuffer sb = new StringBuffer();
        sb.append(eventLocationPrefix(event));
        sb.append(event.getMessage());
        Throwable t = event.getException();
        if (t != null)
            LOG.log(level, sb.toString(), t);
        else
            LOG.log(level, sb.toString());
    }

    private Level logLevelFromPriorityInt(int priority)
    {
        switch (priority)
        {
            case Project.MSG_ERR:
                return Level.ERROR;
            case Project.MSG_WARN:
                return Level.WARN;
            case Project.MSG_INFO:
                return Level.INFO;
            case Project.MSG_DEBUG:
            case Project.MSG_VERBOSE:
                return Level.DEBUG;
            default:
                // In case some new message priority is invented later...
                return Level.WARN;
        }
    }

    private String eventLocationPrefix(BuildEvent event)
    {
        StringBuffer sb = new StringBuffer();
        Task task = event.getTask();
        if (task != null)
        {
            Location location = task.getLocation();
            if (!Location.UNKNOWN_LOCATION.equals(location))
            {
                File file = new File(location.getFileName());
                int lineNumber = location.getLineNumber();
                sb.append(file.getName());
                sb.append(':');
                sb.append(lineNumber);
                sb.append(' ');
            }
        }
        return sb.toString();
    }
    
    private String constructMessageFront(BuildEvent event)
    {
        StringBuffer sb = new StringBuffer(50);
        sb.append(eventLocationPrefix(event));
        if (event.getTask() != null)
        {
            sb.append("{"+event.getTarget()+"}["+
                    event.getTask().getTaskName()+"]");
        }
        else if (event.getTarget() != null)
        {
            sb.append("{"+event.getTarget()+"}");
        }
        return sb.toString();
    }

}
