/*
 * Created on Feb 16, 2007
 * Copyright (c) 2007 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

import com.sas.solutions.brm.util.BRMUtil;

public class LogUtil
{
    /**
      * Disabled constructor - static utility class.
      */
    private LogUtil()
    {
    }

    /**
      * Get a Logger associated with logName. If loggerName is null, return
      * the root logger.
      */
    public static Logger getLogger(String loggerName)
    {
        Logger logger = null;
        if (loggerName == null)
        {
            logger = Logger.getRootLogger();
        }
        else
        {
            logger = Logger.getLogger(loggerName);
        }
        return logger;
    }

    /**
      * @return The name of the Logger that generated the event, or null if
      * there is an error trying to get the logger name.
      */
    public static String getLoggerName(LoggingEvent event)
    {
        return getLoggerName(event, false);
    }

    public static String getLoggerName(LoggingEvent event, 
            boolean reportErrors)
    {
        try
        {
            return event.getLoggerName();
        }
        catch (NoSuchMethodError e)
        {
            if (reportErrors)
            {
                System.err.println(
                    "*** Error getting logger name via event.getLoggerName()");
                throw e;
            }
        }
        try
        {
            return (String)BRMUtil.getDeclaredFieldValue(event, "categoryName");
        }
        catch (Throwable e)
        {
            if (reportErrors)
            {
                System.err.println(
                    "*** Error getting logger name via event.categoryName");
                throw new Error(e);
            }
        }
        return null;
    }

    public static void dumpEventInfo(LoggingEvent event)
    {
        System.out.println(event);
        System.out.println("  Event thread: " + event.getThreadName());
        System.out.println("  Current thread: " + Thread.currentThread());
        System.out.println("  Event classloader: " +
                event.getClass().getClassLoader());
        System.out.println("  Thread classloader: " +
                Thread.currentThread().getClass().getClassLoader());
        System.out.println("  System classloader: " +
                ClassLoader.getSystemClassLoader());
        BRMUtil.dumpObjInfo(event, System.out);
    }
}
