/*
 * Created on Feb 14, 2007
 * Copyright (c) 2007 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import java.rmi.RemoteException;

import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.spi.LoggingEvent;
import org.apache.log4j.spi.ErrorCode;

/**
  * A log4j appender that forwards log messages over an rmi connection to
  * another process.
  *
  * Connect all RemoteAppender instances in this process to an object
  * implementing RemoteLoggingInterface (i.e. a RemoteLoggingServer) by
  * calling RemoteLoggingClient.setLogServer(rli).
  */
public class RemoteAppender extends AppenderSkeleton
{
    boolean locationInfo;

    public RemoteAppender()
    {
        super();
        this.locationInfo = false;
    }

    /**
      * Returns value of the <b>LocationInfo</b> property which
      * determines whether location (stack) info is sent to the remote
      * subscriber.
      */
    public boolean getLocationInfo()
    {
        return locationInfo;
    }

    /**
      * If true, the information sent to the remote subscriber will
      * include caller's location information. By default no location
      * information is sent to the subscriber.
      */
    public void setLocationInfo(boolean locationInfo)
    {
        this.locationInfo = locationInfo;
    }

    public void append(LoggingEvent event)
    {
        if (this.locationInfo)
        {
            event.getLocationInformation();
        }
        try
        {
            RemoteLoggingClient.append(event);
        }
        catch (RemoteException e)
        {
            errorHandler.error(
                    "Error sending log event to RemoteLoggingServer", e,
                    ErrorCode.GENERIC_FAILURE);
        }
    }

    public void close()
    {
        if (this.closed)
            return;
        this.closed = true;
    }

    /**
      * Layout is not required because formatting is done on the remote end.
      */
    public boolean requiresLayout()
    {
        return false;
    }
}
