/*
 * Created on Feb 14, 2007
 * Copyright (c) 2007 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import java.rmi.RemoteException;

import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

/**
  * Forward LoggingEvent messages to a RemoteLoggingServer over RMI.
  */
public class RemoteLoggingClient
{
    private static RemoteLoggingInterface logServer = null;
    private static boolean useLocalFallback = true;

    /**
      * This constructor is disabled -- this class contains only static
      * methods.
      */
    private RemoteLoggingClient()
    {
    }

    public static void setLogServer(RemoteLoggingInterface rli)
    {
        logServer = rli;
    }

    public static RemoteLoggingInterface getLogServer()
    {
        return logServer;
    }

    public static void append(LoggingEvent event) throws RemoteException
    {
        RemoteLoggingInterface rli = logServer;
        if (rli != null)
        {
            try
            {
                rli.append(event);
                return;
            }
            catch (RemoteException e)
            {
                // Controller is shut down, can't send remote log messages
                // Ignore the exception and fall back to local log
            }
            catch (Throwable t)
            {
                System.err.println("Error appending to logServer: " + t);
            }
        }

        if (useLocalFallback)
        {
            // Fallback strategy - log locally since no remote logger
            String loggerName = LogUtil.getLoggerName(event);
            if (loggerName != null && loggerName.startsWith("remote."))
            {
                // Strip "remote." off the front of the logger name.
                // This allows the logger client and server to use the same
                // log4j.properties file without causing infinite loops.
                loggerName = loggerName.substring(7);
            }
            Logger localLogger = LogUtil.getLogger(loggerName);
            localLogger.callAppenders(event);
        }
    }
}
