/*
 * Created on Feb 14, 2007
 * Copyright (c) 2007 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import java.rmi.RemoteException;

import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

/**
  * Remote logging server.
  * This class handles RMI calls containing LogEvent objects, allowing log
  * events generated in one JVM to be logged in another. The reason to use
  * this rather than, say, SocketAppender, is that in my situation I already
  * have an RMI connection -- why create another socket?
  */
public class RemoteLoggingServer 
    implements RemoteLoggingInterface
{
    public RemoteLoggingServer()
    {
    }

    public void append(LoggingEvent event) throws RemoteException
    {
        String loggerName = LogUtil.getLoggerName(event);
        if (loggerName != null && loggerName.startsWith("remote."))
        {
            // Strip "remote." off the front of the logger name.
            // This allows the logger client and server to use the same
            // log4j.properties file without causing infinite loops.
            loggerName = loggerName.substring(7);
        }
        Logger remoteLogger = LogUtil.getLogger(loggerName);
        remoteLogger.callAppenders(event);
    }

    /**
      * Test RemoteLoggingServer.
      * Paradoxically, this process is an rmi client. It is only a server
      * with respect to logging.
      */
    public static void main(String[] args) throws Exception
    {
        System.out.println("Starting RemoteLoggingServer in testing mode");
        RemoteLoggingServer rms = new RemoteLoggingServer();
        java.rmi.server.UnicastRemoteObject.exportObject(rms);
        String host = "localhost";
        int port = RemoteLoggingTesterInterface.PORT;
        String serviceName = RemoteLoggingTesterInterface.NAME;
        System.out.print("Connecting to rmi://" + host + ":" + port + "/"
                + serviceName + " ...");
        java.rmi.registry.Registry registry = 
            java.rmi.registry.LocateRegistry.getRegistry("localhost", port);
        RemoteLoggingTesterInterface rlt = 
            (RemoteLoggingTesterInterface)registry.lookup(serviceName);
        System.out.println("connected");
        rlt.setRemoteLogger(rms);
        System.out.println("RemoteLoggingServer is running");
        System.out.print("Press Enter to quit ...");
        System.in.read();
        rlt.teardown();
        java.rmi.server.UnicastRemoteObject.unexportObject(rms, true);
    }
}
