/* $Id: MigrateConfig.java,v 1.1.4.4.2.1 2007/03/06 23:06:45 davhan Exp $ */
/*
 * Created on Jun 8, 2006
 * Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import com.sas.solutions.brm.Constants;
import com.sas.solutions.brm.errors.BRMConfigurationException;

/**
 * Migration Configuration - tells the migration process what need to be
 * changed and what the new values should be.
 */
public class MigrateConfig
{
    // "Tier" names
    
    //these are the standards.  "extensions" will be below.
    public static final String OMR = Constants.METADATA_COMPONENT;
    public static final String MIDTIER = Constants.WEBAPP_COMPONENT;
    public static final String WORKSPACE = Constants.WORKSPACE_COMPONENT;
    //this one is being split. commenting = disallow?
    //public static final String STOREDPROC = "storedProc";
    public static final String SPAWNER = Constants.OBJECT_SPAWNER_COMPONENT;
    public static final String DAV = Constants.WEBDAV_COMPONENT;
    public static final String REMOTESERVICES = Constants.REMOTE_SERVICES_COMPONENT;
    public static final String ODCS = Constants.ODCS_COMPONENT;
    public static final String MYSQL = Constants.MYSQL_COMPONENT;
    public static final String OLAP = Constants.OLAP_COMPONENT;
    public static final String CONNECT = Constants.CONNECT_COMPONENT;
    
    
    //here are the "extensions"
    public static final String SETINIT = "setinit_component";
    public static final String STOREDPROC_DIRS = "stored_process_dirs";
    public static final String STOREDPROC_SERVER = "stored_process_server_component";
    public static final String STOREDPROC_DATA = "stored_process_data_component";

    public static final String SAS_EXE_DIR = "sas_exe_dir";
    
    Properties migrationProperties = null;
    
    public MigrateConfig() throws BRMConfigurationException
    {
        MigrationConfigParser mcp = new MigrationConfigParser();
        this.migrationProperties = mcp.getMigrationConfig();
    }

    public MigrateConfig(String fileName) throws BRMConfigurationException
    {
        MigrationConfigParser mcp = new MigrationConfigParser();
        try
        {
            this.migrationProperties = mcp.parse(new File(fileName));
        }
        catch (IOException e)
        {
            throw new BRMConfigurationException(
                    "Could not read from file: " + fileName, e);
        }
    }
    
    public String getMetadataFromHost()
    {
        return getFromHost(OMR);
    }
    
    public String getMetadataToHost()
    {
        return getToHost(OMR);
    }
    
    public String getFromHost(String component)
    {
        String key = component+".from.Host";
        return this.migrationProperties.getProperty(key);
    }

    public String getToHost(String component)
    {
        String key = component+".to.Host";
        return this.migrationProperties.getProperty(key);
    }
    
    public String getFromPrefix(String component)
    {
        String key = component+".from.Prefix";
        return this.migrationProperties.getProperty(key);
    }
    
    public String getToPrefix(String component)
    {
        String key = component+".to.Prefix";
        return this.migrationProperties.getProperty(key);
    }
    
    public String getToCommand(String component)
    {
        String key = component+".to.Command";
        return this.migrationProperties.getProperty(key);
    }

    public String getFromValue(String component, String valueName)
    {
        String key = component + ".from." + valueName;
        return this.migrationProperties.getProperty(key);
    }

    public String getToValue(String component, String valueName)
    {
        String key = component + ".to." + valueName;
        return this.migrationProperties.getProperty(key);
    }

    public String getValue(String component, String valueName)
    {
        String key = component + "." + valueName;
        return this.migrationProperties.getProperty(key);
    }

    /**
      * Get all of the key-value pairs in a component section of the
      * migrate.ini file.
      */
    public Map getSection(String component)
    {
        Map result = new HashMap();
        String keyPrefix = component + ".";
        for (Enumeration e = this.migrationProperties.propertyNames();
                e.hasMoreElements(); )
        {
            String key = (String)e.nextElement();
            if (key.startsWith(keyPrefix))
            {
                String k = key.substring(component.length() + 1);
                result.put(k, this.migrationProperties.getProperty(key));
            }
        }
        return result;
    }
}
