package com.sas.solutions.brm.migrate;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.sas.solutions.brm.Constants;
import com.sas.solutions.brm.errors.BRMConfigurationException;

public class MigrationConfigParser {
    
    private static final Logger LOG = Logger.getLogger(MigrationConfigParser.class);

    
    public Properties parse(File in) 
        throws FileNotFoundException, IOException
    {
        Properties p = new Properties();
        BufferedReader br = new BufferedReader(new FileReader(in));
        String line = "";
        String section = "";
        while((line=br.readLine()) != null)
        {
            line = line.trim();
            if (line.length() == 0)
            {
                continue;
            }
            else if (line.startsWith(";"))
            {
                continue;
            }
            else if (line.startsWith("[") && line.endsWith("]"))
            {
                section = line.substring(1, line.length()-1);
                section = section.trim();
            }
            else
            {
                int eqIndex = line.indexOf("=");
                if (eqIndex >= 0)
                {
                    String key = line.substring(0, eqIndex);
                    key = key.trim();
                    String value = line.substring(eqIndex + 1);
                    value = value.trim();
                    p.setProperty(section + "." + key, value);
                }
            }
        }
        return p;
    }
    
    public static void main(String args[]) throws Exception{
        MigrationConfigParser mcp = new MigrationConfigParser();
        Properties p = mcp.parse(new File(args[0]));
        
        System.out.println("PROPERTIES:");
        for(Iterator i = p.keySet().iterator();i.hasNext();){
            Object key = i.next();
            System.out.println(key+"->"+p.getProperty(key.toString()));
        }
    }
    
    public Properties getMigrationConfig() throws BRMConfigurationException
    {
        java.net.URL url =
            ClassLoader.getSystemResource(Constants.MIGRATE_CONFIG_FILE);
        if (url == null)
        {
            throw new BRMConfigurationException(
                    "Could not find config file on classpath: " +
                    Constants.MIGRATE_CONFIG_FILE);
        }
        File f = new File(url.getFile());
        Properties p = new Properties();
        try
        {
            p = parse(f);
        }
        catch(IOException e)
        {
            throw new BRMConfigurationException(
                    "Could not read from file: " + url.getFile(), e);
        }
        return p;
    }
}
