/*
 * Created on Jun 20, 2006
 * Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class ReaderToInputStream extends InputStream
{
    private Reader reader;
    private CharsetEncoder encoder;
    private CharBuffer inBuf;
    private ByteBuffer outBuf;
    
    public ReaderToInputStream(Reader reader)
    {
        this(reader, "UTF-8");
    }
    
    public ReaderToInputStream(Reader reader, String charsetName)
    {
        this.reader = reader;
        Charset cs = Charset.forName(charsetName);
        this.encoder = cs.newEncoder();
        this.inBuf = CharBuffer.allocate(4);
        this.outBuf = ByteBuffer.allocate(16);
        this.outBuf.flip();
    }

    public int read() throws IOException
    {
        // Preconditions:
        // outBuf is set up for reading (getting)
        // inBuf is set up for writing (putting)
        boolean endOfStream = false;
        while (!this.outBuf.hasRemaining() && !endOfStream)
        {
            // No more bytes left in output buffer
            // Read a character and try to generate more bytes
            int unicode_num = this.reader.read();
            if (unicode_num < 0)
            {
                endOfStream = true;
            }
            else
            {
                this.inBuf.put((char)unicode_num);
            }
            this.inBuf.flip();
            this.outBuf.clear();
            this.encoder.encode(this.inBuf, this.outBuf, endOfStream);
            this.inBuf.compact();
            this.outBuf.flip();
        }
        if (!this.outBuf.hasRemaining())
        {
            assert endOfStream;
            return -1;
        }
        int c = (int)this.outBuf.get();
        this.outBuf.compact();
        this.outBuf.flip();
        if (c < 0)
        {
            return 256 + c;
        }
        else
        {
            return c;
        }
    }
    
    public int available()
    {
        try{
        if(reader.ready()){
            return 1;
        }else return 0;
        }catch(IOException e){return 0;}
    }
    
    private static void testUTF8() throws IOException
    {
        String s = "ABC" + ((char)0xa0) + "DEF";
        System.out.println(s);
        StringReader sr = new StringReader(s);
        ReaderToInputStream adapter = new ReaderToInputStream(sr);
        byte[] bytes = new byte[10];
        int n = adapter.read(bytes);
        System.out.println(n + " bytes");
        assert n == 8;
    }
    
    private static void testBuildDOMTree1()
    {
        String s = "<test attr1=\"val1\" attr2=\"val2\"/>";
        System.out.println(s);
        Document doc = MainMigrator.buildDOMTree(s);
        NodeList nodes = doc.getElementsByTagName("test");
        Element element = (Element)nodes.item(0);
        System.out.println(element.getTagName());
    }
    
    private static void testBuildDOMTree2()
    {
        // debugging java.io.UTFDataFormatException
        try
        {
            byte[] bytes = new byte[17466];
            FileInputStream fis = new FileInputStream("C:\\temp\\bytes.txt");
            int count = fis.read(bytes);
            fis.close();
            assert count == bytes.length;
            String s = new String(bytes, "UTF-16BE");
            System.out.println(s);
            Document doc = MainMigrator.buildDOMTree(s);
            System.out.println(doc.getDocumentElement().getTagName());
        }
        catch (Exception e1)
        {
            e1.printStackTrace();
        }
    }
   
    public static void main(String[] args) throws Exception
    {
        testUTF8();
        testBuildDOMTree1();
        testBuildDOMTree2();
    }

}
