/*
 * Created on Apr 12, 2006
 * Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.omr;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * This SAX Handler class is for parsing the result of the IOMI 
 * GetMetadata() method.
 * @author davhan
 */
public class ObjectInfoHandler extends BaseOMRXMLHandler
{
    public static final String TYPE_KEY = GenericMetadataObject.TYPE_KEY;
    
    private static final int STATE_INIT = 0;
    private static final int STATE_IN_LIST = 1;
    public static final int STATE_AFTER_LIST = 2;
    
    private int state;
    private int levelsDeep;
    private int beginLevel;
    private Map objectInfo;
    private List currentAssociatedObjects;

    public ObjectInfoHandler()
    {
        super();
        this.state = STATE_INIT;
        this.levelsDeep = 0;
        this.beginLevel = -1;
        this.objectInfo = new HashMap();
        this.currentAssociatedObjects = null;
    }
    
    public int getState()
    {
        return this.state;
    }

    public String getTypeName()
    {
        return (String)this.objectInfo.get(TYPE_KEY);
    }
    
    public Map getObjectInfo()
    {
        return this.objectInfo;
    }
    
    public void startElement(String uri, String localName, String qName,
            Attributes attributes) throws SAXException 
    {
        ++this.levelsDeep;
        if (this.state == STATE_INIT)
        {
            this.objectInfo.put(TYPE_KEY, qName);
            int numAttributes = attributes.getLength();
            for (int i=0; i<numAttributes; ++i)
            {
                String propertyName = attributes.getQName(i);
                this.objectInfo.put(propertyName, attributes.getValue(i));
            }
            this.state = STATE_IN_LIST;
            this.beginLevel = this.levelsDeep;
            return;
        }
        else if (this.state == STATE_AFTER_LIST)
        {
            return;
        }
        else if (this.state == STATE_IN_LIST && 
                 this.levelsDeep == (this.beginLevel + 1))
        {
            // If we reach this point we must be looking for elements whose tag
            // names are association names.
            this.currentAssociatedObjects = new ArrayList();
            this.objectInfo.put(qName, this.currentAssociatedObjects);
        }
        else if (this.state == STATE_IN_LIST &&
                 this.levelsDeep == (this.beginLevel + 2))
        {
            // If we reach this point we are looking at elements that comprise
            // this list of associated objects.
            Map associatedObjectInfo = new HashMap();
            associatedObjectInfo.put(TYPE_KEY, qName);
            assert(this.currentAssociatedObjects != null);
            this.currentAssociatedObjects.add(associatedObjectInfo);
            int numAttributes = attributes.getLength();
            for (int i=0; i<numAttributes; ++i)
            {
                String propertyName = attributes.getQName(i);
                associatedObjectInfo.put(propertyName, attributes.getValue(i));
            }
        }
    }
    
    public void endElement(String uri, String localName, String qName)
            throws SAXException 
    {
        --this.levelsDeep;
        if (this.state == STATE_IN_LIST && this.levelsDeep == this.beginLevel)
        {
            this.currentAssociatedObjects = null;
        }
        if (this.state == STATE_IN_LIST && this.levelsDeep < this.beginLevel)
        {
            this.state = STATE_AFTER_LIST;
            return;
        }
    }
    
    public static ObjectInfoHandler parseXML(String inputXML)
    {
        ObjectInfoHandler handler = new ObjectInfoHandler();
        parseXML(handler, inputXML);
        return handler;
    }

    public static void main(String[] args)
    {
        String inputXML = 
            "<PhysicalTable Id=\"A2345678.A2000001\" Name=\"New Table\" Desc=\"New Table added hrough API\">\n" +
            "<Columns>\n" +
            "  <Column Id=\"A2345678.A3000001\" Name=\"New Column\" Desc=\"New Column added through API\"/>\n" +
            "  <Column Id=\"A2345678.A3000002\" Name=\"New Column2\" Desc=\"New Column2 added through API\"/>\n" +
            "</Columns>\n" +
            "</PhysicalTable>";
        ObjectInfoHandler listHandler = parseXML(inputXML);
        System.out.println("Input XML: " + inputXML);
        System.out.println("Object Info: " + listHandler.getObjectInfo());
    }

}
