/*
 * THIS FILE WAS DERIVED FROM APACHE ANT, BUT HAS CHANGED.
 */

package com.sas.solutions.brm.tar;

import java.io.IOException;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;

import org.apache.tools.ant.types.EnumeratedAttribute;
import org.apache.tools.bzip2.CBZip2OutputStream;

public  final class TarCompressionMethod extends EnumeratedAttribute {
    
    // permissible values for compression attribute
    /**
     *    No compression
     */
    private static final String NONE = "none";
    /**
     *    GZIP compression
     */
    private static final String GZIP = "gzip";
    /**
     *    BZIP2 compression
     */
    private static final String BZIP2 = "bzip2";
    
    
    /**
     * Default constructor
     */
    public TarCompressionMethod() {
        super();
        setValue(GZIP);
    }
    
    /**
     *  Get valid enumeration values.
     *  @return valid enumeration values
     */
    public String[] getValues() {
        return new String[] {NONE, GZIP, BZIP2 };
    }
    
    /**
     *  This method wraps the output stream with the
     *     corresponding compression method
     *
     *  @param ostream output stream
     *  @return output stream with on-the-fly compression
     *  @exception IOException thrown if file is not writable
     */
    public OutputStream compress(final OutputStream ostream)
    throws IOException {
        final String value = getValue();
        if (GZIP.equals(value)) {
            return new GZIPOutputStream(ostream);
        } else {
            if (BZIP2.equals(value)) {
                ostream.write('B');
                ostream.write('Z');
                return new CBZip2OutputStream(ostream);
            }
        }
        return ostream;
    }
}
