/*
 * THIS FILE WAS DERIVED FROM APACHE ANT, BUT HAS CHANGED.
 */

package com.sas.solutions.brm.tar;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.MissingResourceException;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;

import com.sas.solutions.brm.tar.Tar.TarFileSet;

public class TarUtility {

    private static final Logger LOG = Logger.getLogger(TarUtility.class);
    
    //because i'm going over a for loop w/ this.
    public static final String EMPTY_STRING= "";
    
    private TarCompressionMethod compression = new TarCompressionMethod();
    
    /**
     * Create an archive file from a list of directories. The files in each
     * directory in the list will be included in the archive file,
     * non-recursively.
     * 
     * @param dirList
     *            List of directory path names (String)
     * @param relativeTo
     *            Common parent directory to all directories in dirList, used to
     *            shorten pathnames stored in archive.
     * @param backupSection
     *            Name of subdirectory of backup directory in which the
     *            resulting archive will be placed.
     * @param archiveName
     *            Name of the archive file, without directory part. If it ends
     *            with ".zip", use ZIP format; If it ends with ".tar.gz" or
     *            ".tgz", use tar+gzip format.
     * 
     * @throws IOException
     *             If there is an creating the zip file or the the subdirectory,
     *             or if the configuration properties file cannot be found.
     * @throws MissingResourceException
     *             If there is missing configuration info
     * @throws IllegalArgumentException
     *             If the archiveName does not end in a recognized suffix.
     * @throws IllegalArgumentException
     *             If relativeTo is not a common parent directory of one or more
     *             files in dirList.
     * @throws FileNotFoundException
     *             If one of the directories in dirList does not exist.
     */
    
    public void copyFileSetToArchive(Vector filesets, String archive, Project p) throws IOException {
//      File gzFile = new File(archive);
//        GZIPOutputStream gzip = new GZIPOutputStream(new FileOutputStream(
//                gzFile));
//        TarOutputStream tar = new TarOutputStream(gzip);
        TarOutputStream tar = null;
        tar = new TarOutputStream(
                compression.compress(
                        new BufferedOutputStream(
                                new FileOutputStream(archive))));
        tar.setLongFileMode(TarOutputStream.LONGFILE_GNU);
        
        
        for (Enumeration e = filesets.elements();
        e.hasMoreElements();) {
            TarFileSet fs = (TarFileSet) e.nextElement();
            String[] files = fs.getFiles(p);
            if (LOG.isDebugEnabled())
            {
                LOG.debug(
                        "Copying TarFileSet to archive:\n" +
                        "    Directory: " + fs.getDir(p) + "\n" +
                        "    Files: " + stringArrayToString(files) + "\n" +
                        "    Archive: " + archive + "\n" +
                        "");
            }
            if (files.length > 1 && fs.getFullpath().length() > 0) {
                throw new BuildException("fullpath attribute may only "
                        + "be specified for "
                        + "filesets that specify a "
                        + "single file.");
            }
            for (int i = 0; i < files.length; i++) {
                if(EMPTY_STRING.equals(files[i])) {continue;}
                File f = new File(fs.getDir(p), files[i]);
                if(f.isDirectory()){continue;}
                
                try{
                    FileInputStream in = new FileInputStream(f);
                    
                    TarEntry te = new TarEntry(f);
                    te.setName(calcArchiveName(f,fs.getDir(p)));
                    tar.putNextEntry(te);
                    
                    //At this point, the tar entry is correct
//                  System.out.println("Tar entry is reporting the size: "+te.getName()+":"+te.getSize());
                    byte[] buf = new byte[4096];
                    int len;
                    long total = 0;
                    while ((len = in.read(buf)) > 0)
                    {
                        tar.write(buf, 0, len);
                        total+=len;
                    }
//                  System.out.println("Total bytes written: "+total);
//                  System.out.println("Record size: "+ tar.getRecordSize());
                    tar.closeEntry();
                    in.close();
                }catch(IOException err) {
//                  System.out.println("broken file: "+name);
                    err.printStackTrace();
                    throw new BuildException(err.getMessage()+f.getAbsolutePath());
                }
            }
        }
        tar.close();
//        gzip.close();
    }
    

    
    /**
     * Calculate the archive name for a source file, given the root of all files
     * being stored in the archive.
     * 
     * Example:
     * 
     * sourceFile = "C:\\Program Files\\SAS\\blah\\file.txt" archiveRoot =
     * "C:\\Program Files\\SAS" calcArchiveName(sourceFile, archiveRoot) ->
     * "blah/file.txt"
     * 
     * @param sourceFile
     *            The path to a file to be stored in the archive.
     * @param archiveRoot
     *            The path to the "root" directory of all files to be stored in
     *            the archive. Note: The case of archiveRoot must be the same as
     *            the case of the equivalent directory names in sourceFile.
     *            (This method may be used on case-sensitive Unix filesystems.)
     * 
     * @throws IllegalArgumentException
     *             If archiveRoot is not a parent, grandparent, or identical to
     *             sourceFile.
     * @throws IOException
     *             If getCanonicalFile() fails.
     */
    public static String calcArchiveName(File sourceFile, File archiveRoot)
    throws IOException
    {
        final File origSourceFile = sourceFile;
        final File origArchiveRoot = archiveRoot;
        if (LOG.isDebugEnabled())
        {
            LOG.debug("calcArchiveName(" + origSourceFile + ", " +
                    origArchiveRoot + ")");
        }
        if (System.getProperty("os.name").indexOf("Windows") >= 0)
        {
            // On Unix, File.getCanonicalFile() apparently converts from a
            // symbolicly-linked path to the "real" path, which breaks this
            // code.
            sourceFile = sourceFile.getCanonicalFile();
            archiveRoot = archiveRoot.getCanonicalFile();
        }
        String[] sourceParts = splitFileParts(sourceFile);
        String[] rootParts = splitFileParts(archiveRoot);
        if (rootParts.length > sourceParts.length)
        {
            throw new IllegalArgumentException(
                "archiveRoot is longer than sourceFile\n" +
                "sourceFile: " + sourceFile + "\n" +
                "archiveRoot: " + archiveRoot + "\n" +
                "sourceParts: " + stringArrayToString(sourceParts) + "\n" +
                "rootParts: " + stringArrayToString(rootParts)
            ); /* I18nOK:LINE */
        }
        for (int i = 0; i < rootParts.length; ++i)
        {
            String sourcePart = sourceParts[i];
            String rootPart = rootParts[i];
            if (!rootPart.equals(sourcePart))
            {
                throw new IllegalArgumentException(
                    "Root part " + i + " (" + rootPart + ") " +
                    "doesn't match source part (" + sourcePart + ")" + "\n" +
                    "sourceFile: " + sourceFile + "\n" +
                    "archiveRoot: " + archiveRoot + "\n" +
                    "sourceParts: " + stringArrayToString(sourceParts) + "\n" +
                    "rootParts: " + stringArrayToString(rootParts)
                ); /* I18nOK:LINE */
            }
        }
        StringBuffer result = new StringBuffer();
        for (int i = rootParts.length; i < sourceParts.length; ++i)
        {
            if (i > rootParts.length)
                result.append("/");
            result.append(sourceParts[i]);
        }
        return result.toString();
    }

    /**
      * This is mainly for debugging
      */
    public static String stringArrayToString(String[] stringArray)
    {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for (int i=0; i<stringArray.length; ++i)
        {
            if (i > 0)
                sb.append(", ");
            sb.append(quoteString(stringArray[i]));
        }
        sb.append("]");
        return sb.toString();
    }

    /**
      * This is mainly for debugging
      */
    public static String quoteString(String s)
    {
        final int lastIndex = s.length();
        StringBuffer sb = new StringBuffer();
        sb.append('"');
        for (int i=0; i<lastIndex; ++i)
        {
            char c = s.charAt(i);
            if (c == '"')
                sb.append("\\\"");
            else if (c == '\\')
                sb.append("\\\\");
            else if (c == '\t')
                sb.append("\\t");
            else if (c == '\r')
                sb.append("\\r");
            else if (c == '\n')
                sb.append("\\n");
            else
                sb.append(c);
        }
        sb.append('"');
        return sb.toString();
    }

    /**
     * Split an "abstract" file up into an array of its component parts. The
     * first item of the resulting array is the root directory or base parent
     * directory, the next item is the next subdirectory down, until the final
     * item which is the leaf file or directory name.
     * 
     * @param file
     * @return an array String file parts
     */
    public static String[] splitFileParts(File file)
    {
        List result = new ArrayList();
        while (file != null)
        {
            result.add(file.getName());
            file = file.getParentFile();
        }
        Collections.reverse(result);
        return (String[]) result.toArray(new String[] {});
    }
}
