/* Copyright (c) 2002 by SAS Institute Inc., Cary, NC 27513 */
package com.sas.swing.util;

import javax.swing.SwingConstants;

/**
 *  Extends the <code>javax.swing.Action</code> interface with some
 *  additional properties.
 */
public interface Action extends javax.swing.Action
   {
	   /**
	    *  The key used for storing the long name to associate
	    *  with the action; typically used for tasks in a task pane.
	    *  This complements <code>NAME</code> defined by our parent interface.
	    */
	   public static final String LONG_NAME = "LongName";

	   /**
		*  The key used for storing the displayable label. This value
		*  replaces <code>Action.NAME</code> as the text
		*  displayed for the <code>Action</code>.
    	*  <p>
		*  @see #NAME
    	*/
		public static final String LABEL = "Label"/*I18nOK:EMS*/;

	   /**
	    *  The key used for storing the name of the small icon.
	    *  <p>
	    *  This differs from the <code>SMALL_ICON</code> which is of type
	    *  <code>javax.swing.Icon</code>.  So typically one would set
	    *  <code>SMALL_ICON_NAME</code> and the <code>Action</code>
	    *  implementation will derive <code>SMALL_ICON</code> from it.
	    *  <p>
	    *  @see #SMALL_ICON
	    *  @see #LARGE_ICON_NAME
	    */
	   public static final String SMALL_ICON_NAME = "SmallIconName";

	   /**
    	*  The key used for storing the disabled version of the small icon;
	    *  typically used in tool bars when an action is unavailable.
    	*  <p>
	    *  If not set defaults to <code>SMALL_ICON</code>.
	    *  @see #SMALL_DISABLED_ICON_NAME
	    *  @see #LARGE_DISABLED_ICON
    	*/
   		public static final String SMALL_DISABLED_ICON = "SmallDisabledIcon";

	   /**
	    *  The key used for storing the name of the disabled version of the
	    *  small icon.
	    *  <p>
	    *  This differs from the <code>SMALL_DISABLED_ICON</code> which is of type
	    *  <code>javax.swing.Icon</code>.  So typically one would set
	    *  <code>SMALL_DISABLED_ICON_NAME</code> and the <code>Action</code>
	    *  implementation will derive <code>SMALL_DISABLED_ICON</code> from it.
	    *  <p>
	    *  If not set defaults to <code>SMALL_ICON_NAME</code>.
	    *  @see #SMALL_DISABLED_ICON
	    *  @see #LARGE_DISABLED_ICON_NAME
	    */
	   public static final String SMALL_DISABLED_ICON_NAME = "SmallDisabledIconName";

	   /**
	    *  The key used for storing the text that describes the action's disabled icon.
	    *  <p>
	    *  If not set defaults to <code>SHORT_DESCRIPTION</code>.
	    *  @see #SHORT_DESCRIPTION
	    */
	   public static final String SHORT_DISABLED_DESCRIPTION = "ShortDisabledDescription";

	   /**
	    *  The key used for storing the large icon to associate
	    *  with the action; typically used for shortcut bars.
	    *  This complements <code>SMALL_ICON</code> defined by our parent interface.
	    *  <p>
	    *  If not set defaults to <code>SMALL_ICON</code>.
	    *  @see #LARGE_ICON_NAME
	    *  @see #LARGE_DISABLED_ICON
	    */
	   public static final String LARGE_ICON = "LargeIcon";

	   /**
	    *  The key used for storing the name of the large icon to associate
	    *  with the action.
	    *  <p>
	    *  This differs from the <code>LARGE_ICON</code> which is of type
	    *  <code>javax.swing.Icon</code>.  So typically one would set
	    *  <code>LARGE_ICON_NAME</code> and the <code>Action</code>
	    *  implementation will derive <code>LARGE_ICON</code> from it.
	    *  <p>
	    *  If not set defaults to <code>SMALL_ICON_NAME</code>.
	    *  @see #LARGE_ICON
	    *  @see #LARGE_DISABLED_ICON_NAME
	    *  @see #SMALL_ICON_NAME
	    */
	   public static final String LARGE_ICON_NAME = "LargeIconName";

	   /**
	    *  The key used for storing the disabled version of the large icon;
	    *  typically used in shortcut bars when the shortcut action is unavailable.
	    *  <p>
	    *  If not set defaults to <code>LARGE_ICON</code>.
	    *  @see #LARGE_ICON
	    *  @see #LARGE_DISABLED_ICON_NAME
	    */
	   public static final String LARGE_DISABLED_ICON = "LargeDisabledIcon";

	   /**
	    *  The key used for storing the name of the disabled version of the
	    *  large icon.
	    *  <p>
	    *  This differs from the <code>LARGE_DISABLED_ICON</code> which is of type
	    *  <code>javax.swing.Icon</code>.  So typically one would set
	    *  <code>LARGE_DISABLED_ICON_NAME</code> and the <code>Action</code>
	    *  implementation will derive <code>LARGE_DISABLED_ICON</code> from it.
	    *  <p>
	    *  If not set defaults to <code>LARGE_ICON_NAME</code>.
	    *  @see #LARGE_DISABLED_ICON
	    *  @see #SMALL_DISABLED_ICON_NAME
	    */
	   public static final String LARGE_DISABLED_ICON_NAME = "LargeDisabledIconName";

	   /**
	    *  The key used for storing the text that describes the action's icon.
	    *  <p>
	    *  If not set defaults to <code>LONG_DESCRIPTION</code>.
	    *  @see #LONG_DESCRIPTION
	    */
	   public static final String LONG_DISABLED_DESCRIPTION = "LongDisabledDescription";

	   /**
	    *  Transformation bean components only.
		*  The key used for storing the name of the rollover icon.
	    *  <p>
		*  When set, the anchor tag for the <code>Action</code> will reference the
		*  image with the <code>onMouseOver</code> tag element.
	    */
	   public static final String ROLLOVER_ICON_NAME = "RolloverIconName";

	   /**
	    *  The key used for storing the name of the group that this action
	    *  belongs to; for example, consider a group of menu items that are
	    *  part of a radio box group.
	    */
	   public static final String GROUP_NAME = "GroupName";

	   /**
	    *  The key used for storing the action's type.  The type can be any of
	    *  the <code>TYPE_*</code> constant values.  If no type is specified the
	    *  implementation should default to <code>TYPE_NORMAL</code>.
	    *  @see #TYPE_NORMAL
	    *  @see #TYPE_BOOLEAN
	    */
	   public static final String TYPE = "Type"/*I18nOK:EMS*/;

	   /**
	    *  Constant value for <code>TYPE</code> property which indicates that
	    *  this is a typical, or normal, action that does not require any
	    *  special handling.
	    *  @see #TYPE
	    */
	   public static final String TYPE_NORMAL = "normal";

	   /**
	    *  Constant value for <code>TYPE</code> property which indicates that
	    *  the result of this action is boolean in nature; for example, consider
	    *  a check box menu item.
	    *  @see #TYPE
	    */
	   public static final String TYPE_BOOLEAN = "boolean";

	   /**
    	*  The key used for storing the horizontal alignment of the text ( Action.Name ) relative to the
		*  icon.  The type can be any of the <code>ALIGNMENT_*</code> constant values.
		*  If no type is specified the implementation should default to <code>ALIGNMENT_RIGHT</code>.
    	*  @see #ALIGNMENT_LEFT
	    *  @see #ALIGNMENT_RIGHT
		*  @see #ALIGNMENT_LEADING
		*  @see #ALIGNMENT_TRAILING
		*  @see #ALIGNMENT_CENTER
    	*/
	   public static final String TEXT_ALIGNMENT = "TextAlignment";

	   /**
    	*  Constant value for <code>TEXT_ALIGNMENT</code> property which indicates
		*  where the text ( <code>Action.NAME</code> ) should be positioned relative to the icon.
	    *  @see #TEXT_ALIGNMENT
    	*/
   		public static final String ALIGNMENT_RIGHT = "SwingConstants.RIGHT";

	   /**
    	*  Constant value for <code>TEXT_ALIGNMENT</code> property which indicates
		*  where the text ( Action.NAME ) should be positioned relative to the icon.
	    *  @see #TEXT_ALIGNMENT
    	*/
   		public static final String ALIGNMENT_LEFT = "SwingConstants.LEFT";

	   /**
    	*  Constant value for <code>TEXT_ALIGNMENT</code> property which indicates
		*  where the text ( Action.NAME ) should be positioned relative to the icon.
	    *  @see #TEXT_ALIGNMENT
    	*/
   		public static final String ALIGNMENT_LEADING = "SwingConstants.LEADING";

	   /**
    	*  Constant value for <code>TEXT_ALIGNMENT</code> property which indicates
		*  where the text ( Action.NAME ) should be positioned relative to the icon.
	    *  @see #TEXT_ALIGNMENT
    	*/
   		public static final String ALIGNMENT_TRAILING = "SwingConstants.TRAILING";


	   /**
    	*  Constant value for <code>TEXT_ALIGNMENT</code> property which indicates
		*  where the text ( Action.NAME ) should be positioned relative to the icon.
	    *  @see #TEXT_ALIGNMENT
    	*/
   		public static final String ALIGNMENT_CENTER = "SwingConstants.CENTER";

	   /**
	    *  The key used for storing the class to instantiate
	    *  and execute when the action's <code>actionPerformed</code> method
	    *  is called.
	    *  <p>
	    *  The class must be an instance of <code>com.sas.util.Command</code>.
	    */
	   public static final String COMMAND_CLASS = "CommandClass";

	   /**
		*  The key used for storing a string that represents a URL to go to
	    *  when the action's <code>actionPerformed</code> method is called.
		*  Transformation bean components only.
	    */
	   public static final String URL = "URL";

	   /**
		*  The key used for storing a string that represents the 'alt' attribute
		*  text within an <IMG> tag.
		*  Transformation bean components only.
	    */
	   public static final String ALTERNATE_TEXT = "AlternateText";

		/**
		 *  The key used for storing a string that represents the client
		 *  target in the anchor tag that is rendered for this action.
		 *  Transformation bean components only.
	     */
		public static final String TARGET = "Target"/*I18nOK:EMS*/;

		/**
		 *  The key used for storing the action's style map.
		 *  Http only.
	     */
		public static final String STYLE_MAP = "StyleMap";

	   /**
	    *  The key used for storing an int to be used as the index
		*  of the mnemonic for the action.
	    *  <p>
	    *  This differs from the <code>MNEMONIC_KEY</code> as this sets
		*  the index in the string in which the mnemonic exists. The <code>MNEMONIC_KEY</code>
		*  needs to be set as well in order for the mnemonic index to work.
		*  So typically one would set the <code>MNEMONIC_KEY</code> first and
		*  then set the <code>MNEMONIC_INDEX</code>.
	    *  <p>
	    *  @see javax.swing.Action#MNEMONIC_KEY
	    */
	   public static final String MNEMONIC_INDEX = "MnemonicIndex";

		/**
	    * Provides a way to programmatically perform the action without having
    	* to create an ActionEvent.
	    */
   		public void performAction();

   }