/* Copyright (c) 2002 by SAS Institute Inc., Cary, NC 27513 */
package com.sas.swing.util;

import com.sas.swing.util.Action;

import java.util.List;

/**
 *  An <code>Action</code> that also has a list of other <code>Actions</code>.
 */
public interface ActionList extends Action
   {

   /**
    *  The key used for storing the list of actions.  The list is represented
    *  as <code>java.util.List</code> and can be accessed via the
    *  convenience method <code>getActions()</code>.
    *  @see #getActions
    */
   public static final String ACTIONS = "Actions"/*I18nOK:EMS*/;

   /**
    *  Object to insert into the list of actions to separate the actions
    *  into groups.
    */
   public static final Object SEPARATOR = "ActionList.SEPARATOR";

   /**
    *  Returns the list of actions.
    *  <p>
    *  The elements of the list will be of type
    *  <code>Action</code> or <code>ActionList</code>
    *  or equal to <code>SEPARATOR</code>.
    *  <p>
    *  Convenience method for
    *  <code>(java.util.List)this.getValue (ActionList.ACTIONS)</code>
    *  @return
    *     a mutable <code>java.util.List</code>; will never return <code>null</code>
    */
   public List getActions ();

   }