package com.sas.table;

/**
 * Interface for implementing moving a column on models for tables.
 * @author sasccf Chad Ferguson
 * @date 28May2003
 */

public interface MoveColumnInterface
{
	/**
	 * Move the column at <var>columnIndex</var> to <var>newIndex</var>. The
	 * old column at <var>columnIndex</var> will now be found at
	 * <var>newIndex</var>. The column that used to be at <var>newIndex</var>
	 * is shifted left or right to make room. This will not move any columns if
	 * <var>columnIndex</var> equals <var>newIndex</var>.
	 * @param columnIndex The index of the column to be moved.
	 * @param newIndex The index of the column's new location.
	 * @exception IllegalArgumentException If <var>columnIndex</var> or
	 * <var>newIndex</var> are not in the valid range
	 */
	public void moveColumn(int columnIndex, int newIndex);

	/**
	 * Check to see if the column at <var>columnIndex</var> can be moved to
	 * <var>newIndex</var>.
	 * @param columnIndex The index of the column to be moved.
	 * @param newIndex The index of the column's new location.
	 * @return true if the column can be moved to the new location
	 */
	public boolean isColumnMoveAllowed(int columnIndex, int newIndex);
}