package com.sas.table;

import com.sas.util.SortDirectionInterface;

/**
 * Interface for implementing sorting on models for tables.
 * @author sasccf Chad Ferguson
 * @date 20Apr2001
 */

public interface SortableInterface extends SortDirectionInterface
{
	/**
	 * Each <var>columnName</var> must have a matching <var>direction</var>. If not, then an
	 * IllegalArgumentException will be thrown.
	 * @param columnNames The names of the columns in the order that the columns will be sorted.
	 * @param direction The direction in which to sort the columns. Valid values are
	 * ASCENDING and DESCENDING.
	 */
	public void sort(String columnNames[], int direction[]) throws SortException;

	/**
	 * Returns a boolean indicating whether the columnName is allowed to be sorted.
	 * The default is that all columns are allowed to be sorted.
	 * @param columnName The name of the column to query about whether it is allowed to be
	 * sorted.
	 * @return A boolean indicating whether the column is allowed to be sorted.
	 */
	public boolean isColumnSortable(String columnName);

	/**
	 * Returns a boolean indicating whether sorting is allowed.  Sorting is allowed by default.
	 * @return A boolean indicating whether sorting is allowed.
	 */
	public boolean isSortAllowed();

	/**
	 * Sets whether sorting is allowed.
	 * @param allowed true if sorting is allowed.
	 */
	public void setSortAllowed(boolean allowed);

	/**
	 * Returns an array of string that are the column names that have been sorted.
	 * The <var>sortedColumns</var> array is updated after each call to the sort method.
	 * The length of the array can be from 0 to the number of available columns. An array
	 * of length 0 means that no columns are currently sorted.
	 * @return An array of strings that are the column names on which the data is sorted.
	 */
	public String[] getSortedColumns();

	/**
	 * Returns an array of ints that indicate which direction each column in the
	 * <var>sortedColumns</var> array has been sorted. The array will be the same size of the
	 * <var>sortedColumns</var> array. The <var>sortedDirections</var> array is update
	 * after each call to the sort method.
	 * @return An array of ints that indicate which direction the sorted columns have been
	 * sorted. Valid values are ASCENDING or DESCENDING.
	 */
	public int[] getSortedDirections();
}