// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.table;

 /**
  * Defines an interface for accessing additional information associated
  * with a table's columns.
  * This will usually be implemented in tandem with the primary interface to 
  * tables, <A HREF="StaticTableInterface.html">StaticTableInterface</A>.
  * @author sasdqr Diane Baucom, sasdjb David Biesack, saskjp Ken Peressini
  * @version 1.0
  * @history
  * <dl>
  * <dt>Wed December 17, 1997  <dd>Created (sasdqr).
  * </dl>
  * @see com.sas.table.StaticTableInterface
  * @see com.sas.table.StaticColumnLabelInterface
  * @see com.sas.table.StaticRowInfoInterface
  */

 public interface StaticColumnInfoInterface
    {

    /**
     * Returns an array of the named information that can be returned for this column.
	 * The returned array should be in priority order.  
     * @return
     *    The prioritized array of information names.
     * @exception TableException 
     *    Provides a catch-all for all non-RuntimeExceptions.
     * @exception IndexOutOfBoundsException 
     *    Thrown if index is negative, zero, or greater than the number of 
     *    labels.
     */
    public String[] getColumnInfoNames () throws TableException;


    /**
     * Returns the named information for a column.  The column is identified by the 
     * given index.  
     * @param index 
     *    A one-based index that identifies the column.
	 * @param columnInfoName
	 * 	  The name of the information to return for the column.
     * @return
     *    The specified column's information.
     * @exception TableException 
     *    Provides a catch-all for all non-RuntimeExceptions.
     * @exception IndexOutOfBoundsException 
     *    Thrown if index is negative, zero, or greater than the number of 
     *    labels.
     */
	public Object getColumnInfo (int index, String columnInfoName) throws TableException;
    
    }

