// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

 package com.sas.table;

 /**
  * Conveys exceptions that occur when invoking the methods defined by the
  * StaticTableInterface-family of interfaces.
  * Since an implementation of these interfaces can generate various types of 
  * exceptions (e.g. remote exceptions, I/O exceptions, resource exceptions), a 
  * TableException is used to trap all these variants.
  * @author saskjp Ken Peressini
  * @version 1.0
  * @since 2.0 Added a default class constant for an error code. Applications throwing
  * this exception can set the errorCode to provide more information about why
  * the exception is occuring. 10/2000, Todd Folsom.
  * <dl>
  * <dt>Fri September 5 1997 <dd>Created.
  * </dl>
  * @see com.sas.table.StaticTableInterface
  */

 public class TableException extends Exception
    {
		/** default value for error code */
		public static final int ERROR_CODE_DEFAULT = 0;
		private int errorCode;

	public TableException ()
       {
	   super ();
	   }
    
	public TableException (String s)
       {
	   super (s);
	   }
       
	public TableException (Exception ex)
	{
		super (ex.toString());
	}

/** Returns the error code set by the application throwing the exception.
 *@return the error code
 */
	public int getErrorCode()
	{
		return errorCode;
	}

/** This method sets the error code to any int.
 *@param errorCode	One of the class constants for error code
 */
	public void setErrorCode (int errorCode)
	{
		this.errorCode = errorCode;
    }
}