// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

/**
 * Format and print a message.
 * <p>
 * This class merely provides a convenience
 * for creating a java.text.MessageFormat object and formatting the
 * message and printing to System.out or to a PrintWriter. It is not very efficient to do this,
 * since it creates a new MessageFormat object each time, which
 * is immediately discarded; this can create a lot of garbage
 * if abused.
 * 
 * @see com.sas.text.Message
 * @author <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @version 1.0
 *
 * <!--
 * @support sasdjb
 * @history Created                                    sasdjb 30Mar98
 * -->
 */

package com.sas.text;
import java.io.PrintWriter;

public final class Format
{
  // private constructor so no instances may be created.
  private Format()
  {
  }

  /**
   * Format and print a Message by applying the format directives
   * in the unformatted text to the format args.
   * The existing format args in the object, if any,
   * are replaced with the new args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   */
  public static void print(PrintWriter writer, String pattern, Object arg0)
  {
    writer.print((new Message(pattern, arg0)));
  }

  /**
   * Format and print a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   */
  public static void print(PrintWriter writer, String pattern, Object arg0, Object arg1)
  {
    writer.print((new Message(pattern, arg0, arg1)));
  }

  /**
   * Format and print a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   */
  public static void print(PrintWriter writer, String pattern, Object arg0, Object arg1, Object arg2)
  {
    writer.print((new Message(pattern, arg0, arg1, arg2)));
  }

  /**
   * Format and print a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   * @param arg3 Next value to be formatted with the message.
   */
  public static void print(PrintWriter writer, String pattern,
                       Object arg0, Object arg1, Object arg2, Object arg3)
  {
    writer.print((new Message(pattern, arg0, arg1, arg2, arg3)));
  }


  /**
   * Format and print a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   * @param arg3 Next value to be formatted with the message.
   * @param arg4 Next value to be formatted with the message.
   */
  public static void print(PrintWriter writer, String pattern,
                       Object arg0, Object arg1, Object arg2, Object arg3, Object arg4)
  {
    writer.print((new Message(pattern, arg0, arg1, arg2, arg3, arg4)));
  }


  /**
   * Format and print a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   * @param arg3 Next value to be formatted with the message.
   * @param arg4 Next value to be formatted with the message.
   * @param arg5 Next value to be formatted with the message.
   */
  public static void print(PrintWriter writer, String pattern,
                       Object arg0, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5)
  {
    writer.print((new Message(pattern, arg0, arg1, arg2, arg3, arg4, arg5)));
  }

  /**
   * Format and print (with a trailing newline) a Message by applying the format directives
   * in the unformatted text to the format args.
   * The existing format args in the object, if any,
   * are replaced with the new args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   */
  public static void println(PrintWriter writer, String pattern, Object arg0)
  {
    writer.println((new Message(pattern, arg0)));
  }

  /**
   * Format and print (with a trailing newline) a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   */
  public static void println(PrintWriter writer, String pattern, Object arg0, Object arg1)
  {
    writer.println((new Message(pattern, arg0, arg1)));
  }

  /**
   * Format and print (with a trailing newline) a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   */
  public static void println(PrintWriter writer, String pattern, Object arg0, Object arg1, Object arg2)
  {
    writer.println((new Message(pattern, arg0, arg1, arg2)));
  }

  /**
   * Format and print (with a trailing newline) a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   * @param arg3 Next value to be formatted with the message.
   */
  public static void println(PrintWriter writer, String pattern,
                       Object arg0, Object arg1, Object arg2, Object arg3)
  {
    writer.println((new Message(pattern, arg0, arg1, arg2, arg3)));
  }


  /**
   * Format and print (with a trailing newline) a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   * @param arg3 Next value to be formatted with the message.
   * @param arg4 Next value to be formatted with the message.
   */
  public static void println(PrintWriter writer, String pattern,
                       Object arg0, Object arg1, Object arg2, Object arg3, Object arg4)
  {
    writer.println((new Message(pattern, arg0, arg1, arg2, arg3, arg4)));
  }


  /**
   * Format and print (with a trailing newline) a Message by applying the format directives
   * in the unformatted text to the format args.
   * @param writer the PrintWriter where the message should be printed
   * @param pattern the new raw format string
   * @param arg0 First value to be formatted with the message.
   * @param arg1 Next value to be formatted with the message.
   * @param arg2 Next value to be formatted with the message.
   * @param arg3 Next value to be formatted with the message.
   * @param arg4 Next value to be formatted with the message.
   * @param arg5 Next value to be formatted with the message.
   */
  public static void println(PrintWriter writer, String pattern,
                       Object arg0, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5)
  {
    writer.println((new Message(pattern, arg0, arg1, arg2, arg3, arg4, arg5)));
  }


}
