// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.tools;

import java.beans.*;
import java.lang.reflect.Method;

/**
 * Introspect a class, printing the results to System.out.
 */
public class Introspect
{
  boolean useSASIntrospector = true;
  Class theClass = null;
  String indent = "";

  void indent()
    {
      indent += "  ";
    }
  void dedent()
    {
      indent = indent.substring(2);
    }

  /**
   * Command line interpreter.
   * Recognized command line arguments:
   * <pre>
   * -sas       use com.sas.beans.Introspector
   * -beans     use java.beans.Introspector
   * </pre>
   */
  public static void main(String argv[])
  {
    boolean useSAS = false;
    BeanInfo beanInfo;
    for (int argc = 0; argc < argv.length; argc++)
      {
        if ("-sas".equals(argv[argc]))
          useSAS = true;
        else if ("-beans".equals(argv[argc]))
          useSAS = false;
        else
		{
			System.out.println( "Introspecting : " + argv[argc] );
          new Introspect(argv[argc], useSAS).introspect();
		}
      }
  }

  public Introspect(String className, boolean useSASIntrospector)
    {
      try
        {
          this.useSASIntrospector = useSASIntrospector;
          this.theClass = Class.forName(className);
        }
      catch (Exception e)
        {
          System.out.println("Introspect(String, boolean) exception:"  + e.getMessage());
          System.out.println(e.getMessage());
          e.printStackTrace();
        }
    }

  void introspect()
    {
      BeanInfo beanInfo;
      try
        {
          if (useSASIntrospector)
            {
              System.out.println("use com.sas.beans.Introspector");
              beanInfo = com.sas.beans.Introspector.getBeanInfo(theClass);
            }
          else
            {
              System.out.println("use java.beans.Introspector");
              beanInfo = java.beans.Introspector.getBeanInfo(theClass);
            }
          print(beanInfo);
        }
      catch (Exception e)
        {
          System.out.println("introspect() exception:"  + e.getMessage());
          e.printStackTrace();
        }
    }
  void print(BeanInfo info)
    {
      try
        {
          print(info.getBeanDescriptor());
          indent();
          PropertyDescriptor pda[] = info.getPropertyDescriptors();
          for (int i = 0; i < pda.length; i++)
            print(pda[i]);
          EventSetDescriptor esds[] = info.getEventSetDescriptors();
          for (int i = 0; i < esds.length; i++)
            print(esds[i]);
        }
      catch (Exception e)
        {
			System.out.println("print(BeanInfo) exception:"  + e.getMessage());
			System.err.println(e);
		}
      finally
        {
          dedent();
        }
    }

  void print(BeanDescriptor bd)
    {
      // System.out.println(indent + bd);
      print("bean: ", bd.getBeanClass());
      print("customizer: ", bd.getCustomizerClass());
    }

  void print(EventSetDescriptor esd)
    {
      // System.out.println(indent + esd);
      print("listener type: ", esd.getListenerType());
      indent();
      print(esd.getAddListenerMethod());
      print(esd.getListenerMethodDescriptors());
      print(esd.getListenerMethods());
      print(esd.getRemoveListenerMethod());
      System.out.println(indent + "is in default event set: " + esd.isInDefaultEventSet());
      System.out.println(indent + "is Unicast: " + esd.isUnicast());
      dedent();
    }

  void print(Method methods[])
    {
      indent();
      for (int i = 0; i < methods.length; i++)
        print(methods[i]);
      dedent();
    }

  void print(MethodDescriptor methods[])
    {
      indent();
      for (int i = 0; i < methods.length; i++)
        print(methods[i]);
      dedent();
    }

  void print(Method method)
    {
      System.out.println(indent + "method " + method);
    }

  void print(MethodDescriptor method)
    {
      print(method.getMethod());
    }

  void print(String label, Class aClass)
    {
      System.out.println(indent + label + aClass);
    }

  void print(PropertyDescriptor pd)
    {
      // System.out.println(indent + pd);
      String name = pd.getName();
      Class type = pd.getPropertyType();
      Method readMethod = pd.getReadMethod();
      Method writeMethod = pd.getWriteMethod();
      PropertyEditor pe = null;
	  if ( type != null )
		  pe = PropertyEditorManager.findEditor(type);
      Class propertyEditor = pe == null ? null : pe.getClass();
      String bound = pd.isBound() ? "bound " : "unbound ";
      String constrained = pd.isConstrained() ? "constrained " : "unconstrained ";
      System.out.println(indent + "Property  : " + bound + constrained + type + " " + name);
      System.out.println(indent + "   Read   : " + readMethod);
      System.out.println(indent + "   Write  : " + writeMethod);
      System.out.println(indent + "   Editor : " + (  propertyEditor == null
                                                    ? "none"
                                                    : propertyEditor.getName()));
      System.out.println(indent + "   Hidden : " + pd.isHidden());
      System.out.println(indent + "   Expert : " + pd.isExpert());
    }
}

