package com.sas.util;

/**
* AbstractCommand is an abstract Command that is executable
* and only requires the execute() method to be overridden.
**/
public abstract class AbstractCommand
	implements com.sas.util.Command,
			   java.io.Serializable
{
	static final long serialVersionUID = 2484208964316308902L;
	private String text;

	/**
	* Default Constructor.
	**/
	public AbstractCommand()
	{
	}

	public abstract Object clone() throws CloneNotSupportedException;

	/**
	* Return true, this Command is executable.
	* @return true, this Command is executable
	**/
	public boolean isExecutable()
	{
		return true;
	}

	/**
	* Return false, this Command is not Undoable.
	* @return false, this Command is not Undoable
	**/
	public boolean isUndoable()
	{
		return false;
	}

	/**
	* This does nothing.
	**/
	public void undo()
	{
	}

	/**
	* This does nothing.
	**/
	public abstract void execute( Object arg );

	/**
	* Return the text.
	* @return text
	**/
	public String getText()
	{
		return text;
	}

	/**
	* Set the text of the AbstractCommand.
	* @param txt the text 
	**/
	public void setText( String txt )
	{
		text = txt;
	}
}