package com.sas.util;

import com.sas.util.transforms.ObjectToStringTransform;
import com.sas.MissingValues;

/**
 * @author SDKNBJ
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public abstract class AbstractCondition {

	public static final int OPERATOR_EQ = 1;

	public static final int OPERATOR_GT = 2;

	public static final int OPERATOR_GE = 3;

	public static final int OPERATOR_LT = 4;

	public static final int OPERATOR_LE = 5;

	public static final int OPERATOR_NE = 6;

	public static final int DATATYPE_NUMERIC = 1;

	public static final int DATATYPE_CHARACTER = 2;

	private Object _identifier = null;

	private int _operator = 0;

	private String _leftExpression = null;

	private String _rightExpression = null;

	/**
	 * Returns the identifier.
	 * @return String
	 */
	public Object getIdentifier() {
		return _identifier;
	}

	/**
	 * Returns the leftExpression.
	 * @return String
	 */
	public String getLeftExpression() {
		return _leftExpression;
	}

	/**
	 * Returns the operator.
	 * @return int
	 */
	public int getOperator() {
		return _operator;
	}

	/**
	 * Returns the rightExpression.
	 * @return String
	 */
	public String getRightExpression() {
		return _rightExpression;
	}

	/**
	 * Sets the identifier.
	 * @param identifier The identifier to set
	 */
	public void setIdentifier(Object identifier) {
		_identifier = identifier;
	}

	/**
	 * Sets the leftExpression.
	 * @param leftExpression The leftExpression to set
	 */
	public void setLeftExpression(String leftExpression) {
		_leftExpression = leftExpression;
	}

	/**
	 * Sets the operator.
	 * @param operator The operator to set
	 */
	public void setOperator(int operator) {
		_operator = operator;
	}

	/**
	 * Sets the rightExpression.
	 * @param rightExpression The rightExpression to set
	 */
	public void setRightExpression(String rightExpression) {
		_rightExpression = rightExpression;
	}

	private boolean isNumeric(Object value) {
		boolean numeric = true;
		
		if (value instanceof String) {
			try {
				String s = (String)ObjectToStringTransform.defaultInstance.transform(value);
				double testConvert = Double.parseDouble(s);
			} catch (NumberFormatException e) {
				numeric = false;
			}
		}
		
		return numeric;
	}
	
	/**
	 * Method evaluateCondition.
	 * This method does the actual evaluation, and should be called from any
	 * implementers to figure out if the condition evaluated true or false
	 * @param conditionEvaluationInterface
	 * @param values the row values needed for the evaluation.
	 * @return boolean whether the condition evaluated true or false.
	 */
	protected boolean evaluateCondition(ConditionEvaluationInterface conditionEvaluationInterface, Object[] values) {
		boolean applyFormatting = true;

		if ((_leftExpression != null) && (_rightExpression != null)) {
			
			Comparable lside = null;
			Comparable rside = null;
			boolean lside_MissingValue = false;
			boolean rside_MissingValue = false;

			int columnIndex = conditionEvaluationInterface.getColumnIndex(getLeftExpression());
			int columnType = conditionEvaluationInterface.getColumnType(getLeftExpression());

			Object value = values[columnIndex];

			try {
				if (columnType == DATATYPE_NUMERIC) {
					applyFormatting = isNumeric(value);
				}
				
				if (applyFormatting) {
					if (columnType == DATATYPE_NUMERIC) {
						if (getRightExpression().equals(".")) {
							rside_MissingValue = true;
						}
						else {
							rside = Double.valueOf(getRightExpression());
						}

						if (value instanceof MissingValues) {
							lside_MissingValue = true;
						}
						else {
							lside = Double.valueOf((String)ObjectToStringTransform.defaultInstance.transform(value));
						}
					} else if (columnType == DATATYPE_CHARACTER) {
						rside = getRightExpression();
						lside = (String)ObjectToStringTransform.defaultInstance.transform(value);
					}
				}
			}
			catch (ClassCastException e)
			{
//				e.printStackTrace();
				applyFormatting = false;
			}
			catch (NullPointerException e)
			{
//				e.printStackTrace();
				applyFormatting = false;
			}
			catch (NumberFormatException e)
			{
//				e.printStackTrace();
				applyFormatting = false;
			}

			if (applyFormatting) {
				if (lside != null && rside != null) {
					switch (getOperator()) {
						case OPERATOR_EQ:
							applyFormatting = (lside.compareTo(rside) == 0);
							break;
						case OPERATOR_GT:
							applyFormatting = (lside.compareTo(rside) > 0);
							break;
						case OPERATOR_GE:
							applyFormatting = (lside.compareTo(rside) >= 0);
							break;
						case OPERATOR_LT:
							applyFormatting = (lside.compareTo(rside) < 0);
							break;
						case OPERATOR_LE:
							applyFormatting = (lside.compareTo(rside) <= 0);
							break;
						case OPERATOR_NE:
							applyFormatting = (lside.compareTo(rside) != 0);
							break;
						default:
							applyFormatting = false;
							break;
					}
				}
				else if (rside_MissingValue && lside_MissingValue)
				{
					boolean equals = false;
					if ((columnType == DATATYPE_NUMERIC) &&
						(MissingValues.Dot.equals(value)))
					{
						equals = true;
					}

					switch (getOperator())
					{
						case OPERATOR_EQ:
						case OPERATOR_GE:
						case OPERATOR_LE:
							applyFormatting = equals;
							break;
						case OPERATOR_NE:
							applyFormatting = !equals;
							break;
						default:
							applyFormatting = false;
							break;
					}
				}
				else {
					applyFormatting = false;
				}
			}
		}
		else {
			applyFormatting = false;
		}

		return applyFormatting;
	}

	/**
	 * Method evaluate.
	 * This method evaluates the condition and returns the formatting to be
	 * used.
	 * @param conditionEvaluationInterface
	 * @param values the row values which is needed for the condition evaluation
	 * @return Object the returned formatting, or null if the condition
	 * evaluated false.
	 */
	public abstract Object evaluate(ConditionEvaluationInterface conditionEvaluationInterface, Object[] values);
}
