/* Copyright (c)1999 SAS Institute Inc, Cary NC USA 27513
 * Unpublished- All Rights Reserved.
 */
package com.sas.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
//
// History:
//
// Date    Name            Remarks
// 24Aug99 G Williams      Created.
// 27Feb02 B Beers         S0141718: Move to sas.core.jar.
//
/**
 * @author Gary Williams
 */
public class BASE64
{
    private final static int MAXLINE = 76;
    private final static char b64_array[] = {
        'A','B','C','D','E','F','G','H',
        'I','J','K','L','M','N','O','P',
        'Q','R','S','T','U','V','W','X',
        'Y','Z','a','b','c','d','e','f',
        'g','h','i','j','k','l','m','n',
        'o','p','q','r','s','t','u','v',
        'w','x','y','z','0','1','2','3',
        '4','5','6','7','8','9','+','/'};


    //  Take three bytes from the array, then turn them
    //  into four bytes by looking up the value in the
    //  array.    
    public static String encode( byte[] b )
    {
        
        StringBuffer out = new StringBuffer();
        int i;
        byte b1, b2, b3;
        
        if ( (b == null) || (b.length == 0) ) return null;
        
        for ( i = 0; i < b.length-2; i+=3 )
        {
            b1 = b[i];
            b2 = b[i+1];
            b3 = b[i+2];
            out.append((char)b64_array[(b1 >>> 2) & 0x3F]);
            out.append((char)b64_array[((b1 << 4) & 0x30) + ((b2 >>> 4) & 0xf)]);
            out.append((char)b64_array[((b2 << 2) & 0x3c) + ((b3 >>> 6) & 0x3)]);
            out.append((char)b64_array[b3 & 0x3F]);
        }
        if ( i == b.length - 2 )
        {
            b1 = b[i];
            b2 = b[i+1];
            b3 = 0;
            out.append((char)(b64_array[(b1 >>> 2) & 0x3F]));
            out.append((char)(b64_array[((b1 << 4) & 0x30) + ((b2 >>> 4) & 0xf)]));
            out.append((char)(b64_array[((b2 << 2) & 0x3c) + ((b3 >>> 6) & 0x3)]));
            out.append((char)'=');
        }
        if ( i == b.length - 1 )
        {
            b1 = b[i];
            b2 = 0;
            b3 = 0;
            out.append((char)(b64_array[(b1 >>> 2) & 0x3F]));
            out.append((char)(b64_array[((b1 << 4) & 0x30) + ((b2 >>> 4) & 0xf)]));
            out.append((char)'=');
            out.append((char)'=');
        }
        return new String( out );
    }
    
    // Take four bytes from the string, and turn them into three
    // bytes.
    public static byte[] decode( String in )
    {
        StringReader sr = new StringReader( in );
        ByteArrayOutputStream bo = new ByteArrayOutputStream();
        byte[] out = null;
        int i;
        
        for( ; ; )
        {
            char a, b, c, d;
            int ai=-1, bi=-1, ci=-1, di=-1;
            byte b1, b2,b3;
            // If we get a -1 here, it means the end of the stream on an
            // even boundary.  Break and return the byte array.
            a = nextChar( sr );
            if ( a == (char)-1 ) break;
            // a -1 return on any of the others is an error
            b = nextChar( sr );
            if ( b == (char)-1 ) return null;
            c = nextChar( sr );
            if ( c == (char)-1 ) return null;
            d = nextChar( sr );
            if ( d == (char)-1 ) return null;
            for ( i = 0; i < 64; i++ )
            {
                if ( a == b64_array[i] ) ai = i;
                if ( b == b64_array[i] ) bi = i;
                if ( c == b64_array[i] ) ci = i;
                if ( d == b64_array[i] ) di = i;
            }
            if ( ((ai == -1) && (a != '=')) || 
                 ((bi == -1) && (b != '=')) ||
                 ((ci == -1) && (c != '=')) || 
                 ((di == -1) && (d != '=')) )
            {
                return null;
            }

            b1 = ( (byte)(((ai << 2) & 0xfc) | ((bi >>> 4) & 3)) );
            bo.write( b1 );
            if ( c != '=' )
            {
                b2 = ( (byte)(((bi << 4) & 0xf0) | ((ci >>> 2) & 0xf )) );
                bo.write ( b2 );
                if ( d != '=' )
                {
                    b3 = ( (byte)(((ci << 6) & 0xc0) | (di & 0x3f)) );
                    bo.write( b3 );
                }
            }
            else
                break;
        }
        return bo.toByteArray();
    }
    
    private static char nextChar( StringReader sr )
    {
        try
        {
            char c = (char)sr.read();
            while ( (c == '\n') || (c == ' ') || (c == '\t') || (c == '\r') )
            {
                try{
                c = (char)sr.read();
                }
                catch( IOException ex )
                { return (char)-1; }
            }
            return c;
        }
        catch( IOException ex )
        {  return (char)-1; }
    }
}