package com.sas.util;

/**
 * An interface for exceptions which
 * "wrap" other other exceptions.  For example, a high level
 * interface such as the com.sas.table classes may contain
 * methods which throw TableException. An implementation of
 * the various table interfaces may be remote and the low
 * level implementation may have to catch checked exceptions
 * such as java.rmi.RemoteException, then "wrap" that
 * RemoteException in a TableException. The wrapped exception
 * may be unchecked exceptions if desired.
 * <p>
 * The exception chain is actually a chain of {@link Throwable}s
 * but the vast majority of Throwables which are caught and
 * rethrown as other exceptions derive from {@link Exception}.
 * (Java guidelines recommend not catching {@link Error}s or
 * other non-Exception throwables.) So, while this API
 * supports the chained or target exception being a Throwable,
 * the method is still named <code>getTargetException</code>
 * based on the method of the same name in {@link java.lang.reflect.InvocationTargetException}.
 *
 * @see ChainedException
 * @see UncheckedException
 * @author sasdjb <A HREF="http://sww.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @version 1.0
 */
public interface ChainedExceptionInterface {

  /**
   * Return the Exception (or Throwable) with which this ChainedExceptionInterface
   * was constructed.
   * Note that JDK 1.4 provides a <code>Throwable getCause()</code> method which
   * provides the same behavior as this method, but the <code>getCause()</code> method
   * does not exist prior to 1.4.
   */
  Throwable getTargetException();
}
