package com.sas.util;

/**
 * Implemented by viewers which will allow Conditional Highlighting. The methods in
 * this interface allow a way to set the class which will evaulate the all conditions
 * that are applied to the data model of the viewer.
 * 
 * @see com.sas.util.ConditionEvaluationInterface
 * 
 * @author sasccf
 */
public interface ConditionalHighlightingViewInterface
{
	/**
	 * Sets the class used to evaluate the conditions set on the viewer's data
	 * model. The class must implement com.sas.util.ConditionEvaluationInterface.
	 * @param evaluator the instance of com.sas.util.ConditionEvaluationInterface
	 * that is used to evaluate the conditions on the viewer's data model
	 */
	public void setConditionEvaluator(ConditionEvaluationInterface evaluator);

	/**
	 * Gets the class used to evaluate the conditions set on the viewer's data
	 * model.
	 * @return the instance of com.sas.util.ConditionEvaluationInterface
	 * that is used to evaluate the conditions on the viewer's data model
	 */
	public ConditionEvaluationInterface getConditionEvaluator();
}