// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.util;

import com.sas.util.Comparator;

/**
 * A default <code>Comparator</code> which always returns a constant
 * value and ignores the values.
 * <p>
 * The value {@link #COMPARE_EQUALS <code>ConstantComparator.COMPARE_EQUALS</code>}
 * always return {@link com.sas.util.Comparator#EQUALS}
 * when comparing two items.
 * (Note that creating a comparator
 * which always returns
 * {@link com.sas.util.Comparator#LESS_THAN <code>Comparator.LESS_THAN</code>}
 * or always returns
 * {@link com.sas.util.Comparator#GREATER_THAN <code>Comparator.GREATER_THAN</code>}
 * is probably
 * a dangerous thing to do; it may result in non-terminating sorting algorithms.)
 *
 * @author <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @version 1.0
 *
 */

public class ConstantComparator
	implements Comparator,
			   java.io.Serializable,
			   java.util.Comparator
{
	private int result;

	/** A Comparator which always returns EQUALS */
	public static Comparator COMPARE_EQUALS = new ConstantComparator(EQUALS);

	/**
	 * Construct a new <code>ConstantComparator</code>
	 */
	public ConstantComparator ()
	{
		result = EQUALS;
	}

	/**
	 * Construct a new <code>ConstantComparator</code> which always returns
	 * <var>result</var>
	 * @param result the valeu to return from the {@link #compare
	 * <code>compare</code>} method.
	 */
	public ConstantComparator (int result)
	{
		this.result = result;
	}

	/**
	 * Get the constant value this comparator returns
	 * @return the value used when this comparator was constructed.
	 */
	public int getCompareResult()
	{
		return result;
	}

	/**
	 * Compare two values.
	 * This method does not really compare the values,
	 * but returns the value used when this comparator was constructed.
	 * @param ignored1 a value that is ignored.
	 * @param ignored2 a value that is ignored.
	 * @return the value used when this comparator was constructed.
	 */
	public int compare(Object ignored1, Object ignored2)
	{
		return result;
	}
}
