package com.sas.util;

import com.sas.MissingValues;

public class MissingValueRangeSegment extends com.sas.util.RangeSegment
{
	public static final String RB_KEY = "MissingValueRangeSegment.";

	private MissingValues min;
	private MissingValues max;

	public MissingValueRangeSegment()
	{
		this((MissingValues)null,(MissingValues)null,false,false,null);
	}

	public MissingValueRangeSegment(String min, String max)
	{
		this();

		int length = min.length();
		if(length == 1){
			this.min = MissingValues.get("Dot");
		} else if(length == 2){
			this.min = MissingValues.get(String.valueOf(min.charAt(1)).toUpperCase()/*I18nOK:MTH*/);
		}

		length = max.length();
		if(length == 1){
			this.max = MissingValues.get("Dot");
		} else if(length == 2){
			this.max = MissingValues.get(String.valueOf(min.charAt(1)).toUpperCase()/*I18nOK:MTH*/);
		} else {
			if(this.min != null)
				this.max = this.min;
		}

		if(this.min == null && this.max != null)
			this.min = this.max;
	}

	public MissingValueRangeSegment(String min, String max, boolean minExcl, boolean maxExcl)
	{
		this(min,max,minExcl,maxExcl,null);
	}

	public MissingValueRangeSegment(String min, String max, boolean minExcl, boolean maxExcl, Object value)
	{
		this(min,max);

		setValue(value);
		setMinExclude(minExcl);
		setMaxExclude(maxExcl);
	}

	public MissingValueRangeSegment(MissingValues min, MissingValues max)
	{
		this(min,max,false,false,null);
	}

	public MissingValueRangeSegment(MissingValues min, MissingValues max, boolean minExcl, boolean maxExcl)
	{
		this(min,max,minExcl,maxExcl,null);
	}

	public MissingValueRangeSegment(MissingValues min, MissingValues max, boolean minExcl, boolean maxExcl, Object value)
	{
		super();

		this.min = min;
		this.max = max;

		setValue(value);
		setMinExclude(minExcl);
		setMaxExclude(maxExcl);
	}

	public void setMin(MissingValues min)
	{
		this.min = min;
	}

	public void setMax(MissingValues max)
	{
		this.max = max;
	}

	public MissingValues getMin()
	{
		return min;
	}

	public MissingValues getMax()
	{
		return max;
	}

	/**
	 * Returns 0 if the value is in the range, -1 if the value is
	 * less then min, and 1 if the value is greater then max.
	 */
	public int inRange(Object value)
	{
		if(value != null)
		{
			if(value instanceof com.sas.MissingValues)
				return inRange((MissingValues) value);
			else
				throw new RuntimeException(RB.getStringResource(RB_KEY, "invalidType.txt"));
		}

		return -2;
	}

    public int inRange(MissingValues value)
    {
        int result = 0;
        int vIndex = value.getIndex();

        if(vIndex < min.getIndex() || (minExclude && vIndex == min.getIndex()))
            result = -1;
        else if(vIndex > max.getIndex() || (maxExclude && vIndex == max.getIndex()))
            result =  1;

        return result;
    }
}