package com.sas.util;

/**
 * @author Salman Maher
 */

public class NumericRangeSegment extends com.sas.util.RangeSegment
{
	public static final String RB_KEY = "NumericRangeSegment.";

	private double min;
	private double max;
	
	public NumericRangeSegment()
	{
		this(Double.NEGATIVE_INFINITY,Double.POSITIVE_INFINITY,false,true,null);
	}
	
	public NumericRangeSegment(double min, double max)
	{
		this(min,max,false,true,null);		
	}
	
	public NumericRangeSegment(double min, double max, Object value)
	{
		this(min,max,false,true,value);		
	}
	
	public NumericRangeSegment(double min, double max, boolean minExcl, boolean maxExcl, Object value)
	{
		super();
		
		this.min = min;
		this.max = max;
		
		setValue(value);
		setMinExclude(minExcl);
		setMaxExclude(maxExcl);		
	}
	
	public void setMin(double min)
	{
		this.min = min;
	}
	
	public void setMax(double max)
	{
		this.max = max;
	}
	
	public double getMin()
	{
		return min;
	}
	
	public double getMax()
	{
		return max;
	}
	
	/**
	 * Returns 0 if the value is in the range, -1 if the value is
	 * less then min, and 1 if the value is greater then max. If the
	 * value is null or Double.NaN a -2 is returned.
	 */
	public int inRange(Object value)
	{
		if(value != null)
		{
			if(value instanceof Double)
				return inRange(((Double)value).doubleValue());
			else
				throw new RuntimeException(RB.getStringResource(RB_KEY, "invalidInputType.txt"));
		}
		else
		{
			return -2;
		}
	}
	
	/**
	 * Returns 0 if the value is in the range, -1 if the value is
	 * less then min, and 1 if the value is greater then max. If the
	 * value is equal to Double.NaN a -2 is returned.
	 */
    public int inRange(double value)
    {
        int result = 0;
        
        if(Double.isNaN(value))
            result = -2;
        else if(value < min || (minExclude && value == min))
            result = -1;
        else if(value > max || (maxExclude && value == max))
            result =  1;

        return result;
    }
    
    public String toString()
    {
    	return (String.valueOf(min) + (minExclude? "<":"") +
    	        "-" + (maxExclude? "<":"") + String.valueOf(max));
    }
    
    public String toString(java.text.Format format)
    {
    	return (format.format(new Double(min)) + (minExclude? "<":"") +
    	       "-" + (maxExclude? "<":"") + format.format(new Double(max)));    	
    }
}