/* Copyright (c) 2001 by SAS */

package com.sas.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

import com.sas.text.Message;

public class OrderedMap implements Map
{
	List keyList;
	List valueList;
 
	public OrderedMap()
	{
		keyList = new ArrayList();
		valueList = new ArrayList();
	}

	public OrderedMap( Class modelListClass ) throws InstantiationException, IllegalAccessException
	{
		keyList = (List) modelListClass.newInstance();
		valueList = (List) modelListClass.newInstance();
	}

	//Defined in java.util.Map
	public int size( )
	{
		return keyList.size();
	}

	//Defined in java.util.Map
	public boolean isEmpty( )
	{
		return keyList.isEmpty();
	}

	//Defined in java.util.Map
	public boolean containsKey( Object key )
	{
		return keyList.contains( key );
	}

	//Defined in java.util.Map
	public boolean containsValue( Object value )
	{
		return valueList.contains( value );
	}

	//Defined in java.util.Map
	public Object get( Object key )
	{
		int index = keyList.indexOf( key );
		if ( index == -1 )
		{
			return null;
		}
		return valueList.get( index );
	}

	//Defined in java.util.Map
	public Object put(Object key, Object value )
	{
		Object oldValue = get( key );

		if ( containsKey( key ) )
		{
			remove( key );
		}

		keyList.add( key );
		valueList.add( value );

		return oldValue;
	}

	//Defined in java.util.Map
	public Object remove( Object key )
	{
		Object oldValue = get( key );

		int index = keyList.indexOf( key );
		if ( index == -1 )
		{
			return null;
		}
		keyList.remove( index );
		valueList.remove( index );

		return oldValue;
	}

	//Defined in java.util.Map
	public void putAll( Map map )
	{
		Iterator iterator = map.keySet().iterator();
		while ( iterator.hasNext() )
		{
			Object key = iterator.next();
			put( key, map.get( key ) );
		}
	}

	//Defined in java.util.Map
	public void clear( )
	{
		keyList.clear();
		valueList.clear();
	}

	//Defined in java.util.Map
	public Set keySet()
	{
		return new OrderedMapSet( keyList );
	}

	//Defined in java.util.Map
	public Collection values( )
	{
		return new ArrayList( valueList );
	}

	//Defined in java.util.Map
	public Set entrySet( )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "entrySet()"));
	}

	//Defined in java.util.Map
	public boolean equals( Object otherMap )
	{
		return ( hashCode() == otherMap.hashCode() );
	}

	//Defined in java.util.Map
	public int hashCode( )
	{
		return keyList.hashCode() + valueList.hashCode();
	}

	public List getKeyList()
	{
		return keyList;
	}

}

class OrderedMapSet implements Set
{
	protected Collection collection;

	public OrderedMapSet( Collection collection )
	{
		this.collection = collection;
	}

	public int size()
	{
		return collection.size();
	}

	public boolean isEmpty()
	{
		return (size() == 0);
	}

	public Iterator iterator()
	{
		return collection.iterator();
	}

	public boolean add( Object value )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "add(Object)"));
	}

	public boolean addAll( Collection collection )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "addAll(Collection)"));
	}

	public boolean contains( Object value )
	{
		return collection.contains( value );
	}

	public boolean containsAll( Collection collection )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "containsAll(Collection)"));
	}

	public void clear()
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "clear()"));
	}

	public boolean remove( Object value )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "remove(Object)"));
	}

	public boolean removeAll( Collection collection )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "removeAll(Collection)"));
	}

	public boolean retainAll( Collection collection )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "retainAll(Collection)"));
	}

	public Object[] toArray()
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "toArray()"));
	}

	public Object[] toArray( Object[] type )
	{
		throw new IllegalStateException(Message.format(RB.getStringResource("notSupported.txt"), "toArray(Object[])"));
	}
}

class OrderedMapSetIterator implements Iterator
{
	public static final String RB_KEY = "OrderedMapSetIterator.";

	public OrderedMapSetIterator()
	{
	}

	public boolean hasNext()
	{
		return false;
	}

	public Object next()
	{
		return null;
	}

	public void remove()
	{
		throw new UnsupportedOperationException(Message.format(RB.getStringResource(RB_KEY, "iteratorNotSupported.txt"), "remove()"));
	}
}
