package com.sas.util;

/**
 * Base class for the RangeSegment tree.
 * @author Salman Maher
 */

abstract public class RangeSegment extends java.lang.Object
{	
	protected Object value;
	protected boolean minExclude;
	protected boolean maxExclude;
	
	protected RangeSegment()
	{
		value = null;
		minExclude = false;
		maxExclude = true;
	}

	/**
	 * @param trueOrFalse if true min value will be exclude from
	 *                    the range. If false min value will be 
	 *                    included in the range. default false.
	 */	
	public void setMinExclude(boolean trueOrFalse)
	{
		minExclude = trueOrFalse;
	}
	
	/**
	 * @param trueOrFalse if true max value will be exclude from
	 *                    the range. If false max value will be 
	 *                    included in the range. default false.
	 */
	public void setMaxExclude(boolean trueOrFalse)
	{
		maxExclude = trueOrFalse;
	}
	
	/**
	 * Return true if the min value should be exclude from the range. 
	 * Otherwise false
	 */	
	public boolean getMinExclude()
	{
		return minExclude;
	}
	
	/**
	 * Return true if the max value should be exclude from the range. 
	 * Otherwise false
	 */
	public boolean getMaxExclude()
	{
		return maxExclude;
	}
	
	/**
	 * Set the value for this range
	 */
	public void setValue(Object value)
	{
		this.value = value;
	}
	
	/**
	 * Returns the value for this range.
	 */
	public Object getValue(Object value)
	{
		return this.value;
	}	
	
	/**
	 * Returns 0 if the value is in the range, -1 if the value is
	 * less then min, and 1 if the value is greater then max.
	 */
	abstract public int inRange(Object value);
}