// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.util;

/**
 * <code>SimpleCommand</code> is a simple version of an executable command.
 * Objects which implement the SimpleCommand interface provide
 * an <code>execute(Object arg)</code> method. This is useful for encapulating
 * generic behavior. A <code>SimpleCommand</code> may be bound to
 * a user interface element, such as a button or menu item. The UI
 * event, through a simple adaptor, will invoke the command's execute
 * method to carry out the requested behavior. The behavior is coded
 * as a command so that it is independent of the user interface
 * (button, icon, menu selection, or other UI gesture).
 * <p>
 * See <em>Design Patterns: Elements of Reusable Object Oriented Software</em>
 * This <code>SimpleCommand</code> interface is modelled after the Command pattern.
 *
 * @author <A HREF="http://www.unx.sas.com/~sasdjb/Home.html">David J. Biesack</A>
 * @see com.sas.util.Command
 * @see com.sas.util.CommandsInterface
 * @see com.sas.util.ApplyInterface
 */

public interface SimpleCommand
{
  /**
   * Execute the command.
   * @param arg an argument which the caller wishes
   * to pass to the command. The arg may be <code>null</code> if no information
   * is needed.
   */
  public void execute(Object arg);
}
