package com.sas.util;

/**
 * @author Salman Maher
 */
public class StringRangeSegment extends com.sas.util.RangeSegment
{
	public static final String RB_KEY = "StringRangeSegment.";

	private String min;
	private String max;

	public StringRangeSegment()
	{
		this(null,null,false,true,null);
	}

	public StringRangeSegment(String min, String max)
	{
		this(min,max,false,true,null);
	}

	public StringRangeSegment(String min, String max, Object value)
	{
		this(min,max,false,true,value);
	}

	public StringRangeSegment(String min, String max, boolean minExcl, boolean maxExcl, Object value)
	{
		super();

		this.min = min;
		this.max = max;

		setValue(value);
		setMinExclude(minExcl);
		setMaxExclude(maxExcl);
	}

	public void setMin(String min)
	{
		this.min = min;
	}

	public void setMax(String max)
	{
		this.max = max;
	}

	public String getMin()
	{
		return min;
	}

	public String getMax()
	{
		return max;
	}

	/**
	 * Returns 0 if the value is in the range, -1 if the value is
	 * less then min,1 if the value is greater then max, and -2 if value is NULL.
	 */
	public int inRange(Object value)
	{
		if(value != null)
		{
			if(value instanceof String)
				return inRange((String) value);
			else
				throw new RuntimeException(RB.getStringResource(RB_KEY, "invalidType.txt"));
		}

		return -2;
	}

	/**
	 * Returns 0 if the value is in the range, -1 if the value is
	 * less then min, and 1 if the value is greater then max and -2 if value is NULL.
	 */
    public int inRange(String value)
    {
	    int result = 0;

    	if(value != null)
    	{
	        int sctv = (min == null ?  1 : value.compareTo(min));
	        int ectv = (max == null ? -1 : value.compareTo(max));

	        if(sctv < 0 || (minExclude && sctv == 0))
	        	result = -1;
	        else if(ectv > 0 || (maxExclude && ectv == 0))
	        	result =  1;
        }
        else
        {
        	result = -2;
        }

        return result;
    }

    public String toString()
    {
    	return (min + (minExclude? "<":"") + "-" + (maxExclude? "<":"") + max);
    }

    public String toString(java.text.Format format)
    {
    	return (format.format(min) + (minExclude? "<":"") +
    	        "-" + (maxExclude? "<":"") + format.format(max));
    }
}