/* Copyright (c) 2002 by SAS Institute Inc., Cary, NC 27513 */

package com.sas.util.login;

/**
 * The LoginBeanActionInterface defines a set of methods which act as
 * notification of Action events within the LoginBean.
 * Action events consist of notifications that can occur during the lifetime
 * of the login bean, such as OK, Cancel, Close, Success, Failure and Retry attempts.
 * <P>
 * You implement a handler and set it on the login bean so that you'll receive 
 * notification whenever one of these actions occurs.
 * <P>
 * See the LoginBeanSwingUtil class in this same package for an example that
 * illustrates the use of this interface in a typical scenario.
 */
public interface LoginBeanActionInterface
{
	/**
	 * This action occurs when the user presses the OK button on the LoginBean.
	 */
	public void OKAction();

	/**
	 * This action occurs when the user presses the Cancel button on the LoginBean.
	 */
	public void cancelAction();

	/**
	 * This action occurs when the user presses the Close button on the LoginBean.
	 * The Close button is only visible after the failureAction has occured, and
	 * it allows the user to acknowledge the failure while displaying a failure
	 * message.
	 */
	public void closeAction();

	/**
	 * This action occurs when the user has successfully logged on to the system.
	 */
	public void successAction();

	/**
	 * This action occurs when the user has failed to log on to the system.
	 */
	public void failureAction();

	/**
	 * This action occurs when the user has failed one attempt to log on, but has
	 * additional attempts remaining.
	 */
	public void retryAction();
}
