/* Copyright (c) 2002 by SAS Institute Inc., Cary, NC 27513 */

package com.sas.util.login;

/**
 * The LoginBeanValidateInterface defines the interface for performing
 * validation of credentials entered in via the LoginBean.
 * <P>
 * You implement a validation handler and set it on the login bean so 
 * that the LoginBean can perform validation and handle retry attempts
 * and failures accordingly.
 * <P>
 * See the LoginBeanSwingUtil class in this same package for an example that
 * illustrates the use of this interface in a typical scenario.
 */
public interface LoginBeanValidateInterface
{
	/**
	 * Performs validation of user credentials entered in via the LoginBean.
	 * This method is typically called directly by the LoginBean whenever
	 * validation is required.
	 * @param username The user's username.
	 * @param password The user's password.
	 * @return Returns a boolean value representing the success of the validation.
	 */
	public boolean validate(String username, String password);
}
