// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;

/**
 * The BooleanToCharacterTransform converts an instance of Boolean to a Character.
 * By default, if the value of the Boolean is true, the value of the Character is set to 'T'.
 * Otherwise, the value of the Character returned is set to 'F'.
 * <P>
 * A non-default constructor allows the true character and the false character to be
 * set.
 */
public class BooleanToCharacterTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = 256899152294673648L;

	public static final BooleanToCharacterTransform defaultInstance = new BooleanToCharacterTransform();

	public char trueChar;
	public char falseChar;

	public BooleanToCharacterTransform()
	{
		this( 'T', 'F' );
	}

	public BooleanToCharacterTransform( char trueChar, char falseChar )
	{
		this.trueChar = trueChar;
		this.falseChar = falseChar;
	}

	public Object transform( Object value )
	{
		Boolean bool = (Boolean) value;
		if ( bool.booleanValue() == true )
		{
			return new Character( trueChar );
		}
		return new Character( falseChar );
	}

	public Class getInputClass()
	{
		return Boolean.class;
	}

	public Class getOutputClass()
	{
		return Character.class;
	}
}
