// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;

/**
 * The CharacterToBooleanTransform converts an instance of Character to a Boolean.
 * By default, if the value of the Character is 'T', the value of the Boolean is set to true.
 * Otherwise, the value of the Boolean returned is set to false.
 * <P>
 * A non-default constructor allows the true character and the false character to be
 * set.
 */
public class CharacterToBooleanTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = 256899152294673648L;

	public static final CharacterToBooleanTransform defaultInstance = new CharacterToBooleanTransform();

	public char trueChar;
	public char falseChar;

	public CharacterToBooleanTransform()
	{
		this( 'T', 'F' );
	}

	public CharacterToBooleanTransform( char trueChar, char falseChar )
	{
		this.trueChar = trueChar;
		this.falseChar = falseChar;
	}

	public Object transform( Object value )
	{
		Character c = (Character) value;
		if ( c.charValue() == trueChar )
		{
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}

	public Class getInputClass()
	{
		return Character.class;
	}

	public Class getOutputClass()
	{
		return Boolean.class;
	}
}
