/* Copyright (c) 2001 by SAS Institute Inc., Cary, NC 27513 */
package com.sas.util.transforms;

/**
  * Defines an interface to identify objects that use <em>display transform</em>.
  * A transform takes one object and converts it to another object. The display transform
  * is used for transforming object in a model into a different representation in the viewer.
  * @author drande Drew Anderson
  * @version 3.0
  * @see com.sas.util.transforms.TransformInterface
 */
public interface DisplayTransformInterface
{
	/**
	  * Returns the transform associated with <code>this</code>.
	  * @return
	  *    An object that implements <code>TransformInterface</code>.
	  * @see #setDisplayTransform
	 */
	public TransformInterface getDisplayTransform();

	/**
	  * Sets the transform associated with <code>this</code>.
	  * @param resourceLocator
	  *    An object that implements <code>TransformInterface</code>.
	  * @see #getDisplayTransform
	 */
	public void setDisplayTransform (TransformInterface displayTransform);
}