// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;

import com.sas.util.SimpleTemplate;
import java.util.Map;

/**
 * Transforms a Map by by subsituting the Map keys in the specified <code>templateText</code> with their corresponding Map values.
 * The general form of a template is: 
 * <br>
 *  text %VAR more text
 * <br>
 * where VAR is the name of a key variable defined in the Map supplied to the MapToTemplateStringTransform. 
 */
public class MapToTemplateStringTransform implements TransformInputOutputInterface
{
	private String templateText;
	private SimpleTemplate template;

	public MapToTemplateStringTransform(String templateText)
	{
	 	template = new SimpleTemplate();
		this.templateText = templateText;
	}

	public Object transform( Object input )
	{
		template.setValueDictionary((Map)input);
		template.setTemplate(templateText);
		return template.format();
	}

	public Class getInputClass()
	{
		return Map.class;
	}

	public Class getOutputClass()
	{
		return String.class;
	}

	public void setTemplateText(String templateText)
	{
	 	this.templateText = templateText;
	}

	public String getTemplateText()
	{
		return templateText;
	}
}