
package com.sas.util.transforms;

import com.sas.util.transforms.TransformInterface;
import com.sas.util.transforms.TransformException;
import java.util.Date;

/**
 * MillisecondsStringToDateTransform converts a string containing a number 
 * of milliseconds since the epoch to a Date object.
 * @see java.util.Date
 */
public class MillisecondsStringToDateTransform implements TransformInterface
{
	static final long serialVersionUID = -471023530221988295L;

	public static final MillisecondsStringToDateTransform defaultInstance = new MillisecondsStringToDateTransform();

	/**
	 * Converts a string containing a number of milliseconds since the epoch
	 * to a Date object.
	 * @param value a string containing a number of milliseconds since the epoch
	 * @exception TransformException
	 * @return A java.util.Date representing the date specified in the string
	 */
	public Object transform( Object value ) throws TransformException
	{
		try
		{
			long milliseconds = Long.parseLong( (String) value );
			return new Date( milliseconds );
		}
		catch (NumberFormatException nfe)
		{
			throw new TransformException( nfe.getMessage() );
		}
	}
}
