// Copyright (c) 1999 by SAS Institute Inc., Cary, NC 27513

package com.sas.util.transforms;

/**
 * ObjectArrayToStringArrayTransform to transform an Object array to a String array.
 */
public class ObjectArrayToStringArrayTransform
   implements TransformInputOutputInterface
{
   public static final String RB_KEY = "ObjectArrayToStringArrayTransform.";

   /**
    * Default instance of ObjectArrayToStringArrayTransform.
    */
   public static final ObjectArrayToStringArrayTransform defaultInstance =
      new ObjectArrayToStringArrayTransform();

   /**
    * Transform an Object[] to a String[] using System.arraycopy
    * @param object the object to transform
	* @return a string array containing the object array's items
    */
   public Object transform(Object object)
   {
      if (object instanceof Object[])
      {
		  Object[] array = (Object[])object;
		  String[] stringArray = new String[ array.length ];
		  System.arraycopy( array, 0, stringArray, 0, array.length );
		  return stringArray;
      }
      else
         throw new IllegalArgumentException(RB.getStringResource(RB_KEY, "notAnInstance.txt"));
   }

	public Class getInputClass()
	{
		return Object[].class;
	}

	public Class getOutputClass()
	{
		return String[].class;
	}
}
