// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513
package com.sas.util.transforms;

import com.sas.MissingValues;
import com.sas.util.NumberUtil;

import java.text.DateFormat;
import java.sql.Timestamp;
import java.sql.Time;
import java.util.Locale;

public class ObjectToStringTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = -7670709939329023996L;

	protected Locale locale;
	protected DateFormat dateFormatter;
	protected String nullString;
	protected String dotString;

	public static final ObjectToStringTransform defaultInstance = new ObjectToStringTransform();

	/**
	 * 
	 */
	public ObjectToStringTransform()
	{
		this(Locale.getDefault());
	}

	/**
	 * 
	 * @param locale
	 */
	public ObjectToStringTransform(Locale locale)
	{
		this.nullString = "null";
		this.dotString = ".";
		this.locale = locale;
	}
		
	/**
	 * 
	 */
	public Object transform( Object input )
	{
		if ( input != null )
		{
			if ( input instanceof Double )
			{
				return NumberUtil.toString( ((Double)input).doubleValue(), locale );
			}
			else if ( input instanceof Float )
			{
				return NumberUtil.toString( ((Float)input).floatValue(), locale );
			}
			else if ( input instanceof java.util.Date )
			{
				DateFormat formatter = dateFormatter; 
				if (formatter == null)
				{
					if (input instanceof Time)
						formatter = DateFormat.getTimeInstance(DateFormat.DEFAULT, locale);
					else if (input instanceof java.sql.Date)
						formatter = DateFormat.getDateInstance(DateFormat.DEFAULT, locale);
					else
						formatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, locale);
				}
				return formatter.format((java.util.Date)input);
			}
			else if ( input instanceof MissingValues )
			{
				if ( MissingValues.Dot.equals(input) )
					return dotString;
			}

			return input.toString();
		}
		else
			return nullString;
	}

	public Class getInputClass()
	{
		return Object.class;
	}

	public Class getOutputClass()
	{
		return String.class;
	}
	
	/**
	 * @return DateFormat
	 */
	public DateFormat getDateFormatter() 
	{
		return dateFormatter;
	}

	/**
	 * @return String
	 */
	public String getDotString() 
	{
		return dotString;
	}

	/**
	 * @return String
	 */
	public String getNullString() 
	{
		return nullString;
	}

	/**
	 * Sets the formatter used for when an Object is an instanceof java.util.Date.
	 * @param dateFormatter The dateFormatter to set
	 */
	public void setDateFormatter(DateFormat dateFormatter) 
	{
		this.dateFormatter = dateFormatter;
	}

	/**
	 * Sets the String used when the value is equal to the com.sas.MissingValue.Dot class.
	 * @param dotString The dotString to set
	 */
	public void setDotString(String dotString) 
	{
		this.dotString = dotString;
	}

	/**
	 * Sets the string return when a Object is null.
	 * @param nullString The nullString to set
	 */
	public void setNullString(String nullString) 
	{
		this.nullString = nullString;
	}

}
