// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.util.transforms;

import com.sas.util.CharBuffer;
import com.sas.util.Util;
import com.sas.util.Strings;

/**
 * Transform a java.lang.String[] object to a java.lang.String object.
 */
public class StringArrayToStringTransform implements TransformInputOutputInterface
{
   public static StringArrayToStringTransform defaultInstance = new StringArrayToStringTransform();

   /**
    * Transforms a String[] into a String of comma seperated quoted strings.
    */
   public Object transform(Object object)
   {
      return transform(object, true);
   }

   /**
    * Transforms a String[] into a String.  Use the includeQuotes parameter to specify whether
    * or not the each string in the array is to be quoted in the final string.
    * @param includeQuotes true to include quotes around each string item, false to not include quotes.
    */
   public String transform(Object object, boolean includeQuotes)
   {
      if (object == null)
         return null;

      String[] array = (String[])object;
      CharBuffer buffer = new CharBuffer();

      char delimiter = Util.getListDelimiterChar();

      if (includeQuotes)
      {
         for (int i=0; i<array.length; i++)
           {
             buffer.append(Strings.quote(array[i]));
             buffer.append(delimiter);
           }
      }
      else
      {
         for (int i=0; i<array.length; i++)
           {
             buffer.append(array[i]);
             buffer.append(delimiter);
           }
      }

      if (!buffer.isEmpty())
         buffer.remove(buffer.length() - 1, buffer.length());

      return buffer.toString();
   }

	public Class getInputClass()
	{
		return String[].class;
	}

	public Class getOutputClass()
	{
		return String.class;
	}
}
