// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;

/**
 * The StringToBooleanTransform converts a String to a Boolean object.  If the value of the 
 * input to the transform is null or a blank/empty string, Boolean.FALSE will be returned.
 * Otherwise, Boolean.valueOf() will be called to convert the string.
 * <P>
 * @see java.lang.Boolean#valueOf
 */
public class StringToBooleanTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = -4710235302219882295L;

	public static final StringToBooleanTransform defaultInstance = new StringToBooleanTransform();

	public Object transform( Object input )
	{
		// Return false if null input

		if ( input == null )
			return Boolean.FALSE;

		// Trim off whitespace on both ends of the string

		String trimmedInput = ((String)input).trim();

		// Return false if empty string

		if ( trimmedInput.length() == 0 )
			return Boolean.FALSE;

		return Boolean.valueOf( trimmedInput );
	}

	public Class getInputClass()
	{
		return String.class;
	}

	public Class getOutputClass()
	{
		return Boolean.class;
	}
}
