// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;


/**
 * The StringToByteTransform converts a String to a Byte object.  If the value of the 
 * input to the transform is null or a blank/empty string, zero will be returned.
 * Otherwise, Byte.valueOf() will be called to convert the string.
 * <P>
 * @see java.lang.Byte#valueOf(String)
 */
public class StringToByteTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = 4539523488635769533L;

	public static final StringToByteTransform defaultInstance = new StringToByteTransform();

	public Object transform( Object input )
	{
		// null strings return zero
		if ( input == null )
			return new Byte( (byte) 0);

		String trimmedInput = ((String)input).trim();

		// empty strings return zero
		if ( trimmedInput.length() == 0 )
		{
			return new Byte( (byte) 0);
		} // if

		return Byte.valueOf( trimmedInput );
	}

	public Class getInputClass()
	{
		return String.class;
	}

	public Class getOutputClass()
	{
		return Byte.class;
	}
}
