/* $Id: StringToDateTransform.java,v 1.4 2003/09/04 23:29:06 sasspl Exp $ */
/* Copyright (c) 2003 by SAS Institute Inc., Cary, NC 27513 */
package com.sas.util.transforms;

import com.sas.util.transforms.TransformException;

import java.text.ParseException;
import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;

public class StringToDateTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = -7670709939329023996L;

	protected Locale locale;
	protected DateFormat dateFormatter;
	protected String nullString;
	protected String dotString;

	public static final StringToDateTransform defaultInstance = new StringToDateTransform();

	/**
	 * 
	 */
	public StringToDateTransform()
	{
		this(Locale.getDefault());
	}

	/**
	 * 
	 * @param locale
	 */
	public StringToDateTransform(Locale locale)
	{
		this.nullString = "null";
		this.dotString = ".";
		this.locale = locale;
	}
		
	/**
	 * 
	 */
	public Object transform( Object input ) throws TransformException
	{
		if ( input != null )
		{
			if (dateFormatter == null)
			{
				dateFormatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, locale);
			}

			try
			{
				return dateFormatter.parse((String)input);
			}
			catch (ParseException e)
			{
				TransformException te = new TransformException(e.getMessage());
				te.initCause(e);
				throw te;
			}			
		}

		return nullString;
	}

	public Class getInputClass()
	{
		return String.class;
	}

	public Class getOutputClass()
	{
		return Date.class;
	}
	
	/**
	 * @return DateFormat
	 */
	public DateFormat getDateFormatter() 
	{
		return dateFormatter;
	}

	/**
	 * Sets the formatter used for when an Object is an instanceof java.util.Date.
	 * @param dateFormatter The dateFormatter to set
	 */
	public void setDateFormatter(DateFormat dateFormatter) 
	{
		this.dateFormatter = dateFormatter;
	}
}
