// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;

/**
 * The StringToIntegerTransform converts a String to a Integer object.  If the value of the 
 * input to the transform is null or a blank/empty string, zero will be returned.
 * Otherwise, Integer.valueOf() will be called to convert the string.
 * <P>
 * @see java.lang.Integer#valueOf
 */
public class StringToIntegerTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = -7757250351074975763L;

	public static final StringToIntegerTransform defaultInstance = new StringToIntegerTransform();
	
	public Object transform( Object input )
	{
		// null strings return zero
		if ( input == null )
			return new Integer(0);

		String trimmedInput = ((String)input).trim();

		// empty strings return zero
		if ( trimmedInput.length() == 0 )
		{
			return new Integer(0);
		} // if

		return Integer.valueOf( trimmedInput );
	}
	
	public Class getInputClass()
	{
		return String.class;
	}

	public Class getOutputClass()
	{
		return Integer.class;
	}
}
