// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;


/**
 * The StringToLongTransform converts a String to a Long object.  If the value of the 
 * input to the transform is null or a blank/empty string, zero will be returned.
 * Otherwise, Long.valueOf() will be called to convert the string.
 * <P>
 * @see java.lang.Long#valueOf
 */
public class StringToLongTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = 4539523488635769533L;

	public static final StringToLongTransform defaultInstance = new StringToLongTransform();

	public Object transform( Object input )
	{
		// null strings return zero
		if ( input == null )
			return new Long(0);

		String trimmedInput = ((String)input).trim();

		// empty strings return zero
		if ( trimmedInput.length() == 0 )
		{
			return new Long(0);
		} // if

		return Long.valueOf( trimmedInput );
	}

	public Class getInputClass()
	{
		return String.class;
	}

	public Class getOutputClass()
	{
		return Long.class;
	}
}
