/* Copyright (c) 2003 by SAS Institute Inc., Cary, NC 27513 */
package com.sas.util.transforms;

import com.sas.util.transforms.TransformException;

import java.text.DateFormat;
import java.util.Locale;
import java.text.ParseException;
import java.sql.Time;

public class StringToSQLTimeTransform extends StringToDateTransform
{
	static final long serialVersionUID = -7670709939329023996L;

	public static final StringToSQLTimeTransform defaultInstance = new StringToSQLTimeTransform();

	/**
	 * 
	 */
	public StringToSQLTimeTransform()
	{
		this(Locale.getDefault());
	}

	/**
	 * 
	 * @param locale
	 */						
	public StringToSQLTimeTransform(Locale locale)
	{
		super(locale);
	}
		
	/**
	 * 
	 */
	public Object transform( Object input ) throws TransformException
	{
		if ( input != null )
		{
			if (dateFormatter == null)
			{
				dateFormatter = DateFormat.getTimeInstance(DateFormat.DEFAULT, locale);
			}

			try
			{
				return new Time(((java.util.Date)dateFormatter.parse((String)input)).getTime());
			}
			catch (ParseException e)
			{
				TransformException te = new TransformException(e.getMessage());
				te.initCause(e);
				throw te;
			}			
		}

		return nullString;
	}

	public Class getOutputClass()
	{
		return Time.class;
	}	
}
