/* Copyright (c) 2003 by SAS Institute Inc., Cary, NC 27513 */
package com.sas.util.transforms;

import com.sas.util.transforms.TransformException;

import java.text.ParseException;
import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;

public class StringToSQLTimestampTransform extends StringToDateTransform
{
	static final long serialVersionUID = -7670709939329023996L;

	public static final StringToSQLTimestampTransform defaultInstance = new StringToSQLTimestampTransform();

	/**
	 * 
	 */
	public StringToSQLTimestampTransform()
	{
		this(Locale.getDefault());
	}

	/**
	 * 
	 * @param locale
	 */						
	public StringToSQLTimestampTransform(Locale locale)
	{
		super(locale);
	}
		
	/**
	 * 
	 */
	public Object transform( Object input ) throws TransformException
	{
		if ( input != null )
		{
			if (dateFormatter == null)
			{
				dateFormatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, locale);
			}

			try
			{
				return new java.sql.Timestamp(((java.util.Date)dateFormatter.parse((String)input)).getTime());
			}
			catch (ParseException e)
			{
				TransformException te = new TransformException(e.getMessage());
				te.initCause(e);
				throw te;
			}			
		}

		return nullString;
	}

	public Class getOutputClass()
	{
		return java.sql.Timestamp.class;
	}	
}
