// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.util.transforms;


/**
 * The StringToShortTransform converts a String to a Short object.  If the value of the 
 * input to the transform is null or a blank/empty string, zero will be returned.
 * Otherwise, Short.valueOf() will be called to convert the string.
 * <P>
 * @see java.lang.Short#valueOf(String)
 */
public class StringToShortTransform implements TransformInputOutputInterface
{
	static final long serialVersionUID = 4539523488635769533L;

	public static final StringToShortTransform defaultInstance = new StringToShortTransform();

	public Object transform( Object input )
	{
		// null strings return zero
		if ( input == null )
			return new Short( (short) 0);

		String trimmedInput = ((String)input).trim();

		// empty strings return zero
		if ( trimmedInput.length() == 0 )
		{
			return new Short( (short) 0);
		} // if

		return Short.valueOf( trimmedInput );
	}

	public Class getInputClass()
	{
		return String.class;
	}

	public Class getOutputClass()
	{
		return Short.class;
	}
}
