// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.util.transforms;

/**
 * An transformation interface which allows the caller to
 * perform some transformation action on an object.
 * This is useful when enumerating over collections, etc.
 * A transform may modify the Object passed in and return it, or it
 * may return a different Object.
 * <P>
 * Transforms are used in property linking to convert the value of
 * the source property to the class of the target property.
 * Transforms are also used in viewers such as the ListBox to 
 * effect how Objects are displayed in the viewer.
 * <P>
 * @author David Biesack
 * @version 1.0
 */

/*
 * @support sasdjb
 * @history
 * <dl>          
 * <dt>Fri Sep 27 1996  <dd>Created
 * <dt>sasdjb Fri Apr 18 1997 <dd> Added com.sas.util.transforms.TransformException
 * </dl>
 */

import com.sas.util.transforms.TransformException;

public interface TransformInterface extends java.io.Serializable
{
        /**
         * Does something with/to an object and returns the passed in
		 * Object or another Object.  
		 *
         * @param o The object which should be used
         * @exception com.sas.util.transforms.TransformException if the
         *            transform function fails.
         */
        Object transform(Object o) throws TransformException;
}