// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513


package com.sas.visuals;

/**
* An interface for objects that can be painted on other components.
* A PainterInterface object can be called from the paint() method of a visual
* component and using the Graphics of the other component will paint in the given area.
**/
public interface PainterInterface extends java.io.Serializable
{
	/**
	* The paint method called to paint this object.
	* @param component the component being painted on.
	* @param g the Graphics context
	* @param x the x position of the area to paint
	* @param y the y position of the area to paint
	* @param width the width of the area to paint
	* @param height the height of the area to paint
	**/
	public void paint(java.awt.Component component, java.awt.Graphics g, int x, int y, int width, int height );
	
	/**
	* Get the insets of used by the PainterInterface object when it is painted.
	* @return the insets used when the PainterInterface object is painted
	**/
	public java.awt.Insets getInsets();
}
