// Copyright (c) 1998 by SAS Institute Inc., Cary, NC 27513

package com.sas.beans;

import java.util.Vector;

/**
 * PropertyChangeEvent is a specialization of java.beans.PropertyChangeEvent which is exclusively
 * for IDE property changed events. In addition to the normal PropertyChangeEvent functionality,
 * it maintains a changed history for link loop avoidance.
 * @author Diane Baucom
 * @version 1.0
 * @history
 * <dl>
 * <dt>Mon Mar 24 1997  <dd>Created	- sasdqr
 * </dl>
 */
public class PropertyChangeEvent extends java.beans.PropertyChangeEvent
{
	static final long serialVersionUID = 6681050398947982876L;

	/**
	 * The history of attribute changed events.  This is used for link loop
	 * avoidance.
	 */
	public Vector m_ChangeHistory;
	
	/**
	 * Constructs a PropertyChangeEvent
	 * @see java.beans.PropertyChangeEvent
	 */
	public PropertyChangeEvent (Object source, String propertyName, Object oldValue, Object newValue)
	{
		super( source, propertyName, oldValue, newValue );
	}

	protected String paramString()
	{
		return "source=" + getSource() + ", property=" + getPropertyName() +
			", oldValue=" + getOldValue() + ", newValue=" + getNewValue();
	}

	public String toString()
	{
		return getClass().getName() + "[ " + paramString() + " ]";
	}
} // PropertyChangeEvent
